////////////////////////////////////////////////////////////////////////////////
// MODULE DrawMaster:   Pairing interface to GTour        G.Kaniuk 06/06/04
//
#include <vector>
#include <string>

#include "Tour.h"
#include "PairMaster.h"

#ifndef DrawMasterH
#define DrawMasterH

namespace DrawMaster										// CONVERTORS
{
	using std::string;

	using Tours::GPlayerPtr;

	string points_string(const GPlayerPtr P);
   string wins_string(const GPlayerPtr P);
	string homegames_string(const GPlayerPtr P);
}
//---------------------------------------------------------------------------

namespace DrawMaster										// REGISTER SUPPORT
{
	using Tools::ints;
   using Tools::intsi;

   using Tours::GTour;

	void ChangeGrade
   (
   	const GTour& Tour,
      ints  oGrd, ints nGrd,ints  oMMi, ints oMMs, ints& nMMi, ints& nMMs
   );
}
//---------------------------------------------------------------------------

namespace DrawMaster										// PRE-DRAW SUPPORT
{
	using Tours::GTour;

   enum 													  // Bar setting report
   {
   	brSet, brSleep, brLoBar, brHiBar,
      brNoMcmon, brManual,
      brOrder, brKomi,
      brNoBar
   };

   struct DGroup
   {
   	int Size;
      ints Score;

      DGroup(){ Size = 0; Score = 0; }
      
      DGroup(int size, ints score)
      {
      	Size  = size;
         Score = score;
      }
   };

   bool GetBarPops(GTour* Tour, int& LoPop, int& HiPop);
   
	int SetBar(GTour* Tour, int& BarPop);

   int SetInitialScores(ints NovBar, ints KomBar, ints MacBar, ints& BarDepth, GTour* Tour);

	void SetPlayerStates(void);

   bool StartDraw(PairMaster::PairCount& ModelCount);
}
//---------------------------------------------------------------------------

namespace DrawMaster										// POST DRAW SUPPORT
{
   using std::vector;
   using std::list;

   using Tours::GBoard;
   using PairMaster::PairCount;

   enum Comment
   {
      coRepeat,   // Played before in an earlier round
      coClub,     // Same club when there are others
      coLand,     // Same country when there are others

      coUneven,   // 0<MMS diff <=Komidiff
      coHcap,     // MMS > Komidiff
      coOrder,    // Bad board order
      coCount
   };

   struct GHandicap										// KOMI MANAGEMENT
   {
   	int Board;											// The board number
      int Stones;    									// Stones given to black > 0
      int Komi;      									// Komi given: + to black, - to white;

      GHandicap()
      {
      	Board = Stones = Komi = 0;
      }

      GHandicap(int board, int stones, int komi)
      {
      	Board = board;  Stones = stones;  Komi = komi;
      }

      GHandicap(int board, string komistones);

      bool operator==(const GHandicap& handicap)
      {
      	return Board == handicap.Board;
      }

      bool operator!=(const GHandicap& handicap)
      {
      	return Board != handicap.Board;
      }

      bool operator < (const GHandicap& handicap)
      {
      	return Board < handicap.Board;
      }

      bool SameAs(const GHandicap& newH)
      {
      	return Stones == newH.Stones && Komi == newH.Komi;
      }
   };

   void GameComment(Tours::GBoard Board, std::vector<bool>& Note);
   void DrawEditInfo(int black, int white, bool& Repeat, bool& BadOrder);

   string ForgeKomi(const GPlayerPtr Black, const GPlayerPtr White);
   int ParseKomi(string komi, int& Stones, int& Komi);

   int BoardCount(void);
   int FindBoard(const size_t pid);

   ints McMahonScore(int Round, GPlayerPtr Player);

   void SetMcMahonScores(void);
   
   void SetNewRound(void);
   void ResetRound(void);
   
   int AllocateBoards(int& PairedCount);
   void ListBoards(void);
   void LayoutBoards(void);
   
   void AnalyseDraw(PairMaster::PairCount& ActualCount);
   
   bool ClearDraw(void);
   bool ClearDraw(Tours::PlayerState State);

	bool ChangeDraw
   (
   	vector<GBoard>& NewPairs,		// New pairing after edit
      list<GHandicap>OldKomis,		// Original komi list
      list<GHandicap>NewKomis			// New komi list
   );

   void StartManualDraw(PairCount& ModelCount);
   void CancelManualDraw(void);
   void EndFullManualDraw(PairCount& ActualCount);

   int ManualDraw(const int b, const GBoard& Board, string& Handicap);

   bool CycleDraw
   (
   	const Byte PairMode, const int PairedCountRemaining,
      PairCount& Actual, vector<PairCount>& Actuals
   );
   
   void ChangeStyle(void);
}
//---------------------------------------------------------------------------

namespace DrawMaster										// RESULTS SUPPORT
{
   bool EditResult(int Board, Tools::intsi Bres, Tools::intsi Wres);

   intsi GetCurrentResult(GPlayerPtr Player);

   void RestoreResult(GPlayerPtr Player);

   int Restore(GPlayerPtr Player);
}
//---------------------------------------------------------------------------

#endif
