////////////////////////////////////////////////////////////////////////////////
// MODULE EditPlayerForm:       Register edit dialogue        G.Kaniuk 05/06/02
//
#include <string>
#include <vcl.h>
#pragma hdrstop

#include "EdPlayerIDH.h"
#include "EdPlayerRES.h"
#include "Format.h"
#include "Strings.h"
#include "PlayerMaster.h"
#include "RegMaster.h"
#include "StringsGUI.h"
#include "GoDrawGUI.h"
#include "EditPlayerForm.h"

extern void __fastcall BeginCommand(int idcmd);
extern void __fastcall EndCommand(int idcmd);

extern void __fastcall EditPlayer(const int player);
extern String __fastcall ShortPlayer(const int player);

extern void __fastcall RegGridEndPlayerEdit(const bool);
extern void __fastcall RegGridLoadPlayer(Tours::GPlayerPtr, const int);

extern void __fastcall SetRegGridStatusDormant(void);

extern void __fastcall ReloadPairResult(void);

extern void __fastcall ClearGridSelections(void); 		// GK 6.3.9

extern bool RegClosed(void);
//---------------------------------------------------------------------------
#pragma package(smart_init)

#pragma resource "*.dfm"

//---------------------------------------------------------------------------
// CONSTRUCTION
//---------------------------------------------------------------------------
using PlayerMaster::EditPlayerTube;
using Tools::to_string;
using Tools::s_S;
   
enum { etNameEd=34, elNameEd=116, ewNameEd=241, ehNameEd=21 };
enum { gtGameGr=152,glGameGr=8,   gwGameGr=398, ghGameGr=158 };


static const int lwNameList = 290;
static const int lcNameList[4] = { 2, 128, 156, 232 };

static const std::string pidFmt("0000");

__fastcall TEditPlayerFrm::TEditPlayerFrm(TComponent* Owner):TForm(Owner)
{
	Hide();

   IdBox->Tag 		= cbPid;
   TickPanel->Tag = cbTick;
   GradeBox->Tag 	= cbGrade;
   ClubBox->Tag 	= cbClub;
   LandBox->Tag 	= cbLand;
   GroupBox->Tag 	= cbGroup;
   NoteBox->Tag 	= cbNote;

   BehaveLbl->Caption = LoadStr(IDB_Behaviour);

   Tie1Lbl->Caption	 = EditPlayerTube.TieLabel(1);
   Tie2Lbl->Caption	 = EditPlayerTube.TieLabel(2);

   HelpFile 			 = GoDraw::HelpFilePath(String("EdPlayer.hlp"));

   if(PlayerGrpBox)
   {
   	NameBox = new GNameBox
      (
      	etNameEd, elNameEd, ehNameEd, ewNameEd,
         lwNameList, lcNameList,

         (TWinControl*)(PlayerGrpBox), this
      );
      NameBox->Editor->Tag = cbName;
      NameBox->Editor->TabOrder = 4;
      NameBox->Editor->HelpContext  = IDH_NameBox;

      GameGrid = new GGameGrid
      (
       	gtGameGr, glGameGr, ghGameGr, gwGameGr,

         (TWinControl*)(PlayerGrpBox)
      );
      GameGrid->Tag = cbGame;
      GameGrid->TabOrder = 11;
      GameGrid->HelpContext = IDH_GamesGrid;

      if(TickPanel)
      {
         TickBox = new GTickBox((TWinControl*)(TickPanel));

         TickBox->Tag = cbTick;

         TickBox->TabOrder = 3;

         TickBox->HelpContext = IDH_TickBox;
   	}

      if(iMMSText && MMSpin)
      {
         MMSpin->Tag 		= cbMMS;

         MMSpin->TabStop  	= true;
         MMSpin->TabOrder 	= 10;

         MMSpin->Min 		= PlayerMaster::mmsMin;
  			MMSpin->Max       = PlayerMaster::mmsMax;
   		MMSpin->Increment = Tours::mc_scale;
      }

      if(CloseBtn && RegScrollBar)
      {
      	RegScrollBar->LargeChange = 20;
         RegScrollBar->SmallChange = 1;
         RegScrollBar->Min         = 1;
         RegScrollBar->Max         = 100;
         RegScrollBar->Position    = 1;

         RegScrollBar->TabStop = false;
         RegTrackBar->TabStop = false;
      }
   }

   RegScrollBar->HelpContext	= IDH_RegScrollBar;
   HelpBtn->HelpContext 		= IDH_HelpBtn;
   CloseBtn->HelpContext 		= IDH_CloseBtn;
   AutoCompBox->HelpContext 	= IDH_AutoCompBox;
   CloseBox->HelpContext 		= IDH_CloseBox;
   RestoreBtn->HelpContext 	= IDH_CancelBtn;
   OKBtn->HelpContext 			= IDH_OkBtn;
   IdBox->HelpContext 			= IDH_IdBox;
   GradeBox->HelpContext 		= IDH_GradeBox;
   ClubBox->HelpContext 		= IDH_AreaBox;
   LandBox->HelpContext 		= IDH_AreaBox;
   GroupBox->HelpContext 		= IDH_GroupBox;
   NoteBox->HelpContext 		= IDH_NoteBox;
   iMMSText->HelpContext 		= IDH_iMMSText;
   cMMSText->HelpContext 		= IDH_cMMSText;
   Tie1Text->HelpContext 		= IDH_Tie1Text;
   Tie2Text->HelpContext 		= IDH_Tie2Text;
   BalPanel->HelpContext 		= IDH_Unbalances;

   TailDeleted = false;
	ReadOnly    = false;
   Modified 	= 0;
   ModifiedBox = 0;
   Report      = 0;

   GradeEntryText = String();
   GroupEntryText = String();

   RestoreBtn->Hint = LoadStr(IDB_RestoreHint);
   RestoreBtn->ShowHint = true;

   OKBtn->Hint = LoadStr(IDB_SaveHint);
   OKBtn->ShowHint = true;
}

__fastcall TEditPlayerFrm::~TEditPlayerFrm()
{
	delete GameGrid;
	delete NameBox;
   delete TickBox;
}

//---------------------------------------------------------------------------
// LOADING
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::LoadLists(void)
{
   using Tools::load_str;
   using Tools::S_s;
   using ToolsGUI::erOperr;
   using ToolsGUI::erUnknown;
   using ToolsGUI::ETour;
	using ToolsGUI::ShowMessage;

	try
   {
   	EditPlayerTube.LoadListsFromAlgo(IdBox->Items, GradeBox->Items, GroupBox->Items);
   	EditPlayerTube.LoadRegionsFromFile(ClubBox->Items, LandBox->Items);

      NameBox->ListBox->SetSorted(false);
   		EditPlayerTube.LoadPlayersFromFile(NameBox->ListBox->Items);
      NameBox->ListBox->SetSorted(true);
	}
  	catch(ETour& Eload)
   {
   	// The LoadSOMETHING routines Throw an exception
      // with an appropriate message

   	ShowMessage(Eload.Etype, S_s(Eload.Message));
   }
   catch(...)
   {
   	ShowMessage(erUnknown, load_str(IDB_leUnknown));
   }
}

void __fastcall TEditPlayerFrm::UpdateLists(void)
{
   PlayerMaster::EditPlayerTube.UpdateListsFromTour
   (
   	IdBox->Items, NameBox->ListBox->Items,
      ClubBox->Items, LandBox->Items,
      NoteBox->Items
   );
}

void __fastcall TEditPlayerFrm::ConnectFindTubeLists(void)
{
   using RegMaster::FindTube;

   FindTube.Lists[RegMaster::fiName]  = NameBox->ListBox->Items;
   FindTube.Lists[RegMaster::fiGrade] = GradeBox->Items;
   FindTube.Lists[RegMaster::fiClub]  = ClubBox->Items;
   FindTube.Lists[RegMaster::fiLand]  = LandBox->Items;
   FindTube.Lists[RegMaster::fiGroup] = GroupBox->Items;
   FindTube.Lists[RegMaster::fiNote]  = NoteBox->Items;

   // No List for fiRound
   // The List for fiMMS is internally generated
}

void __fastcall TEditPlayerFrm::Load()
{
   using Tools::load_str;
   using ToolsGUI::erOperr;
   using ToolsGUI::ShowMessage;

   using PlayerMaster::epiNew;
   using PlayerMaster::epiExists;

	// SET UP THE SCROLL BAR AT CHANGE OF STATE
   const int State 		 = EditPlayerTube.State;
   const int Row         = EditPlayerTube.PlayerRow;
   const int PlayerCount = EditPlayerTube.PlayerCount;
   const int MaxScroll   = EditPlayerTube.NewBlocked ? PlayerCount : PlayerCount+1;
   const int PageSize    = 20;
   const int PlayerPos   = Row <= MaxScroll ? Row : MaxScroll;

   switch(State)
   {
      case PlayerMaster::epiNew:
      {
      	SetScrollBarParams(MaxScroll, 1, MaxScroll, PageSize);
         SetTrackBarParams(MaxScroll, 1, MaxScroll, PageSize);
      }
      break;

      case PlayerMaster::epiExists:
      {
      	SetScrollBarParams(PlayerPos, 1, MaxScroll, PageSize);
         SetTrackBarParams(PlayerPos, 1, MaxScroll, PageSize);
      }
      break;

      default:
      {
      	ShowMessage(erOperr, load_str(IDB_BadTubeState));

         return;
      }
   }

   // THE PLAYER DETAILS HAVE ALREADY BEEN LOADED INTO THE TUBE
   // LOAD THE USER EDITABLE TUBE FIELDS

   NameBox->Load(EditPlayerTube.Name);

   Load(IdBox,		EditPlayerTube.PlayerID);
   Load(GradeBox, EditPlayerTube.Grade);
   Load(ClubBox,	EditPlayerTube.Club);
   Load(LandBox,	EditPlayerTube.Land);
   Load(GroupBox,	EditPlayerTube.Group);
   Load(NoteBox,	EditPlayerTube.Note);

   TickBox->Reg = EditPlayerTube.Reg;

   // THERE ARE NON-EDITABLE FIELDS
   Tie1Lbl->Caption  = EditPlayerTube.TieLabel(1);
   Tie2Lbl->Caption  = EditPlayerTube.TieLabel(2);

   Tie1Text->Caption	= EditPlayerTube.TieText(1);
   Tie2Text->Caption	= EditPlayerTube.TieText(2);

    MMSpin->Position	= EditPlayerTube.MMSi;
   iMMSText->Caption	= EditPlayerTube.iMMSText(MMSpin->Position);
   cMMSText->Caption	= EditPlayerTube.cMMSText();

   ColBalLbl->Caption= EditPlayerTube.ColBalText();
   OutBalLbl->Caption= EditPlayerTube.UnevenText();
   UDBalLbl->Caption = EditPlayerTube.UpDnBalText();

   // LOAD THE GAMES
   GameGrid->RowCount = EditPlayerTube.Rounds+1;
   GameGrid->Forge();
   GameGrid->Load();

   Report   = 0;
   Modified = 0;

   ForgeCaption();
}

void __fastcall TEditPlayerFrm::SetBoxFocus(void)
{
   switch(PlayerMaster::EditPlayerTube.PlayerCol)
   {
      case 1: if(TickBox->Enabled) TickBox->SetFocus(); 			break;
      case 2: if(NameBox->Enabled)NameBox->Editor->SetFocus();	break;
      case 3: if(GradeBox->Enabled)GradeBox->SetFocus(); 		break;
      case 4: if(ClubBox->Enabled)ClubBox->SetFocus(); 			break;
      case 5: if(LandBox->Enabled)LandBox->SetFocus(); 			break;
      case 6: if(GameGrid->Enabled)GameGrid->SetFocus(); 		break;
      case 7: if(GroupBox->Enabled)GroupBox->SetFocus(); 		break;
      case 8: if(NoteBox->Enabled)NoteBox->SetFocus(); 			break;

      default: if(IdBox->Enabled)IdBox->SetFocus(); 				break;
   }
}
void __fastcall TEditPlayerFrm::Save(void)
{
   // PRESERVE THE EXISTING PLAYER TUBE AS WE MIGHT NEED TO RESET AUTOMATICALLY
   PlayerMaster::SavedPlayerTube = EditPlayerTube;

	// HARVEST BASIC PLAYER DETAILS
   EditPlayerTube.PlayerID = IdBox->Items->Strings[EditPlayerTube.PlayerRow-1];

   EditPlayerTube.Name		= NameBox->Editor->Text;
   EditPlayerTube.Grade		= GradeBox->Text;
   EditPlayerTube.Club		= ClubBox->Text;
   EditPlayerTube.Land		= LandBox->Text;
   EditPlayerTube.Group		= GroupBox->Text;
   EditPlayerTube.Note		= NoteBox->Text;

   // HARVEST REGISTRATION AND ROUNDS DATA
   EditPlayerTube.Reg 		= TickBox->Reg;
   EditPlayerTube.MMSi		= MMSpin->Position;

   // HARVEST GAMES GRID
   GameGrid->HarvestPlay(EditPlayerTube.Playing);
}


bool __fastcall TEditPlayerFrm::TextChanged(void)
{
	using Tools::cDot;

	const String EditorText =  NameBox->Editor->Text;

   if(EditPlayerTube.Name 		!= EditorText)			return true;
   if(EditPlayerTube.Grade		!= GradeBox->Text)	return true;
   if(EditPlayerTube.Club 		!= ClubBox->Text)		return true;
   if(EditPlayerTube.Land 		!= LandBox->Text) 	return true;
   if(EditPlayerTube.Group 	!= GroupBox->Text)	return true;
   if(EditPlayerTube.Note 		!= NoteBox->Text) 	return true;
   if(EditPlayerTube.Reg		!= TickBox->Reg)		return true;
   if(EditPlayerTube.MMSi		!= MMSpin->Position) return true;

   for(int r=0; r<EditPlayerTube.Rounds; ++r)
   {
   	bool Playing = GameGrid->Cells[1][r+1][1] != cDot;

   	if(EditPlayerTube.Playing[r] != Playing) 		return true;
   }

   return false;
}

void __fastcall TEditPlayerFrm::ForgeCaption(void)
{
   using PlayerMaster::epiNew;

   const String Starred = Modified ? String("*") : String();

   if(EditPlayerTube.State==epiNew)
   {
   	String NewPid = IdBox->Items->Strings[EditPlayerTube.PlayerCount];

      Caption = LoadStr(IDB_NewPlayer) + Starred + NewPid;
   }
   else
		Caption = LoadStr(IDB_KnownPlayer) + Starred + EditPlayerTube.Name;
}

void __fastcall TEditPlayerFrm::OnFormShow(TObject *Sender)
{
	using PlayerMaster::epiNew;
   using PlayerMaster::epiExists;

	// THIS IS THE OnShow Event handler
	switch(EditPlayerTube.State)
   {
   	case PlayerMaster::epiVirgin:   Caption = LoadStr(IDB_Ready);			break;

      case PlayerMaster::epiNew: 	  ForgeCaption();								break;

      case PlayerMaster::epiExists:   ForgeCaption(); 							break;

      default:
      {
      	Caption = LoadStr(IDB_BadFormState);

         return;
      }
   }

   NameBox->Show();
   TickBox->Show();
   MMSpin->Show();
   GameGrid->Show();
}

void __fastcall TEditPlayerFrm::Update(void)
{
	TickBox->Repaint();

	TForm::Update();
}

void __fastcall TEditPlayerFrm::SetAction(const bool permitted)
{
   const bool active       = permitted && !RegClosed();

	AutoCompBox->Enabled 	= active;
   CloseBox->Enabled 		= active;

   HelpBtn->Enabled 			= true;
   CloseBtn->Enabled 		= true;
   RegScrollBar->Enabled 	= true;

   IdBox->Enabled 			= true;

   TickBox->Enabled     = active;
   TickPanel->Enabled 	= active;
   GradeBox->Enabled 	= active;
   ClubBox->Enabled 		= active;
   LandBox->Enabled 		= active;
   GroupBox->Enabled 	= active;
   NoteBox->Enabled 		= active;
   MMSpin->Enabled 		= permitted;
   GameGrid->Enabled 	= active;

   RestoreBtn->Enabled  = permitted;
   OKBtn->Enabled  		= permitted;

   NameBox->SetEnabled(active);

   if(!TickBox->Enabled)
   	TickBox->Selected = GTickBox::ftGrey;
   else
      TickBox->Selected = EditPlayerTube.PlayerCol==cbTick
      						? GTickBox::ftSelect
                        : GTickBox::ftFree;
}

void TEditPlayerFrm::UpdateModified(const int idcontrol, const bool modified)
{
	const Tools::bits OldModified = Modified;

	Tools::set_bit(idcontrol, modified, Modified);

   if(!OldModified && Modified) ForgeCaption();	// Modified is a bits
   if( OldModified && !Modified) ForgeCaption();
}

//---------------------------------------------------------------------------
// SCROLLBAR SUPPORT
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::SetScrollBarPos(const int pos)
{
   RegScrollBar->OnScroll    = 0;
   	RegScrollBar->Position = pos;				// Triggers both OnChange and OnScroll
   RegScrollBar->OnScroll    = OnScrollEvent;
}

void __fastcall TEditPlayerFrm::SetScrollBarParams
(
	const int pos, const int minpos, const int maxpos, const int pagesize
)
{
   RegScrollBar->OnScroll = 0;

      RegScrollBar->LargeChange = pagesize;
      RegScrollBar->SmallChange = 1;
      RegScrollBar->SetParams(pos, minpos, maxpos);

   RegScrollBar->OnScroll = OnScrollEvent;
}


void __fastcall TEditPlayerFrm::OnScrollEvent
(
	TObject *Sender, TScrollCode ScrollCode, int &ScrollPos
)
{
	TScrollBar* ScrollBar = dynamic_cast<TScrollBar*>(Sender);

   if(!ScrollBar) return;

   switch(ScrollCode)
   {
   	case scTrack:
      {
      	Caption = ShortPlayer(ScrollPos);
         SetText(IdBox, s_S(to_string(pidFmt, ScrollPos)));
      }
      break;

   	case scEndScroll:
         if(RegTrackBar->Position != ScrollBar->Position)
         	RegTrackBar->Position = ScrollBar->Position;
      break;

      default: return;
   }
}

void __fastcall TEditPlayerFrm::SetTrackBarParams
(
	const int pos, const int minpos, const int maxpos, const int pagesize
)
{
	RegTrackBar->OnChange = 0;
      RegTrackBar->Min      = minpos;
      RegTrackBar->Max      = maxpos;
      RegTrackBar->PageSize = pagesize;
      RegTrackBar->Position = pos;
   RegTrackBar->OnChange = OnTrackChange;
}

void __fastcall TEditPlayerFrm::SetTrackBarPos(const int pos)
{
	RegTrackBar->OnChange     = 0;
   	RegTrackBar->Position  = pos;				// Triggers OnChange
   RegTrackBar->OnChange     = OnTrackChange;

   RegTrackBar->SetFocus();
}

void __fastcall TEditPlayerFrm::OnTrackChange(TObject *Sender)
{
   const int TrackPos = RegTrackBar->Position;

   // THERE IS NOTHING TO DO: ENSURE SCROLLBAR IS UP TO DATE WITH TRACKBAR
   if(TrackPos == EditPlayerTube.PlayerRow)
   {
   	if(RegScrollBar->Position != TrackPos)
      	RegScrollBar->Position = TrackPos;

      return;
   }

   if(ValidPlayerEdit())
   {
   	// TRY TO CHANGE PLAYER TO NEW POSITION
      ChangeToPlayer(EditPlayerTube.PlayerCol, TrackPos);

      // USER MAY HAVE CANCELLED THE CHANGE
      if(EditPlayerTube.PlayerRow != TrackPos)
         SetTrackBarPos(EditPlayerTube.PlayerRow);

      // KEEP THE SCROLLBAR THUMB TAB UP TO DATE
   	RegScrollBar->Position = EditPlayerTube.PlayerRow;

      // TRACKBAR RETAINS FOCUS FOR FUTURE KEY INPUT
      RegTrackBar->SetFocus();
   }
   else
   {
   	if(RegScrollBar->Position != EditPlayerTube.PlayerRow)
      	RegScrollBar->Position = EditPlayerTube.PlayerRow;

      ForgeCaption();
      
      SetText(IdBox, s_S(to_string(pidFmt, RegScrollBar->Position)));
   }
}

void __fastcall TEditPlayerFrm::OnTrackKeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
{
	switch(Key)
   {
   	case VK_ESCAPE:
      {
      	// RETURN TO LAST LOADED PLAYER
         ::EditPlayer(EditPlayerTube.PlayerRow);

         SetScrollBarPos(EditPlayerTube.PlayerRow);

         SetTrackBarPos(EditPlayerTube.PlayerRow);

         SetBoxFocus();
      }
      break;

      case VK_RETURN:
      {
         // WE ARE FINISHED SCROLLING AND WANT TO EDIT
         if(EditPlayerTube.State==PlayerMaster::epiNew)
            NameBox->Editor->SetFocus();
         else
            SetBoxFocus();
      }
      break;

      default:
      {
      	int p = Position;

         ++p;
      }
      return;
   }

   Key = 0;
}

//---------------------------------------------------------------------------
// SUPPORT FOR STANDARD COMBO BOXES
//---------------------------------------------------------------------------
void TEditPlayerFrm::SelectText(TComboBox* combo, const int start, const int length)
{
   combo->SelStart = start>=0 ? start : 0;
   combo->SelLength = length>=0 ? length : 0;
}

void TEditPlayerFrm::SetText(TComboBox* combo, const String text)
{
	const bool OnChangeHandled = combo->OnChange != 0;

 	combo->OnChange = 0;				 				// Setting Text will trigger OnChange!

   combo->Text = text;								// Initialise the Combo

   if(OnChangeHandled)
   	combo->OnChange = OnChangeComboText;	// Allow user to change now
}

void TEditPlayerFrm::Load(TComboBox* combo, const String text)
{
	// SETS Text AND ItemIndex FOR THE combo
   if(text.IsEmpty())
   {
   	SetItemIndex(combo, -1);
   	SetText(combo, String());

      return;
   }

   // THE ROUTINE FindInList GUARANTEES TO DO BOTH
   // FOR NON-EMPTY text
   FindInList(combo, text);
}

void TEditPlayerFrm::LoadExact(TComboBox* combo, const String text)
{
	// SETS Text AND ItemIndex FOR THE combo
   if(text.IsEmpty())
   {
   	SetItemIndex(combo, -1);
   	SetText(combo, String());

      return;
   }

   // THE ROUTINE FindInList GUARANTEES TO DO BOTH
   // FOR NON-EMPTY text
   FindExactInList(combo, text);
}

void TEditPlayerFrm::SetItemIndex(TComboBox* combo, const int itemindex)
{
	const bool OnChangeHandled = combo->OnChange != 0;

 	combo->OnChange = 0;				 				// Setting ItemIndex triggers OnChange!
   {
      combo->ItemIndex = itemindex;				// Safely set it
   }
   if(OnChangeHandled)
   	combo->OnChange = OnChangeComboText;	// Allow user to change now
}


void TEditPlayerFrm::EscapeCombo(TComboBox* combo)
{
	switch(combo->Tag)
   {
      case cbPid:   Load(IdBox,   EditPlayerTube.PlayerID); break;
      case cbClub:  Load(ClubBox, EditPlayerTube.Club);		break;
      case cbLand:  Load(LandBox, EditPlayerTube.Land); 		break;
      case cbNote:  Load(NoteBox, EditPlayerTube.Note); 		break;

      case cbGrade: NewGrade(EditPlayerTube.Grade);			break;
      case cbGroup: NewGroup(EditPlayerTube.Group);			break;

      default: 	  Report = PlayerMaster::peCount;    		return;
   }

   Report = 0;

   UpdateModified(combo->Tag, false);	ModifiedBox = (TObject*)combo;

   SelectText(combo, combo->Text.Length(),0);
}

void TEditPlayerFrm::FindInList(TComboBox* combo, const String text)
{
    const int TextLength = text.Length();
   int ItemIndex 			= combo->ItemIndex;

   if(TextLength==0)		// We do not need to search for empty text
   	return;

   // ItemIndex MAY ALREADY BE SET TO IDENTIFY text IN ITEMS
   if(ItemIndex>=0)
   {
   	const String& ItemText = combo->Items->Strings[ItemIndex];

   	if(ItemText == text)
      {
      	SetText(combo,ItemText);

      	return;
      }
   }

	ItemIndex = -1;

   for(int i=0; i<combo->Items->Count; i++)
   {
      const String ItemText = combo->Items->Strings[i].UpperCase();

      if(TextLength <= ItemText.Length())			// Possible substring
         if(ItemText.Pos(text.UpperCase())==1)  // So ItemText begins with text
         {
          	ItemIndex = i;		 break;				// Found
         }
   }

   if(ItemIndex >= 0)
   	SetItemIndex(combo, ItemIndex);	// Sets Text to found text
   else
   {
   	SetItemIndex(combo, -1);			// Automatically clears the text

      SetText(combo, text);				// So we must reset it
   }
}

void TEditPlayerFrm::FindExactInList(TComboBox* combo, const String text)
{
   int ItemIndex 			= combo->ItemIndex;

   if(text.IsEmpty())		// We do not need to search for empty text
   	return;

   const String utext = text.UpperCase(); String ItemText;

   // ItemIndex MAY ALREADY BE SET TO IDENTIFY text IN ITEMS
   if(ItemIndex>=0)
   {
   	ItemText = combo->Items->Strings[ItemIndex];

   	if(utext == ItemText.UpperCase())
      {
      	SetText(combo,ItemText);

      	return;
      }
   }

	ItemIndex = -1;

   for(int i=0; i<combo->Items->Count; i++)
   {
      ItemText = combo->Items->Strings[i];

      if(utext == ItemText.UpperCase())
      {
         ItemIndex = i;		 break;				// Found
      }
    }

   if(ItemIndex >= 0)
   	SetItemIndex(combo, ItemIndex);	// Sets Text List text
   else
   {
   	SetItemIndex(combo, -1);			// Automatically clears the text

      SetText(combo, text);				// So we must reset it
   }
}

bool TEditPlayerFrm::TextModified(TComboBox* combo)
{
   switch(combo->Tag)
   {
   	case cbGrade: return combo->Text != EditPlayerTube.Grade;

      case cbClub:  return combo->Text != EditPlayerTube.Club;
      case cbLand:  return combo->Text != EditPlayerTube.Land;

      case cbGroup: return combo->Text != EditPlayerTube.Group;
      case cbNote:  return combo->Text != EditPlayerTube.Note;

      default: 	  return false;
   }
}

void TEditPlayerFrm::ClearUnknownPlayerFields(void)
{
   Load(GradeBox, EditPlayerTube.Grade);
   Load(ClubBox,	EditPlayerTube.Club);
   Load(LandBox,	EditPlayerTube.Land);
   Load(GroupBox,	EditPlayerTube.Group);
   Load(NoteBox,	EditPlayerTube.Note);

   SetSpinScores(0, 0);
}

//---------------------------------------------------------------------------
// EVENTS FOR STANDARD COMBO BOXES
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::OnEnterCombo(TObject *Sender)
{
	TComboBox* ComboBox = dynamic_cast<TComboBox*>(Sender);

   if(!ComboBox)return;

   // THE MMSi IS ONLY UPDATED IF THE GRADE OR GROUP CHANGES
   if(ComboBox == GradeBox)GradeEntryText = ComboBox->Text;
   if(ComboBox == GroupBox)GroupEntryText = ComboBox->Text;	// Used only to prevent spurious MMS updates

   // SCAN FOR ERROR ON PREVIOUS COMPONENT
	UserError(ComboBox->Tag);
}

void __fastcall TEditPlayerFrm::OnExitCombo(TObject *Sender)
{

	TComboBox* ComboBox = dynamic_cast<TComboBox*>(Sender);

   if(!ComboBox)return;

   // IF ANY PREVIOUS ERROR, Report WILL ALREADY HAVE BEEN SET
   // AND WE ARE EXITING TO THE ERROR FIELD 
   if(Report)
   	return;

   bool ChangedText = false;

   // PARSE TEXT AND SET ERROR REPORT ON FAILURE
   switch(ComboBox->Tag)
   {
   	case cbPid:	ValidPid(IdBox->Text);	break;

      case cbGrade:
      {
      	NewGrade(ComboBox->Text);

         ChangedText = ComboBox->Text != EditPlayerTube.Grade;
      }
      break;

      case cbClub:
      {
      	ValidClub(ComboBox->Text);

         ChangedText = ComboBox->Text != EditPlayerTube.Club;
      }
      break;

      case cbLand:
      {
      	ValidLand(ComboBox->Text);

         ChangedText = ComboBox->Text != EditPlayerTube.Land;
      }
      break;

      case cbGroup:
      {
      	NewGroup(ComboBox->Text);

         ChangedText = ComboBox->Text != EditPlayerTube.Group;
      }
      break;

      case cbNote:
      {
      	ValidNote(ComboBox->Text);

         ChangedText = ComboBox->Text != EditPlayerTube.Note;
      }
      break;

      default: Beep(); Caption = Caption + String(" ERR 002");  return;
   }

   // THE TARGET MAY HAVE LOST FOCUS IF THERE WAS AN ERROR
   // ERROR REPORTED TO USER WHEN CLICKING ANOTHER FIELD

   UpdateModified(ComboBox->Tag, ChangedText);  ModifiedBox = Sender;
}
//---------------------------------------------------------------------------

void __fastcall TEditPlayerFrm::OnChangeComboText(TObject* Sender)
{
	TComboBox* ComboBox = dynamic_cast<TComboBox*>(Sender);

   if(!ComboBox)return;

   const String utext = ComboBox->Text;		// Current user's text

   const int SelStart = ComboBox->SelStart;	// New position of cursor AFTER the text change

   if(TailDeleted)									// BackSpace key
   {
   	TailDeleted = false;

      UpdateModified(ComboBox->Tag, TextModified(ComboBox));

      ModifiedBox = Sender;

      return;
   }

   if(AutoCompBox->Checked)
   {
   	FindInList(ComboBox, utext);			// Text is set to found text

      if(ComboBox->ItemIndex>=0)				// User's text was found
      {
         const int pos = utext.Length();

         SelectText(ComboBox, pos, ComboBox->Text.Length() - pos);
      }
      else
         ComboBox->SelStart = SelStart;
   }

   UpdateModified(ComboBox->Tag, TextModified(ComboBox));
   ModifiedBox = Sender;
}
//---------------------------------------------------------------------------


void __fastcall TEditPlayerFrm::IdBoxKeyDown(WORD &Key, TShiftState Shift)
{
	using Tools::load_str;
   using ToolsGUI::erOperr;
   using ToolsGUI::erWarn;

   const bool UpDnKey = Key==VK_UP || Key==VK_DOWN || Key==VK_PRIOR || Key==VK_NEXT;

   // PARSE THE TEXT FOR SYNTAX AND RANGE
   if(Key==VK_RETURN || UpDnKey)
	   if(IdBox->Text != EditPlayerTube.PlayerID)  // Type a new number or step
         if(!ValidPid(IdBox->Text))
         {
            Key = 0;

            UserError(cbPid);

            return;
         }

	if(Key == VK_RETURN)
   {
   	Key = 0;

      if(IdBox->Text == EditPlayerTube.PlayerID)	// Same number
      {
      	OKBtn->Click();	return;						// Just completing an edit
      }

      // ENSURE PLAYER ID ACTUALLY EXISTS
      FindInList(IdBox, IdBox->Text);

      const int IdPlayer  = IdBox->ItemIndex+1;

      if(IdBox->ItemIndex<0)
      {
         ShowMessage(erOperr, load_str(IDB_PlayerDeleted));

         return;
      }

      if(EditPlayerTube.NewBlocked)
      	if(IdPlayer == EditPlayerTube.PlayerCount+1)
         {
         	ShowMessage(erOperr, load_str(IDB_NoNewPlayer));

            SetItemIndex(IdBox, EditPlayerTube.PlayerRow - 1);
            
            return;
         }

      ChangeToPlayer(EditPlayerTube.PlayerCol, IdPlayer);

      return;
   }

   if(UpDnKey)
   {
   	const int KeyNow       = Key;
   	const int PlayerCount  = EditPlayerTube.PlayerCount;
      const int MaxPlayerRow = EditPlayerTube.NewBlocked ? PlayerCount : PlayerCount+1;

   	int PlayerRow = IdBox->ItemIndex+1;	// Not yet updated

      Key = 0;

      switch(KeyNow)
      {
      	case VK_UP:
         {
         	--PlayerRow;	if(PlayerRow<1) return;
         }
         break;

         case VK_DOWN:
         {
         	++PlayerRow;	if(PlayerRow>MaxPlayerRow) return;
         }
         break;

         case VK_PRIOR:
         {
         	PlayerRow -= 20;	if(PlayerRow<1) return;
         }
         break;

         case VK_NEXT:
         {
         	PlayerRow += 20;	if(PlayerRow>MaxPlayerRow) return;
         }
         break;

			default: return;
      }

      ChangeToPlayer(EditPlayerTube.PlayerCol, PlayerRow);

      return;
   }
}

void __fastcall TEditPlayerFrm::OnComboKeyDown
(TObject *Sender, WORD &Key, TShiftState Shift)
{
	TComboBox* ComboBox = dynamic_cast<TComboBox*>(Sender);

   if(!ComboBox)return;

   if(ComboBox == IdBox)
   {
   	IdBoxKeyDown(Key, Shift);

      if(Key==0) return;
   }


	switch(Key)
   {
		case VK_ESCAPE: EscapeCombo(ComboBox);	break;

      case VK_RETURN:
      {
      	Key = 0;

         OKBtn->Click();
		}
      return;

      case VK_DELETE: case VK_BACK:
      {
      	const int SelEnd = ComboBox->SelStart + ComboBox->SelLength;

      	TailDeleted = SelEnd == ComboBox->Text.Length();
      }
      return;

      default: return;
   }

   Key = 0;
}

void __fastcall TEditPlayerFrm::OnClickCombo(TObject *Sender)
{
   using ToolsGUI::OperatorError;

 	TComboBox* ComboBox = dynamic_cast<TComboBox*>(Sender);

   if(!ComboBox)return;

   // IF SOME OTHER FIELD HAS ERROR WE ABORT THE CLICK
   if(UserError(ComboBox->Tag)) return;

   switch(ComboBox->Tag)
   {
   	case cbPid:
      {
      	const int IdPlayer =  IdBox->ItemIndex+1;

         if(IdPlayer == EditPlayerTube.PlayerCount+1)
         	if(EditPlayerTube.NewBlocked)
            {
            	OperatorError(IDB_NoNewPlayer);

               SetItemIndex(IdBox, EditPlayerTube.PlayerRow - 1);

               return;
            }

      	ChangeToPlayer(EditPlayerTube.PlayerCol, IdPlayer);
      }
      break;

   	case cbGrade:  NewGrade(ComboBox->Text); break;

      case cbGroup:	NewGroup(ComboBox->Text); break;

      default: break;
   }
}
//---------------------------------------------------------------------------
// MMS SPIN BUTTON
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::OnClickMMSpin(TObject* Sender, TUDBtnType Button)
{
	// DO NOT TRY TO DO ANYTHING HERE INVOLVING ANY SetFocus()
   // OR ANY ShowMessage();

	UpdateMMS(cbMMS);

   UpdateModified(cbMMS, MMSpin->Position != EditPlayerTube.MMSi);
}

//---------------------------------------------------------------------------
// PARSERS
//---------------------------------------------------------------------------
bool __fastcall TEditPlayerFrm::ValidPid(const String Pid)
{
   // WE JUST PARSE FOR OBVIOUS SYNTAX ERRORS AND OUT OF RANGE
   // WE WAIT FOR RETURN KEY TO CHANGE PLAYER

   String PidF = Pid;

   Report =  EditPlayerTube.ValidPid(PidF);	if(Report)return false;

   Load(IdBox, PidF);

   if(IdBox->ItemIndex < 0)
   {
      Report = PlayerMaster::pePidDel;	return false;
   }
   else
      return true;
}

bool __fastcall TEditPlayerFrm::ValidName(const String Name)
{
	String NameF = Name;

   Report  = EditPlayerTube.ValidName(NameF);	if(Report) return false;

   NameBox->Load(NameF);

   return true;
}

void __fastcall TEditPlayerFrm::NewGrade(const String Grade)
{
	String GradeF = Grade;

   Report = EditPlayerTube.ValidGrade(GradeF);	if(Report) return;

   Load(GradeBox, GradeF);

   if(GradeF.IsEmpty())
      SetSpinScores(0,0);
   else
   	if(GradeEntryText != GradeF)
			UpdateMMS(cbGrade);

   GradeEntryText = GradeF;
}

bool __fastcall TEditPlayerFrm::ValidClub(const String Club)
{
	String ClubF = Club;

	Report = EditPlayerTube.ValidClub(ClubF);  if(Report) return false;

   LoadExact(ClubBox, ClubF);

   return true;
}

bool __fastcall TEditPlayerFrm::ValidLand(const String Land)
{
	String LandF = Land;

	Report = EditPlayerTube.ValidLand(LandF);  if(Report) return false;

   LoadExact(LandBox, LandF);

   return true;
}

void __fastcall TEditPlayerFrm::NewGroup(const String Group)
{
	String GroupF = Group;

   Report = EditPlayerTube.ValidGroup(GroupF);  if(Report) return;

   Load(GroupBox, GroupF);

   if(!GroupF.IsEmpty())
   	if(GroupF != GroupEntryText)
         UpdateMMS(cbGroup);

   GroupEntryText = GroupF;
}

bool __fastcall TEditPlayerFrm::ValidNote(const String Note)
{
	String NoteF = Note;

   Report = EditPlayerTube.ValidNote(NoteF);  if(Report) return false;

   LoadExact(NoteBox, NoteF);

   return true;
}

bool __fastcall TEditPlayerFrm::UserError(const int cbox)
{
	using Tools::load_str;
   using Tools::to_string;

   using ToolsGUI::OperatorError;
   using ToolsGUI::erOperr;

   using PlayerMaster::pePidZero;
   using PlayerMaster::pePidBig;
   using PlayerMaster::pePidDel;
   using PlayerMaster::pePidBad;
   using PlayerMaster::peReg;
   using PlayerMaster::peName;
   using PlayerMaster::peGrade;
   using PlayerMaster::peClub;
   using PlayerMaster::peLand;
   using PlayerMaster::peGroup;
   using PlayerMaster::peNote;
   using PlayerMaster::peCount;

   if(!Report) return false;

   // THIS ROUTINE IS CALLED WHEN USER CLICKS A CONTROL
   // WE SUPPRESS ERROR ALREADY REPORTED ON THIS CONTROL
   switch(Report)
   {
   	case pePidZero: case pePidBig:case pePidDel: case pePidBad:
      	if(cbox==cbPid) Report = 0;
      break;

      case peReg:  if(cbox==cbTick) Report = 0; break;
      case peName: if(cbox==cbName) Report = 0; break;
      case peGrade:if(cbox==cbGrade)Report = 0; break;
      case peClub: if(cbox==cbClub) Report = 0; break;
      case peLand: if(cbox==cbLand) Report = 0; break;
      case peGroup:if(cbox==cbGroup)Report = 0; break;
      case peNote: if(cbox==cbNote) Report = 0; break;

      default:	Report = peCount;
   }
   if(!Report) return false;

   // SHOW THE VALID RANGE FOR PLAYER NUMBERS
   if(Report == pePidBig)
   {
   	const int nextpid = EditPlayerTube.PlayerInfo;

   	const std::string prange = load_str(IDB_pePidBig) + to_string(nextpid);

      ShowMessage(erOperr, prange);

      IdBox->SetFocus();

      return true;
   }

   // SHOW THE ERROR MESSAGE AND RETURN FOCUS TO THE OFFENDING FIELD
   ShowMessage(erOperr, load_str(IDB_peNoError + Report));

   switch(Report)
   {
   	case pePidZero: case pePidDel: case pePidBad:
      	IdBox->SetFocus();
      break;

      case peReg:TickBox->SetFocus(); 			 	break;
      case peName:NameBox->Editor->SetFocus();	break;
      case peGrade:GradeBox->SetFocus();			break;
      case peClub:ClubBox->SetFocus();				break;
      case peLand:LandBox->SetFocus();				break;
      case peGroup:GroupBox->SetFocus();			break;
      case peNote:NoteBox->SetFocus();				break;

      default: RestoreBtn->SetFocus();
   }
   return true;
}

bool __fastcall TEditPlayerFrm::MissingFields(void)
{
	using ToolsGUI::WhiteSpace;
   using ToolsGUI::OperatorError;

	if(WhiteSpace(IdBox->Text))
   {
   	OperatorError(IDB_MissingPid);  IdBox->SetFocus();  return true;
   }

   if(WhiteSpace(NameBox->Editor->Text))
   {
   	OperatorError(IDB_MissingName);  NameBox->Editor->SetFocus();  return true;
   }

   if(WhiteSpace(GradeBox->Text))
   {
   	OperatorError(IDB_MissingGrade);  GradeBox->SetFocus();  return true;
   }

	if(WhiteSpace(ClubBox->Text))
   {
   	OperatorError(IDB_MissingClub);  ClubBox->SetFocus();  return true;
   }

   if(WhiteSpace(LandBox->Text))
   {
   	OperatorError(IDB_MissingLand);  LandBox->SetFocus();  return true;
   }

   return false;
}


bool __fastcall TEditPlayerFrm::ValidPlayerEdit(void)
{
	// THIS ROUTINE FORCES ERROR DETECTION WHEN CLICKING ON RegGrid

	// SOMETHING MUST HAVE BEEN MODIFIED FOR THERE TO BE ANY ERROR
	if(!Modified) return true;

   // SOME FIELDS CAN BE LEGITIMATELY CLEARED, BUT NOT ALL
	if(MissingFields()) return false;

   // THE NAME FIELD WAS MODIFIED
   GNameEd* Editor =  dynamic_cast<GNameEd*>(ModifiedBox);

   if(Editor && Editor == NameBox->Editor)
   {
      if(!ValidName(Editor->Text))
      {
      	UserError(cbValidEdit);

         return false;
      }
      else
      	return true;
   }

   // THE TickBox MMSPin, OR GameGrid CAN BE MODIFIED WITHOUT ERROR
   TComboBox* Combo = dynamic_cast<TComboBox*>(ModifiedBox);

   if(!Combo) return true;

   // SO, SOME COMBO WAS MODIFIED
   while(true)
   {
   	if(Combo==IdBox)   { ValidPid(IdBox->Text);    break; }
      if(Combo==ClubBox) { ValidClub(ClubBox->Text); break; }
      if(Combo==LandBox) { ValidLand(LandBox->Text); break; }
      if(Combo==NoteBox) { ValidNote(NoteBox->Text); break; }

      if(Combo==GradeBox){ NewGrade(GradeBox->Text); break; }
      if(Combo==GroupBox){ NewGroup(GroupBox->Text); break; }

      Report = PlayerMaster::peCount;					  break;
   }

   if(Report)
   {
   	UserError(cbValidEdit);		return false;
   }
   else
   	return true;
}

bool __fastcall TEditPlayerFrm::InvalidPlayerEditMessage(void)
{
	using Tools::load_str;
	using ToolsGUI::WhiteSpace;
   using ToolsGUI::OperatorError;
   using ToolsGUI::erOperr;

	// SOMETHING MUST HAVE BEEN MODIFIED FOR THERE TO BE ANY ERROR
	if(!Modified) return false;

   // THE NAME FIELD WAS MODIFIED
   GNameEd* Editor =  dynamic_cast<GNameEd*>(ModifiedBox);

   if(Editor && Editor == NameBox->Editor)
   {
   	if(WhiteSpace(NameBox->Editor->Text))
   	{
   		OperatorError(IDB_MissingName);	return true;
   	}

      if(!ValidName(Editor->Text))
      {
      	ShowMessage(erOperr, load_str(IDB_peNoError + Report));

         return true;
      }
      else
      	return false;
   }


   // THE TickBox, MMSPin, OR GameGrid CAN BE MODIFIED WITHOUT ERROR
   TComboBox* Combo = dynamic_cast<TComboBox*>(ModifiedBox);

   if(!Combo) return false;

   // SO, SOME COMBO WAS MODIFIED - WE REPORT ON MISSING FIELDS OR ERROR
   int IDBmissing = 0;

   while(true)
   {
   	if(Combo==IdBox)
      {
      	if(WhiteSpace(IdBox->Text)) { IDBmissing = IDB_MissingPid; break; }

      	ValidPid(IdBox->Text);   break;
      }

      if(Combo==ClubBox)
      {
      	if(WhiteSpace(ClubBox->Text)) { IDBmissing = IDB_MissingClub; break; }

       	ValidClub(ClubBox->Text); break;
      }

      if(Combo==LandBox)
      {
      	if(WhiteSpace(LandBox->Text)) { IDBmissing = IDB_MissingLand; break; }

      	ValidLand(LandBox->Text); break;
      }

      if(Combo==GradeBox)
      {
      	if(WhiteSpace(GradeBox->Text)) { IDBmissing = IDB_MissingGrade; break; }

      	NewGrade(GradeBox->Text); break;
      }

      if(Combo==GroupBox){ NewGroup(GroupBox->Text); break; }
      if(Combo==NoteBox) { ValidNote(NoteBox->Text); break; }

      Report = PlayerMaster::peCount;					  break;
   }

   if(IDBmissing)
   {
   	OperatorError(IDBmissing);  return true;
   }

   if(Report == PlayerMaster::pePidBig)
   {
   	const int nextpid = EditPlayerTube.PlayerInfo;

   	const std::string prange = load_str(IDB_pePidBig) + to_string(nextpid);

      ShowMessage(erOperr, prange);

      return true;
   }

   if(Report)
   {
   	 OperatorError(IDB_peNoError + Report);  return true;
   }

   return false;
}

//---------------------------------------------------------------------------
// SUPPORT FOR MMS SPIN BUTTON
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::UpdateMMS(const int cbtag )
{
	using std::vector;
   using std::string;

   // WE NEED SLEEPING ROUNDS FOR CURRENT McScore
   vector<bool>playing(EditPlayerTube.Rounds);
   GameGrid->HarvestPlay(playing);

   int mmi, mms;

   switch(cbtag)
   {
   	case cbGrade: 	case cbGroup: // A CHANGE IN GRADE OR GROUP MAY OVERRIDE MiScore
   		EditPlayerTube.mmScores(GradeBox->Text, GroupBox->Text, mmi, mms);
      break;

      case cbMMS:	case cbGame:	// A CHANGE IN MiScore OR GAMES CHANGES McScore
      {
      	mmi = MMSpin->Position;

   		EditPlayerTube.mcScore(GradeBox->Text, GroupBox->Text, mmi, mms);
      }
      break;

      default: return;
   }

   SetSpinScores(mmi, mms);
}

void __fastcall TEditPlayerFrm::SetSpinScores(const int mmi, const int mms)
{
	using Tours::mms_ustring;
   using Tools::s_S;

	MMSpin->Position = mmi;

   iMMSText->Caption = s_S(mms_ustring(mmi));
   cMMSText->Caption = s_S(mms_ustring(mms));
}

//---------------------------------------------------------------------------
// PROCESS THE EDIT
//---------------------------------------------------------------------------
enum { NewPlayer = true, KnownPlayer = false };

bool __fastcall TEditPlayerFrm::MakePlayer(void)
{
	using std::string;

	using ToolsGUI::ShowMessage;
   using ToolsGUI::erOperr;
   using ToolsGUI::erInfo;
   using Tools::load_str;
   using Tools::to_string;
   using Tools::sCrLf;

   using PlayerMaster::epiNew;
   using PlayerMaster::MEditPlayerTube;

   // PARSE THE TUBE DATA, SETTING Player FIELDS IF ALL OK
   const int ReportPlayer = EditPlayerTube.MakePlayer();

   // ON ANY ERROR REFOCUS TO ERROR FIELD
   if(ReportPlayer == PlayerMaster::peNameDup)
   {
      ShowMessage(erOperr, EditPlayerTube.DuplicationError());

      NameBox->Editor->SetFocus();

      EditPlayerTube = PlayerMaster::SavedPlayerTube;

      Load();

      return false;
   }

   if(ReportPlayer == PlayerMaster::pePidBig)
   {
   	const int nextpid = EditPlayerTube.PlayerInfo;

   	const std::string prange = load_str(IDB_pePidBig) + to_string(nextpid);

      ShowMessage(erOperr, prange);

      IdBox->SetFocus();

      return false;
   }

   if(ReportPlayer)
   {
   	int ErrorReport = IDB_MAKEPLAYER + PlayerMaster::peCount;

   	if(ReportPlayer == PlayerMaster::peGroup)
      	ErrorReport = IDB_GROUPS + EditPlayerTube.GroupError;
      else
      	ErrorReport = IDB_MAKEPLAYER + ReportPlayer;

   	ShowMessage(erOperr, load_str(ErrorReport));

      switch(ReportPlayer)
      {
         case PlayerMaster::pePidZero: IdBox->SetFocus();				break;
         case PlayerMaster::pePidBad:	OKBtn->SetFocus();				break;
         
         case PlayerMaster::peReg:		TickPanel->SetFocus(); 			break;
         case PlayerMaster::peName:		NameBox->Editor->SetFocus();	break;
         case PlayerMaster::peGrade:	GradeBox->SetFocus(); 			break;
         case PlayerMaster::peClub:   	ClubBox->SetFocus(); 			break;
         case PlayerMaster::peLand:   	LandBox->SetFocus(); 			break;

         case PlayerMaster::peGroup:  	GroupBox->SetFocus(); 			break;
         case PlayerMaster::peNote:   	NoteBox->SetFocus(); 			break;
         case PlayerMaster::peMMSi:   	MMSpin->SetFocus(); 				break;

         default:	OKBtn->SetFocus();
      }

      return false;
   }

   // EPILOGUE
   TourMaster::GoTour.PopulationCount();

	if(EditPlayerTube.State == epiNew)
   {
   	// LOAD THE NEW PLAYER TO THE REGISTER
      ::RegGridLoadPlayer(&EditPlayerTube.Player, EditPlayerTube.PlayerRow);

      // UPDATE RegGRid SHOWING NEW PLAYER AT END OF REGISTER
      ::RegGridEndPlayerEdit(NewPlayer);
   }
   else
   {
   	// UPDATE RegGRid SHOWING EDITED PLAYER
      ::RegGridEndPlayerEdit(KnownPlayer);
   	::RegGridLoadPlayer(&EditPlayerTube.Player, EditPlayerTube.PlayerRow);
   }

   // ADD ANY NEW ITEMS TO THE COMBO LISTS
   EditPlayerTube.UpdateLists
   (
   	IdBox->Items, NameBox->ListBox->Items,
      ClubBox->Items, LandBox->Items,
      NoteBox->Items
   );

   string infomsg  = string();
   string grademsg = string();

   if(EditPlayerTube.GradeChanged)
   	if(EditPlayerTube.Player.Playing(EditPlayerTube.Round))
   		grademsg = load_str(IDB_PLAYERINFO+RegMaster::prGradeChanged);

   if(EditPlayerTube.PlayerInfo)
   {
   	switch(EditPlayerTube.PlayerInfo)
      {
      	case RegMaster::prJoin: TickBox->Reg = GTickBox::rtJoin; break;
         case RegMaster::prLeave: TickBox->Reg = GTickBox::rtLeave; break;
         case RegMaster::prRestored: TickBox->Reg = GTickBox::rtReg; break;
         case RegMaster::prFullyRestored: TickBox->Reg = GTickBox::rtReg; break;

         default: break;
      }

      TickBox->Repaint();

      infomsg = load_str(IDB_PLAYERINFO+EditPlayerTube.PlayerInfo);
   }

   string editmsg = string();

   if(!infomsg.empty() && !grademsg.empty())
   	editmsg = infomsg + string(sCrLf) + grademsg;
   else
   	editmsg = infomsg + grademsg;

   if(!editmsg.empty())
   	ShowMessage(erInfo, editmsg);

   return true;
}


void __fastcall TEditPlayerFrm::ChangeToPlayer(const int playrow)
{
   using ToolsGUI::OperatorWarning;

	// THERE IS A REQUEST TO CHANGE TO player FOR EDITING
   // IF CURRENT PLAYER TEXT HAS CHANGED, USER MAY WANT TO
   // FIRST SAVE, AND THEN CHANGE OTHERWISE DONT CHANGE

   if(UserError(cbChangeTo)) return;

   if(Modified)
   {
      if(MissingFields()) return;

   	bool CancelChange = OperatorWarning(IDB_PlayerChanged);

      if(!CancelChange)
      {
      	Save();

         ::BeginCommand(TourMaster::cmdEnter);
         	if(!MakePlayer()) CancelChange = true;
      	::EndCommand(TourMaster::cmdEnter);
      }

      if(CancelChange)
      {
         Load(IdBox,  EditPlayerTube.PlayerID);

         ForgeCaption();

         return;
      }
   }

   ::EditPlayer(playrow);

   ForgeCaption();
}

void __fastcall TEditPlayerFrm::ChangeToPlayer(const int playcol, const int playrow)
{
   PlayerMaster::EditPlayerTube.PlayerCol = playcol;

   ChangeToPlayer(playrow);
}

bool  __fastcall TEditPlayerFrm::UnsavedPlayerEdit(const int idmessage)
{
   using PlayerMaster::epiVirgin;

	// IS THE FORM ACTIVE
	if(EditPlayerTube.State==epiVirgin)
   	return false;

	// THERE MAY HAVE BEEN A USER TYPO
   if(!ValidPlayerEdit())
   	return true;

	if(PlayerMaster::EditPlayerTube.State != PlayerMaster::epiVirgin)
      if(Modified)
      {
      	if(idmessage)ToolsGUI::OperatorError(idmessage);

         if(OKBtn->Enabled)
   			FocusControl(OKBtn);
         else
      		FocusControl(HelpBtn);

         return true;
      }

   return false;
}

bool __fastcall TEditPlayerFrm::ClosedPlayerEdit(const int idmessage)
{
   using PlayerMaster::epiVirgin;

   // SAVE DATA IN THE TUBE IF USER WANTS TO
   if(EditPlayerTube.State==epiVirgin) return true;

   if(Modified)						
   {
      bool CancelChange = ToolsGUI::OperatorWarning(idmessage);

      if(!CancelChange)
      {
         Save();

         ::BeginCommand(TourMaster::cmdEnter);
            if(!MakePlayer()) CancelChange = true;
         ::EndCommand(TourMaster::cmdEnter);
      }

      if(CancelChange)
      {
         Load(IdBox,  EditPlayerTube.PlayerID);	return false;
      }
   }

   Close();

   return true;
}


//---------------------------------------------------------------------------
// EDIT CONTROL
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::OnOKBtnClick(TObject *Sender)
{
   using PlayerMaster::epiNew;

   // THERE MAY HAVE BEEN A USER TYPO
   if(UserError(cbOkClick))
   	return;

   // ENSURE MANDATORY FIELDS FILLED
   if(MissingFields())
   	return;

   // SAVE USER'S DATA IN THE TUBE
   if(Modified)
   	Save();
   else
   {
   	if(CloseBox->Checked)Close();

   		return;
   }


   // PROCESS THE PLAYER DATA
   bool ProcessedPlayerOK = false;

   ::BeginCommand(TourMaster::cmdEnter);
   	ProcessedPlayerOK = MakePlayer();
   ::EndCommand(TourMaster::cmdEnter);

   if(ProcessedPlayerOK)
   {
      if(EditPlayerTube.State == epiNew)
      {
         // RESET FOR A NEW PLAYER
         EditPlayerTube.LoadPlayer(2, EditPlayerTube.PlayerRow+1);

         Load();

         NameBox->Editor->SetFocus();
      }
      else
      	IdBox->SetFocus();

      Modified = 0;

      ::ReloadPairResult();

      ForgeCaption();
   }

   if(CloseBox->Checked)
      Close();
}

void __fastcall TEditPlayerFrm::OnCancelBtnClick(TObject *Sender)
{
	// RELOAD THE ENTRY PLAYER - IDENTIFIED BY PlayerRow
   Load();

   Report = 0;

   Update();
}

void __fastcall TEditPlayerFrm::OnHelpBtnClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_EDPLAY); }catch(...){ }
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// FORM CLOSING
//---------------------------------------------------------------------------
void __fastcall TEditPlayerFrm::OnCloseBtnClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TEditPlayerFrm::OnFormClose(TObject *Sender, TCloseAction &Action)
{
	using ToolsGUI::ShowMessage;
   using ToolsGUI::erWarn;
   using Tools::load_str;

   using PlayerMaster::MEditPlayerTube;
	using PlayerMaster::EditPlayerTube;
   using PlayerMaster::epiVirgin;

   // CLOSE THE HELP IF OPEN
   try{ Application->HelpCommand(HELP_QUIT, 0); }catch(...){ }

   // NOW TRY TO CLOSE THE FORM
	// The form is NOT allowed to close if
   // 1. OK was pressed and Do not close on OK was checked
   // 2. OK was pressed and there is an error in some field

   // The form IS allowed to close if
   // The Close button was pressed
   // The widows x menu was pressed
   // The 'Do not close on OK' box was NOT checked

   // We need to check that user is not in the middle of editing.


   if(EditPlayerTube.State != epiVirgin)
      if(Modified)
         switch(ShowMessage(erWarn, load_str(IDB_Editing)))
         {
            case IDOK: 		ModalResult = mrOk; break;

            case IDCANCEL: ModalResult = mrNone; break;

            default: 		ModalResult = mrOk;
         }
      else
         ModalResult = mrOk;
   else
   	ModalResult = mrOk;


	if(ModalResult == mrOk)
   {
   	// IT IS OK TO CLOSE THE FORM.
      // WE NEVER ACTUALLY CLOSE IT TO FREE ITS MEMORY, JUST MAKE IT INVISIBLE

      EditPlayerTube = MEditPlayerTube(); // Clear everything
   	EditPlayerTube.State = epiVirgin;

      if(ReadOnly)
      {
      	ReadOnly = false;		// GK 6.3.9 Allow next invocation of EditPlayer to modify

      	ClearGridSelections();	// GK 6.3.9 Pairgrid and ResGrid
      }
      
      Report = 0;

      Hide();

      Action = caNone;

      ::SetRegGridStatusDormant();
   }
   else
   	// IT IS NOT OK TO CLOSE THE FORM - CARRY ON EDITING
      Action = caNone;
}
//---------------------------------------------------------------------------






