//---------------------------------------------------------------------------
#ifndef EditPlayerFormH
#define EditPlayerFormH

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <ExtCtrls.hpp>
#include <Grids.hpp>
#include <Buttons.hpp>

#include "Maths.h"
#include "EditPlayerGUI.h"

//---------------------------------------------------------------------------
// THE EDIT PLAYER FORM
//---------------------------------------------------------------------------
enum
{
	cbValidEdit=-4, cbChangeTo=-3, cbOkClick=-2, cbUnsaved=-1,
	cbPid, cbTick,
   cbName, cbGrade, cbClub, cbLand,
   cbGroup, cbNote, cbMMS,  cbGame,
   cbCount
};

class TEditPlayerFrm : public TForm
{
__published:	// IDE-managed Components
	TGroupBox *PlayerGrpBox;
	TButton *CloseBtn;
	TButton *HelpBtn;
	TScrollBar *RegScrollBar;
	TButton *RestoreBtn;
	TButton *OKBtn;
	TComboBox *IdBox;
	TComboBox *ClubBox;
	TComboBox *LandBox;
	TComboBox *GroupBox;
	TComboBox *GradeBox;
	TStaticText *cMMSText;
	TStaticText *Tie1Text;
	TStaticText *Tie2Text;
	TLabel *iMMSLbl;
	TLabel *cMMSLbl;
	TLabel *Tie1Lbl;
	TLabel *Tie2Lbl;
	TPanel *BalPanel;
	TLabel *OutBalLbl;
	TLabel *UDBalLbl;
	TLabel *ColBalLbl;
	TStringGrid *GameGridVCL;
	TCheckBox *AutoCompBox;
	TCheckBox *CloseBox;
	TComboBox *NoteBox;
	TLabel *ClubLbl;
	TLabel *LandLbl;
	TLabel *GroupLbl;
	TLabel *NoteLbl;
	TLabel *PidLbl;
	TLabel *RegLbl;
	TLabel *NameLbl;
	TLabel *GradeLbl;
	TPanel *TickPanel;
	TStaticText *iMMSText;
	TStaticText *BehaveLbl;
	TTrackBar *RegTrackBar;
	TUpDown *MMSpin;
	void __fastcall OnFormShow(TObject *Sender);
	void __fastcall OnFormClose(TObject *Sender, TCloseAction &Action);
	void __fastcall OnCloseBtnClick(TObject *Sender);

   void __fastcall OnClickCombo(TObject *Sender);
	void __fastcall OnChangeComboText(TObject *Sender);
	void __fastcall OnEnterCombo(TObject *Sender);
	void __fastcall OnExitCombo(TObject *Sender);
	void __fastcall OnComboKeyDown(TObject *Sender, WORD &Key, TShiftState Shift);

   void __fastcall OnOKBtnClick(TObject *Sender);
	void __fastcall OnCancelBtnClick(TObject *Sender);

	void __fastcall OnScrollEvent(TObject *Sender, TScrollCode ScrollCode, int &ScrollPos);
	void __fastcall OnTrackChange(TObject *Sender);
	void __fastcall OnTrackKeyUp(TObject *Sender, WORD &Key, TShiftState Shift);
	void __fastcall OnClickMMSpin(TObject *Sender, TUDBtnType Button);
	void __fastcall OnHelpBtnClick(TObject *Sender);

private:								// AUTOCOMPLETION MANAGEMENT
	bool TailDeleted;

   void FindInList(TComboBox* combo, const String text);
   void FindExactInList(TComboBox* combo, const String text);

   void EscapeCombo(TComboBox*);
	void SelectText(TComboBox* combo, const int start, const int length);
   void SetItemIndex(TComboBox* combo, const int itemindex);
   bool TextModified(TComboBox* combo);

   void DrawNamePip(void);

public:
   bool ReadOnly;		  		// No Editing allowed, just scrolling
   
   Tools::bits Modified;	// A bit is set when any field modified - detected on exit
   TObject* ModifiedBox;   // The control that was modified
   int Report;					// Internal User error mechanics
   int GroupError;			// Group error details

   String GradeEntryText;	// Prevent spurious MMS updates when the text has not changed
   String GroupEntryText;  // since the last access to GradeBox or GroupBox

	GNameBox*  NameBox;
   GTickBox*  TickBox;
   GGameGrid* GameGrid;

	__fastcall TEditPlayerFrm(TComponent* Owner);
   __fastcall ~TEditPlayerFrm();

   void __fastcall Update(void);

   void __fastcall ForgeCaption();
   void __fastcall LoadLists(void);
   void __fastcall UpdateLists(void);
   void __fastcall ConnectFindTubeLists(void);
   
   void __fastcall Load(void);
   void __fastcall Save(void);
   void __fastcall SetAction(const bool active);

   bool __fastcall TextChanged(void);

   bool __fastcall ValidPid(const String Pid);
   bool __fastcall ValidName(const String Name);
   void __fastcall NewGrade(const String Grade);
   bool __fastcall ValidClub(const String Club);
   bool __fastcall ValidLand(const String Land);
   void __fastcall NewGroup(const String Group);
   bool __fastcall ValidNote(const String Note);

   bool __fastcall UserError(const int cbox);
   bool __fastcall MissingFields(void);

   void __fastcall UpdateMMS(const int combotag);
   void __fastcall SetSpinScores(const int mmi, const int mms);

   void __fastcall SetScrollBarParams(const int p, const int min, const int max, const int pagesize);
   void __fastcall SetScrollBarPos(const int pos);
   void __fastcall SetTrackBarParams(const int pos, const int min, const int max, const int pagesize);
   void __fastcall SetTrackBarPos(const int pos);

   bool __fastcall MakePlayer(void);
   void __fastcall ChangeToPlayer(const int playrow);
   void __fastcall ChangeToPlayer(const int playcol, const int playrow);
   void __fastcall SetBoxFocus(void);

   bool __fastcall ValidPlayerEdit(void);
   bool __fastcall InvalidPlayerEditMessage(void);
   bool __fastcall UnsavedPlayerEdit(const int idmessage);
   bool __fastcall ClosedPlayerEdit(const int idmessage);

   void __fastcall IdBoxKeyDown(WORD &Key, TShiftState Shift);

   void Load(TComboBox* combo, const String text);
   void LoadExact(TComboBox* combo, const String text);
   void SetText(TComboBox* combo, const String text);
   void UpdateModified(const int idcontrol, const bool modified);
   void ClearUnknownPlayerFields(void);
};
//---------------------------------------------------------------------------
#endif
