////////////////////////////////////////////////////////////////////////////////
// MODULE EditPlayerGUI: Support classes for EditPlayerForm  G.Kaniuk 15/02/08
//

#include <vcl.h>
#include <winuser.h>
#include <ComCtrls.hpp>
#pragma hdrstop

#include "GraphicsAPI.h"
#include "Strings.h"
#include "Format.h"
#include "StringsGUI.h"
#include "ETour.h"
#include "EdPlayerRES.h"
#include "PlayerMaster.h"
#include "RegMaster.h"

#include "EditPlayerForm.h"

// extern void __fastcall EditPlayer(const int player);
// extern void __fastcall ShowMsg(const char* msg);

using PlayerMaster::EditPlayerTube;

//---------------------------------------------------------------------------
// THE REGISTERED CONTROL
//---------------------------------------------------------------------------
__fastcall GTickBox::GTickBox(TWinControl* parent)             // CLASS GTickBox
:TCustomControl(parent->Owner)
{
   Visible    	= false;
   Parent     	= parent;
   Width 		= Parent->Width - 2;
   Height		= Parent->Height - 2;
   Top         = 1;
   Left        = 1;

   Hint       	= LoadStr(IDB_RegHint);
   ShowHint   	= true;
   TabStop    	= true;
   TabOrder   	= 0;

   OnClick   	= OnClickTick;
   OnEnter   	= OnEnterTick;
   OnExit		= OnExitTick;

   EditPlayerForm = (TEditPlayerFrm*)(Parent->Owner);

   for(int ft=0; ft<ftCount; ft++)
   for(int rt=0; rt<rtCount; rt++)
   {
   	Ticks[ft][rt] = new Graphics::TBitmap();
      Ticks[ft][rt]->Transparent = true;
   }

   Reg 	   = rtNoReg;
   Selected = ftFree;

   Clicked  = false;
   Entered = false;

	DrawTicks();
}

__fastcall GTickBox::~GTickBox()
{
	for(int ft=0; ft<ftCount; ft++)
   for(int rt=0; rt<rtCount; rt++)
   	delete Ticks[ft][rt];
}

const static TPoint TickPoints[6] =
{
   Point( 1,8), Point(6,13), Point(15,4),
   Point(13,2), Point(6, 9), Point( 3,6)
};

const static int wTick = 14, hTick = 11;

void GTickBox::DrawTick
(
   int Selected,						// Whether Free,Selected, or Gray
   int Registered,               // Tick or blob
   int oL, int oT,               // Offsets

   Graphics::TBitmap* BitMap
)
{
   TRect Area(0,0, BitMap->Width, BitMap->Height);

   TColor OldBrushColor = BitMap->Canvas->Brush->Color, clBrush;

   BitMap->TransparentColor     = Brush->Color;

   switch(Selected)
   {
   	case ftFree: 	clBrush = clWindow; 	  break;
      case ftSelect:	clBrush = clHighlight; break;
      case ftGrey:	clBrush = clGrayText;  break;

      default:			clBrush = clRed;
   }
   BitMap->Canvas->Brush->Color = clBrush;

   BitMap->Canvas->FillRect(Area);  Repaint();

   if(Registered)
   {
      const int TickH = hTick, TickW = wTick;

      int Left = (BitMap->Width - TickW)/2;    if(Left<0) Left = 0;
      int Top  = (BitMap->Height - TickH)/2;   if(Top<0)  Top  = 0;

      Left += oL;    Top += oT;

      TPoint TickPts[6];

      for(int k=0; k<6; k++)
      {
         TPoint P = TickPoints[k];

         TickPts[k] = Point(Left+(TickW*P.x)/wTick, Top+(TickH*P.y)/hTick);
      }

      TColor clTick;

      switch(Registered)
      {
         case GTickBox::rtReg:   clTick = clYellow; break;
         case GTickBox::rtJoin:  clTick = clBlack;  break;
         case GTickBox::rtLeave: clTick = clWhite;  break;

         default: clTick = clGreen;  break;
      }

      BitMap->Canvas->Brush->Color = clTick;
      BitMap->Canvas->Polygon(TickPts, 5);
   }
   else
   {
      int BlobW = (5*BitMap->Width/16), BlobH = (5*BitMap->Height)/16;

      if(BlobH<BlobW)
         BlobW = BlobH;
      else
         BlobH = BlobW;

      int Left = (BitMap->Width - BlobW)/2 + oL;
      int Top  = (BitMap->Height - BlobH)/2 + oT;

      BitMap->Canvas->Brush->Color = clYellow;
      BitMap->Canvas->Ellipse(Left, Top, Left+BlobW-1, Top+BlobH-1);
   }
   BitMap->Canvas->Brush->Color = OldBrushColor;
}

void GTickBox::DrawTicks(void)
{
   // Unregistered
   Ticks[ftFree][rtNoReg]->Width = Width;
   Ticks[ftFree][rtNoReg]->Height = Height;
   DrawTick(ftFree, rtNoReg, -3, -1, Ticks[ftFree][rtNoReg]);

   Ticks[ftSelect][rtNoReg]->Width = Width;
   Ticks[ftSelect][rtNoReg]->Height = Height;
   DrawTick(ftSelect, rtNoReg, -1, -1, Ticks[ftSelect][rtNoReg]);

   Ticks[ftGrey][rtNoReg]->Width = Width;
   Ticks[ftGrey][rtNoReg]->Height = Height;
   DrawTick(ftGrey, rtNoReg, -3, -1, Ticks[ftGrey][rtNoReg]);

   // Registered
   Ticks[ftFree][rtReg]->Width = Width-1;
   Ticks[ftFree][rtReg]->Height = Height-1;
   DrawTick(ftFree, rtReg, -3, -3, Ticks[ftFree][rtReg]);

   Ticks[ftSelect][rtReg]->Width = Width-1;
   Ticks[ftSelect][rtReg]->Height = Height-1;
   DrawTick(ftSelect,rtReg, -1, -3, Ticks[ftSelect][rtReg]);

   Ticks[ftGrey][rtReg]->Width = Width-1;
   Ticks[ftGrey][rtReg]->Height = Height-1;
   DrawTick(ftGrey,rtReg, -1, -3, Ticks[ftGrey][rtReg]);

   // Player Joining Draw
   Ticks[ftFree][rtJoin]->Width = Width;
   Ticks[ftFree][rtJoin]->Height = Height;
   DrawTick(ftFree, rtJoin, -2, -3, Ticks[ftFree][rtJoin]);

   Ticks[ftSelect][rtJoin]->Width = Width;
   Ticks[ftSelect][rtJoin]->Height = Height;
   DrawTick(ftSelect, rtJoin, -2, -3, Ticks[ftSelect][rtJoin]);

   Ticks[ftGrey][rtJoin]->Width = Width;
   Ticks[ftGrey][rtJoin]->Height = Height;
   DrawTick(ftGrey, rtJoin, -2, -3, Ticks[ftGrey][rtJoin]);

   // Player Leaving Draw
   Ticks[ftFree][rtLeave]->Width = Width;
   Ticks[ftFree][rtLeave]->Height = Height;
   DrawTick(ftFree, rtLeave, -2, -3, Ticks[ftFree][rtLeave]);

   Ticks[ftSelect][rtLeave]->Width = Width;
   Ticks[ftSelect][rtLeave]->Height = Height;
   DrawTick(ftSelect, rtLeave, -2, -3, Ticks[ftSelect][rtLeave]);

   Ticks[ftGrey][rtLeave]->Width = Width;
   Ticks[ftGrey][rtLeave]->Height = Height;
   DrawTick(ftGrey, rtLeave, -2, -3, Ticks[ftGrey][rtLeave]);
}

void __fastcall GTickBox::Paint(void)
{
	const TRect RegArea = TRect(0, 0, Parent->Width-1, Parent->Height-1);

   Canvas->FillRect(RegArea);

   if(Selected<0 || Selected >= ftCount) return;
   if(Reg<0 || Reg >= rtCount) return;

   Canvas->Draw(0, 0, Ticks[Selected][Reg]);
}

void __fastcall GTickBox::KeyDownSpecial(TCMWantSpecialKey& Message)
{
	switch(Message.CharCode)
   {
   	case VK_ESCAPE: TCustomControl::Dispatch((void*)&Message);  break;
      case VK_RETURN: TCustomControl::Dispatch((void*)&Message);  break;

      default:  Message.Result = 1;
   }
}

void __fastcall GTickBox::KeyDown(Word& Key, TShiftState Shift)
{
   switch(Key)
   {
   	case VK_ESCAPE:
      {
         Reg = PlayerMaster::EditPlayerTube.Reg;

         EditPlayerForm->UpdateModified(cbTick, false);
         EditPlayerForm->ModifiedBox = (TObject*)this;
         EditPlayerForm->Report = 0;

         Repaint();
      }
      break;

      case VK_RETURN:
      	EditPlayerForm->OKBtn->Click();
      break;

      case VK_TAB:
      {
      	Selected = ftFree;  

         if(Shift.Contains(ssShift))
         	EditPlayerForm->IdBox->SetFocus();
         else
         	EditPlayerForm->NameBox->Editor->SetFocus();
      }
      break;

      default:
      	// ANY KEY TOGGLES, BUT WE WANT TO BE ABLE TO
         // SHIFT+TAB AWAY FROM THE TICK BOX
         if(!Shift.Contains(ssShift))Click();
   }
   Key = 0;
}

void __fastcall GTickBox::OnClickTick(TObject* Sender)
{
   Clicked = true;	// We book the event for EnterTick

   SetFocus();			// This triggers previous control exit - who may discover error
   						// It then triggers EnterTick who can then deal with the error

   if(!Entered) return;

   // TOGGLE
   if(EditPlayerTube.Roundl == 0)
   {
      Reg = Reg==rtNoReg ? rtReg : rtNoReg;

      EditPlayerForm->UpdateModified(cbTick, Reg != EditPlayerTube.Reg);
      EditPlayerForm->ModifiedBox = (TObject*)this;
   }

	// SELECT THE TICK IMAGE
   Selected = ftSelect;  Repaint();
}

void __fastcall GTickBox::OnEnterTick(TObject *Sender)
{
	// SELECT THE TICK IMAGE
   if(!Clicked)
   {
   	SetFocus();
      Selected = ftSelect;  Repaint();
   }

   // TEST FOR USER ERROR ON SOME OTHER FIELD
   Entered = ! EditPlayerForm->UserError(cbTick);
}


void __fastcall GTickBox::OnExitTick(TObject *Sender)
{
   // IF ANY PREVIOUS ERROR, Report WILL ALREADY HAVE BEEN SET
   // AND WE ARE EXITING TO THE ERROR FIELD
   Entered = false;
   Clicked = false;
   
   if(EditPlayerForm->Report)
   	return;

   if(EditPlayerTube.InvalidReg(Reg))
   	EditPlayerForm->Report = PlayerMaster::peReg;

	// DESELECT THE TICK IMAGE
   Selected = ftFree;  Repaint();
}

//---------------------------------------------------------------------------
// THE PIP BUTTON
//---------------------------------------------------------------------------

__fastcall GNamePip::GNamePip(GNameBox* combo)
:TBitBtn(combo->Parent->Owner)
{
   Visible = false;

   Parent  = combo->Parent;
   Layout  = blGlyphLeft;
   Kind    = bkCustom;
   Style   = bsNew;
   Caption = String();
   Spacing = 0;
   Margin  = 1;
   TabStop = false;

   Combo   = combo;

   OnClick = OnClickPip;
}

void GNamePip::DrawPip(void)
{
	// DRAW DOWNWARD POINTING ARROW
   Glyph->Height = (3*Height)/17;
   Glyph->Width  = (5*Width)/13;

   Glyph->Canvas->Pen->Color = clBlack;

   Glyph->Canvas->MoveTo(             0, 0);
   Glyph->Canvas->LineTo(Glyph->Width  , 0);

   Glyph->Canvas->MoveTo(             1, 1);
   Glyph->Canvas->LineTo(Glyph->Width-1, 1);

   Glyph->Canvas->MoveTo(             2, 2);
   Glyph->Canvas->LineTo(Glyph->Width-2, 2);
}

void __fastcall GNamePip::OnClickPip(TObject *Sender)
{
	GNameList* PlayerListBox = Combo->ListBox;

   TEditPlayerFrm* EditPlayerForm = Combo->EditPlayerForm;

   if(!PlayerListBox) return;

   if(EditPlayerForm->Report)
      if(EditPlayerForm->Report != PlayerMaster::peName)
      	{
         	(void)EditPlayerForm->UserError(cbName);   return;
         }

   // USER FINISHED WITH LIST
   if(PlayerListBox->Visible)
   {
      PlayerListBox->Hide();

      Combo->Editor->SetFocus();

      return;
   }

   // A SELECTED ITEM IS POSITIONED AT THE TOP OF THE BOX
   if(PlayerListBox->ItemIndex >= 0)
   	PlayerListBox->TopIndex = PlayerListBox->ItemIndex;
   else
      PlayerListBox->TopIndex = 0;

   PlayerListBox->BringToFront();
	PlayerListBox->Show();
   PlayerListBox->SetFocus();
}

//---------------------------------------------------------------------------
// THE LIST BOX
//---------------------------------------------------------------------------
__fastcall GNameList::GNameList
(
	const int listcols[4], GNameBox* combo
)
:TCustomListBox(combo->Parent->Owner)
{
   Visible        = false;

   Parent         = combo->Parent;
   Combo          = combo;

   Style          = lbOwnerDrawFixed;
   IntegralHeight = true;
   Sorted         = false;
   TabStop        = true;
   
   Background     = Color;

   NavKey         = false;

   for(int i=0; i<4; i++) ListCols[i] = listcols[i];

   OnExit  = OnListExit;
   OnClick = OnListClick;

}
//-------------------------------------------------------------------

MESSAGE void __fastcall GNameList::CNDrawItem(TMessage& message)
{
	// THIS REMOVES THE FOCUS RECTANGLE FROM THE LIST BOX ITEM
   LPDRAWITEMSTRUCT drawitem = reinterpret_cast<LPDRAWITEMSTRUCT>(message.LParam);

   drawitem->itemState &= ~ODS_FOCUS;

   TCustomListBox::Dispatch(&message);
}

void __fastcall GNameList::DrawItem
(
   int index, const TRect& area, TOwnerDrawState drawstate
)
{
	using std::string;

	// DRAW A SOLID HIGHLIGHTED BAR ACROSS ALL COLUMNS FOR SELECTED ITEM
   TColor clFont  = Canvas->Font->Color;

   if(drawstate.Contains(odSelected))
   {
      if(clFont==clHighlight)
         Canvas->Font->Color = TColor(clHighlight/2 + 2);

      Canvas->Brush->Color = clHighlight;
   }
   Canvas->FillRect(area);

   // UNPACK THE TABBED TEXT AND DRAW EACH TEXT FIELD
   const int AreaTop = area.top + 2, AreaLeft = area.left + 2;

   const String& Itemi = Items->Strings[index];

   try
   {
   	using std::string;
      using Tools::s_S;
      using Tools::S_s;
      using Tools::unpack_tabbed_exact;

      string packed = S_s(Itemi), fields[4];

      const int F = unpack_tabbed_exact(packed, 4, fields);

      if(F==4)
      	for(int f=0; f<4; f++)
         	Canvas->TextOut(AreaLeft+ListCols[f], AreaTop, s_S(fields[f]));
   	else
      	Canvas->TextOut(AreaLeft, AreaTop, Itemi);
   }
   catch(...)
   {
      Canvas->TextOut(AreaLeft, AreaTop, Itemi);
   }

   if(drawstate.Contains(odSelected))
   {
      Canvas->Brush->Color = Background;
      Canvas->Font->Color  = clFont;
   }
}
//---------------------------------------------------------------------------

void __fastcall GNameList::KeyDownSpecial(TCMWantSpecialKey& Message)
{
	switch(Message.CharCode)
   {
   	case VK_ESCAPE: TCustomListBox::Dispatch((void*)&Message);  break;
      case VK_RETURN: TCustomListBox::Dispatch((void*)&Message);  break;

      default:  Message.Result = 1;
   }
}

void __fastcall GNameList::KeyDown(Word &Key,TShiftState Shift)
{
   NavKey = false;

   switch(Key)
   {
      case VK_ESCAPE:
      {
         NavKey = true;		Close();
      }
      break;

      case VK_RETURN:
         Click();
      break;

      case VK_TAB:
      {
      	TEditPlayerFrm* Form = Combo->EditPlayerForm;

         Click();

         if(Shift.Contains(ssShift))
         	Form->TickPanel->SetFocus();
         else
         	Form->GradeBox->SetFocus();
      };
      break;

      case VK_UP: case VK_DOWN:
      {
      	NavKey = true;

         TCustomListBox::KeyDown(Key, Shift);  // Triggers click

         SetFocus();
      }
      return;

      default:	TCustomListBox::KeyDown(Key, Shift); return;
   }

   Key = 0;
}
//---------------------------------------------------------------------------

void __fastcall GNameList::OnListClick(TObject* Sender)
{
	GNameEd* Editor = Combo->Editor;

   TEditPlayerFrm* EditPlayerForm = Combo->EditPlayerForm;

	if(NavKey)
   {
      NavKey = false;	return;
   }

   if(Visible)Close();

   // SET PLAYER DETAILS INCLUDING MMS
   UnpackItem();

   Editor->SelectText(0, Editor->Text.Length());

   // CHECK THE NAME PICKED FROM THE LIST
   // THE NAME MAY GET MODIFIED IF THE LIST HAD INCORRECT
   // CAPITALISATION FOR EXAMPLE
   String Name = Editor->Text;

   EditPlayerForm->Report == EditPlayerTube.ValidName(Name);

   if(!EditPlayerForm->Report)Combo->Load(Name);
}

void __fastcall GNameList::OnListExit(TObject* Sender)
{
	Close();
}

void GNameList::SelectItem(void)
{
   Close();
}

void GNameList::UnpackItem(void)
{
	using Tools::s_S;
   using Tools::S_s;
   using Tools::bits;
   using Tools::set_bit;

   using std::string;
   using PlayerMaster::epiNew;
   using PlayerMaster::epiExists;

	TEditPlayerFrm* Form = Combo->EditPlayerForm;

	const string item = S_s(Items->Strings[ItemIndex]);

   enum 																			// V6.5.0: Duplicated from PlayerMaster.cpp
   {
   	hdName, hdGrade, hdClub, hdLand, hdCount, hdRate = hdCount, hdMax = hdCount+1
   };

	string items[hdMax];	const int nused =  Tools::unpack_tabbed(item, hdMax, items);

   if(nused<hdCount)															// Some missing fields
   	for(int i=nused; i<hdCount; i++) items[i]= string("?");

   const bool rated = nused==hdMax;

   bits Modified = 0;

   // WE SET ALL FIELDS FOR A NEW PLAYER
   if(EditPlayerTube.State == epiNew)
   {
      Form->NameBox->Editor->SetText(s_S(items[hdName]));

      Form->Load(Form->GradeBox, s_S(items[hdGrade]));
      Form->Load(Form->ClubBox, s_S(items[hdClub]));
      Form->Load(Form->LandBox, s_S(items[hdLand]));

      Form->SetText(Form->GroupBox, String());
      Form->SetText(Form->NoteBox, String());
      Form->NewGrade(s_S(items[hdGrade]));

      if(rated)
      {
      	const string lan("<"),ran(">"),lsq("["),rsq("]"),sp(" ");

      	const string lgrade = lan+items[hdGrade]+ran; 			// Pervious tournament grade
      	const string sgor   = items[hdRate];						// Current rating

         string strength, tgrade;

         if(RegMaster::make_rated_grade(sgor, strength, tgrade))
         {
         	const string srated = lsq+strength+rsq + sp;

      		Form->SetText(Form->NoteBox, s_S(srated+lgrade));

            Form->Load(Form->GradeBox, s_S(tgrade));
         }
      }

      set_bit(cbName,  true, Modified);
      set_bit(cbGrade, true, Modified);
      set_bit(cbClub,  true, Modified);
      set_bit(cbLand,  true, Modified);
   }

   // FOR AN EXISTING PLAYER, WE CHANGE THE NAME ONLY
   if(EditPlayerTube.State == epiExists)
   {
      Combo->Editor->SetText(s_S(items[0]));

      set_bit(cbName,  true, Modified);
   }

   Form->Modified = Modified;
}

bool GNameList::Find(const String text)
{
   const int TextLength = text.Length();
   const String utext   = text.UpperCase();

   if(TextLength==0)		// We do not need to search for empty text
   	return false;

   // ItemIndex MAY ALREADY BE SET TO IDENTIFY text IN ITEMS
   if(ItemIndex>=0)
   {
   	const String& ItemText = Items->Strings[ItemIndex].UpperCase();

      if(ItemText.Pos(utext)==1)
      	return true;
   }

   // SEARCH FOR utext EMBEDDED AT BEGINNING OF ITEMS
	int Itemi = -1;

   for(int i=0; i<Items->Count; i++)
   {
      const String ItemText = Items->Strings[i].UpperCase();

      if(TextLength <= ItemText.Length())			// Possible substring
         if(ItemText.Pos(utext)==1)					// So ItemText begins with text
         {
          	Itemi = i;		 break;					// Found
         }
   }

   ItemIndex = Itemi;

   return Itemi >= 0;
}

void GNameList::SetSorted(bool sorted)
{
   Sorted = sorted;
}

void GNameList::Close(void)
{
	Hide();

   Combo->Editor->SetFocus();
}

void GNameList::Nudge(int dir)
{
	int Itemi = ItemIndex;

	if(dir == 0) return;

   if(Itemi < 0)
   {
   	if(dir<0) Itemi = 0;
      if(dir>0) Itemi = Items->Count-1;
   }
   else
   {
   	Itemi += dir;

      if(Itemi<0) 				Itemi = 0;
      if(Itemi>=Items->Count) Itemi = Items->Count-1;
   }

   ItemIndex = Itemi;
}

void GNameList::Page(int dir)
{
	const int ListEnd = Items->Count-1;

	int Topi = TopIndex, Boti, Itemi = ItemIndex;

	if(dir == 0) return;

   if(Itemi < 0)
   {
   	Topi = 0;
      Itemi = 0;
   }
   else
   {
   	if(dir<0)
      {
      	Boti = Itemi;

      	Topi = Boti-VisibleItems+1;

         if(Topi<0) Topi = 0;

         Itemi = Topi;
      }

      if(dir>0)
      {
      	Topi = Itemi;

         Boti = Topi+VisibleItems-1;

         if(Boti>ListEnd) Boti = ListEnd;

         Itemi = Boti;
      }
   }

   TopIndex  = Topi;
   ItemIndex = Itemi;
}

//---------------------------------------------------------------------------
// THE EDITOR
//---------------------------------------------------------------------------
__fastcall GNameEd::GNameEd(GNameBox* combo)
:TEdit(combo->Parent->Owner)
{
	Visible = false;

	Parent = combo->Parent;

   Text  = String();

   TabStop = true;

   OnChange  = OnChangeText;
   OnClick   = OnClickEditor;
   OnEnter   = OnEnterEditor;
   OnExit    = OnExitEditor;
   OnKeyDown = OnKeyDownEditor;

   TailDeleted		  = false;
   RequireSelectAll = false;

   Combo = combo;
}

void GNameEd::SelectText(const int start, const int length)
{
	SelStart = start>=0 ? start : 0;
   SelLength = length>=0 ? length : 0;
}

void GNameEd::SetText(const String text)
{
	const bool OnChangeHandled = OnChange != 0;

 	OnChange = 0;				 			// Setting Text will trigger OnChange!

   Text = text;							// Initialise the Editor

   if(OnChangeHandled)
   	OnChange = OnChangeText;		// Allow user to change now
}

void GNameEd::Escape(void)			// USER ESCAPES AFTER TYPING IN EDITOR
{
   Combo->Load(EditPlayerTube.Name);

   SelectText(Text.Length(), 0);

   Combo->EditPlayerForm->UpdateModified(cbName, false);
   Combo->EditPlayerForm->ModifiedBox = (TObject*)this;
   Combo->EditPlayerForm->Report = 0;
}

void __fastcall GNameEd::TEditKeyDown(WORD &Key, TShiftState Shift)
{
   // KeyDown CALLS THE HANDLER ATTACHED TO OnKeyDown
   // SO TO USE THE BASE TEDit KeyDown,
   // WE MUST FIRST DETACH OUR HANDLER

   bool OnKeyDownHandled = OnKeyDown != 0;

   OnKeyDown = 0;	TEdit::KeyDown(Key, Shift);

   if(OnKeyDownHandled)
      OnKeyDown = OnKeyDownEditor;
}

void __fastcall GNameEd::OnKeyDownEditor(TObject *Sender, WORD &Key, TShiftState Shift)
{
	TEditPlayerFrm* Form = Combo->EditPlayerForm;

   switch(Key)
   {
      case VK_ESCAPE:  Escape();  				break;

      case VK_RETURN:  Form->OKBtn->Click(); break;

      case VK_UP: case VK_DOWN:
      {
      	int dir = 0;

         if(Key==VK_UP)   dir = -1;
         if(Key==VK_DOWN) dir =  1;

         Combo->ListBox->Nudge(dir);
         Combo->ListBox->Click();
      }
      break;

      case VK_PRIOR: case VK_NEXT:
      {
      	int dir = 0;

         if(Key==VK_PRIOR)dir = -1;
         if(Key==VK_NEXT) dir =  1;

         Combo->ListBox->Page(dir);
         Combo->ListBox->Click();
      }
      break;

      case VK_DELETE: case VK_BACK:
      {
         TailDeleted = SelStart+SelLength == Text.Length();

         TEditKeyDown(Key, Shift);

         return;
      }

      default:
      {
         TEditKeyDown(Key, Shift);

         return;
      }
   }

   Key = 0;
}

void __fastcall GNameEd::OnClickEditor(TObject *Sender)
{
   if(Combo->EditPlayerForm->UserError(cbName)) return;
   
	bool Matched;

	int Item = ToolsGUI::FindNearest(Text, Combo->ListBox->Items, Matched);

	Combo->ListBox->ItemIndex = Item;

   if(RequireSelectAll)
   {
   	SelectAll();

      RequireSelectAll = false;
   }
}

void __fastcall GNameEd::OnEnterEditor(TObject *Sender)
{
   if(SelLength != Text.Length())
   	RequireSelectAll = true;
}


void __fastcall GNameEd::OnExitEditor(TObject *Sender)
{
	String Name = Text;

   Combo->EditPlayerForm->ValidName(Name);
}

void __fastcall GNameEd::OnChangeText(TObject* Sender)
{
	const String utext = Text;							// User's current ext
   const int upos = Text.Length();

   GNameList* ListBox = Combo->ListBox;

   if(!ListBox) return;

   Combo->EditPlayerForm->UpdateModified(cbName, Text != EditPlayerTube.Name);
   Combo->EditPlayerForm->ModifiedBox = (TObject*)this;
   
   if(TailDeleted) 										// Tail deleted by Del or BackSpace key
   {
      TailDeleted = false;

      return;
   }

   if(Combo->EditPlayerForm->AutoCompBox->Checked)
   	if(ListBox->Find(utext))						// So the text is embedded in the list
      {
         ListBox->UnpackItem();						// SAFELY Resets Text to completed text

         SelectText(upos, Text.Length()-upos);	// Select tail
      }
      else
         // THE MODIFIED NAME IS NOT ON LIST
         // If editing an existing player leave all other fields untouched.
         // If entering a new player, clear all other fields.

      	if(EditPlayerTube.State==PlayerMaster::epiNew)
         	Combo->EditPlayerForm->ClearUnknownPlayerFields();
}

//---------------------------------------------------------------------------
// THE COMBO
//---------------------------------------------------------------------------
__fastcall GNameBox::GNameBox
(
   const int edtop, const int edleft,				// Editor geometry
   const int edheight, const int edwidth,

   const int lbwidth,									// Listbox geometry
   const int lbcols[4],

   TWinControl* parent,
   TEditPlayerFrm*  editplayerfrm
)
{
	Parent         = parent;
   EditPlayerForm = editplayerfrm;

   // CREATE THE THREE COMPONENTS OF THE COMBO
	Editor = 0;  PipBtn = 0;  ListBox = 0;

   Editor = new GNameEd(this);
   PipBtn = new GNamePip(this);
   ListBox = new GNameList(lbcols, this);

   if(!Editor || !PipBtn || !ListBox)
   {
   	delete ListBox;	ListBox = 0;
      delete PipBtn;		PipBtn = 0;
   	delete Editor;		Editor = 0;

      return;
   }

   // MOUNT THE EDITOR
   Editor->Top = edtop;  		Editor->Left = edleft;
   Editor->Width = edwidth;   Editor->Height = edheight;

   // MOUNT THE PIP BUTTON
   PipBtn->Width  = 13*Editor->Width/251;
   PipBtn->Height = Editor->Height - 4;

   PipBtn->Top    = Editor->Top+2;
   PipBtn->Left   = Editor->Left + Editor->Width - PipBtn->Width - 2;

	PipBtn->DrawPip();

   // MOUNT THE LISTBOX
   ListBox->Width  = lbwidth;
   ListBox->Height = (ListBox->ItemHeight+3)*7;
   ListBox->Top    = Editor->Top + Editor->Height - 1;
   ListBox->Left   = Editor->Left;

   TRect Client = ListBox->ClientRect;

	ListBox->VisibleItems  = (Client.Bottom - Client.Top)/ListBox->ItemHeight;

	SetEnabled(true);   
}

__fastcall GNameBox::~GNameBox()
{
  	delete ListBox;	ListBox = 0;
   delete PipBtn;		PipBtn = 0;
   delete Editor;		Editor = 0;
}

void GNameBox::SetEnabled(const bool enabled)
{
	Editor->Enabled  = enabled;
   PipBtn->Enabled  = enabled;
   ListBox->Enabled = enabled;

   Enabled = enabled;
}

void GNameBox::Show()
{
	Editor->Show();
   PipBtn->Show();
   ListBox->Hide();
}

void GNameBox::Load(String name)
{
	using ToolsGUI::FindNearest;

   // LOAD TEXT INTO EDITOR
	Editor->SetText(name);

   // LISTBOX IS INVISIBLE - SET IT TO NEAREST MATCH
   bool exact;

   ListBox->ItemIndex = FindNearest(name, ListBox->Items, exact);
}
//---------------------------------------------------------------------------
// THE GAMES GRID
//---------------------------------------------------------------------------
static const int GameCols[8] = { 40, 40, 120, 40,60,40,40,40 };

__fastcall GGameGrid::GGameGrid
(
   const int top, const int left,
   const int height, const int width,
   TWinControl* parent
)
:TStringGrid(parent->Owner)
{
	Parent   			= parent;
	Visible  			= false;
   Top					= top;
   Left 					= left;
   Width					= width;
   Height				= height;
   DefaultRowHeight	= 21;
   FixedCols			= 1;
   FixedRows 			= 1;
   ScrollBars			= ssVertical;
   TabStop 				= true;
   ColCount          = 8;
   RowCount          = 7;

   Options.Clear();
   Options<<goFixedVertLine<<goFixedHorzLine<<goVertLine<<goHorzLine;
   Options<<goThumbTracking;

   NavKey = false;

   // CONSTRUCT THE TICKS
	for(int st = 0; st<stCount; st++)
  	for(int pt = 0; pt<ptCount; pt++)
  	{
   	Ticks[st][pt] =  new Graphics::TBitmap();

      Ticks[st][pt]->Transparent = true;
   }

   EditPlayerForm =  dynamic_cast<TEditPlayerFrm*>(parent->Owner);
}

__fastcall GGameGrid::~GGameGrid()
{
	for(int st = 0; st<stCount; st++)
  	for(int pt = 0; pt<ptCount; pt++)
   	delete Ticks[st][pt];
}

void GGameGrid::Forge(void)
{
   EditPlayerTube.LoadTour();

   // COLUMN HEADINGS
   for(int col=0; col<ColCount; col++)
      Cells[col][0] = LoadStr(IDB_ROUNDSHEAD+col);

   // ROW HEADINGS
   RowCount = EditPlayerTube.Rounds+1;

   for(int row=1; row<RowCount; row++)
      Cells[0][row] = String(row);

   // ADJUST SIZE OF OPPONENT COLUMN
   int ColArea = 0;


   ScrollBars = RowCount <= VisibleRowCount+1 ? ssNone : ssVertical;

   for(int col=0; col<ColCount; col++)
      if(col != 2)ColArea += GameCols[col];

   const int OppWidth = ClientWidth - ColArea - 8;

   for(int col=0; col<ColCount; col++)
      if(col != 2)
         ColWidths[col] = GameCols[col];

   ColWidths[2] = OppWidth>40 ? OppWidth : GameCols[2];

  // DRAW THE TICKS
	for(int st = 0; st<stCount; st++)
  	for(int pt = 0; pt<ptCount; pt++)
  	{
  		Graphics::TBitmap*& Tick = Ticks[st][pt];

      Tick->Width  = ColWidths[1]-1; Tick->Height = RowHeights[0]-1;

  		DrawTick(st,pt, Tick);
   }
}

void GGameGrid::Load(void)
{
	for(int row = 1; row<RowCount; row++)
 	for(int col = 1; col<ColCount; col++)
   {
   	Cells[col][row] = EditPlayerTube.GameText(row, col);

      String Scr = Cells[col][row];

      Scr=String(";");
   }

   // NO CELL SHOULD BE SELECTED UNTIL CLICKED
   TGridRect NoSelect;

   NoSelect.Top    = -1;   NoSelect.Left  = -1;
   NoSelect.Bottom = -1;   NoSelect.Right = -1;

   Selection = NoSelect;
}

void GGameGrid::HarvestPlay(std::vector<bool>&playing)
{
   for(size_t r=0; r<playing.size(); ++r)
   {
   	char playchar = Cells[1][r+1][1];

   	playing[r] = playchar != Tools::cDot;
   }
}

void GGameGrid::UpdateModified(void)
{
   if(Col != 1) return;

	bool RoundModified = false;

   for(int row=1; row<=EditPlayerTube.Rounds; row++)
   {
   	const String& sPlaying = Cells[1][row];

   	const bool Playing = sPlaying[1] == 'Y';

      if(Playing != EditPlayerTube.Playing[row-1])
      {
      	RoundModified = true;	break;
      }
   }

	EditPlayerForm->UpdateModified(cbGame, RoundModified);
   EditPlayerForm->ModifiedBox = (TObject*)this;
}

static const ToolsGUI::Justify AlignGames[8] =
{
	ToolsGUI::jyCentre,		// Round
   ToolsGUI::jyCentre,		// Playing
   ToolsGUI::jyLeft,			// Opponent
   ToolsGUI::jyCentre,		// Colour
   ToolsGUI::jyLeft,			// Up/Dn
   ToolsGUI::jyCentre,		// Stones
   ToolsGUI::jyCentre,		// Komi
   ToolsGUI::jyCentre		// Result
};


void GGameGrid::DrawTick
(
   int state,						// Whether Free, Selected
   int kind,						// Whether Normal, Greyed, Protected

   Graphics::TBitmap* BitMap
)
{
   TRect Area(0,0, BitMap->Width, BitMap->Height);

   TColor OldBrushColor = BitMap->Canvas->Brush->Color, clBrush;

   BitMap->TransparentColor = Brush->Color;

   switch(state)
   {
      case stFree:     	clBrush = clWindow;  	break;
   	case stSelected:	clBrush = clHighlight;  break;
      case stGrey:      clBrush = clGrayText;	break;

      default:				clBrush = clRed;
   }

   BitMap->Canvas->Brush->Color = clBrush;
   BitMap->Canvas->FillRect(Area);

   TPoint O(11, 3), Tickpts[6];

   for(int p=0; p<6; p++)
   {
   	Tickpts[p] = TickPoints[p];

      Tickpts[p].x += O.x;   Tickpts[p].y += O.y;
   }

   switch(kind)
   {
   	case ptNormal:    clBrush = clYellow;    	break;
      case ptGreyed:  	clBrush = clGrayText;	break;
      case ptProtected:	clBrush = clGray;			break;

   	default: 			clBrush = clBlue; 		break;
   }

   BitMap->Canvas->Brush->Color = clBrush;

   BitMap->Canvas->Polygon(Tickpts, 5);

   BitMap->Canvas->Brush->Color = OldBrushColor;
}

void __fastcall GGameGrid::DrawCell
(
	int col, int row,
   const Windows::TRect &cell,
   Grids::TGridDrawState state
)
{
	using ToolsGUI::jyCentre;
	using ToolsGUI::DrawAlignedText;

	if(row==0)
   {
   	DrawAlignedText(Canvas->Handle, jyCentre, cell, Cells[col][row]);

      return;
   }

   if(row>0 && col != 1)
   {
      String CellText = Cells[col][row];
 // BUG CellText==NULL
 
   	if(CellText[1] == '.')
   		DrawAlignedText(Canvas->Handle, jyCentre, cell, CellText);
		else
    		DrawAlignedText(Canvas->Handle, AlignGames[col], cell, CellText);

      return;
   }

   const bool ViewPastRound = EditPlayerTube.Round != EditPlayerTube.Roundl;

   int tickstate;

   if(!Enabled)
   	tickstate = stGrey;
   else
   	tickstate = state.Contains(gdSelected) ? stSelected : stFree;

   if(row>0 && col == 1)
   {
   	const bool Playing  = Cells[col][row][1] == 'Y';

   	if(Playing)
      {
         const bool tpast      = row < EditPlayerTube.Round;
         const bool tprotected = row > EditPlayerTube.Round && ViewPastRound;

         int ticktype = ptNormal;

         if(tpast) ticktype = ptGreyed;

         if(tprotected) ticktype = ptProtected;

      	Canvas->Draw(cell.Left, cell.Top, Ticks[tickstate][ticktype]);
   	}
      else
      {
      	const TColor OldBrushColor = Canvas->Brush->Color;

         TColor clBrush;

         switch(tickstate)
         {
            case stFree:     	clBrush = clWindow;  	break;
            case stSelected:	clBrush = clHighlight;  break;
            case stGrey:      clBrush = clGrayText;	break;

            default:				clBrush = clRed;
         }
         Canvas->Brush->Color = clBrush;

         Canvas->FillRect(cell);

      	DrawAlignedText(Canvas->Handle, clBrush, jyCentre, cell, Cells[col][row]);

         Canvas->Brush->Color = OldBrushColor;
      }
   }
}


bool __fastcall GGameGrid::SelectCell(int col, int row)
{
   if(col<1 || row<1) return false;

   return true;
}

void __fastcall GGameGrid::SetFocus(void)
{
	const int row = PlayerMaster::EditPlayerTube.Round;

   TGridRect GridCell;

   GridCell.Top    = row;   GridCell.Left  = 1;
   GridCell.Bottom = row;   GridCell.Right = 1;

   TStringGrid::SetFocus();
   
   Selection = GridCell;
}

void __fastcall GGameGrid::Click(void)
{
   if(EditPlayerForm->UserError(cbGame))
   {
   	TGridRect NoSelect;

      NoSelect.Top    = -1;   NoSelect.Left  = -1;
      NoSelect.Bottom = -1;   NoSelect.Right = -1;

      Selection = NoSelect;

   	return;
   }
   
	const bool NoClick = Row<PlayerMaster::EditPlayerTube.Round;

   // CLICK WAS TRIGGERED BY A KEY
   if(NavKey)
   {
   	NavKey = false;

      if(ArrowKey)ArrowKey = false;

      return;
   }

   // ONLY CLICK ON PLAYING COLUMN
	if(Row==0 || Col != 1 || NoClick)
   {
   	TStringGrid::Click(); return;
   }

   // CANNOT CHANGE THE PAST
   if(Row < EditPlayerTube.Round)
   	return;

   // WHILE VIEWING PAST ROUNDS CANNOT CHANGE THE FUTURE
   if(EditPlayerTube.Round != EditPlayerTube.Roundl)
   	if(Row > EditPlayerTube.Round)
      	return;

   // TOGGLE PLAYING
   String sPlaying = Cells[1][Row];

   if(sPlaying[1] == 'Y')
   	sPlaying = LoadStr(IDB_NoText);
   else
   	sPlaying = LoadStr(IDB_Yes);

   Cells[1][Row] = sPlaying;

   InvalidateCell(Col,Row);

   EditPlayerForm->UpdateMMS(cbGame);

   // UPDATE TickBox->Reg IF WE ARE EDITING THE CURRENT ROUND
   if(Row == EditPlayerTube.Round)
   {
   	using GTickBox::rtReg;
   	using GTickBox::rtJoin;
      using GTickBox::rtLeave;

      int& TickReg = EditPlayerForm->TickBox->Reg;

   	bool NowPlaying = sPlaying[1] == 'Y';
   	bool WasPlaying = EditPlayerTube.Playing[Row-1];

      if( WasPlaying ==  NowPlaying)			// Player Status restored
      	TickReg = EditPlayerTube.Reg;
      else
      	switch(EditPlayerTube.Reg)				// Player Status disturbed
         {
            case rtJoin:  TickReg = rtReg;   /* Undo Joining */ 	 break;
            case rtLeave: TickReg = rtReg;   /* Undo Leaving */	 break;
            case rtReg:   TickReg = NowPlaying ? rtJoin : rtLeave; break;

            default: 	  TickReg = EditPlayerTube.Reg;
         }

         EditPlayerForm->TickBox->Repaint();
	}

   // MODIFIED ROUND
   UpdateModified();
}


void __fastcall GGameGrid::KeyDown(Word &Key, TShiftState Shift)
{
	NavKey = false;
   ArrowKey = false;

	switch(Key)
   {
   	case VK_ESCAPE:
      	Cells[Col][Row] = EditPlayerTube.GameText(Row,Col);
         InvalidateCell(Col,Row);
      break;

      case VK_RETURN:
      	EditPlayerForm->OKBtn->Click();
      break;

      default:
      	if(0x30 <= Key && Key <= 0x5B)
         	Click();
         else
         {
         	NavKey = true;

            ArrowKey = Key == VK_UP || Key == VK_DOWN;

         	TStringGrid::KeyDown(Key, Shift);

            Repaint();

            return;
         }
   }

   Key = 0;

   UpdateModified();
}
//---------------------------------------------------------------------------


