////////////////////////////////////////////////////////////////////////////////
// MODULE  EditPlayerGUI   INTERFACE TO EditPlayerForm    G.Kaniuk 02/04/08
//

#include <string>

#ifndef EditPlayerGUIH
#define EditPlayerGUIH

//---------------------------------------------------------------------------
// TICK CONTROL
//---------------------------------------------------------------------------

class TEditPlayerFrm;

class GTickBox : public TCustomControl
{
	public:
   	enum{ ftFree, ftSelect, ftGrey, ftCount};		  		// Selected
      enum{ rtNoReg, rtReg, rtJoin, rtLeave, rtCount };  // Registered

   private:
   	TEditPlayerFrm*  EditPlayerForm;

      Graphics::TBitmap*Ticks[ftCount][rtCount];

      void DrawTick
      (
         int Selected,				// Whether Free or Selected
         int Registered,         // Tick or blob
         int oL, int oT,         // Offsets

         Graphics::TBitmap* BitMap
      );

      void DrawTicks(void);

   protected:
      virtual void __fastcall Paint(void);

      virtual void __fastcall OnClickTick(TObject *Sender);
      virtual void __fastcall OnEnterTick(TObject *Sender);
      virtual void __fastcall OnExitTick(TObject *Sender);

   public:
      __fastcall GTickBox(TWinControl* parent);
      __fastcall ~GTickBox();

      DYNAMIC void __fastcall KeyDown(Word& Key, TShiftState Shift);
      void __fastcall KeyDownSpecial(TCMWantSpecialKey& Message);

      BEGIN_MESSAGE_MAP
         MESSAGE_HANDLER(CM_WANTSPECIALKEY, TCMWantSpecialKey, KeyDownSpecial)
      END_MESSAGE_MAP(TCustomControl)

   public:
    	int  Reg;
      int  Selected;
      bool Clicked;
      bool Entered;
};

//---------------------------------------------------------------------------
// NameBox Combo
//
// Combobox with Autocompletion and multiple columns for the NameBox
//
// The Combo is constructed from an Edit Box, a Pip button, and a List box as this
// is the only combination that gives all the features we need.
//
// Autocompletion is managed via the GComboBox member LastText which gets set
// on entry to the Combo, and is the text in the combo just before any key is
// pressed.  The user changes the Combo.Text and this is used as the search in
// the List box for an item starting with Combo.Text  If found the list item
// is accepted, with the non-matching part selected.  At this point LastText is
// set to the users Combo.text(the unhighlighted part of the text in the Combo).
//
// The ESCAPE key always sets the Combo.Text to EntryText.

// The class implementations are found in ComboGUI.cpp
//---------------------------------------------------------------------------
class GNamePip;
class GNameEd;
class GNameList;
class GNameBox;


class GNamePip : public TBitBtn					// THE Pip BUTTON
{
	private:
   	GNameBox* Combo;

   protected:
      void __fastcall OnClickPip(TObject *Sender);

   public:
      __fastcall GNamePip(GNameBox* combo);

      virtual void DrawPip();
};
//---------------------------------------------------------------------


class GNameEd : public TEdit			 			// THE Edit BOX
{
   private:
      GNameBox* Combo;
      bool RequireSelectAll;
      bool TailDeleted;

   protected:
      virtual void __fastcall OnKeyDownEditor(TObject *Sender, WORD &Key, TShiftState Shift);

      virtual void __fastcall OnClickEditor(TObject *Sender);
      virtual void __fastcall OnChangeText(TObject *Sender);
      virtual void __fastcall OnEnterEditor(TObject *Sender);
		virtual void __fastcall OnExitEditor(TObject *Sender);
      
      virtual void __fastcall TEditKeyDown(WORD &Key, TShiftState Shift);
	public:
   	void SelectText(const int start, const int length);
      void SetText(const String text);
      void Escape(void);

   public:
      __fastcall GNameEd(GNameBox* combo);
};
//---------------------------------------------------------------------------

class GNameList: public TCustomListBox             // THE List BOX
{
   friend GNamePip;
   friend GNameEd;
   friend GNameBox;

   private:
      GNameBox* Combo;      	// Controller

      int VisibleItems;       // number of visible lines

      int ListCols[4];  		// Display positions of the listbox columns

      TColor Background;      // Brush Color of Listbox at construction

      bool NavKey;            // True to suppress Click()

   protected:
   	DYNAMIC void __fastcall KeyDown(Word &Key,TShiftState Shift);
      MESSAGE void __fastcall CNDrawItem(TMessage& message);
      virtual void __fastcall DrawItem(int Index, const TRect& Rect, TOwnerDrawState State);

      void __fastcall KeyDownSpecial(TCMWantSpecialKey& Message);
      void __fastcall OnListClick(TObject* Sender);
      void __fastcall OnListExit(TObject* Sender);

      virtual void SelectItem(void);
      virtual void UnpackItem(void);
      virtual void Close(void);
      virtual bool Find(const String text);

   public:
      BEGIN_MESSAGE_MAP
         MESSAGE_HANDLER(CN_DRAWITEM, TMessage, CNDrawItem)
         MESSAGE_HANDLER(CM_WANTSPECIALKEY, TCMWantSpecialKey, KeyDownSpecial)
      END_MESSAGE_MAP(TCustomListBox)

      __fastcall GNameList(const int listcols[4], GNameBox* combo);

      void SetSorted(bool sorted);

      void Nudge(int dir);
      void Page(int dir);
};
//------------------------------------------------------------------------------


class GNameBox                         	  	// THE Combo BOX
{
   friend GNamePip;
   friend GNameEd;
   friend GNameList;
   
   private:
   	TWinControl* Parent;
      TEditPlayerFrm*  EditPlayerForm;

   public:
   	int Enabled;

      GNamePip*  PipBtn;
      GNameList* ListBox;
      GNameEd*   Editor;

   public:
      __fastcall GNameBox
      (
         const int edtop,
         const int edleft,				// Editor geometry
         const int edheight, const int edwidth,

         const int lbwidth,									// Listbox geometry
         const int lbcols[4],

         TWinControl* parent,
         TEditPlayerFrm*  editplayerfrm
      );
      __fastcall ~GNameBox();

      void SetEnabled(const bool enabled);

      void Load(String name);

      void Show();
};
//---------------------------------------------------------------------------
// THE GAMES GRID
//---------------------------------------------------------------------------
class GGameGrid : public TStringGrid
{
   private:
   	enum { ptNormal, ptGreyed, ptProtected, ptCount};	// Tick type
      enum { stFree, stSelected, stGrey, stCount };		// Tick state

      TEditPlayerFrm* EditPlayerForm;

   	bool NavKey;													// Suppress spurious Click
      bool ArrowKey;													// Capture EntryText for up/dn arrow keys

      Graphics::TBitmap* Ticks[stCount][ptCount];

      void DrawTick(const int State, const int Type, Graphics::TBitmap* BitMap);

   	virtual void __fastcall DrawCell
      (
         int col, int row, const Windows::TRect &cell,
         Grids::TGridDrawState state
      );

      DYNAMIC void __fastcall KeyDown(Word &Key, TShiftState Shift);
      DYNAMIC void __fastcall Click(void);

      virtual bool __fastcall SelectCell(int col, int row);

	public:
   	__fastcall GGameGrid
      (
      	const int top, const int left,
         const int height, const int width,
         TWinControl* parent
      );

      __fastcall ~GGameGrid();

      virtual void __fastcall SetFocus(void);

	public:
   	void Forge(void);
   	void Load(void);
      void HarvestPlay(std::vector<bool>& playing);
      void UpdateModified(void);
};
//---------------------------------------------------------------------------
#endif