////////////////////////////////////////////////////////////////////////////////
// MODULE  GoRGUI:    RankList in various formats for export  G.Kaniuk 25/02/03
//
#include <string>
#include <vector>

#include <vcl.h>
#pragma hdrstop

#include "GoDrawRES.h"
#include "GoErrMsgRES.h"
#include "GoFormRES.h"
#include "FileRES.h"

#include "GoDrawGUI.h"
#include "StringsGUI.h"

#include "Strings.h"
#include "Format.h"
#include "TourMaster.h"
#include "RegMaster.h"

#include "ExportGUI.h"
#include "GoRForm.h"

#pragma package(smart_init)

namespace GoRGUI                       // IMPLEMENTATION OF THE GoR FORMAT
{
   using std::string;
   using std::vector;
   
   using TourMaster::GTourMaster;
   using TextSmith::RankFile;

   static const string fmtRnd("########");
   static const string fmtOpp("####");
   static const string fmtPos("####");

   static const string snoplay("      0");
   static const string Spaces("  ");

   static const string PartHead("GR CO CLU    MMS WIN    ");

   static std::string sosos_string(Tools::ints mms)
   {
   	const string sosos_fmt = string("########");

      int imms = mms/Tours::mc_scale;
      int fmms = Tools::modulus(mms)%Tours::mc_scale;

      std::string sfmms = (0<fmms && fmms<4)
                   ? string(1, Tools::cFractional[fmms])
                   : string(Tools::sSpace);

      std::string simms = Tools::to_string(sosos_fmt, imms);

      return simms + sfmms;
   }

   static string make_gor_name(string& tname)
   {
   	// tname has syntax fname[, gname]
      // fname may have spaces, gname may have spaces
      // . space to be replaced with _
      // . [,] to be replaced with space.

      string name = Tools::trim(tname);  Tools::strip_null(name);

      string::iterator nf = name.begin(), ne = name.end();

      while(nf != ne)
      {
      	switch(*nf)
         {
         	case ' ': *nf = '_'; break;
            case ',': *nf = ' '; break;

            default:				 ;
         }
         ++nf;
      }
      return name;
   }

   Tools::ints TieShift(Tours::GTour* tour, Tours::tie_t ties)
   {
      size_t Rounds = tour->Rules.Rounds;
      switch(ties)
      {
         case Tours::tbNone:   return 0;
         case Tours::tbWins:   return 0;
         case Tours::tbCuss:   return Rounds*tour->Rules.MinGrade;
         case Tours::tbCusp:   return 0;
         case Tours::tbSoS:    return Rounds*tour->Rules.MinGrade;
         case Tours::tbSoDoS:  return Rounds*tour->Rules.MinGrade/2;
         case Tours::tbSoSoS:  return Rounds*Rounds*tour->Rules.MinGrade;

         default:       return 0;
      }
   }

   GoRRankFile::GoRRankFile(string& fname, GTourMaster* tour):RankFile(fname,tour)
   {
      FileName = Tools::strip_ext(tour->FileName) + GoDraw::GoRExt;
   }

   string GoRRankFile::game_string(Tours::GGame& game)
   {
      int oppid = game.Oppid>0 ? (int)Tour->Listed[game.Oppid]->Rank : 0;

      char res, col;

      bool simple = true;

      switch(game.Result)
      {
         case Tours::grWin:  res = '+'; break;
         case Tours::grLose: res = '-'; break;
         case Tours::grJigo: res = '='; break;

         case Tours::grFwin: res = '+'; simple = false; break;
         case Tours::grDlos: res = '-'; simple = false; break;
         case Tours::grSleep:res = '='; simple = false; break;

         default: res = '?';            simple = false;
      }

      switch(game.Colour)
      {
         case Tours::gcBlack: col = 'b'; break;
         case Tours::gcWhite: col = 'w'; break;

         default: col = '?';
      }

      const int hcap = Tools::modulus(game.Stones)/Tours::mc_scale;

      const string komi = Tools::to_string(hcap);

      string sgame;

      if(simple)
      {
         sgame = Tools::to_string(fmtOpp, oppid);

         sgame.push_back(res);
         sgame.push_back('/');
         sgame.push_back(col);
      }
      else
      {
         sgame = snoplay;
         sgame.push_back(res);
      }
      return sgame+komi;
   }

   void GoRRankFile::HeadLine()
   {
   	using TourMaster::GoTour;
      
   	const string mbar = Tours::grade_ustring(GoTour.Rules.MacBar);
      const string mpop = Tools::to_string(GoTour.MacPop);

		string Bar(";McMahon Bar ");

      Bar += mbar + string("  players ") + mpop;

      Add(";CL[X]");
      Add(";EV[tourname]");
      Add(";PC[UK, place]");
      Add(";DT[2011-mm-d1,2011-mm-d2]");
      Add(";HA[H9]");
      Add(";KM[7.5]");
      Add(";TM[70]");
      Add(Bar);

      string Heading(";");

      Heading += Tools::load_str(IDL_GPOSN) + Tools::sSpace;
      Heading += Tools::load_str(IDL_IName) + string(23, Tools::cSpace);
      Heading += PartHead + Tools::sSpace;
      Heading += Tools::load_str(IDL_GNONE + Tour->Rules.TieOne);
      Heading += Tools::load_str(IDL_GNONE + Tour->Rules.TieTwo);

      for(int r=0; r<(int)Tour->Roundl; r++)
         Heading += Tools::to_string(fmtRnd, r+1) + Spaces;

      Add(Heading);
   }

   void GoRRankFile::RankLine
   (
      bool newgrp, int p, int pos,
      Tours::GPlayerPtr P
   )
   {
      using Tools::ints;
      using Tools::sSpace;
      using Tools::to_string;
      using Tools::parse_extended_family;
      using Tools::trim;

      using Tours::grade_string;
      using Tours::wins_string;
      using Tours::mms_string;

      //string Given, Middle, Family, name = trim(P->Name), gor_name = make_gor_name(P->Name);

      //parse_extended_family(name, Given, Middle, Family);

      string Club = P->Club, Land = P->Land;

      if(RatingClubList)
      	//ToolsGUI::OfficialCode(ClubList, Tour->Clubs, Club);
         ToolsGUI::OfficialClubCode(RatingClubList, Tour->Clubs, Club);

      if(LandList)
      	ToolsGUI::OfficialCode(LandList, Tour->Lands, Land);

      const ints McZero = Tour->Rules.MinGrade;                   // Minimum MacMahon Score
      const ints McOne  = TieShift(Tour, Tour->Rules.TieOne);     // Tiebreak conversion
      const ints McTwo  = TieShift(Tour, Tour->Rules.TieTwo);

      string Blank = string();

      string Line = to_string(fmtPos, p) + sSpace;

      //Line += Family + sSpace + Given + sSpace + Middle;
      Line += make_gor_name(P->Name);
      Line.resize(32, Tools::cSpace);

      // Grade+Country+Club
      Line += grade_string(P->Grade) + sSpace;
      Line += Land + sSpace;
      Line += Club + sSpace + sSpace;

      // MMscore + wins
      Line += mms_string(P->McScore - McZero) + sSpace;
      Line += wins_string(P->Wins()) + Spaces;

      // TieOne+TieTwo
      const ints TieOne=P->TieOne-McOne, TieTwo=P->TieTwo-McTwo;
      
      if(Tour->Rules.TieOne == Tours::tbSoSoS)
      	Line += sosos_string(TieOne) + Spaces;
      else
         Line += mms_string(TieOne) + sSpace;

      if(Tour->Rules.TieTwo == Tours::tbSoSoS)
      	Line += sosos_string(TieTwo) + Spaces;
      else
         Line += mms_string(TieTwo) + Spaces;

      // Games
      for(size_t r=0; r<Tour->Roundl; r++)
         Line += game_string(P->Games[r])+ Spaces;

      Add(Line);

      if(!(p%5)) Add(Blank);
   }
}


namespace GoRGUI
{
   using std::string;
   using std::vector;

   GoRTube::GoRTube(const GoRRankFile& GoRRank)
   {
      TourName = "Tourname";  // TODO:  Tourname specified at new := Tours(GoRRank.Tour->TourId) , Where Tours is a stringlist

      FileName = GoRRank.FileName;

      Lines = GoRRank.Lines;
   }

   void GoRTube::LoadFrom(const string SourceName, TMemo* Memo)
   {
      FileName = SourceName;

      Lines->Clear();

      for(int i =0; i<Memo->Lines->Count; i++)
         Lines->Append(Memo->Lines->Strings[i]);
   }


   void GoRTube::CancelRankList()
   {
      FileName = Tools::sEmpty;
   }

}


namespace ExportGUI
{
   using std::string;

   using Tools::S_s;
   using Tools::s_S;

   using TourMaster::GoTour;

   using TextSmith::RankFile;

   void Throw(ToolsGUI::EType etype, short idproc, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDM_FileExport, idproc, idmsg);
   }

   String ExportTitle(int TitleId)
   {
   	return LoadStr(TitleId) + Tools::s_S(TourMaster::GoTour.TourNameYear());
   }



   void ExportRankCmd(TSaveDialog* SaveDlg)
   {
      using GoDraw::RankExt;

      using TourMaster::GoTour;

      if(GoTour.State < Tours::tsRegNow)
      {
      	ToolsGUI::OperatorError(IDF_NoExport);   return;
      }

      // GK REGSORT 2015 2015 int OldSortSense = GoTour.SortSense, OldSortOrder = GoTour.SortOrder;
      try
      {
         string RankFileName = Tools::strip_ext(GoTour.FileName) +  RankExt;

         SaveDlg->Filter   = LoadStr(IDL_FRankList) + s_S(RankExt);
         SaveDlg->FileName = s_S(RankFileName);
         SaveDlg->Title    = ExportTitle(IDM_RnkTitle);

         if(SaveDlg->Execute())
         {
            RankFileName = Tools::strip_ext(S_s(SaveDlg->FileName)) + RankExt;

            TextSmith::StdRankFile StdRank(RankFileName, &GoTour);

            StdRank.BuildRankList();

            TourMaster::GoTour.PostTourInfo(IDF_RankFiled);
         }
      }
      catch(Exception& Err)
      {
         ToolsGUI::Serve(Err, IDM_RnkExport, IDM_RnkExport);
      }
      catch(...)
      {
         ToolsGUI::ShowWin32Err(IDM_WebExport, IDE_NoMemory);
      }
      // GK REGSORT 2015 RegMaster::RegSort(OldSortOrder, OldSortSense);
   }


   void ExportWebCmd(TSaveDialog* SaveDlg)
   {
      using GoDraw::WebExt;

      using TourMaster::GoTour;

      if(GoTour.State < Tours::tsRegNow)
      {
      	ToolsGUI::OperatorError(IDF_NoExport);   return;
      }

      // GK REGSORT 2015 int OldSortSense = GoTour.SortSense, OldSortOrder = GoTour.SortOrder;
      try
      {
         string RankFileName = Tools::strip_ext(GoTour.FileName) + WebExt;

         SaveDlg->Filter   = LoadStr(IDL_FWebList) + s_S(WebExt);
         SaveDlg->FileName = s_S(RankFileName);
         SaveDlg->Title    = ExportTitle(IDM_WebTitle);

         if(SaveDlg->Execute())
         {
            RankFileName = Tools::strip_ext(S_s(SaveDlg->FileName)) + WebExt;

            TextSmith::WebRankFile WebRank(RankFileName, &GoTour);

            WebRank.BuildRankList();

            TourMaster::GoTour.PostTourInfo(IDF_WebFiled);
         }
      }
      catch(Exception& Err)
      {
         ToolsGUI::Serve(Err, IDM_WebExport, IDM_WebExport);
      }
      catch(...)
      {
         ToolsGUI::ShowWin32Err(IDM_WebExport, IDE_NoMemory);
      }
      // GK REGSORT 2015 RegMaster::RegSort(OldSortOrder, OldSortSense);
   }

   void ExportRatingCmd
   (
      TComponent* Owner, TSaveDialog* SaveDlg
   )
   {
   	using TourMaster::GoTour;

      if(GoTour.State < Tours::tsRegNow)
      {
      	ToolsGUI::OperatorError(IDF_NoExport);   return;
      }

      // GK REGSORT 2015 int OldSortSense = GoTour.SortSense, OldSortOrder = GoTour.SortOrder;
      TGoRFrm* GoRFrm = NULL;

      string GoRFile = string();

      GoRGUI::GoRRankFile RankFile(GoRFile, &GoTour);

      try
      {
      	Screen->Cursor = crHourGlass;
         
         String RatingClubFile = GoDraw::FilePath(GoDraw::GoClubCodeR);

         RankFile.RatingClubList = new TStringList();

   		RankFile.RatingClubList->LoadFromFile(RatingClubFile);

         RankFile.BuildRankList();

         GoRGUI::GoRTube RankTube(RankFile);

         GoRFrm = new TGoRFrm(Owner, SaveDlg, &RankTube);   // Updates RankFile Lines

         GoRFrm->Caption = ExportTitle(IDM_GoRTitle);

         GoRFrm->ShowModal();

         if(GoRFrm->ModalResult == mrOk)
            TourMaster::GoTour.PostTourInfo(IDF_GoRFiled);
         else
            RankTube.CancelRankList();

         RankFile.FileName = RankTube.FileName;

         delete GoRFrm;                            			// The list is filed when ~RankFile is invoked !
         delete RankFile.RatingClubList;

			Screen->Cursor = crDefault;         
      }
      catch(Exception& Err)
      {
         delete GoRFrm;
         delete RankFile.RatingClubList;
         Screen->Cursor = crDefault;
         ToolsGUI::Serve(Err, IDM_GoRExport, IDM_GoRExport);
      }
      catch(...)
      {
         delete GoRFrm;
         delete RankFile.RatingClubList;
         Screen->Cursor = crDefault;
         ToolsGUI::ShowWin32Err(IDM_GoRExport, IDE_NoMemory);
      }
      // GK REGSORT 2015  RegMaster::RegSort(OldSortOrder, OldSortSense);
   }
}
//---------------------------------------------------------------------------


