////////////////////////////////////////////////////////////////////////////////
// MODULE FileMUI:       File Menu  Interface       G.Kaniuk 14/05/04
//
#include <vcl.h>
#include <systdate.h>
#include <filectrl.hpp>

#pragma hdrstop


#include "Strings.h"
#include "ETourRES.h"
#include "GoErrMsgRES.h"
#include "GoInfoRES.h"
#include "FileRES.h"

#include "ETour.h"
#include "GraphicsAPI.h"
#include "FilesGUI.h"

#include "GoDrawGUI.h"
#include "RegListsGUI.h"
#include "RegPanelGUI.h"

#include "TourStream.h"
#include "PrefsMaster.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "PlayerMaster.h"

#include "NewForm.h"

#include "MenuMaster.h"
#include "FileMUI.h"

extern void __fastcall LoadEditLists(void);
extern void __fastcall UpdateEditLists(void);
extern bool __fastcall PlayerTextChanged(void);
extern void __fastcall SetCancelBtnCaption(void);
extern void __fastcall SetPairBtnCaption(void);

#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace FileMUI                                  // ERROR MANAGEMENT
{
   static void Throw(ToolsGUI::EType etype, short idevent, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_FileMUI, idevent, idmsg);
   }

   static int Serve(Exception& Err, short idevent)
   {
      return ToolsGUI::Serve(Err, IDE_FileMUI, idevent);
   }
}
//---------------------------------------------------------------------------


namespace FileMUI
{
   void __fastcall LoadDraw(PaneGUI::GPairPane* PairPane)
   {
   	PairPane->LoadFromTour();

      PairPane->PairGrid->Repaint();
      PairPane->ResGrid->Repaint();
   }

	void __fastcall StartClearGrids						// SHOW GRIDS FROM CLEARED GoTour
   (
      RegGUI::GRegGrid* RegGrid, PaneGUI::GPairPane* PairPane
   )
   {
   	RegGrid->StartSortHeader();
   	RegGrid->LoadFromTour();
      RegGrid->Repaint();

      LoadDraw(PairPane);
   }

   void __fastcall LoadRegister(RegGUI::GRegGrid* RegGrid)
   {
      RegGrid->StartSortHeader();
   	RegGrid->ShowInfo(IDI_LdReg);
      RegGrid->LoadFromTour();
      RegGrid->Repaint();
   }

	void __fastcall NewCmd								// NEW TOURNAMENT
   (
   	TComboBox* RoundBox,
      RegGUI::GRegGrid* RegGrid,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using Tools::S_s;
      
   	using ToolsGUI::EditCell;
      using ToolsGUI::EditingGrid;

   	using TourMaster::GoTour;
      using PrefsMaster::PrefsTube;
      using RegPanelGUI::LoadRounds;

      TNewFrm* NewForm;

      // DEACTIVATE GRIDS
      if(EditCell)
         if(EditingGrid(PairPane->ResGrid))
         	EditCell->HardDismount();

      PlayerMaster::EditPlayerTube.State = PlayerMaster::epiVirgin;

      try
      {
      	FileMaster::MNewTube NewTube;										// Setup info for TNewFrm

      	NewForm = new TNewFrm(RegGrid->Owner, NewTube);				// Display NewFrm

         if(NewForm->ShowModal() == mrOk)
         {
            try
            {
            	std::string SeedFile = Tools::S_s(NewTube.SeedFile);

               Screen->Cursor = crHourGlass;

            	if(!SeedFile.empty())
               {
               	Tours::Load(SeedFile, GoTour);   					// Pick names etc ...

                  GoTour.CleanSeed();										// Remove opponents
               }
               else
               	GoTour.NewStart(Tours::tsTourStart);				// Standard 3 rounder, brings down all lists

               GoTour.TourName  = Tools::S_s(NewTube.TourName);	// Fixed size guaranteed
            	GoTour.Year      = NewTube.Year.ToInt();

            	GoTour.Save(Tools::S_s(NewTube.TourFile));

               ::LoadEditLists();
               ::UpdateEditLists();

               Screen->Cursor = crDefault;
            }
            catch(...)
            {
            	Screen->Cursor = crDefault;

            	Throw(ToolsGUI::erOperr,IDM_FileNew ,IDM_FileNewErr);
            }

            NewTube.SaveTourList();												// Operator may have added a tournament

            PrefsTube.TourFile = NewTube.TourFile;							// Preserve for auto load
            PrefsTube.SaveToFile();

            LoadRounds(RoundBox);

            StartClearGrids(RegGrid, PairPane);								// Finalise columns and show

            GoTour.PostTourInfo(IDM_NewTourFile);
         }
         delete NewForm;
      }
      catch(Exception& Err)
      {
      	Serve(Err, IDM_FileNew);

         delete NewForm;
      }
      catch(...)
      {
     		ToolsGUI::ShowWin32Err(IDE_FileMUI, IDM_FileNew);

         delete NewForm;
      }
   }

   void __fastcall OpenTourFile                      	// OPEN EXISTING TOURNAMENT
   (
      String FileName,
      TComboBox* RoundBox,
      RegGUI::GRegGrid* RegGrid, PaneGUI::GPairPane* PairPane
   )
   {
      using Tools::S_s;
      using Tools::s_S;

      using PrefsMaster::PrefsTube;
      using TourMaster::GoTour;
      using Tours::skEntry;

      using ToolsGUI::gsUp;

      using RegPanelGUI::LoadRounds;
      using GoDraw::FilePath;

      try
      {
      	Screen->Cursor = crHourGlass;

         GoTour.Load(Tools::S_s(FileName));				// LOAD THE TOURNAMENT

         PrefsTube.TourFile = s_S(GoTour.FileName);	// USED FOR AUTO LOAD

         PrefsTube.SaveToFile();

         LoadRounds(RoundBox);

         LoadDraw(PairPane);

         LoadRegister(RegGrid);

         Screen->Cursor = crDefault;

         GoTour.ClearTourInfo();

         if(GoTour.State != Tours::tsPastRound)
      		GoTour.PostTourInfo(IDI_Ready);
         else
         	GoTour.PostTourInfo(IDI_ReadyPast);
      }
      catch(Exception& Err)
      {
      	Screen->Cursor = crDefault;

         ToolsGUI::Serve(Err, IDM_FileOpen, IDM_FileOpenErr);
      }
      catch(...)
      {
      	Screen->Cursor = crDefault;

     		ToolsGUI::ShowWin32Err(IDE_FileMUI, IDM_FileOpenErr);
      }
   }

   void __fastcall OpenCmd                      			// OPEN EXISTING TOURNAMENT
   (
      TOpenDialog* OpenDlg,
      TComboBox* RoundBox,
      RegGUI::GRegGrid* RegGrid,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using ToolsGUI::EditCell;
      using ToolsGUI::EditingGrid;

      using TourMaster::GoTour;

   	// DEACTIVATE GRIDS
      if(EditCell)
         if(EditingGrid(PairPane->ResGrid))
         	EditCell->HardDismount();

      PlayerMaster::EditPlayerTube.State = PlayerMaster::epiVirgin;

      if(OpenDlg->Execute())
      {
      	OpenTourFile(OpenDlg->FileName, RoundBox, RegGrid, PairPane);

         ::LoadEditLists();
         ::UpdateEditLists();
      }
   }


   void __fastcall CloseCmd										// CLOSE CURRENT TOURNAMENT & CLEAR PREFS.TOURFILE
   (
      RegGUI::GRegGrid* RegGrid, PaneGUI::GPairPane* PairPane
   )
   {
   	using std::string;
      using TourMaster::GoTour;
      using PrefsMaster::PrefsTube;

      using ToolsGUI::EditCell;
      using ToolsGUI::EditingGrid;

      // DEACTIVATE GRIDS
      if(EditCell)
         if(EditingGrid(PairPane->ResGrid))
         	EditCell->HardDismount();

   	string warning = string();

   	switch(GoTour.State)
      {
      	case Tours::tsEditNow:    warning = Tools::load_str(IDM_WarnEdit);   break;
         case Tours::tsForcedPair: warning = Tools::load_str(IDM_WarnForced); break;

         default: warning = string();
      }

      if(!warning.empty())
      {
      	warning += Tools::load_str(IDM_WarnChoices);

         if(ShowMessage(ToolsGUI::erWarn, warning) != IDOK)
         {
         	GoTour.PostTourInfo(IDM_ClosureCancelled);
            return;
         }
      }

      PairPane->UpsetList->Hide();							// We may have been editing

      PrefsTube.FinishTourFile();                     // Clear Tourfile entry

      TourMaster::GoTour.NewStart(Tours::tsVirgin);	// Standard rules and clear all lists

      ::SetPairBtnCaption();
      ::SetCancelBtnCaption();								// V6.5.0 state is now virgin

      StartClearGrids(RegGrid, PairPane);					// Redisplay all grids as new

      GoTour.PopulationCount();								// Clear the counts

      GoTour.ClearTourInfo();
      GoTour.PostTourInfo(IDM_TourClosed);
   }

   bool __fastcall CloseGodraw								// CLOSE APPLICATION AFTER WARNING
   (
      RegGUI::GRegGrid* RegGrid
   )
   {
   	using std::string;
      using Tools::load_str;
      using TourMaster::GoTour;

   	string warning = string(), leader = load_str(IDM_ExitWarning);

      while(true)
      {
   		if(PlayerMaster::EditPlayerTube.State != PlayerMaster::epiVirgin)
         	if(::PlayerTextChanged())
            {
               warning += leader + load_str(IDM_ExitRegEd);  break;
            }

         if(GoTour.State == Tours::tsEditNow)
         {
         	warning += leader + load_str(IDM_ExitDrawEd);  break;
         }

         if(GoTour.State == Tours::tsForcedPair)
         {
         	warning += leader + load_str(IDM_ExitForced);  break;
         }

         break;
      }

      if(warning.empty())
      	return true;
      else
      {
      	warning += load_str(IDM_ExitChoices);

         if(ShowMessage(ToolsGUI::erWarn, warning) == IDOK)
         	return true;												// We will exit
         else
         	return false;												// Cancel exit
      }
   }
}
//---------------------------------------------------------------------------

namespace FileMUI                                  // BACKUP TO FLOPPY
{
	void __fastcall Backup(String Location, TStatusBar* StatusBar)
   {
   	using Tools::s_S;
      using Tools::load_str;

      using ToolsGUI::erOperr;
      using ToolsGUI::CopyFile;

   	using TourMaster::GoTour;

      using GoDraw::AppPath;
      using GoDraw::Backup;
      using GoDraw::GoDrawIni;

      try
      {
      	String Sep = String(Tools::sSep);

         GoDraw::ShowTourInfo(StatusBar, IDM_BackingUp);

      	// TOURNAMENT FILE
         if(GoTour.State >= Tours::tsTourStart)
         {
   			String TourFileName   = Tools::s_S(GoTour.FileName);
            String BackupFileName = Location + Sep + ExtractFileName(TourFileName);

            if(!CopyFile(TourFileName, BackupFileName))
            	Throw(ToolsGUI::erOperr, IDM_FileBackup, IDM_NoBackupTour);
         }

         // INITIALISATION FILES
         {
         	String Source[6], Sink[6];

            bool CopyFailed = false;

            Source[0] = AppPath +  Sep + s_S(GoDraw::GoDrawIni);
            Sink[0]   = Location + Sep + s_S(GoDraw::GoDrawIni);

            Source[1] = AppPath +  Sep + s_S(GoDraw::GoTourList);
            Sink[1]   = Location + Sep + s_S(GoDraw::GoTourList);

            Source[2] = AppPath +  Sep + s_S(GoDraw::GoClubCode);
            Sink[2]   = Location + Sep + s_S(GoDraw::GoClubCode);

            Source[3] = AppPath +  Sep + s_S(GoDraw::GoClubCodeR);
            Sink[3]   = Location + Sep + s_S(GoDraw::GoClubCodeR);

            Source[4] = AppPath +  Sep + s_S(GoDraw::GoLandCode);
            Sink[4]   = Location + Sep + s_S(GoDraw::GoLandCode);

            Source[5] = AppPath +  Sep + s_S(GoDraw::GoPlayers);
            Sink[5]   = Location + Sep + s_S(GoDraw::GoPlayers);

            for(int f=0; f<6; f++)
            	if(!CopyFile(Source[f], Sink[f]))
               	CopyFailed = true;

            if(CopyFailed)
            	GoDraw::ShowTourInfo(StatusBar, IDM_NoBackupGDI);
            else
            	GoDraw::ShowTourInfo(StatusBar, IDM_BackedUp);
         }
      }
      catch(...)
      {
      	ShowMessage(erOperr, load_str(IDM_NoBackupAct));

         GoDraw::ShowTourInfo(StatusBar, IDM_NoBackup);
      }
   }

   void __fastcall BackupCmd(TStatusBar* StatusBar)
   {
   	using TourMaster::GoTour;

      if(GoTour.State < Tours::tsTourStart)
      {
      	ToolsGUI::OperatorError(IDM_EmptyBackup); return;
      }

      String Location = PrefsMaster::PrefsTube.BackupPath;

      if(!DirectoryExists(Location))
      {
      	ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDM_NoBackupLoc));

         return;
      }

      Backup(Location, StatusBar);
   }

   void __fastcall SaveCopyCmd(TSaveDialog* SaveCopyDlg, TStatusBar* StatusBar)
   {
   	using TourMaster::GoTour;

      if(SaveCopyDlg->FileName.IsEmpty())
      	SaveCopyDlg->FileName = Tools::s_S(GoTour.FileName);

   	if(SaveCopyDlg->Execute())
      {
      	String TourFile = Tools::s_S(GoTour.FileName);

         String TourCopy = SaveCopyDlg->FileName;

         if(ToolsGUI::CopyFile(TourFile, TourCopy))
         	GoDraw::ShowTourInfo(StatusBar, IDM_SavedCopyAs);
         else
         	ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDM_NoCopyAs));
      }
   }
   //---------------------------------------------------------------------------

}
namespace FileMUI                                  // IMPORT:
{                                                  // ENTER ALL PLAYERS IN A
   void __fastcall ImportCmd                       // TAB-DELIMITED TEXT FILE
   (
      TOpenDialog* ImportDlg,
      TComboBox* RoundBox,
      RegGUI::GRegGrid* RegGrid,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using Tools::S_s;
      using Tools::s_S;

   	using ToolsGUI::EditCell;
      using ToolsGUI::EditingGrid;

      using TourMaster::GoTour;

   	// DEACTIVATE GRIDS
      if(EditCell)
         if(EditingGrid(PairPane->ResGrid))
         	EditCell->HardDismount();

      PlayerMaster::EditPlayerTube.State = PlayerMaster::epiVirgin;
      
      if(ImportDlg->Execute())
      {
         GoDraw::AutoBackup();

      	RegGUI::ImportPlayerList(S_s(ImportDlg->FileName), RoundBox, RegGrid, PairPane);

         ::UpdateEditLists();
		}
   }
}
//------------------------------------------------------------------------------



