////////////////////////////////////////////////////////////////////////////////
// MODULE GoDrawForm:       GoDraw Application        G.Kaniuk 05/04/02
//
#include <FileCtrl.hpp>
#include <vcl.h>
#include <stdlib.h>
#include <time.h>

#pragma hdrstop

#include "TourMaster.h"
#include "MenuMaster.h"

#include "ETourRES.h"
#include "GoErrMsgRES.h"
#include "GoInfoRES.h"
#include "GoFormRES.h"
#include "GoDrawRES.h"
#include "EdPlayerRES.h"
#include "GoDrawFormIDH.h"

#include "ETour.h"

#include "GraphicsAPI.h"
#include "PrinterAPI.h"
#include "PrefsMaster.h"
#include "PlayerMaster.h"

#include "RegListsGUI.h"
#include "RegPanelGUI.h"
#include "ExportGUI.h"
#include "GoDrawGUI.h"

#include "FileMUI.h"
#include "SettingsMUI.h"
#include "TourMUI.h"
#include "PrintMUI.h"
#include "HelpMUI.h"

#include "PairPanelGUI.h"

#include "GoDrawForm.h"
//
//---------------------------------------------------------------------------
//
#pragma package(smart_init)
#pragma resource "*.dfm"
//
//---------------------------------------------------------------------------
// SERVICES
//
TGoDrawFrm *GoDrawFrm;

static void Throw(ToolsGUI::EType etype, short idevent, short idmsg)
{
   ToolsGUI::ThrowError(etype, IDE_GoDrawForm, idevent, idmsg);
}

static int Serve(Exception& Err, short idevent)
{
   return ToolsGUI::Serve(Err, IDE_GoDrawForm, idevent);
}

void __fastcall SetFormText(void)
{
	GoDrawFrm->SetFormText();
}

void __fastcall ClearTourInfo()
{
	GoDrawFrm->ClearTourInfo();
}

void __fastcall ShowTourStatus(int messid)
{
	TourMaster::GoTour.PostTourInfo(messid);

	GoDrawFrm->ShowTourStatus();
}

void __fastcall ShowTourInfo(int messid)
{
   GoDrawFrm->ClearTourInfo();

	TourMaster::GoTour.PostTourInfo(messid);

	GoDrawFrm->ShowTourInfo();
}

void __fastcall ShowMsg(const char* msg)						// GK 6.5.1
{
   GoDrawFrm->ShowMsg(msg);
}

void __fastcall SetGridColour(void)
{
	using PrefsMaster::PrefsTube;

	GoDrawFrm->RegGrid->Color  = PrefsTube.PaneColor;
   GoDrawFrm->PairGrid->Color = PrefsTube.PaneColor;
   GoDrawFrm->ResGrid->Color  = PrefsTube.PaneColor;
}

void __fastcall TGoDrawFrm::SetHelpContextID(void)
{
   FindBtn->HelpContext     	= IDH_REGPANEL_Find;
   RegAllBtn->HelpContext 		= IDH_REGPANEL_RegAll;
   UnRegAllBtn->HelpContext 	= IDH_REGPANEL_UnRegAll;
   FastRegBtn->HelpContext 	= IDH_REGPANEL_FastReg;
   CloseBtn->HelpContext 		= IDH_REGPANEL_Close;
   PrRegBtn->HelpContext 		= IDH_REGPANEL_PrReg;
   ReOpenBtn->HelpContext 		= IDH_REGPANEL_ReOpen;
   RegAllBtn->HelpContext 		= IDH_REGPANEL_Restore;
   RoundBox->HelpContext 		= IDH_REGPANEL_Round;

   PairBtn->HelpContext 		= IDH_PAIRPANEL_Draw;
   ManualBtn->HelpContext 		= IDH_PAIRPANEL_Manual;
   EditBtn->HelpContext 		= IDH_PAIRPANEL_Edit;
   SaveBtn->HelpContext 		= IDH_PAIRPANEL_Save;
   CancelBtn->HelpContext 		= IDH_PAIRPANEL_Cancel;
   PrDrawBtn->HelpContext 		= IDH_PAIRPANEL_PrDraw;
   PrRankBtn->HelpContext 		= IDH_PAIRPANEL_PrRank;
   PrPrizesBtn->HelpContext 	= IDH_PAIRPANEL_PrPrizes;

 	MacBox->HelpContext 			= IDH_BARS_MacMahon;
 	KomiBox->HelpContext 		= IDH_BARS_Handicap;
 	NoviceBox->HelpContext 		= IDH_BARS_Novice;
}
//---------------------------------------------------------------------------

// DISPLAY TOOLS
void __fastcall TGoDrawFrm::ClearTourInfo()
{
   TourMaster::GoTour.ClearTourInfo();

   StatusBar->Panels->Items[0]->Text = Tools::sEmpty;
}

void __fastcall TGoDrawFrm::ShowTourInfo()
{
   using TourMaster::GoTour;

   if(GoTour.TourInfo() <= 1) return;

   StatusBar->Panels->Items[0]->Text = LoadStr(GoTour.TourInfo());
}

void __fastcall TGoDrawFrm::ShowTourInfo(String Message)
{
   using TourMaster::GoTour;

   GoTour.ClearTourInfo();
   GoTour.PostTourInfo(1);

   StatusBar->Panels->Items[0]->Text = Message;
}

void __fastcall TGoDrawFrm::ShowTourStatus()
{
   using TourMaster::GoTour;

   if(GoTour.TourInfo())
   	StatusBar->Panels->Items[0]->Text = LoadStr(GoTour.TourInfo());

   StatusBar->Panels->Items[1]->Text = LoadStr(IDI_SVirgin + GoTour.State);
}

void __fastcall TGoDrawFrm::ShowMsg(const char* msg)		 // GK 6.5.1
{
   using TourMaster::GoTour;

   GoTour.ClearTourInfo();

   StatusBar->Panels->Items[0]->Text = Tools::s_S(msg);
}


void __fastcall TGoDrawFrm::SetFormText()
{
	using Tools::s_S;
   using Tools::sMinus;
   using Tools::sSpace;
   using Tools::sBra;
   using Tools::sKet;

   using TourMaster::GoTour;
   using PlayerMaster::EditPlayerTube;

   if(GoTour.State > Tours::tsVirgin)
   {
		String NameYear = s_S(GoTour.TourName).Trim() + sMinus + String(GoTour.Year);

	   Caption = Tools::s_S(GoDraw::GoDrawCap) + sSpace + NameYear + sSpace;
	   Caption = Caption + String(sSpace) + sBra + s_S(GoTour.FileName) + sKet;
   }
   else
   	Caption = Tools::s_S(GoDraw::GoDrawCap);

   // UPDATE THE ROUND NUMBER DISPLAY
   {
         if(GoTour.Round>0)
            RoundBox->ItemIndex = GoTour.Round-1;
         else
            RoundBox->Clear();

         RoundBox->SelLength = 0;
         RoundBox->SelStart = 0;
   }

   GoDraw::SetPopulations(PopGroup);
   GoDraw::SetBars(BarGroup);
}


void __fastcall LoadPairGrid(void)
{
	GoDrawFrm->PairGrid->LoadFromTour();
}

void __fastcall LoadResultGrid(void)
{
	GoDrawFrm->ResGrid->LoadFromTour();
}

void __fastcall LoadPair(int board)
{
	GoDrawFrm->PairGrid->LoadPair(board);
}

void __fastcall LoadResult(int board)
{
	GoDrawFrm->ResGrid->LoadResult(board);
}
//------------------------------------------------------------------------------
//
// COMMAND CONTROL
//
void __fastcall DisableRegPanel(void)
{
	for(int c=0; c<GoDrawFrm->RegPanel->ControlCount; c++)
   {
   	TControl* RegPanelControl = GoDrawFrm->RegPanel->Controls[c];

      RegPanelControl->Enabled = false;
   }
}

void __fastcall EnableRegPanel(void)
{
	for(int c=0; c<GoDrawFrm->RegPanel->ControlCount; c++)
   {
   	TControl* RegPanelControl = GoDrawFrm->RegPanel->Controls[c];

      RegPanelControl->Enabled = true;
   }

   GoDrawFrm->RoundBox->Enabled = false;
}

void __fastcall TGoDrawFrm::BeginCommand(void) 				// Commands which do not change state
{
	using TourMaster::GoTour;

   ClearTourInfo();

   OldState = GoTour.State;

   GoTour.OldRegOrder =  GoTour.SortOrder;	// GK REGSORT 2015
   GoTour.OldRegSense =  GoTour.SortSense;
}

void __fastcall TGoDrawFrm::BeginCommand(int IdCmd)
{
	using TourMaster::GoTour;
   using TourMaster::EnabledCommands;

   BeginCommand();

	if(!EnabledCommands[IdCmd][GoTour.State])
   	ToolsGUI::BugReport(IDE_GoDrawForm, IDE_BeginCommand, IDE_BadCommand);
}

void __fastcall TGoDrawFrm::EndCommand(int idCmd, int regControl)
{
	using TourMaster::GoTour;

   if(idCmd>TourMaster::cmdNone)
		if(GoTour.State != OldState)
      	EnableCriticalControls();

   // GK REGSORT 2015
   // Commands with  regcon=regOldSort might need to temporarily change the register order,
   // as well as making minor player edits. For these we restore the original sort order,
   // and then re-load the register.
   //
   // Commands with  regcon=regNewSort have sorted the register into a required new order, and
   // loaded the grid. For these we just need to repaint.

   if(GoTour.SortOrder != GoTour.OldRegOrder ||  GoTour.SortSense != GoTour.OldRegSense)
   {
   	if(regControl==TourMaster::regOldSort)												// Restoretheold order
      	RegGrid->LoadSorted(GoTour.OldRegOrder,  GoTour.OldRegSense);
      RegGrid->Repaint();
   }

   EdPlayerFrm->SetAction(!EdPlayerFrm->ReadOnly);

   if(EdPlayerFrm->Visible)EdPlayerFrm->Update();

   SetFormText();
   ShowTourStatus();
}


void __fastcall BeginCommand(int IdCmd)		// Global versions needed
{                                            // by register and results
	GoDrawFrm->BeginCommand(IdCmd);
}

void __fastcall BeginCommand(void)
{
	GoDrawFrm->BeginCommand();
}

void __fastcall EndCommand(int IdCmd)
{
	GoDrawFrm->EndCommand(IdCmd);
}

bool __fastcall TGoDrawFrm::IgnoreClick(void)
{
	bool ignored = IgnoringClick;

   IgnoringClick = false;

   return ignored;
}




//------------------------------------------------------------------------------
// EDIT PLAYER FORM SUPPORT
//------------------------------------------------------------------------------
void __fastcall TGoDrawFrm::EditPlayer(const int playerow)
{
	using Tools::load_str;

	using ToolsGUI::ShowMessage;
   using ToolsGUI::erBug;

	using PlayerMaster::EditPlayerTube;
   using TourMaster::GoTour;
   
   TEditPlayerFrm* EditPlayerFrm = GoDrawFrm->EdPlayerFrm;

   if(!EditPlayerFrm)
   {
      ShowMessage(erBug, load_str(IDB_NoEditFrm));

      return;
   }

   // CHECK THE player RANGE
   if(playerow<1 || playerow >(int)GoTour.Players+1)
   {
   	ShowMessage(erBug, load_str(IDB_CorruptPlayerNum));

      return;
   }

   int& PlayerRow = EditPlayerTube.PlayerRow;
   int& PlayerCol = EditPlayerTube.PlayerCol;

   RegGrid->InvalidateRegRow(PlayerRow);

   EditPlayerTube.LoadPlayer(playerow);

   // THE FORM ITSELF SUPPLIES A METHOD TO SET ALL ITS TEXT
   EditPlayerFrm->Load();
   EditPlayerFrm->SetAction(!EditPlayerFrm->ReadOnly);

   if(!EditPlayerFrm->Visible)
   	EditPlayerFrm->Show();
   else
   	EditPlayerFrm->Update();

   // TELL RegGrid WE ARE NOW EDITING
   RegGrid->Status = EditPlayerFrm->ReadOnly ? RegGUI::rsDormant : RegGUI::rsEditing;
   RegGrid->SelectGridCell(PlayerCol, PlayerRow);
   RegGrid->InvalidateRegRow(PlayerRow);

   RegGrid->Repaint();
}

void __fastcall ClearGridSelections(void) 		// GK 6.3.9 
{
	GoDrawFrm->PairGrid->NoCellSelected();
   GoDrawFrm->ResGrid->NoCellSelected();
}

void __fastcall ViewPlayer(const int blackwhite, const int player)
{
	using PlayerMaster::EditPlayerTube;

   EditPlayerTube.PlayerCol = 2;		// Stops Beep() in SelectGridCell by selecting Name

	GoDrawFrm->EditPlayer(player);

   GoDrawFrm->EdPlayerFrm->ReadOnly = true;
   GoDrawFrm->EdPlayerFrm->SetAction(false);

   String Title = String();

   if(blackwhite<0) Title = LoadStr(IDB_BlackPlayer);
   if(blackwhite>0) Title = LoadStr(IDB_WhitePlayer);

   GoDrawFrm->EdPlayerFrm->Caption = Title + EditPlayerTube.Name;
}

void __fastcall TGoDrawFrm::RegGridLoadPlayer(Tours::GPlayerPtr player, const int row)
{
	RegGrid->LoadPlayer(player, row);
}

void __fastcall TGoDrawFrm::RegGridEndPlayerEdit(const bool newplayer)
{
	RegGrid->InvalidateRegRow(RegGrid->Row);

	RegGrid->EndPlayerEdit(newplayer);
}

void __fastcall ReloadPairResult(void)
{
	using PlayerMaster::EditPlayerTube;

   const int b = DrawMaster::FindBoard(EditPlayerTube.Player.Pid);

   if(b>=0)
   {
      GoDrawFrm->PairGrid->LoadPair(b);
      GoDrawFrm->ResGrid->LoadResult(b);
   }
}

void __fastcall EditPlayer(const int player)
{
	GoDrawFrm->EditPlayer(player);
}

void __fastcall EditPlayerAndFocus(const int player)
{

	GoDrawFrm->EditPlayer(player);
   GoDrawFrm->EdPlayerFrm->SetBoxFocus();
}

void __fastcall RegGridLoadPlayer(Tours::GPlayerPtr player, const int row)
{
	GoDrawFrm->RegGridLoadPlayer(player, row);
}

void __fastcall RegGridEndPlayerEdit(const bool newplayer)
{
	GoDrawFrm->RegGridEndPlayerEdit(newplayer);
}

void __fastcall TGoDrawFrm::SetRegGridStatusDormant(void)
{
	RegGrid->Status = RegGUI::rsDormant;

   RegGrid->InvalidateRegRow(RegGrid->Row);
}

void __fastcall SetRegGridStatusDormant(void)
{
	GoDrawFrm->SetRegGridStatusDormant();
}

bool __fastcall ValidPlayerEdit(void)
{
	return GoDrawFrm->EdPlayerFrm->ValidPlayerEdit();
}

bool __fastcall PlayerTextChanged(void)
{
	return GoDrawFrm->EdPlayerFrm->TextChanged();
}

bool  __fastcall UnsavedPlayerEdit(const int idmessage)
{
	return GoDrawFrm->EdPlayerFrm->UnsavedPlayerEdit(idmessage);
}

bool  __fastcall ClosedPlayerEdit(const int idmessage)
{
	return GoDrawFrm->EdPlayerFrm->ClosedPlayerEdit(idmessage);
}

bool __fastcall TGoDrawFrm::DismissedPlayerEdit(const int idmessage)
{
	if(!EdPlayerFrm->Visible)
   	return true;

	if(EdPlayerFrm->UnsavedPlayerEdit(idmessage))
   	return false;
   else
   	EdPlayerFrm->Close();

   return true;
}

bool __fastcall DismissedPlayerEdit(const int idmessage)
{
	return GoDrawFrm->DismissedPlayerEdit(idmessage);
}
void __fastcall TGoDrawFrm::LoadPlayerIDs(void)
{
	TEditPlayerFrm* EdPlayerForm = GoDrawFrm->EdPlayerFrm;

	if(EdPlayerForm)
   	PlayerMaster::EditPlayerTube.LoadPlayerIDs(EdPlayerForm->IdBox->Items);
}

void __fastcall LoadPlayerIDs(void)
{
	GoDrawFrm->LoadPlayerIDs();
}

void __fastcall TGoDrawFrm::SetIdBoxItemIndex(const int row)
{
	TEditPlayerFrm* EdPlayerForm = GoDrawFrm->EdPlayerFrm;

	if(EdPlayerForm)
   	EdPlayerForm->IdBox->ItemIndex = row-1;
}

void __fastcall SetIdBoxItemIndex(const int row)
{
	GoDrawFrm->SetIdBoxItemIndex(row);
}

String __fastcall TGoDrawFrm::ShortPlayer(const int row)
{
	return RegGrid->ShortPlayer(row);
}

String __fastcall ShortPlayer(const int row)
{
	return GoDrawFrm->ShortPlayer(row);
}

void __fastcall ResumePlayerAndFocus(void)
{
	GoDrawFrm->EdPlayerFrm->Show();;
   GoDrawFrm->EdPlayerFrm->SetBoxFocus();
}

void __fastcall TGoDrawFrm::ChangeToPlayer(const int row)
{
	// THE FORM HAS ALREADY BEEN LOADED, BUT USER NOW
   // CHANGES TO ANOTHER PLAYER BY CLICKING ON A DIFFERENT
   // ROW AND POSSIBLY A DIFFERENT COLUMN IN THE GRID
   RegGrid->InvalidateRegRow(RegGrid->Row);
   	EdPlayerFrm->Show();
   	EdPlayerFrm->ChangeToPlayer(row);
   RegGrid->InvalidateRegRow(RegGrid->Row);

   RegGrid->Status = EdPlayerFrm->ReadOnly ? RegGUI::rsDormant : RegGUI::rsEditing;
}

void __fastcall ChangeToPlayerAndFocus(void)
{
	const int Row = GoDrawFrm->RegGrid->Row;

   int& PlayerRow = PlayerMaster::EditPlayerTube.PlayerRow;

	GoDrawFrm->RegGrid->SelectGridCell(GoDrawFrm->RegGrid->Col, Row);

   GoDrawFrm->RegGrid->InvalidateRegRow(PlayerRow);

	GoDrawFrm->ChangeToPlayer(Row);

   GoDrawFrm->RegGrid->InvalidateRegRow(PlayerRow);

   GoDrawFrm->EdPlayerFrm->SetBoxFocus();
}

void __fastcall LoadEditLists(void)
{
	GoDrawFrm->EdPlayerFrm->LoadLists();
}

void __fastcall UpdateEditLists(void)
{
	GoDrawFrm->EdPlayerFrm->UpdateLists();
}

void __fastcall ReloadPlayer(void)
{
	// THE PLAYER DETAILS LOADED IN EDIT FORM MAY HAVE CHANGED
   // FOLLOWING A DRAW OR RESULT EDIT
	using PlayerMaster::EditPlayerTube;

   TEditPlayerFrm* EditPlayerForm = GoDrawFrm->EdPlayerFrm;

	if(EditPlayerTube.State != PlayerMaster::epiVirgin)
   {
      EditPlayerTube.LoadPlayer(EditPlayerTube.PlayerRow);

   	EditPlayerForm->Load();

   	EditPlayerForm->SetAction(!EditPlayerForm->ReadOnly);
   }
}
//---------------------------------------------------------------------------
//
// GODRAW FORM STARTUP
//
__fastcall TGoDrawFrm::TGoDrawFrm(TComponent* Owner):TForm(Owner)
{
	StatusBar->Panels->Items[0]->Text = String();
}

void __fastcall TGoDrawFrm::FormCreate(TObject *Sender)
{
   using namespace GoDraw;

   using PrefsMaster::PrefsTube;
   using TourMaster::GoTour;
   using Tools::S_s;
   using Tools::s_S;

   // FormResize() is spuriously triggered during the startup process
   // UsingForm is kept false until we are really ready
   UsingForm = false;
   MaximisedForm = false;
   
   // HELP
   Application->HintHidePause = 6000;

   Application->HelpFile = HelpFilePath(s_S(GoDrawHlp));

   SetHelpContextID();

   // ERROR RECORDING
   ::AppLog.StartUp(Version, GoDrawLog);

   // PARSERS
   Tools::set_lexical_classes();

   // INITIALISE PRINTER
   GoDraw::StartPrinter();

   // CREATE PANES
   try
   {
      RegGrid   = new RegGUI::GRegGrid(RegGroup,  StatusBar, FastRegBtn);
      PairGrid  = new PairGUI::GPairGrid(PairGroup,  StatusBar);
      UpsetList = new UpsetGUI::GUpsetList(PairGroup);
      ResGrid   = new ResGUI::GResGrid(PairGroup,  StatusBar, PopGroup);

      PairPane  = new PaneGUI::GPairPane(PairGrid, ResGrid, UpsetList, ImageList, Timer);

      EdPlayerFrm = new TEditPlayerFrm(this);
      SeekFrm     = new TSeekFrm(this, RegGrid, RegMaster::FindTube);

      RegPopup    = new TourMUI::MRegPopup(RegGrid);
      PairPopup   = new TourMUI::MPairPopup(this, PairPane);

      RegGrid->PopupMenu  = RegPopup;
      PairGrid->PopupMenu = PairPopup;
      ResGrid->PopupMenu  = PairPopup;

      RegGrid->ConnectGroups(PopGroup, BarGroup);

      PairGroup->ControlStyle = PairGroup->ControlStyle<<csDisplayDragImage;

      ToolsGUI::ClearEditCell();
   }
   catch(...)
   {
      Throw(ToolsGUI::erOperr, IDE_NewPaneFields, IDE_NoMemory);
   }


   // INITIAL DIRECTORY FOR TOURNAMENT FILES
   StartToursDirectory(OpenDlg);

   StartUpForm();

   Repaint();

   if(!::PostMessage(Handle, UWM_END_FORM_CREATE, 1, 0))
   	Throw(ToolsGUI::erBug, IDE_MainFormCreate, IDE_NoPost);
}

void __fastcall TGoDrawFrm::EndFormCreate(TMessage& Message)
{
	using PlayerMaster::EditPlayerTube;
	using PrefsMaster::PrefsTube;
   using TourMaster::GoTour;
   using GoDraw::FilePath;
   using GoDraw::GoTours;
   using FileMUI::OpenTourFile;
   using Tools::S_s;

   // FONTS The preferences specify the
   // required printer font. Once loaded we need to check that the font still exists,
   // and if it does not then revert to system font -SelectFontName("");

   FastRegBtn->Caption = LoadStr(IDF_StartFastReg);
   
   Repaint();

   Message.Result = 1;

   // LOAD TOURNAMENT FILE
   try
   {
      PrefsTube.LoadFromFile();									// THIS FILE TO BE KEPT SHORT

      PrefsTube.AppPath = GoDraw::AppPath;

      if(PrefsTube.BackupPath.IsEmpty())						// ENSURE BACKUP PATH IS VALID
      	PrefsTube.BackupPath = FilePath(GoTours);

      if(!DirectoryExists(PrefsTube.BackupPath))
      	PrefsTube.BackupPath = FilePath(GoTours);

      if(!PrefsTube.TourFile.IsEmpty())						// LOAD THE RECENTLY USED TOURNAMENT
      	OpenTourFile
         (
         	PrefsTube.TourFile, RoundBox,
            RegGrid, PairPane
         );
      else
      {
      	GoTour.NewStart(Tours::tsVirgin);
      	GoTour.PopulationCount();

			FileMUI::StartClearGrids(RegGrid, PairPane);		// Show headers

         GoTour.ClearTourInfo();
      	GoTour.PostTourInfo(IDI_Ready);
      }


      if(EdPlayerFrm)												// LOAD THE LISTS FOR ALL THE COMBOS
      {
      	EditPlayerTube.PlayerCount = GoTour.Players;
         EditPlayerTube.LoadTour();

         EdPlayerFrm->ConnectFindTubeLists();
      	EdPlayerFrm->LoadLists();
         EdPlayerFrm->UpdateLists();
      }

      SaveCopyDlg->InitialDir = OpenDlg->InitialDir;
      SaveCopyDlg->FileName = String();

      GoDraw::StartBars(MacBox, KomiBox, NoviceBox);

      SetFormText();

   	SetGridColour();

      // INITIALISE RANDOM NUMBER GENERATOR	V6.5.0
      time_t timenow = 0;
      std::srand((unsigned)std::time(&timenow));

      // SET UP INITIAL STATE AND ENABLE REQUIRED BUTTONS
   	TagCriticalControls();

   	EnableCriticalControls();

      PairPanelGUI::SetCancelBtnCaption(CancelBtn);

      // DEFINE THE FORM SIZE
      UsingForm = true;

      if(PrefsTube.ReuseWindow)
      {
         if(PrefsTube.NormalWindow())				// Form was manually sized, but
         {                                      // not maximised
         	MaximisedForm = false;

         	BoundsRect = PrefsTube.WindowLocn;
            	ReBuildForm();
            PrefsTube.WindowLocn = BoundsRect;

            PrefsTube.SaveToFile();

      		ShowTourStatus();
         }
         else												// Form was manually maximised
         {
         	MaximisedForm = true;

         	WindowState = wsMaximized;          // Triggers FormResize
         }
      }
      else													// Form has design size
      {
      	ReBuildForm();

      	PrefsTube.WindowLocn = BoundsRect;

      	ShowTourStatus();
      }

      PrefsTube.SaveToFile();

   	RegGrid->SelectGridCell(RegGUI::rfName, RegGrid->RowCount-1);
   }
   catch(ToolsGUI::ETour& Estart)
   {
   	Serve(Estart, IDE_MainFormCreate);
   }
   catch(...)
   {
      std::string fatalerror = ToolsGUI::FormWin32Error(IDE_LoadRegLists, IDE_MainFormCreate);

      throw Exception(Tools::s_S(fatalerror));
   }
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FormDestroy(TObject *Sender)
{
   UsingForm = false;

   delete EdPlayerFrm;
   delete PairPane;
   delete ResGrid;
   delete PairGrid;
   delete RegGrid;
   delete UpsetList;

   delete SeekFrm;
   delete RegPopup;
   delete PairPopup;

   PrinterAPI::DeletePrinter();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FormResize(TObject *Sender)
{
	using PrefsMaster::PrefsTube;

   if(UsingForm)
   {
		ReBuildForm();

      if(MaximisedForm)
      	PrefsTube.WindowLocn = TRect(-1,-1,-1,-1);
      else
      	PrefsTube.WindowLocn = BoundsRect;

   	PrefsTube.SaveToFile();

      ShowTourStatus();
   }
}

void __fastcall TGoDrawFrm::WndProc(TMessage& Message)
{
	if(Message.Msg == WM_SYSCOMMAND)
      if(Message.WParam == SC_MAXIMIZE)
      	MaximisedForm = true;

   TForm::WndProc(Message);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FormClose(TObject *Sender, TCloseAction &Action)
{
	using TourMaster::GoTour;
   using PrefsMaster::PrefsTube;

   if(WindowState == wsMaximized)
   	PrefsTube.WindowLocn = TRect(-1,-1,-1,-1);
   else
      PrefsTube.WindowLocn = BoundsRect;

   if(FileMUI::CloseGodraw(RegGrid))
   {
   	if(GoTour.State == Tours::tsTourDone)			// Fresh start next time
      	PrefsTube.FinishTourFile();
      else
         PrefsTube.SaveToFile();

   	UsingForm = false;									// For some reason FormResize is triggered at form closure.
                                                   // This cancels RebuildForm

      Application->HelpCommand(HELP_QUIT, 0);		// Close any open help

   	::AppLog.Add(Tools::load_str(IDS_Shutdown));	// Report to error log

      Action = caFree;										// Form closes
   }
   else
   	Action = caNone;										// Form stays
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FormShow(TObject *Sender)
{
	if(FindBtn->Enabled)FindBtn->SetFocus();
}

void __fastcall TGoDrawFrm::ComboBoxDropDown(TObject *Sender)
{
	using PlayerMaster::EditPlayerTube;
   using PlayerMaster::epiVirgin;

   if(UnsavedPlayerEdit(IDF_OkCancelEdit))
      IgnoringClick = true;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//
// FILE MENU
//
void __fastcall TGoDrawFrm::FileOpenClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

   BeginCommand(TourMaster::cmdOpen);
      FileMUI::OpenCmd(OpenDlg, RoundBox, RegGrid, PairPane);
   EndCommand(TourMaster::cmdOpen);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileNewClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

   BeginCommand(TourMaster::cmdNew);
      FileMUI::NewCmd(RoundBox, RegGrid, PairPane);
   EndCommand(TourMaster::cmdNew);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileCloseClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

   BeginCommand(TourMaster::cmdCloseT);
      FileMUI::CloseCmd(RegGrid, PairPane);
   EndCommand(TourMaster::cmdCloseT);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::SaveCopyAsClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

	ClearTourInfo();
   	Screen->Cursor = crHourGlass;
			FileMUI::SaveCopyCmd(SaveCopyDlg, StatusBar);
      Screen->Cursor = crDefault;
 	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileBackupClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

	ClearTourInfo();
   	Screen->Cursor = crHourGlass;
			FileMUI::BackupCmd(StatusBar);
      Screen->Cursor = crDefault;
 	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileImportClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

   // ImportCmd sorts reg in PID order	GK REGSORT 2015
   BeginCommand(TourMaster::cmdEnter);
      FileMUI::ImportCmd(ImportDlg, RoundBox,RegGrid, PairPane);
   EndCommand(TourMaster::cmdEnter, TourMaster::regNewSort);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileRankListClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

	BeginCommand();
		ExportGUI::ExportRankCmd(ExportDlg);
	EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileWebResultsClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

   BeginCommand();
		ExportGUI::ExportWebCmd(ExportDlg);
   EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileRatingsClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

   BeginCommand();
		ExportGUI::ExportRatingCmd(this, ExportDlg);
   EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FileExitClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_FilePlayerEd)) return;

	Close();		// Fires the OnClose Event implemented in FormClose
}
//---------------------------------------------------------------------------
//
// SETTINGS MENU
//

void __fastcall TGoDrawFrm::SettingsRulesClick(TObject *Sender)
{
	using TourMaster::GoTour;

	if(!DismissedPlayerEdit(IDI_SettPlayerEd)) return;

	ClearTourInfo();
   {
      OldState = GoTour.State;

		SettingsMUI::RulesCmd(this, RegGrid, PairGrid, ResGrid);

      if(GoTour.State != OldState)EnableCriticalControls();

   	SetFormText();
   }
	ShowTourStatus();
}

void __fastcall TGoDrawFrm::SettingsTrainingClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_SettPlayerEd)) return;

	ClearTourInfo();
		SettingsMUI::TrainingCmd(this, ResGrid);
      SetFormText();
	ShowTourStatus();
}

void __fastcall TGoDrawFrm::SettingsPreferencesClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_SettPlayerEd)) return;

	ClearTourInfo();
		SettingsMUI::PrefsCmd(this, PairGrid);
	ShowTourStatus();
}
//---------------------------------------------------------------------------
//
// TOOLS MENU
//
void __fastcall TGoDrawFrm::ToolsMuClick(TObject *Sender)
{
   using ToolsGUI::EditingCell;
   using TrainMaster::MTrainTube;
   using TourMaster::GoTour;
   using Tours::stMcMahon;
   using Tours::stBarLow;

   const Tours::style_t Style = GoTour.Rules.Style;

	const bool CanSimClr  = !EditingCell() && MTrainTube().CanChangeResults();

   ToolsSimulate->Enabled = CanSimClr;
   ToolsClear->Enabled    = CanSimClr;
   ToolsGroups->Enabled   = Style==stMcMahon || Style==stBarLow;
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ToolsGroupsClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_ToolPlayerEd)) return;

	ClearTourInfo();
   	TourMUI::GroupsForm(this, RegGrid, 0);
   ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ToolsRankClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_ToolPlayerEd)) return;

   BeginCommand();
   	TourMUI::RankListForm(this, RegGrid);
   EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ToolsPrizesClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_ToolPlayerEd)) return;

   BeginCommand();
   	TourMUI::PrizeListForm(this, RegGrid);
   EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------


void __fastcall TGoDrawFrm::ToolsSimClick(TObject *Sender)
{
 	if(!DismissedPlayerEdit(IDI_ToolPlayerEd)) return;

   BeginCommand(TourMaster::cmdResult);
		TourMUI::SimulateResultsCmd(ResGrid);
   EndCommand(TourMaster::cmdResult);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ToolsClearClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_ToolPlayerEd)) return;

	BeginCommand(TourMaster::cmdResult);
		TourMUI::ClearResultsCmd(ResGrid);
   EndCommand(TourMaster::cmdResult);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ToolsMonitorClick(TObject *Sender)
{
	PairPanelGUI::PairMonitorCmd(this);
}
//---------------------------------------------------------------------------
//
// HELP MENU
//
void __fastcall TGoDrawFrm::HelpOClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_HELP_OVERVIEW); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::HelpMClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_MENUS_INDEX); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::HelpIClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_GODRAW_INDEX); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::HelpRClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_REGED_INDEX); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::HelpEClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_ERRORMESSAGES); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::HelpAboutClick(TObject *Sender)
{
	HelpMUI::ShowAboutForm(this);
}
//------------------------------------------------------------------------------
//
// REGPANEL
//
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FindBtnClick(TObject *Sender)
{
	ClearTourInfo();
		RegPanelGUI::FindCmd(SeekFrm, RegGrid);
	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::FastRegBtnClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   BeginCommand(TourMaster::cmdFastReg);
      RegGrid->ManageFastReg();
   EndCommand(TourMaster::cmdFastReg);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::RegAllBtnClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   BeginCommand(TourMaster::cmdRegAll);
      RegPanelGUI::RegPlayersCmd(RegAllBtn, RegGrid, RegPanelGUI::rpRegAll);
   EndCommand(TourMaster::cmdRegAll);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::UnRegAllBtnClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   BeginCommand(TourMaster::cmdUnRegAll);
      RegPanelGUI::RegPlayersCmd(UnRegAllBtn, RegGrid, RegPanelGUI::rpUnRegAll);
   EndCommand(TourMaster::cmdUnRegAll);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::CloseBtnClick(TObject *Sender)
{
	//if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return; 		// GK REGSORT 2015
   if(!DismissedPlayerEdit(IDF_RegEdClose)) return;

   BeginCommand(TourMaster::cmdClose);
      RegPanelGUI::CloseRegCmd(BarGroup, PopGroup, RegGrid);
   EndCommand(TourMaster::cmdClose, TourMaster::regNewSort);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ReOpenBtnClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_RegEdOpen)) return;

   BeginCommand(TourMaster::cmdReOpen);
      RegPanelGUI::OpenRegCmd(RegGrid);
   EndCommand(TourMaster::cmdReOpen);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::RestoreBtnClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   BeginCommand(TourMaster::cmdRestore);
      RegPanelGUI::RestoreCmd(RegGrid);
   EndCommand(TourMaster::cmdRestore);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::RoundBoxClick(TObject *Sender)
{
	if(IgnoreClick()) return;

   BeginCommand(TourMaster::cmdRound);
      RegPanelGUI::ChangeRoundCmd(RoundBox, RegGrid, PairPane);
   EndCommand(TourMaster::cmdRound);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::BarClick(TObject *Sender)
{
	TComboBox* BarCom = (TComboBox*)Sender;

   int cmdBar;

   switch(BarCom->Tag)
   {
      case GoDraw::McBarEd: cmdBar = TourMaster::cmdMacBar;	break;
      case GoDraw::KoBarEd: cmdBar = TourMaster::cmdKomBar;	break;
      case GoDraw::NuBarEd: cmdBar = TourMaster::cmdNovBar;	break;

      default: cmdBar = 0;
   }

   BeginCommand(cmdBar);
      GoDraw::ParseBar(BarGroup, BarCom, StatusBar, RegGrid);
   EndCommand(cmdBar);
}

void __fastcall TGoDrawFrm::BarChange(TObject *Sender)
{
	TComboBox* BarCom = (TComboBox*)Sender;

   int cmdBar;

   switch(BarCom->Tag)
   {
      case GoDraw::McBarEd: cmdBar = TourMaster::cmdMacBar;	break;
      case GoDraw::KoBarEd: cmdBar = TourMaster::cmdKomBar;	break;
      case GoDraw::NuBarEd: cmdBar = TourMaster::cmdNovBar;	break;

      default: cmdBar = 0;
   }

   BeginCommand(cmdBar);
      GoDraw::ShowTourInfo(StatusBar, IDF_BarChanged);					// GK 6.5.1
   EndCommand(cmdBar);
}


//---------------------------------------------------------------------------
// PAIR PANEL
//
//---------------------------------------------------------------------------

void __fastcall  SetPairBtnCaption(void)
{
	PairPanelGUI::SetPairBtnCaption(GoDrawFrm->PairBtn);
}

void __fastcall  SetCancelBtnCaption(void)
{
	PairPanelGUI::SetCancelBtnCaption(GoDrawFrm->CancelBtn);
}

void __fastcall TGoDrawFrm::PairBtnClick(TObject *Sender)
{
	using TourMaster::GoTour;

	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   // GK REGSORT 2015 This terminates a manual pairing
   // In which case the sort order was disturbed by Start ManualPairing
   if(GoTour.State==Tours::tsForcedPair)
   {
   	GoTour.SortOrder = GoTour.OldRegOrder;
      GoTour.SortSense = GoTour.OldRegSense;
   }

   BeginCommand(TourMaster::cmdDraw);
      PairPanelGUI::PairCmd(this, PairBtn, CancelBtn, PairPane);
      RegPanelGUI::LoadRounds(RoundBox);
   EndCommand(TourMaster::cmdDraw);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::ManualBtnClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   BeginCommand(TourMaster::cmdForced);
      PairPanelGUI::ManualPairingCmd(this, PairBtn, CancelBtn, PairPane);
      {
      	const int NewCount = RoundBox->Items->Count+1;
      	RoundBox->Items->Add(String(NewCount));
      	RoundBox->ItemIndex =  NewCount - 1;
      }
   EndCommand(TourMaster::cmdForced);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::EditDrawClick(TObject *Sender)
{
	if(UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

   BeginCommand(TourMaster::cmdEdit);
   	PairPanelGUI::EditDrawCmd(CancelBtn, PairPane);
   EndCommand(TourMaster::cmdEdit);
}

void __fastcall TGoDrawFrm::SaveDrawClick(TObject *Sender)
{
   BeginCommand(TourMaster::cmdSave);

   	PairPanelGUI::SaveDrawCmd(CancelBtn, PairPane);	// MIGHT switch RestoreRegStatus on

      if(TourMaster::GoTour.RestoreRegStatus)
      	RegPanelGUI::RestoreRegStatus(RegGrid);		// WILL switch RestoreRegStatus off

   EndCommand(TourMaster::cmdSave);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::CancelBtnClick(TObject *Sender)
{
	using TourMaster::GoTour;
   using Tours::tsDrawDone;
   using Tours::tsPartDraw;
   
   if(GoTour.State==tsDrawDone || GoTour.State==tsPartDraw)	// Interpret button as Cancel Draw
   {
      BeginCommand(TourMaster::cmdDcancel);
      	PairPanelGUI::CancelDrawCmd(CancelBtn, PairPane);
         RegPanelGUI::LoadRounds(RoundBox);
   	EndCommand(TourMaster::cmdDcancel); 
   }
   else 																		// In ForcedPair or EditNow
   {                                                  		// Interpret button as Cancel Edit
   	BeginCommand(TourMaster::cmdEcancel);           		// Button is disabled in any other state
      	PairPanelGUI::CancelEditCmd(PairBtn, CancelBtn, PairPane);
   	EndCommand(TourMaster::cmdEcancel);
   }
}
//---------------------------------------------------------------------------
//
// PRINTING
//
void __fastcall TGoDrawFrm::PrRegBtnClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

	// GK REGSORT 2015 ClearTourInfo();
   // May disturb reg order.
   // We keep printed reg and screen reg in sync
   BeginCommand();
		PrintMUI::PrintRegisterCmd(RegGrid, false);
   EndCommand(TourMaster::cmdNone, TourMaster::regNewSort);
	// GK REGSORT 2015 ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrDrawBtnClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

	ClearTourInfo();
		PrintMUI::PrintDrawResCmd(TextSmith::drDraw, false);
	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrRankBtnClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

   // GK REGSORT 2015 Restore original reg order
   BeginCommand();
		PrintMUI::PrintRankCmd(false);
   EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrPrizesBtnClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

   // GK REGSORT 2015 Restore original reg order
	BeginCommand();
		PrintMUI::PrintPrizeCmd(false);
	EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopRegClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

   // GK REGSORT 2015 We keep printed reg and screen reg in sync
   BeginCommand();
		PrintMUI::PrintRegisterCmd(RegGrid, true);
   EndCommand(TourMaster::cmdNone, TourMaster::regNewSort);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopDrawClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

	ClearTourInfo();
		PrintMUI::PrintDrawResCmd(TextSmith::drDraw, true);
	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopResultsClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

	ClearTourInfo();
		PrintMUI::PrintDrawResCmd(TextSmith::drResult, true);
	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopRankListClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

   // GK REGSORT 2015 Restore original reg order
   BeginCommand();
		PrintMUI::PrintRankCmd(true);
   EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopPrizesClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

   // GK REGSORT 2015 Restore original reg order
   BeginCommand();
		PrintMUI::PrintPrizeCmd(true);
	EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopCardsClick(TObject *Sender)
{
	if(!DismissedPlayerEdit(IDI_PrinPlayerEd)) return;

   // GK REGSORT 2015 Restore original reg order
   BeginCommand();
		PrintMUI::PrintCardsCmd(this, true);
	EndCommand(TourMaster::cmdNone);
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopTestClick(TObject *Sender)
{
	ClearTourInfo();
		PrintMUI::PrintTestCmd();
	ShowTourStatus();
}
//---------------------------------------------------------------------------

void __fastcall TGoDrawFrm::PrShopSetupClick(TObject *Sender)
{
	ClearTourInfo();
		PrintMUI::PrintSetupCmd();
	ShowTourStatus();
}
//---------------------------------------------------------------------------
//									END
//---------------------------------------------------------------------------

