////////////////////////////////////////////////////////////////////////////////
// MODULE GoDrawGUI:       Application - Wide StartUp Support  G.Kaniuk 05/04/02
//

#include <math.h>
#include <vcl.h>
#include <vcl\filectrl.hpp>
#pragma hdrstop


#include "GoErrMsgRES.h"
#include "GoDrawRES.h"
#include "GoInfoRES.h"
#include "GoFormRES.h"
#include "FileRES.h"


#include "Strings.h"
#include "Format.h"

#include "ETour.h"
#include "PrinterAPI.h"
#include "GraphicsAPI.h"
#include "FilesGUI.h"

#include "TourStream.h"
#include "PrefsMaster.h"
#include "TourMaster.h"
#include "MenuMaster.h"
#include "RegMaster.h"
#include "DrawMaster.h"


#include "GoDrawGUI.h"
//---------------------------------------------------------------------------

#pragma package(smart_init)
//------------------------------------------------------------------------------

extern ToolsGUI::ErrorLog AppLog;

namespace GoDraw                          // EXCEPTIONS
{
   using std::string;

   using ToolsGUI::Forge;
   using ToolsGUI::ETour;
   using ToolsGUI::EType;
   using ToolsGUI::erBug;

   static void Throw(EType etype, short idmodule, short idproc, short idmsg)
   {
   	ToolsGUI::ThrowError(etype, idmodule, idproc, idmsg);
      string message = Forge(etype, idmodule, idproc, idmsg);

      if(etype==erBug)::AppLog.Add(message);

      throw ETour(etype, message);
   }
}

namespace GoDraw                          // GODRAW RUN TIME SETTINGS
{
   using Tools::sEmpty;
   using Tools::s_S;
   using Tools::S_s;

   using ToolsGUI::GRatio;

   String Backup    = "Backup"; 					// Regular backup location
   String Removable = "A:";					   // Can change in preferences
   String BackupExt = ".bak";						// Appended to full Tournament filename

   String AppPath   = GetCurrentDir();       // Program Location

   String FilePath(String FileRoot)          // Files In GoDraw folder
   {
      return AppPath + String(Tools::sSep) + FileRoot;
   }

   String FilePath(string FileRoot)
   {
      return AppPath + String(Tools::sSep) + s_S(FileRoot);
   }

   String AutoBackupPath(string FileRoot)		// ... /*.gdt.bak
   {
   	return s_S(FileRoot) + String(BackupExt);
   }

   void AutoBackup(void)
   {
   	const String TourFile = s_S(TourMaster::GoTour.FileName);
   	const String BackupFile = TourFile + BackupExt;

      if(!ToolsGUI::CopyFile(TourFile, BackupFile))
      	ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDM_NoCopyAs));
   }
}

namespace GoDraw                          // GLOBAL MAGIC CONSTANTS
{
   using std::string;
                                                     // TODO: OBSOLETE
   const int defFormHeight = 600;                     // GoDraw sized at construction
   const int defFormWidth  = 800;
   const int defFontSize   =   8;                     // Main grid font size - Points
   const int defStrideSize =  14;                     // Grid row stride - Points

                                                      // PRECIOUS FILES
   const string GoDefTour   = "NoTour.gdt";           // Default name - no such file
   const string GoTours     = "Tours";                // Directory for godraw tournament files
   const string GoHelp      = "Help";						// Directory for all help files

   const string GoDrawCap   = "GoDraw - ";
   const string GoDrawHlp   = "GoDraw.hlp";           // Compiled Help
   const string GoDrawIni   = "GoDrawINI.gdi";        // Preferences

   const string GoTourList  = "GoTours.gdi";				// Tournament list
   const string GoLandCode  = "GoLandCode.gdi";       // Country codes
   const string GoClubCode  = "GoClubCode.gdi";       // Club Codes
   const string GoClubCodeR = "GoClubCodeR.gdi";      // Tab delimited 4 letter

   const string GoPlayers   = "GoPlayers.gdi";        // Player Details

                                                      // VOLATILE FILES
   const string GoDrawLog   = "GoDrawLOG.txt";        // Error Log
   const string DummyFile   = "Dummy.txt";            // Dummy File name

                                                      // LOW PROFILE STRINGS
   const string Version     = "Version 6.5.1";        // Manual version update

                                                      // EXPORTED FILES
   const string RankExt     = ".rnk";                 // Standard rank list
   const string WebExt      = ".web";                 // Includes handicaps for web
   const string GoRExt      = ".h9";                  // Translated to GOR format

   String TourFilePath(String TourFile)
   {
   	String ToursPath = FilePath(GoTours);

      return ToursPath + String(Tools::sSep) + TourFile;
   }

   String HelpFilePath(String HelpFile)
   {
   	return FilePath(GoHelp) + String(Tools::sSep) + HelpFile;
   }

   
}
//------------------------------------------------------------------------------

namespace GoDraw                                   // STATUS BAR
{
   void ShowTourState(TStatusBar* Status)
   {
      int IDState = IDI_SVirgin + TourMaster::GoTour.State;

      Status->Panels->Items[1]->Text = LoadStr(IDState);
   }

   void ShowTourInfo(TStatusBar* Status, int MessID)
   {
      Status->Panels->Items[0]->Text = LoadStr(MessID);
      Status->Update();
   }
}
//------------------------------------------------------------------------------

namespace GoDraw                                         // STARTUP
{
   void StartPrinter(void)
   {
      try
      {
         PrinterAPI::CreatePrinter(PrefsMaster::PrefsTube.PrFont);

         PrefsMaster::PrefsTube.PrFont = PrinterAPI::GetPrinterFontName();
      }
      catch(Exception& Eprint)
      {
          ToolsGUI::Serve(Eprint, IDE_GoDraw, IDE_StartPrinter);
      }
   }

   void StartToursDirectory(TOpenDialog* OpenDlg)
   {
      using Tools::sSep;
      using Tools::s_S;

      try
      {
         OpenDlg->InitialDir = AppPath + String(Tools::sSep)+ s_S(GoTours);

         if(!DirectoryExists(OpenDlg->InitialDir))
            if(!CreateDir(OpenDlg->InitialDir))
               Throw(ToolsGUI::erOperr, IDE_GoDraw, IDE_StartToursDirectory, IDE_NoCreateDir);
      }
      catch(ETour& EDir)
      {
         ToolsGUI::Serve(EDir, IDE_GoDraw, IDE_StartToursDirectory );
      }
   }

   void StartBars(TComboBox* MBox, TComboBox* KBox, TComboBox* NBox)
   {
   	using Tools::trim;
      using Tools::s_S;

   	using Tours::pro_grade;
      using Tours::mc_scale;
      using Tours::grade_string;

      using TourMaster::GoTour;
      
      MBox->Items->Clear();
      KBox->Items->Clear();
      NBox->Items->Clear();

      int gmax = pro_grade + 8*mc_scale, gmin = -99*mc_scale;

      for(int g = gmax; g>=gmin; g -= mc_scale)
      {
         String Grade = s_S(trim(grade_string(g)));

         MBox->Items->Add(Grade);
         KBox->Items->Add(Grade);
         NBox->Items->Add(Grade);
      }

      String MText    = s_S(trim(grade_string(GoTour.Rules.MacBar)));
      MBox->ItemIndex = MBox->Items->IndexOf(MText);

      String KText    = s_S(trim(grade_string(GoTour.Rules.KomiGroup)));
      KBox->ItemIndex = KBox->Items->IndexOf(KText);

      String NText    = s_S(trim(grade_string(GoTour.Rules.MinGrade)));
      NBox->ItemIndex = NBox->Items->IndexOf(NText);
   }

}
//------------------------------------------------------------------------------

namespace GoDraw                                         // UPDATE TEXT
{
	void SetPopulations(TGroupBox* PopGrp)
   {
   	using TourMaster::GoTour;

   	for(int c=0; c<PopGrp->ControlCount; c++)
      {
      	TLabel* PopLabel = (TLabel*)(PopGrp->Controls[c]);

         switch(c)
         {
            case  1:PopLabel->Caption = String(GoTour.RegCount);      		break;
            case  3:PopLabel->Caption = String(GoTour.Players);       		break;
            case  5:PopLabel->Caption = String(GoTour.LandCount);     		break;
            case  7:PopLabel->Caption = String(GoTour.ClubCount);     		break;
            case  9:PopLabel->Caption = String(GoTour.PlayNowCount);  		break;
            case 11:PopLabel->Caption = String(GoTour.PlayNextCount); 		break;
            case 13:PopLabel->Caption = String((GoTour.MissingCount+1)/2); break;
            case 15:PopLabel->Caption = String(GoTour.PrizeCount);    		break;

            default:;
         }
      }
   }

   void SetBars(TGroupBox* MacGrp)
   {
  		using TourMaster::GoTour;

   	for(int c=0; c<MacGrp->ControlCount; c++)
      {
         if((c+1)%2==0 && c<6)
         {
            TLabel* PopLabel = (TLabel*)(MacGrp->Controls[c]);

            switch(c)
            {
               case 1: PopLabel->Caption = String(GoTour.MacPop);    break;
               case 3: PopLabel->Caption = String(GoTour.KomiPop);   break;
               case 5: PopLabel->Caption = String(GoTour.NovicePop); break;

               default:;
            }
         }

         if(c >= 6)
         {
            using Tools::s_S;

            TComboBox* BarBox = (TComboBox*)(MacGrp->Controls[c]);

            String BarText;

            switch(c)
            {
               case 6: BarText = s_S(Tours::grade_ustring(GoTour.Rules.MacBar));    break;
               case 7: BarText = s_S(Tours::grade_ustring(GoTour.Rules.KomiGroup)); break;
               case 8: BarText = s_S(Tours::grade_ustring(GoTour.Rules.MinGrade));  break;

               default: ;
            }

            BarBox->ItemIndex = BarBox->Items->IndexOf(BarText);
         }
      }
   }

   void ParseBar
   (
   	TGroupBox* BarGrp, TComboBox* BarBox, TStatusBar* StatusBar,

      RegGUI::GRegGrid* RegGrid
   )
   {
   	using std::string;

      using Tools::s_S;
      using Tools::S_s;
      using Tools::trim;
   	using Tools::ints;
      using Tools::load_str;
      using Tools::to_string;

   	using TourMaster::GoTour;
      using DrawMaster::brSet;
      using DrawMaster::SetInitialScores;

      // PICK UP THE GRADE WHICH WE KNOW TO BE VALID
   	string bgrade = trim(S_s(BarBox->Items->Strings[BarBox->ItemIndex]));

      ints bval;	RegMaster::parse_grade(bgrade, bval);

      // GET THE CHANGED BAR
      bool NewMac=false, NewNov=false;

      ints MacBar = GoTour.Rules.MacBar;
      ints KomBar = GoTour.Rules.KomiGroup;
      ints NovBar = GoTour.Rules.MinGrade;

      switch(BarBox->Tag)
      {
      	case McBarEd: MacBar = bval; 	NewMac = true;	break;
         case KoBarEd: KomBar = bval; 						break;
         case NuBarEd: NovBar = bval;  NewNov = true; break;

         default:
         {
         	ToolsGUI::OperatorError(IDF_UnknownBar);

            return;
         }
      }

      // ENSURE BARS ORDERED
      if(NovBar>KomBar || KomBar>MacBar)
      {
      	ToolsGUI::OperatorError(IDF_BarOrder);

         return;
      }

      // NEW BARS AND GROUP CODES
 		GoTour.Rules.MacBar    = MacBar;
      GoTour.Rules.KomiGroup = KomBar;
      GoTour.Rules.MinGrade  = NovBar;


      // UPDATE INITIAL SCORES IF EITHER TOP OR BOTTOM BAR CHANGED
      ints BarDepth = 0;

      if(NewNov || NewMac)
      	SetInitialScores(NovBar, KomBar,MacBar, BarDepth, &GoTour);

      // GROUP CODES FOR MCMAHON GROUP MAY CHANGE
      if(NewMac)
      	RegMaster::SetMacGroupCodes();

 		GoTour.SaveTournament();

      // UPDATE BAR DISPLAY
      GoTour.PopulationCount();	GoDraw::SetBars(BarGrp);

      if(BarBox->Tag != McBarEd)
      {
      	GoTour.PostTourInfo(IDF_BarEdit);	return;
      }

      // GK BARDEPTH 19/11/2015
      // MCMAHON REPORT ON EXTREME SETTINGS
      const ints LimBarDepth=2*Tours::mc_scale, MaxBarDepth=3*Tours::mc_scale;

      int LoPop, HiPop;

      if(DrawMaster::GetBarPops(&GoTour, LoPop, HiPop))
      {
      	const bool bdeep = BarDepth>=MaxBarDepth;
         const bool hipop = GoTour.MacPop>HiPop;
         const bool lopop = GoTour.MacPop<LoPop;

      	int Report = 0;

         // DURING REGISTRATION
         while(GoTour.State<Tours::tsRegClosed)
         {
            //if(bdeep) { Report = IDF_BarDepth; break; }  GK 2018 4.6.3
            Report = IDF_BarPopCount;

            if(hipop) { Report = IDF_HiBar; break; }
            if(lopop) { Report = IDF_LoBar; break; }
            break;
         }
         //if(GoTour.State<Tours::tsRegClosed) 				GK 2018 4.6.3
         	//ToolsGUI::OperatorWarning(IDF_BarPopCount);

      	// AT CLOSE OF REG
         while(GoTour.State==Tours::tsRegClosed)
         {
            if(bdeep)									{ Report = IDF_BarDepth; break; }
            if(BarDepth>=LimBarDepth && hipop)	{ Report = IDF_HiPopHint; break; }
            if(BarDepth==0 && lopop)	 			{ Report = IDF_LoPopHint; break; }
            break;
         }

         if(GoTour.State==Tours::tsRegClosed)
         {
            string BarReport;

            switch(Report)
            {
               case IDF_BarDepth:
                  BarReport = load_str(IDF_BarDepth);
               break;

               case IDF_HiPopHint:
               {
                  BarReport = load_str(IDF_HiPopHint);
                     size_t lHiPop = BarReport.find('#', 0);
                     string sHiPop = to_string(HiPop);
                  BarReport.replace(lHiPop, 1, sHiPop);
               }
               break;

               case IDF_LoPopHint:
               {
                  BarReport = load_str(IDF_LoPopHint);
                     size_t lLoPop = BarReport.find('#', 0);
                     string sLoPop = to_string(LoPop);
                  BarReport.replace(lLoPop, 1, sLoPop);
               }
               break;

               default: GoTour.PostTourInfo(IDF_BarEdit); break;
            }

            if(Report)
               ToolsGUI::ShowMessage(ToolsGUI::erInfo, BarReport);
         }
         else
         	GoDraw::ShowTourInfo(StatusBar, Report);

         RegGrid->LoadFromTour();
      }
   }
}
//------------------------------------------------------------------------------

namespace GoDraw                                      // PRINT CONTROL
{                                                     // FILES
   const string RegPri      = "RegPRI.txt";           // Register print
   const string DrawPri     = "DrawPRI.txt";          // Draw print
   const string ResPri      = "ResPRI.txt";           // Results print
   const string RankPri     = "RankPRI.txt";          // Rank print
   const string PrizePri    = "PrizePRI.txt";         // Prizes print
   const string CardsPri    = "CardsPRI.txt";         // Cards print
   const string RulesPri    = "RulesPri.txt";			// Rules print
}
//------------------------------------------------------------------------------








