////////////////////////////////////////////////////////////////////////////////
// MODULE GoDrawGUI:       Application-Wide support            G.Kaniuk 05/04/02
//
#include <string>

#include <forms.hpp>
#include <grids.hpp>
#include <classes.hpp>
#include <graphics.hpp>

#include "ETour.h"

#include "RegGUI.h"
#include "PairPaneGUI.h"

#include "TourMaster.h"

#ifndef GoDrawGUIH
#define GoDrawGUIH
//---------------------------------------------------------------------------

namespace GoDraw
{
   struct GPoint
   {
      int i,j;                                   // Graphic notation - i across, j is down

      GPoint(){ i=0; j=0; }

      GPoint(const int pI, const int pJ){ i=pI; j=pJ; }

      GPoint operator+ (const GPoint Q){ return GPoint(i+Q.i, j+Q.j); }
   };

}
//------------------------------------------------------------------------------

namespace GoDraw                                 // GLOBAL DATA
{
   using std::string;

   extern const int defCoverage;                 // Percent screen dimension
   extern const int defFormHeight, defFormWidth; // Design size on 800 x 600
   extern const int defFontSize;                 // Points
   extern const TColor defPaneColor;             // Sunken parts of the pane

                                                // FILE NAMES
   extern const string GoExt;                   // Tournament file - binary
   extern const string GoBakExt;                // Tournament file - binary
   extern const string GoDefTour;               // Default Empty tournament file
   extern const string GoTours;                 // Directory for files .gdt
   extern const string GoTourList;					// Tournament list
   extern const string GoHelp;						// Help files directory

   extern const string GoDrawCap;               // Caption 
   extern const string GoDrawLog;               // Error Log
   extern const string GoDrawIni;               // Preferences
   extern const string GoDrawHlp;               // Compiled Help

   extern const string GoLandCode;              // Country codes
   extern const string GoClubCode;              // Club Codes
   extern const string GoClubCodeR;					// Tab delimited 4 letter

   extern const string GoPlayers;               // Player Details

                                                // LOW PROFILE STRINGS
   extern const string Version;                 // Manual version update

                                                // EXPORTED FILES
   extern const string RankExt;                 // Standard rank list
   extern const string WebExt;                  // Includes handicaps for web
   extern const string GoRExt;                  // Translated to GOR format

                                                // VARIABLES
   extern String AppPath;                       // Program Directory
   extern String Backup; 								// Regular backup location
   extern String Removable;					   	// Can change in preferences
}
//------------------------------------------------------------------------------

namespace GoDraw                                 // GLOBAL ROUTINES
{
   using ToolsGUI::EType;
   using ToolsGUI::ETour;

   // void Throw(EType etype, short idmodule, short idproc, short idmsg);

   void ShowTourState(TStatusBar* Status);
   void ShowTourInfo(TStatusBar* Status, int MessID);

   String FilePath(String fileroot);
   String FilePath(string FileRoot);
   
   String TourFilePath(String TourFile);
   String HelpFilePath(String HelpFile);

   void AutoBackup(void);
}

namespace GoDraw                                 // STARTUP
{
   void StartPrinter(void);
   void StartToursDirectory(TOpenDialog* OpenDlg);
   void StartBars(TComboBox* MBox, TComboBox* KBox, TComboBox* NBox);
}

namespace GoDraw                                 // UPDATE TEXT
{
	enum
   {
   	NoBarEd, McBarEd,KoBarEd,NuBarEd
   };
   
	void SetPopulations(TGroupBox* PopGrp);

   void SetBars(TGroupBox* BarGrp);

   void ParseBar
   (
   	TGroupBox* BarGrp, TComboBox* BarBox, TStatusBar* StatusBar,

      RegGUI::GRegGrid* RegGrid
   );
}
//------------------------------------------------------------------------------

namespace GoDraw                                      // PRINT CONTROL
{                                                     // FILES
   extern const string RegPri;          					// Register print
   extern const string DrawPri;         					// Draw print
   extern const string ResPri;          					// Results print
   extern const string RankPri;         					// Rank print
   extern const string PrizePri;        					// Prizes print
   extern const string CardsPri;        					// Cards print
   extern const string RulesPri; 		   				// Rules print
   extern const string DummyFile;       					// Dummy File name

	void DeletePrintFiles(void);   
}
//------------------------------------------------------------------------------

#endif

