////////////////////////////////////////////////////////////////////////////////
// MODULE GoDrawGeom:       Calculate Godraw geometry        G.Kaniuk 27/04/04
//
#include <vcl.h>
#include <math.h>

#pragma hdrstop

#include "GraphicsAPI.h"
#include "GoDrawForm.h"

#pragma package(smart_init)
//------------------------------------------------------------------------------

// GLOBAL namespace                    // SCALING

const int minScreenW = 800;         // Minimum Screen working area
const int minScreenH = 600;

const int minFontSiz =  8;          // Defines row stride
const int minTaskBar = 32;          // Hor or vertical
const int minGrout   =  8;          // Defines offsets
const int minFiller  =  4;          // Fine detail offsets
const int minPack    =  2;          // Packing

const int minPairW   = 279;         // Main Group Box Widths
const int minRegW    = 489;
const int minMacW    = 180;

const int minPanelH  =  81;         // Main Component Heights
const int minGridH   = 258;
const int minMacH    = 113;         // Allow 6 rows
const int minStatusH =  22;         // StatusBar
const int minLabel   =  13;         // Label Heights
const int minGradeW  =  49;         // Bar grade width
const int minPopStep =  98;         // PopGroup stride

void __fastcall TGoDrawFrm::StartUpForm(void)
{
   int ScreenT, ScreenL, ScreenB, ScreenR, ScreenW, ScreenH;

   ToolsGUI::GetScreenWorkingArea(ScreenT,ScreenL,ScreenB,ScreenR);

   ScreenW = ScreenR - ScreenL;   ScreenH = ScreenB - ScreenT;

   if(ScreenW <= minScreenW || ScreenH <= minScreenH)
      WindowState = wsMaximized;
   else
      WindowState = wsNormal;
}

void __fastcall TGoDrawFrm::MinGroupWidths(void)
{
   // REGISTER PANE WIDTHS
   RegGroup->Left   = minGrout;
   RegGroup->Width  = minRegW;

   RegPanel->Left   = minGrout;
   RegPanel->Width  = minRegW - 2*minGrout;

   RegGrid->Left    = RegPanel->Left;
   RegGrid->Width   = RegPanel->Width;

   BarGroup->Left   = RegGrid->Left;
   BarGroup->Width  = minMacW;

   PopGroup->Left   = BarGroup->Left + BarGroup->Width + minGrout;
   PopGroup->Width  = RegGrid->Width - minGrout - BarGroup->Width;


   // PAIR PANE WIDTHS
   PairGroup->Left  = minClientWidth - minGrout - minPairW;
   PairGroup->Width = minPairW;

   PairPanel->Left  = minGrout;
   PairPanel->Width = minPairW - 2*minGrout;

   PairGrid->Left   = PairPanel->Left;
   PairGrid->Width  = PairPanel->Width;

   ResGrid->Left    = PairGrid->Left;
   ResGrid->Width   = PairGrid->Width;

   // STATUS BAR
   StatusBar->Left  = RegGroup->Left;
   StatusBar->Width = minGrout + RegGroup->Width + PairGroup->Width;
   StatusBar->Panels->Items[0]->Width = PairGroup->Left - RegGroup->Left;
}

void __fastcall TGoDrawFrm::ScaleGroupWidths(void)
{
   // WE SCALE ALL THE COMPONENT WIDTHS, BUT NOT THE GROUT
   // THE PAIR GROUP IS RIGHT ALIGNED
   // ROUNDING ERROR IS ACCUMULATED IN THE CENTRE

   int minFixedW = 3*minGrout;

   int minVariableW = minRegW + minPairW;
   int newVariableW = ClientWidth - minFixedW;

   ToolsGUI::GRatio WdScale(newVariableW, minVariableW);

   int newRegW = WdScale(minRegW), newPairW = WdScale(minPairW);
   int newMacW = WdScale(minMacW);

   // REGISTER PANE WIDTHS
   RegGroup->Left   = minGrout;
   RegGroup->Width  = newRegW;

   RegPanel->Left   = minGrout;
   RegPanel->Width  = newRegW - 2*minGrout;

   RegGrid->Left    = RegPanel->Left;
   RegGrid->Width   = RegPanel->Width;

   BarGroup->Left   = RegGrid->Left;
   BarGroup->Width  = newMacW;

   PopGroup->Left   = BarGroup->Left + BarGroup->Width + minGrout;
   PopGroup->Width  = RegGrid->Width - minGrout - BarGroup->Width;


   // PAIR PANE WIDTHS
   PairGroup->Left  = ClientWidth - minGrout - newPairW;
   PairGroup->Width = newPairW;

   PairPanel->Left  = minGrout;
   PairPanel->Width = newPairW - 2*minGrout;

   PairGrid->Left   = PairPanel->Left;
   PairGrid->Width  = PairPanel->Width;

   ResGrid->Left    = PairGrid->Left;
   ResGrid->Width   = PairGrid->Width;

   // STATUS BAR
   StatusBar->Left  = RegGroup->Left;
   StatusBar->Width = minGrout + RegGroup->Width + PairGroup->Width;
   StatusBar->Panels->Items[0]->Width = PairGroup->Left - RegGroup->Left;
}

void __fastcall TGoDrawFrm::MinGroupHeights(void)
{
   // PAIR PANE HEIGHTS
   PairGroup->Height  = 5*minGrout + minPanelH + minGridH + minMacH;

   PairPanel->Top    = 2*minGrout;
   PairPanel->Height = minPanelH;

   PairGrid->Top     = PairPanel->Top + PairPanel->Height + minGrout;
   PairGrid->Height  = minGridH;

   ResGrid->Top      = PairGrid->Top + PairGrid->Height + minGrout;
   ResGrid->Height   = minMacH;

   // REGISTER PANE HEIGHTS
   RegGroup->Height  = PairGroup->Height;

   RegPanel->Top     = PairPanel->Top;
   RegPanel->Height  = PairPanel->Height;

   RegGrid->Top      = PairGrid->Top;
   RegGrid->Height   = PairGrid->Height;

   BarGroup->Top     = ResGrid->Top;
   BarGroup->Height  = ResGrid->Height;

   PopGroup->Top     = BarGroup->Top;
   PopGroup->Height  = BarGroup->Height;

   // STATUS BAR
   StatusBar->Left   = RegGroup->Left;
   StatusBar->Top    = RegGroup->Top + RegGroup->Height + minFiller;
   StatusBar->Height = minStatusH;
}


void __fastcall TGoDrawFrm::ScaleGroupHeights(void)
{
   // WE DO NOT SCALE THE CONTROL PANEL, BAR GROUP OR GROUT
   int minFixedH = 6*minGrout + minPanelH + minMacH + minStatusH; // Total fixed height within client area
   int newGridH = ClientHeight - minFixedH;                       // Only the Grid height is adjusted

   // PAIR PANE HEIGHTS
   PairGroup->Height = 5*minGrout + minPanelH + newGridH + minMacH;

   PairPanel->Top    = 2*minGrout;
   PairPanel->Height = minPanelH;

   PairGrid->Top     = PairPanel->Top + PairPanel->Height + minGrout;
   PairGrid->Height  = newGridH;

   ResGrid->Top      = PairGrid->Top + PairGrid->Height + minGrout;
   ResGrid->Height   = minMacH;

   // REGISTER PANE HEIGHTS
   RegGroup->Height  = PairGroup->Height;

   RegPanel->Top     = PairPanel->Top;
   RegPanel->Height  = PairPanel->Height;

   RegGrid->Top      = PairGrid->Top;
   RegGrid->Height   = PairGrid->Height;

   BarGroup->Top     = ResGrid->Top;
   BarGroup->Height  = ResGrid->Height;

   PopGroup->Top     = BarGroup->Top;
   PopGroup->Height  = BarGroup->Height;

   // STATUS BAR
   StatusBar->Top    = RegGroup->Top + RegGroup->Height + minFiller;
}

void __fastcall TGoDrawFrm::ReBuildRegPanel(void)
{
   const int BtnW = (RegPanel->Width - 7*minGrout)/6;          // Main buttons width
   const int BtnH = RegPanel->Height - 2*minGrout;             // Main buttons height
   const int BtnS = (BtnH - minGrout)/2;                       // Small buttons height

   const int RndH = (BtnH - 3*minFiller)/2;                    // Components for Round button
   const int RndW = (2*BtnW)/3;

   const int FndW = (3*BtnW)/4;											// Find button width
   const int FrgW = (5*BtnW)/4;											// Fast reg button width

   FindBtn->Left = minGrout;                             FindBtn->Width = FndW;
   FindBtn->Top  = minGrout;            						FindBtn->Height = BtnH;

   RegAllBtn->Left = FindBtn->Left + FndW + minGrout;    RegAllBtn->Width = BtnW;
   RegAllBtn->Top  = minGrout;                           RegAllBtn->Height = BtnS;

   UnRegAllBtn->Left = RegAllBtn->Left;						UnRegAllBtn->Width = BtnW;
   UnRegAllBtn->Top  = RegAllBtn->Top + BtnH - BtnS;     UnRegAllBtn->Height = BtnS;

   FastRegBtn->Left = RegAllBtn->Left + BtnW + minGrout; FastRegBtn->Width = FrgW;
   FastRegBtn->Top  = minGrout;      					      FastRegBtn->Height = BtnS;

   CloseBtn->Left = RegAllBtn->Left + BtnW + minGrout;   CloseBtn->Width = FrgW;
   CloseBtn->Top  = UnRegAllBtn->Top;                    CloseBtn->Height = BtnS;

   RoundPan->Left = RegPanel->Width - minGrout - BtnW;   RoundPan->Width  = BtnW;
   RoundPan->Top  = minGrout;                            RoundPan->Height = BtnH;
   RoundLbl->Left = (BtnW-RndW)/2;                       RoundLbl->Width  = RndW;
   RoundLbl->Top  = minGrout;                            RoundLbl->Height = RndH;
   RoundBox->Left = (BtnW-RndW)/2;                       RoundBox->Width  = RndW;
   RoundBox->Top  = BtnH - minGrout - BtnS;              RoundBox->Height = RndH;

   ReOpenBtn->Left = RoundPan->Left - minGrout - BtnW;   ReOpenBtn->Width = BtnW;
   ReOpenBtn->Top = minGrout;                            ReOpenBtn->Height = BtnS;
   RestoreBtn->Left = ReOpenBtn->Left;                   RestoreBtn->Width = BtnW;
   RestoreBtn->Top = ReOpenBtn->Top + BtnH - BtnS;       RestoreBtn->Height = BtnS;

   PrRegBtn->Left = ReOpenBtn->Left - minGrout - BtnW;   PrRegBtn->Width = BtnW;
   PrRegBtn->Top = minGrout;                             PrRegBtn->Height = BtnH;
}

void __fastcall TGoDrawFrm::ReBuildPairPanel(void)
{
   int BtnW = (PairPanel->Width - 4*minGrout)/3;         // Main buttons width
   int BtnH = PairPanel->Height - 2*minGrout;            // Main buttons height
   int BtnS = (BtnH - 2*minFiller)/3;                    // Small buttons height

   PairBtn->Left        = minGrout;
   PairBtn->Width       = BtnW;
   PairBtn->Top         = minGrout;
   PairBtn->Height      = BtnH;

   PrDrawBtn->Left      = PairPanel->Width - minGrout - BtnW;
   PrDrawBtn->Width     = BtnW;
   PrDrawBtn->Top       = minGrout;
   PrDrawBtn->Height    = BtnS;

   PrPrizesBtn->Left    = PrDrawBtn->Left;
   PrPrizesBtn->Width   = BtnW;
   PrPrizesBtn->Top     = minGrout + BtnH - BtnS;
   PrPrizesBtn->Height  = BtnS;

   PrRankBtn->Left   = PrDrawBtn->Left;
   PrRankBtn->Width  = BtnW;
   PrRankBtn->Top    = minGrout + BtnS + minFiller;
   PrRankBtn->Height = BtnH - 2*BtnS - 2*minFiller;

   ManualBtn->Left   = PairBtn->Left + BtnW + minGrout;
   ManualBtn->Width  = PairPanel->Width - 2*BtnW - 4*minGrout;
   ManualBtn->Top    = minGrout;
   ManualBtn->Height = BtnS;

   EditBtn->Left     = ManualBtn->Left;
   EditBtn->Width    = (ManualBtn->Width - minFiller)/2;
   EditBtn->Top      = PrRankBtn->Top;
   EditBtn->Height   = PrRankBtn->Height;

   SaveBtn->Left     = EditBtn->Left + ManualBtn->Width - EditBtn->Width;
   SaveBtn->Width    = EditBtn->Width;
   SaveBtn->Top      = EditBtn->Top;
   SaveBtn->Height   = EditBtn->Height;

   CancelBtn->Left   = ManualBtn->Left;
   CancelBtn->Width  = ManualBtn->Width;
   CancelBtn->Top    = PrPrizesBtn->Top;
   CancelBtn->Height = PrPrizesBtn->Height;
}

void __fastcall TGoDrawFrm::ReBuildBars(void)
{
   int lblGrout = 4;

   int barGrout = (BarGroup->Height - 3*MacBox->Height)/4;
   int fixedLabel = MacPopLbl->Left + MacPopLbl->Width;

   int BoxLeft = (fixedLabel + BarGroup->Width - minGradeW)/2;

   NoviceBox->Left   = BoxLeft;
   NoviceBox->Top    = BarGroup->Height - barGrout - NoviceBox->Height;
   NoviceLbl->Top    = NoviceBox->Top + lblGrout;
   NovicePopLbl->Top = NoviceLbl->Top;

   KomiBox->Left     = BoxLeft;
   KomiBox->Top      = NoviceBox->Top - barGrout - KomiBox->Height;
   KomiLbl->Top      = KomiBox->Top + lblGrout;
   KomiPopLbl->Top   = KomiLbl->Top;

   MacBox->Left      = BoxLeft;
   MacBox->Top       = KomiBox->Top - barGrout - MacBox->Height;
   MacLbl->Top       = MacBox->Top + lblGrout;
   MacPopLbl->Top    = MacLbl->Top;
}

void __fastcall TGoDrawFrm::ReBuildPops(void)
{
   int lSize = RegPopLbl->Left +  RegPopLbl->Width - RegLbl->Left;
   int rSize = PlayersPopLbl->Left +  PlayersPopLbl->Width - PlayersLbl->Left;

   int popGrout = (PopGroup->Width - lSize - rSize)/3;

   int topReg = 16, topArea = 37, topPlay = 67, topRes = 88;

   RegLbl->Left         = popGrout;
   RegLbl->Top          = topReg;

   RegPopLbl->Left      = RegLbl->Left + minPopStep;
   RegPopLbl->Top       = topReg;

   PlayersLbl->Left     = 2*popGrout + lSize;
   PlayersLbl->Top      = topReg;

   PlayersPopLbl->Left  = PlayersLbl->Left + minPopStep;
   PlayersPopLbl->Top   = topReg;


   LandLbl->Left     = RegLbl->Left;
   LandLbl->Top      = topArea;

   LandPopLbl->Left  = RegPopLbl->Left;
   LandPopLbl->Top   = topArea;

   ClubLbl->Left     = PlayersLbl->Left;
   ClubLbl->Top      = topArea;

   ClubPopLbl->Left  = PlayersPopLbl->Left;
   ClubPopLbl->Top   = topArea;


   NowLbl->Left      = RegLbl->Left;
   NowLbl->Top       = topPlay;

   NowPopLbl->Left   = RegPopLbl->Left;
   NowPopLbl->Top    = topPlay;

   NextLbl->Left     = PlayersLbl->Left;
   NextLbl->Top      = topPlay;

   NextPopLbl->Left  = PlayersPopLbl->Left;
   NextPopLbl->Top   = topPlay;


   ResLbl->Left      = RegLbl->Left;
   ResLbl->Top       = topRes;

   ResPopLbl->Left   = RegPopLbl->Left;
   ResPopLbl->Top    = topRes;

   PrizeLbl->Left    = PlayersLbl->Left;
   PrizeLbl->Top     = topRes;

   PrizePopLbl->Left = PlayersPopLbl->Left;
   PrizePopLbl->Top  = topRes;
}

void __fastcall TGoDrawFrm::ReBuildForm(void)
{
   if(!UsingForm)return;                 	// System closedown

   UsingForm = false;							// Prevent spurious calls due to FormResize
   													// triggered when we adjust the StatusBar

   // WE NEED TO PREVENT VCL SCREWING THE SCROLL
   RegGrid->LeftCol  = 1;  RegGrid->TopRow   = 1;
   PairGrid->LeftCol = 1;  PairGrid->TopRow  = 1;
   ResGrid->LeftCol  = 1;  ResGrid->TopRow   = 1;

   if(!LockWindowUpdate(GetDesktopWindow()))return;

   if(HorzScrollBar) HorzScrollBar->Position = 0;      // Prevent fat margin
   if(VertScrollBar) VertScrollBar->Position = 0;


   minClientWidth  = minGrout + minRegW + minGrout + minPairW + minGrout;

   minClientHeight = 2*minGrout + minPanelH +
                       minGrout + minGridH +
                       minGrout + minMacH +
                     2*minGrout + minStatusH;

   if(ClientHeight<=minClientHeight)
      MinGroupHeights();
   else
      ScaleGroupHeights();

   if(ClientWidth<=minClientWidth)
      MinGroupWidths();
   else
      ScaleGroupWidths();

   ReBuildRegPanel();
   ReBuildPairPanel();
   ReBuildBars();
   ReBuildPops();

   LockWindowUpdate(NULL);

   RegGrid->ReBuild();        PairPane->ReBuild();

   RegGrid->Show();

   PairGrid->Show();

   if(PairPane->EditState > PaneGUI::psEditOff)
   	UpsetList->Show();
   else
   	ResGrid->Show();

   UsingForm = true;
}

//------------------------------------------------------------------------------
