////////////////////////////////////////////////////////////////////////////////
// MODULE GoDrawStates:       Manage GoDraw States        G.Kaniuk 05/06/04
//

#include <vcl.h>
#pragma hdrstop

#include "TourMaster.h"
#include "GoDrawForm.h"

void __fastcall TagPlayerDelete(void)
{
}

void __fastcall TGoDrawFrm::TagCriticalControls(void)
{
	StateControls[TourMaster::cmdNone] 		= 0;
   StateControls[TourMaster::cmdNew] 		= (TComponent*)FileNew;							// ACTIVE - CHANGING STATE
   StateControls[TourMaster::cmdOpen] 		= (TComponent*)FileOpen;
   StateControls[TourMaster::cmdEnter] 	= (TComponent*)EdPlayerFrm->OKBtn;
   StateControls[TourMaster::cmdDel] 		= 0;

   StateControls[TourMaster::cmdClose] 	= (TComponent*)CloseBtn;
   StateControls[TourMaster::cmdReOpen] 	= (TComponent*)ReOpenBtn;
   StateControls[TourMaster::cmdRestore] 	= (TComponent*)RestoreBtn;
   StateControls[TourMaster::cmdRound] 	= (TComponent*)RoundBox;

   StateControls[TourMaster::cmdDraw] 		= (TComponent*)PairBtn;
   StateControls[TourMaster::cmdForced] 	= (TComponent*)ManualBtn;
   StateControls[TourMaster::cmdEdit] 		= (TComponent*)EditBtn;

   StateControls[TourMaster::cmdSave] 		= (TComponent*)SaveBtn;
   StateControls[TourMaster::cmdDcancel] 	= (TComponent*)CancelBtn;
   StateControls[TourMaster::cmdEcancel] 	= (TComponent*)CancelBtn;
   StateControls[TourMaster::cmdResult] 	= 0;
   StateControls[TourMaster::cmdCloseT] 	= (TComponent*)FileClose;

   StateControls[TourMaster::cmdFind]     = (TComponent*)FindBtn;							// STATE DEPENDENT AVAILABILITY
	StateControls[TourMaster::cmdRegAll]   = (TComponent*)RegAllBtn;
   StateControls[TourMaster::cmdUnRegAll] = (TComponent*)UnRegAllBtn;
	StateControls[TourMaster::cmdFastReg]  = (TComponent*)FastRegBtn;

 	StateControls[TourMaster::cmdMacBar] 	= (TComponent*)MacBox;
 	StateControls[TourMaster::cmdKomBar] 	= (TComponent*)KomiBox;
 	StateControls[TourMaster::cmdNovBar] 	= (TComponent*)NoviceBox;

   StateControls[TourMaster::cmdImport] 	= (TComponent*)FileImport;
}


void __fastcall TGoDrawFrm::EnableCriticalControls(void)
{
	using Tours::tsRegNow;
   using Tours::tsTourDone;
   using Tours::stMcMahon;
   using Tours::stBarLow;
   using Tours::style_t;
   
   using TourMaster::cmdEnter;
   using TourMaster::cmdResult;
	using TourMaster::cmdDcancel;
   using TourMaster::cmdEcancel;
   using TourMaster::EnabledCommands;
   using TourMaster::GoTour;

   const Tours::TourState State = GoTour.State;

	for(int cmd = 0; cmd<TourMaster::cmdCount; ++cmd)
   {
   	TComponent* StateCmd = StateControls[cmd];

   	if(StateCmd)
      {
       	TMenuItem* Mitem = dynamic_cast<TMenuItem*>(StateCmd);
         TControl*  Witem = dynamic_cast<TControl*>(StateCmd);

         if(Mitem)
         	Mitem->Enabled = EnabledCommands[cmd][State];

         if(Witem)
         	Witem->Enabled = EnabledCommands[cmd][State];
       }
   }

   // CANCEL DRAW/EDIT BUTTON
   CancelBtn->Enabled =
   	EnabledCommands[cmdDcancel][State] || EnabledCommands[cmdEcancel][State];

   // WE NEED TO INHIBIT SOME MENU ITEMS
   TMenuItem* MainItems = GoDrawFrm->MainMenu->Items;

   // FILE MENU LOCKS
   TMenuItem* FileItems  = MainItems->Items[0];
   TMenuItem* ImportItem = FileItems->Find("Import ...");

   if(ImportItem)
   	if(State == Tours::tsRegClosed || State == Tours::tsDrawDone)
      	ImportItem->Enabled = false;
      else
      	ImportItem->Enabled = EnabledCommands[cmdEnter][State];

   // TOOLS MENU LOCKS
   TMenuItem* ToolItems = MainItems->Items[3];

   TMenuItem* SimItem   = ToolItems->Find("Simulate Results");
   TMenuItem* ClrItem   = ToolItems->Find("Clear Results");
   TMenuItem* GrpItem   = ToolItems->Find("Groups");

   const bool ToolEnabled = EnabledCommands[cmdResult][State];
   const style_t Style    = GoTour.Rules.Style;
   const bool GrpsEnabled = Style==stMcMahon || Style==stBarLow;

   if(SimItem) SimItem->Enabled = ToolEnabled;
   if(ClrItem) ClrItem->Enabled = ToolEnabled;

   if(GrpItem)
   	if(GrpsEnabled)
   		GrpItem->Enabled = tsRegNow<=State && State<=tsTourDone;
      else
         GrpItem->Enabled = false;

   // EDIT PROTECTION
   // The Tag's for each grid are checked by Click()
   // to protect the grid from unwanted edits.

   // PairGrid editing is allowed
   // only if the state is edit or forced pair
   //
   PairGrid->Tag = State == Tours::tsEditNow
                                ||
                   State == Tours::tsForcedPair;

	if(PairGrid->Tag)
   {
   	EdPlayerFrm->ReadOnly = true;			// View only
 
   	RegGrid->Tag = false;					// Delete not possible
      ResGrid->Tag = false;
   }
   else												// No clicking during Pair edit
   {
   	const bool EditAllowed = EnabledCommands[cmdEnter][State];

      EdPlayerFrm->ReadOnly = !EditAllowed;

   	RegGrid->Tag 			 =  EditAllowed;

      ResGrid->Tag 			 =  EditAllowed;
   }

   RegGrid->ShowHint = RegGrid->Status == RegGUI::rsFastReg;
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
