//---------------------------------------------------------------------------

#include <vcl.h>

#pragma hdrstop
#include "GroupRES.h"
#include "GroupsIDH.h"

#include "GoDrawGUI.h"
#include "GroupForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------

const int GroupFields = 4;
const int GroupColPos[4] = { 0, 20, 110, 138 };

__fastcall JListBox::JListBox
(
	TGroupFrm* groupform,	TComponent* owner,

   const int left, const int top, const int width, const int height
)
:RegPanelGUI::MListBox(owner, GroupFields, GroupColPos)
{
   Clear();

   GroupForm = groupform;

   MultiSelect = true;		ExtendedSelect = true;
   		Width = width;				  Height = height;
    		 Left = left;					  Top = top;

   clBackground    = Color;
   clFont          = Canvas->Font->Color;
   clHighlightFont = clFont==clHighlight ? TColor(clHighlight/2 + 2) : clFont;
}

void __fastcall JListBox::DblClick(void)
{
	using TourGroupsMaster::MGroupsTube::idNone;
   using TourGroupsMaster::MGroupsTube::gaNone;

	if(this == GroupForm->PlayerBox)
   {
   	const int Index = GroupForm->PlayerBox->ItemIndex;

      const String Player =  GroupForm->PlayerBox->Items->Strings[Index];

      int pId, pCode;

      if(GroupForm->GroupsTube.ParseGroup(Player, pId, pCode))
      {
         GroupForm->IdCombo->ItemIndex = pId;
         GroupForm->CodeCombo->ItemIndex = pCode;

         GroupForm->ShowMembers(Player);

         if(pId < idNone && pCode < gaNone)
         {
            GroupForm->AddBtn->Enabled = true;
            GroupForm->RemoveBtn->Enabled = true;

         	GroupForm->ShowInfo(IDK_Valid);
         }
         else
         {
         	GroupForm->AddBtn->Enabled = false;
            GroupForm->RemoveBtn->Enabled = false;

         	GroupForm->BeepInfo(IDK_NoPlayerGroup);
         }
      }
      else
         GroupForm->BeepInfo(IDK_BadGroup);
   }
}

void __fastcall JListBox::Click(void)
{
   if(this == GroupForm->PlayerBox)
   {
   	// CLICKING THE MEMBERS DESELECTS THE PLAYERS
   	if(GroupForm->MemberBox->SelCount>0)
      {
      	GroupForm->MemberBox->Items->BeginUpdate();

         for(int p=0; p<GroupForm->MemberBox->Items->Count; p++)
         	if(GroupForm->MemberBox->Selected[p])
               GroupForm->MemberBox->Selected[p] = false;

         GroupForm->MemberBox->Items->EndUpdate();
      }
   }

   // CLICKING THE PLAYERS DESELECTS THE MEMBERS
   if(this == GroupForm->MemberBox)
   {
   	if(GroupForm->PlayerBox->SelCount>0)
      {
      	GroupForm->PlayerBox->Items->BeginUpdate();

         for(int p=0; p<GroupForm->PlayerBox->Items->Count; p++)
         	if(GroupForm->PlayerBox->Selected[p])
            	GroupForm->PlayerBox->Selected[p] = false;

         GroupForm->PlayerBox->Items->EndUpdate();
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::ClearInfo(void)
{
   GroupStatusBar->Panels->Items[0]->Text = String();
}

void __fastcall TGroupFrm::ShowInfo(const int messid)
{
   GroupStatusBar->Panels->Items[0]->Text = LoadStr(messid);
}

void __fastcall TGroupFrm::BeepInfo(const int messid)
{
	// Beep();

   GroupStatusBar->Panels->Items[0]->Text = LoadStr(messid);
}

void __fastcall TGroupFrm::ShowMembers(const String& Group)
{
   MemberBox->Clear();	MemberBox->ItemIndex = -1;

   if(Group[1]==Tools::cSpace && Group[2]==Tools::cSpace)
   	return;

   MemberBox->Items->BeginUpdate();

   for(int p=0; p<PlayerBox->Items->Count; p++)
   {
   	const String& Player = PlayerBox->Items->Strings[p];

      if(Player[1]==Group[1] && Player[2]==Group[2])
      	MemberBox->Items->Add(Player);
   }
   MemberBox->Items->EndUpdate();
}

void __fastcall TGroupFrm::ClearMembers(void)
{
	MemberBox->Clear();

   MemberBox->ItemIndex = -1;
}
//---------------------------------------------------------------------------

__fastcall TGroupFrm::TGroupFrm
(
	TComponent* Owner,
   TourGroupsMaster::MGroupsTube& groupstube
): TForm(Owner), GroupsTube(groupstube)
{
	using TourGroupsMaster::MGroupsTube::idNone;
   using TourGroupsMaster::MGroupsTube::gaNone;

	using RegPanelGUI::MListBox;

	// FORM TEXT
	Caption = LoadStr(IDK_Groups) + GroupsTube.TourNameYear;

   // PLAYERS,MEMBERS LIST BOXES
   try
   {
   	PlayerBox = new JListBox(this, GroupPlayersBox, 16, 19, 265, 404);
      MemberBox = new JListBox(this, GroupOpsBox, 		16, 56, 265, 339);

      PlayerBox->SetSorted(false);	PlayerBox->TabStop = false;
      MemberBox->SetSorted(false);  MemberBox->TabStop = false;
   }
   catch(...)
   {
      OkBtn->Enabled = false;
      AddBtn->Enabled = false;	RemoveBtn->Enabled = false;
      IdCombo->Enabled = false;	CodeCombo->Enabled = false;

      BeepInfo(IDK_NoBoxes);

      delete PlayerBox;
      delete MemberBox;

   	return;
   }

   // HELP
   HelpFile = GoDraw::HelpFilePath("Groups.hlp");

   PlayerBox->HelpContext 	 = IDH_TMPlayers;
   MemberBox->HelpContext 	 = IDH_TMMembers;
   GroupOpsBox->HelpContext = IDH_TMGroupOps;
   IdCombo->HelpContext 	 = IDH_TMGrID;
   CodeCombo->HelpContext 	 = IDH_TMGrATTR;
   AddBtn->HelpContext 		 = IDH_TMAdd;
   RemoveBtn->HelpContext 	 = IDH_TMRemove;
   OkBtn->HelpContext 		 = IDH_TMOK;

   // GROUP ID COMBO
   IdCombo->Clear();

   for(int i=0; i<GroupsTube.GroupIDs.Length()-1; i++)
   {
   	String GroupId(" "); 	GroupId[1] = GroupsTube.GroupIDs[i+1];

   	IdCombo->Items->Add(GroupId);
   }
   IdCombo->Items->Add(LoadStr(IDK_NOID));

   // GROUP ATTRIBUTE COMBO
   CodeCombo->Clear();
   
   for(int i=0; i<GroupsTube.GroupCodes.Length(); i++)
   	CodeCombo->Items->Add(LoadStr(IDK_GATTRIB + i));

   // IDENTIFY STARTUP GROUP
   int GroupIndex = GroupsTube.GroupIndex, GroupCodeIndex = GroupsTube.GroupCodeIndex;

   String Group;	GroupsTube.IdForgeGroup(GroupIndex, GroupCodeIndex, Group);

   if(GroupsTube.SelectedPlayer>0)
   {
      IdCombo->ItemIndex = GroupIndex;
      CodeCombo->ItemIndex = GroupCodeIndex;

      StartIndex   = GroupsTube.SelectedPlayer-1;
   }
   else
   {
      IdCombo->ItemIndex   = idNone;
      CodeCombo->ItemIndex = gaNone;

      StartIndex  = -1;
   }

   // LOAD PLAYERS AND ANY GROUP MEMBERS
   const bool NonEmptyGroup = GroupsTube.SelectedPlayer > 0
   												 &&
   									IdCombo->ItemIndex != idNone;

   ShowInfo(IDK_LoadingP);

   for(int p=1; p<=GroupsTube.Players; p++)
   {
   	const String PlayerText = GroupsTube.PlayerText(p);

   	PlayerBox->Items->Add(PlayerText);

      if(NonEmptyGroup)
      	if(Group[1]==PlayerText[1])
            if(Group[2]==PlayerText[2])
            	MemberBox->Items->Add(PlayerText);
   }

   PlayerBox->Show();
   MemberBox->Show();

   if(GroupsTube.SelectedPlayer > 0)
   {
      PlayerBox->Selected[GroupsTube.SelectedPlayer-1] = true;

      PlayerBox->ItemIndex = GroupsTube.SelectedPlayer;
   }

   if(MemberBox->Items->Count>0)
   {
   	AddBtn->Enabled = true;		RemoveBtn->Enabled = true;

      MemberBox->ItemIndex = -1;
	}
   else
   	if(GroupIndex<idNone)
      {
      	AddBtn->Enabled = true;		RemoveBtn->Enabled = false;
      }
      else
      {
      	AddBtn->Enabled = false;		RemoveBtn->Enabled = false;
      }

   OkBtn->Enabled = false;

   if(NonEmptyGroup)
   	ShowInfo(IDK_ReadyP);
   else
   	BeepInfo(IDK_ReadyG);
}

__fastcall TGroupFrm::~TGroupFrm()
{
	delete PlayerBox;
   delete MemberBox;
}
//---------------------------------------------------------------------------

int __fastcall TGroupFrm::CodeAnalysis(String& Group)
{
   int Id = IdCombo->ItemIndex, Code = CodeCombo->ItemIndex;

   const int Report = GroupsTube.CodeForgeGroup(Id, Code, Group);

   IdCombo->ItemIndex = Id;	CodeCombo->ItemIndex = Code;

   return Report;
}

int __fastcall TGroupFrm::IdAnalysis(String& Group)
{
   int Id = IdCombo->ItemIndex, Code = CodeCombo->ItemIndex;

   const int Report = GroupsTube.IdForgeGroup(Id, Code, Group);

   IdCombo->ItemIndex = Id;	CodeCombo->ItemIndex = Code;

   return Report;
}

void __fastcall TGroupFrm::CodeComboClick(TObject *Sender)
{
	using TourGroupsMaster::MGroupsTube::fgValid;
   using TourGroupsMaster::MGroupsTube::fgEmpty;

	ClearInfo();

   String Group;

   const int Report = CodeAnalysis(Group);

   if(Report <= fgEmpty)
   {
      if(Report==fgValid)
      {
      	ShowMembers(Group);

         AddBtn->Enabled = true;		RemoveBtn->Enabled = true;
      }
      else
      {
      	ClearMembers();

         AddBtn->Enabled = true;		RemoveBtn->Enabled = false;
      }

      if(Report>=2)
      	ToolsGUI::OperatorError(IDK_GFORGE+Report);
      else
      	ShowInfo(IDK_GFORGE+Report);
   }
   else
   {
   	ClearMembers();

      AddBtn->Enabled = false;		RemoveBtn->Enabled = false;

      BeepInfo(IDK_GFORGE+Report);
   }

   OkBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::IdComboClick(TObject *Sender)
{
	using TourGroupsMaster::MGroupsTube::fgValid;
   using TourGroupsMaster::MGroupsTube::fgEmpty;

	ClearInfo();

   String Group;

   const int Report = IdAnalysis(Group);

   if(Report <= fgEmpty)
   {
      if(Report==fgValid)
      {
      	ShowMembers(Group);

         AddBtn->Enabled = true;		RemoveBtn->Enabled = true;
      }
      else
      {
      	ClearMembers();

         AddBtn->Enabled = true;		RemoveBtn->Enabled = false;
      }

      if(Report>=2)
      	ToolsGUI::OperatorError(IDK_GFORGE+Report);
      else
      	ShowInfo(IDK_GFORGE+Report);

   }
   else
   {
   	ClearMembers();

      BeepInfo(IDK_GFORGE+Report);

      AddBtn->Enabled = false;		RemoveBtn->Enabled = false;
   }

   OkBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::AddBtnClick(TObject *Sender)
{
	ClearInfo();

   if(!PlayerBox->SelCount)
   {
   	BeepInfo(IDK_NoPlayers);	return;
   }

   String Group;	const int Report = IdAnalysis(Group);

   if(Report>TourGroupsMaster::MGroupsTube::fgEmpty)
   {
   	AddBtn->Enabled = false;		RemoveBtn->Enabled = false;

   	BeepInfo(IDK_GFORGE + Report);	return;
   }

   PlayerBox->Items->BeginUpdate();
   MemberBox->Items->BeginUpdate();

	for(int p=0; p<PlayerBox->Items->Count; p++)
   {
   	String Player = PlayerBox->Items->Strings[p];

   	if(PlayerBox->Selected[p])									// So player was selected
         if(MemberBox->Items->IndexOf(Player)<0)			// Is not already a member
         {
            Player[1] = Group[1];  Player[2] = Group[2];

            PlayerBox->Items->Strings[p] = Player;			// New group for Player

            MemberBox->Items->Add(Player);					// Add to Members
         }
   }
   MemberBox->Items->EndUpdate();
   PlayerBox->Items->EndUpdate();

   RemoveBtn->Enabled = true;

   OkBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::RemoveBtnClick(TObject *Sender)
{
	using Tools::cSpace;
   
	ClearInfo();

   if(!MemberBox->SelCount)
   {
   	BeepInfo(IDK_NoMembers);	return;
   }

   String Group;	const int Report = IdAnalysis(Group);

   if(Report>TourGroupsMaster::MGroupsTube::fgEmpty)
   {
   	AddBtn->Enabled = false;		RemoveBtn->Enabled = false;

   	BeepInfo(IDK_GFORGE + Report);	return;
   }

   PlayerBox->Items->BeginUpdate();
	for(int m=0; m<MemberBox->Items->Count; m++)
      if(MemberBox->Selected[m])									// So member was selected
      {
         String Player = MemberBox->Items->Strings[m];	// Update Player List

         const int p = PlayerBox->Items->IndexOf(Player);

         Player[1] = cSpace;  Player[2] = cSpace;

         PlayerBox->Items->Strings[p] = Player;			   // Cleared group for Player
      }
   PlayerBox->Items->EndUpdate();

   ShowMembers(Group);
   
   if(!MemberBox->Items->Count)
   {
   	AddBtn->Enabled = true;		RemoveBtn->Enabled = false;

      BeepInfo(IDK_GroupEmptied);
   }
   OkBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::HelpBtnClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_TMGROUPS); }catch(...){ }	
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::FormClose(TObject *Sender, TCloseAction &Action)
{
	try{ Application->HelpCommand(HELP_QUIT, 0); }catch(...){ }		
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::OnMouseWheelUpDn(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
	Handled = true;	
}
//---------------------------------------------------------------------------

void __fastcall TGroupFrm::FormMouseWheel(TObject *Sender,
      TShiftState Shift, int WheelDelta, TPoint &MousePos, bool &Handled)
{
	Handled = true;	
}
//---------------------------------------------------------------------------

