////////////////////////////////////////////////////////////////////////////////
// MODULE MenuMaster:  Interface Menus, Popups to GTourMaster G.Kaniuk 22/05/04
//
#include <stdio.h>
#include <vcl.h>
#include <string>
#include <algorithm>
#include <functional>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "GoDrawRES.h"
#include "SetsRES.h"
#include "TourRES.h"
#include "PairRES.h"

#include "Strings.h"
#include "Format.h"
#include "Sort.h"

#include "GoDrawGUI.h"
#include "Regmaster.h"
#include "DrawMaster.h"
#include "TourStream.h"
#include "Model.h"

#include "MenuMaster.h"
//------------------------------------------------------------------------------
#pragma package(smart_init)
//------------------------------------------------------------------------------

namespace MenuMaster												// TOOLS
{
	const String NoValue("  .."), NoOpp("  ...");

 	void OpInfo(TStatusBar* StatusBar, String Message)
   {
   	StatusBar->Panels->Items[0]->Text = Message;
   }

 	void OpInfo(TStatusBar* StatusBar, int MessID)
   {
   	StatusBar->Panels->Items[0]->Text = LoadStr(MessID);
   }

	void OpErr(TStatusBar* StatusBar, int MessID)
   {
   	Beep();

   	StatusBar->Panels->Items[0]->Text = LoadStr(MessID);
   }

   void ClearBar(TStatusBar* StatusBar)
   {
   	StatusBar->Panels->Items[0]->Text = String();
   }

   void SelectFirstItem(TComboBox* ComboBox)							// This is only used after
   {                                                           // typing the first char into
   	String Source = ComboBox->Text.Trim().LowerCase();       // the ComboBox.  It leaves the
                                                               // the text after the first char
      ComboBox->ItemIndex = ComboBox->Items->IndexOf(Source);  // selected

      ComboBox->DroppedDown = true;

      if(ComboBox->DroppedDown)
      	return;

      if(ComboBox->ItemIndex < 0)
      {
      	int SourceLength = Source.Length();

         for(int i = 0; i<ComboBox->Items->Count; i++)
         {
            String Item = ComboBox->Items->Strings[i];

            if(Source == Item.SubString(1, SourceLength).LowerCase())
            {
               ComboBox->ItemIndex = i;
               ComboBox->Text      = Item;

               break;
            }
         }
      }

      if(ComboBox->ItemIndex >= 0)
      {
      	ComboBox->SelStart = 1;
         ComboBox->SelLength = ComboBox->Text.Length()-1;
      }
   }

   void LoadCombo(TComboBox* ComboBox, int Count, String* Strings, int Selected)
   {
   	ComboBox->Clear();

      for(int i=0; i<Count; i++)
      	ComboBox->Items->Add(Strings[i]);

      if(0<=Selected && Selected<Count)
      	ComboBox->ItemIndex = Selected;
   }

   void LoadRounds(TComboBox* RoundBox, int Rounds, int sRound)
   {
      if(Rounds<1 || Rounds>Tours::max_rounds) return;

   	RoundBox->Clear();

      if(sRound>0)
      {
         for(int r = 0; r<Rounds; r++)
            RoundBox->Items->Add(String(r+1));

         if(sRound<=Rounds)
            RoundBox->ItemIndex = sRound - 1;
      }
      else
      {
      	RoundBox->Items->Add(String(0));
         RoundBox->ItemIndex = 0;
      }
   }

   void LoadPrizes(TComboBox* PrizeBox, int Rounds, int sPrize)
   {
   	char prize[5];

      PrizeBox->Clear();

      for(int w=0; w<=2*Rounds; w++)
      {
      	int mainprize =w/2;
         char halfprize = w%2 ? Tools::cHalf : Tools::cSpace;

         sprintf(prize, "%2d%c%c", mainprize, halfprize, Tools::cSpace);

         PrizeBox->Items->Add(String(prize));
      }

      if(0 <= sPrize && sPrize <= 2*Rounds)
      	PrizeBox->ItemIndex = sPrize;
   }

   void LoadGrades(TComboBox* GradeBox, int Grade)
   {
   	using Tools::trim;
      using Tools::s_S;

   	using Tours::pro_grade;
      using Tours::mc_scale;
      using Tours::grade_string;

   	GradeBox->Clear();

      int gmax = pro_grade + 8*mc_scale, gmin = -99*mc_scale;

      for(int g = gmax; g>=gmin; g -= mc_scale)
      {
         String Grade = s_S(trim(grade_string(g)));

         GradeBox->Items->Add(Grade);
      }

	   String GradeTxt =  s_S(trim(grade_string(Grade)));

      GradeBox->ItemIndex = GradeBox->Items->IndexOf(GradeTxt);
	}

   String TieName(int Level, int nTie)
   {
   	if(Level!=1 && Level !=2) return String();

      if(nTie<0 || nTie>=Tours::tbSize) return String();

      if(nTie>0)
         return LoadStr(IDM_TIEBREAK + nTie);

      if(Level==1)return LoadStr(IDT_notie1);
      if(Level==2)return LoadStr(IDT_notie2);

      return String();
   }

   String TieValue(int nTie, int vTie)
   {
   	using Tools::s_S;

   	if(nTie<=0 || nTie>=Tours::tbSize) return String();

      if(nTie == Tours::tbWins) return s_S(Tours::wins_ustring(vTie));

      return s_S(Tours::mms_ustring(vTie));
   }

   String GameOpponent(Tours::GGame& Game)
   {
   	Tours::GPlayerPtr Opp = TourMaster::GoTour.Listed[Game.Oppid];

      if(GoTour.State <= Tours::tsRegClosed)return String();

      return Game.Oppid>0 ? Tools::s_S(GoTour.Name(Opp)) : NoOpp;
   }

   String GameColour(Tours::GGame& Game)
   {
   	if(GoTour.State <= Tours::tsRegClosed)return String();

   	switch((int)Game.Colour)
      {
      	case Tours::gcBlack: return String("B");
         case Tours::gcWhite: return String("W");

         default: 				return NoValue;
      }
   }

   String GameParity(Tours::GGame& Game)
   {
   	if(GoTour.State <= Tours::tsRegClosed)return String();

      if(Game.Result==Tours::grSleep) return NoValue;

   	switch((int)Game.Parity)
      {
      	case Tours::gdUp:   return String("Up");
         case Tours::gdDown: return String("Down");
         case Tours::gdEven: return String("Even");

         default:				  return NoValue;
      }
   }

   String GameStones(Tours::GGame& Game)
   {
   	if(!Game.Stones) return NoValue;

      return String((int)Game.Stones/Tours::mc_scale);
   }

   String GameKomi(Tours::GGame& Game)
   {
   	if(!Game.Komi) return NoValue;

      return String((int)Game.Komi/Tours::mc_scale);
   }

   String GameResult(Tours::GGame& Game)
   {
   	switch(Game.Result)
      {
      	case Tours::grLose: 	return String("0");
         case Tours::grJigo:  return String("J");
         case Tours::grWin:   return String("1");
         case Tours::grUsolv: return String("Unfin");
         case Tours::grDlos:  return String("Dloss");
         case Tours::grFwin:  return String("Fwin");
         case Tours::grVoid:  return String("Void");
         case Tours::grSleep: return String("N");
         case Tours::grNone:	return String("Y");

         default:					return String();
      }
   }

   String McScore(int mms)
   {
   	return Tools::s_S(Tours::mms_ustring(mms));
   }
}

namespace MenuMaster												// CLASS MStringList
{
	void MStringList::StartCombo(TComboBox* ListBox)
   {
      ListBox->Sorted = StringList->Sorted;

      ListBox->Clear();

      for(int i = 0; i<StringList->Count; i++)
      	ListBox->Items->Add(StringList->Strings[i]);

      ListBox->Text = String();

   	ListBox->ItemIndex = -1;
   }

   void MStringList::StartCombo(TComboBox* ListBox, String& Selected)
   {
		StartCombo(ListBox);

      if(Selected.IsEmpty()) return;

      ListBox->ItemIndex = ListBox->Items->IndexOf(Selected);

      if(ListBox->ItemIndex >= 0)
      	ListBox->Text = ListBox->Items->Strings[ListBox->ItemIndex];
      else
      	ListBox->Text = "Please Select";
   }
}
//------------------------------------------------------------------------------



namespace FileMaster						// FILE MENU TUBES
{
	using TourMaster::GoTour;

   MNewTube::MNewTube()
   {
   	unsigned short sDay, sMonth, sYear;

   	TDateTime DateNow = TDateTime().CurrentDate();

      DateNow.DecodeDate(&sYear, &sMonth, &sDay);

      Year         = String(sYear);
      TourFile 	 = String();
      SeedFile		 = String();
   	TourName 	 = String();	TourList.StringList->Clear();
      
      ToursDir 	 = GoDraw::FilePath(GoDraw::GoTours);
      TourListFile = GoDraw::FilePath(GoDraw::GoTourList);

      if(FileExists(TourListFile))
      	TourList.StringList->LoadFromFile(TourListFile);
   }

   bool MNewTube::ForgeTourFileName(void)
   {
   	using Tools::s_S;
      using Tools::S_s;
      using Tools::strip_white_space;

   	TourFile = String();

      if(Year.IsEmpty()) return false;

   	String sTour = s_S(strip_white_space(S_s(TourName)));	if(sTour.IsEmpty()) return false;

      String fRoot = sTour + String(Tools::sUline)
      							+ Year.SubString(3,2)			// Year guaranteed 4 digits
                           + Tools::s_S(Tours::TourExt);

      TourFile = GoDraw::TourFilePath(fRoot);

      return true;
   }

   void  MNewTube::SaveTourList(void)
   {
   	String sTourName = TourName.Trim();    if(sTourName.IsEmpty()) return;

      if(TourList.StringList->IndexOf(sTourName) >= 0) return;

      TourList.StringList->Add(sTourName);

      if(FileExists(TourListFile))
      	TourList.StringList->SaveToFile(TourListFile);
   }
}
//------------------------------------------------------------------------------

namespace TourFormMaster										// PAIR DETAILS TUBE
{
	using TourMaster::GoTour;
   using Tools::s_S;
   using DrawMaster::coCount;

   MPairTube::MPairTube(const int board, const GBoard& pairboard)
   {
      if(board<1) return;

      // TITLE
      TourNameYear = s_S(GoTour.TourNameYear());

      // BOARD AND ROUND
      PairBoard  = pairboard;		Board = board; 	Round = GoTour.Round;

      BoardRound  = LoadStr(IDT_Board) + String(Board) + String("     ");
      BoardRound += LoadStr(IDT_Round) + String(Round);

      // COMMENTS
      std::vector<bool>Notes(coCount);

      Comments.resize(coCount);

      DrawMaster::GameComment(PairBoard, Notes);

      int CommentNow = 0;

      for(int k=0; k<coCount; k++)
      {
      	Comments[k] = String();

      	if(Notes[k])                                          // Observe that
         	Comments[CommentNow++] = LoadStr(IDT_COMMENT + k); // CommentNow<=k
      }
   }

   String MPairTube::Pair(int r, int c)
   {
   	using std::string;
      
   	using Tools::s_S;
      using Tools::to_string;
      
      using Tours::mms_ustring;
      using Tours::mc_scale;
      using Tours::GPlayer;

   	using TourMaster::GoTour;

      using DrawMaster::wins_string;
      using DrawMaster::points_string;
      using DrawMaster::homegames_string;

   	const String Empty = String();

   	if(c<0 || c>2) return Empty;

      GPlayer* Black = GoTour.Listed[PairBoard.Black];
      GPlayer* White = GoTour.Listed[PairBoard.White];

      int Round = GoTour.Round;

      string Bmmg = string(), Bmmi = string();
      string Wmmg = string(), Wmmi = string();

      string Bcbal = string(), Bmbal = string(), Bmout = string();
      string Wcbal = string(), Wmbal = string(), Wmout = string();

      if(Round>0)
      {
      	if(Black)
         {
         	Bmmg  = mms_ustring(Black->McMahonScore(Round-1));
            Bmmi  = mms_ustring(Black->MiScore);
            Bmbal = to_string(Black->Mbal);
            Bmout = to_string(Black->Mout);
            Bcbal = to_string(Black->Cbal);
         }

         if(White)
         {
         	Wmmg  = mms_ustring(White->McMahonScore(Round-1));
            Wmmi  = mms_ustring(White->MiScore);
            Wmbal = to_string(White->Mbal);
            Wmout = to_string(White->Mout);
            Wcbal = to_string(White->Cbal);
         }
      }

      switch(r)
      {
      	case 0:
         {
         	if(c==0) return Empty;
            if(c==1) return LoadStr(IDT_BLACK);
            if(c==2) return LoadStr(IDT_WHITE);
         }
         break;

         case 1:
         {
         	if(c==0) return String("Name");
            if(c==1) return s_S(GoTour.ShortNameGrade(Black)); // TODO: if Black==NULL?
            if(c==2) return s_S(GoTour.ShortNameGrade(White));
         }
         break;

         case 2:
         {
         	if(c==0) return String("Location");
            if(c==1) return s_S(GoTour.ShortLandClub(Black));
            if(c==2) return s_S(GoTour.ShortLandClub(White));
         }
         break;

         case 3:
         {
         	if(c==0) return LoadStr(IDT_MMI);
            if(c==1) return s_S(Bmmi);
            if(c==2) return s_S(Wmmi);
         }
         break;

         case 4:
         {
         	if(c==0) return LoadStr(IDT_MMC);
            if(c==1) return s_S(Bmmg);
            if(c==2) return s_S(Wmmg);
         }
         break;

         case 5:
         {
         	if(c==0) return LoadStr(IDT_Points);
            if(c==1) return s_S(points_string(Black));
            if(c==2) return s_S(points_string(White));
         }
         break;

         case 6:
         {
         	if(c==0) return LoadStr(IDT_Wins);
            if(c==1) return s_S(wins_string(Black));
            if(c==2) return s_S(wins_string(White));
         }
         break;

         case 7:
         {
         	if(c==0) return LoadStr(IDT_Mbal);
            if(c==1) return s_S(Bmbal);
            if(c==2) return s_S(Wmbal);
         }
         break;

         case 8:
         {
         	if(c==0) return LoadStr(IDT_Cbal);
            if(c==1) return s_S(Bcbal);
            if(c==2) return s_S(Wcbal);
         }
         break;

         case 9:
         {
         	if(c==0) return LoadStr(IDT_Mout);
            if(c==1) return s_S(Bmout);
            if(c==2) return s_S(Wmout);
         }
         break;

         case 10:
         {
         	if(c==0) return LoadStr(IDT_Home);
            if(c==1) return s_S(homegames_string(Black));
            if(c==2) return s_S(homegames_string(White));
         }
         break;

      	default: return Empty;
      }
      return Empty;
   }
}
//---------------------------------------------------------------------------

namespace MonitorMaster
{
	using std::vector;
	using PairMaster::PairCount;
   using TourMaster::GoTour;
   using Tools::s_S;
   
	MMonitorTube::MMonitorTube(PairCount& model, PairCount& actual, vector<PairCount>& trials)
   {
   	Counts = new TStringList();			// MonitorForm will catch if reqd

      const String Unset = String(-1);

      bool Valid = Tours::tsRegClosed < GoTour.State
      										&&
   					  GoTour.State < Tours::tsPastRound;

      // TITLE
      TourName = s_S(GoTour.TourNameYear());

      // MODEL COUNTS
      Counts->Add(Valid ? String(model.Uneven)    : Unset);
      Counts->Add(Valid ? String(model.SameClub)  : Unset);
      Counts->Add(Valid ? String(model.SameLand)  : Unset);

      // ACTUAL COUNTS
      Counts->Add(Valid ? String(actual.Uneven)	  : Unset);
      Counts->Add(Valid ? String(actual.SameClub) : Unset);
      Counts->Add(Valid ? String(actual.SameLand) : Unset);

      // TRIALS
      for(size_t k=0; k<trials.size(); k++)
      {
      	Counts->Add(Valid ? String(trials[k].Uneven)  : Unset);
         Counts->Add(Valid ? String(trials[k].SameClub): Unset);
      	Counts->Add(Valid ? String(trials[k].SameLand): Unset);
      }

      // SLEEPERS DISPLAY
      GatherSleepers();
   }


   void MMonitorTube::GatherSleepers(void)
   {
   	using TourMaster::GoTour;

      // SPACE FOR THE SLEEPER LIST
   	SleepCount = GoTour.Players - GoTour.PlayNowCount;

      if(SleepCount<=0) SleepCount = 0;

      Sleepers.resize(SleepCount);

      if(!SleepCount) return;

      // COLLECT SLEEPERS
      int i=0;

      for(size_t p=1; p<=GoTour.Players; p++)
         if(GoTour.Listed[p]->Sleeping(GoTour.Round))
         	Sleepers[i++] = iSleep(p);

      // SORT IN NAME ORDER
      vector<iSleep>::iterator bsleep = Sleepers.begin();
      vector<iSleep>::iterator esleep = Sleepers.end();

      std::sort(bsleep, esleep, std::less<iSleep>());
   }

   String MMonitorTube::GetSleeperField(const int col, const int row)
   {
   	using TourMaster::GoTour;
      using Tools::spaces;

   	if(row<0) return String();

   	if(row==0)
      	switch(col)
         {
         	case 0:  return LoadStr(IDQ_Player);
            case 1:  return LoadStr(IDQ_Rounds);
            default: return String();
         }

      if(row>(int)Sleepers.size())return String();

      Tours::GPlayerPtr P = GoTour.Listed[Sleepers[row-1].index];

      switch(col)
      {
      	case 0:	return s_S(GoTour.ShortNameGrade(P));
         case 1:	return s_S(GoTour.Rounds(P));
         default:	return String();
      }
   }
}
//---------------------------------------------------------------------------

namespace TourRankMaster                                         // ONLINE RANKLIST
{
	using TourMaster::GoTour;
   using Tools::to_string;
   
	static const string fmtPid  = " 0000";      // Player id
   static const string fmtRnd  = "######";
   static const string noGame  = " ... ";
   static const string noField = "?";
   
   MRankTube::MRankTube()
   {
      // GK 2015 RankSort disturbs Sorted OldSortSense = GoTour.SortSense; OldSortOrder = GoTour.SortOrder;
      Active = Tours::RankSort(&GoTour);

      Fields = GoTour.Rules.Rounds + 11;

      TourNameYear = GoTour.TourNameYear();
   }

   string MRankTube::GetField(int col, int row)
   {
   	using Tools::load_str;
      using Tools::pad_right;
      
      using Tours::Score;
      
   	using Tours::grade_string;
      using Tours::wins_string;
      using Tours::mms_string;
      using Tours::result_char;

   	if(row==0)
      {
      	int IDT_Tie1 = GoTour.Rules.TieOne ? IDL_GNONE + GoTour.Rules.TieOne : IDT_NOTIE1;
         int IDT_Tie2 = GoTour.Rules.TieTwo ? IDL_GNONE + GoTour.Rules.TieTwo : IDT_NOTIE2;

      	switch(col)
         {
         	case 0: return load_str(IDL_GRANK);
            case 1: return pad_right(load_str(IDT_HID), 8);
            case 2: return pad_right(load_str(IDT_HNAME),32);
            case 3: return load_str(IDL_Grade);
            case 4: return load_str(IDL_Club);
            case 5: return load_str(IDL_Land);
            case 6: return load_str(IDL_Group);
            case 7: return load_str(IDL_GWINS);
            case 8: return load_str(IDL_GMMS);
            case 9: return load_str(IDT_Tie1);
            case 10: return load_str(IDT_Tie2);

            default: if(col>10+(int)GoTour.Rules.Rounds) return string();
         }

         return Tools::to_string(fmtRnd, col-10);
      }

      Tours::GPlayerPtr P = GoTour.GetPlayerPtr(row);  if(!P) return noField;

      Tools::ints mms;

      const int Round = GoTour.Round;

      if(Round<1)
      	mms = P->MiScore;
      else
       	mms = P->McScore;

      switch(col)
      {
      	case  0: return to_string(P->Rank);
         case  1: return to_string(fmtPid, (int)P->Pid) + "  ";
         case  2: return GoTour.Name(P);
         case  3: return grade_string(P->Grade);
         case  4: return P->Club;
         case  5: return P->Land;
         case  6: return GoTour.Group(P);
         case  7: return wins_string(GoTour.WinsValue(P));
         case  8: return mms_string(mms);
         case  9: return GoTour.Rules.TieOne ? mms_string(P->TieOne) : string();
         case 10: return GoTour.Rules.TieTwo ? mms_string(P->TieTwo) : string();

         default: if(col>10+(int)GoTour.Rules.Rounds) return string();
      }

      const size_t round = col-10;

      if(round <= GoTour.Round)
         if(P->Awake(round))
         {
            Tours::GGame& Game = P->Games[round-1];

            if(Game.Oppid>0)
            {
               int OppRankId = (int)GoTour.Listed[Game.Oppid]->Rank;

               if(OppRankId>0)
                  return to_string(OppRankId) + string(1, result_char(Game.Result));
            }
         }

      return noGame;
   }
}
//---------------------------------------------------------------------------

namespace TourPrizeMaster                                         // ONLINE RANKLIST
{
	using TourMaster::GoTour;
   using Tools::to_string;
   
	static const string fmtPid  = " 0000";      // Player id
   static const string noField = string("?");

   static string Spaces(const int n)
   {
   	return string(n, Tools::cSpace);
   }

   MPrizeTube::MPrizeTube()
   {
      // GK REGSORT 2015 OldSortSense = GoTour.SortSense; OldSortOrder = GoTour.SortOrder;
      Active = Tours::PrizeSort(&GoTour);

      Fields = 8;

      TourNameYear = GoTour.TourNameYear();
   }

   string MPrizeTube::GetField(int col, int row)
   {
   	using Tools::load_str;
      using Tools::cSpace;

      using Tours::Score;

   	using Tours::grade_string;
      using Tours::wins_string;
      using Tours::mms_string;
      using Tours::result_char;

   	if(row==0)
      switch(col)
      {
         case 0: return string(10, cSpace);
         
         case 1: return load_str(IDT_HID);
         case 2: return load_str(IDT_HNAME);
         case 3: return load_str(IDL_Grade);
         case 4: return load_str(IDL_Club);
         case 5: return load_str(IDL_Land);
         case 6: return load_str(IDT_Wins);
         case 7: return load_str(IDT_Rank);

         default: return string();
      }

      Tours::GPlayerPtr P = GoTour.GetPlayerPtr(row);  if(!P) return noField;

      switch(col)
      {
      	case  0: return to_string(row);
         case  1: return to_string(fmtPid, (int)P->Pid);
         case  2: return GoTour.Name(P);
         case  3: return Spaces(2)+GoTour.Grade(P);
         case  4: return Spaces(2)+GoTour.Club(P);
         case  5: return Spaces(2)+GoTour.Land(P);
         case  6: return Spaces(2)+wins_string(GoTour.WinsValue(P));
         case  7: return Spaces(2)+to_string(P->Rank);

         default: return string();
      }
   }
}
//---------------------------------------------------------------------------

namespace TrainMaster												// TRAINING
{
	using std::string;

   using TourMaster::GoTour;

   MTrainTube::MTrainTube()
   {
   	TourNameYear  = GoTour.TourNameYear();
      
   	Byte drawmode = PairMaster::GetPairMode(GoTour.PairParam);
      Byte partdraw = PairMaster::GetPartDraw(GoTour.PairParam);

      DrawMode 	  = (int)drawmode;
      PartDraw 	  = (int)partdraw;
   }

   bool MTrainTube::SimulateResults(void)
   {
   	using TourMaster::GoTour;

   	for(size_t b=1; b<GoTour.Pair.size(); b++)
      	if(!Model::SimulateGame(b))
         	return false;

      GoTour.PopulationCount();

      GoTour.SetState();
      
      return true;
   }

   bool MTrainTube::ClearResults(void)
   {
   	using TourMaster::GoTour;

   	for(size_t b=1; b<GoTour.Pair.size(); b++)
      	if(!DrawMaster::EditResult(b, Tours::grNone, Tours::grNone))
         	return false;

      GoTour.PopulationCount();

      GoTour.SetState();

      return true;
   }

   void MTrainTube::HarvestModes(void)
   {
   	Byte drawmode = (Byte)DrawMode;
      Byte partdraw = (Byte)PartDraw;

   	PairMaster::PutPairMode(drawmode, GoTour.PairParam);
      PairMaster::PutPartDraw(partdraw, GoTour.PairParam);
   }

   bool MTrainTube::CanChangeResults(void)
   {
   	using TourMaster::cmdResult;
      
   	const bool CanEditRes = TourMaster::EnabledCommands[cmdResult][GoTour.State];
      const bool CanSimClr  = DrawMode != PairMaster::pmNormal;

      return CanEditRes && CanSimClr;
   }
}
//---------------------------------------------------------------------------

namespace TourGroupsMaster											// GROUPS
{
	using std::string;
   
	using Tools::s_S;
   using Tools::S_s;
   using Tools::spaces;

   using Tours::GPlayerPtr;
   using TourMaster::GoTour;

   MGroupsTube::MGroupsTube(const int selectedPlayer)
   {
   	TourNameYear = s_S(GoTour.TourNameYear());

      Players = GoTour.Players;

      // GROUP CODES
      GroupCodes = s_S(Tours::GroupCodes + spaces(1));

      const int codeCount = GroupCodes.Length();

      GroupCodes[codeCount-1] = GroupCodes[codeCount-2];

      // GROUP ID'S
      GroupIDs = String("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ ");

      const int idCount = GroupIDs.Length();

      // ESTABLISH SELECTED PLAYER'S INDEXES
      SelectedPlayer = 0;

      GroupIndex = idNone;
      GroupCodeIndex = gaNone;

      if(selectedPlayer>0)
      {
      	SelectedPlayer = selectedPlayer;

         GPlayerPtr Pselected = GoTour.GetPlayerPtr(selectedPlayer);

      	const string& group = Pselected->Group;

         char groupId = group[0], groupCode = group[1];

         int id,code;

         for(id=1; id<=idCount; id++)
         	if(GroupIDs[id]==groupId)
            	break;
         GroupIndex = id-1;

         for(code=1; code<=codeCount; code++)
         	if(GroupCodes[code] == groupCode)
            	break;
         GroupCodeIndex = code-1;

         if(GroupIndex == 0)								// Top MM group
         	GroupCodeIndex = codeCount-2;

         if(1<=GroupIndex && GroupIndex<=9)			// Supergroup
         	GroupCodeIndex = codeCount-1;
      }
      else
      {
      	GroupIndex = idCount - 1;
         GroupCodeIndex = codeCount - 1;
      }

      // GATHER EXISTING LIST OF GROUPS
      Groups.clear();

      list<string>::iterator sgroup = Groups.begin();
      list<string>::iterator fgroup = Groups.begin();

      for(int p=1; p<=Players; p++)
      {
      	const string group = GoTour.Group(GoTour.GetPlayerPtr(p));

         if(!Groups.size())
         	Groups.push_back(group);
         else
         {
         	fgroup = find(sgroup, Groups.end(), group);

            if(fgroup == Groups.end())
            	Groups.push_back(group);
         }
      }
   }

   bool MGroupsTube::ParseGroup
   (
   	const String& player,	// Player picked from list

   	int& groupId,				// Index to the GroupId combo
      int& groupCode				// Index to the GroupAttribute combo
   )
   {
   	groupId = GroupIDs.Length()-1;				// No group by default
      groupCode = GroupCodes.Length()-1;

   	if(player.Length()<2) return false;

      const char id = player[1], code = player[2];

      // FIND THE groupId
      for(int i=1; i<=GroupIDs.Length(); i++)
      	if(id == GroupIDs[i])
         {
         	groupId = i-1; break;
         }

      // FIND THE groupCode
      for(int i=1; i<=GroupCodes.Length(); i++)
      	if(code == GroupCodes[i])
         {
         	groupCode = i-1; break;
         }

      if(1<= groupId && groupId <= 9)				// Supergroup
      	if(groupCode == GroupCodes.Length()-3)
         	++groupCode;
         else
         	return false;

      return true;										// Always a valid Group
   }

   int MGroupsTube::IdForgeGroup
   (
   	int& groupId,			// Index to the GroupId combo
      int& groupCode,	   // Index to the GroupAttribute combo

      String &Group			// Constructed from Indexes
   )
   {
      string group(2, Tools::cSpace);  Group = s_S(group);

      if(groupId<0 || groupCode<0)
      	return fgBadLoIndex;

      if(groupId>=GroupIDs.Length() || groupCode>=GroupCodes.Length())
      	return fgBadHiIndex;

      if(groupId==idTop)
      	groupCode = gaTop;

      if(idOne<=groupId && groupId<=idNine)
      	groupCode = gaSuper;

      if(idA<=groupId && groupId<=idZ)
      	if(groupCode > gaPref)
         {
         	groupCode = gaNone;

         	return fgWantNormal;
         }

      if(groupId==idNone)
      {
      	groupCode = gaNone;

         return fgWantGroup;
      }

      group[0] = GroupIDs[groupId+1];
      group[1] = GroupCodes[groupCode+1];

      Group = s_S(group);

      if(find(Groups.begin(), Groups.end(), group) == Groups.end())
      	return fgEmpty;
      else
      	return fgValid;
   }

   int MGroupsTube::CodeForgeGroup
   (
   	int& groupId,			// Index to the GroupId combo
      int& groupCode,		// Index to the GroupAttribute combo

      String &Group			// Constructed from Indexes
   )
   {
   	string group(2, Tools::cSpace);  Group = s_S(group);

      if(groupId<0 || groupCode<0)
      	return fgBadLoIndex;

      if(groupId>=GroupIDs.Length() || groupCode>=GroupCodes.Length())
      	return fgBadHiIndex;

      if(groupCode == gaTop)
      	groupId = idTop;

      if(groupCode == gaSuper)
      	if(groupId==0 || groupId>9)
         {
         	groupId = idNone;

            return fgWantDigit;
         }

      if(gaMust <= groupCode && groupCode <= gaPref)
      	if(groupId<idA || groupId>idZ)
         {
         	groupId = idNone;

         	return fgWantAlpha;
         }

      if(groupCode == gaNone)
      {
      	groupId = idNone;

         return fgWantGroup;
      }

      group[0] = GroupIDs[groupId+1];
      group[1] = GroupCodes[groupCode+1];

      Group = s_S(group);

      if(find(Groups.begin(), Groups.end(), group) == Groups.end())
      	return fgEmpty;
      else
      	return fgValid;
   }

  	String MGroupsTube::PlayerText(const int p)
   {
      using Tools::tab_add;
      using Tools::sEmpty;

   	GPlayerPtr P = GoTour.GetPlayerPtr(p);  // In register sort order

      string players(P->Group);					 // Tabbed fields

      tab_add(players, GoTour.ShortName(P));
      tab_add(players, GoTour.Grade(P));
      tab_add(players, GoTour.ShortLandClub(P));

      return s_S(players);
   }

   void MGroupsTube::Add(const String& Group)
   {
   	const string group = S_s(Group);

      list<string>::iterator fgroup = find(Groups.begin(), Groups.end(), group);

      if(fgroup == Groups.end())
      	Groups.push_back(group);
   }

   void MGroupsTube::Remove(const String& Group)
   {
   	const string group = S_s(Group);

      list<string>::iterator fgroup = find(Groups.begin(), Groups.end(), group);

      if(fgroup != Groups.end())
      	Groups.erase(fgroup);
   }

   bool MGroupsTube::Find(const String& Group)
   {
   	const string group = S_s(Group);

      return find(Groups.begin(), Groups.end(), group) != Groups.end();
   }
}
//---------------------------------------------------------------------------

namespace TourCloseRegMaster
{
	MCloseRegTube::MCloseRegTube()
   {
   	TourNameYear = Tools::s_S(TourMaster::GoTour.TourNameYear());

      Action = RegMaster::crDelete;
   }
}
//---------------------------------------------------------------------------

namespace TourJoinMaster
{
	using Tours::GPlayerPtr;

	using TourMaster::GoTour;

	MJoinDrawTube::MJoinDrawTube()
   {
   	TourNameYear = Tools::s_S(TourMaster::GoTour.TourNameYear());

      PlayerCount  = GoTour.Players;
   	JoiningCount = GoTour.JoiningCount;
   }

   bool MJoinDrawTube::JoiningPlayer(const int p)
   {
   	bool PlayNow, PlayNext;

      if(p<1 || p>PlayerCount) return false;

      Tours::GPlayer& Player = *GoTour.Listed[p];

      GoTour.PlayNowNext(Player, PlayNow, PlayNext);

      return !PlayNow && PlayNext;
   }

   String MJoinDrawTube::JoiningPlayerText(const int p)
   {
   	if(p<1 || p>PlayerCount) return String();

      GPlayerPtr Player = GoTour.Listed[p];

      std::string PackedPlayer(GoTour.Name(Player));

      Tools::tab_add(PackedPlayer, GoTour.Grade(Player));
      Tools::tab_add(PackedPlayer, GoTour.Rounds(Player));

      return Tools::s_S(PackedPlayer);
   }
}
//---------------------------------------------------------------------------

