////////////////////////////////////////////////////////////////////////////////
// MODULE MenuMaster:  Interface Menus, Popups to GTourMaster G.Kaniuk 22/05/04
//

#include <vector>
#include <string>
#include <list>
#include <vcl.h>
#include <comctrls.hpp>

#include "TourMaster.h"
#include "PairMaster.h"

#ifndef MenuMasterH
#define MenuMasterH

namespace MenuMaster													// GENERAL TOOLS
{
   using std::list;

	using TourMaster::GoTour;

   struct MStringList
   {
   	TStringList* StringList;

      MStringList()
      {
      	 StringList = new TStringList();

          StringList->Sorted = true;
      };

      ~MStringList() { delete StringList; };

      void StartCombo(TComboBox* ListBox);
      void StartCombo(TComboBox* ListBox, String& Selected);
   };

   void OpInfo(TStatusBar* StatusBar, String Message);
   void OpInfo(TStatusBar* StatusBar, int MessID);
   void OpErr(TStatusBar* StatusBar, int MessID);
   void ClearBar(TStatusBar* StatusBar);

   void SelectFirstItem(TComboBox* ComboBox);

   void LoadCombo(TComboBox* ComboBox, int Count, String* Strings, int Selected);
   void LoadRounds(TComboBox* RoundBox, int Rounds, int sRound);
   void LoadPrizes(TComboBox* PrizeBox, int Rounds, int sPrize);
   void LoadGrades(TComboBox* GradeBox, int Grade);

   String TieName(int Level, int nTie);
   String TieValue(int nTie, int vTie);

	String Playing(int Round);

   String McScore(int mms);
}
//------------------------------------------------------------------------------

namespace FileMaster						// FILE MENU TUBES
{
	using MenuMaster::MStringList;
   
   struct MNewTube
   {
      String Year;						// Initial year setting from clock
      String TourName;					// Empty for operator to fill
      String SeedFile;					// Absolute existing file name
      String TourFile;					// The absolute tournament filename
      String TourListFile;				// Absolute TournamentList filename

      String ToursDir;				   // Tours directory

      MStringList TourList;			// List of tournaments

      MNewTube();							// Loads tourlist from GoTours.gdi

      bool ForgeTourFileName();		// Construct from tour and year
      void SaveTourList();				// Operator may have added to list
   };
}
//------------------------------------------------------------------------------

namespace TourFormMaster										// PAIR DETAILS TUBE
{
	using std::vector;
   using Tours::GPlayer;
   using Tours::GBoard;

   class MPairTube
   {
   	public:
       	int Board, Round;

         GBoard PairBoard;

      	String TourNameYear;
         String BoardRound;

         std::vector<String>Comments;

      public:
      	MPairTube(const int board, const GBoard& pairboard);

         String Pair(int r, int c);
   };
}
//------------------------------------------------------------------------------

namespace MonitorMaster										// DRAW MONITOR
{
	using std::vector;
	using PairMaster::PairCount;

   struct iSleep
   {
   	int index;

      iSleep()				 { index = 0; }
      iSleep(const int i){ index = i; }

      bool operator<(const iSleep q)
      {
         const Tours::GPlayerPtr P = TourMaster::GoTour.Listed[index];
         const Tours::GPlayerPtr Q = TourMaster::GoTour.Listed[q.index];

         return P->Name < Q->Name;
      }

   };

   bool operator<(const iSleep& p, const iSleep& q)
   {
      const Tours::GPlayerPtr P = TourMaster::GoTour.Listed[p.index];
      const Tours::GPlayerPtr Q = TourMaster::GoTour.Listed[q.index];

      return P->Name < Q->Name;
   }

   class MMonitorTube
   {
   	private:
      	vector<iSleep> Sleepers;

         void GatherSleepers(void);

   	public:
   		String TourName;

         TStringList* Counts;

      public:
         MMonitorTube(){ TourName = String(); Counts = NULL; }

         ~MMonitorTube(){ delete Counts; }

         MMonitorTube(PairCount& model, PairCount& actual, vector<PairCount>& trials);

      public:
         int SleepCount;

         String GetSleeperField(const int col, const int row);

	};
}
//------------------------------------------------------------------------------

namespace TourRankMaster												// SHOW RANKLIST
{
	using std::string;
   using std::vector;
   using TourMaster::GTourMaster;

   class MRankTube
   {
   	public:
         int Active;
         int Fields;
         
         int OldSortSense, OldSortOrder;

         string TourNameYear;

   	public:
   		MRankTube();

         string GetField(int col, int row);
   };
}
//---------------------------------------------------------------------------

namespace TourPrizeMaster												// SHOW RANKLIST
{
	using std::string;
   using std::vector;
   using TourMaster::GTourMaster;

   class MPrizeTube
   {
   	public:
         int Active;
         int Fields;
         
         int OldSortSense, OldSortOrder;

         string TourNameYear;

   	public:
   		MPrizeTube();

         string GetField(int col, int row);
   };
}
//---------------------------------------------------------------------------

namespace TrainMaster												// TRAINING
{
	using std::string;

   class MTrainTube
   {
   	public:
         string TourNameYear;

         int  DrawMode;			// pmNormal, pmModel
         int  PartDraw;			// Prob range for an incomplete draw

   	public:
   		MTrainTube();

         bool SimulateResults(void);
         bool ClearResults(void);
         void HarvestModes(void);
         bool CanChangeResults(void);
   };
}
//---------------------------------------------------------------------------

namespace TourGroupsMaster											// GROUPS
{
	using std::list;
   using std::string;
   
	class MGroupsTube
   {
   	public:
      	enum
         {
            gaMust, gaMustNot, gaPrefNot, gaPref, gaTop, gaSuper, gaNone
         };

         enum
         {
            idTop=0, idOne=1, idNine=9, idA=10,idZ=35, idNone=36
         };

         enum
         {
         	fgValid, fgEmpty,
            fgWantNormal, fgWantAlpha, fgWantDigit, fgWantGroup,
            fgBadLoIndex, fgBadHiIndex
         };

   	public:
   		String TourNameYear;

         String GroupCodes;									// *~-+::% 	(%==space means no code chosen)
         String GroupIDs;										// 0-9A-Z%

         list<string>Groups;									// The current list of Groups known to the form

         int GroupIndex, GroupCodeIndex;					// These are Combo DropDownList indexes

         int Players;
         
         int SelectedPlayer;									// Player selected at startup

      public:
   		MGroupsTube(const int selectedPlayer);

         String PlayerText(const int p);					// . . G.Family(7d) Club CC

         int IdForgeGroup(int& groupId, int& groupCode, String& Group);
         
         int CodeForgeGroup(int& groupId, int& groupCode, String& Group);
         
         bool ParseGroup(const String& player, int& groupId, int& groupCode);

         void Add(const String& Group);
         void Remove(const String& Group);
         bool Find(const String& Group);

   };
}
//---------------------------------------------------------------------------

namespace TourCloseRegMaster								// CLOSE REG
{
	class MCloseRegTube
   {
   	public:
      	String TourNameYear;

         int Action;

      public:
      	MCloseRegTube();
   };
}
//---------------------------------------------------------------------------

namespace TourJoinMaster									// JOINING DRAW
{
	class MJoinDrawTube
   {
   	public:
      	String TourNameYear;

         int PlayerCount;	// Total number of players

         int JoiningCount;	// Number of players joining

      public:
      	MJoinDrawTube();

         bool JoiningPlayer(const int p);			// Identify joining player

         String JoiningPlayerText(const int p);	// Given Family  Grade  1 2 3 _ 5
   };
}
//---------------------------------------------------------------------------

#endif
