////////////////////////////////////////////////////////////////////////////////
// MODULE NewForm:       New Tournament Dialogue        G.Kaniuk 05/04/02
//
#include <vcl.h>
#pragma hdrstop

#include "FileNewIDH.h"
#include "FileRES.h"

#include "ETour.h"
#include "GoDrawGUI.h"
#include "NewForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

//---------------------------------------------------------------------------
__fastcall SComboBox::SComboBox(TComponent* Owner)
:MComboBox(Owner, 176,16, 177, 21)
{
	Name     = String("TourBox");
   
   TabOrder = 3;

   NewForm = (TNewFrm*)Owner;

   KillOnChange = false;

   OnKeyPress = NewForm->OnKeyEnter;
}

void __fastcall SComboBox::OnClickList(TObject* Sender)
{
	NewForm->CancelRenamed(Sender);

   OnChange = 0;

   	Text = Items->Strings[ItemIndex];

      KillOnChange = true;

   OnChange = OnChangeEd;
}

void __fastcall SComboBox::OnChangeEd(TObject* Sender)
{
	if(KillOnChange)
   {
   	KillOnChange = false;

      return;
   }

	NewForm->CancelRenamed(Sender);

   MComboBox::OnChangeEd(Sender);
}

//---------------------------------------------------------------------------

bool __fastcall TNewFrm::ParseNameYear(void)
{
	using std::string;

	if(TourBox->Text.IsEmpty() || YearEd->Text.IsEmpty())
   {
   	MenuMaster::OpErr(StatusBar, IDM_ReqTourNameYear);

      return false;
   }

   string tourname = Tools::S_s(TourBox->Text);
   string year     = Tools::S_s(YearEd->Text);

   if(!TourMaster::parse_tourname(tourname))
   {
   	MenuMaster::OpErr(StatusBar, IDM_BadTourName);

      return false;
   }

   if(!TourMaster::parse_year(year))
   {
   	MenuMaster::OpErr(StatusBar, IDM_BadYear);

      return false;
   }

   NewTube.TourName = Tools::s_S(tourname);
   NewTube.Year     = Tools::s_S(year);

   return true;
}

//---------------------------------------------------------------------------
__fastcall TNewFrm::TNewFrm(TComponent* Owner, FileMaster::MNewTube& newtube)
: TForm(Owner), NewTube(newtube)
{
	TourBox = new SComboBox(this);

	HelpFile 				  = GoDraw::HelpFilePath(String("FileNew.hlp"));

   TourBox->HelpContext   = IDH_FNTournamentName;
   YearEd->HelpContext    = IDH_FNYear;
   SeedBtn->HelpContext   = IDH_FNSeedFrom;
   RenameBtn->HelpContext = IDH_FNRename;
   OkBtn->HelpContext     = IDH_FNOk;
   HelpBtn->HelpContext   = IDH_FNHelp;
   CancelBtn->HelpContext = IDH_FNCancel;

   Renamed = false;

	YearEd->Text = NewTube.Year;

   NewTube.TourList.StartCombo((TComboBox*)TourBox);

   TourBox->ItemIndex = 0;   Scrolled = false;

   MenuMaster::OpInfo(StatusBar, IDM_SelTourName);

   ToolsGUI::OperatorInfo(IDM_FileNewStart);							// V6.5.0
}
//---------------------------------------------------------------------------


void __fastcall TNewFrm::OkBtnClick(TObject *Sender)
{
	ModalResult = mrNone;

   if(!ParseNameYear())
   	return;

   if(!Renamed)
   	if(!NewTube.ForgeTourFileName())
      	return;

   if(FileExists(NewTube.TourFile))
   	if(ToolsGUI::WarnOverwrite(Tools::S_s(NewTube.TourFile))!=IDYES)
      	return;

   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::RenameBtnClick(TObject *Sender)
{
   if(!ParseNameYear()) return;

   if(!NewTube.ForgeTourFileName()) return;

   MenuMaster::ClearBar(StatusBar);

   SaveDlg->FileName   = NewTube.TourFile;
   SaveDlg->InitialDir = NewTube.ToursDir;

	if(SaveDlg->Execute())
   {
   	NewTube.TourFile = SaveDlg->FileName;

      Renamed = true;
   }

   RenameLbl->Caption = "...\\" + ExtractFileName(NewTube.TourFile);
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::SeedBtnClick(TObject *Sender)
{
	MenuMaster::ClearBar(StatusBar);

   if(ToolsGUI::OperatorWarning(IDM_FileNewSeed)) return;		// V6.5.0

   OpenDlg->FileName   = String();
   OpenDlg->InitialDir = NewTube.ToursDir;

	if(OpenDlg->Execute())
   {
   	NewTube.SeedFile = OpenDlg->FileName;

      Renamed = false;
   }
   else
   	NewTube.SeedFile = String();

   SeedLbl->Caption = "...\\" + ExtractFileName(NewTube.SeedFile);
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::HelpBtnClick(TObject *Sender)
{
	MenuMaster::ClearBar(StatusBar);

   try{ Application->HelpCommand(HELP_CONTEXT, IDH_FNFILENEW); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::FormClose(TObject *Sender, TCloseAction &Action)
{
	delete TourBox;

	try{ Application->HelpCommand(HELP_QUIT, 0); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::CancelRenamed(TObject *Sender)
{
	Renamed = false;

   RenameLbl->Caption = String();
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::CancelBtnClick(TObject *Sender)
{
	ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TNewFrm::OnKeyEnter(TObject *Sender, char &Key)
{
	MenuMaster::OpInfo(StatusBar, IDM_NewEnter);
}
//---------------------------------------------------------------------------

