////////////////////////////////////////////////////////////////////////////////
// MODULE  PairGUI:    Draw Pane Support                     G.Kaniuk 06/04/02
//
#include <string>
#include <list>

#include <vcl.h>
#include <ctype.h>

#pragma hdrstop

#include "GoInfoRES.h"
#include "GoDrawRES.h"
#include "GoErrMsgRES.h"
#include "PairRES.h"

#include "GraphicsAPI.h"
#include "Strings.h"
#include "ETour.h"
#include "PrefsMaster.h"
#include "TourMaster.h"
#include "PairPaneGUI.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

namespace PairGUI     // MAGIC CONSTANTS FOR 600 x 800 RESOLUTION SCREEN
{
	using ToolsGUI::OperatorError;
   using ToolsGUI::OperatorInfo;
   
   static const int defPairHeight = 241, defPairWidth = 259;
   static const int defPairTop    = 105, defPairLeft  =  14;

   static const int cwBoard = 24, cwName = 87, cwKomi = 42, cwNote = 100;

   static const int PairHeadID[pfFields] =
   {
      IDL_Board, IDL_Black, IDL_Komi, IDL_White, IDL_Comm
   };

   static const GGrid::ColSize ColSizes[pfFields] =
   {
      { cwBoard, 0 },
      { cwName,  1 },  { cwKomi,  0 },  { cwName,  1 },
      { cwNote,  1 }
   };
}
//---------------------------------------------------------------------------

namespace PairGUI          // ERROR MANAGEMENT
{
   void Throw(ToolsGUI::EType etype, short idproc, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_PairGUI, idproc, idmsg);
   }
}
//---------------------------------------------------------------------------

namespace PairGUI
{

   __fastcall GKomiEd::GKomiEd(GKomi* field):ToolsGUI::GEdit(field)
   {
      PairGrid = field->PairGrid;

      OnChange 	= NULL;
      Text 			= String();
      AutoSelect  = true;
		Name 			= LoadStr(IDL_GHCAP);

      MaxLength	= 7;
      Hint        = LoadStr(IDI_SampleKomi_H);
      ShowHint    = true;
      OnChange    = OnChangeEd;
   }

   void __fastcall GKomiEd::KeyPress(char& Key)
   {
   	// Suppress invalid non-ascii junk
      if(0x00 <= Key && Key <= 0x1F){ Beep(); return; } 				// Control
      if(Key >= 0x7F)               { Beep(); Key = 0; return; }	// Non-ascii
   }

   void __fastcall GKomiEd::OnChangeEd(TObject *Sender)
   {
   	using Tools::cPlus;
      using Tools::cMinus;

   	// Check all chars
      for(int k=1;k<=Text.Length(); k++)
      {
      	char Key = Text[k];

      	if(!isdigit(Key) && Key != cPlus && Key != cMinus)
         {
         	ToolsGUI::OperatorError(IDQ_BadKomiChar);

            SelStart  = k-1;
            SelLength = 1;

            return;
         }
      }
   }
}

namespace PairGUI                                  // CLASS GKomi
{
   using ToolsGUI::GGrid;
   using ToolsGUI::GField;

   __fastcall GKomi::GKomi(GPairGrid* pairgrid):GField(pairgrid)
   {
   	PairGrid = pairgrid;
      
      Editor = new GKomiEd(this);
   }

   __fastcall GKomi::~GKomi( )
   {
      delete Editor;
   }

   void __fastcall GKomi::FieldKeyDown(Word &Key, TShiftState Shift)
   {
      switch(Key)
      {
         case VK_TAB:
         {
         	if(PairGrid->Parse(VK_RETURN))
            {
            	Dismount();

            	PairGrid->SetFocus();
            }
            else
            	Key = 0;
         }
         break;

         default:
            ToolsGUI::GField::FieldKeyDown(Key, Shift);
            return;
      }
      Key = 0;
   }

   void __fastcall GKomi::Load()
   {
      ((TEdit*)Editor)->Text = CellText;
      ((TEdit*)Editor)->Modified = true;
   }

   void __fastcall GKomi::Store()
   {
      using std::string;
      using Tools::s_S;
      using Tools::S_s;

      Grid->Cells[ColM][RowM]
      =
      s_S(Tools::strip_white_space(S_s(((TEdit*)Editor)->Text)));
   }

   
   bool __fastcall GKomi::SameValue(void)
   {
      return ((TEdit*)Editor)->Text == CellText;
   }

   
   void __fastcall GKomi::ShowEd()
   {
      if(Editor)
      {
      	Editor->Color = PrefsMaster::PrefsTube.EditColor;
         Editor->Show();
      }
   }


   bool __fastcall GPairGrid::Parse(const Word Key)
   {
   	if(Key==VK_RETURN)
      {
         int Fail = PairPane->EditKomi(Fields[pfKomi]->Editor->Text);

         if(Fail == 0){ ShowInfo(IDQ_KomiEdited);       return true;  }

         if(Fail < 0 ){ OperatorError(IDQ_BadKomiChar); return false; }
         if(Fail > 0 ){ OperatorError(IDQ_KOMI+1+Fail); return false; }
      }
      return true;
   }
}
//---------------------------------------------------------------------------

namespace PairGUI                                  // CLASS GPairGrid
{
   using ToolsGUI::GGrid;

   using std::string;

    __fastcall GPairGrid::GPairGrid          // Startup constructor
   (
      TGroupBox* parent,
      TStatusBar*  statusbar
   )
   : GGrid((TWinControl*)parent, statusbar, (int)pfFields)
   {
         Top = defPairTop;      Left = defPairLeft;
      Height = defPairHeight;  Width = defPairWidth;

      hScrollBar  = true;

      Fields[pfKomi] = new GKomi(this);

      ColWidths[pfBoard] = cwBoard;
      ColWidths[pfBlack] = cwName;
      ColWidths[pfWhite] = cwName;

      for(int f=0; f<pfFields; f++)
         Cells[f][0] = LoadStr(PairHeadID[f]);

      SortEnabled = false;

      for(int f=0; f<pfFields; f++)
         SortDir[f] = ToolsGUI::gsNo;

      SortDir[pfBoard] = ToolsGUI::gsUp;
      
      PairKey = TourMaster::pkBoardUp;          // Ascending board order

      // DRAG AND CLICK IMPLEMENTATION
      tBlackRow = tBlackCol = tPairedRow = 0;   // Last paired players

      BlackRow  = &tBlackRow;                   // Re-assigned when
      BlackCol  = &tBlackCol;                   // Pairgrid is attached
      PairedRow = &tPairedRow;                  // PairPane

      DragKind     = dkDrag;
      DragMode     = dmManual;

      NormalStyle  = ControlStyle;
   }

   __fastcall GPairGrid::~GPairGrid()
   {
      delete (GKomi*)(Fields[pfKomi]);
   }

   void __fastcall GPairGrid::InvalidatePlayer(int col, int row)
   {
      InvalidateCell(col,row);
   }
   void __fastcall GPairGrid::InvalidateBoard(int board)
   {
      InvalidateRow(board);
   }

   void __fastcall GPairGrid::DrawHeading(int col, int row, const TRect &cell)
   {
   	using Tours::tsEditNow;
      using Tours::tsForcedPair;
      
   	using TourMaster::GoTour;

      DrawAlignedText(Canvas->Handle, ToolsGUI::jyCentre, cell, Cells[col][row]);

      if(GoTour.State==tsEditNow || GoTour.State==tsForcedPair)
      	return;
   }

   void __fastcall GPairGrid::DrawOutlinedCell(Windows::TRect cell, String name, TColor border)
   {
      using Windows::TRect;
      using ToolsGUI::jyLeft;
      using ToolsGUI::DrawAlignedText;


      TColor OldPenColor = Canvas->Pen->Color;
      int    OldPenWidth  = Canvas->Pen->Width;

      Canvas->Pen->Color = border;
      Canvas->Pen->Width = 2;

      Canvas->Rectangle(cell);

      cell.left += 3;   cell.right -= 2;
      cell.top  += 3;   cell.bottom -= 2;

      DrawAlignedText(Canvas->Handle, jyLeft, cell, name);

      Canvas->Pen->Color = OldPenColor;
      Canvas->Pen->Width = OldPenWidth;
   }

   void __fastcall GPairGrid::DrawCell
   (
      int col, int row,
      const Windows::TRect &cell,
      Grids::TGridDrawState state
   )
   {
      using Windows::TRect;

      using ToolsGUI::DrawAlignedText;

      using ToolsGUI::Justify;
      using ToolsGUI::jyCentre;
      using ToolsGUI::jyLeft;
      using ToolsGUI::jyRight;

      using PaneGUI::GClickPlayer;
      using PaneGUI::GPairPane;

      if(row==0)            								// Headings Centred
      {
         DrawHeading(col, row, cell);

         return;
      }

      TRect Rect = cell;

      Justify Posn;

      switch(col)
      {
         case pfBoard:                             // Board numbers Right
         {
            Rect.Right -= 4;   Posn = jyRight;
         }
         break;

         case pfKomi:
            Posn = jyCentre;                       // Komi Centred
         break;

         default:                                  // Names, comment Left
         {
            if(row>0)
            {
               if(row==*BlackRow && col==*BlackCol)
               {
                 DrawOutlinedCell(cell, Cells[col][row], clBlack);
                 return;
               }

               if(row==*PairedRow)
               {
                  DrawOutlinedCell(CellRect(pfBlack, row), Cells[pfBlack][row], clBlack);
                  DrawOutlinedCell(CellRect(pfWhite, row), Cells[pfWhite][row], clBlack);
                  return;
               }
            }

            Rect.Left += 2;  Rect.Right -= 2;
            Posn = jyLeft;
         }
      }

      DrawAlignedText(Canvas->Handle, Posn, Rect, Cells[col][row]);
   }
   //------------------------------------------------------------------------------

   void __fastcall GPairGrid::Click(void)
   {
   	ClearInfo();

      if(!Tag || !EditPossible())					 // Editing forbidden by State
      {                                          // Or already editing something else
      	ToolsGUI::ResetToEditCell();

      	ClearInfo();

         OperatorError(IDI_NoPairEd);

         return;
      }

      if(PairPane->EditState == PaneGUI::psEditOff)      // EDIT in Edit Mode only
      {
         OperatorError(IDI_NoEditMode);  return;
      }
   }


   void __fastcall GPairGrid::ClickPlayer(void)          // Click a cell
   {
   	ClearInfo();
      
   	if(Row==0)
      {
      	NoCellSelected();
         
      	return;
      }

      if(Col == pfNote)                                  // CANNOT Edit comments
      {
         OperatorError(IDI_NoEditNote);  return;
      }

      if(Col==pfKomi)                                    // CAN edit Komi if not editing pair
      if
      (
         PairPane->EditState == PaneGUI::psEditOn
                             ||
         PairPane->EditState == PaneGUI::psPaired
      )
      {
         ToolsGUI::GGrid::Click();   return;
      }
      else
      {
         OperatorError(IDI_KomiEdit); return;
      }

      GKomi* Komi = (GKomi*)Fields[pfKomi];              // CAN edit pair only if not editing Komi

      if(Komi->State != ToolsGUI::fsDormant)
      {
         ToolsGUI::GGrid::Click();                       // RESET to Komi field if changed

         if(!Komi->SameValue())
         {
            OperatorError(IDI_EditKomi);

            return;
         }
      }

      if(Col!=pfBlack && Col!=pfWhite)
      {
      	OperatorError(IDI_BlackWhite);  return;
      }

      if(Cells[Col][Row].IsEmpty())
      {
         OperatorError(IDI_NoEmpty);  return;
      }

      PairPane->ClickPlayers.PairEdit(PaneGUI::soGrid);
   }

   void __fastcall GPairGrid::KeyDown(Word &Key, TShiftState ShiftState)
   {
      using ToolsGUI::GGrid;

      using PaneGUI::GClickPlayer;
      using PaneGUI::GPairPane;

      if(Col<0 || Row<0) { Key = 0; return; }

      ClearInfo();

      if(SelectedFieldScrolled())
      {
         ExposeSelectedCell();   Key = 0;    return;
      }

      switch(Key)
      {
         case VK_RETURN:
         {
            KeyClick = false;    KeyPressed = 0;

            if(Col==pfKomi)
            	ClickPlayer();
            else
            	Click();
         }
         break;

         case VK_ESCAPE:
         {
            if(PairPane->EditState == PaneGUI::psSelect)
            {
               PairPane->ClickPlayers.CancelPairEdit();

               NoCellSelected(); PairPane->UpsetList->ItemIndex = -1;
               Repaint();

               PairPane->UpsetList->Repaint();

               ShowInfo(IDI_CancelEdit);
            }
         }
         break;

         default:  GGrid::KeyDown(Key, ShiftState); return;
      }
      Key = 0;
   }


   void __fastcall GPairGrid::RowChange(int Control)
   {
      OperatorError(IDI_EditKomi);
   }

   bool __fastcall GPairGrid::ColChange(int Control)
   {
      OperatorError(IDI_EditKomi);

      return false;
   }

   void __fastcall GPairGrid::GridChange(void)
   {
      OperatorError(IDI_FinishKomi);
   }

   void __fastcall GPairGrid::NoDownArrow(void)
   {
      ShowInfo(IDI_EndBoards);
   }

   void __fastcall GPairGrid::NotShowing(void)
   {
      Beep();	ShowInfo(IDI_HiddenKomi);
   }
}
//------------------------------------------------------------------------------

namespace PairGUI                                  // DRAG IMPLEMENTATION
{
   void __fastcall GPairGrid::DragStartE
   (
      TObject* Source,                             // This PairGrid
      TDragObject* &DragObject                     // GDragPlayer
   )
   {
      PairPane->DragPlayer->PhantomDrag = true;

      PairPane->DragPlayer->StartGrid = true;
      PairPane->DragPlayer->StartList = false;

      PairPane->DragPlayer->SoCol = -1;            // Others detect problems
      PairPane->DragPlayer->SoRow = -1;            // using negative value
      PairPane->iSource           = -1;
      PairPane->jSource           = -1;

      String Text    = String();                   // Cope with attempt to drag
      TRect  Area    = TRect(0,0,0,0);             // anything other than Black
      TCursor Cursor = crDefault;                  // or White player

      // IDENTIFY SELECTED CELL
      TPoint O = ClientOrigin;
      int iC=0, jC=0; POINT P;                     // POINT is a pair of long's and incompatible with TPoint!

      if(GetCursorPos(&P)){ iC = P.x - O.x;  jC = P.y - O.y; }

      if(iC>0 && jC>0)
      {
         int col, row;  MouseToCell(iC, jC, col, row);

         PairPane->iSource = iC;    PairPane->DragPlayer->SoCol = col;
         PairPane->jSource = jC;    PairPane->DragPlayer->SoRow = row;

         if(row > 0)
         {
            if(col==pfBlack || col==pfWhite)          // Can click or drag
            {
               Text = Cells[col][row];
               Area = CellRect(col,row);

               Cursor = crArrow;
            }
            else
               Cursor = crArrow;
         }
      }

      PairPane->DragPlayer->StartDragPlayer(Source, Cursor, Text, Area);

      DragObject = PairPane->DragPlayer;
   }


   void __fastcall GPairGrid::DragOverE
   (
      TObject* Sink,                               // This PairGrid
      TObject* Source,                             // Either Grid or List
      int i, int j,                                // Current cursor
      TDragState State,                            // dsDragEnter, Leave, Move
      bool &Accept                                 // Possible drop
   )
   {
      if(PairPane->DragPlayer->StartGrid)          // Drag from grid detected in DragStartE
      {
         if(!PairPane->HasDragged)
         {
            PairPane->HasDragged = PairPane->DragDetected(i,j);

            return;
         }

         if(PairPane->DragPlayer->PhantomDrag)
         {
            PairPane->DragPlayer->CursorNow = crArrow;

            return;
         }

         int col,row;  MouseToCell(i,j,col, row);

         if(col==pfBlack || col==pfWhite)
            PairPane->DragPlayer->CursorNow = crDrag;
         else
            PairPane->DragPlayer->CursorNow = crNoDrop;

         return;
      }

      if(PairPane->DragPlayer->StartList)                                   // Drag started on list
      {
         int col,row;  MouseToCell(i,j,col, row);

         if(col==pfBlack || col==pfWhite)
            PairPane->DragPlayer->CursorNow = crDrag;
         else
            PairPane->DragPlayer->CursorNow = crNoDrop;

         return;
      }

      PairPane->DragPlayer->CursorNow = crUpArrow;

      // We leave Accept true

      // TODO
      // OBSOLETE?
      // Start the scroll timer as appropritae if
      // it is not already running
   }

   void __fastcall GPairGrid::DragDropE
   (
      TObject* Sink,                               // This PairGrid
      TObject* Source,                             // Either Grid or List
      int i, int j                                 // Current cursor
   )
   {
      // PairPane->DragPlayer->Timer->Enabled = false;  OBSOLETE

      // GDragPlayer::Finish handles all drop processing
   }
}
// -----------------------------------------------------------------------------


namespace PairGUI                                  // ACTIONS
{
	void __fastcall GPairGrid::EscapeKomi(void)
   {
       Word EscKey = VK_ESCAPE;	TShiftState Shift;

       Fields[pfKomi]->FieldKeyDown(EscKey, Shift.Clear());
   }

 	void __fastcall GPairGrid::FocusKomi(void)
   {
   	GKomiEd* KomiEd = (GKomiEd*)Fields[pfKomi]->Editor;

      KomiEd->SelStart = KomiEd->Text.Length();

      KomiEd->SetFocus();
   }

   void __fastcall GPairGrid::ReBuild()
   {
      GGrid::ReBuild(ColSizes);
      
      sColBlack = ColWidths[pfBoard];
      eColBlack = sColBlack + ColWidths[pfBlack];
      sColWhite = eColBlack + ColWidths[pfKomi];
      eColWhite = sColWhite + ColWidths[pfWhite];

      if(TourMaster::GoTour.State != Tours::tsEditNow)
         NoCellSelected();
   }

   void __fastcall GPairGrid::LoadPairRow(int b)
   {
   	using Tools::s_S;
      using Tours::GBoard;
      using Tours::GPlayerPtr;
   	using TourMaster::GoTour;

   	GBoard& Board = GoTour.Pair[b];

      GPlayerPtr Black = GoTour.Listed[Board.Black];
      GPlayerPtr White = GoTour.Listed[Board.White];

      string sBlack, sWhite;

      switch(PrefsMaster::PrefsTube.ScreenNameStyle)
      {
         case PrefsMaster::dpSufMMS:
         {
            sBlack = GoTour.ShortNameMMS(Black);
            sWhite = GoTour.ShortNameMMS(White);
         }
         break;

         case PrefsMaster::dpSufGrade:
         {
            sBlack = GoTour.ShortNameGrade(Black);
            sWhite = GoTour.ShortNameGrade(White);
         }
         break;

         default:
         {
            sBlack = GoTour.ShortName(Black);
            sWhite = GoTour.ShortName(White);
         }
      }

      Cells[pfBoard][b] = String(b);
      Cells[pfBlack][b] = s_S(sBlack);
      Cells[pfWhite][b] = s_S(sWhite);
      Cells[pfKomi][b]  = s_S(GoTour.Komi(Board));
      Cells[pfNote][b]  = s_S(GoTour.GameNote(Board));
   }

   void __fastcall GPairGrid::LoadFromTour(void)
   {
      using Tools::s_S;
      using TourMaster::GoTour;
      using Tours::GPlayerPtr;

      size_t PairSize = GoTour.Pair.size();

      RowCount = PairSize>1 ? PairSize : 2;

      Cells[0][0] = Tools::sEmpty;
      Cells[0][1] = Tools::sEmpty;

      for(size_t f=1; f<pfFields; f++)
      {
         Cells[f][0]          = LoadStr(PairHeadID[f]);
         Cells[f][RowCount-1] =  String();

         Cells[f][1]          =  String();
      }

      for(size_t b=1; b<PairSize; b++)LoadPairRow(b);

      ReBuild();             // Board count changed

      TopRow = 1;

      NoCellSelected();
   }

   void __fastcall GPairGrid::LoadPair(int b)
   {
   	using TourMaster::GoTour;
      using Tools::s_S;

    	if(RowCount != (int)GoTour.Pair.size()) return;
   	if(b<1 || b >= RowCount) return;

      LoadPairRow(b);

      InvalidateRow(b);
   }

   void __fastcall GPairGrid::ClearPairing(void)
   {
      using TourMaster::GoTour;

      Cells[0][0] = String();

      for(size_t f=1; f<pfFields; f++)
         Cells[f][0] = LoadStr(PairHeadID[f]);

      for(int b=1; b<RowCount; b++)
      {
      	Cells[0][b] = String(b);

      	for(size_t f=1; f<pfFields; f++)
         	Cells[f][b] = String();
      }
   }

   void __fastcall GPairGrid::CommentKomi(int board, int black, int white)
   {
      using Tools::s_S;

      using Tours::GPlayerPtr;
      using Tours::GBoard;

      using TourMaster::GoTour;

      using DrawMaster::coCount;
      using DrawMaster::DrawEditInfo;
      using DrawMaster::ForgeKomi;

      GPlayerPtr Black = black ? GoTour.Listed[black] : NULL;
      GPlayerPtr White = white ? GoTour.Listed[white] : NULL;

      if(black && white)												// BOTH players on board
      {
      	// COMMENT ON DUFF PAIRINGS
      	{
      		bool Repeat, BadOrder;

         	DrawEditInfo(black, white, Repeat, BadOrder);

         	if( Repeat && !BadOrder)OperatorInfo(IDQ_RepeatGame);
         	if(!Repeat &&  BadOrder)OperatorInfo(IDQ_BadOrder);
            if( Repeat &&  BadOrder)OperatorInfo(IDQ_BadRepeatOrder);
            if(!Repeat && !BadOrder) ClearInfo();
         }

         Cells[pfKomi][board] = s_S(ForgeKomi(Black, White));
         Cells[pfNote][board] = s_S(GoTour.GameNote(GBoard(black,white)));
      }
      else																	// ONE or BOTH players missing
      {
      	Cells[pfKomi][board] = String();
         ClearInfo();
      }
   }
}
//---------------------------------------------------------------------------
















