////////////////////////////////////////////////////////////////////////////////
// MODULE  PairGUI    Draw Pane Support for both GPairGrid and GUpsetList                    G.Kaniuk 06/04/02
//
#include <grids.hpp>
#include <classes.hpp>

#include "Tour.h"
#include "GridGUI.h"

#ifndef PairGUIH
#define PairGUIH

namespace PaneGUI
{
   class GPairPane;
}

namespace PairGUI                         // PAIRGRID
{
	using Tours::GBoard;
   using ToolsGUI::GGrid;
   using ToolsGUI::GField;
   using ToolsGUI::GRatio;

   enum PairFields
   {
      pfBoard, pfBlack, pfKomi, pfWhite, pfNote, pfFields
   };

   class GPairGrid;
   class GKomi;
   
	class GKomiEd : public ToolsGUI::GEdit
   {
   	friend GKomi;
      friend GPairGrid;

      private:
         GPairGrid* PairGrid;

      private:
         DYNAMIC void __fastcall KeyPress(char& Key);
         virtual void __fastcall OnChangeEd(TObject *Sender);

      public:
         __fastcall GKomiEd(GKomi* Field);
   };

   class GKomi : public GField
   {
      friend GPairGrid;
      friend GKomiEd;
      
      private:
         GPairGrid* PairGrid;

      protected:
      	void __fastcall FieldKeyDown(Word &Key, TShiftState Shift);
         void __fastcall ShowEd(void);
         void __fastcall Load();
         void __fastcall Store();
         bool __fastcall SameValue();

      public:
         __fastcall GKomi(GPairGrid* pairgrid);
         __fastcall ~GKomi();
   };

   class GPairGrid : public GGrid
   {
   	friend GKomi;
      
      public:
         int tBlackRow, tBlackCol, tPairedRow;  // Used until PairGrid attached to PairPane
         int *BlackRow, *BlackCol, *PairedRow;  // Pair Click support

         int PairKey;                           // Column sort order

         PaneGUI::GPairPane* PairPane;          // Controller for Drag and Click

         TControlStyle NormalStyle;             // Extra reqd for editing

      private:
         int sColBlack, eColBlack;              // Locate Black column
         int sColWhite, eColWhite;              // Locate White column
         
         void __fastcall DrawHeading(int col, int row, const TRect &cell);
         
         void __fastcall DrawOutlinedCell(Windows::TRect cell, String name, TColor border);

         void __fastcall DrawCell
         (
            int col, int row,
            const Windows::TRect& rect,
            Grids::TGridDrawState state
         );

         DYNAMIC void __fastcall KeyDown(Word &Key, Classes::TShiftState Shift);

      private:
         void __fastcall RowChange(int Control);
         bool __fastcall ColChange(int Control);

         void __fastcall GridChange(void);

         void __fastcall NoDownArrow(void);
         void __fastcall NotShowing(void);

         bool __fastcall Parse(const Word Key);

      public:
         __fastcall GPairGrid(TGroupBox* parent, TStatusBar*  statusbar);

         __fastcall ~GPairGrid();

         DYNAMIC void __fastcall Click(void);
         
         void __fastcall DragStartE(TObject* Source, TDragObject* &DragObject);
         void __fastcall DragOverE(TObject* Sink, TObject* Source, int i, int j, TDragState State, bool &Accept);
         void __fastcall DragDropE(TObject* Sink, TObject* Source, int i, int j);

         void __fastcall InvalidatePlayer(int col, int row);
         void __fastcall InvalidateBoard(int board);

         void __fastcall ReBuild(void);
         void __fastcall ClickPlayer(void);
         void __fastcall LoadPairRow(int b);
         void __fastcall LoadFromTour(void);
         void __fastcall ClearPairing(void);
         void __fastcall EscapeKomi(void);

         void __fastcall FocusKomi(void);

         void __fastcall LoadPair(int b);

         void __fastcall CommentKomi(int board, int black, int white);
   };
}
//---------------------------------------------------------------------------
#endif
 