////////////////////////////////////////////////////////////////////////////////
// MODULE PairMaster: GTour pairing support 							G.Kaniuk 01/06/05
////////////////////////////////////////////////////////////////////////////////
#include <vector>

#include "Maths.h"
#include "Tour.h"

#ifndef PairMasterH
#define PairMasterH

////////////////////////////////////////////////////////////////////////////////
// PAIRING INTERFACE
//
namespace PairMaster
{
	struct PairCount		// Draw monitor
   {
   	int Uneven;			// Count uneven games
      int SameClub;		// Count same club games
      int SameLand;		// Count same country games

      PairCount()
      {
      	Uneven = SameClub = SameLand = -1;
      }

      void Clear()
      {
      	Uneven = SameClub = SameLand = -1;
      }
   };

   typedef std::vector<PairCount> PairCounts;

   void KomiStones
   (
   	const Tours::GPlayerPtr Black, const Tours::GPlayerPtr White,
      int& Komi, int& Stones
   );

	void PairPlayers(int one, int two);

   void ManualPairing (size_t b, size_t black, size_t white, int Stones, int Komi);
   void ManualHandicap(size_t black, size_t white, int Stones, int Komi);

   void UnpairPlayers(int one, int two);
   void UnpairPlayer(size_t one);

   bool PlayedBefore(const int One, const int Two, const int round);
}

////////////////////////////////////////////////////////////////////////////////
// PAIRING PARAMETERS
//
namespace PairMaster
{
	using Tools::ints;
   using Tools::real;
   
	enum
   {
   	size_monitor = 32,	// Number of rows of monitor data

   	def_trials = 3,
      def_parity = 1,
      def_area   = 2,
      def_colour = 3,
      def_alpha  = 1,
      def_beta   = 1,
      def_gamma  = 1
   };

   enum	// Pairing mode
   {
   	pmNormal, pmModel, pmTest
   };

   enum	// Results mode
   {
   	rmManual, rmSim
   };

	struct PairParams
   {                             	// ADVANCED
   	int Trials;							// The number of trials to improve pairing
      int Parity, Area, Colour;  	// The order of sorting
      real Alpha;                   // Uneven weight
      real Beta;                    // Area weight
      real GammaP, GammaM;          // Colour weight

      										// SETTINGS
      Byte PairMode;						// Normal, Training, Test
      Byte PartDraw;					   // Controls prob partial draw

      ints Monitor[3*size_monitor];	// MONITOR

      PairParams();

      PairParams(Byte* Stream);

      void PutStream(Byte* Stream);
      void GetStream(Byte* Stream);
   };

   void PutTrials(int trials, Byte* Stream);
   int GetTrials(Byte* Stream);

   void PutPairMode(Byte pairmode, Byte* Stream);
   Byte GetPairMode(Byte* Stream);

   void PutPartDraw(Byte partdraw, Byte* Stream);
   Byte GetPartDraw(Byte* Stream);

   void PutMonitor(PairCount& Model, PairCount& Actual, PairCounts& Trials, Byte* Stream);
   void GetMonitor(Byte* Stream, PairCount& Model, PairCount& Actual, PairCounts& Trials);
   void ClearMonitor(Byte* Stream);

	void SetDefaultParameters(Byte* DefPars);

   void ClearOpponents(void);
}
//
////////////////////////////////////////////////////////////////////////////////
#endif



