////////////////////////////////////////////////////////////////////////////////
// MODULE  PairPaneGUI    Panel containing Pair and Res grids G.Kaniuk 23/02/04
//
#include <memory>
#include <string>
#include <vcl.h>

#pragma hdrstop
#pragma package(smart_init)


#include "GoInfoRES.h"
#include "PairRES.h"
#include "ETour.h"
#include "Sort.h"
#include "PrefsMaster.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "DrawMaster.h"
#include "PairPaneGUI.h"

//------------------------------------------------------------------------------

namespace PaneGUI             // MAGIC CONSTANTS FOR PANE
{
	using ToolsGUI::OperatorError;
   using PairGUI::pfBlack;
   using PairGUI::pfWhite;
   using PairGUI::pfKomi;
   using PairGUI::pfNote;
   using PrefsMaster::PrefsTube;
}

namespace PaneGUI             // THE CLICK PLAYERS OBJECT
{
	using Tours::GBoard;

   __fastcall GClickPlayer::GClickPlayer()
   {
      PairPane = 0;

      PlayerName = String();

      BoardPaired = 0;
   }

   __fastcall GClickPlayer::~GClickPlayer()
   {
   }
   //--------------------------------------------------------


   void __fastcall GClickPlayer::ResetPairEdit(void)
   {
      BoardPaired = 0;

      FirstCol = FirstRow = 0;   FirstIndex = -1;

      FirstOnGrid = false;       FirstOnList = false;

      PairPane->EditState = psEditOn;
   }

   void __fastcall GClickPlayer::CancelPairEdit(void)
   {
      FirstCol = FirstRow = 0;   FirstIndex = -1;

      FirstOnGrid = false;       FirstOnList = false;

      PairPane->EditState = psEditOn;
   }

   void __fastcall GClickPlayer::PairEdit(int source)
   {
      if(PairPane->EditState == psEditOn ||PairPane->EditState == psPaired)
         ResetPairEdit();

      if(PairPane->EditState == psEditOn)
      {
         if(source==soGrid)
         {
            FirstOnGrid = true;
            FirstCol    = PairPane->PairGrid->Col;
            FirstRow    = PairPane->PairGrid->Row;
            PlayerName  = PairPane->PairGrid->Cells[FirstCol][FirstRow];

            PairPane->PairGrid->Repaint();
         }

         if(source==soList)
         {
            FirstOnList = true;
            FirstIndex  = PairPane->UpsetList->ItemIndex;
            PlayerName  = PairPane->UpsetList->Items->Strings[FirstIndex];
         }

         PairPane->EditState = psSelect;

         return;
      }

      if(PairPane->EditState == psSelect)                   // We now have the second click of a pair
      {
         if(source == soGrid)                               // The second click is on the Grid
         {
            if(FirstOnGrid)
            {
               GridGridPair(PairPane->PairGrid->Col, PairPane->PairGrid->Row);

               FirstCol = FirstRow = 0;                     // This prevents the old Black selection showing
            }

            if(FirstOnList)
               ListGridPair(PairPane->PairGrid->Col, PairPane->PairGrid->Row);
         }

         if(source == soList)                               // The second click is on the List
         {
            if(FirstOnGrid)
            {
               GridListPair(PairPane->UpsetList->ItemIndex);

               FirstCol = FirstRow = 0;
            }

            if(FirstOnList)
               ListListPair(PairPane->UpsetList->ItemIndex);
         }

         PairPane->EditState = psPaired;

         PairPane->PairGrid->NoCellSelected();   PairPane->PairGrid->Repaint();
         PairPane->UpsetList->ItemIndex = -1;    PairPane->UpsetList->Repaint();
      }

      const GBoard& Board = PairPane->Pairs[BoardPaired];
      PairPane->PairGrid->CommentKomi(BoardPaired, Board.Black, Board.White);
   }
   //---------------------------------------------------------------------------

   // BLACK SELECTED ON GRID, WHITE SELECTED ON GRID
   void __fastcall GClickPlayer::GridGridPair(int Col, int Row)
   {
      if(FirstCol==Col && FirstRow==Row)                  // [FirstCol,FirstRow] is Black
      {
         ResetPairEdit();

         OperatorError(IDI_NoSelfPair);  return;
      }

      int WhiteNumber = PairPane->GetPlayerNumber(Col,Row), UpsetNumber;

      String WhiteName = PairPane->PairGrid->Cells[Col][Row], UpsetName;

      if(FirstCol == pfBlack)                              // PlayerName = Cells[pfBlack][FirstRow]
      {
         BoardPaired = FirstRow;

         if(Col == pfWhite && Row == FirstRow)             // No Change in Pair
         {
            PairPane->EditState = psPaired;   return;
         }

         UpsetNumber = PairPane->GetPlayerNumber(pfWhite,FirstRow);  // Old White upset
         UpsetName   = PairPane->PairGrid->Cells[pfWhite][FirstRow];

         PairPane->PairGrid->Cells[pfWhite][FirstRow] = WhiteName;   // New White
         PairPane->SetPlayerNumber(pfWhite, FirstRow, WhiteNumber);

         PairPane->PairGrid->Cells[Col][Row] = String();             // Clear White selection
         PairPane->SetPlayerNumber(Col, Row, 0);
      }
      else                                                  // PlayerName = Cells[pfWhite][FirstRow]
      {
         int BlackNumber;

         BoardPaired = FirstRow;

         UpsetNumber = PairPane->GetPlayerNumber(pfBlack,FirstRow);  // Old Black upset
         UpsetName   = PairPane->PairGrid->Cells[pfBlack][FirstRow];

         PairPane->PairGrid->Cells[pfBlack][FirstRow] = PlayerName;   // New Black
         BlackNumber = PairPane->GetPlayerNumber(pfWhite, FirstRow);
         PairPane->SetPlayerNumber(pfBlack, FirstRow, BlackNumber);

         PairPane->PairGrid->Cells[pfWhite][FirstRow] = WhiteName;    // New White
         PairPane->SetPlayerNumber(pfWhite, FirstRow, WhiteNumber);

         if(Row != FirstRow)                                // White selected else where
         {
            PairPane->PairGrid->Cells[Col][Row] = String();
            PairPane->SetPlayerNumber(Col,Row,0);
         }
         else                                                     // Just a swop
            UpsetNumber = 0;
      }

      if(UpsetNumber > 0)
         PairPane->UpsetList->AddPlayer(UpsetName, UpsetNumber);

      PairPane->EditState = psPaired;   // Edit Completed
   }
   //---------------------------------------------------------------------------


   // BLACK SELECTED ON LIST, WHITE SELECTED ON GRID
   void __fastcall GClickPlayer::ListGridPair(int Col, int Row)
   {
      using PairGUI::pfBlack;
      using PairGUI::pfWhite;

      String UpsetName;  int UpsetNumber;

      BoardPaired = Row;

      if(Col==pfBlack)                                      // Black will move to White
      {
         UpsetName = PairPane->PairGrid->Cells[pfWhite][Row];         // White is upset
         UpsetNumber = PairPane->GetPlayerNumber(pfWhite, Row);

         PairPane->PairGrid->Cells[pfWhite][Row] =  PairPane->PairGrid->Cells[pfBlack][Row];
         PairPane->SetPlayerNumber(pfWhite, Row, PairPane->GetPlayerNumber(pfBlack, Row));
      }
      else                                                  // White stays fixed
      {
         UpsetName = PairPane->PairGrid->Cells[pfBlack][Row];         // Black is upset
         UpsetNumber = PairPane->GetPlayerNumber(pfBlack, Row);
      }

      PairPane->PairGrid->Cells[pfBlack][Row] = PlayerName;
      PairPane->SetPlayerNumber(pfBlack, Row, PairPane->Upsets[FirstIndex]);

      if(UpsetNumber>0)
         PairPane->UpsetList->ReplacePlayer(FirstIndex, UpsetName, UpsetNumber);
      else
         PairPane->UpsetList->RemovePlayer(FirstIndex);

      PairPane->EditState = psPaired;
   }
   //---------------------------------------------------------------------------

   // BLACK SELECTED ON GRID, WHITE SELECTED ON LIST
   void __fastcall GClickPlayer::GridListPair(int Index)
   {
      using PairGUI::pfBlack;
      using PairGUI::pfWhite;

      String UpsetName;   int UpsetNumber;

      BoardPaired = FirstRow;

      if(FirstCol==pfBlack)              // Black stays fixed, White is upset
      {
         UpsetName = PairPane->PairGrid->Cells[pfWhite][FirstRow];
         UpsetNumber = PairPane->GetPlayerNumber(pfWhite,FirstRow);
      }
      else                                // Black is Upset, White moves to Black
      {
         UpsetName = PairPane->PairGrid->Cells[pfBlack][FirstRow];
         UpsetNumber = PairPane->GetPlayerNumber(pfBlack,FirstRow);

         PairPane->PairGrid->Cells[pfBlack][FirstRow] = PlayerName;

         PairPane->SetPlayerNumber(pfBlack, FirstRow, PairPane->GetPlayerNumber(pfWhite, FirstRow));
      }

      PairPane->PairGrid->Cells[pfWhite][FirstRow] = PairPane->UpsetList->Items->Strings[Index];
      PairPane->SetPlayerNumber(pfWhite,FirstRow, PairPane->Upsets[Index]);

      if(UpsetNumber>0)
         PairPane->UpsetList->ReplacePlayer(Index, UpsetName, UpsetNumber);
      else
         PairPane->UpsetList->RemovePlayer(Index);

      PairPane->EditState = psPaired;
   }
   //---------------------------------------------------------------------------

   // BLACK SELECTED ON LIST, WHITE SELECTED ON LIST
   void __fastcall GClickPlayer::ListListPair(int Index)
   {
      using PairGUI::pfBlack;
      using PairGUI::pfWhite;
      using PairGUI::pfBoard;

      BoardPaired = 0;

      String UpsetName;  int UpsetNumber;

      // AVOID CLICK ON SAME PLAYER
      if(FirstIndex == Index)
      {
         OperatorError(IDI_NoSelfPair);

         ResetPairEdit();

         return;
      }

      // SEARCH FOR A FREE SEAT
      for(size_t b=1; b<PairPane->Pairs.size(); b++)
      {
         if(!PairPane->Pairs[b].Black)                 // Black Free
         {
            UpsetName =  PairPane->PairGrid->Cells[pfWhite][b];
            UpsetNumber = PairPane->GetPlayerNumber(pfWhite,b);

            PairPane->PairGrid->Cells[pfBlack][b] = PlayerName;
            PairPane->SetPlayerNumber(pfBlack, b, PairPane->Upsets[FirstIndex]);

            PairPane->PairGrid->Cells[pfWhite][b] = PairPane->UpsetList->Items->Strings[Index];
            PairPane->SetPlayerNumber(pfWhite,b, PairPane->Upsets[Index]);

            BoardPaired = b;

            break;
         }

         if(!PairPane->Pairs[b].White)       			// White Free
         {
            UpsetName =  PairPane->PairGrid->Cells[pfBlack][b];
            UpsetNumber = PairPane->GetPlayerNumber(pfBlack,b);

            PairPane->PairGrid->Cells[pfBlack][b] = PlayerName;
            PairPane->SetPlayerNumber(pfBlack, b, PairPane->Upsets[FirstIndex]);

            PairPane->PairGrid->Cells[pfWhite][b] = PairPane->UpsetList->Items->Strings[Index];
            PairPane->SetPlayerNumber(pfWhite,b, PairPane->Upsets[Index]);

            BoardPaired = b;

            break;
         }
      }

      // REMOVE THE CLICKED PAIR FROM THE UPSET LIST
      if(Index>FirstIndex)
      {
         PairPane->UpsetList->RemovePlayer(Index);   PairPane->UpsetList->RemovePlayer(FirstIndex);
      }
      else
      {
         PairPane->UpsetList->RemovePlayer(FirstIndex);   PairPane->UpsetList->RemovePlayer(Index);
      }

      // ADD ANY NEW UPSET PLAYER
      if(UpsetNumber>0)
         PairPane->UpsetList->AddPlayer(UpsetName, UpsetNumber);

      // WE NOW SCROLL THE PAIRING IF LATEST BOARD OUT OF VIEW
      const int Stride = PairPane->PairGrid->VisibleRowCount;

      if(BoardPaired > Stride)
      	PairPane->PairGrid->TopRow = BoardPaired-Stride + 1;

      // ADD A BOARD NUMBER AS THESE ARE CLEAR DURING MANUAL PAIRING
      PairPane->PairGrid->Cells[pfBoard][BoardPaired] = String(BoardPaired);
       
      PairPane->EditState = psPaired;
   }
}
//------------------------------------------------------------------------------

namespace PaneGUI                         // DRAG PLAYER - PUBLIC INTERFACE
{
   __fastcall GDragPlayer::GDragPlayer()
   {
      StartList = StartGrid = false;

      PairPane = NULL;
      ImageList = NULL;                   // Owned by form, set each edit

      BitName = new Graphics::TBitmap();  // We change the size and drawn text
      BitName->Transparent = false;       // at each drag

      SoCol = SoRow = SoIndex = 0;

      PlayerName   = String();
      CursorNow    = crDefault;
      ImageShowing = false;
   }

   __fastcall GDragPlayer::~GDragPlayer()
   {
      delete BitName;
   }

   // ROUTINE StartDragPlayer called by OnStartDrag to initialise the object
   //
   void __fastcall GDragPlayer::StartDragPlayer
   (
      TObject* Source,        // Source of drag is either PairGrid or UpsetList
      TCursor Cursor,         // Use this cursor while dragging
      String& Text,           // A non-empty string if image to be shown
      TRect&  Area            // The rectangle enclosing the text
   )
   {
      PairPane->iDelta = (GetSystemMetrics(SM_CXDRAG)+1)/2;     // MS help claims the drag rectangle
      PairPane->jDelta = (GetSystemMetrics(SM_CYDRAG)+1)/2;     // is centred on the drag point

      // WE WANT SCREEN CO-ORDINATES FOR THE HOT AREAS
      const TPoint PaneO = PairPane->PairGrid->Parent->ClientOrigin;
      const TPoint GridO = PairPane->PairGrid->ClientOrigin;
      const TPoint ListO = PairPane->UpsetList->ClientOrigin;

      jT = PaneO.y;
      jU = GridO.y;
      jL = GridO.y + PairPane->PairGrid->ClientHeight;
      jB = ListO.y;
      jE = ListO.y + PairPane->UpsetList->Height;

      iL = PaneO.x;
      iS = GridO.x;
      iE = GridO.x + PairPane->PairGrid->ClientWidth;
      iR = PaneO.x + PairPane->PairGrid->Parent->Width;

      CursorNow = Cursor;

      PairPane->HasDragged = false;

      Timer->Enabled = false;

      // GENERATE THE IMAGE IF REQUIRED
      if(Text.IsEmpty())
      {
         PlayerName   = String();
         ImageShowing = false;
      }
      else
      {
         GPairGrid*  SoGrid = dynamic_cast<GPairGrid*>(Source);
         GUpsetList* SoList = dynamic_cast<GUpsetList*>(Source);

         TCanvas* Canvas = NULL;

         if(SoGrid) Canvas = SoGrid->Canvas;
         if(SoList) Canvas = SoList->Canvas;

         PlayerName = Text;

         PhantomDrag = Text.IsEmpty();

         if(Canvas != NULL)
            NewImage(Area, Canvas);
         else
            ImageShowing = false;
      }
      PairPane->PairGrid->ClearInfo();
   }
}
//------------------------------------------------------------------------------


namespace PaneGUI                         // DRAG PLAYER - PRIVATE TOOLS
{
   enum
   {
      ColScrollTime  = 300,
      PageScrollTime = 500
   };

   void __fastcall GDragPlayer::ShowDragImage(void)
   {
      ImageList->ShowDragImage();
   }

   void __fastcall GDragPlayer::HideDragImage(void)
   {
      ImageList->HideDragImage();
   }

   TDragImageList* __fastcall GDragPlayer::GetDragImages(void)
   {
      return ImageShowing ? ImageList : NULL;
   }


   // ROUTINE NewImage called by StartDragPlayer to convert player name
   // to a bitmap image in the ImageList
   //
   void __fastcall GDragPlayer::NewImage
   (
      TRect Area,      // Cell or Item area

      TCanvas* Canvas  // Supplied by Grid or List
   )
   {
      if(PlayerName.IsEmpty())
      {
         ImageShowing = false;  return;
      }

      if(PairPane->iSource<0 || PairPane->jSource<0)
      {
         ImageShowing = false;  return;
      }

      TSize NameArea =  Canvas->TextExtent(PlayerName);

      BitName->Width  = NameArea.cx+2;
      BitName->Height = NameArea.cy+2;

      BitName->Canvas->Brush->Color =  clWhite;
      BitName->Canvas->TextOut(1,1, PlayerName);

      ImageList->Clear();
      ImageList->Width  = BitName->Width;             // This is supposed to clear
      ImageList->Height = BitName->Height;            // the image list - but it doesn't always!

      ImageShowing = false;                           // Detect image creation

      int iO,jO;                                      // Hot spot corrections

      if(StartGrid)
      {
         iO = -1;    jO = -1;
      }

      if(StartList)
      {
         iO = 1;     jO = 1;
      }

      if(ImageList->Add(BitName, NULL)==0)            // Only one image with index 0
      {
         int iC = PairPane->iSource - Area.left + iO; // Set hotspot so image
         int jC = PairPane->jSource - Area.top + jO;  // overlays original text

         ImageShowing = ImageList->SetDragImage(0, iC,jC);
      }
   }

   // ROUTINE ScrollTimer event handler for scrolling
   //
   void __fastcall GDragPlayer::ScrollTimer(TObject* Sender)
   {
      if(HotT){ PairPane->PairGrid->NextGridPage(-1); return; }
      if(HotB){ PairPane->PairGrid->NextGridPage( 1); return; }

      if(HotL){ PairPane->PairGrid->NextGridCol(-1);  return; }
      if(HotR){ PairPane->PairGrid->NextGridCol( 1);  return; }
   }

   // ROUTINE GetDragCursor called whenever the mouse moves
   //
   TCursor  __fastcall GDragPlayer::GetDragCursor(bool accepted, int i, int j)
   {
      if(PhantomDrag)return CursorNow;

      // CANNOT DROP OUTSIDE GRID OR LIST
      TCursor Cursor = CursorNow;

      const bool iPane = iL <= i && i <= iR;		// Inside pane horizontally
      const bool iGrid = iS <= i && i <= iE;    // Inside grid horizontally

      const bool jGrid = jU <= j && j <= jL;		// Inside grid vertically
      const bool jList = jB <= j && j <= jE;		// Inside list vertically

      const bool InGrid = iGrid && jGrid;			// Grid interior
      const bool InList = iGrid && jList;			// List interior

      // DO NOT SCROLL IN GRID OR LIST
      if(InGrid || InList)
      {
         Timer->Enabled = false;

         return Cursor;
      }

      // ALREADY SCROLLING OUTSIDE GRID OR LIST
      if(Timer->Enabled)
      	return crNo;

      // DETERMINE IF IN HOT AREA
      HotT = iPane && jT <= j && j <= jU;
      HotB = iPane && jL <= j && j <= jB;

      HotL = jGrid && iL <= i && i <= iS;
      HotR = jGrid && iE <= i && i <= iR;

      while(true)
      {
         if(HotL || HotR)
            { Timer->Enabled = true; Timer->Interval = ColScrollTime;   break; }

         if(HotT || HotB)
            { Timer->Enabled = true; Timer->Interval = PageScrollTime;  break; }

         break;
      }

      return crNo;
   }

   // ROUTINE Finished called at mouse up, at which point we finally
   // know whether we have had a drag or a click
   //
   void __fastcall GDragPlayer::Finished(TObject* Target, int i, int j, bool accepted)
   {
      Timer->Enabled = false;

      if(!PairPane->HasDragged)                    // PROCESS THE CLICK
      {
         if(StartGrid)
         {
            PairPane->PairGrid->SelectGridCell(SoCol,SoRow);
            PairPane->PairGrid->ClickPlayer();
         }

         if(StartList)
         {
            PairPane->UpsetList->ItemIndex = SoIndex;
            PairPane->UpsetList->ClickPlayer();
         }
      }
      else                                         // PROCESS THE DRAG
      {
      	if(PairPane->EditState==psPaired)
         {
         	PairPane->ClickPlayers.BoardPaired=0;

            PairPane->PairGrid->Invalidate();
         }

         DragPlayer(Target, i, j);
      }

      StartList = StartGrid = false;
   }
   //---------------------------------------------------------------------------

   void __fastcall GDragPlayer::DragPlayer(TObject* Sink, int i, int j)
   {
      if(i<=0 || j<=0) return;
      if(StartList && SoIndex<0) return;
      if(StartGrid && (SoCol<0 || SoRow<0)) return;

      if(PlayerName.IsEmpty()) return;

      GPairGrid*  SiGrid = dynamic_cast<GPairGrid*>(Sink);
      GUpsetList* SiList = dynamic_cast<GUpsetList*>(Sink);

      if(SiGrid == PairPane->PairGrid)    // DROP ON GRID TO A BLACK OR WHITE CELL
      {
         int col, row;  PairPane->PairGrid->MouseToCell(i, j, col, row);

         if(row<1 || (col != PaneGUI::pfBlack && col != PaneGUI::pfWhite)) return;

         int UpsetNumber  = PairPane->GetPlayerNumber(col, row);

         String UpsetName =  PairPane->PairGrid->Cells[col][row];

         if(StartGrid)
            GridGridDrop(col,row, UpsetNumber, UpsetName);

         if(StartList)
            ListGridDrop(col, row, UpsetNumber, UpsetName);

         return;
      }

      if(SiList == PairPane->UpsetList)   // DROP ON LIST ANYWHERE
      {
         if(StartGrid)
            GridListDrop();

         if(StartList)
            ListListDrop();

         return;
      }
   }
   //---------------------------------------------------------------------------

   void __fastcall GDragPlayer::GridGridDrop
   (
      int ColDrop, int RowDrop,

      int UpsetNumber, String UpsetName
   )
   {
      // NO MOVEMENT
      if(SoCol==ColDrop && SoRow==RowDrop) return;

      // DROP THE PLAYER
      PairPane->PairGrid->Cells[ColDrop][RowDrop] = PlayerName;
      PairPane->PairGrid->Cells[pfKomi][RowDrop] = String();
      PairPane->PairGrid->Cells[pfNote][RowDrop] = String();

      PairPane->PairGrid->Cells[SoCol][SoRow]  = String();
      PairPane->PairGrid->Cells[pfKomi][SoRow] = String();
      PairPane->PairGrid->Cells[pfNote][SoRow] = String();

      // UPDATE Pairs
      PairPane->SetPlayerNumber
      (
         ColDrop, RowDrop, PairPane->GetPlayerNumber(SoCol, SoRow)
      );
      PairPane->SetPlayerNumber(SoCol, SoRow, 0);

      // UPDATE UpsetList AND Upsets only when we drop onto non-empty cell
      if(PairPane->UpsetList && UpsetNumber>0)
      {
         PairPane->UpsetList->Items->Add(UpsetName);   // Upset name to back of list

         PairPane->Upsets[PairPane->UpsetList->Items->Count-1] = UpsetNumber;
      }

      const GBoard& Board = PairPane->Pairs[RowDrop];
      PairPane->PairGrid->CommentKomi(RowDrop, Board.Black, Board.White);
   }
   //---------------------------------------------------------------------------


   void __fastcall GDragPlayer::ListGridDrop
   (
      int ColDrop, int RowDrop,

      int UpsetNumber,  String UpsetName
   )
   {
      // DROP THE PLAYER
      PairPane->PairGrid->Cells[ColDrop][RowDrop] = PlayerName;
      PairPane->PairGrid->Cells[pfKomi][RowDrop] = String();
      PairPane->PairGrid->Cells[pfNote][RowDrop] = String();

      // UPDATE PAIRS
      PairPane->SetPlayerNumber(ColDrop, RowDrop, PairPane->Upsets[SoIndex]);

      if(UpsetNumber>0) // LIST SOURCE REPLACED BY NEW                                           // We can re-use the Item
      {                                                           // which was dropped onto grid
         PairPane->UpsetList->Items->Strings[SoIndex] = UpsetName;
         PairPane->Upsets[SoIndex] = UpsetNumber;
      }
      else              // LIST SOURCE DELETED                                                        // We dropped onto empty cell
      {
         PairPane->UpsetList->Items->BeginUpdate();
                                                                  // So Contract UpsetList
         int ItemEnd = PairPane->UpsetList->Items->Count-1;

         if(SoIndex != ItemEnd)                               // Move end to middle
         {
            String EndName = PairPane->UpsetList->Items->Strings[ItemEnd];

            PairPane->UpsetList->Items->Strings[SoIndex] = EndName;

            PairPane->Upsets[SoIndex] = PairPane->Upsets[ItemEnd];
         }

         PairPane->UpsetList->Items->Delete(ItemEnd);

         PairPane->UpsetList->Items->EndUpdate();
      }

      const GBoard& Board = PairPane->Pairs[RowDrop];
      PairPane->PairGrid->CommentKomi(RowDrop, Board.Black, Board.White);
   }
   //---------------------------------------------------------------------------

   void __fastcall GDragPlayer::GridListDrop(void)
   {
      // THE SOURCE PLAYER IS UPSET
      int UpsetNumber = PairPane->GetPlayerNumber(SoCol, SoRow);

      // UPDATE PAIRS
      PairPane->PairGrid->Cells[SoCol][SoRow]  = String();
      PairPane->PairGrid->Cells[pfKomi][SoRow] = String();
      PairPane->PairGrid->Cells[pfNote][SoRow] = String();

      PairPane->SetPlayerNumber(SoCol, SoRow, 0);

      // ADD UPSET PLAYER TO BACK OF UPSET LIST
      PairPane->UpsetList->Items->Add(PlayerName);
      PairPane->Upsets[PairPane->UpsetList->Items->Count-1] = UpsetNumber;
   }

   void __fastcall GDragPlayer::ListListDrop(void)
   {
   }
}
//------------------------------------------------------------------------------

namespace PaneGUI             // INTERNAL TOOLS
{
   int __fastcall GPairPane::GetPlayerNumber(int col, int row)
   {
   	switch(col)
      {
      	case PairGUI::pfBlack: 	return Pairs[row].Black;
         case PairGUI::pfWhite: 	return Pairs[row].White;

      	default: 					return 0;
      }
   }

   void __fastcall GPairPane::SetPlayerNumber(int col, int row, int number)
   {
   	if(number<0) return;

   	switch(col)
      {
      	case PairGUI::pfBlack:	Pairs[row].Black = number; break;
         case PairGUI::pfWhite: 	Pairs[row].White = number; break;

      	default: return;
      }
   }

   // ROUTINE DragDetected more reliable than API::DragDetect
   // It is called by OnDragOver
   //
   bool __fastcall GPairPane::DragDetected(int i, int j)
   {
      if(iSource<0 || jSource<0) return false;

      bool detecting = true;

      while(detecting)
      {
         j -= jSource;  if(j < -jDelta) break;  if(j > jDelta) break;
         i -= iSource;  if(i < -iDelta) break;  if(i > iDelta) break;

         return false;
      }

      if(EditState == PaneGUI::psSelect)
      {
         ClickPlayers.CancelPairEdit();

         PairGrid->NoCellSelected();   UpsetList->ItemIndex = -1;
         PairGrid->Repaint();          UpsetList->Repaint();

         PairGrid->ShowInfo(IDI_CancelEdit);
      }

      return true;
   }

}
//---------------------------------------------------------------------------

namespace PaneGUI             // MAIN INTERFACE
{
   __fastcall GPairPane::GPairPane
   (
      PairGUI::GPairGrid* pairgrid,
      ResGUI::GResGrid* resgrid,
      UpsetGUI::GUpsetList* upsetlist,
      TImageList* imagelist,
      TTimer* timer
   )
   {
      EditState = psEditOff;

      DragPlayer            = new GDragPlayer();
      DragPlayer->PairPane  = this;

      DragPlayer->ImageList = imagelist;
      DragPlayer->Timer     = timer;

      DragPlayer->Timer->OnTimer = DragPlayer->ScrollTimer;


      ClickPlayers.PairPane = this;

      PairGrid = pairgrid;    PairGrid->PairPane   = this;
      ResGrid  = resgrid;
      UpsetList = upsetlist;  UpsetList->PairPane  = this;

      PairGrid->BlackCol  = &ClickPlayers.FirstCol;
      PairGrid->BlackRow  = &ClickPlayers.FirstRow;
      PairGrid->PairedRow = &ClickPlayers.BoardPaired;

      Pairs.resize(0);
   }

   __fastcall GPairPane::~GPairPane()
   {
      delete DragPlayer;
   }

   void __fastcall GPairPane::AttachHandlers()
   {
      // THESE HANDLERS ARE ATTACHED ONLY DURING EDITING
      PairGrid->OnStartDrag  = PairGrid->DragStartE;
      PairGrid->OnDragOver   = PairGrid->DragOverE;
      PairGrid->OnDragDrop   = PairGrid->DragDropE;

      PairGrid->ControlStyle = PairGrid->NormalStyle<<csDisplayDragImage;
      PairGrid->DragMode     = dmAutomatic;
   }

   void __fastcall GPairPane::DetachHandlers()
   {
   	// DETACH HANDLERS AT END OF EDIT
      PairGrid->ControlStyle = PairGrid->NormalStyle;
      PairGrid->DragMode     = dmManual;

      PairGrid->OnStartDrag = NULL;
      PairGrid->OnDragOver  = NULL;
      PairGrid->OnDragDrop  = NULL;
   }

   void __fastcall GPairPane::ReBuild(void)
   {
   	PairGrid->ReBuild();		ResGrid->ReBuild();

      if(EditState > psEditOff)
      	UpsetList->Mount(ResGrid);
   }
}
//--------------------------------------------------------------------------

namespace PaneGUI             // PANEL BUTTON INTERFACE
{
	int __fastcall GPairPane::EditKomi(String komitext)
   {
   	using std::string;
      using Tools::S_s;
      using Tools::trim;

      int Stones, Komi;

      string komi = trim(S_s(komitext));

      return DrawMaster::ParseKomi(komi, Stones,Komi);
   }


   bool __fastcall GPairPane::StartEdit(void)		// DRAW EDIT
   {
   	using std::string;
      using Tools::S_s;

      using PairGUI::pfKomi;

      using TourMaster::GoTour;
      using DrawMaster::GHandicap;

   	// ALLOW DRAGGING
      AttachHandlers();

      // THE PAIRING REPRESENTATION
      int Boards = GoTour.Pair.size() - 1;         // Original Number of boards in use

      ClickPlayers.ResetPairEdit();                // Editing now allowed

      try														// May not cope with very large Players
      {
      	if(Boards<GoTour.PlayNowCount/2)				// We know that PlayNowCount is even
         	Boards = GoTour.PlayNowCount/2;

	      Pairs.resize(Boards+1);                   // Pairs represents current pairing
         														// Pairs[0] not used

	      Upsets.resize(2*Boards);            		// Contains id's of upset players

         OldKomis.clear();									// Boards with komi are added to list
		}
      catch(...)
      {
      	return false;
      }

      // RESULTS GRID IS REPLACED BY UPSET LIST
      ResGrid->Hide();

      UpsetList->Clear();
         UpsetList->Mount(ResGrid);

      // REMOVE Leaving PLAYERS FROM PAIRS AND ADD Joining TO UPSET LIST
      size_t Round = GoTour.Round;	if(Round<1 || Round>GoTour.Rules.Rounds)
      										return false;

      StartPairs(Round);	StartUpsets(Round);

      // DISPLAY ADJUSTED GRID
      PairGrid->ReBuild();
      PairGrid->SelectGridCell(1, 1);
      PairGrid->ExposeSelectedCell();
      PairGrid->Invalidate();

      PairGrid->SetFocus();

      return true;
   }

   bool __fastcall GPairPane::SaveEdit(void)
   {
   	using std::string;

      using Tools::S_s;
      using PairGUI::pfKomi;

      if(!PairGrid || !ResGrid || !UpsetList) return false;

      // THERE MUST BE NO UPSET PLAYERS
      if(UpsetList->Items->Count > 0) return false;

      // ENSURE ALL BOARDS HAVE EITHER TWO PLAYERS OR NONE
      for(size_t b = 1; b<Pairs.size(); b++)
      {
      	Tours::GBoard& Board = Pairs[b];

         if(Board.Black && !Board.White)return false;
         if(Board.White && !Board.Black)return false;
      }

      // HARVEST NEW KOMIS
      NewKomis.clear();

      for(size_t b = 1; b<Pairs.size(); b++)
      {
         string Komi = S_s(PairGrid->Cells[pfKomi][b]);

   		if(!Komi.empty())												// Gather NewKomi
         	NewKomis.push_back(GHandicap(b,Komi));
      }

      DrawMaster::ChangeDraw(Pairs, OldKomis, NewKomis);

      DrawMaster::LayoutBoards();									// Set up for results

      OldKomis.clear();   NewKomis.clear();

		return CancelEdit();
   }


   bool __fastcall GPairPane::CancelEdit(void)
   {
      if(!PairGrid || !ResGrid || !UpsetList) return false;

      DetachHandlers();

      UpsetList->Hide();

      PairGrid->EscapeKomi();

      EditState = psEditOff;

      ClickPlayers.FirstCol = ClickPlayers.FirstRow = ClickPlayers.BoardPaired = 0;

      Pairs.resize(0);	Upsets.resize(0);

      OldKomis.clear(); NewKomis.clear();

      return true;
   }

   bool __fastcall GPairPane::StartManualPairing(void)
   {
   	using Tools::s_S;
   	using Tours::GPlayerPtr;
   	using TourMaster::GoTour;

      if(!ResGrid || !UpsetList) return false;

      // DEFINE Pairs AND Upsets SIZES
      int Boards = GoTour.PlayNowCount/2;	// We know that PlayNowCount is even

      try											// May not cope with very large Players
      {
	      Pairs.resize(Boards+1);          // Pairs represents current pairing Pairs[0] not used
	      Upsets.resize(2*Boards);         // Contains id's of upset players
		}
      catch(...)
      {
      	return false;
      }

      // CLEAR PAIRING GRID
      PairGrid->RowCount = Pairs.size()>1 ? Pairs.size() : 2;
      	PairGrid->ClearPairing();
      		PairGrid->ReBuild();

      // PUT ALL PLAYERS IN DRAW ON UPSET LIST IN RANK ORDER
      // GK 2015 const int OldOrder = GoTour.SortOrder, OldSense = GoTour.SortSense;

      Tours::RankSort(&GoTour);
      	if(Upsets.size() > GoTour.Sorted.size())
         	return false;

      UpsetList->Clear();

      for(size_t p=1; p <= GoTour.Players; p++)
      {
      	int pUpset = GoTour.Sorted[p];

         if(GoTour.PlayingThisRound(pUpset))
         {
         	if(UpsetList->Items->Count > (int)Upsets.size())
      			return false;

         	GPlayerPtr Player = GoTour.Listed[pUpset];

         	String UpsetName;

            switch(PrefsTube.ScreenNameStyle)
            {
               case PrefsMaster::dpSufMMS:
               	UpsetName = s_S(GoTour.ShortNameMMS(Player));
               break;

               case PrefsMaster::dpSufGrade:
                  UpsetName = s_S(GoTour.ShortNameGrade(Player));
               break;

               default:
                  UpsetName = s_S(GoTour.ShortName(Player));
            }

      		UpsetList->AddPlayer(UpsetName, pUpset);
         }
      }
      // GK 2015 RegMaster::RegSort(OldOrder, OldSense);

      ResGrid->Hide();	UpsetList->Mount(ResGrid);

      // SWITCH EDITING ON
      AttachHandlers();

      ClickPlayers.ResetPairEdit();

      // DISPLAY NEXT ROUND NUMBER
      return true;
   }

   bool __fastcall GPairPane::EndManualPairing(int& PairedCount)
   {
   	using Tools::S_s;
   	using DrawMaster::ManualDraw;
      using PairGUI::pfKomi;

      PairedCount = 0;

      if(!ResGrid || !UpsetList) return false;

      int Report = 0;

   	for(size_t b = 1; b<Pairs.size(); b++)
      {
      	Tours::GBoard& Paired = Pairs[b];

         std::string Handicap = S_s(PairGrid->Cells[pfKomi][b]);

         if(Paired.Black && Paired.White)
         {
            Report = ManualDraw(b, Paired, Handicap);

               if(Report) break;

            PairedCount += 2;
         }
      }

      if(Report < 0 )ToolsGUI::OperatorError(IDQ_BadKomiChar);
   	if(Report > 0 )ToolsGUI::OperatorError(IDQ_KOMI+1+Report);

      if(!Report)
         return CancelEdit();
      else
      {
         PairGrid->FocusKomi();

         return false;
      }
   }

   void __fastcall GPairPane::LoadFromTour(void)
   {
   	PairGrid->LoadFromTour();

      ResGrid->LoadFromTour();
         if(!ResGrid->Visible)
         	ResGrid->Show();
   }
}

namespace PaneGUI             // PAIRS LIST AND UPSET LIST SUPPORT
{
   void __fastcall GPairPane::StartPairs(int Round)
   {
   	using Tours::GBoard;
   	using TourMaster::GoTour;
      using PairGUI::pfKomi;
      using PairGUI::pfNote;

   	int Boards = Pairs.size()-1, OldBoards = GoTour.Pair.size()-1;

      if(Boards > OldBoards)PairGrid->RowCount = Boards+1;

   	for(int b = 1; b<=Boards; b++)								// By definition in StartEdit()
      if(b<=OldBoards)  												// we know that OldBoards <= Boards
      {
         Pairs[b] = GoTour.Pair[b];     							// Edit will permute player id's

         GBoard& Pair = Pairs[b];

         if(Pair.Black)
            if(GoTour.Listed[Pair.Black]->Leaving(Round))
            {
               Pair.Black = 0;

               PairGrid->Cells[pfBlack][b] = String();
               PairGrid->Cells[pfKomi][b]  = String();
               PairGrid->Cells[pfNote][b]  = String();
            }

         if(Pair.White)
            if(GoTour.Listed[Pair.White]->Leaving(Round))
            {
               Pair.White = 0;

               PairGrid->Cells[pfWhite][b] = String();
               PairGrid->Cells[pfKomi][b]  = String();
               PairGrid->Cells[pfNote][b]  = String();
            }

         std::string Komi = Tools::S_s(PairGrid->Cells[pfKomi][b]);

   		if(!Komi.empty())												// Gather Komi
         	OldKomis.push_back(DrawMaster::GHandicap(b,Komi));
      }
      else
      {
      	Pairs[b] =  GBoard(0,0);									// Upset players fill these

         PairGrid->Cells[PairGUI::pfBoard][b] = String(b);
         PairGrid->Cells[PairGUI::pfBlack][b] = String( );
         PairGrid->Cells[PairGUI::pfWhite][b] = String( );
         PairGrid->Cells[PairGUI::pfKomi][b]  = String( );
         PairGrid->Cells[PairGUI::pfNote][b]  = String( );
      }
   }

   void __fastcall GPairPane::StartUpsets(int Round)
   {
   	using Tools::s_S;
   	using TourMaster::GoTour;

      int Boards = Pairs.size()-1;

      for(int u = 0; u<2*Boards; u++)          				   // Edit sets and permutes Upsets
      	 Upsets[u] = 0;

      for(size_t p = 1; p<=GoTour.Players; p++)					// Add joining/unpaired players to UpsetList
      {
      	Tours::GPlayerPtr Player = GoTour.Listed[p];

         if(Player->Joining(Round) || Player->Unpaired(GoTour.Round))
         {
         	String UpsetName;

            switch(PrefsTube.ScreenNameStyle)
            {
               case PrefsMaster::dpSufMMS:
               	UpsetName = s_S(GoTour.ShortNameMMS(Player));
               break;

               case PrefsMaster::dpSufGrade:
                  UpsetName = s_S(GoTour.ShortNameGrade(Player));
               break;

               default:
                  UpsetName = s_S(GoTour.ShortName(Player));
            }

         	UpsetList->AddPlayer(UpsetName, p);
         }
      }
   }
}
//------------------------------------------------------------------------------







