////////////////////////////////////////////////////////////////////////////////
// MODULE  PairPaneGUI    Support for Drag and Click editing G.Kaniuk 14/04/04
//

#include <memory>
#include <vector>
#include <list>

#include "PairGUI.h"
#include "ResGUI.h"
#include "UpsetGUI.h"
#include "DrawMaster.h"

#ifndef PairPaneGUIH
#define PairPaneGUIH

namespace PaneGUI
{
   enum              // LOCAL EDITING STATES
   {
      psEditOff,     // Editing not allowed
      psEditOn,      // Editing enabled but inactive
      psSelect,      // Black chosen, waiting for White
      psPaired       // Black and White chosen
   };

   enum              // SOURCE
   {
      soGrid,        // Started on PairGrid
      soList         // Started on UpsetList
   };

   class GPairPane;

   class GClickPlayer                  // CLICK PLAYER EDITING
   {
      public:
         GPairPane* PairPane;          // Gateway to pairgrid and upsetlist

         String PlayerName;            // Whatever the source, this is the dragged name

         int  FirstCol, FirstRow;      // Cell Location for FirstOnGrid
         int  FirstIndex;              // Item index for FirstOnList
         bool FirstOnGrid;             // First click of a pair  on Grid
         bool FirstOnList;             // First click of a pair on List

         int  BoardPaired;             // The Board of the latest pairing

      public:
         __fastcall GClickPlayer();
         __fastcall ~GClickPlayer();

         void __fastcall StartEdit(int rowcount);
         void __fastcall EndEdit(void);
         void __fastcall ResetPairEdit(void);
         void __fastcall CancelPairEdit(void);
         void __fastcall PairEdit(int source);

      public:
         void __fastcall GridGridPair(int ColSec, int RowSec);
         void __fastcall ListGridPair(int ColSec, int RowSec);
         void __fastcall GridListPair(int IndexSec);
         void __fastcall ListListPair(int IndexSec);

         void __fastcall ClickPlayers(void);
   };
}
//---------------------------------------------------------------------------


namespace PaneGUI                      // DRAG PLAYER EDITING
{
   class GDragPlayer : public TDragObject
   {
      friend class GPairPane;

      private:
         String PlayerName;            // The source player

         Graphics::TBitmap* BitName;   // Bitmap of players name
         TImageList* ImageList;        // Owned by the form

         bool ImageShowing;            // Whether to show drag image

         int jT,jU,jL,jB,jE;				// j-Boundaries of Hot areas
         int iL,iS,iE,iR;					// i-Boundaries of Hot areas

      public:
         GPairPane* PairPane;          // Gateway to PairGrid and UpsetList
         TCursor CursorNow;            // The current drag cursor

         bool StartList, StartGrid;    // Identify drag source

         int SoCol, SoRow;             // Identify cell when dragging from Grid
         int SoIndex;                  // Identify item when dragging from List

         bool PhantomDrag;             // Only non-empty Black or White are not phantom

         bool HotT,HotB, HotL,HotR;    // Scrolling regions
         TTimer* Timer;                // Scroll Timer


      public:
         __fastcall GDragPlayer();
         __fastcall ~GDragPlayer();

         void __fastcall StartDragPlayer(TObject* Source, TCursor Cursor, String& Text, TRect&  Area);
         void __fastcall ScrollTimer(TObject* Sender);

      private:
         TCursor __fastcall GetDragCursor(bool accepted, int i, int j);
         void __fastcall NewImage(TRect Area, TCanvas* Canvas);
         TDragImageList* __fastcall GetDragImages(void);
         void __fastcall ShowDragImage(void);
         void __fastcall HideDragImage(void);

         void __fastcall Finished(TObject* Target, int i, int j, bool accepted);
         void __fastcall DragPlayer(TObject* Sink, int i, int j);
         void __fastcall GridGridDrop(int col, int row, int Unum, String Uname);
         void __fastcall ListGridDrop(int col, int row, int Unum, String Uname);
         void __fastcall GridListDrop(void);
         void __fastcall ListListDrop(void);
   };
}
//---------------------------------------------------------------------------

namespace PaneGUI                      // OVERALL MANAGEMENT
{
   using std::vector;
   using std::list;

   using PairGUI::GPairGrid;
   using ResGUI::GResGrid;
   using UpsetGUI::GUpsetList;

   using ToolsGUI::GRatio;
   using ToolsGUI::GGrid;

   using DrawMaster::GHandicap;

   class GPairPane
   {
      friend class GDragPlayer;
      
      public:
         GPairGrid*  PairGrid;         // Set by form - Gets scrolled by the timer event
         GResGrid*   ResGrid;          // Set by form
         GUpsetList* UpsetList;        // Set by form

         vector<Tours::GBoard>Pairs;	// Model the current pairing
         vector<int>Upsets;            // Model the current list of upset players

         list<GHandicap> OldKomis;		// The original komi list
         list<GHandicap> NewKomis;		// The komi list after editing

         int  EditState;               // Local state

         bool HasDragged;              // DragDetected

         int iSource, jSource;         // Cursor position at StartDrag

         GClickPlayer ClickPlayers;    // Manage Editing by clicking
         GDragPlayer* DragPlayer;      // Manage Editing by dragging

      private:
         int iDelta, jDelta;           // Drag rectangle set at StartDrag

         void __fastcall AttachHandlers(void);
         void __fastcall DetachHandlers(void);

      public:
         __fastcall GPairPane
         (
            GPairGrid* pairgrid, GResGrid* resgrid, GUpsetList* upsetlist,
            TImageList* imagelist, TTimer* timer
         );

         __fastcall ~GPairPane();


      public:
      	void __fastcall ReBuild(void);
         void __fastcall LoadFromTour(void);

         int  __fastcall EditKomi(String komitext);
         
         bool __fastcall StartEdit(void);
         bool __fastcall SaveEdit(void);
         bool __fastcall CancelEdit(void);
         void __fastcall StartPairs(int Round);
         void __fastcall StartUpsets(int Round);

         bool __fastcall StartManualPairing(void);
         bool __fastcall EndManualPairing(int& PairedCount);

         bool __fastcall DragDetected(int i, int j);

         int  __fastcall GetPlayerNumber(int col, int row);
         void __fastcall SetPlayerNumber(int col, int row, int number);
   };

}

//---------------------------------------------------------------------------
#endif
