////////////////////////////////////////////////////////////////////////////////
// MODULE PairPanelGUI:     All Events on PairPanel      G.Kaniuk 15/05/04
//
#include <vector>
#include <vcl.h>
#pragma hdrstop
#pragma package(smart_init)

#include "GoErrMsgRES.h"
#include "TourRES.h"
#include "PairRES.h"

#include "Tour.h"
#include "MenuMaster.h"
#include "TourMaster.h"
#include "DrawMaster.h"
#include "PairMaster.h"
#include "PairTool.h"
#include "PairMUI.h"
#include "TourMUI.h"
#include "Model.h"
#include "Pair.h"

#include "GoDrawGUI.h"
#include "PairPanelGUI.h"

extern void __fastcall ReloadPlayer(void);

//
//------------------------------------------------------------------------------
//
namespace PairPanelGUI									// MAGIC CONSTANTS
{
	enum
   {
   	pPairBtn,
      pPrDrawBtn, pPrRankBtn, pPrPrizesBtn,
      pForcedBtn, pEditBtn, pSaveBtn, pCancelBtn
   };

   static void Throw(ToolsGUI::EType etype, short idProc, short idMess)
   {
   	ToolsGUI::ThrowError(etype, IDE_PairEvent, idProc, idMess);
   }

   static Tours::TourState OldTourState;

   static PairMaster::PairCount ModelCount, ActualCount;
}
//------------------------------------------------------------------------------

namespace PairPanelGUI										// PAIR MONITOR
{
   void __fastcall PairMonitorCmd(TComponent* Owner)
   {
   	using TourMaster::GoTour;
   	using PairMaster::PairCount;

      const int TrialCount = PairMaster::GetTrials(GoTour.PairParam);

      if(TrialCount < 1 || TrialCount>PairMaster::size_monitor) return;


      std::vector<PairCount>Actuals(TrialCount);

      PairCount ModelCount, ActualCount;

      PairMaster::GetMonitor(GoTour.PairParam, ModelCount, ActualCount, Actuals);

   	PairMUI::MonitorForm(Owner, ModelCount, ActualCount, Actuals);
   }
}
//------------------------------------------------------------------------------

namespace PairPanelGUI										// PAIR
{
   void __fastcall SetPairBtnCaption(TButton* PairBtn)					// V6.5.0
   {
      const Tours::TourState& TourState(TourMaster::GoTour.State);

      const int idcaption = TourState==Tours::tsForcedPair
                          ? IDQ_EndManualBtn
                          : IDQ_DrawBtn;

      PairBtn->Caption = LoadStr(idcaption);
   }

   void __fastcall SetCancelBtnCaption(TButton* CancelBtn)				// V6.5.0
   {
   	const Tours::TourState& TourState(TourMaster::GoTour.State);

      int idcaption = IDQ_CancelBtn;

      switch(TourState)
      {
      	case Tours::tsDrawDone:   idcaption = IDQ_CancelDrawBtn;	  break;
         case Tours::tsForcedPair: idcaption = IDQ_CancelManualBtn; break;
         case Tours::tsPartDraw:   idcaption = IDQ_CancelDrawBtn;   break;
         case Tours::tsEditNow:    idcaption = IDQ_CancelEditBtn;   break;

         default: idcaption = IDQ_CancelBtn;
      }

      CancelBtn->Caption = LoadStr(idcaption);
   }

   void __fastcall PairCmd
   (
   	TComponent* Owner, TButton* DrawBtn,  TButton* CancelBtn,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using Tours::tsForcedPair;

   	using Tours::GPlayerPtr;
   	using TourMaster::GoTour;

      const size_t Round = GoTour.Round;
      
      int PairedCount = 0, IDQ_DrawReport = 0;

      GoTour.ClearTourInfo();

      PairMaster::PairParams Pars(GoTour.PairParam);

      std::vector<PairMaster::PairCount>Actuals(Pars.Trials);

      try
      {
         // PROLOGUE
         if(GoTour.State == tsForcedPair)									// MANUAL PAIRING
         {
            if(ToolsGUI::EditingCell())									// In tsForcedPair state
            {                                            			// the only cell you could
               Beep(); GoTour.PostTourInfo(IDQ_EndManualPair);   	// be editing is the komi
                                                         			// in the pair grid
               PairPane->PairGrid->FocusKomi();

               return;
            }

            DrawBtn->Caption = LoadStr(IDQ_DrawBtn);

            if(!PairPane->EndManualPairing(PairedCount))				// Dump unpaired
            	Throw(ToolsGUI::erBug, IDQ_PairCmd, IDQ_NoManual);

            GoTour.Roundl = GoTour.Round;
      	}
         else																		// NORMAL PAIRING
         {
         	if(GoTour.Round>0 && !(GoTour.PlayNextCount%2) && GoTour.JoiningCount)
               if(!TourMUI::JoinDrawAct(Owner))
               {
                  Beep(); GoTour.PostTourInfo(IDT_DrawCancelled);

                  return;
               }

         	if(!DrawMaster::StartDraw(ModelCount))
              Throw(ToolsGUI::erOperr, IDQ_PairCmd, IDQ_NoUnevenDraw);
         }
         GoDraw::AutoBackup();
         
			// PAIRING CYCLE
         if(PairedCount < GoTour.PlayNowCount)								// Pair remaining players
         {
        		if
            (
            	!DrawMaster::CycleDraw
               (
               	Pars.PairMode, GoTour.PlayNowCount-PairedCount,
                  ActualCount, Actuals
               )
            )
               if(GoTour.TourInfo())
               {
               	DrawMaster::ResetRound();

                  SetCancelBtnCaption(CancelBtn);

               	Throw(ToolsGUI::erOperr, IDQ_PairCmd, GoTour.TourInfo());
               }
               else
               	Throw(ToolsGUI::erBug, IDQ_PairCmd, IDQ_NoPairing);

            // ATTEMPT TO MINIMISE MMS DIFF ON TOP BOARDS
            if(PairedCount==GoTour.PlayNowCount)
            try
            {
            	if(PairTool::ImprovePairing()<0)
            		Throw(ToolsGUI::erLog, IDQ_PairCmd, IDQ_NoPairing);
            }
            catch(Exception& Err)
            {
            	ToolsGUI::Serve(Err, IDQ_PairCmd, IDQ_NoPairing);
            }

            if(DrawMaster::AllocateBoards(PairedCount)<0)
         		Throw(ToolsGUI::erBug, IDQ_PairCmd, IDQ_NoBoards);
         }
         else																			// Full Manual pairing
         {
            DrawMaster::EndFullManualDraw(ActualCount);

            GoTour.SortOrder = -1;											// GK REGSORT 2015  trigger register re-sort
         }

         // AUGMENT BOARD LAYOUT IF NECESSARY
         DrawMaster::LayoutBoards();

			// EPILOGUE
         PairMaster::PutMonitor(ModelCount, ActualCount, Actuals, GoTour.PairParam);

         if(PairedCount==GoTour.PlayNowCount)
         {
         	GoTour.State   = Tours::tsDrawDone;

            switch(Pars.PairMode)
            {
            	case PairMaster::pmNormal:	IDQ_DrawReport = IDQ_DrawComplete;			break;
               case PairMaster::pmModel:	IDQ_DrawReport = IDQ_DrawCompleteModel;	break;
               case PairMaster::pmTest:	IDQ_DrawReport = IDQ_DrawCompleteTest;		break;

               default:							IDQ_DrawReport = IDQ_DrawComplete;
         	}
         }
         else
         {
         	GoTour.State   = Tours::tsPartDraw;
         	IDQ_DrawReport = IDQ_ShortUnpaired;
         }

         GoTour.SaveTournament();

         PairPane->LoadFromTour();

         GoTour.PopulationCount();

         // FINALE
			if(PairedCount != GoTour.PlayNowCount)
         {
            int IDQ_Unpair = IDQ_LongUnpaired;

            if(Pars.PairMode != PairMaster::pmNormal)
            	IDQ_Unpair = PairedCount ? IDQ_ModelUnpair : IDQ_ModelAllUnpair;

				ToolsGUI::WarnOK(Tools::load_str(IDQ_Unpair));
         }
         else
      		PairMUI::MonitorForm(Owner, ModelCount, ActualCount, Actuals);

         ::ReloadPlayer();

         GoTour.PostTourInfo(IDQ_DrawReport);
      }
      catch(Exception& Err)
      {
      	if(GoTour.Round != Round)
         	DrawMaster::ResetRound();

         ToolsGUI::Serve(Err, IDQ_PairCmd, IDQ_NoPair);
      }
      catch(...)
      {
      	if(GoTour.Round != Round)
         	DrawMaster::ResetRound();

     		ToolsGUI::ShowWin32Err(IDE_PairEvent, IDQ_PairCmd);
      }

       SetCancelBtnCaption(CancelBtn);
   }
}
//------------------------------------------------------------------------------

namespace PairPanelGUI										// CANCEL DRAW
{
   void __fastcall CancelDrawCmd(TButton* CancelBtn, PaneGUI::GPairPane* PairPane)
   {
   	using TourMaster::GoTour;

   	try
      {
   		if(DrawMaster::ClearDraw())
         {
            GoTour.SaveTournament();

            DrawMaster::ListBoards();

      		PairPane->LoadFromTour();

            GoTour.PostTourInfo(IDQ_DrawUndo);
         }
         else
         	Throw(ToolsGUI::erBug, IDQ_CancelDrawCmd, IDQ_NoResetRound);

         ::ReloadPlayer();
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_PairPanelGUI, IDQ_CancelDrawCmd);
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_PairPanelGUI, IDQ_CancelDrawCmd);
      }

      SetCancelBtnCaption(CancelBtn);
   }
}
//------------------------------------------------------------------------------

namespace PairPanelGUI										// EDIT DRAW
{
	void __fastcall EditDrawCmd(TButton* CancelBtn, PaneGUI::GPairPane* PairPane)
   {
   	using TourMaster::GoTour;

   	OldTourState = GoTour.State;

      GoDraw::AutoBackup();

   	try
      {
      	if(GoTour.State == Tours::tsEditForced)
      		if(GoTour.PlayNowCount%2)
         		Throw(ToolsGUI::erOperr, IDQ_EditCmd, IDQ_NoUnevenEdit);

      	if(!PairPane->StartEdit())
         	Throw(ToolsGUI::erOperr, IDQ_EditCmd, IDE_NoMemory);

      	GoTour.State = Tours::tsEditNow;

         GoTour.PostTourInfo(IDQ_EditNow);
      }
      catch(Exception& Err)
      {
         PairPane->CancelEdit();

         GoTour.State = OldTourState;

      	ToolsGUI::Serve(Err,IDE_PairPanelGUI,IDQ_EditCmd);
      }
      catch(...)
      {
         PairPane->CancelEdit();

         GoTour.State = OldTourState;

      	ToolsGUI::ShowWin32Err(IDE_PairPanelGUI, IDQ_EditCmd);
      }

      SetCancelBtnCaption(CancelBtn);
   }
}
//------------------------------------------------------------------------------

namespace PairPanelGUI										// CANCEL EDIT
{
   void __fastcall CancelEditCmd
   (
   	TButton* DrawBtn, TButton* CancelBtn,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using TourMaster::GoTour;

      if(ToolsGUI::EditingCell())
      {
      	GoTour.PostTourInfo(IDQ_Cancelling);

         PairPane->PairGrid->FocusKomi();

      	return;
      }

      PairPane->CancelEdit();

      int IDQ_Undo = 0;

      if(GoTour.State == Tours::tsForcedPair)
      {
      	DrawMaster::CancelManualDraw();

         IDQ_Undo = IDQ_ForcedUndo;
      }
      else
      	IDQ_Undo = IDQ_EditUndo;

      GoTour.State = OldTourState;

      PairPane->LoadFromTour();

      DrawBtn->Caption = LoadStr(IDQ_DrawBtn);

      GoTour.PostTourInfo(IDQ_Undo);

       SetCancelBtnCaption(CancelBtn);
   }
}
//------------------------------------------------------------------------------

namespace PairPanelGUI										// SAVE DRAW
{
   void __fastcall SaveDrawCmd(TButton* CancelBtn, PaneGUI::GPairPane* PairPane)
   {
   	using TourMaster::GoTour;

      if(ToolsGUI::EditingCell())
      {
      	ToolsGUI::OperatorError(IDQ_Editing);

         PairPane->PairGrid->FocusKomi();

      	return;
      }

   	if(!PairPane->SaveEdit())
      {
      	ToolsGUI::OperatorError(IDQ_NoSave);

      	return;
      }

      GoTour.PopulationCount();
         GoTour.SetState();
         	GoTour.SaveTournament();

      PairPane->LoadFromTour();

      ::ReloadPlayer();

      GoTour.PostTourInfo(IDQ_Saved);

      SetCancelBtnCaption(CancelBtn);
   }
}
//---------------------------------------------------------------------------

namespace PairPanelGUI								// MANUAL PAIRING
{
   void __fastcall ManualPairingCmd
   (
   	TComponent* Owner,  TButton* DrawBtn, TButton* CancelBtn,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using Tours::stAllPlay;
      using Tours::stAllPlayHcap;
      
   	using Tours::GPlayerPtr;
   	using TourMaster::GoTour;
		using PairMaster::PairCount;

      const size_t Round = GoTour.Round;

      try
      {
      	OldTourState = GoTour.State;

         const int OddParity =  GoTour.Round>0 ? GoTour.PlayNextCount%2 : GoTour.PlayNowCount%2;

      	if(OddParity)
         	Throw(ToolsGUI::erOperr, IDQ_PairCmd, IDQ_NoUnevenDraw);

         if(GoTour.Rules.Style==stAllPlay || GoTour.Rules.Style==stAllPlayHcap)
         	Throw(ToolsGUI::erOperr, IDQ_PairCmd, IDQ_NoAllPlayMan);

         if(GoTour.Round>0 && GoTour.JoiningCount)
            if(!TourMUI::JoinDrawAct(Owner))
            {
               Beep(); GoTour.PostTourInfo(IDT_DrawCancelled);

               return;
            }

         ++GoTour.Round;										// Roundl stays until Draw

         GoTour.PopulationCount();

      	DrawMaster::StartManualDraw(ModelCount);

			if(!PairPane->StartManualPairing())
         	Throw(ToolsGUI::erOperr, IDQ_ForcedCmd, IDQ_NoForcedMemory);

         GoTour.State = Tours::tsForcedPair;

         DrawBtn->Caption = LoadStr(IDQ_EndManualBtn);

          GoTour.PostTourInfo(IDQ_ManualReady);
      }
      catch(Exception& Err)
      {
      	if(GoTour.Round != Round)
         	DrawMaster::ResetRound();

         ToolsGUI::Serve(Err, IDQ_ForcedCmd, IDQ_NoForcedMemory);
      }
      catch(...)
      {
      	if(GoTour.Round != Round)
         	DrawMaster::ResetRound();

     		ToolsGUI::ShowWin32Err(IDE_PairEvent, IDQ_ForcedCmd);
      }

      SetCancelBtnCaption(CancelBtn);
	}
}
//---------------------------------------------------------------------------

 