////////////////////////////////////////////////////////////////////////////////
// MODULE PairSort: All sort functions for the pairing dll  G.Kaniuk 16/10/04
////////////////////////////////////////////////////////////////////////////////
#include <functional>
#include <vector>
#include <string>

#include "Tour.h"

#ifndef PairSortH
#define PairSortH
//
////////////////////////////////////////////////////////////////////////////////
// ENVIRON
//
namespace PairSort
{
	using std::string;
   using std::vector;

	extern vector<int> Sorted;
   extern const string noc;

   void StartSort(int Size);
   void EndSort(void);

   bool ValidSortRange(int a, int b);

	Tours::GPlayerPtr SortedPlayer(const int p);
   Tours::GPlayerPtr Player(const int p);

   void ScanSorted(int a, int b);
}


////////////////////////////////////////////////////////////////////////////////
// SORT ROUTINES
//
namespace PairSort
{
   using Tools::ints;
   
   typedef vector<int>::iterator Iterator;

   void Throw(int ID_message);
   
   void McMahonSort(const int a, int& b);
   void ModMcDiffSort(const int a, int& b, const ints mcscore);

   void KomiSort(const int a, int& b, const bool komiplayer);
   
   void AreaSort(const int a, int& b);
   void AreaSizeSort(const int a, const int b);
   bool AlienAreaSort
   (
   	const int a, int& b,
      const string alienland, const string alienclub
   );

   enum { saForward=1, saReverse=-1 };	
   void SameAreaSort(const int sortdir, const int a, int& b);
   
   void GroupCodeSort(const int a, int& b);
   bool NextGroupCode
   (
   	const int rs, const int re,			// Range of players
      string& GroupCode,						// The code found
      int& a, int& b								// The group range:
   );                                     // rs <= a<=b <= re
   void CompatibleSort(const int a, int& b, const char tag, string compatcode);

   void PlayerIdSort(const int pa, const int pb);

   void PoolSort(const int a, int& b);

   enum { mrNone, mrUp, mrDown, mrOut }; 				// Aliases PMcMahon::pcClass
   int MibSort(const int a, int& b, const int rule);
   bool ClassSort(const int a, int& b, const int rule);

   enum { crGrey, crBlack, crWhite };
   void CibSort(const int a, int& b, const int rule);

   void BusySort(const int a, int& b);
}
#endif
//
////////////////////////////////////////////////////////////////////////////////
