////////////////////////////////////////////////////////////////////////////////
// MODULE PlayerMaster:Interface EditPlayerForm to GTourMaster G.Kaniuk 05/02/08
//
#include <stdio.h>
#include <vcl.h>
#include <string>
#include <algorithm>
#include <functional>
#pragma hdrstop

#include "TourRES.h"
#include "SetsRES.h"
#include "EdPlayerRES.h"
#include "GoErrmsgRES.h"
#include "GoFormRES.h"

#include "ETour.h"
#include "Strings.h"
#include "Format.h"
#include "GodrawGUI.h"
#include "StringsGUI.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "PlayerMaster.h"

// GK REGSORT 2015 extern void __fastcall EndCommand(int idcmd);

//---------------------------------------------------------------------------
// ERROR HANDLING
//---------------------------------------------------------------------------
namespace PlayerMaster
{
	using std::string;

   using Tools::load_str;
   using Tools::sCrLf;
   using Tools::sQuery;
   using Tools::S_s;
   using Tools::s_S;
   using Tools::trim;
   using Tools::uppercase;
   using Tools::strip_white_space;

   using Tools::ints;
   using Tools::compactify;



	using ToolsGUI::ETour;
   using ToolsGUI::erOperr;

   enum				// Errors in LoadPlayerFromFile
   {
   	leNoError, leCannotLoad,
      leFewLines, leBadHeader,
      leBadLine
   };

   static void Throw(ToolsGUI::EType etype, short idproc, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_PlayerMaster, idproc, idmsg);
   }

   static void Throw(const int eload, const int eline)
   {
   	using std::string;

   	string message = Tools::load_str(IDB_LOADPLAYERLIST + eload);

      if(eload==leBadLine)
      {
         int posline = message.find(string(Tools::sQuery), 0);

         message.replace(posline, 1,Tools::to_string(eline));
      }

      throw ETour(erOperr, message);
   }

   static void ThrowSyntax(int filetype,  std::string filename, int line)
   {
   	using std::string;

      string message = load_str(filetype)+string(sCrLf) +
                       string(3, Tools::cSpace) + filename+string(sCrLf) +
                       load_str(IDE_BadSyntax)+ Tools::to_string(line);

      throw ETour(erOperr, message);
   }
}

//---------------------------------------------------------------------------
// TEXT SOURCE
//---------------------------------------------------------------------------
namespace PlayerMaster
{
	using TourMaster::GoTour;

   const std::string pidFmt("0000");

   static String TieName(int Level, int nTie)
   {
   	if(Level!=1 && Level !=2) return String();

      if(nTie<0 || nTie>=Tours::tbSize) return String();

      if(nTie>0)
         return LoadStr(IDM_TIEBREAK + nTie);

      if(Level==1)return LoadStr(IDT_notie1);
      if(Level==2)return LoadStr(IDT_notie2);

      return String();
   }

   static String TieValue(int nTie, int vTie)
   {
   	using Tools::s_S;

   	if(nTie<=0 || nTie>=Tours::tbSize) return String();

      if(nTie == Tours::tbWins) return s_S(Tours::wins_ustring(vTie));

      return s_S(Tours::mms_ustring(vTie));
   }

   static String GamePlaying(Tours::GGame& Game)
   {
   	using Tours::grSleep;
      
   	const int Result = (int)Game.Result;

      return Result != grSleep ? LoadStr(IDB_Yes) : LoadStr(IDB_NoText);
   }

   static String GameOpponent(Tours::GGame& Game)
   {
   	Tours::GPlayerPtr Opp = TourMaster::GoTour.Listed[Game.Oppid];

      if(GoTour.State <= Tours::tsRegClosed)return String();

      return Game.Oppid>0 ? Tools::s_S(GoTour.Name(Opp)) : LoadStr(IDB_NoText);
   }

   static String GameColour(Tours::GGame& Game)
   {
   	if(GoTour.State <= Tours::tsRegClosed)return String();

   	switch((int)Game.Colour)
      {
      	case Tours::gcBlack: return String("B");
         case Tours::gcWhite: return String("W");

         default:
         	if(!Game.Oppid)
            	return LoadStr(IDB_NoText);	// Normal
            else
            	return String("?");				// Not yet updated
      }
   }

   static String GameParity(Tours::GGame& Game)
   {
   	if(GoTour.State <= Tours::tsRegClosed)return String();

      if(Game.Result==Tours::grSleep) return LoadStr(IDB_NoText);

   	switch((int)Game.Parity)
      {
      	case Tours::gdUp:   return String("Up");
         case Tours::gdDown: return String("Down");
         case Tours::gdEven: return String("Even");

         default:				  return LoadStr(IDB_NoText);
      }
   }

   static String GameStones(Tours::GGame& Game)
   {
   	if(!Game.Stones) return LoadStr(IDB_NoText);

      return String((int)Game.Stones/Tours::mc_scale);
   }

   static String GameKomi(Tours::GGame& Game)
   {
   	if(!Game.Komi) return LoadStr(IDB_NoText);

      return String((int)Game.Komi/Tours::mc_scale);
   }

   static String GameResult(Tours::GGame& Game)
   {
   	switch(Game.Result)
      {
      	case Tours::grLose: 	return String("0");
         case Tours::grJigo:  return String("J");
         case Tours::grWin:   return String("1");
         case Tours::grUsolv: return String("Unfin");
         case Tours::grDlos:  return String("Dloss");
         case Tours::grFwin:  return String("Fwin");
         case Tours::grVoid:  return String("Void");
         case Tours::grSleep: return String("...");
         case Tours::grNone:	return String("?");

         default:					return String();
      }
   }

   static String McScore(int mms)
   {
   	return Tools::s_S(Tours::mms_ustring(mms));
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - CONSTRUCTION AND SIMPLE LISTS
//---------------------------------------------------------------------------
namespace PlayerMaster
{
	using Tools::s_S;
   using Tools::S_s;
   using TourMaster::GoTour;

	MEditPlayerTube::MEditPlayerTube(void)
   {
   	TourName = String();

      PlayerID = String();
      Name     = LoadStr(IDB_Name);
      Grade    = LoadStr(IDB_Grade);
      Club     = LoadStr(IDB_Club);
      Land		= LoadStr(IDB_Land);
      Note		= LoadStr(IDB_Note);

      GradeChanged = false;
      GradeEnabled = false;
      MMSiEnabled  = false;
      NewBlocked   = false;
      State 		 = epiVirgin;

      PlayerCount  = -1;
      PlayerRow    = -1;
      PlayerCol    = -1;
      PlayerDup    = -1;

      GroupError 	 = 0;
   }

   void MEditPlayerTube::LoadTour(void)
   {
   	Rounds = GoTour.Rules.Rounds;	// Manage games grid
   	Round  = GoTour.Round;
      Roundl = GoTour.Roundl;
   }

   void MEditPlayerTube::LoadPlayerIDs(TStrings* ids)
   {
   	using Tools::to_string;
      using Tools::s_S;
      using Tours::GPlayerPtr;

      ids->Clear();

      PlayerCount = TourMaster::GoTour.Players;

      for(int p=1; p<=PlayerCount; p++)
      {
         GPlayerPtr player = GoTour.Listed[GoTour.Sorted[p]];

         ids->Add(s_S(to_string(pidFmt, (int)player->Pid)));
      }

      ids->Add(s_S(to_string(pidFmt, GoTour.NewPid)));
   }

   void MEditPlayerTube::LoadListsFromAlgo
   (
   	TStrings* ids,			// PlayerID Combo
      TStrings* grades,    // Grade Combo
      TStrings* groups     // Groups Combo
   )
   {
      using Tools::s_S;

      // PlayerID Combo
      LoadPlayerIDs(ids);

      // GRADE COMBO
      {
         using Tours::mc_scale;
         using Tours::pro_grade;
         using Tours::def_mingrade;
         using Tours::grade_ustring;

         grades->Clear();

         grades->Add(s_S(grade_ustring(Tours::zero_pro)));
         grades->Add(s_S(grade_ustring(Tours::zero_dan)));
         grades->Add(s_S(grade_ustring(Tours::zero_kyu)));

         const int gmax = pro_grade + 8*mc_scale, gmin = -99*mc_scale;

         for(int g = gmax; g>=gmin; g -= mc_scale)
         {
            std::string grade = grade_ustring(g);

            grades->Add(s_S(grade));
         }
      }

      // GROUP COMBO
      {
         groups->Clear();

         String Prefer(Tools::cPlus), PreferNot(Tools::cMinus);
         String Must(Tools::cStar), MustNot(Tools::cTilde);

         for(char GroupId = 'A'; GroupId <= 'Z'; GroupId++)
         {
            String GroupName(GroupId);

            groups->Add(GroupName + PreferNot);
            groups->Add(GroupName + Prefer);

            groups->Add(GroupName + MustNot);
            groups->Add(GroupName + Must);
         }
         groups->Add(LoadStr(IDB_None));
         groups->Add(" ");
         groups->Add(String());
      }
      for(int s=0; s<=9; s++)
      	groups->Add(String(s)+ " ");
      groups->Add(String());
   }

   void MEditPlayerTube::LoadRegionFromFile  // HAPPENS ONCE AT STARTUP
   (
      const int codesize, const String& filename,
      TStrings* regions
   )
   {
      TStringList* CodeFile = NULL;

      try
      {
         CodeFile = new TStringList();

         CodeFile->LoadFromFile(filename);
      }
      catch(...)
      {
         delete CodeFile;

         Throw(ToolsGUI::erOperr, IDE_LoadFromFile, IDE_NoMemory);
      }

      for(int i=0; i<CodeFile->Count; i++)
      {
         const String Item = CodeFile->Strings[i];

         if(Item.Length()<codesize+2)
            ThrowSyntax(IDE_BadCodeFile, S_s(filename), i+1);

         const String Name = Item.SubString(codesize+2, Item.Length()-(codesize+1));

         if(!regions->Count)            	// List is empty
            regions->Add(Name);				// So add the name
         else
         	if(regions->IndexOf(Name)<0)	// Name is not in list
            	regions->Add(Name);			// Add unique names to list
      }

      delete CodeFile;
   }

   void MEditPlayerTube::LoadRegionsFromFile
   (
   	TStrings* clubs,		// Club Combo
      TStrings* lands      // Country Combo
   )
   {
   	using GoDraw::FilePath;

    	const String clubfile = FilePath(GoDraw::GoClubCode);

   	if(FileExists(clubfile))
      	LoadRegionFromFile(3, clubfile, clubs);


    	const String landfile = FilePath(GoDraw::GoLandCode);

      if(FileExists(landfile))
      	LoadRegionFromFile(2, landfile, lands);
   }

   void MEditPlayerTube::UpdateLists
   (
   	TStrings* ids, TStrings* names,
      TStrings* clubs, TStrings* lands,
      TStrings* notes
   )
   {
   	using Tools::s_S;
      using Tools::to_string;
      using Tools::pack_tabbed;

   	using ToolsGUI::AddNewItem;
      using ToolsGUI::FindInTabbedList;

      String NewPid = s_S(to_string(pidFmt, GoTour.NewPid));

   	AddNewItem(NewPid, ids);
      AddNewItem(Club, clubs);
      AddNewItem(Land, lands);
      AddNewItem(Note, notes);

      // WE NEED TO PAKCAGE name~grade~club~land
      // TO ADD TO names
      {
      	string items[4], packeditem;

      	items[0] = S_s(Name);  items[1] = S_s(Grade);
         items[2] = S_s(Club);  items[3] = S_s(Land);

      	Tools::pack_tabbed(4, items, &packeditem);

         // FIND THE NAME IN THE TABBED LIST
         String PlayerItem = s_S(packeditem);

         int index = FindInTabbedList(Name, names);

         if(index>=0)								// Name was found in list
         	names->Delete(index);            // We assume other details out of date

         AddNewItem(PlayerItem, names);
      }
   }

   void MEditPlayerTube::UpdateListsFromTour
   (
   	TStrings* ids, TStrings* names,
      TStrings* clubs, TStrings* lands,
      TStrings* notes
   )
   {
   	using Tours::GPlayerPtr;

      const int OldState = State;                     // LoadExistingPlayer will disturb the state

      for(size_t p=1; p <= GoTour.Players; p++)
      {
      	GPlayerPtr player = GoTour.GetPlayerPtr(p);	// March through list in current sort order

         if(player)
         {
         	Player = *player;

         	LoadExistingPlayer(p);

         	UpdateLists(ids, names, clubs, lands, notes);
         }
      }

      State = OldState;                               // Reset the state 
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - LOADING THE PLAYER LIST
//
// At V6.5.0 we provide access to the player rating
// If the rating field in the player list is present we use that to set
// the tournament entry grade. The note field on the player form shows the
// old tournament grade and the rating.
//
//---------------------------------------------------------------------------
namespace PlayerMaster                          // PARSING TOOLS
{
   using std::string;
   using std::list;
   using std::vector;

   using Tools::unpack_tabbed;
   using Tools::pack_tabbed;

   using Tools::load_str;

   using Tools::S_s;
   using Tools::s_S;

   using ToolsGUI::erBug;
   using ToolsGUI::erOperr;

   enum { hdName, hdGrade, hdClub, hdLand, hdCount, hdRate = hdCount, hdMax = hdCount+1};
   enum { ehEmpty=-1, ehTooFew=-2, ehNoName=-3, ehNoGrade=-4, ehNoClub=-5, ehNoLand=-6 };

   static const int HeadID[hdMax] =          // The Register Header (V6.5.0 was  HeadID[hdCount])
   {
      IDB_INameFG, IDB_IGrade, IDB_IClub, IDB_ILand, IDB_IRating
   };

   static int ParseHeader              // FOR REGIMPORT AND NAMEFIELD SUPPORT
   (
      string& Head,                    // The supplied header
      vector<int>& FieldMap          	// Identifies the fields name,grade, club, country, and rating
   )
   {
      using Tools::trim;

      // PREPARE THE HEADER NAMES
      if(Head.empty())return ehEmpty;

      string HeadName[hdMax];

      for(int h=0; h<hdMax; h++)
      	HeadName[h] = Tools::load_str(HeadID[h]);

      // THE COLUMNS ARE SELECTED FROM THE HEADERS IN THE FILE
      list<string>Headers;

      int ItemCount = unpack_tabbed(Head, hdMax, Headers);		// Number of non-empty items
      if(ItemCount<hdCount) return ehTooFew;

      for(int f=0; f<hdMax; f++)FieldMap[f] = -1;         		// FieldMap indexed by field

      int item = 0;                                         	// Index to an item in the Headers list

      list<string>::iterator sHead = Headers.begin();
      list<string>::iterator eHead = Headers.end();

      // SEARCH FOR THE REQUIRED HEADERS AND SET UP THE FIELD MAP
      while(sHead != eHead)
      {
         for(int h=0; h<hdMax; h++)
         {
           	if(HeadName[h] == uppercase(*sHead))
            {
               FieldMap[h] = item;	break;
            }
         }
         ++sHead;  ++item;
      }
      if(FieldMap[hdName]<0)  return ehNoName;
      if(FieldMap[hdGrade]<0) return ehNoGrade;
      if(FieldMap[hdClub]<0)  return ehNoClub;
      if(FieldMap[hdLand]<0)  return ehNoLand;
      																			// An empty rating field is not an error
      return item; 															// Number of used fields (return hdCount; )
   }

   static void LowerGradeSuffix(string& GradeText)
   {
   	const int Suffix = GradeText.size()-1;

      GradeText[Suffix] = Tools::lowercase(GradeText[Suffix]);
   }

   static bool ForgePlayerItem   // RE-FORMAT FOR LOADING NAMEFIELD LISTBOX
   (
      vector<int> FieldMap,    	// Identified by ParseHeader
      string& PlayerIn,          // Possibly in weird order
      string& PlayerOut          // "Family Given~Grade~Club~Country"
   )
   {
      using Tools::compactify;
      using Tools::compactify_name;

      const string space = string(Tools::sSpace);
      const string comma = string(Tools::sComma);

      // UNPACK PlayerIn TO GET A ROW OF STRINGS
      list<string> ListItemsIn; vector<string>ItemsIn;
      string ItemsOut[hdMax];

      const int ItemCount = unpack_tabbed(PlayerIn, hdMax, ListItemsIn);
      if(ItemCount<hdCount) return false;
      ItemsIn.resize(ItemCount);

      list<string>::iterator sItem = ListItemsIn.begin();

      for(int f=0; f<ItemCount; f++)
         ItemsIn[f] = compactify(*sItem++);

      // WE WANT THE NAME IN THE FORM: FAMILY,Given Middle
      string FullName = ItemsIn[FieldMap[hdName]];
      if(!RegMaster::make_family(FullName)) return false;
      ItemsOut[hdName] = FullName;

      // PICK THE REMAINING FIELDS
      ItemsOut[hdGrade] = ItemsIn[FieldMap[hdGrade]];  LowerGradeSuffix(ItemsOut[hdGrade]);
      ItemsOut[hdClub]  = ItemsIn[FieldMap[hdClub]];
      ItemsOut[hdLand]  = ItemsIn[FieldMap[hdLand]];

      ItemsOut[hdRate]  = ItemsIn[FieldMap[hdRate]];				// Possibly empty

      // PACK THEM IN THE CORRECT ORDER
      pack_tabbed(hdMax, ItemsOut, &PlayerOut);

      return true;
   }


   void MEditPlayerTube::LoadPlayersFromFile(TStrings* Players)
   {
   	using Tools::S_s;
      using TourMaster::GoTour;

      // LOAD THE ENTIRE FILE
      try
      {
         Players->LoadFromFile(GoDraw::FilePath(GoDraw::GoPlayers));
      }
      catch(...)
      {
         Throw(leCannotLoad, 0);
      }

      // ESTABLISH NUMBER OF LINES IN THE FILE
      int Lines = Players->Count;   			// This includes the header
      if(Lines<2)return Throw(leFewLines,1);	// which does not appear in the list box

      // ANALYSE THE HEADER TO IDENTIFY Name-parts, Grade, Club, Country, Rating
      vector<int> FieldMap(hdMax,-1);

      string Head = S_s(Players->Strings[0]);

      const int FieldsUsed = ParseHeader(Head, FieldMap);

      if(FieldsUsed < hdCount)Throw(leBadHeader, 1); 				// Need at least Name, Grade Club, LAnd

      Players->Delete(0);	--Lines;

      // REPLACE EACH LINE IN players
      for(int line=0; line<Lines; line++)
      {
         string PlayerLine = S_s(Players->Strings[line]), Fplayer;

         if(!ForgePlayerItem(FieldMap, PlayerLine, Fplayer))
           Throw(leBadLine, line+1);

         Players->Strings[line] = s_S(Fplayer);
      }
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - LOADING ALL FIELD TEXT
//---------------------------------------------------------------------------
namespace PlayerMaster
{
	enum { rsNoReg, rsReg, rsJoin, rsLeave, rsRegsize };

   void MEditPlayerTube::LoadNewPlayer(void)
   {
   	using Tours::psRegistered;

   	// FILLS ALL PLAYER SPECIFIC TUBE FIELDS FOR A NEW PLAYER
      PlayerID = s_S(Tours::pid_string(GoTour.NewPid));
      Name		= String();
      Grade		= String();
      Club		= String();
      Land		= String();
      Group		= String();
      Note		= String();

      MMSi  	= 0;

      State    = epiNew;

      Reg 		= Player.State >= psRegistered ? rsReg : rsNoReg;
   }

   void MEditPlayerTube::LoadExistingPlayer(const size_t row)
   {
   	using Tours::psRegistered;

      PlayerID = s_S(GoTour.Number(&Player));
      Name     = s_S(Player.Name);
      Grade    = s_S(GoTour.Grade(&Player));
      Club     = s_S(GoTour.Club(&Player));
      Land		= s_S(GoTour.Land(&Player));
      Group		= s_S(GoTour.Group(&Player));
      Note		= s_S(GoTour.Note(&Player));

      MMSi 		= Player.MiScore;
      MMSc     = Player.McScore;

      State   	= epiExists;

      Reg 	 	= Player.State >= psRegistered ? rsReg : rsNoReg;

      if(Roundl>0)
      {
      	if(Player.Joining(Round)) Reg = rsJoin;
      	if(Player.Leaving(Round)) Reg = rsLeave;
      }
   }

   void MEditPlayerTube::LoadPlayer(const size_t row)
   {
   	using Tours::grSleep;
      using Tours::grNone;
   	using Tours::psEntered;
      using Tours::psSleep;
      using Tours::tsRegClosed;
      using Tours::tsPastRound;

   	using Tours::GPlayerPtr;
      using Tours::GPlayer;

   	if(row > GoTour.Players+1) return;

      PlayerCount = GoTour.Players;

   	TourName 	= s_S(GoTour.TourNameYear());

      Rounds 		= GoTour.Rules.Rounds;
      Roundl 		= GoTour.Roundl;
      Round  		= GoTour.Round;

      Playing.resize(Rounds);

      if(row <= GoTour.Players)
      {
   		GPlayerPtr player = GoTour.Listed[GoTour.Sorted[row]];

         if(!player) return;

         Player = *player;

         PlayerRow = row;

         LoadExistingPlayer(row);
      }
      else
      {
      	Player = GPlayer(GoTour.NewPid, Rounds);

         Player.State = GoTour.State >tsRegClosed ? psSleep : psEntered;

         for(int r=0; r<Rounds; r++)
      		Player.Games[r].Result = r<Round ? grSleep : grNone;

         PlayerRow = GoTour.Players+1;
         PlayerCol = 2;

         LoadNewPlayer();
      }

      for(int r=0; r<Rounds; r++)
      	Playing[r] = Player.Games[r].Result != grSleep;

      GradeEnabled = false;
      MMSiEnabled  = false;

      NewBlocked = GoTour.State==tsRegClosed || GoTour.State==tsPastRound;
   }

   void MEditPlayerTube::LoadPlayer(const size_t col, const size_t row)
   {
   	LoadPlayer(row);

   	PlayerCol = col;
   }

   String MEditPlayerTube::cMMSText(void)
   {
   	return(McScore(Player.McScore));
   }

   String MEditPlayerTube::iMMSText(const int mms)
   {
   	return(McScore(mms));
   }

   String MEditPlayerTube::TieLabel(const int level)
   {
   	int ntie;

      if(level == 1) ntie = GoTour.Rules.TieOne;
      if(level == 2) ntie = GoTour.Rules.TieTwo;

   	return TieName(level, ntie);
   }

   String MEditPlayerTube::TieText(const int level)
   {
   	int vtie;

      if(level == 1) vtie = (int)Player.TieOne;
      if(level == 2) vtie = (int)Player.TieTwo;

   	return TieValue(level, vtie);
   }

   String MEditPlayerTube::ColBalText(void)
   {
   	return LoadStr(IDB_ColBal) + String((int)Player.Cbal);
   }

   String MEditPlayerTube::UpDnBalText(void)
   {
   	return LoadStr(IDB_UpDnBal) + String((int)Player.Mbal);
   }

   String MEditPlayerTube::UnevenText(void)
   {
   	return LoadStr(IDB_Uneven) + String((int)Player.Mout);
   }

   String MEditPlayerTube::GameText(const int r, const int c)
   {
   	const String NoText = LoadStr(IDB_NoText);
      const String Yes    = LoadStr(IDB_Yes);

      if(r<0 || c>=8 || State == epiVirgin) return String();

      if(r>Rounds && c<8) return NoText;

      // NEW PLAYER
      if(State==epiNew)
      	if(c==pfPlaying && r>Roundl)
         	return Yes;
         else
         	return NoText;

      // EXISTING PLAYER
   	Tours::GGame& Game = Player.Games[r-1];

      if(c != pfPlaying && r>Roundl)
      	return NoText;
         
      switch(c)
      {
      	case 1: return GamePlaying(Game);
      	case 2: return GameOpponent(Game);
         case 3: return GameColour(Game);
         case 4: return GameParity(Game);
         case 5: return GameStones(Game);
         case 6: return GameKomi(Game);
         case 7: return GameResult(Game);

         default: return String();
      }
   }
}

//---------------------------------------------------------------------------
// VALIDATION AND PARSING OF USER TEXT
//---------------------------------------------------------------------------
namespace PlayerMaster
{
	using Tours::GPlayer;

   using RegMaster::make_family;
   using RegMaster::parse_grade;
   using RegMaster::make_region;
   using RegMaster::parse_group;
   using RegMaster::parse_pid;
   using RegMaster::FindPlayerName;

   enum { size_club = 3, size_land = 2 };

   bool MEditPlayerTube::InvalidReg(const int reg)
   {
   	return reg < rsNoReg || reg >= rsRegsize;
   }

   int MEditPlayerTube::ValidPid(String& Pid)
   {
      // WE JUST PARSE FOR OBVIOUS SYNTAX ERRORS AND OUT OF RANGE
      string idtext = strip_white_space((S_s(Pid)));  int pid;

      switch (parse_pid(idtext, pid))
      {
         case RegMaster::pidBig:
         {
         	PlayerInfo = pid;

            return PlayerMaster::pePidBig;
         }

         case RegMaster::pidZero:
            return PlayerMaster::pePidZero;

         case RegMaster::pidBad:
            return PlayerMaster::pePidBad;

         default:
         {
         	Pid = s_S(idtext); PlayerInfo = pid;

            return 0;
         }
      }
   }

   int MEditPlayerTube::ValidName(String& NameF)
   {
      string name = S_s(NameF);  if(name.empty()) return 0;

      if(!RegMaster::make_family(name)) return peName;

      NameF = s_S(name);

      return 0;
   }

   int MEditPlayerTube::MakeName(String& NameF, GPlayer& player)
   {
   	using RegMaster::FindPlayerName;

   	int Report = ValidName(NameF);      if(Report>0)return Report;

      std::string name = S_s(NameF);

      const int iplayer = FindPlayerName(uppercase(name));

      if(iplayer && iplayer != PlayerRow)
      {
         PlayerDup = GoTour.Listed[GoTour.Sorted[iplayer]]->Pid;

         return peNameDup;
      }

      player.Name = name;

      return 0;
   }

   int MEditPlayerTube::ValidGrade(String& GradeF)
   {
      string sgrade = S_s(GradeF);   if(sgrade.empty()) return 0;

      ints vgrade;

      if(!parse_grade(sgrade, vgrade))return peGrade;

      GradeF = s_S(sgrade);

      return 0;
   }

   int MEditPlayerTube::MakeGrade(String& GradeF, GPlayer& player)
   {
      string sgrade = S_s(GradeF);

      if(!parse_grade(sgrade, player.Grade))
      	return peGrade;

      GradeF = s_S(sgrade);

      return 0;
   }

   int MEditPlayerTube::ValidClub(String& ClubF)
   {
   	string sclub = compactify(S_s(ClubF)), clubcode;   if(sclub.empty()) return 0;

      if(!make_region(size_club, sclub, clubcode)) return peClub;

      ClubF = s_S(sclub);

      return 0;
   }

   int MEditPlayerTube::MakeClub(String& ClubF, GPlayer& player)
   {
   	string sclub = S_s(ClubF), clubcode;

      if(!make_region(size_club, sclub, clubcode)) return peClub;

      player.Club = clubcode;

      return 0;
   }

   int MEditPlayerTube::ValidLand(String& LandF)
   {
   	string sland = compactify(S_s(LandF)), landcode;	if(sland.empty()) return 0;

      if(!make_region(size_land, sland, landcode)) return peLand;

      LandF = s_S(sland);

      return 0;
   }

   int MEditPlayerTube::MakeLand(String& LandF, GPlayer& player)
   {
   	string sland = S_s(LandF), landcode;

      if(!make_region(size_land, sland, landcode)) return peLand;

      player.Land = landcode;

      return 0;
   }

   int MEditPlayerTube::ValidGroup(String& GroupF)
   {
   	GroupError = 0;

      if(GroupF == LoadStr(IDB_None))
   	{
   		GroupF = String();	return 0;
   	}

      string group = trim(S_s(GroupF));  char grpid;  int grptype;

      if(!parse_group(group, grpid, grptype))						// GK 6.4.2 split supergroup errors
      {
         GroupError = -grptype;											// Error details

         return peGroup;													// Error indication
      }

      GroupF = s_S(trim(group));

      return 0;
   }

   int MEditPlayerTube::ValidNote(String& NoteF)
   {
      string note = compactify(S_s(NoteF));

      if(!Tools::is_ascii(note)) return peNote;

      NoteF = s_S(note);

      return 0;
   }

   int MEditPlayerTube::ValidMMS(const int McScore)
   {
   	if(McScore<mmsMin || McScore>mmsMax) return peMMSi;

      return 0;
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - SET THE Player
//---------------------------------------------------------------------------
namespace  PlayerMaster
{
   int MEditPlayerTube::MakePlayer(void)
   {
      using Tours::tsRegClosed;

      using RegMaster::start_new_player;
      using RegMaster::player_round_change;
      using RegMaster::player_rounds_changed;
      using RegMaster::end_player_edit;

      // THE FIELD Player WAS INITIALISED TO THE LOADED PLAYER
      // OR TO A NEW PLAYER WHO WON'T PLAY THIS ROUND
      // WE LEAVE Player ALONE UNTIL THE EDIT IS VALIDATED
      GPlayer player = Player;

   	// THE PLAYER ID
      {
      	const int Report = ValidPid(PlayerID);  if(Report>0) return Report;

        	if(PlayerInfo != (int)Player.Pid)
         	return pePidBad;
		}

      // REGISTRATION
      {
      	if(InvalidReg(Reg)) return peReg;

         // THE TickBox IS DISABLED AFTER CLOSE OF REG, SO LEAVE THE PLAYER State ALONE
         if(GoTour.State < tsRegClosed)
         {
         	if(Reg==rsNoReg) player.State = Tours::psEntered;
            if(Reg==rsReg)   player.State = Tours::psRegistered;
         }
      }

      // NAME
      {
      	const int Report = MakeName(Name, player);

         if(Report>0)return Report;
      }

      // GRADE
      {
      	const int Report = MakeGrade(Grade, player);

         if(Report>0)return Report;

         if(GoTour.State > tsRegClosed)
         	GradeChanged = player.Grade != Player.Grade;
         else
         	GradeChanged = false;
      }

      // CLUB
      {
      	const int Report = MakeClub(Club, player);

         if(Report>0)return Report;
      }

      // LAND
      {
      	const int Report = MakeLand(Land, player);

         if(Report>0)return Report;
      }

      // GROUP
      {
      	int Report = ValidGroup(Group);  if(Report>0) return Report;

         if(!Group.IsEmpty())
         	player.Group = S_s(Group);
         else
            player.Group = string("  ");
      }

      // NOTE
      {
       	int Report = ValidNote(Note);  if(Report>0) return Report;

      	player.Note = S_s(Note);
      }

      // ROUNDS PLAYED
      {
      	for(int r=0; r<Rounds; r++)
      		player_round_change(Playing[r], player.Games[r].Result);
      }

      // INITIAL AND CURRENT MMS
      {
      	const int Report = ValidMMS(MMSi);	if(Report>0)
         	return Report;

         player.MiScore = (ints)MMSi;

         player.McScore = player.McMahonScore(Roundl);
      }

      // EPILOGUE
      if(State == epiNew)
      {
         // SET MM SCORES AND ADD PLAYER TO LISTS
      	start_new_player(player);

         GoTour.AddPlayer(player);

         ++GoTour.NewPid;
      }

      // PLAYER EDIT MIGHT TRIGGER A GoTour STATE CHANGE
      PlayerInfo = RegMaster::prNone;

      if(GoTour.State > Tours::tsRegClosed)
      	player_rounds_changed(Player, player, PlayerInfo);

      // LOAD THE EDITED PLAYER AND SAVE TO FILE
      end_player_edit(Player,player, PlayerRow, PlayerInfo);

      Player = player;

   	return peNoError;
   }

   string MEditPlayerTube::DuplicationError(void)
   {
   	string message = load_str(IDB_peNameDup);

      int pospid = message.find(string(sQuery), 0);

      message.replace(pospid,1, Tours::pid_string(PlayerDup));

      return message;
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - UPDATING MM SCORES
//---------------------------------------------------------------------------
namespace PlayerMaster
{
	using std::string;
   using std::vector;
   using Tools::ints;

	using Tours::GPlayer;

   using RegMaster::parse_grade;
   using RegMaster::initial_score;

   static bool ValidGradeGroup
   (
   	const String Pgrade, const String Pgroup, GPlayer& player
   )
   {
   	string sgrade = S_s(Pgrade), sgroup = S_s(Pgroup);

      ints vgrade = 0;   char igrp;  int tgrp;

      if(!parse_grade(sgrade, vgrade)) return false;
      if(!parse_group(sgroup, igrp, tgrp)) return false;

      player.Grade = vgrade;  player.Group = sgroup;

      return true;
   }

   void MEditPlayerTube::mmScores
   (
   	const String Pgrade, const String Pgroup, int& mmi, int& mms
   )
   {
   	if(Pgrade.IsEmpty()) { mmi = mms = 0; return; }

      GPlayer player = Player;

      if(!ValidGradeGroup(Pgrade,Pgroup,player)) return;

      initial_score(player);

      mmi = player.MiScore;

      mms = player.McMahonScore(Round);
   }

   void MEditPlayerTube::mcScore
   (
   	const String Pgrade, const String Pgroup, const int mmi, int& mms
   )
   {
      GPlayer player = Player;

      if(!ValidGradeGroup(Pgrade,Pgroup,player)) return;

      player.MiScore = mmi;

      mms = player.McMahonScore(Round);
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - operators
//---------------------------------------------------------------------------
namespace PlayerMaster
{
   MEditPlayerTube& MEditPlayerTube::operator=(const MEditPlayerTube& T)
   {
      State 		= T.State;
      NewBlocked 	= T.NewBlocked;

      Rounds 		= T.Rounds;
      Round 		= T.Round;
      Roundl 		= T.Roundl;

      PlayerCount = T.PlayerCount;
      PlayerRow 	= T.PlayerRow;
      PlayerCol 	= T.PlayerCol;
      Player 		= T.Player;

      TourName 	= T.TourName;

      PlayerID 	= T.PlayerID;

      Name 			= T.Name;
      Club 			= T.Club;
      Land 			= T.Land;
      Note 			= T.Note;

      Grade 		= T.Grade;
      Group 		= T.Group;

      PlayerInfo 	= T.PlayerInfo;
      PlayerDup 	= T.PlayerDup;

      Reg 			= T.Reg;

      MMSi 			= T.MMSi;
      MMSc 			= T.MMSc;

      Playing 		= T.Playing;

      GradeEnabled 	= T.GradeEnabled;
      MMSiEnabled 	= T.MMSiEnabled;
      GradeChanged 	= T.GradeChanged;

      return *this;
   }
}

//---------------------------------------------------------------------------
// THE EditPlayerTube - THE ONE AND ONLY TUBE
//---------------------------------------------------------------------------
namespace PlayerMaster
{
   MEditPlayerTube EditPlayerTube;
   MEditPlayerTube SavedPlayerTube;
}

bool RegClosed(void)
{
   return TourMaster::GoTour.State==Tours::tsRegClosed;
}
//---------------------------------------------------------------------------

