////////////////////////////////////////////////////////////////////////////////
// MODULE PlayerMaster:Interface EditPlayerForm to GTourMaster G.Kaniuk 05/02/08
//

#include <vector>
#include "TourMaster.h"

#ifndef PlayerMasterH
#define PlayerMasterH

namespace PlayerMaster
{
	using std::string;
   using std::vector;
   
	using Tours::min_grade;
   using Tours::pro_grade;
   using Tours::max_rounds;
   using Tours::mc_scale;
   using Tours::GPlayer;

	enum
   {
   	mmsMin = min_grade,
      mmsMax = (pro_grade + 9*mc_scale) + mc_scale*max_rounds
   };

	enum				// States of the EditPlayerForm
   {
      epiVirgin,  // State when form is invisible - at construction or after closing
   	epiNew,		// Entering new player
      epiExists,	// Editing known player
   };

   enum				// Data fields for each round corresponding
   {					// to column numbers in the GameGrid
   	pfRound,  pfPlaying, pfOpp,
      pfColour, pfUpDn,
      pfStones, pfKomi, pfResult,
      pfFields
   };

   enum				// Errors arising from MakePlayer
   {
   	peNoError,
      pePidZero, pePidBig, pePidDel, pePidBad,
      peReg,
      peName, peNameDup,
      peGrade, peClub, peLand, peGroup,
      peGroupForbid, peGroupSpecial, peGroupSuper, peGroupOther,
      peNote,
      peMMSi,
      peCount
   };

	class MEditPlayerTube
   {
   	private:
         void LoadNewPlayer(void);
         void LoadExistingPlayer(const size_t row);

         int MakeName(String& Name, GPlayer& player);
         int MakeGrade(String& Grade, GPlayer& player);
   		int MakeClub(String& Club, GPlayer& player);
			int MakeLand(String& Land, GPlayer& player);

   	public:
      	int State;										// The epiState of the form

         bool NewBlocked;								// New players cannot be added

      	int Rounds, Round, Roundl;					// From GTourMaster

         int PlayerCount;								// Total number of players
         int PlayerRow;									// Row number of loaded player
         int PlayerCol;									// Column number of loaded player

         GPlayer Player;								// The current player

         String TourName;								// In Title bar

         String PlayerID;								// This is a search key - not an index
         String Name, Club, Land, Note;  			// All in combo's needing data for lists

         String Grade, Group;							// These combos generate their own lists
         int GroupError;                        // Detailed group errors

         int PlayerInfo;								// User reports
         int PlayerDup;									// Player duplicated

         int Reg;											// Registered state of player

         int MMSi;										// Initial McMahon score
         int MMSc;										// Current McMahon score

         std::vector<bool>Playing;					// Whether playing in a particular round

         bool GradeEnabled;							// Can change grade
         bool MMSiEnabled;								// Can change initial MMS
         bool GradeChanged;							// The grade was changed - report to user

      public:
      	MEditPlayerTube(void);

         void LoadTour(void);

         void LoadPlayerIDs(TStrings* ids);

         void LoadListsFromAlgo
         (
         	TStrings* ids, TStrings* grades, TStrings* groups
         );

         void LoadRegionFromFile
   		(
      		const int codesize, const String& filename,
      		TStrings* region
   		);

         void LoadRegionsFromFile(TStrings* clubs,TStrings* lands);
         void LoadPlayersFromFile(TStrings* players);

         void UpdateLists
         (
            TStrings* ids,   TStrings* names,
            TStrings* clubs, TStrings* lands,
            TStrings* notes
         );

         void UpdateListsFromTour
         (
            TStrings* ids,   TStrings* names,
            TStrings* clubs, TStrings* lands,
            TStrings* notes
         );

         String PidText(void);

         String cMMSText(void);
         String iMMSText(const int mms);
         String TieLabel(const int level);
         String TieText(const int level);
         String ColBalText(void);
         String UpDnBalText(void);
         String UnevenText(void);
         String GameText(int r, int c);

         void LoadPlayer(const size_t row);
         void LoadPlayer(const size_t col, const size_t row);

         bool InvalidReg(const int reg);

         int ValidPid(String& Pid);
         int ValidName(String& NameF);
         int ValidGrade(String& GradeF);
   		int ValidClub(String& ClubF);
         int ValidLand(String& LandF);
   		int ValidGroup(String& GroupF);
      	int ValidNote(String& NoteF);
         int ValidMMS(const int McScore);

         int MakePlayer(void);

         std::string DuplicationError(void);

         void MEditPlayerTube::mmScores
         (
            const String Pgrade, const String Pgroup, int& mmi, int& mms
         );

         void MEditPlayerTube::mcScore
         (
            const String Pgrade, const String Pgroup, const int mmi, int& mms
         );

         MEditPlayerTube& operator=(const MEditPlayerTube& T);
   };

   extern MEditPlayerTube EditPlayerTube;
   extern MEditPlayerTube SavedPlayerTube;
}
//---------------------------------------------------------------------------
#endif
