//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "GoPrefsRES.h"
#include "PrefsIDH.h"

#include "Strings.h"
#include "Etour.h"

#include "GoDrawGUI.h"

#include "PrinterAPI.h"
#include "PrefForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------

using Tools::load_str;
using Tools::S_s;

using ToolsGUI::ShowMessage;
using ToolsGUI::erOperr;

static void Throw(ToolsGUI::EType etype, short idevent, short idmsg)
{
   ToolsGUI::ThrowError(etype, IDE_PrefForm, idevent, idmsg);
}
//---------------------------------------------------------------------------

static Graphics::TBitmap* NewGlyph(const int Width, const int Height, const TColor Colour)
{
	Graphics::TBitmap* Patch = new Graphics::TBitmap();

   Patch->Width = Width;  Patch->Height = Height;

   Patch->Canvas->Brush->Color = Colour;
   Patch->Canvas->Pen->Color = clBlack;

   TRect Area(0,0, Height-1, Height-1);	Patch->Canvas->Rectangle(Area);

   Patch->TransparentColor = Patch->Canvas->Pixels[Height-1][Height-1];

   return Patch;
}

static void SetPatchColour(const TColor Colour, Graphics::TBitmap* Patch)
{
   Patch->Canvas->Brush->Color = Colour;
   Patch->Canvas->Pen->Color = clBlack;

   TRect Area(0,0, Patch->Height-1, Patch->Height-1);	Patch->Canvas->Rectangle(Area);
}
//---------------------------------------------------------------------------

__fastcall TPrefFrm::TPrefFrm
(
   TComponent* Owner,

   PrefsMaster::GPrefsTube& prefs
): TForm(Owner), PrefsTube(prefs)
{
	PrefPageControl->ActivePageIndex = 0;

   // HELP SETTINGS
  	HelpFile 				  	= GoDraw::HelpFilePath(String("Preferences.hlp"));

   WishMemo->HelpContext 	= IDH_Memo;
   FontBox->HelpContext 	= IDH_PrFont;
   DrawBox->HelpContext 	= IDH_PrName;
   RankBox->HelpContext 	= IDH_PrRankList;
   ColourGrp->HelpContext 	= IDH_ClGrids;
   BackupGrp->HelpContext  = IDH_Backup;
   BehaveGrp->HelpContext  = IDH_Behaviour;

   // MEMO
   String PrefName = PrefsTube.AppPath + PrefsMaster::WishFile;

   if(FileExists(PrefName))
      WishMemo->Lines->LoadFromFile(PrefName);
   else
      WishMemo->Lines->Strings[0] = LoadStr(IDU_WishList);

   // BACKUP
   DriveBox->DirList = BackupPathList;

   if(DirectoryExists(PrefsTube.BackupPath))
   {
   	DriveBox->Drive           = PrefsTube.BackupPath[1];
   	BackupPathList->Directory = PrefsTube.BackupPath;
   }
   else
   {
      DriveBox->Drive           = GoDraw::AppPath[1];
   	BackupPathList->Directory = GoDraw::AppPath;
   }

   // COLOUR
   {
   	const int clWidth = 14, clHeight = 14;

   	ColorDlg->Color = TColor(0);

      clGrid  = PrefsTube.PaneColor;
      clEdit  = PrefsTube.EditColor;
      clSleep = PrefsTube.SleepColor;
      clEntry = PrefsTube.EntryColor;

      GridPatch  = NewGlyph(clWidth, clHeight, clGrid);   GridColBtn->Glyph = GridPatch;
      EditPatch  = NewGlyph(clWidth, clHeight, clEdit);   EditColBtn->Glyph = EditPatch;
      SleepPatch = NewGlyph(clWidth, clHeight, clSleep);  SleepColBtn->Glyph = SleepPatch;
      EntryPatch = NewGlyph(clWidth, clHeight, clEntry);  EntryColBtn->Glyph = EntryPatch;
   }

   // SCREEN - DRAW NAME FORMAT
   switch(PrefsTube.ScreenNameStyle)
   {
   	case PrefsMaster::dpSufMMS:	ScrMMSRadio->Checked=true;   break;
   	case PrefsMaster::dpSufGrade: ScrGradeRadio->Checked=true; break;
   	case PrefsMaster::dpSufNone:  ScrNameRadio->Checked=true;  break;

      default: ScrMMSRadio->Checked=false;
      			ScrGradeRadio->Checked=true;
      			ScrNameRadio->Checked=false;
   }

   // SCREEN - DRAW COMMENTS
   FlagLandBox->Checked	= PrefsTube.ScreenFlagLand;
   FlagClubBox->Checked	= PrefsTube.ScreenFlagClub;

   // PRINT - DRAW NAME FORMAT
   {
   	if(PrefsTube.NameStyle==PrefsMaster::dpSufMMS)
      	MMSRadio->Checked=true;

   	if(PrefsTube.NameStyle==PrefsMaster::dpSufGrade)
      	GradeRadio->Checked=true;

   	if(PrefsTube.NameStyle== PrefsMaster::dpSufNone)
      	NameRadio->Checked=true;
   }


   // PRINT - RANKLIST REGION STYLE
   {
   	ClubBox->ItemIndex = PrefsTube.ClubStyle;
      LandBox->ItemIndex = PrefsTube.LandStyle;
   }

   // PRINTERFONT
   {
      using namespace PrinterAPI;

      using Tools::S_s;

      try
      {
      	StartPrinter();
         	SysPrinterFont = String();
         	SelectFontName(&SysPrinterFont);
         	ListFonts(FontBox->Items);
      	EndPrinter();

         int Index = FontBox->Items->IndexOf(prefs.PrFont);

         SysFontIndex = FontBox->Items->IndexOf(SysPrinterFont);

         if(Index<0)
            if(SysFontIndex<0)
            	ShowMessage(erOperr, load_str(IDU_MissingSystemFont));
            else
            {
            	Index =  SysFontIndex;
               ShowMessage(erOperr, load_str(IDU_MissingFont));
            }
            
         FontBox->ItemIndex = Index;
      }
      catch(...)
      {
      	FontBox->Clear();
         FontBox->Enabled = false;
         ShowMessage(erOperr, load_str(IDU_NoFonts));
      }
   }

   // BEHAVIOUR CHECK BOXES
   {
   	ReuseWindowBox->Checked = PrefsTube.ReuseWindow;

   }
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::PrFactoryBtnClick(TObject *Sender)
{
	if(SysFontIndex>=0)
   	FontBox->ItemIndex = SysFontIndex;
   else
   	ShowMessage(erOperr, load_str(IDU_MissingSystemFont));
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::OkBtnClick(TObject *Sender)
{
   String PrefName = PrefsTube.AppPath + PrefsMaster::WishFile;

   if(FileExists(PrefName))
      WishMemo->Lines->SaveToFile(PrefName);
   else
      WishMemo->Lines->Strings[0] = LoadStr(IDU_WishList);

   // PRINTER
   PrefsTube.PrFont 		= FontBox->Items->Strings[FontBox->ItemIndex];

   PrefsTube.ClubStyle  = ClubBox->ItemIndex;
   PrefsTube.LandStyle	= LandBox->ItemIndex;

   if(MMSRadio->Checked)PrefsTube.NameStyle   = PrefsMaster::dpSufMMS;
   if(GradeRadio->Checked)PrefsTube.NameStyle = PrefsMaster::dpSufGrade;
   if(NameRadio->Checked)PrefsTube.NameStyle  = PrefsMaster::dpSufNone;

   //SCREEN
   PrefsTube.ScreenFlagClub   = FlagClubBox->Checked;
   PrefsTube.ScreenFlagLand   = FlagLandBox->Checked;

   if(ScrMMSRadio->Checked)PrefsTube.ScreenNameStyle   = PrefsMaster::dpSufMMS;
   if(ScrGradeRadio->Checked)PrefsTube.ScreenNameStyle = PrefsMaster::dpSufGrade;
   if(ScrNameRadio->Checked)PrefsTube.ScreenNameStyle  = PrefsMaster::dpSufNone;

   // GRID COLOURS
   PrefsTube.PaneColor = clGrid;
   PrefsTube.EditColor = clEdit;
   PrefsTube.SleepColor = clSleep;
   PrefsTube.EntryColor = clEntry;

   // BACKUP
   PrefsTube.BackupPath = BackupPathList->Directory;

   // BEHAVIOUR
   PrefsTube.ReuseWindow = ReuseWindowBox->Checked;
   
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::CancelBtnClick(TObject *Sender)
{
   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------


void __fastcall TPrefFrm::DrawFactBtnClick(TObject *Sender)
{
   MMSRadio->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::RankFactBtnClick(TObject *Sender)
{
	ClubBox->ItemIndex = PrefsMaster::rcClubName;
   LandBox->ItemIndex = PrefsMaster::rlLandCode;
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::ScrDrawFactBtnClick(TObject *Sender)
{
	ScrMMSRadio->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::GridColBtnClick(TObject *Sender)
{
   ColorDlg->Color = clGrid;

   if(ColorDlg->Execute())
   {
   	clGrid = ColorDlg->Color;

   	SetPatchColour(clGrid, GridColBtn->Glyph);
   }
}

void __fastcall TPrefFrm::EditColBtnClick(TObject *Sender)
{
   ColorDlg->Color = clEdit;

   if(ColorDlg->Execute())
   {
   	clEdit = ColorDlg->Color;

   	SetPatchColour(clEdit, EditColBtn->Glyph);
   }
}

void __fastcall TPrefFrm::SleepColBtnClick(TObject *Sender)
{
   ColorDlg->Color = clSleep;

   if(ColorDlg->Execute())
   {
   	clSleep = ColorDlg->Color;

   	SetPatchColour(clSleep, SleepColBtn->Glyph);
   }
}

void __fastcall TPrefFrm::EntryColBtnClick(TObject *Sender)
{
   ColorDlg->Color = clEntry;

   if(ColorDlg->Execute())
   {
   	clEntry = ColorDlg->Color;

   	SetPatchColour(clEntry, EntryColBtn->Glyph);
   }
}

void __fastcall TPrefFrm::ColourFactBtnClick(TObject *Sender)
{
   clGrid  = PrefsMaster::defPaneColor;
   clEdit  = PrefsMaster::defEditColor;
   clSleep = PrefsMaster::defSleepColor;
   clEntry = PrefsMaster::defEntryColor;

	SetPatchColour(clGrid,  GridColBtn->Glyph);
   SetPatchColour(clEdit,  EditColBtn->Glyph);
   SetPatchColour(clSleep, SleepColBtn->Glyph);
   SetPatchColour(clEntry, EntryColBtn->Glyph);
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::HelpBtnClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT,  IDH_SMPREFS); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::FormClose(TObject *Sender, TCloseAction &Action)
{
	delete GridPatch;
   delete EditPatch;

	try{ Application->HelpCommand(HELP_QUIT, 0); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::BehaveFactoryBtnClick(TObject *Sender)
{
	ReuseWindowBox->Checked = PrefsMaster::defReuseWindow;
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::OnChangePath(TObject *Sender)
{
	// THIS IS TRIGGERED WHEN BackupPathList IS CHANGED
   // This is NOT triggered when an invalid drive was chosen
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::OnClickPath(TObject *Sender)
{
	String PathSelected = BackupPathList->GetItemPath(BackupPathList->ItemIndex);

   BackupPathList->Directory = PathSelected;
}
//---------------------------------------------------------------------------

void __fastcall TPrefFrm::OnChangeDrive(TObject *Sender)
{
	// This does NOT get triggered if you choose an invalid drive
}
//---------------------------------------------------------------------------

