////////////////////////////////////////////////////////////////////////////////
// MODULE PrefsMaster:  Interface Preferences to GTourMaster  G.Kaniuk 07/06/04
//

#include <vcl.h>
#pragma hdrstop

#include "GoErrmsgRES.h"
#include "PrinterRES.h"
#include "GoPrefsRES.h"

#include "ETour.h"
#include "GraphicsAPI.h"
#include "GoDrawGUI.h"
#include "GoDrawGeom.h"
#include "PrefsMaster.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace PrefsMaster										// PREFERENCE DEFAULTS
{
   const TColor defPaneColor   = clWindow;						// Grid color
   const TColor defEditColor   = clYellow;						// Edit field background
   const TColor defSleepColor  = clInactiveCaptionText;		// Not playing
   const TColor defEntryColor  = clActiveCaption;				// NewPlayer
   const TColor defRegSelColor = clHighlight;					// Reg field background color when selected

   const int defClubStyle = rcClubName;							// Show clubs in ranklist
   const int defLandStyle = rlLandCode;							// Show country code in ranklist
   const int defNameStyle = dpSufMMS;								// Suffix is (MMS)

   const int defClubFormat = rcClubName;							// Interpret club field input as full clubname
   const int defLandFormat = rlLandName;

   const bool defReuseWindow = false;								// Start up in screen centre
   const bool defFlagLand    = false;								// Ignore same country pairing comments
   const bool defFlagClub    = true;								// Include same club pairing comments

   const String WishFile    = "\\GoWish.txt";
   const String defLandName = "UK";
}
//---------------------------------------------------------------------------

namespace PrefsMaster			// PREFERENCE CLASS - SETTING
{
   GPrefsTube::GPrefsTube()
   {
   	AppPath     = GoDraw::AppPath;
      BackupPath  = String();

      PaneColor   = defPaneColor;
      EditColor   = defEditColor;
      SleepColor  = defSleepColor;
      EntryColor  = defEntryColor;
      RegSelColor = defRegSelColor;

      PrFont    = String("Arial"); //LoadStr(IDP_Arial);
      TourFile  = String();
                               			//PRINT
      ClubStyle = defClubStyle;
      LandStyle = defLandStyle;
      NameStyle = defNameStyle;
      ClubFormat  = defClubFormat;
      LandFormat  = defLandFormat;

      ScreenFlagClub  = defFlagClub;	// DISPLAY
      ScreenFlagLand  = defFlagLand;
      ScreenNameStyle = defNameStyle;

      DefLandName = defLandName;

      ReuseWindow = defReuseWindow;

      WindowLocn = TRect(0,0, ::minScreenW, ::minScreenH);
      
      AutoBackup    = false;
      JoiningPrompt = false;
   }

   String GPrefsTube::ValueOf(int item, TStringList* strlist)
   {
   	String Item = LoadStr(item);	if(Item.IsEmpty()) return String();

      return strlist->Values[Item];
   }

   void GPrefsTube::SetColor(String Value, TColor& Color)
   {
   	if(Value.IsEmpty())
      	return;

   	int ColorOfValue = Value.ToInt();

      if(Tools::siBounds(clMin,clMax)(ColorOfValue))
      	Color = (TColor)ColorOfValue;
   }

   void GPrefsTube::SetPrinterFont(String Value)
   {
   	if(!Value.IsEmpty())
      	PrFont = Value;
   }

   void GPrefsTube::SetTourFile(String Value)
   {
   	using Tools::sCrLf;

   	if(Value.IsEmpty()){ TourFile = String();	return; }

      if(!FileExists(Value))
      {
      	String MissingFile = LoadStr(IDU_PrefsMissing) + String(sCrLf);
         MissingFile += 		Value                     + String(sCrLf);
         MissingFile +=       LoadStr(IDU_PrefsManual);

      	ShowMessage(ToolsGUI::erOperr, Tools::S_s(MissingFile));

         TourFile = String();
      }
      else
      	TourFile = Value;
   }

   void GPrefsTube::SetClubStyle(String Value, int& value)
   {
   	value = defClubStyle;	if(Value.IsEmpty()) return;

      int StyleOfValue = Value.ToInt();

      if(Tools::siBounds(rcNoClub,rcClubName)(StyleOfValue))
      	value = StyleOfValue;
   }

   void GPrefsTube::SetLandStyle(String Value, int& value)
   {
   	value = defLandStyle;	if(Value.IsEmpty()) return;

      int StyleOfValue = Value.ToInt();

      if(Tools::siBounds(rlNoLand,rlLandName)(StyleOfValue))
      	value = StyleOfValue;
   }

   void GPrefsTube::SetNameStyle(String Value, int& value)
   {
   	value = defNameStyle;	if(Value.IsEmpty()) return;

      int StyleOfValue = Value.ToInt();

      if(Tools::siBounds(dpSufMMS,dpSufNone)(StyleOfValue))
      	value = StyleOfValue;
   }

   void GPrefsTube::SetBool(String Value, bool& TruthValue)
   {
   	TruthValue = false;	if(Value.IsEmpty()) return;

      TruthValue = Value.ToInt() != 0;
   }

   void GPrefsTube::SetBackupPath(String Value)
   {
   	BackupPath = Value;  // Analyse when actually backing up
   }

   void GPrefsTube::SetWindowRect(String Value)
   {
   	using std::string;
      using Tools::S_s;
      using Tools::next_int;

      // THE DEFAULT IS THE MINIMUM DESIGNED SCREEN
      WindowLocn = TRect(0,0, ::minScreenW, ::minScreenH);

      // THE ACTUAL SCREEN
      int swaT, swaL, swaB, swaR;
      ToolsGUI::GetScreenWorkingArea(swaT,swaL,swaB,swaR);

      // THE DESIRED FORM LOCATION
      string value = Tools::S_s(Value);

      string::iterator vs = value.begin(), ve = value.end();

      int top, left, bottom, right;

      if(!next_int(vs, ve, left))   return;
      if(!next_int(vs, ve, top))    return;

      if(!next_int(vs, ve, right))  return;
      if(!next_int(vs, ve, bottom)) return;

      if(top==-1 && bottom==-1 && left==-1 && right==-1)
      {
      	WindowLocn = TRect(-1,-1,-1,-1);

         return;
      }

      // ENSURE REQUIRED FORM FITS
      if(bottom<=top || top<swaT || bottom>swaB) return;
      if(right<=left || left<swaL || right>swaR) return;

      // WE CAN NOW ACCEPT THE VALUES PARSED FROM THE STRING
      WindowLocn = TRect(left, top, right, bottom);
   }

   String GPrefsTube::WindowRect(void)
   {
   	String Space(" ");

      String sWindowRect =  String(WindowLocn.left) + Space + String(WindowLocn.top)
      											 				 + Space +
      				  			String(WindowLocn.right) + Space + String(WindowLocn.bottom);

      return sWindowRect;
   }

   bool GPrefsTube::NormalWindow(void)
   {
   	if(WindowLocn.left < 0)  return false;
      if(WindowLocn.right < 1) return false;
      if(WindowLocn.top < 0) 	 return false;
      if(WindowLocn.bottom < 1)return false;

      return true;
   }

}
//---------------------------------------------------------------------------

namespace PrefsMaster			  // SAVING AND LOADING
{
   void GPrefsTube::SaveToFile()
   {
   	const String EQ = String("=");

      TStringList* List = 0;

   	try
      {
      	List = new TStringList();

	      List->Sorted = true;

	      List->Add(LoadStr(IDU_PaneColor)+EQ+String(PaneColor));
	      List->Add(LoadStr(IDU_EditColor)+EQ+String(EditColor));
 	      List->Add(LoadStr(IDU_SleepColor)+EQ+String(SleepColor));
	      List->Add(LoadStr(IDU_EntryColor)+EQ+String(EntryColor));

	      List->Add(LoadStr(IDU_PrintFont)+EQ+PrFont);
	      List->Add(LoadStr(IDU_TourFile)+EQ+TourFile);

	      List->Add(LoadStr(IDU_ClubStyle)+EQ+String(ClubStyle));
	      List->Add(LoadStr(IDU_LandStyle)+EQ+String(LandStyle));
         List->Add(LoadStr(IDU_NameStyle)+EQ+String(NameStyle));

	      List->Add(LoadStr(IDU_ScreenFlagClub)+EQ+String((int)ScreenFlagClub));
	      List->Add(LoadStr(IDU_ScreenFlagLand)+EQ+String((int)ScreenFlagLand));
         List->Add(LoadStr(IDU_ScreenNameStyle)+EQ+String(ScreenNameStyle));

         List->Add(LoadStr(IDU_JoiningPrompt)+EQ+String((int)JoiningPrompt));
         List->Add(LoadStr(IDU_AutoBackup)+EQ+String((int)AutoBackup));
         List->Add(LoadStr(IDU_ReuseWindow)+EQ+String((int)ReuseWindow));

         List->Add(LoadStr(IDU_WindowRect)+EQ+WindowRect());
         List->Add(LoadStr(IDU_BackupPath)+EQ+BackupPath);

         List->SaveToFile(GoDraw::FilePath(GoDraw::GoDrawIni));

         delete List;
      }
      catch(...)
      {
      	delete List;

         ToolsGUI::ShowMessage(ToolsGUI::erBug, Tools::load_str(IDE_NoMemory));
      }
   }

   void GPrefsTube::LoadFromFile()
   {
   	if(FileExists(GoDraw::FilePath(Tools::s_S(GoDraw::GoDrawIni))))
      {
      	TStringList* List = 0;

         try
         {
            List = new TStringList();

            List->LoadFromFile(GoDraw::FilePath(GoDraw::GoDrawIni));

            SetColor(ValueOf(IDU_PaneColor,List), PaneColor);
            SetColor(ValueOf(IDU_EditColor,List), EditColor);
            SetColor(ValueOf(IDU_SleepColor,List), SleepColor);
            SetColor(ValueOf(IDU_EntryColor,List), EntryColor);

            SetPrinterFont(ValueOf(IDU_PrintFont,List));
            SetTourFile(ValueOf(IDU_TourFile,List));

            SetClubStyle(ValueOf(IDU_ClubStyle,List), ClubStyle);
            SetLandStyle(ValueOf(IDU_LandStyle,List), LandStyle);
            SetNameStyle(ValueOf(IDU_NameStyle,List), NameStyle);

            SetBool(ValueOf(IDU_ScreenFlagClub,List), ScreenFlagClub);
            SetBool(ValueOf(IDU_ScreenFlagLand,List), ScreenFlagLand);
            SetNameStyle(ValueOf(IDU_ScreenNameStyle,List), ScreenNameStyle);

            SetBool(ValueOf(IDU_JoiningPrompt,List), JoiningPrompt);
            SetBool(ValueOf(IDU_AutoBackup,List), AutoBackup);
            SetBool(ValueOf(IDU_ReuseWindow,List), ReuseWindow);

            SetWindowRect(ValueOf(IDU_WindowRect, List));
            SetBackupPath(ValueOf(IDU_BackupPath,List));

            delete List;
         }
         catch(...)
         {
         	delete List;

            ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDE_BadPrefs));
         }
      }
      else
      	SaveToFile();
   }

   void GPrefsTube::FinishTourFile(void)
   {
   	TourFile = String();

      SaveToFile();
   }

   GPrefsTube PrefsTube;          // Preferences
}

//------------------------------------------------------------------------------


 