////////////////////////////////////////////////////////////////////////////////
// MODULE PrefsMaster:  Interface Preferences to GTourMaster  G.Kaniuk 07/06/04
//

#ifndef PrefsMasterH
#define PrefsMasterH

namespace PrefsMaster
{
	enum
   {
      rcNoClub, rcClubCode, rcClubName
   };

   enum
   {
      rlNoLand, rlLandCode, rlLandName
   };

   enum
   {
   	dpSufMMS, dpSufGrade, dpSufNone
   };

   class GPrefsTube          			// GLOBALLY AVAILABLE USER SETTINGS
   {
   	public:
         String AppPath;				// Application program path contains prefs files
         String BackupPath;         // Manual backup path
         String TourFile;        	// Empty at virgin start
         String DefLandName;			// Default country name

         TColor PaneColor;       	// Input colour in main form
         TColor EditColor;				// Edit field background
         TColor SleepColor;			// Rounds field not playing
         TColor EntryColor;			// New player on last line of register
         TColor RegSelColor;			// Reg field selected colour

         String PrFont;          	// Supplied with Arial

         									// PRINTING
         int  ClubStyle;     			// Club control in Ranklist  print
         int  LandStyle;     			// Country control in Ranklist print
         int  NameStyle;				// Player name format in Draw/Results print

         int  ClubFormat;				// Whether club input is full name or code
         int  LandFormat;				// Whether country input is full name or code

         									// ON-SCREEN DISPLAY
         bool ScreenFlagClub;      	// Same club comment
         bool ScreenFlagLand;      	// Same country comment
         int  ScreenNameStyle;		// Player name format in Draw/Upset display


         bool AutoBackup;				// Automatic backup of tour file only at critical points
         bool ReuseWindow;				// Re use window settings
         TRect WindowLocn;          // Main screen window location

         bool JoiningPrompt;			// Prompt at draw if players joining this round

         GPrefsTube();

         void LoadFromFile(void);				// If no file, create standard defaults
      	void SaveToFile(void);
         
         void FinishTourFile(void);

         bool NormalWindow(void);				// Maximised when LocateWindow = (-1,-1,-1,-1)

		private:
         String ValueOf(int item, TStringList* strlist);

         void SetColor(String Value, TColor& Color);
         void SetPrinterFont(String Value);
         void SetTourFile(String Value);
         void SetClubStyle(String Value, int& value);
         void SetLandStyle(String Value, int& value);
         void SetNameStyle(String Value, int& value);

         void SetBackupPath(String Value);
         void SetWindowRect(String Value);

         void SetBool(String Value, bool& TruthValue);

         String WindowRect(void);
   };

   extern const String WishFile;				// Users wish list file

   extern const TColor defPaneColor;      // Grid color
   extern const TColor defEditColor;		// Edit field background
   extern const TColor defSleepColor;		// Rounds not playing
   extern const TColor defEntryColor;		// New player


   extern const int defClubStyle;			// Show clubs in ranklist
   extern const int defLandStyle;			// Don't show country in ranklist
   extern const int defNameStyle;			// MMS form

   extern const bool defReuseWindow;		// Start up in screen centre
   extern const bool defFlagLand;			// Ignore same country pairing comments
   extern const bool defFlagClub;			// Include same club pairing comments

   extern GPrefsTube PrefsTube;				// The one and only
}

//---------------------------------------------------------------------------
#endif
 