////////////////////////////////////////////////////////////////////////////////
// MODULE PrintCard:  PrintCards from CardsPRI.txt 		G.Kaniuk 		14/08/05
//
#define STRICT 			// THIS IS ESSENTIAL TO GET THE CORRECT TYPE FOR HFONT!!!
#include <windows.h>
#include <vcl.h>
#include <string>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "PrRES.h"

#include "Strings.h"
#include "Format.h"

#include "ETour.h"
#include "PrinterAPI.h"
#include "PrintCards.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace PrintCards							// CARD GEOMETRY
{
	using std::string;

	using PrinterAPI::paTopLeft;
   using PrinterAPI::paTopMid;
   using PrinterAPI::paTopRight;
   using PrinterAPI::paMidLeft;
   using PrinterAPI::paCentre;
   using PrinterAPI::paMidRight;
   using PrinterAPI::paBotLeft;
   using PrinterAPI::paBotMid;

   using PrinterAPI::PrTextArea;
   using PrinterAPI::PrSheet;

   using PrinterAPI::pfNormal;

	enum
   {
   	cardFields = 19, maxRounds=12, maxCardItems=61
   };

   struct PrGame					  			// LAYOUT FOR ONE GAME
   {
   	int hpos, vpos;			  			// Top left corner game cell
      int hsize, vsize;    	  			// Dimensions game cell

      PrTextArea aOpponent;	  			// Opponent number
      PrTextArea aSub;       				// Colour
      PrTextArea aSup;						// Direction
      PrTextArea aResult;       			// Same height as opponent

      PrGame()
      {
      	hpos = vpos = 0;
         hsize = vsize = 0;
      }

      PrGame	// Game Template at 0,0
      (
      	const int Hsize, const int Vsize
      );

      PrGame	// Game shifted to Hpos, Vpos
      (
      	const int Hpos, const int Vpos,
         const int Round,
         PrGame& Template
      );

      void displace(const int dh, const int dv);
   };

   PrGame::PrGame			// GAME TEMPLATE AT 0,0
   (
      const int Hsize, const int Vsize
   )
   {
      hsize = Hsize;		vsize = Vsize;					// Game cell size
      hpos  = 0;			vpos	= 0;
      
   	const int hopp = (29*hsize)/48;					// Opponent id
      const int hres = hsize/6;							// Result
      const int hsuff = (25*hres)/32;   				// Dir, colour
      const int hmarg = hsuff/2;							// Left hand in game cell

      // OPPONENT AND RESULT ON SAME LINE
      aOpponent.align = paMidLeft;
      aOpponent.area  = TRect(hmarg, 1, hmarg + hopp-1,  vsize-1);

      aResult.align = paMidLeft;
      aResult.area  = TRect(hmarg+hopp+hsuff, 1, hsize-1, vsize-1);

      // DIRECTION AND COLOUR
      const int vmid = vsize/2;

      aSup.align = paBotMid;
      aSup.area = TRect(hmarg+hopp,      1, hmarg+hopp+hsuff-1, vmid-1);

      aSub.align = paTopMid;
      aSub.area = TRect(hmarg+hopp, vmid+1, hmarg+hopp+hsuff-1, vsize-1);
   }

   PrGame::PrGame								// Game shifted to Game Area
   (
      const int Hpos, const int Vpos,	// Origin of game area
		const int Round,						// Decides position of game
      PrGame& Template						// Template origin is (0,0)
   )
   {
   	const int h = Round%4, 	v = Round/4;

      *this = Template;						// Copy template

      hpos = Hpos + h*Template.hsize;	// New co-ordinates
      vpos = Vpos + v*Template.vsize;

      this->displace(hpos, vpos);		// Move
   }

   void PrGame::displace(const int hstep, const int vstep)
   {
   	aOpponent = aOpponent.displace(hstep,vstep);
      aSup	    = aSup.displace(hstep,vstep);
      aSub	    = aSub.displace(hstep,vstep);
      aResult   = aResult.displace(hstep,vstep);
   }

   class PrCard								// LAYOUT FOR ONE CARD
   {
   	private:
      	PrSheet* csheet;

   	public:
         int roundcount, roundnow;		// Future rounds _____; Non-existent are empty
         int hcount, vcount;				// horizontal, vertical card count
         int hpos, vpos;					// Top left corner
         int hsize, vsize;					// Card dimensions
         int hstride, vstride;			// Steps to next card origin
         int vline;							// Line height
         int hgame;							// Game line width
         int hbals;							// Balances line width

         int normalFH, normalFW;			// Normal font size
         int scriptFH, scriptFW;			// super-sub script font size

         PrTextArea aMMStit,aMMS;		// MMS
         PrTextArea aCbal,aMbal,aMout;	// Balances
         PrTextArea aName, aRegion;		// Player ID
         PrGame Games[maxRounds];		// All rounds

      public:
         // Calculates the template at 0,0
      	PrCard(const int RoundCount, const int RoundNow, PrSheet* CardSheet);

         // Produce a copy of the template shifted to (Hpos,Vpos)
         PrCard(const int Hpos, const int Vpos, const PrCard& Template);

         void DrawBorder(void);
         void DrawText(string* items);
         void DrawText(const PrTextArea posn, string text);
         void DrawGame(const PrGame game, string* items);
         void DrawWaitingGame(const PrGame game);
   };

   PrCard::PrCard(const int RoundCount, const int RoundNow, PrSheet* CardSheet)
   {
   	csheet = CardSheet;

      roundcount = RoundCount;  roundnow = RoundNow;

   	// PHYSICAL ASPECT DETERMINES CARD COUNT AND SIZE
      hcount=0, vcount=0;

      if(csheet->Vsize>=csheet->Hsize)
      {
      	hcount = 3;		vcount = 8;		// Portrait
      }
      else
      {
      	hcount = 3; 	vcount = 4;		// Landscape
      }

      const int harea = csheet->Harea, varea = csheet->Varea;

      hstride = harea/hcount;		vstride = varea/vcount;
      hsize = hstride-8;			vsize = vstride-8;

      // THE MAIN PARTITION
      vline = vsize/5;
      hgame = (9*hsize)/11;
      hbals = (2*hsize)/11;

      // PLAYER ID AREA
      aName.align   = paMidLeft;
      aName.area    = TRect(12,1, hgame-1, vline-1);
      aRegion.area  = TRect(12, vline, hgame-1, 2*vline-1);

      // MMS AREA
      aMMStit.area = TRect(hgame+1, 1, hsize-1, vline-1);
      aMMS.area    = TRect(hgame+1, vline, hsize-1, 2*vline-1);

      // BALANCES
      aCbal.area = TRect(hgame+1, 2*vline, hsize-1, 3*vline-1);
      aMbal.area = TRect(hgame+1, 3*vline, hsize-1, 4*vline-1);
      aMout.area = TRect(hgame+1, 4*vline, hsize-1,   vsize-1);

		// GAMES
      PrGame Template(hgame/4, vline);

      const int hgpos = 0, vgpos = 2*vline;

      for(int r=0; r<maxRounds; r++)
      	Games[r] = PrGame(hgpos,vgpos, r, Template);

      // CALCULATE NORMAL FONT SIZE
      int FontHeight = CardSheet->StdFontHeight;
      int FontWidth  = Template.hsize/8;
      int GamesWidth = 4*8*FontWidth;

      if(FontHeight >= vline-2)
         FontHeight = vline-2;

      if(GamesWidth >= hgame-2)
         FontWidth = ((hgame-2)*FontWidth)/GamesWidth;

      // SUB/SUPER SCRIPT FONT SIZE
      normalFH = FontHeight;			normalFW = FontWidth;
      scriptFH = (5*FontHeight)/8; 	scriptFW = (5*FontWidth)/8;
   }

   PrCard::PrCard(const int Hpos, const int Vpos, const PrCard& Template)
   {
   	*this = Template;

      hpos = Hpos;  vpos = Vpos;					// The step

      // MOVE TEXT AREAS
      aName   = aName.displace(hpos,vpos);
      aRegion = aRegion.displace(hpos,vpos);
      aMMStit = aMMStit.displace(hpos,vpos);
      aMMS    = aMMS.displace(hpos,vpos);
      aCbal   = aCbal.displace(hpos,vpos);
      aMbal   = aMbal.displace(hpos,vpos);
      aMout   = aMout.displace(hpos,vpos);

      // MOVE GAMES
      for(int r=0; r<maxRounds; r++)
      	Games[r].displace(hpos,vpos);
   }

   void PrCard::DrawBorder()
   {
      // HORIZONTAL LINES
      int h = hpos, v = vpos;
      csheet->MoveTo(h, v);  csheet->LineTo(h+hsize-1, v);

      v += 2*vline;
      csheet->MoveTo(h, v);  csheet->LineTo(h+hsize-1, v);

      v += 3*vline;
      csheet->MoveTo(h, v);  csheet->LineTo(h+hsize-1, v);

      // VERTICAL LINES
      h = hpos;  v = vpos;
      csheet->MoveTo(h, v);  csheet->LineTo(h, v+vsize-1);

      h += hgame;
      csheet->MoveTo(h, v);  csheet->LineTo(h, v+vsize-1);

      h += hbals;
      csheet->MoveTo(h, v);  csheet->LineTo(h, v+vsize-1);
   }

   void PrCard::DrawText(const PrTextArea posn, string text)
   {
   	csheet->DrawTextCell(posn.area, posn.align, text);
   }

   void PrCard::DrawGame(const PrGame game, string* items)
   {
   	csheet->NewFont(scriptFH, scriptFW, pfNormal);
      DrawText(game.aSub, 		items[1]);
      DrawText(game.aSup, 		items[2]);

   	csheet->NewFont(normalFH, normalFW, pfNormal);
   	DrawText(game.aOpponent,items[0]);
      DrawText(game.aResult, 	items[3]);
   }

   void PrCard::DrawWaitingGame(const PrGame game)
   {
   	TRect aWait = game.aOpponent.area;

      aWait.right = game.aResult.area.right;

      csheet->DrawTextCell(aWait, paBotLeft, string(6, Tools::cUline));
   }

   void PrCard::DrawText(string* Items)
   {
   	// PLAYER IDENTIFICATION
      DrawText(aName, Items[0]);	DrawText(aRegion, Items[1]);

      // MMS
      DrawText(aMMStit,Items[2]);DrawText(aMMS,Items[3]);

      // BALANCES
      DrawText(aCbal, Items[4]);	DrawText(aMbal, Items[5]); DrawText(aMout, Items[6]);

      // GAMES PLAYED
      for(int r=0; r<roundnow; r++)
      {
      	DrawGame(Games[r], Items+7 + 4*r);
      }

   	// GAMES TO COME
      for(int f=roundnow; f<roundcount; f++)
      	DrawWaitingGame(Games[f]);
   }
}

namespace PrintCards							// PRINTING
{
	using Tools::unpack_tabbed;
   using Tools::to_posint;
   using Tools::S_s;

   static bool AnalyseCardsHeader
   (
   	TStringList* cardsfile, PrSheet& cardsheet,

      int& roundcount, int& round, int& frplayer, int& toplayer
   )
   {
   	//	CARDS	ROUND	5	OF	6	British Go Congress	2000
		//	PLAYERS	1	TO	8
      string CardHead1Raw = S_s(cardsfile->Strings[0]), CardHead2Raw = S_s(cardsfile->Strings[1]);
      string CardHead1[7], CardHead2[4];
      if(unpack_tabbed(CardHead1Raw, 7, CardHead1) != 7) return false;
      if(unpack_tabbed(CardHead2Raw, 4, CardHead2) != 4) return false;

      string sround    = CardHead1[2], sroundcount = CardHead1[4];
      string sfrplayer = CardHead2[1], stoplayer   = CardHead2[3];

      roundcount = to_posint(sroundcount.begin(), sroundcount.end());
		round      = to_posint(sround.begin(), sround.end());

      frplayer   = to_posint(sfrplayer.begin(), sfrplayer.end());
      toplayer   = to_posint(stoplayer.begin(), stoplayer.end());

      return true;
   }

   void PrCardsPage
   (
   	const int iFromPlayer, const int iToPlayer,
      TStringList* CardsFile, PrCard& CardPlate
   )
   {
   	int h=0, v=0;

   	for(int p = iFromPlayer; p<=iToPlayer; p++)
      {
      	string CardRaw = S_s(CardsFile->Strings[p+1]);

         string CardItems[maxCardItems];	for(int i=0; i<maxCardItems; i++) CardItems[i] = string();

         const int ItemCount = unpack_tabbed(CardRaw, maxCardItems, CardItems);

         if(ItemCount > maxCardItems) return;

      	const int hpos = h*CardPlate.hstride,  vpos = v*CardPlate.vstride;

      	PrCard CardNow = PrCard(hpos, vpos, CardPlate);

         CardNow.DrawText(CardItems);

         CardNow.DrawBorder();

         ++h;

         if(h==CardPlate.hcount)
         {
         	h = 0;  ++v;

            if(v==CardPlate.vcount)
            	break;
         }
      }
   }

   
	void PrintCardList(bool PrDlg, String& CardsPri)
   {
   	TStringList* CardsFile = 0;

   	try
      {
         CardsFile = new TStringList();

         CardsFile->LoadFromFile(CardsPri);

         PrinterAPI::PrSheet CardsSheet;

         int RoundCount, RoundNow, FromPlayer, ToPlayer;

         if(!AnalyseCardsHeader(CardsFile, CardsSheet, RoundCount, RoundNow, FromPlayer, ToPlayer))
         	throw(Exception(String()));

         PrCard CardPlate(RoundCount, RoundNow, &CardsSheet);

         const int PlayerCount = ToPlayer - FromPlayer + 1;

         const int CardsPerPage = CardPlate.hcount*CardPlate.vcount;

         int pFrom, pTo;

         pFrom = FromPlayer;
         pTo   = PlayerCount>CardsPerPage ? pFrom+CardsPerPage-1 : ToPlayer;

         CardsSheet.NewFont(CardPlate.normalFH, CardPlate.normalFW, pfNormal);

         PrinterAPI::StartDoc(Tools::load_cstr(IDM_PrintCards));

         while(true)
         {
            PrinterAPI::StartPage();
            	PrCardsPage(pFrom-FromPlayer+1, pTo-FromPlayer+1, CardsFile, CardPlate);
            PrinterAPI::EndPage();

            if(pTo < ToPlayer)
            {
               pFrom = pTo + 1;
               pTo   = pFrom + CardsPerPage-1;

               if(pTo > ToPlayer) pTo = ToPlayer;
            }
            else
            	break;
         }

         PrinterAPI::EndDoc();

         delete CardsFile;
      }
      catch(...)
      {
      	delete CardsFile;

         PrinterAPI::Throw(ToolsGUI::erOperr, IDM_PrintCardsCmd, IDM_NoCardsPrMemory);
      }
   }

}
//---------------------------------------------------------------------------
