////////////////////////////////////////////////////////////////////////////////
// MODULE PrintDrawRes:  Draw or Results from DrawPRI.txt, ResPRI.txt
//								G.Kaniuk 		17/08/05
//
#define STRICT 			// THIS IS ESSENTIAL TO GET THE CORRECT TYPE FOR HFONT!!!
#include <windows.h>
#include <vcl.h>

#pragma hdrstop

#include "GoErrMsgRES.h"
#include "PrRES.h"

#include "Strings.h"
#include "Format.h"

#include "ETour.h"

#include "PrinterAPI.h"
#include "PrintDrawRes.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace PrintDrawRes
{
	using std::string;
   using std::list;

   using Tools::cSpace;
   using Tools::S_s;
   using Tools::to_posint;
   using Tools::to_string;
   using Tools::unpack_tabbed;
   using Tools::spaces;

   using PrinterAPI::paTopLeft;
   using PrinterAPI::paTopMid;
   using PrinterAPI::paTopRight;
   using PrinterAPI::paMidLeft;
   using PrinterAPI::paMidRight;
   using PrinterAPI::paCentre;

   using PrinterAPI::pfBold;
   using PrinterAPI::pfNormal;

   using PrinterAPI::PrSheet;
   using PrinterAPI::PrTextArea;

	enum				// FONT SIZES
   {
   	fsHead = 14, fsLine = 13, fsStd = 11
   };

   enum				// LINE SPACING FACTORS
   {
   	dlFont = 64, duFont = 140, duSpace = 128
   };

	enum				// HEADING STRUCTURE
   {
   	nHeadT = 5, // ROUND	1	DRAW	London Open	2025
      nHeadC = 4, // ROOMS	4	BOARDS	77
      nHeadR = 6, // ROOM	R	BOARDS	1	TO	16
      nHeadD = 4, // BOARD	BLACK	HANDICAP	WHITE
      nBoard = 4	// 66	_	Ken Dackombe(-22)	2 stn_	Zita Galambos(-19)
   };

   struct BoardRange
   {
   	string IdRoom;					// Id of non-empty room
   	int fromBoard, toBoard;		// Range of boards in room
   };

   // FIRST HEADING LINE
   static string strTo("  -  ");
   static string DrRound, DrTour, DrRoom, DrBoards, DrRoomBoards;

   // SECOND HEADING LINE - COLUMN HEADERS
   static string DrBlack, DrKomi, DrWhite;

   // FIELD TEMPLATES
   static PrTextArea aTitle, aTour, aRoombs;	// Fixed areas
   static PrTextArea aBlack, aWhite, aKomi;

   static PrTextArea aBoards[nBoard];			// Items for 1 board

   static int headFontHeight, headFontWidth;	// Requested header font
   static int HeadFontHeight, HeadFontWidth; // Achieved header font
   static int HeadHeight;							// Total Header step

   static int lineFontHeight, lineFontWidth;	// Requested line font
   static int LineFontHeight, LineFontWidth; // Achieved line font
   static int LineHeight,     SpaceHeight;   // Line and space step

   static int AnalysePairHeader
   (
   	TStringList* drfile,

      PrSheet& drsheet,

      list<BoardRange>& rooms,

      int& boardcount, int& headercount, int& batchcount, int& textpos
   )
   {
   	// GET LAYOUT PARAMETERS
      string rawLayout = S_s(drfile->Strings[1]);
      string Layout[nHeadC];
      if(unpack_tabbed(rawLayout, nHeadC, Layout) != nHeadC)
      	return -101;

      DrBoards = Layout[2];

      string DrRoomCount = Layout[1], DrBoardCount = Layout[3];

      const int roomcount  = to_posint(DrRoomCount.begin(), DrRoomCount.end());

      boardcount = to_posint(DrBoardCount.begin(),DrBoardCount.end());

      if(roomcount<1 || boardcount<1)
      	return -102;

      // BUILD THE ROOM LIST
      rooms.clear();

      string RoomBoard[nHeadR], FromBoard, ToBoard;

      int boardsum = 0;

      for(int r=0; r<roomcount; r++)
      {
      	string rawRoom = S_s(drfile->Strings[2+r]);
         if(unpack_tabbed(rawRoom, nHeadR, RoomBoard) != nHeadR)
         	return -103;

         if(r==0) DrRoom = RoomBoard[0];

         FromBoard = RoomBoard[3];  ToBoard = RoomBoard[5];

         BoardRange rBoards;

         rBoards.IdRoom    = RoomBoard[1];

         rBoards.fromBoard = to_posint(FromBoard.begin(), FromBoard.end());
         rBoards.toBoard   = to_posint(ToBoard.begin(),   ToBoard.end());

         boardsum += rBoards.toBoard - rBoards.fromBoard + 1;

         // USE ONLY ENOUGH ROOMS FOR ALL BOARDS
         if(boardsum >= boardcount)
         {
         	rBoards.toBoard -= boardsum - boardcount;

            rooms.push_back(rBoards);

            break;
         }

         // ENSURE ENOUGH BOARDS ALLOCATED
         if(r == roomcount-1)
         	if(boardsum < boardcount)
            	rBoards.toBoard += boardcount - rBoards.toBoard;

         rooms.push_back(rBoards);
      }
      headercount = roomcount + 3;

      // VERTICAL SPACE REQUIRED FOR 13 POINT BOLD HEADINGS
      const int StdFontHeight = drsheet.FontHeight;	// 11 point
      const int StdFontWidth  = drsheet.FontWidth;

      headFontHeight = (fsHead*StdFontHeight)/fsStd;
      headFontWidth  = (fsHead*StdFontWidth)/fsStd;

      drsheet.NewFont(headFontHeight, headFontWidth, pfBold);

      HeadFontHeight = drsheet.FontHeight;
      HeadFontWidth  = drsheet.FontWidth;

      HeadHeight = 5*HeadFontHeight;

      textpos = drsheet.Tmargin + HeadHeight;

      // LINE FONT SIZE: 11pt FONT 100 CHARS PER LINE
      {
      	const int printwidth = drsheet.Rmargin - drsheet.Lmargin;

      	lineFontWidth = (fsLine*StdFontWidth)/fsStd;

         if(100*lineFontWidth >= printwidth)
         	lineFontWidth = printwidth/100;
         
      	lineFontHeight = (fsLine*StdFontHeight)/fsStd;

         drsheet.NewFont(lineFontHeight,lineFontWidth, pfNormal);

         LineFontWidth  = drsheet.FontWidth;
         LineFontHeight = drsheet.FontHeight;
         LineHeight     = (duFont*LineFontHeight)/dlFont;

         SpaceHeight    = (duSpace*LineFontHeight)/dlFont;
      }
      
     	// BATCHES OF 5
      batchcount = drsheet.BatchesPerPage(duFont, dlFont, duSpace, HeadHeight);

   	// TEMPLATE FOR PAIRS ON A BOARD
		const int ActiveWidth = drsheet.Rmargin - drsheet.Lmargin;

      const int wBoardNo = (11*ActiveWidth)/144;
      const int wPlayer  = (50*ActiveWidth)/144;
      const int wKomi    = ActiveWidth - wBoardNo - 2*wPlayer;

      aBoards[0] = PrTextArea(drsheet.Lmargin, wBoardNo, paTopLeft);
      aBoards[1] = aBoards[0] + PrTextArea(0, wPlayer, paTopLeft);
      aBoards[2] = aBoards[1] + PrTextArea(0, wKomi,   paTopLeft);
      aBoards[3] = aBoards[2] + PrTextArea(0, wPlayer, paTopLeft);

      for(int i=0; i<4; i++)
      {
      	aBoards[i].area.top    = textpos;
         aBoards[i].area.bottom = textpos + LineHeight;
      }

      // TEMPLATE FOR FIRST HEADER LINE
      string rawHeadT = S_s(drfile->Strings[0]);
      string HeadT[nHeadT];
      if(unpack_tabbed(rawHeadT, nHeadT, HeadT) != nHeadT) return false;

      DrRound  = HeadT[0] + spaces(1) + HeadT[1];
      DrTour   = HeadT[3] + spaces(1) + HeadT[4];

      const int wRound = (2*ActiveWidth)/12, wRoombs = (6*ActiveWidth/12);
      const int wTour  = ActiveWidth - wRound - wRoombs;

      aTitle  = PrTextArea(drsheet.Lmargin, wRound, paTopLeft);
      aTour   = aTitle + PrTextArea(0, wTour, paTopMid);
      aRoombs = aTour  + PrTextArea(0, wRoombs, paTopRight);

      int Thead = drsheet.Tmargin, Bhead = drsheet.Tmargin + HeadFontHeight;

      aTitle.area.top  = Thead;	aTitle.area.bottom  = Bhead;
      aTour.area.top   = Thead;	aTour.area.bottom   = Bhead;
      aRoombs.area.top = Thead;	aRoombs.area.bottom = Bhead;


      // TEMPLATE FOR COLUMN HEADINGS
      string rawColHead = S_s(drfile->Strings[headercount-1]);
      string ColHead[nHeadD];
      if(unpack_tabbed(rawColHead, nHeadD, ColHead) != nHeadD)
      	return -105;

      DrBlack = ColHead[1];  DrKomi = ColHead[2];	DrWhite = ColHead[3];

      Thead += 3*HeadFontHeight; Bhead += 3*HeadFontHeight;

      aBlack = aBoards[1]; aBlack.area.top = Thead;  aBlack.area.bottom = Bhead;
      aKomi  = aBoards[2]; aKomi.area.top  = Thead;  aKomi.area.bottom  = Bhead;
      aWhite = aBoards[3]; aWhite.area.top = Thead;  aWhite.area.bottom = Bhead;

      return 0;
   }

   static void PrDrawResHead
   (
   	const string IdRoom,
      const int BoardFrom, const int BoardTo,

      PrSheet& DrSheet
   )
   {
   	// ALL HEADER LINES BOLD
      DrSheet.NewFont(headFontHeight, headFontWidth, pfBold);

      const string DrRoomId = DrRoom + spaces(1) + IdRoom;
      const string DrRange = to_string(BoardFrom) + strTo + to_string(BoardTo);

      DrRoomBoards = DrRoomId + spaces(6) + DrBoards + spaces(2) + DrRange;

      aTitle.DrawText(DrRound, DrSheet);
      aTour.DrawText(DrTour, DrSheet);
      aRoombs.DrawText(DrRoomBoards, DrSheet);

      aBlack.DrawText(DrBlack, DrSheet);
      aKomi.DrawText(DrKomi, DrSheet);
      aWhite.DrawText(DrWhite, DrSheet);
   }

   static bool PrBoards
   (
   	const int HeaderCount, const int BatchCount,
      const int BoardFrom,  const int BoardTo,
      const int TextPos,

      TStringList* DrFile, PrSheet& DrSheet
   )
   {
   	// ALL BOARD LINES LARGER FONT
      DrSheet.NewFont(lineFontHeight, lineFontWidth, pfNormal);

      // BATCH CONTROL
      int TextTopNow = TextPos, BatchLine = 0;

      const int nBoardItems = nBoard+2;													// Include separate result fields to be concatenated

      for(int b = BoardFrom; b <= BoardTo; b++)
      {
      	string RawLine = S_s(DrFile->Strings[b-1+HeaderCount]);					// Board numbering starts at 1
         string BoardItems[nBoardItems];
         if(unpack_tabbed(RawLine, nBoardItems, BoardItems) < nBoardItems-1)	// Missing Komi is allowed
         	return false;

         string PairItems[nBoard];

         PairItems[0] = BoardItems[0];
         PairItems[1] = BoardItems[1] + spaces(2) + BoardItems[2];
         PairItems[2] = BoardItems[3];
         PairItems[3] = BoardItems[4] + spaces(2) + BoardItems[5];

         for(int i=0; i<nBoard; i++)
         {
         	aBoards[i].area.top 	 = TextTopNow;
            aBoards[i].area.bottom = TextTopNow + LineHeight;

            aBoards[i].DrawText(PairItems[i], DrSheet);
         }

         ++BatchLine;

         if(BatchLine == 5)
         {
         	BatchLine = 0;

         	TextTopNow += LineHeight+SpaceHeight;
         }
         else
         	TextTopNow += LineHeight;
      }
      return true;
   }


   void PrintPairing(bool PrDlg, String PairPri)
   {
   	TStringList* PairFile = 0;

   	try
      {
         PairFile = new TStringList();

         PairFile->LoadFromFile(PairPri);

         PrinterAPI::PrSheet PairSheet;

         int BoardCount, HeaderCount, BatchCount, TextPos;

         list<BoardRange>RoomList;

         const int repHeader = AnalysePairHeader(PairFile, PairSheet, RoomList, BoardCount, HeaderCount, BatchCount,TextPos);

         if(repHeader<0)
         {
         	string errMsg = string("ERROR ")+ Tools::to_string(repHeader);
            
            ToolsGUI::ShowMessage(ToolsGUI::erBug, errMsg);

         	PrinterAPI::Throw(ToolsGUI::erBug, IDM_PrintDrawCmd, IDM_NoDrawPrHeader);
         }

         const int BoardsPerPage = 5*BatchCount;

         list<BoardRange>::iterator RoomNow = RoomList.begin();
         list<BoardRange>::iterator RoomEnd = RoomList.end();

      	PrinterAPI::StartDoc(Tools::load_cstr(IDM_PrintDrawRes));

         while(RoomNow != RoomEnd)
         {
         	const int fromBoard = RoomNow->fromBoard, toBoard = RoomNow->toBoard;

            const int BoardsInRoom = toBoard - fromBoard + 1;

            int fBoard, tBoard, BoardsPrinted = 0;

            fBoard = fromBoard;
            tBoard = fromBoard + BoardsPerPage - 1;

            if(BoardsInRoom < BoardsPerPage) tBoard = toBoard;

            while(BoardsPrinted < BoardsInRoom)
            {
               PrinterAPI::StartPage();

               PrDrawResHead(RoomNow->IdRoom, fBoard, tBoard, PairSheet);

               if(!PrBoards(HeaderCount, BatchCount, fBoard, tBoard, TextPos, PairFile, PairSheet))
                  PrinterAPI::Throw(ToolsGUI::erBug, IDM_PrintDrawCmd, IDM_NoDrawPrBoards);

               PrinterAPI::EndPage();

               BoardsPrinted += tBoard - fBoard + 1;

               fBoard += BoardsPerPage;
               tBoard += BoardsPerPage;

               if(BoardsInRoom-BoardsPrinted<BoardsPerPage)				// Short page
               {
               	tBoard = toBoard;		if(tBoard < fBoard) break;		// Insulation
               }
            }

            ++RoomNow;
         }

         PrinterAPI::EndDoc();

         delete PairFile;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintDrawCmd, IDM_NoDrawPrMemory);
      }
      catch(...)
      {
      	delete PairFile;

         PrinterAPI::Throw(ToolsGUI::erOperr, IDE_LoadFromFile, IDE_NoMemory);
      }
   }

}
//---------------------------------------------------------------------------



