////////////////////////////////////////////////////////////////////////////////
// MODULE PrintMUI:  Interface to printing system 		G.Kaniuk 		12/01/05
//

#include <vcl.h>
#pragma hdrstop

#include "PrRES.h"

#include "ETour.h"
#include "Strings.h"
#include "Format.h"
#include "Sort.h"
#include "Tourmaster.h"
#include "RegMaster.h"
#include "TextSmith.h"

#include "GoDrawGUI.h"
#include "CardsForm.h"
#include "PrintReg.h"
#include "PrintDrawRes.h"
#include "PrintRank.h"
#include "PrintPrize.h"
#include "PrintCards.h"
#include "TestPage.h"
#include "PrinterAPI.h"
#include "PrintMUI.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

namespace PrintMUI
{
	using std::string;
	using Tools::s_S;
   using Tools::S_s;
   using TourMaster::GoTour;

	void PrintErrorReport(short idproc, short idmsg)
   {
   	ToolsGUI::ErrorReport(IDM_PrintMUI,  idproc, idmsg);
   }
}

namespace PrintMUI								// PrintRegisterCmd
{
	void PrintRegisterCmd(GRegGrid* RegGrid, bool PrinterDlg)
   {
   	String PriFile = GoDraw::FilePath(GoDraw::RegPri);
      string priFile = S_s(PriFile);

   	try
      {
      	if(!GoTour.Players)
         {
         	ToolsGUI::OperatorError(IDM_NoRegister);

            return;
         }

         // CREATE A NEW REGISTER PRINT FILE
         const int OldSortSense =  GoTour.SortSense;
         {
         	RegMaster::RegSort(GoTour.SortOrder, ToolsGUI::gsUp);

         	TextSmith::RegisterPriFile RegPrint(priFile, &GoTour);

         	RegPrint.BuildTextList(GoTour.Players);

            // GK REGSORT 2015  RegMaster::RegSort(GoTour.SortOrder, OldSortSense);
         }

         // A REQUESTED PRINTER DIALOGUE CAN BE CANCELLED BY USER
         if(PrinterDlg)
         {
         	const bool Started = PrinterAPI::StartNewPrinter();

            if(!Started)								// Restore original order
            {
            	RegGrid->LoadSorted(GoTour.SortOrder, OldSortSense);

            	return;
            }
         }
         else
            PrinterAPI::StartPrinter();

         // PRINT THE REGISTER PriFile
         PrintReg::PrintRegister(PrinterDlg, PriFile);

         PrinterAPI::EndPrinter();

         RegGrid->LoadSorted(GoTour.SortOrder, GoTour.SortSense);

         GoTour.PostTourInfo(IDM_RegPub);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintRegCmd, IDM_NoRegPrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintRegCmd, IDM_NoRegPrMemory);
      }
   }
}
//---------------------------------------------------------------------------

namespace PrintMUI								// PrintDrawResCmd
{
	bool WarnedEditingDrawAct(void)
   {
    	if(GoTour.State != Tours::tsEditNow &&  GoTour.State != Tours::tsForcedPair)
      	return false;

      if(ToolsGUI::OperatorWarning(IDM_WarnEdDraw)) return false;	// Pressed OK- print anyway

      GoTour.PostTourInfo(IDM_PrintAborted);								// Presed Cancel

      return true;																// Abandon print
   }
   
	void PrintDrawResCmd(int Mode, bool PrinterDlg)
   {
      using GoDraw::DrawPri;
      using GoDraw::ResPri;
      using TextSmith::drDraw;

   	String PriFile = GoDraw::FilePath(Mode == drDraw ? DrawPri : ResPri);
      string priFile = S_s(PriFile);

      int IDM_PrintPub = Mode == drDraw ? IDM_DrawPub : IDM_ReslPub;
      int IDM_NoPub    = Mode == drDraw ? IDM_NoDraw  : IDM_NoResl;

      int Boards = (int)GoTour.Pair.size()-1;

   	try
      {
      	if(Boards<=0 || GoTour.Round <= 0)
         {
         	ToolsGUI::OperatorError(IDM_NoPub);

            return;
         }

         if(WarnedEditingDrawAct()) return;

         // CREATE A NEW PAIRING PRINT FILE
         try
         {
         	TextSmith::DrawResPriFile DrawResPrint(Mode, priFile, &GoTour);

         	DrawResPrint.BuildTextList(Boards);
         }
         catch(...)
         {
            PrintErrorReport(IDM_PrintDrawCmd, IDM_NoDrawPrFile);

            return;
         }

         // A REQUESTED PRINTER DIALOGUE CAN BE CANCELLED BY USER
         if(!PrinterDlg)
         	PrinterAPI::StartPrinter();
         else
            if(!PrinterAPI::StartNewPrinter()) return;

         // NOW PRINT THE PAIRING PriFile
         PrintDrawRes::PrintPairing(PrinterDlg, PriFile);

         PrinterAPI::EndPrinter();
         
         GoTour.PostTourInfo(IDM_PrintPub);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintDrawCmd, IDM_NoDrawPrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintDrawCmd, IDM_NoDrawPrPossible);
      }
   }
}
//---------------------------------------------------------------------------

namespace PrintMUI								// PrintRankCmd
{
	void PrintRankCmd(bool PrinterDlg)
   {
   	String PriFile = GoDraw::FilePath(GoDraw::RankPri);
      string priFile = S_s(PriFile);

      // GK REGSORT 2015  int OldSortSense = GoTour.SortSense, OldSortOrder = GoTour.SortOrder;
   	try
      {
      	if(!GoTour.Players)
         {
         	ToolsGUI::OperatorError(IDM_NoRank);

            return;
         }

         if(WarnedEditingDrawAct()) return;

         // CREATE A NEW RANKLIST PRINT FILE
         {
         	TextSmith::StdRankFile StdRank(priFile, &GoTour);

         	StdRank.BuildRankList();
         }

         // A REQUESTED PRINTER DIALOGUE CAN BE CANCELLED BY USER
         if(!PrinterDlg)
         	PrinterAPI::StartPrinter();
         else
            if(!PrinterAPI::StartNewPrinter()) return;

         // NOW PRINT THE RANK LIST
         PrintRank::PrintRankList(PrinterDlg, PriFile);

         PrinterAPI::EndPrinter();

         GoTour.PostTourInfo(IDM_RankPub);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintRankCmd, IDM_NoRankPrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintRankCmd, IDM_NoRankPrMemory);
      }
      // GK REGSORT 2015  RegMaster::RegSort(OldSortOrder, OldSortSense);
   }
}
//---------------------------------------------------------------------------

namespace PrintMUI								// PrintPrizeCmd
{
	void PrintPrizeCmd(bool PrinterDlg)
   {
   	String PriFile = GoDraw::FilePath(GoDraw::PrizePri);
      string priFile = S_s(PriFile);

      // GK REGSORT 2015  int OldSortSense = GoTour.SortSense, OldSortOrder = GoTour.SortOrder;
   	try
      {
      	if(!GoTour.PrizeCount)
         {
         	ToolsGUI::OperatorError(IDM_NoPrizes);

            return;
         }

         if(WarnedEditingDrawAct()) return;

         // CREATE NEW PRIZE PRINT FILE
         {
            Tours::PrizeSort(&GoTour);

            TextSmith::PrizePriFile PrizePrint(priFile, &GoTour);

            PrizePrint.BuildTextList(GoTour.PrizeCount);
         }

         // A REQUESTED PRINTER DIALOGUE CAN BE CANCELLED BY USER
         if(!PrinterDlg)
         	PrinterAPI::StartPrinter();
         else
            if(!PrinterAPI::StartNewPrinter()) return;

         //  DISPLAY THE THRESHOLD
         {
           	std::string threshmsg = Tools::load_str(IDM_PRIZETHRESHOLD);

            threshmsg += Tours::wins_ustring(GoTour.Rules.Prizes);

            threshmsg += Tools::load_str(IDM_PRIZEWINS);

         	ShowMessage(ToolsGUI::erInfo, threshmsg);
         }

         // NOW PRINT THE PRIZELIST
         PrintPrize::PrintPrizeList(PrinterDlg, PriFile);

         PrinterAPI::EndPrinter();

         GoTour.PostTourInfo(IDM_PrizePub);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintPrizeCmd, IDM_NoPrizePrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintPrizeCmd, IDM_NoPrizePrMemory);
      }
      // GK REGSORT 2015  RegMaster::RegSort(OldSortOrder, OldSortSense);
   }
}
//---------------------------------------------------------------------------

namespace PrintMUI								// PrintCardsCmd
{
	void PrintCardsCmd(TComponent* Owner, bool PrinterDlg)
   {
   	String PriFile = GoDraw::FilePath(GoDraw::CardsPri);
      string priFile = S_s(PriFile);

      // GK REGSORT 2015 int OldSortSense = GoTour.SortSense, OldSortOrder = GoTour.SortOrder;
   	try
      {
      	int Active = Tours::RankSort(&GoTour);

      	if(!Active)
         {
         	ToolsGUI::OperatorError(IDM_NoCards);

            return;
         }

         if(WarnedEditingDrawAct()) return;
         
         int Pfirst = 1, Plast = 24;	// Default range

         TCardsFrm* CardsForm = new TCardsFrm(Owner, GoTour.Players, Pfirst, Plast);  // GK 2015 Allow all players in range

         if(CardsForm->ShowModal() == mrOk)
         {
         	// THE CARD RANGE HAS BEEN VALIDATED
         	Pfirst = CardsForm->FirstEd->Text.ToInt();
            Plast  = CardsForm->LastEd->Text.ToInt();

            if(Plast < Pfirst) Plast = Pfirst;	// Just in case!

            // CREATE NEW CARDS PRINT FILE
            {
            	TextSmith::CardsPriFile CardsPrint(priFile, Pfirst, Plast, &GoTour);

            	CardsPrint.BuildTextList(Plast - Pfirst + 1);
            }

            // A REQUESTED PRINTER DIALOGUE CAN BE CANCELLED BY USER
            if(!PrinterDlg)
               PrinterAPI::StartPrinter();
            else
               if(!PrinterAPI::StartNewPrinter()) return;

            PrintCards::PrintCardList(PrinterDlg, PriFile);

            PrinterAPI::EndPrinter();

         	// GK REGSORT 2015  RegMaster::RegSort(OldSortOrder, OldSortSense);
         	GoTour.PostTourInfo(IDM_CardsPub);
         }
         delete CardsForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintCardsCmd, IDM_NoCardsPrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintCardsCmd, IDM_NoCardsPrMemory);
      }
      // GK REGSORT 2015  RegMaster::RegSort(OldSortOrder, OldSortSense);
   }
}
//---------------------------------------------------------------------------

namespace PrintMUI								// PrintTestCmd
{
	void PrintTestCmd(void)
   {
   	try
      {
      	PrinterAPI::StartNewPrinter();
         	PrinterAPI::PrintTestPage();
         PrinterAPI::EndPrinter();

         GoTour.PostTourInfo(IDM_TestPub);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintTestCmd, IDM_NoTestPrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintTestCmd, IDM_NoTestPrMemory);
      }
   }
}
//---------------------------------------------------------------------------

namespace PrintMUI								// PrintSetupCmd
{
	void PrintSetupCmd(void)
   {
   	try
      {
      	if(PrinterAPI::SetupPrinter())
         	GoTour.PostTourInfo(IDM_PrinterPub);
         else
            GoTour.PostTourInfo(IDM_Ready);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDM_PrintSetupCmd, IDM_NoSetupPrMemory);
      }
      catch(...)
      {
      	PrintErrorReport(IDM_PrintSetupCmd, IDM_NoSetupPrMemory);
      }
   }
}
//---------------------------------------------------------------------------




