////////////////////////////////////////////////////////////////////////////////
// MODULE PrintPrize:  PrintPrize from PrizePRI.txt 		G.Kaniuk 		11/09/05
//
#define STRICT 			// THIS IS ESSENTIAL TO GET THE CORRECT TYPE FOR HFONT!!!
#include <windows.h>
#include <string>

#include <vcl.h>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "PrRES.h"

#include "Strings.h"
#include "Format.h"

#include "ETour.h"

#include "PrinterAPI.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace PrintPrize
{
	using std::string;

   using Tools::S_s;
   using Tools::unpack_tabbed;
   using Tools::spaces;
   using Tools::to_string;
   using Tools::to_posint;

	using PrinterAPI::paTopLeft;
   using PrinterAPI::paTopMid;
   using PrinterAPI::paTopRight;
   using PrinterAPI::paMidLeft;
   using PrinterAPI::paCentre;
   using PrinterAPI::paMidRight;
   using PrinterAPI::pfBold;
   using PrinterAPI::pfNormal;

   using PrinterAPI::PrAlign;
   using PrinterAPI::PrSheet;
   using PrinterAPI::PrTextArea;

   enum								// LINE SPACING
   {
   	rlFont = 64, ruFont = 78, ruSpace = 60
   };

   enum								// FIELD COUNTS
   {
   	prizeMainHead = 8,
      prizeCols = 8
   };

   enum								// CHARACTER COUNTS
   {
   	nNum=6, nPid=7,			// Line number, playerID
      nName=28,					// Name
      nGrade=10,					// Grade
      nClub=20,					// Club name
      nLand=20,               // Country name
      nWin=6,						// Include jigo
      nRank=7,						// Player's rank number
      nSpace=1						// Space
   };

   static string PrizeTitle, PrizeTour, PrizePlayers, PrizeRange;
   static string PrizeColText[prizeCols];

   static TRect aPrizeTitle, aPrizeTour, aPrizeRange;		// Fixed areas per page
   static PrTextArea aPrizeText[prizeCols];					// Column positions

   static int LineFontHeight, LineFontWidth;
   static int HeadFontHeight, HeadFontWidth;

   static bool AnalysePrizeHeader
   (
   	TStringList* prizefile, PrSheet& prizesheet,

      int& prizecount, int& batchcount, int& textpos
   )
   {
   	// EXTRACT THE FIRST ROW
      // PRIZE LIST	ROUND	3	Fourty Two	2005	PRIZES	6	Players
   	const string PrizeHeadRaw = S_s(prizefile->Strings[0]);
      string PrizeHead[prizeMainHead];
      if(unpack_tabbed(PrizeHeadRaw, prizeMainHead, PrizeHead) != prizeMainHead) return false;

      string PrizeList  = PrizeHead[0], Round = PrizeHead[1], RoundNo = PrizeHead[2];

      PrizeTitle = PrizeList + spaces(3) + Round + spaces(1) + RoundNo + spaces(10);

      PrizeTour = PrizeHead[3] + spaces(1) + PrizeHead[4];

      string PrizeCount = PrizeHead[6];

      prizecount = to_posint(PrizeCount.begin(), PrizeCount.end());

      PrizePlayers = PrizeHead[7];

      PrizeRange = PrizePlayers + string("  999 - 999");

      // EXTRACT COLUMN HEADINGS
      const string PrizeColRaw = S_s(prizefile->Strings[1]);
      if(unpack_tabbed(PrizeColRaw, prizeCols, PrizeColText) != prizeCols) return false;

      // REQUIRED FONT WIDTH - DC IS CREATED WITH 11 POINT FONT USED HERE
      const int MaxLineChars = nNum+nSpace+nPid +2*nSpace+
      							    nName+nGrade +nSpace+nClub+nSpace+nLand +
                               3*nSpace+nWin+nSpace+nRank;

      const int ActiveWidth 	= prizesheet.Rmargin - prizesheet.Lmargin;

      int ReqFontHeight = prizesheet.StdFontHeight;;		// 11 point
      int ReqFontWidth  = ActiveWidth/MaxLineChars;

      prizesheet.NewFont(ReqFontHeight, ReqFontWidth, pfNormal);

      LineFontHeight = prizesheet.FontHeight;				// Achieved size
      LineFontWidth  = prizesheet.FontWidth;

      // ALLOCATE FIELD SIZES
      const int wSpace = nSpace*LineFontWidth;
      const int wNum   = nNum*LineFontWidth;
      const int wPid   = nPid*LineFontWidth;
      const int wName  = nName*LineFontWidth;
      const int wGrade = nGrade*LineFontWidth;
      const int wClub  = nClub*LineFontWidth;
      const int wLand  = nLand*LineFontWidth;
      const int wWin   = nWin*LineFontWidth;
      const int wRank  = nRank*LineFontWidth;


      aPrizeText[0] = PrTextArea(prizesheet.Lmargin,        wNum,  paTopLeft);
      aPrizeText[1] = aPrizeText[0] +  PrTextArea(  wSpace, wPid,  paTopLeft);
      aPrizeText[2] = aPrizeText[1] +  PrTextArea(2*wSpace, wName, paTopLeft);
      aPrizeText[3] = aPrizeText[2] +  PrTextArea(			0, wGrade,paTopMid);
      aPrizeText[4] = aPrizeText[3] +  PrTextArea(  wSpace, wClub, paTopLeft);
      aPrizeText[5] = aPrizeText[4] +  PrTextArea(  wSpace, wLand, paTopLeft);
      aPrizeText[6] = aPrizeText[5] +  PrTextArea(3*wSpace, wWin,  paTopRight);
      aPrizeText[7] = aPrizeText[6] +  PrTextArea(  wSpace, wRank, paTopRight);

      // TEMPLATE FOR MAIN HEADING
      prizesheet.NewFont(ReqFontHeight, ReqFontWidth, pfBold);

      HeadFontWidth = prizesheet.FontWidth;
      HeadFontHeight = prizesheet.FontHeight;

      int wTitle, wTour, wRange, hText;

      prizesheet.GetTextArea(PrizeTitle, wTitle, hText);
      prizesheet.GetTextArea(PrizeRange, wRange, hText);

      wTour = ActiveWidth - wTitle - wRange;

		if(wTour<0)
      {
      	wTitle = (3*ActiveWidth)/8;
         wTour  = ActiveWidth/4;
         wRange = (3*ActiveWidth)/8;
      }

      const int Left = prizesheet.Lmargin, Top = prizesheet.Tmargin;

      aPrizeTitle = TRect(Left, 						 Top, Left+wTitle, 					Top+HeadFontHeight);
      aPrizeTour  = TRect(aPrizeTitle.Right+1,   Top, aPrizeTitle.Right+1+wTour, Top+HeadFontHeight);
      aPrizeRange = TRect(aPrizeTour.Right+1,    Top, prizesheet.Rmargin, 			Top+HeadFontHeight);

      // FIVE LINES PER BATCH
      const int HeadHeight = (7*HeadFontHeight)/2;

      batchcount = prizesheet.BatchesPerPage(ruFont, rlFont, ruSpace, HeadHeight);

      textpos = prizesheet.Tmargin + HeadHeight;

      return true;
   }

   static void PrPrizeHead
   (
   	const int FromPlayer, const int ToPlayer,

      PrSheet& PrizeSheet
   )
   {
   	using Tools::trim;

   	// ALL HEADER LINES 11 PT BOLD
      PrizeSheet.NewFont(HeadFontHeight, HeadFontWidth, pfBold);
      const int HeadFontH = PrizeSheet.FontHeight;

   	//FIRST HEADER LINE
      string Range = to_string(FromPlayer) + string(" - ") + to_string(ToPlayer);

   	PrizeRange = PrizePlayers + spaces(3) + Range;

      PrizeSheet.DrawTextCell(aPrizeTitle, paMidLeft,   PrizeTitle);
      PrizeSheet.DrawTextCell(aPrizeTour,  paCentre,    PrizeTour);
      PrizeSheet.DrawTextCell(aPrizeRange, paMidRight,  PrizeRange);

      // COLUMN HEADINGS
      const int Top = PrizeSheet.Tmargin + 2*HeadFontH;

      for(int col=0; col<prizeCols; col++)
      {
      	PrTextArea& Prize = aPrizeText[col];

         Prize.area.top    = Top;
         Prize.area.bottom = Top + HeadFontHeight;

         PrizeSheet.DrawTextCell(Prize.area, Prize.align, PrizeColText[col]);
      }
	}

   static void PrPrizeLines
   (
      const int FromPlayer, const int ToPlayer,
      const int TextPos,

      TStringList* PrizeFile, PrSheet& PrizeSheet
   )
   {
   	// ALL RANK LINES HAVE STANDARD SIZE FONT
      PrizeSheet.NewFont(LineFontHeight, LineFontWidth, pfNormal);

      const int FontHeight  = PrizeSheet.FontHeight;
      const int LineHeight  = (ruFont*FontHeight)/rlFont;
      const int SpaceHeight = (ruSpace*FontHeight)/rlFont;

      // BATCH CONTROL
      int TextTopNow = TextPos, BatchLine = 0;

      // PRINT ALL LINES IN PAGE
      string PrizeItems[prizeCols];

      for(int p=FromPlayer; p<=ToPlayer; p++)
      {
      	string RawLine = Tools::S_s(PrizeFile->Strings[p+1]);

         for(int i=0; i<prizeCols; i++) PrizeItems[i] = string();

         unpack_tabbed(RawLine, prizeCols, PrizeItems);

      	for(int i=0; i<prizeCols; i++)
         {
         	const string& Item = PrizeItems[i];

            if(!Item.empty())
            {
               PrTextArea Prize = aPrizeText[i];

               Prize.area.top    =  TextTopNow;
               Prize.area.bottom =  TextTopNow + LineHeight;

               PrizeSheet.DrawTextCell(Prize.area, Prize.align, PrizeItems[i]);
            }
         }

         ++BatchLine;

         if(BatchLine == 5)
         {
         	BatchLine = 0;

         	TextTopNow += LineHeight+SpaceHeight;
         }
         else
         	TextTopNow += LineHeight;
      }
   }

   void PrintPrizeList(bool PrDlg, String& PrizePri)
   {
   	TStringList* PrizeFile = 0;

   	try
      {
         PrizeFile = new TStringList();

         PrizeFile->LoadFromFile(PrizePri);

         PrinterAPI::PrSheet PrizeSheet;

         int PlayerCount, BatchCount, TextPos;

         if(!AnalysePrizeHeader(PrizeFile, PrizeSheet, PlayerCount, BatchCount, TextPos))
         	throw(Exception(String()));

         const int PlayersPerPage = 5*BatchCount;

         int pFrom = 1;
         int pTo   = PlayerCount>PlayersPerPage ? PlayersPerPage : PlayerCount;

         PrinterAPI::StartDoc(Tools::load_cstr(IDM_PrintPrizes));

         while(true)
         {
            PrinterAPI::StartPage();

            	PrPrizeHead(pFrom, pTo, PrizeSheet);

            	PrPrizeLines(pFrom, pTo, TextPos, PrizeFile, PrizeSheet);

            PrinterAPI::EndPage();

            if(pTo < PlayerCount)
            {
               pFrom = pTo + 1;
               pTo   = pFrom + PlayersPerPage-1;

               if(pTo > PlayerCount) pTo = PlayerCount;
            }
            else
            	break;
         }

         PrinterAPI::EndDoc();

         delete PrizeFile;
      }
      catch(...)
      {
      	delete PrizeFile;

         PrinterAPI::Throw(ToolsGUI::erOperr, IDM_PrintPrizeCmd, IDM_NoPrizePrMemory);
      }
   }
}
//---------------------------------------------------------------------------


