////////////////////////////////////////////////////////////////////////////////
// MODULE PrintRank:  PrintRank from RankPRI.txt 		G.Kaniuk 		21/08/05
//
#define STRICT 			// THIS IS ESSENTIAL TO GET THE CORRECT TYPE FOR HFONT!!!
#include <windows.h>

#include <vcl.h>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "PrRES.h"

#include "Strings.h"
#include "Format.h"

#include "ETour.h"

#include "PrinterAPI.h"
#include "PrintRank.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace PrintRank
{
	using std::string;

   using Tools::cZero;
   using Tools::S_s;
   using Tools::to_posint;
   using Tools::to_string;
   using Tools::unpack_tabbed;

   using PrinterAPI::paTopLeft;
   using PrinterAPI::paTopMid;
   using PrinterAPI::paTopRight;
   using PrinterAPI::paMidLeft;
   using PrinterAPI::paCentre;
   using PrinterAPI::paMidRight;
   using PrinterAPI::pfBold;
   using PrinterAPI::pfNormal;

   using PrinterAPI::PrAlign;
   using PrinterAPI::PrSheet;
   using PrinterAPI::PrTextArea;

   enum				// FONT SIZES
   {
   	fsHead = 11, fsLine = 10, fsStd = 11
   };

   enum
   {
   	rlFont = 64, ruFont = 78, ruSpace = 60
   };

   enum
   {
   	rankMainHead = 8,
      rankCols = 22
   };

   enum
   {
   	nPosn=7, nNum = 7,		// Posn, number
      nName=25,					// Name
      nGrade=8,					// Grade
      nClubF=18,					// Club name
      nLandF=14,              // Country name
      nClubC=5,					// Club code
      nLandC=4,					// Land code
      nResult=6,					// ddddR
      nWin=5,						// Include jigo
      nMMS=7,						// -30h
      nTie=7,						// -300h
      nSpace=1						// Space
   };

   static TRect aRankTitle, aRankTour, aRankRange;		// Fixed areas per page

   static PrTextArea Rgeom[rankCols+1];						// Rank line and heading metrics

   static string RankTitle, RankTour, RankPlayers, RankRange, RankGrade;
   static string RankColText[rankCols];

   static int RankColTextCount;
   static int ClubFormat, LandFormat, Tie1Fmt,Tie2Fmt;
   static bool TrimClub, TrimLand;
   static int  RankColsUsed;
   static int RoundCount;
   static int LineFontHeight, LineFontWidth;

   static bool AnalyseRankHeader
   (
   	TStringList* rankfile, PrSheet& ranksheet,

      int& playercount, int& batchcount, int& textpos
   )
   {
   	// EXTRACT THE FIRST ROW
      // RANK LIST	ROUND	3	Epsom	2000	PLAYERS	46	0000
   	const string RankHeadRaw = S_s(rankfile->Strings[0]);
      string RankHead[rankMainHead];
      if(unpack_tabbed(RankHeadRaw, rankMainHead, RankHead) != rankMainHead) return false;

      string RankList  = RankHead[0], Round = RankHead[1], RoundNo = RankHead[2];
      string RankCount = RankHead[6];

      string strSpace(" ");

      RankTour    = RankHead[3] + strSpace + RankHead[4];
      RankPlayers = RankHead[5];

      playercount = to_posint(RankCount.begin(), RankCount.end());

      string RankCode = RankHead[7];

      ClubFormat = (int)(RankCode[0]-cZero); LandFormat = (int)(RankCode[1]-cZero);
      Tie1Fmt    = (int)(RankCode[2]-cZero); Tie2Fmt    = (int)(RankCode[3]-cZero);


      // EXTRACT THE COLUMN HEADINGS
      // POSN	NO	NAME	GRADE	CLUB COUNTRY	 1	 2	 3	WIN	MMS TIE1 TIE 2
   	const string RankColRaw = S_s(rankfile->Strings[1]);

      RankColTextCount = unpack_tabbed(RankColRaw, rankCols, RankColText);

      // CLUB COUNTRY SPACE
      int numClub = 0, numLand = 0, numLocn  = 0;

      if(ClubFormat==0 && LandFormat==0)  numLocn = 0;
      if(ClubFormat==1 && LandFormat==0)  numLocn = numClub = nClubC;
      if(ClubFormat==2 && LandFormat==0)  numLocn = numClub = nClubF;

      if(ClubFormat==0 && LandFormat==1)  numLocn = numLand = nLandC;
      if(ClubFormat==1 && LandFormat==1){ numClub = nClubC;	numLand = nLandC; numLocn = nClubC+nSpace+nLandC; }
      if(ClubFormat==2 && LandFormat==1){ numClub = nClubF; numLand = nLandC; numLocn = nClubF+nSpace+nLandC; }

      if(ClubFormat==0 && LandFormat==2)  numLocn = numLand = nLandF;
      if(ClubFormat==1 && LandFormat==2){ numClub = nClubC; numLand = nLandF; numLocn = nClubC+nSpace+nLandF; }
      if(ClubFormat==2 && LandFormat==2){ numClub = nClubF; numLand = nLandF; numLocn = nClubF+nSpace+nLandF; }

      // TIEBREAK SPACE
      int numTie1 = 0, numTie2 = 0, numTies = 0;

      if(Tie1Fmt)
      {
      	numTie1 = numTies = nTie;

         if(Tie2Fmt)
         {
         	numTie2 = nTie;

         	numTies += nSpace + nTie;
         }
      }


   	// THE MAXIMUM NUMBER OF CHARACTERS PER LINE
      RoundCount = to_posint(RoundNo.begin(), RoundNo.end());

      const int numResults = RoundCount*nResult + (RoundCount-1)*nSpace;

      int MaxLineChars = nPosn+nNum + 2*nSpace + nName
      					  + nGrade + nSpace + numLocn + 2*nSpace + numResults + 2*nSpace
                       + nWin+nSpace+nMMS + nSpace + numTies;

      // REQUIRED FONT WIDTH - DC IS CREATED WITH 11 POINT FONT
      const int ActiveWidth 	= ranksheet.Rmargin - ranksheet.Lmargin;

      const int StdFontHeight = ranksheet.FontHeight;		// 11 point
      const int StdFontWidth  = ranksheet.FontWidth;

      int ReqFontHeight =  (fsLine*StdFontHeight)/fsStd;	// Desired size
      int ReqFontWidth  =  (fsLine*StdFontWidth)/fsStd;

      int MaxFontWidth = ActiveWidth/MaxLineChars;

     	if(ReqFontWidth > MaxFontWidth)							// Need to reduce width
      	ReqFontWidth = MaxFontWidth;

      ranksheet.NewFont(ReqFontHeight, ReqFontWidth, pfNormal);

      LineFontHeight = ranksheet.FontHeight;					// Achieved size
      LineFontWidth  = ranksheet.FontWidth;

      TrimClub = TrimLand = false;

      if(LineFontWidth*MaxLineChars> ActiveWidth)			// Cannot honour font request
      {																	// Replace Club/Land names with codes
      	const int OldMaxLineChars = MaxLineChars;

      	MaxLineChars -= numLocn;

      	if(ClubFormat == 2) { ClubFormat = 1; numLocn -= nClubF; TrimClub = true; }
         if(LandFormat == 2) { LandFormat = 1; numLocn -= nLandF; TrimLand = true; }

         MaxLineChars += numLocn;

         if(MaxLineChars < OldMaxLineChars)
         {
         	MaxFontWidth = ActiveWidth/MaxLineChars;

     			if(ReqFontWidth > MaxFontWidth)ReqFontWidth = MaxFontWidth;

      		ranksheet.NewFont((fsLine*StdFontHeight)/fsStd, ReqFontWidth, pfNormal);

      		LineFontHeight = ranksheet.FontHeight;
      		LineFontWidth  = ranksheet.FontWidth;
         }
		}

      // FIELD WIDTHS EACH LINE AND HEADING ARE THE SAME
      const int wPosn   = nPosn*LineFontWidth;
      const int wNum    = nNum*LineFontWidth;
      const int wName   = nName*LineFontWidth;
      const int wGrade  = nGrade*LineFontWidth;
      const int wClub   = numClub*LineFontWidth;
      const int wLand   = numLand*LineFontWidth;
      const int wResult = nResult*LineFontWidth;
      const int wWins   = nWin*LineFontWidth;
      const int wMMS    = nMMS*LineFontWidth;
      const int wTie1   = numTie1*LineFontWidth;
      const int wTie2   = numTie2*LineFontWidth;
      const int wSpace  = nSpace*LineFontWidth;
      const int wSpCL   = wClub>0 && wLand>0 ? wSpace : 0;
      const int wSpT12	= wTie1>0 && wTie2>0 ? wSpace : 0;

      // RANK COLUMN GEOMETRY
      int rCol = 0, wLast;

      Rgeom[0] = PrTextArea(ranksheet.Lmargin,    wPosn,  paTopRight);
      Rgeom[1] = Rgeom[0] + PrTextArea(0, 		  wNum, 	 paTopRight);
      Rgeom[2] = Rgeom[1] + PrTextArea(2*wSpace,  wName,  paTopLeft);
      Rgeom[3] = Rgeom[2] + PrTextArea(0, 		  wGrade, paTopMid);

      Rgeom[4] = Rgeom[3] + PrTextArea(wSpace, 	  wClub,	paTopLeft);	// wClub may be zero
      Rgeom[5] = Rgeom[4] + PrTextArea(wSpCL,     wLand,	paTopLeft); // wLand may be zero

      rCol = 6;

      Rgeom[rCol] = Rgeom[rCol-1] + PrTextArea(2*wSpace, wResult,  paTopLeft);	++rCol;

      for(int r=1; r<RoundCount; r++)
      {
      	Rgeom[rCol] = Rgeom[rCol-1] + PrTextArea(wSpace, wResult, paTopLeft);

         ++rCol;
		}

      Rgeom[rCol] = Rgeom[rCol-1] + PrTextArea(2*wSpace, wWins,	paTopMid);	++rCol;
      Rgeom[rCol] = Rgeom[rCol-1] + PrTextArea(wSpace,   wMMS,	paTopLeft);	++rCol;

      Rgeom[rCol] = Rgeom[rCol-1] + PrTextArea(wSpace, 	wTie1,	paTopLeft);	++rCol;
      Rgeom[rCol] = Rgeom[rCol-1] + PrTextArea(wSpT12, 	wTie2,	paTopLeft);	++rCol;

      RankColsUsed = rCol;


      // VERTICAL SPACE REQUIRED FOR 11 POINT BOLD HEADINGS
      ranksheet.NewFont(StdFontHeight, StdFontWidth, pfBold);

      const int HeadFontHeight = ranksheet.FontHeight;

      const int HeadHeight = (7*HeadFontHeight)/2;


      // TEMPLATE FOR FIRST HEADER LINE
      int wTitle, wTour, wRange, wGap, hText;

      RankTitle = RankList+strSpace+strSpace+strSpace + Round+strSpace+RoundNo;

      RankRange = RankPlayers + string("  9999 - 9999");

      ranksheet.GetTextArea(RankTitle, wTitle, hText);
      ranksheet.GetTextArea(RankTour,  wTour,  hText);
      ranksheet.GetTextArea(RankRange, wRange, hText);

      wGap = ActiveWidth - wTitle - wTour - wRange;
      wGap = wGap>2 ? wGap/2 : 1;

      int Left = ranksheet.Lmargin, Top = ranksheet.Tmargin;

      aRankTitle = TRect(Left,				 	   Top, Left+wTitle, 					 Top+hText);
      aRankTour  = TRect(aRankTitle.right+wGap, Top, aRankTitle.right+wGap+wTour, Top+hText);
      aRankRange = TRect(aRankTour.right+wGap,  Top, aRankTour.right+wGap+wRange, Top+hText);

      // FIVE LINES PER BATCH
      ranksheet.NewFont(LineFontHeight, LineFontWidth, pfNormal);

      batchcount = ranksheet.BatchesPerPage(ruFont, rlFont, ruSpace, HeadHeight);

      textpos = ranksheet.Tmargin + HeadHeight;

		return true;
   }

   static void PrRankHead
   (
   	const int FromPlayer, const int ToPlayer,

      PrSheet& RankSheet
   )
   {
   	using Tools::trim;

   	// ALL HEADER LINES 11 PT BOLD
      int HeadFontHeight = RankSheet.StdFontHeight;
      int HeadFontWidth  = RankSheet.StdFontWidth;

      RankSheet.NewFont(HeadFontHeight, HeadFontWidth, pfBold);
      HeadFontHeight = RankSheet.FontHeight;
      HeadFontWidth  = RankSheet.FontWidth;

   	//FIRST HEADER LINE
      string Range = to_string(FromPlayer) + string(" - ") + to_string(ToPlayer);

   	RankRange = RankPlayers + string("   ") + Range;

      RankSheet.DrawTextCell(aRankTitle, paMidLeft,   RankTitle);
      RankSheet.DrawTextCell(aRankTour,  paCentre,    RankTour);
      RankSheet.DrawTextCell(aRankRange, paMidRight,  RankRange);

      // SECOND HEADER LINE - SCALE WIDTH FOR REDUCED LINE FONT
      HeadFontHeight = RankSheet.StdFontHeight;
      HeadFontWidth  = (RankSheet.StdFontWidth + LineFontWidth)/2;

      RankSheet.NewFont(HeadFontHeight, HeadFontWidth, pfBold);

      HeadFontHeight = RankSheet.FontHeight;
      HeadFontWidth  = RankSheet.FontWidth;

      const int Top = aRankTitle.top + 2*HeadFontHeight;

      int col;

      PrTextArea HeadGeom;

      // FIRST COLS - POSN NO NAME GRADE FIXED
      for(col=0; col<4; col++)
      {
      	HeadGeom = Rgeom[col];

         HeadGeom.area.top = Top;
         HeadGeom.area.bottom = Top + HeadFontHeight;

         RankSheet.DrawTextCell(HeadGeom.area, HeadGeom.align, RankColText[col]);
      }

      // CLUB/COUNTRY HEADERS JOINED
      string ClubText = RankColText[4], LandText = RankColText[5], HeadText;

      const PrTextArea& ClubGeom = Rgeom[4], LandGeom = Rgeom[5];

      if(!ClubText.empty() && !LandText.empty())
      	HeadText = ClubText + string("/") + LandText;
      else
         HeadText = ClubText + LandText;		// One or both empty

      HeadGeom             = ClubGeom;

      HeadGeom.area.top    = Top;
      HeadGeom.area.bottom = Top + HeadFontHeight;
      HeadGeom.area.right  = LandGeom.area.right;

      if(!HeadText.empty())
      	RankSheet.DrawTextCell(HeadGeom.area, HeadGeom.align, HeadText);

      // NOW RESULTS, WIN, MMS
      for(col=6; col< 6 + RoundCount + 2; col++)
      {
      	HeadGeom = Rgeom[col];

         HeadGeom.area.top = Top;	HeadGeom.area.bottom = Top + HeadFontHeight;

         if(!HeadText.empty())
         	RankSheet.DrawTextCell(HeadGeom.area, HeadGeom.align, RankColText[col]);
      }

      // TIEBREAK HEADERS JOINED
      const string Tie1Text = trim(RankColText[col]), Tie2Text = trim(RankColText[col+1]);

      const PrTextArea& Tie1Geom = Rgeom[col], Tie2Geom = Rgeom[col+1];

      if(!Tie2Text.empty())										// Therefore Tie1 present
      	HeadText = Tie1Text + string("/") + Tie2Text;
      else
         HeadText = Tie1Text;										// Tie1 may be empty

      HeadGeom             = Tie1Geom;

      HeadGeom.area.top    = Top;
      HeadGeom.area.bottom = Top + HeadFontHeight;
      HeadGeom.area.right  = Tie2Geom.area.right;

      if(!HeadText.empty())
      	RankSheet.DrawTextCell(HeadGeom.area, HeadGeom.align, HeadText);
   }

   static void PrRankLines
   (
      const int FromPlayer, const int ToPlayer,
      const int TextPos,

      TStringList* RankFile, PrSheet& RankSheet
   )
   {
   	using PrinterAPI::PrAlign;
      using PrinterAPI::paTopMid;

       // ALL RANK LINES HAVE REDUCED SIZE FONT
      RankSheet.NewFont(LineFontHeight, LineFontWidth, pfNormal);

      const int FontHeight  = RankSheet.FontHeight;
      const int LineHeight  = (ruFont*FontHeight)/rlFont;
      const int SpaceHeight = (ruSpace*FontHeight)/rlFont;

      // BATCH CONTROL
      int TextTopNow = TextPos, BatchLine = 0;

      // PRINT ALL LINES IN PAGE
      string RankItems[rankCols];

      for(int p=FromPlayer; p<=ToPlayer; p++)
      {
      	string RawLine = Tools::S_s(RankFile->Strings[p+1]);

         for(int i=0; i<RankColsUsed; i++) RankItems[i] = string();

         unpack_tabbed(RawLine, RankColsUsed, RankItems);

         if(TrimClub)
         {
         	string& pClub = RankItems[4];

            pClub = pClub.substr(0,3);
         }

         if(TrimLand)
         {
         	string& pLand = RankItems[5];

            pLand = pLand.substr(0,2);
         }

      	for(int i=0; i<RankColsUsed; i++)
         {
         	const string& Item = RankItems[i];

            if(!Item.empty())
            {
               PrTextArea Geom = Rgeom[i];

               Geom.area.top    =  TextTopNow;
               Geom.area.bottom =  TextTopNow + LineHeight;

               RankSheet.DrawTextCell(Geom.area, Geom.align, RankItems[i]);
            }
         }

         ++BatchLine;

         if(BatchLine == 5)
         {
         	BatchLine = 0;

         	TextTopNow += LineHeight+SpaceHeight;
         }
         else
         	TextTopNow += LineHeight;
      }
   }

	void PrintRankList(bool PrDlg, String& RankPri)
   {
   	TStringList* RankFile = 0;

   	try
      {
         RankFile = new TStringList();

         RankFile->LoadFromFile(RankPri);

         PrinterAPI::PrSheet RankSheet;

         int PlayerCount, BatchCount, TextPos;

         if(!AnalyseRankHeader(RankFile, RankSheet, PlayerCount, BatchCount, TextPos))
         	throw(Exception(String()));

         const int PlayersPerPage = 5*BatchCount;

         int pFrom,pTo;

         pFrom = 1;
         pTo   = PlayerCount>PlayersPerPage ? PlayersPerPage : PlayerCount;

         PrinterAPI::StartDoc(Tools::load_cstr(IDM_PrintRank));

         while(true)
         {
            PrinterAPI::StartPage();

            	PrRankHead(pFrom, pTo, RankSheet);

            	PrRankLines(pFrom, pTo, TextPos, RankFile, RankSheet);

            PrinterAPI::EndPage();

            if(pTo < PlayerCount)
            {
               pFrom = pTo + 1;
               pTo   = pFrom + PlayersPerPage-1;

               if(pTo > PlayerCount) pTo = PlayerCount;
            }
            else
            	break;
         }

         PrinterAPI::EndDoc();

         delete RankFile;
      }
      catch(...)
      {
      	delete RankFile;

         PrinterAPI::Throw(ToolsGUI::erOperr, IDM_PrintRankCmd, IDM_NoRankPrMemory);
      }
   }
}
//---------------------------------------------------------------------------



