////////////////////////////////////////////////////////////////////////////////
// MODULE PrintReg:  PrintRegister from RegPri.txt 		G.Kaniuk 		10/08/05
//
#define STRICT 			// THIS IS ESSENTIAL TO GET THE CORRECT TYPE FOR HFONT!!!
#include <windows.h>

#include <vcl.h>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "PrRES.h"

#include "Strings.h"
#include "Format.h"

#include "ETour.h"

#include "PrinterAPI.h"
#include "PrintReg.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace PrintReg
{
	using std::string;

   using Tools::S_s;
   using Tools::to_posint;
   using Tools::to_string;
   using Tools::unpack_tabbed;
   using Tools::spaces;

   using PrinterAPI::paTopLeft;
   using PrinterAPI::paTopMid;
   using PrinterAPI::paTopRight;

   using PrinterAPI::pfBold;
   using PrinterAPI::pfNormal;

   using PrinterAPI::PrSheet;
   using PrinterAPI::PrTextArea;

   enum				// FONT SCALINGS
   {
   	rlFont = 64, ruFont = 71, ruSpace = 77
   };

   enum				// TEXT AREA SCALINGS
   {
   	sId=9, sReg=2, sName=50, sClub=30, sLand=10, sActive=180
	};

   enum				// HEADER SCALING
   {
   	sTitle=60,	sTour=87, sPages=33
   };

   enum				// FIELD COUNTS
   {
   	regItems1 = 6,
   	regItems2 = 9,
      regCols   = 8		// Excluding Tick
   };

   enum				// HEADER FIELDS
   {
   	riTitle, riTour, riRoundCount, riPage, riOf, riPlayers, riHcount
   };

   enum				// COLUMNS
   {
   	riId, riReg, riName, riGrade, riClub, riLand, riRounds, riGroup, riNote, riCount
   };

   // HEADER TEMPLATES
   static PrTextArea aRegTitle, aRegTour, aRegPage; // Fixed areas per page
   static PrTextArea aRegHeads[regCols-1];

   // LINE TEMPLATE
   static PrTextArea aRegLine[riCount];				 // Current line

   // HEADER TEXT
   static string RegTitle, RegTour, RegRounds, RegPage, RegOf, RegPlayers;
   static string RegOfPages, RegPageCount;
   static string RegColText[regCols-1];

   // FONT SIZES
   static int headFontHeight, headFontWidth;			// Requested header font
   static int HeadFontHeight, HeadFontWidth;			// Supplied header font

   static int lineFontHeight, lineFontWidth;			// Requested line font
   static int LineFontHeight, LineFontWidth;			// Supplied line font
   static int LineHeight,     SpaceHeight;         // Batch print spacing

   static bool AnalyseRegHeader
   (
   	TStringList* regfile, PrSheet& regsheet,

      int& playercount, int& pagecount, int& batchcount, int& textpos
   )
   {
   	// EXTRACT THE FIRST ROW
      // Line 1: REGISTER_SORTED_BY_ID ~ London_Open_2008 ~ 3 ~ PAGE~OF~134
   	const string RegRaw1 = S_s(regfile->Strings[0]);
      string RegHead1[riHcount];
      if(unpack_tabbed(RegRaw1, regItems1, RegHead1) != regItems1) return false;

      RegTitle 	 = RegHead1[riTitle];
      RegTour  	 = RegHead1[riTour];
      RegRounds    = RegHead1[riRoundCount];
      RegPage  	 = RegHead1[riPage];
      RegOf			 = RegHead1[riOf];
      RegPlayers 	 = RegHead1[riPlayers];

      const size_t Rounds = to_posint(RegRounds.begin(), RegRounds.end());

      // EXTRACT SECOND HEADER ROW
      // Line 2: ID	[REG]	NAME	GRADE	CLUB\COUNTRY	ROUNDS	GROUP	NOTE
      const string RegRaw2 = S_s(regfile->Strings[1]);
      string RegHeads2[regItems2];
      if(unpack_tabbed(RegRaw2, regItems2, RegHeads2) != regItems2) return false;

      RegColText[0] = RegHeads2[riId];
      RegColText[1] = RegHeads2[riName];
      RegColText[2] = RegHeads2[riGrade];
      RegColText[3] = RegHeads2[riClub] + string("/") + RegHeads2[riLand];
      RegColText[4] = RegHeads2[riRounds];
      RegColText[5] = RegHeads2[riGroup];
      RegColText[6] = RegHeads2[riNote];

      string strGrade  = RegColText[2], strGroup = RegColText[5];
      string strRounds = RegColText[4];

      // FONT SCALING BASED ON MEASUREMENT
      // 28 CHARS OCCUPIES 48mm, ACTIVE WIDTH IS 182mm
      {
      	const int activewidth =  regsheet.Rmargin - regsheet.Lmargin;

         const int namewidth = (sName*activewidth)/sActive;

         lineFontHeight = regsheet.StdFontHeight;
         lineFontWidth  = namewidth/33;

         
         headFontHeight = (12*lineFontHeight)/11;
      	headFontWidth  = lineFontWidth;
      }

      // SPACE REQUIRED FOR 12 POINT BOLD HEADINGS
      regsheet.NewFont(headFontHeight, headFontWidth, pfBold);

      HeadFontHeight = regsheet.FontHeight;
      HeadFontWidth  = regsheet.FontWidth;

      const int HeadHeight = (7*HeadFontHeight)/2;

      int wGrade, wRounds, wGroup, hText;

      regsheet.GetTextArea(strGrade,  wGrade,  hText);
      regsheet.GetTextArea(strRounds, wRounds, hText);
      regsheet.GetTextArea(strGroup,  wGroup,  hText);

      // TEMPLATE FOR FIRST HEADER LINE
      const int wActive = regsheet.Rmargin-regsheet.Lmargin;

      const int wTitle = (sTitle*wActive)/sActive;
      const int wTour  = (sTour*wActive)/sActive;
      const int wPages = (sPages*wActive)/sActive;

      const int Left = regsheet.Lmargin, Top = regsheet.Tmargin;

      aRegTitle = PrTextArea(Left, wTitle, paTopLeft);
      aRegTitle.area.top = Top;  aRegTitle.area.bottom = Top+HeadFontHeight;

      aRegTour  = aRegTitle + PrTextArea(0, wTour, paTopMid);
      aRegTour.area.top = Top;  aRegTour.area.bottom = Top+HeadFontHeight;

      aRegPage  = aRegTour + PrTextArea(0,  wPages, paTopMid);
      aRegPage.area.top = Top;  aRegPage.area.bottom = Top+HeadFontHeight;

      // TEMPLATE FOR COLUMNS
      regsheet.NewFont(lineFontHeight, lineFontWidth, pfNormal);

      LineFontHeight = regsheet.FontHeight;
      LineFontWidth  = regsheet.FontWidth;

      // WIDTH OF ROUNDS COLUMN - DEPENDS ...
      string strRoundp = string("9");		// Maximal width Pattern
      string strSpace  = spaces(1);

      for(size_t r=1; r<Rounds; r++)
         if(r<10)
            strRoundp += string(" 9");
         else
            strRoundp += string(" 10");

      int wRoundp;	regsheet.GetTextArea(strRoundp, wRoundp, hText);
      int wSpace;    regsheet.GetTextArea(strSpace,  wSpace,  hText);

      const int wRound = (wRoundp>wRounds ? wRoundp : wRounds);

      // WE CAN NOW SET UP THE TEXT AREAS EACH COLUMN
      aRegLine[riId]    = PrTextArea(Left, (sId*wActive)/sActive, paTopLeft);
      aRegLine[riReg]   = aRegLine[riId]    + PrTextArea(0,      (sReg*wActive)/sActive,  paTopLeft);
      aRegLine[riName]  = aRegLine[riReg]   + PrTextArea(2*wSpace,(sName*wActive)/sActive, paTopLeft);
      aRegLine[riGrade] = aRegLine[riName]  + PrTextArea(0,       wGrade, 						paTopMid);
      aRegLine[riClub]  = aRegLine[riGrade] + PrTextArea(2*wSpace,(sClub*wActive)/sActive, paTopLeft);
      aRegLine[riLand]  = aRegLine[riClub]  + PrTextArea(wSpace, (sLand*wActive)/sActive, paTopLeft);
      aRegLine[riRounds]= aRegLine[riLand]  + PrTextArea(wSpace, 	wRound, 						paTopLeft);
      aRegLine[riGroup] = aRegLine[riRounds]+ PrTextArea(2*wSpace,  wGroup, 					paTopMid);

      int wUsed = 2*wSpace, wNote;

      for(int f=0; f<riCount-1; f++)
      	wUsed += aRegLine[f].area.right - aRegLine[f].area.left + 1;

      wNote = wActive - wUsed;  if(wNote<0) wNote = 0;

      aRegLine[riNote]  = aRegLine[riGroup] + PrTextArea(2*wSpace,  wNote, paTopLeft);

      // LINE HEIGHTS
      LineHeight  = (ruFont*LineFontHeight)/rlFont;
      SpaceHeight = (ruSpace*LineFontHeight)/rlFont;

      textpos = regsheet.Tmargin + HeadHeight;

      for(int f=0; f<regCols; f++)
      {
      	aRegLine[f].area.top    = textpos;
         aRegLine[f].area.bottom = textpos + LineHeight;
      }

      // TEMPLATE FOR SECOND HEADER LINE
      aRegHeads[0]   = aRegLine[riId];
      aRegHeads[1]   = aRegLine[riName];
      aRegHeads[2]   = aRegLine[riGrade];
      aRegHeads[3]   = aRegLine[riClub];
      aRegHeads[4]   = aRegLine[riRounds];
      aRegHeads[5]   = aRegLine[riGroup];
      aRegHeads[6]   = aRegLine[riNote];

      aRegHeads[3].area.right = aRegLine[riLand].area.right;

      for(int f=0; f<regCols-1; f++)
      {
      	aRegHeads[f].area.top    = Top + 2*HeadFontHeight;
         aRegHeads[f].area.bottom = Top + 3*HeadFontHeight;
      }

      // BATCHES OF 5 PER PAGE
      batchcount = regsheet.BatchesPerPage(ruFont, rlFont, ruSpace, HeadHeight);

      // NUMBER OF PAGES
      int playersperpage = 5*batchcount;

      playercount = to_posint(RegPlayers.begin(),RegPlayers.end() );

      pagecount = (playercount + playersperpage - 1)/playersperpage;

      RegOfPages = spaces(1) + RegOf + spaces(1) + to_string(pagecount);

		return true;
   }

   static void PrRegHead
   (
   	const int PageNum,

      PrSheet& RegSheet
   )
   {
   	// ALL HEADER LINES BOLD
      RegSheet.NewFont(headFontHeight, headFontWidth, pfBold);

   	//FIRST HEADER LINE
   	RegPageCount = RegPage + spaces(1) + to_string(PageNum) + RegOfPages;

      aRegTitle.DrawText(RegTitle, RegSheet);
      aRegTour.DrawText(RegTour, RegSheet);
      aRegPage.DrawText(RegPageCount, RegSheet);

      // SECOND HEADER LINE
      for(int i=0; i<regCols-1; i++)
      	aRegHeads[i].DrawText(RegColText[i], RegSheet);
   }

   static bool PrRegLines
   (
   	const int page,
      const int PlayerCount, const int BatchCount, const int TextPos,

      TStringList* RegFile, PrSheet& RegSheet
   )
   {
   	using PrinterAPI::PrAlign;
      using PrinterAPI::paTopMid;

       // ALL REGISTER LINES NORMAL FONT
      RegSheet.NewFont(lineFontHeight, lineFontWidth, pfNormal);

      // BATCH CONTROL
      const int LinesPerPage = BatchCount*5;

      int FromPlayer = (page-1)*LinesPerPage + 1;
      int ToPlayer	 =  FromPlayer + LinesPerPage - 1;

      if(ToPlayer>=PlayerCount) ToPlayer = PlayerCount;

      int TextTopNow = TextPos, BatchLine = 0;

      // WIDE PEN
      HPEN hPen;	hPen = CreatePen(PS_SOLID, 3, 0);
                  	 if(hPen==NULL) return false;

      HGDIOBJ hOldPen; hOldPen = SelectObject(RegSheet.hDevice, (HGDIOBJ)hPen);
                      if(hOldPen==NULL) return false;

		const string strTick("R");
                            
      bool eSyntax = false;

      // PRINT ALL LINES IN PAGE
      for(int p=FromPlayer; p<=ToPlayer; p++)
      {
      	const string RawLine = S_s(RegFile->Strings[p+1]);

         string RegItems[riCount];

         if(unpack_tabbed(RawLine, riCount, RegItems) < riCount-2)	// Only Group and Note
         { eSyntax = true;	 break; }                                 // may be empty

      	for(int i=0; i<riCount; i++)
         {
         	PrTextArea iRegLine = aRegLine[i];

         	iRegLine.area.top    = TextTopNow;
            iRegLine.area.bottom = TextTopNow + (i!=riReg ? LineHeight : LineFontHeight/2);

            if(i != riReg)
            	iRegLine.DrawText(RegItems[i], RegSheet);
            else
            	if(RegItems[riReg]== strTick)
               	RegSheet.DrawTick(iRegLine.area);
         }

         ++BatchLine;

         if(BatchLine == 5)
         {
         	BatchLine = 0;

         	TextTopNow += LineHeight+SpaceHeight;
         }
         else
         	TextTopNow += LineHeight;
      }

      if(SelectObject(RegSheet.hDevice, hOldPen)==NULL) return false;

      if(!DeleteObject((HGDIOBJ)hPen)) return false;

      return !eSyntax;
   }


	void PrintRegister(bool PrDlg, String& RegPri)
   {
   	TStringList* RegFile = 0;

   	try
      {
         RegFile = new TStringList();

         RegFile->LoadFromFile(RegPri);

         PrinterAPI::PrSheet RegSheet;

         int PlayerCount, PageCount, BatchCount, TextPos;

         if(!AnalyseRegHeader(RegFile, RegSheet, PlayerCount, PageCount,BatchCount,TextPos))
         	throw(Exception(String()));

      	PrinterAPI::StartDoc(Tools::load_cstr(IDM_PrintRegister));

         for(int page=1; page<=PageCount; page++)
         {
         	PrinterAPI::StartPage();

         		PrRegHead(page, RegSheet);

               if(!PrRegLines(page, PlayerCount, BatchCount, TextPos, RegFile, RegSheet))
               	throw(Exception(String()));

         	PrinterAPI::EndPage();
         }

         PrinterAPI::EndDoc();

         delete RegFile;
      }
      catch(...)
      {
      	delete RegFile;

         PrinterAPI::Throw(ToolsGUI::erOperr, IDE_LoadFromFile, IDE_NoMemory);
      }
   }
}
//---------------------------------------------------------------------------



