////////////////////////////////////////////////////////////////////////////////
// MODULE  RegGUI:    Register Pane Support                     G.Kaniuk 05/04/02
//
#include <string>
#include <list>
#include <vector>
#include <stdlib.h>
#include <vcl.h>
#pragma hdrstop

#include "Tour.h"

#include "ETourRES.h"
#include "GoDrawRES.h"
#include "GoErrMsgRES.h"
#include "GoInfoRES.h"
#include "GoFormRES.h"
#include "EdPlayerRes.h"

#include "GraphicsAPI.h"
#include "Strings.h"
#include "Format.h"
#include "ETour.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "PrefsMaster.h"
#include "PlayerMaster.h"

#include "GoDrawGUI.h"

extern void __fastcall BeginCommand(int IdCmd); 				// IN GoDrawForm.cpp
extern void __fastcall EndCommand(int IdCmd);
extern void __fastcall EnableRegPanel(void);
extern void __fastcall DisableRegPanel(void);
extern void __fastcall EditPlayerAndFocus(const int player);
extern void __fastcall ResumePlayerAndFocus(void);
extern void __fastcall ChangeToPlayerAndFocus(void);
extern void __fastcall SetIdBoxItemIndex(const int row);
extern bool __fastcall ClosedPlayerEdit(const int idmessage);
extern bool __fastcall UnsavedPlayerEdit(const int idmessage);
extern bool __fastcall ValidPlayerEdit(void);

//extern void __fastcall ShowMsg(const char* msg);
//---------------------------------------------------------------------------
// MAGIC CONSTANTS FOR 600 x 800 RESOLUTION SCREEN
//---------------------------------------------------------------------------
namespace RegGUI
{
   using std::string;

  	using ToolsGUI::GGrid;
   using ToolsGUI::DrawAlignedText;

   using ToolsGUI::jyLeft;
   using ToolsGUI::jyCentre;
   using ToolsGUI::jyRight;

   using TourMaster::GoTour;
   using PlayerMaster::EditPlayerTube;
   using PlayerMaster::epiVirgin;
   
   static const int defRegHeight = 258, defRegWidth = 473;
   static const int defRegTop    = 105, defRegLeft  =  8;

   enum                                            // Minimum Column Widths
   {
      cwId = 35,  cwStatus=26,
      cwName=140, cwGrade=34, cwClub=74, cwLand=44, cwRounds=90,
      cwGroup=26, cwNote=80
   };

   static const GGrid::ColSize ColSizes[rfFields] =
   {
      { cwId,    0 },  { cwStatus, 0 },
      { cwName,  1 },  { cwGrade,  0 },
      { cwClub,  1 },  { cwLand,   1 },
      { cwRounds,0 },  { cwGroup,  0 },
      { cwNote,  1 }
   };

   static const int minColWidths[rfFields] =
   {
      cwId,    cwStatus,
      cwName,  cwGrade,  cwClub, cwLand, cwRounds,
      cwGroup, cwNote
   };

   static const bool adjCol[rfFields] =
   {
      false,   false,
      true,    true,     true,   true,   true,
      false,   true
   };

   static const int RegHeadID[rfFields] =          // The Register Header
   {
      IDL_Player, IDL_Regis,
      IDL_Name,   IDL_Grade, IDL_Club, IDL_Land,
      IDL_Rounds, IDL_Group, IDL_Note
   };

   static const int RegNewID[rfFields] =           // The new player line
   {
      IDL_New,    IDL_Reg,
      IDL_Name,   IDL_Grade, IDL_Club, IDL_Land,
      IDL_Rounds, IDL_Group, IDL_Note
   };

   static const int RegSort[rfFields] =            // First time sorting
   {
       1,   // Player number ascending
       1,   // First registered, hen the rest
       1,   // Natural Name order
      -1,   // Grade descending
       1,   // Natural club name order
       1,   // Natural country name order
       1,   // First Playing in all, then in all but one etc
      -1,   // Supergroup, McMahon, then individual
       1    // Lexical
   };

   static const ToolsGUI::Justify jyAlign[rfFields] =
   {
      jyRight, jyCentre,
      jyLeft,  jyCentre, jyCentre, jyCentre,
      jyLeft,  jyCentre, jyLeft
   };

   const string numFmt("0000");                   // Player Id

   const char* RegCodes = ".RJL";						// Fast analysis of Reg Cell

   string RegHead(int f)
   {
      if(f<0 || f>=rfFields) return string();

      return Tools::load_str(RegHeadID[f]);
   }
}
//---------------------------------------------------------------------------

namespace RegGUI                             // ERROR MANAGEMENT
{
	using Tools::load_str;
   using ToolsGUI::erOperr;
   using ToolsGUI::ShowMessage;
   using ToolsGUI::OperatorInfo;
   using ToolsGUI::OperatorError;
   using ToolsGUI::OperatorWarning;

   void Throw(ToolsGUI::EType etype, short idproc, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_RegGUI, idproc, idmsg);
   }
}
//---------------------------------------------------------------------------
// CLASS GRegGrid  CONSTRUCTION AND LOADING
//---------------------------------------------------------------------------
namespace RegGUI
{
   using ToolsGUI::GRatio;

   __fastcall GRegGrid::GRegGrid             // Startup constructor
   (
      TWinControl* parent,
      TStatusBar*  statusbar,
      TButton* fastregbtn
   )
   : ToolsGUI::GGrid(parent, statusbar, rfFields)
   {
          Top = defRegTop;
         Left = defRegLeft;
       Height = defRegHeight;
        Width = defRegWidth;

      OnExit  = OnRegGridExit;

      FastRegBtn = fastregbtn;

      RegOptions = Options;

      Options << goRangeSelect;

      RegRangeOptions = Options;

      RowCount = 2;

      Hint = LoadStr(IDF_HintFastReg);

      ShowHint = false;

      for(int f=0; f<rfFields; f++)
         ColWidths[f] = minColWidths[f];

      for(int f=0; f<rfFields; f++)
         Cells[f][0] = LoadStr(RegHeadID[f]);

      Cells[0][1] = String();

      for(int f=1; f<rfFields; f++)
         Cells[f][1] = LoadStr(RegNewID[f]);

      Cells[rfRounds][1] = "1  2  3";

      //OnSelectCell = SelectCellEvent;

      RTick = new Graphics::TBitmap();    RTick->Transparent = true;
      JTick = new Graphics::TBitmap();    JTick->Transparent = true;
      LTick = new Graphics::TBitmap();    LTick->Transparent = true;

      FTick = new Graphics::TBitmap();    FTick->Transparent = true;
      UTick = new Graphics::TBitmap();    UTick->Transparent = true;
      WTick = new Graphics::TBitmap();    WTick->Transparent = true;

      SortEnabled = true;

      for(int f=rfStatus; f<rfFields; f++)
         SortDir[f] = ToolsGUI::gsNo;

      SortDir[rfNumber] = ToolsGUI::gsUp;

      Status = rsDormant;

      EditPlayerId = 0;

      IgnoreClick = false;
   }

   __fastcall GRegGrid::~GRegGrid()
   {
   	delete RTick;
      delete JTick;
      delete LTick;
      delete FTick;

      delete UTick;
      delete WTick;
   }

   void __fastcall GRegGrid::OnRegGridExit(TObject *Sender)
   {
   	if(BlockSelected())
      	SelectGridCell(Selection.Left, Selection.Top);
   }

   void __fastcall GRegGrid::LoadSorted(int sortorder, int sortdir)		// GK REGSORT 2015
   {
   	RegMaster::RegSort(sortorder, sortdir);
      LoadFromTour();
      for(int f=rfNumber; f<rfFields; f++)
         SortDir[f] = f==sortorder ? sortdir : ToolsGUI::gsNo;
   }

   void __fastcall GRegGrid::ReBuild()                       				// ADJUST SHAPE
   {
   	using RegMaster::FindTube;

      ToolsGUI::GGrid::ReBuild(ColSizes);

      DrawTicks();
   }

   void __fastcall GRegGrid::ConnectGroups(TGroupBox* popgroup, TGroupBox* bargroup)
   {
   	PopGroup = popgroup;
      BarGroup = bargroup ;
   }

   void __fastcall GRegGrid::StartSortHeader(void)
   {
   	SortEnabled = true;

      for(int f=rfStatus; f<rfFields; f++)
         SortDir[f] = ToolsGUI::gsNo;

      SortDir[rfNumber] = ToolsGUI::gsUp;

      GoTour.SortOrder = Tours::skEntry;
      GoTour.SortSense = ToolsGUI::gsUp;
   }

   void __fastcall GRegGrid::LoadPlayer(Tours::GPlayerPtr P, const int row)
   {
   	using std::string;

      using Tools::s_S;
      using Tools::to_string;
      using Tools::sSpace;

   	Cells[rfNumber][row] = s_S(to_string(numFmt, (int)P->Pid) + sSpace);
      Cells[rfStatus][row] = s_S(GoTour.Registered(P));

      Cells[rfName][row]   = s_S(P->Name);
      Cells[rfGrade][row]  = s_S(GoTour.Grade(P));
      Cells[rfClub][row]   = s_S(GoTour.Club(P));
      Cells[rfLand][row]   = s_S(GoTour.Land(P));

      Cells[rfRounds][row] = s_S(GoTour.Rounds(P));
      Cells[rfGroup][row]  = s_S(GoTour.Group(P));
      Cells[rfNote][row]   = s_S(P->Note);
   }

   String __fastcall GRegGrid::ShortPlayer(const int row)
   {
      const String Space = String("  ");

   	String PlayerLine = Cells[rfNumber][row] + Space;

      if(row==RowCount-1)
      {
      	std::string playernum = Tools::to_string(numFmt, row);

      	return Tools::s_S(playernum);
      }

      switch(GoTour.SortOrder)
      {
      	case Tours::skEntry:                                      break;
      	case Tours::skTicked: PlayerLine += Cells[rfStatus][row]; break;
      	case Tours::skGrade:  PlayerLine += Cells[rfGrade][row];  break;
         case Tours::skClub:   PlayerLine += Cells[rfClub][row];   break;
         case Tours::skLand:   PlayerLine += Cells[rfLand][row];   break;
         case Tours::skRounds: PlayerLine += Cells[rfRounds][row]; break;
         case Tours::skGroup:  PlayerLine += Cells[rfGroup][row];  break;
         case Tours::skNote:   PlayerLine += Cells[rfNote][row];   break;

         default: return PlayerLine;
      }

      PlayerLine += Space + Cells[rfName][row];

      return PlayerLine;
   }
}
//------------------------------------------------------------------------------
// DRAWING
//------------------------------------------------------------------------------
namespace RegGUI
{
   enum
   {
      rtNoReg, rtReg, rtJoin, rtLeave					// Values for Registered
   };

   enum
   {
   	ftFree,
   	ftSelect
   };

   const static TPoint TickPoints[6] =
   {
      Point( 1,8), Point(6,13), Point(15,4),
      Point(13,2), Point(6, 9), Point( 3,6)
   };

   const static int wTick = 16, hTick = 13;


   static void __fastcall DrawTick
   (
   	int Selected,						// Whether Free or Selected
      int Registered,               // Tick or blob
      int oL, int oT,               // Offsets
      Graphics::TBitmap* BitMap
   )
   {
      TRect Area(0,0, BitMap->Width, BitMap->Height);

      TColor OldBrushColor = BitMap->Canvas->Brush->Color;

      BitMap->TransparentColor     = PrefsMaster::PrefsTube.PaneColor;

      if(Selected)
      	BitMap->Canvas->Brush->Color = PrefsMaster::PrefsTube.RegSelColor;
      else
     		BitMap->Canvas->Brush->Color = PrefsMaster::PrefsTube.PaneColor;

      BitMap->Canvas->FillRect(Area);

      if(Registered)
      {
         int TickW = BitMap->Width, TickH = (8*BitMap->Height)/10;

         if(TickW < TickH) TickH = TickW;    TickW = (TickH*wTick)/hTick;

         int Left = (BitMap->Width - TickW)/2;    if(Left<0) Left = 0;
         int Top  = (BitMap->Height - TickH)/2;   if(Top<0)  Top  = 0;

         Left += oL;    Top += oT;

         TPoint TickPts[6];

         for(int k=0; k<6; k++)
         {
            TPoint P = TickPoints[k];

            TickPts[k] = Point(Left+(TickW*P.x)/wTick, Top+(TickH*P.y)/hTick);
         }

         TColor clTick;

         switch(Registered)
         {
         	case rtReg:   clTick = clYellow; break;
            case rtJoin:  clTick = clBlack;  break;
            case rtLeave: clTick = clWhite;  break;

            default: 	  clTick = clGreen;  break;
         }

         BitMap->Canvas->Brush->Color = clTick;
         BitMap->Canvas->Polygon(TickPts, 5);
      }
      else
      {
         int BlobW = (5*BitMap->Width)/16, BlobH = (5*BitMap->Height)/16;

         if(BlobH<BlobW)
            BlobW = BlobH;
         else
            BlobH = BlobW;

         int Left = (BitMap->Width - BlobW)/2 + oL;
         int Top  = (BitMap->Height - BlobH)/2 + oT;

         BitMap->Canvas->Brush->Color = clYellow;
         BitMap->Canvas->Ellipse(Left, Top, Left+BlobW-1, Top+BlobH-1);
      }
      BitMap->Canvas->Brush->Color = OldBrushColor;
   }

   void __fastcall GRegGrid::DrawTicks(void)
   {
      const int CellW = ColWidths[rfStatus];
      const int CellH = RowHeights[1];

      TRect RegCell = TRect(0,0,CellW, CellH);

      // Simple tick for grid
      RTick->Width = CellW;   RTick->Height = CellH;
      DrawTick(ftFree, rtReg, 2, -1, RTick);

      // Player Joining Draw
      JTick->Width = CellW;   JTick->Height = CellH;
      DrawTick(ftFree, rtJoin, 2, -1, JTick);

      // Player Leaving Draw
      LTick->Width = CellW;   LTick->Height = CellH;
      DrawTick(ftFree, rtLeave, 2, -1, LTick);

      // Focused tick for RegEd
      FTick->Width = CellW;   FTick->Height = CellH;
      DrawTick(ftSelect,rtReg, 2, -1, FTick);
      //FTick->Canvas->DrawFocusRect(RegCell);


      // Unfocused unregistered for RegGrid
      WTick->Width = CellW;   WTick->Height = CellH;
      DrawTick(ftFree, rtNoReg, 2, -1, WTick);

      // Focused unregistered for RegEd
      UTick->Width = CellW;   UTick->Height = CellH;
      DrawTick(ftSelect, rtNoReg, 2, -1, UTick);
   }

   void __fastcall GRegGrid::DrawReg(bool selected, int row, const TRect& cell)
   {
   	const bool FastReg = Status == rsFastReg;

      const String RegStatus =  Cells[rfStatus][row];

      if(RegStatus.IsEmpty())	// Unregistered
      {
         if(selected && FastReg)
            Canvas->Draw(cell.left,cell.top, UTick);
         else
            Canvas->Draw(cell.left,cell.top, WTick);

      	return;
      }

      const char RegChar = RegStatus[1];

      for(int i=1; i<4; i++)
      	if(RegChar == TourMaster::RegCode[i][0])
            switch(i)
            {
               case 1:	// Registered
               	if(selected && FastReg)
                     Canvas->Draw(cell.left,cell.top, FTick);
                  else
                     Canvas->Draw(cell.left,cell.top, RTick);
               return;

               case 2:	// Joining
               	Canvas->Draw(cell.left,cell.top, JTick);
               return;

               case 3:	// Leaving
               	Canvas->Draw(cell.left,cell.top, LTick);
               return;

               default:	return;
            }
   }

   void __fastcall GRegGrid::DrawHeading(int col, int row, const TRect &cell)
   {
      DrawAlignedText(Canvas->Handle, jyCentre, cell, Cells[col][row]);

      int SortColDir = GRegGrid::SortDir[col];

      int Ttop = cell.Top + 2;
      int Tleft = cell.Right - SortUp->Width - 2;

      if(SortColDir == ToolsGUI::gsDn) Canvas->Draw( Tleft, Ttop, SortDn);
      if(SortColDir == ToolsGUI::gsUp) Canvas->Draw( Tleft, Ttop, SortUp);
   }

   void __fastcall GRegGrid::DrawNewPlayer(int col, int row, const TRect &cell)
   {
      using PrefsMaster::PrefsTube;
      
      TColor      OldColor = Canvas->Font->Color;
      TFontStyles OldStyle = Canvas->Font->Style;

      Canvas->Font->Color =  GoTour.PastRound() ? clGrayText : PrefsTube.EntryColor;

      if(Cells[0][row].IsEmpty())
         Canvas->Font->Style = TFontStyles()<<fsItalic;
      else
         Canvas->Font->Style = TFontStyles();

      DrawAlignedText(Canvas->Handle, jyCentre, cell, Cells[col][row]);

      Canvas->Font->Color = OldColor;
      Canvas->Font->Style = OldStyle;
   }

   void __fastcall GRegGrid::DrawSelection(int row, TRect Cell)
   {
      TFontStyles OldStyle = Canvas->Font->Style;

      Canvas->Font->Style  = TFontStyles();

      if(row == EditPlayerTube.PlayerRow)
      	if(EditPlayerTube.State != epiVirgin)
         	Canvas->Font->Style  = TFontStyles()<<fsBold;

      Cell.Right -= 2;

      DrawAlignedText(Canvas->Handle, jyRight, Cell, Cells[rfNumber][row]);

      Canvas->Font->Style  = OldStyle;
   }

   void __fastcall GRegGrid::DrawName(const TColor clBack, const int row, TRect Cell)
   {
      Cell.Left += 2;

      DrawAlignedText(Canvas->Handle, jyAlign[rfName], Cell, Cells[rfName][row]);
   }

   void __fastcall GRegGrid::DrawCell
   (
      int col, int row, const TRect &cell,
      TGridDrawState state
   )
   {
      using ToolsGUI::Justify;
      using PrefsMaster::PrefsTube;

      if(col<0 || row<0) return;

      if(row==0){ DrawHeading(col, row, cell); return; }

      if(row==RowCount-1){ DrawNewPlayer(col, row, cell); return; }

      const bool selected = state.Contains(gdSelected) && col>0;

   	const TColor clBack = selected ? PrefsTube.RegSelColor:PrefsTube.PaneColor;

      if(0<row && row <  RowCount-1)
      switch(col)
      {
         case rfNumber: DrawSelection(row, cell);  	 break;
         case rfStatus: DrawReg(selected, row,  cell); break;
         case rfName:   DrawName(clBack, row, cell);   break;

         default:
            DrawAlignedText(Canvas->Handle, jyAlign[col], cell, Cells[col][row]);
      }
   }
}

//------------------------------------------------------------------------------
// CLICK
//------------------------------------------------------------------------------
namespace RegGUI
{
   void __fastcall GRegGrid::MouseDown          // HANDLER MouseDown
   (                                            // Click on Col or Row 0
      TMouseButton Button, TShiftState Shift,
      int i, int j
   )
   {
      // TGridCoord CellNow = MouseCoord(i, j);
      // const int col = CellNow.X, row = CellNow.Y;
      // GGrid::MouseDown will manage any clicks on row 0 to sort.
      // Otherwise it just calls TStringGrid::MouseDown to ultimately
      // call GRegGrid::Click

      GGrid::MouseDown(Button,Shift, i,j);
   }

   void __fastcall GRegGrid::Click(void)
   {
   	using ToolsGUI::ResetToEditCell;
      using TourMaster::RegCode;

      // SOME KEYS GENERATE A SPURIOUS CLICK
      if(IgnoreClick)
      {
      	IgnoreClick = false;	return;
      }

      // NOTHING TO EDIT IF NO TOURNAMENT SELECTED
      if(GoTour.State == Tours::tsVirgin)
      {
      	OperatorError(IDI_NoPlayerEd);

         return;
      }

      // THE Tag IS SET BY EnableCriticalControls FOR EACH GRID
      const bool CannotEdit = !EditPossible() || (!Tag  && GoTour.Round>0);

      if(CannotEdit)					 // Editing forbidden by State
      {                           // Or already editing something else
      	ResetToEditCell();

         ClearInfo();

         OperatorError(IDI_NoRegEd);

         return;
      }

      // WE DONT ALLOW ADDING PLAYERS WHILE VIEWING PAST ROUND
      if(Row==RowCount-1)
      	if(GoTour.PastRound())
         {
            OperatorError(IDI_NoAddPast);

            return;
         }

      // BLOCK SELECTION IS USED FOR DELETE
      const bool RowSelected = Selection.Left != Selection.Right;
      const bool ColSelected = Selection.Top  != Selection.Bottom;

      if(RowSelected && !ColSelected)				// Just a single row
		{
      	if(Selection.Bottom == RowCount-1)
            NoCellSelected();       				// Cannot select the last row

      	return;
      }

      if(ColSelected)									// We have a block or a single column
      {
      	if(Selection.Bottom == RowCount-1)
         {
         	TGridRect NewSelection = Selection;

         	NewSelection.Bottom = RowCount-2;	// Excise last row from selection

            Selection = NewSelection;
         }

         return;
      }

		// WE HAVE CLICKED A SINGLE CELL



      // THIS IS THE ONLY COLUMN WHERE A CLICK DOES SOMETHING
      // DIRECT TO THE REGISTER
      if(Col == rfStatus)
      {
         if(Row == RowCount-1)return;		// Nothing happens here

      	// CANNOT CHANGE REGISTRATION AFTER CLOSE
         if(GoTour.State>=Tours::tsRegClosed)
         {
            OperatorError(IDI_NoRegStatus);	return;
         }

         // CLOSE THE EDIT FORM IF REGISTERING ON THE GRID
         const int ClickedRow = Row;

         if(::ClosedPlayerEdit(IDB_PlayerChanged))
         {
         	SelectGridCell(rfStatus,ClickedRow);

         	ToggleRegStatus(Row);
         }
         else
         	return;

         if(Status==rsFastReg)ShowInfo(IDB_FastRegKeys);

         return;
      }
      else
      	if(Status==rsFastReg)
         {
      		OperatorInfo(IDF_ClickFastReg);

            return;
         }

      // USER MAY HAVE CLICKED ON A DIFFERENT ROW WHILE ALREADY EDITING.
      // WE OFFER TO SAVE FIRST BEFORE MOVING.
      StartPlayerEdit();
   }
}
//------------------------------------------------------------------------------
// KEYS
//------------------------------------------------------------------------------
namespace RegGUI
{
   void __fastcall GRegGrid::KeyDown(Word &Key, TShiftState ShiftState)
   {
   	using TourMaster::cmdDel;
      using TourMaster::EnabledCommands;

      if(Col<0 || Row<0) { Key = 0; return; }

      if(Status==rsFastReg)
      	ShowInfo(IDB_FastRegKeys);
      else
         ClearInfo();

      switch(Key)
      {
         case VK_RETURN:
            Click();
         break;

         case VK_ESCAPE:
         {
            TStringGrid::KeyDown(Key, ShiftState);    // Remove selection if any
                                                		// Sets Key to zero
            if(BlockSelected())
            	SelectGridCell(Selection.Left, Selection.Top);

            if(Col==rfStatus && Status==rsFastReg)
            	EscapeFastReg();

            InvalidateRow(EditPlayerId);
         }
         break;

         case VK_UP: case VK_DOWN: case VK_LEFT: case VK_RIGHT: case VK_TAB:
         {
            	IgnoreClick = true;

               InvalidateCell(Col,Row);
            		TStringGrid::KeyDown(Key, ShiftState);
               InvalidateCell(Col,Row);

               Repaint();
			}
         return;

         case VK_NEXT: case VK_PRIOR:
         {
         	IgnoreClick = true;

         	TStringGrid::KeyDown(Key, ShiftState);

            if(Row == RowCount-1)
            	SelectGridCell(Col, Row-1);
         }
         return;

         case VK_DELETE:
    			if
            (
            				Status != rsFastReg
                        		 &&
                  EnabledCommands[cmdDel][GoTour.State]
            )
            	if(::ClosedPlayerEdit(IDI_DelPlayerEd))
               {
                  ::BeginCommand(cmdDel);
                     DeleteSelectedPlayers();
                  ::EndCommand(cmdDel);
               }
               else
               	break;
            else
            	OperatorError(IDI_NoDeletion);
         break;

         case VK_R:
         {
         	Key = 0;

            if(Col == rfStatus)
            {
            	ToggleRegStatus(Row);

               if(Status==rsFastReg)NextFastReg();
            }

            return;
         }

         default: Key = 0; return;
      }
      Key = 0;
   }

   void __fastcall GRegGrid::ManageFastReg(void)
   {
   	switch(Status)
      {
         case rsDormant:
         {
      		if(!::ClosedPlayerEdit(IDB_FastRegEditing))return;

         	if(GoTour.RegCount == (int)GoTour.Players)
      		{
      			OperatorInfo(IDB_AlreadyReg);	return;
      		}

         	::DisableRegPanel();

         	FastRegBtn->Caption = LoadStr(IDF_EndFastReg);
            FastRegBtn->Enabled = true;

            StartFastReg();
         }
         break;

         case rsFastReg:
         	EscapeFastReg();
         break;

         default:
         	ShowMessage(ToolsGUI::erBug, Tools::load_str(IDF_BadFastReg));
      }
   }

   void __fastcall GRegGrid::StartFastReg(void)
   {
      // SELECT FIRST UNREGISTERD PLAYER
      SelectGridCell(rfStatus,0);

      Status = rsFastReg;

      NextFastReg();


      // WE ARE NOW IN FAST REG MODE
      SortEnabled = false;

      EditPlayerId = Row;

      OperatorInfo(IDB_FastRegMode);

      ShowHint = true;
   }

   bool __fastcall GRegGrid::NextFastReg(void)
   {
      if(Status != rsFastReg)return false;

      // IDENTIFY NEXT UNREGISTERED PLAYER
      const int PlayerCount = GoTour.Players;

      int Player = Row + 1;

      if(Player > PlayerCount) Player = 1;

      bool FoundPlayer = false;

      for(int p=Player; p<=PlayerCount; p++)
      	if(Cells[rfStatus][p].IsEmpty())
         {
         	Player = p;  FoundPlayer = true;

            break;
         }

      if(!FoundPlayer)
      {
      	if(Player != 1)
         {
         	if(GoTour.RegCount == PlayerCount)
            {
            	OperatorInfo(IDB_AllReg);

               EscapeFastReg();

               return false;
            }

            bool Cancel = OperatorWarning(IDB_RegSearched);

            if(Cancel)
            {
            	EscapeFastReg();

               return false;
            }
            else
            {
            	SelectGridCell(rfStatus,0);

               return NextFastReg();
            }
         }

         if(Player == 1)
         {
         	OperatorInfo(IDB_AllReg);

            EscapeFastReg();

            return false;
         }
      }

      // FOCUS REGISTER CELL
      InvalidateCell(rfStatus,Row);
      	SelectGridCell(rfStatus, Player);
         ExposeSelectedCell();
         SetFocus();
      InvalidateCell(rfStatus,Row);

      // KEEP REMINDING USER HOW TO STOP FAST REG
      ShowInfo(IDB_FastRegKeys);
      
      return true;
   }

   void __fastcall GRegGrid::EscapeFastReg(void)
   {
   	ShowHint = false;

      Status = rsDormant;    EditPlayerId = 0;

      SortEnabled = true;

      ShowInfo(IDI_NormalEd);

      InvalidateCell(rfStatus,Row);

      SetFocus();

      FastRegBtn->Caption = LoadStr(IDF_StartFastReg);

      ::EnableRegPanel();
   }
}
//------------------------------------------------------------------------------
// TOOLS
//------------------------------------------------------------------------------
namespace RegGUI
{
   void __fastcall GRegGrid::InvalidateRegRow(int row)
   {
   	if(0<=row && row<RowCount)
      	InvalidateRow(row);
   }

   void __fastcall GRegGrid::NoDownArrow(void)
   {
      ShowInfo(IDI_EndPlayer);
   }

   void __fastcall GRegGrid::SortColumn(int col)
   {
   	using ToolsGUI::EditCell;

      // DISMOUNT THE CURRENT EDIT FIELD - IT IS GOING TO MOVE
      int PlayerNum  = 0;
      if(Status != rsDormant)
      	PlayerNum = GoTour.Sorted[Row];

      // SORT BY THE COLUMN
		if(col==GoTour.SortOrder)						// Click on current sorted order
      	GGrid::SortColumn(col);						// Flip the sort direction
      else
      	SortDir[col] = ToolsGUI::gsUp;			// New col, start ascending

      LoadSorted(col, SortDir[col]); 				// GK REGSORT 2015

      // FIND WHERE FIELD GONE AND REMOUNT IT
      for(int row=1; row<RowCount; row++)
      	if((int)GoTour.Sorted[row] == PlayerNum)				// PlayerNum=0 if NOT editing
         {
            EditPlayerTube.PlayerRow = row;

            ::SetIdBoxItemIndex(row);

            SelectGridCell(CurFieldId, row);						// Tell grid new location

            ExposeSelectedCell();
         }
   }

   String __fastcall GRegGrid::FullRounds(void)
   {
      int Rounds = GoTour.Rules.Rounds, Round = GoTour.Round;

      std::vector<bool>playing(Rounds);

      for(int r=0; r<Rounds; r++)	  	// Do not play in any past round including this one
      	playing[r] =  r >= Round;  	// Do play in future rounds

      string rtext;

      RegMaster::rounds_string(playing,  rtext);

      return Tools::s_S(rtext);
   }

   void __fastcall GRegGrid::SetHeaderRow(void)
   {
      for(int f=0; f<rfFields; f++)
         Cells[f][0] = LoadStr(RegHeadID[f]);
   }

   void __fastcall GRegGrid::SetNewPlayerRow(void)
   {
      for(int f=rfName; f<rfFields; f++)
      {
         Cells[f][RowCount-1] = LoadStr(RegNewID[f]);
      }
      Cells[rfNumber][RowCount-1] = String();
      Cells[rfStatus][RowCount-1] = String();
      Cells[rfRounds][RowCount-1] = FullRounds();
   }
}
//------------------------------------------------------------------------------
// EDITING
//------------------------------------------------------------------------------
namespace RegGUI
{
	void __fastcall GRegGrid::StartPlayerEdit(void)
   {
   	using PlayerMaster::EditPlayerTube;

   	EditPlayerTube.PlayerCol = Col;

      switch(Status)
      {
      	case rsDormant:
         {
            Status = rsEditing;

            ::EditPlayerAndFocus(Row);
            
            InvalidateRegRow(Row);
         }
         break;

         case rsEditing:
         {
            const int EdRow = EditPlayerTube.PlayerRow;

            if(!::ValidPlayerEdit())					// Return to offending field
            	return;

            if(Row == EdRow)								// Clicked on same row already loaded
               ::ResumePlayerAndFocus();
            else												// Clicked on a different row
               ::ChangeToPlayerAndFocus();			// Gets warned if text changed
         }
         break;

         default: return;
      }
   }

   void __fastcall GRegGrid::EndPlayerEdit(bool NewPlayer)
   {
      using Tools::s_S;
      using Tools::to_string;

      if(Status == rsFastReg) return;

      EditPlayerId = 0;

      Status = rsDormant;

      if(NewPlayer)
      {
         RowCount = RowCount+1;

         // MIGHT BE THE FIRST PLAYER ENTERED
         if(GoTour.State == Tours::tsTourStart)
         	GoTour.State =  Tours::tsRegNow;

         // FILLING THE REGISTER WINDOW TRIGGERS A REBUILD
         if(GoTour.Players==(size_t)BodyRowCount())
         	ReBuild();

         RowHeights[RowCount-1] = RowHeights[RowCount-2];

         // PREPARE FOR NEXT NEW PLAYER
         SetNewPlayerRow();

         SelectGridCell(rfName, RowCount-1); ExposeSelectedCell();

         LeftCol = 1;
      }
      Repaint();
   }

   bool __fastcall GRegGrid::DeleteSelectedPlayers()
   {
      int RowStart = Selection.Top, RowEnd = Selection.Bottom;

      int RegTopRow = TopRow, RegLeftCol = LeftCol;

     	int IDI_Report = 0;

      // YOU CANNOT ALWAYS DELETE
      while(true)
      {
      	if(RowStart<0)
         	{ IDI_Report = IDI_NoDel; break; }

         TGridRect FullSelect;
         FullSelect.Top = RowStart;  FullSelect.Bottom = RowEnd;
         FullSelect.Left = rfStatus; FullSelect.Right = rfNote;

         Selection = FullSelect;

         if(RowEnd == RowCount-1) --RowEnd;

         if(RowEnd<RowStart){ IDI_Report = IDI_NoDelAllowed; break; }

         const int Round = GoTour.Round;

         for(int r = RowStart; r<= RowEnd; r++)
         {
            if(GoTour.ActivePlayer(GoTour.Sorted[r]))
            {
               IDI_Report = IDI_NoDelActive;	break;
            }

            Tours::GPlayerPtr Player = GoTour.GetPlayerPtr(r);

            if(Round>0)
            {
               if(Player->Joining(Round)){ IDI_Report = IDI_NoDelJoin;  break; }
               if(Player->Leaving(Round)){ IDI_Report = IDI_NoDelLeave;	break; }
            }
         }
         break;
      }

      if(IDI_Report)
      {
      	OperatorError(IDI_Report);

         SelectGridCell(rfStatus, Row);

         return false;
      }

      string DelWarnMessage;

      const size_t Range = RowEnd-RowStart+1;

      if(Range==1)
      	DelWarnMessage = load_str(IDI_DelCurrent);
      else
      	if(Range < GoTour.Players)
      		DelWarnMessage = load_str(IDI_DelSelected);
         else
            DelWarnMessage = load_str(IDI_DelAll);

      DelWarnMessage += Tools::load_str(IDI_DelOK);
      DelWarnMessage += Tools::load_str(IDI_DelCancel);

      if(ShowMessage(ToolsGUI::erWarn, DelWarnMessage)==IDOK)
      try
      {
      	RegMaster::delete_players(RowStart, RowEnd);

      	LoadFromTour();

         LeftCol = RegLeftCol;						// Reposition Col

         if(RowCount>BodyRowCount()+1)				// Keep TopRow fixed
            TopRow = RegTopRow;
         else												// Scroll to top
            if(RowCount>1)
               TopRow = 1;

         GoDraw::SetPopulations(PopGroup);		// Display new population count
         GoDraw::SetBars(BarGroup);

         SelectGridCell(rfStatus, RowStart);

         if(Range==1)
      		ShowInfo(IDI_DelPlayer);
      	else
      		if(Range < GoTour.Players)
      			ShowInfo(IDI_DelPlayers);
         	else
            	ShowInfo(IDI_DelAllPlayers);

         return true;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_RegGUI, IDE_DeleteSelectedPlayers);

         return false;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_RegGUI, IDE_DeleteSelectedPlayers);

         return false;
      }
      else
      {
      	SelectGridCell(rfStatus, Row);

      	return false;
      }
   }

   void __fastcall GRegGrid::ToggleRegStatus(const int row)
	{
   	if(row <= 0 || row>= RowCount-1) return;

      const int iReg = RegMaster::toggle_reg_status(row);

      if(iReg<0)
      {
         OperatorError(IDB_NoToggleReg); return;
      }

      ::BeginCommand(TourMaster::cmdEnter);
      	GoTour.PopulationCount();

      	Cells[rfStatus][row] = String(TourMaster::RegCode[iReg]);

      	Repaint();
      ::EndCommand(TourMaster::cmdEnter);

      return;
	}
}
//---------------------------------------------------------------------------
#pragma package(smart_init)











