////////////////////////////////////////////////////////////////////////////////
// MODULE  RegGUI    Register Support                     G.Kaniuk 05/04/02
//
#include <string>
#include <grids.hpp>
#include <classes.hpp>

#include "Tour.h"
#include "ETour.h"
#include "GridGUI.h"
#include "EditPlayerForm.h"

#ifndef RegGUIH
#define RegGUIH

namespace RegGUI
{
   using std::string;

   void Throw(ToolsGUI::EType etype, short idproc, short idmsg);

   enum                    // SPECIAL KEYS
   {
      VK_P = 0x50,         // Toggle Playing in GRounds
      VK_R = 0x52          // Toggle Registered in RegGrid
   };

   enum                    // REGISTER STATUS CONTROLS DISPLAY
   {
      rsDormant,           // No active field
      rsEditing,           // Entering new player or editing old
      rsFastReg            // Auto advance after reg
   };

   enum GRegFields
   {
      rfNumber, rfStatus,
      rfName,   rfGrade, rfClub, rfLand, rfRounds,
      rfGroup,  rfNote,
      rfFields
   };

   extern const string numFmt;                   // Player Id

   string RegHead(int f);
}

namespace RegGUI
{
   class GRegGrid : public ToolsGUI::GGrid
   {
   	private:
         Graphics::TBitmap* RTick;  // RegGrid ticked
         Graphics::TBitmap* FTick;  // Register field focused tick
         Graphics::TBitmap* UTick;  // Register field focused unticked
         Graphics::TBitmap* WTick;  // RegGrid unticked

         Graphics::TBitmap* JTick;  // Player Joins
         Graphics::TBitmap* LTick;  // Player Leaves

      public:
         TGridOptions RegOptions;         // Modified for fast reg mode
         TGridOptions RegRangeOptions;    // Allow range select

         bool IgnoreClick;                // In fast reg mode
         int EditPlayerId;

      public:
         int Status;

         TGroupBox *PopGroup, *BarGroup;

         TButton* FastRegBtn;
         
      public:
         __fastcall GRegGrid(TWinControl* parent, TStatusBar* statusbar, TButton* fastregbtn);
         __fastcall ~GRegGrid();

         void __fastcall OnRegGridExit(TObject *Sender);

         void __fastcall LoadPlayer(Tours::GPlayerPtr P, const int row);

         void __fastcall LoadSorted(int sortorder, int sortdir);				// GK REGSORT 2015
         void __fastcall LoadFromTour();
         void __fastcall LoadNewRounds();
         void __fastcall LoadNewGroups();
         void __fastcall UpdateRegLists();
         void __fastcall ReBuild();
         
         void __fastcall ConnectGroups(TGroupBox* popgroup, TGroupBox* bargroup);

      private:
         void __fastcall DrawHeading(int col, int row, const TRect &cell);
         void __fastcall DrawNewPlayer(int col, int row, const TRect &cell);
         void __fastcall DrawSelection(int row, TRect cell);
         void __fastcall DrawReg(bool selected, int row, const TRect& cell);
         void __fastcall DrawName(const TColor clback, int row, TRect Cell);
         void __fastcall DrawTicks(void);

         void __fastcall DrawCell
         (
            int col, int row,                // Identify cell
            const Windows::TRect &rect,      // Cell geometry
            Grids::TGridDrawState state      // Whether selected
         );

         DYNAMIC void __fastcall KeyDown(Word &Key, TShiftState Shift);
			DYNAMIC void __fastcall Click(void);
         DYNAMIC void __fastcall MouseDown(TMouseButton Button, TShiftState Shift, int X, int Y);

      public:
      	void __fastcall StartPlayerEdit(void);
      	void __fastcall EndPlayerEdit(bool NewPlayer);
         void __fastcall ToggleRegStatus(const int row);
         void __fastcall NoDownArrow(void);
         void __fastcall SortColumn(int col);

         void __fastcall ManageFastReg(void);
         void __fastcall StartFastReg(void);
         bool __fastcall NextFastReg(void);
         void __fastcall EscapeFastReg(void);


      public:
      	String __fastcall FullRounds(void);
         void __fastcall StartSortHeader(void);
         void __fastcall SetHeaderRow(void);
         void __fastcall SetNewPlayerRow(void);

         void __fastcall FocusEditCell(void);

         void __fastcall InvalidateRegRow(int row);

         void __fastcall PackageLists(void);

         bool __fastcall DeleteSelectedPlayers(void);

         String __fastcall ShortPlayer(const int row);
   };
}
//---------------------------------------------------------------------------
#endif




