////////////////////////////////////////////////////////////////////////////////
// MODULE  RegListsGUI  Loading register listboxes     G.Kaniuk 30/04/03
//

#include <string>                                                     
#include <list>
#include <vector>
#include <vcl.h>
#pragma hdrstop

#include "Strings.h"
#include "Format.h"
#include "ETourRES.h"
#include "GoDrawRES.h"
#include "GoErrMsgRES.h"
#include "GoInfoRES.h"

#include "Tour.h"
#include "ByteStream.h"
#include "TourStream.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "PrefsMaster.h"

#include "GoDrawGUI.h"
#include "PairPaneGUI.h"
#include "FileMUI.h"

#include "RegListsGUI.h"

extern void __fastcall LoadPlayerIDs(void);

#pragma package(smart_init)
//---------------------------------------------------------------------------

namespace RegGUI                          // PARSING TOOLS
{
   using std::string;
   using std::list;
   using std::vector;

   using Tools::unpack_tabbed;
   using Tools::pack_tabbed;

   using Tools::load_str;

   using Tools::S_s;
   using Tools::s_S;

   // using ToolsGUI::erBug;					// REG IMPORT Require graceful exception GK 6.5.1
   using ToolsGUI::erOperr;

   void ThrowSyntax(int filetype,  string filename, int line)
   {
      string message = Tools::load_str(filetype)+string(Tools::sCrLf) +
                       string(3, Tools::cSpace) + filename+string(Tools::sCrLf) +
                       Tools::load_str(IDE_BadSyntax)+ Tools::to_string(line);

      throw ToolsGUI::ETour(erOperr, message);
   }

   static void LowerGradeSuffix(string& GradeText)
   {
   	const int Suffix = GradeText.size()-1;

      GradeText[Suffix] = Tools::lowercase(GradeText[Suffix]);
   }

}
//---------------------------------------------------------------------------

namespace RegGUI                              // MAIN FORM INTERFACE
{
   void __fastcall GRegGrid::LoadFromTour()
   {
      using std::string;

      using Tools::s_S;
      using Tools::to_string;
      using Tools::sSpace;

      using TourMaster::GoTour;

      // ROWS CONTAIN HEADER + PLAYERS + NEWPLAYER
      if(RowCount != (int)GoTour.Players + 2)
      	RowCount = GoTour.Players + 2;

      // LOAD THE REGISTER GRID CELLS
      for(size_t row=1; row<=GoTour.Players; row++)
      {
         Tours::GPlayer* P = GoTour.GetPlayerPtr(row);

         LoadPlayer(P, row);
      }

      // THE LAST ROW OF THE GRID IS THE NEW PLAYER ENTRY
      SetNewPlayerRow();

      ReBuild();

      NoCellSelected();

      // UPDATE EditPlayerFrm ID's
      ::LoadPlayerIDs();
   }

   void __fastcall GRegGrid::LoadNewRounds()
   {
      using std::string;

      using Tools::s_S;

      using TourMaster::GoTour;

      for(size_t row=1; row<=GoTour.Players; row++)
      {
         Tours::GPlayer* P = GoTour.GetPlayerPtr(row);

         Cells[rfRounds][row] = s_S(GoTour.Rounds(P));
      }

      SetNewPlayerRow();
   }

   void __fastcall GRegGrid::LoadNewGroups()
   {
      using TourMaster::GoTour;

      for(size_t row=1; row<=GoTour.Players; row++)
      {
         Tours::GPlayerPtr P = GoTour.GetPlayerPtr(row);

         Cells[rfGroup][row] = Tools::s_S(GoTour.Group(P));
      }

      SetNewPlayerRow();
   }
}
//---------------------------------------------------------------------------

namespace RegGUI        								// PLAYER DETAILS IMPORT
{
	using std::string;
	using std::list;

   using Tools::cTab;
   using Tools::sCrLf;
   using Tools::s_S;
   using Tools::S_s;
	using Tools::unpack_tabbed;
   using Tools::load_str;
   using Tools::to_string;
   using Tools::trim;
   using Tools::compactify;
   using Tools::strip_white_space;
   using Tools::uppercase;
   using Tools::spaces;
   using Tools::legal_name_syntax;

   using Tours::size_name;
   using Tours::size_note;

   using TourMaster::GoTour;

   enum { cTwid, cRet, cNex, cNcont };							// REG IMPORT 6.4.1

   const char vcontrols[cNcont] = "~\/";

   char vcontrol(const char c)
   {
   	char vc = Tools::cStar;

   	switch(c)
      {
      	case '\t': vc = vcontrols[cTwid];	break;
         case '\r': vc = vcontrols[cRet];		break;
         case '\n': vc = vcontrols[cNex];		break;

         default:	  vc = Tools::cColon;
      }
      return vc;
   }

   const string make_error_visible(const string line) 	// REG IMPORT 6.4.1
   {
   	enum { max_line_size = 512 };

      const char cV='^',	cB='<', cE='>';					// Unknown invisible, line wrapper

      const char fansi = Tools::cSpace, lansi = Tools::cTilde;
      const char* empty = "<>";

   	if(line.empty()) return string(empty);

      size_t linesize = line.size();

      if(linesize>max_line_size) linesize = max_line_size;

      string vline(linesize+2, 0);

      size_t c = 1;
      for(c=1; c<=linesize; c++)
      {
      	const char lc = line[c-1];

         if(lc>lansi){ vline[c] = cV;  continue; }	   	// DEL or Non ansi character
         if(lc>fansi){ vline[c] = lc;  continue; } 		// Display as is

         vline[c] = vcontrol(lc);								// Display control chars visibly
      }
      vline[0] = cB; vline[c] = cE;								// Enclose text

      return vline;
   }
   
   void ThrowHeaderSyntax(const string filename, const string header, const int IDI_report)
   {
   	const string vheader = make_error_visible(header);

   	string message = string();

      message += load_str(IDI_HeadErr)+filename+string(sCrLf)+string(sCrLf);
      message += vheader + string(sCrLf)+string(sCrLf);
      message += load_str(IDI_report);
      message += load_str(IDI_SeeHeaderDocs);

      Beep(); throw ToolsGUI::ETour(erOperr, message);
   }

   void ThrowPlayerError
   (
   	const string filename, const string line,
      const int p, string name,
      const int IDI_report
   )
   {
   	const string linend = string(sCrLf)+string(sCrLf);

   	string message = string();

      message += load_str(IDI_PlayErr)+to_string(p)+linend;

      message += filename + linend;
      message += make_error_visible(line) + linend;

      switch(IDI_report)
      {
         case IDI_PlayerListDup: case IDI_PlayerTourDup:
         {
         	string dupmess = load_str(IDI_report);

            const int namepos = dupmess.find(string("%"),0);

            dupmess.replace(namepos,1, name);

            message += dupmess;
         }
         break;

      	default: message += load_str(IDI_report) + linend;
      }
      message += load_str(IDI_SeePlayerDocs);

      Beep(); throw ToolsGUI::ETour(erOperr, message);
   }

   static void ShowPlayerCount(RegGUI::GRegGrid* RegGrid, const int count)
   {
   	string message = load_str(IDI_ImportCount);

      int cpos = message.find(string("%"), 0);

      message.replace(cpos, 1, to_string(count));

      RegGrid->ClearInfo();	RegGrid->ShowInfo(s_S(message));

      GoTour.ClearTourInfo();
   }

   // NAME PROCESSING ALGORITHMS
   enum
   {
   	algGivenFamily,				// Construct Family,Given from separate fields
      algFamily,						// Just Family.  So Fred Bloggs is error.
      algNameGF,						// Interpret as Given Family
      algNameFG,						// Construct Family,Given 
      algNames
   };
   static int NameAlgorithm = algNames;


   // FIELD IDENTIFIERS
	enum
   {
   	imPid, 	imReg,
      imGiven, imFamily, imName,  imNameGF, imNameFG,
      imGrade, imClub,	 imLand,
      imRounds,imGroup,  imNote,  imRating,

      imFields
   };

   // FIELD HEADERS
   //
   // ID			is used to report syntax errors if present, not encoded
   // REG		is respected up to close of reg; otherwise set to registerd
   // NAME		is alias for NAME-GF;  repeat names ignored and flag error
   // CLUB		whether full or code decided by preferences
   // COUNTRY	whether full or code decided by preferences
   // ROUNDS	any earlier rounds played are ignored
   // NOTE	   includes current rating if known
   //

   static const int HeadKeysID[imFields] =
   {
      IDL_IPlayer, IDL_IReg,

      IDL_IGiven, IDL_IFamily, IDL_IName, IDL_INameGF, IDL_INameFG,

      IDL_IGrade,

      IDL_IClub, IDL_ILand,

      IDL_IRounds, IDL_IGroup, IDL_INote, IDL_IRating
   };

   // FIELD IDENTIFICATION
   static int RegFieldMap[imFields], ItemMap[imFields];

   static int &M_Pid    = RegFieldMap[imPid];
   static int &M_Reg    = RegFieldMap[imReg];
   static int &M_Given  = RegFieldMap[imGiven];
   static int &M_Family = RegFieldMap[imFamily];
   static int &M_Name   = RegFieldMap[imName];
   static int &M_NameGF = RegFieldMap[imNameGF];
   static int &M_NameFG = RegFieldMap[imNameFG];
   static int &M_Grade  = RegFieldMap[imGrade];
   static int &M_Club   = RegFieldMap[imClub];
   static int &M_Land   = RegFieldMap[imLand];
   static int &M_Rounds = RegFieldMap[imRounds];
   static int &M_Group  = RegFieldMap[imGroup];
   static int &M_Note   = RegFieldMap[imNote];
   static int &M_Rating = RegFieldMap[imRating];

   static int FieldCount = 0;  // Defined by header analysis


	static int ParsePlayersHeader(const string headline, int& Used)
   {
   	// GET THE POSSIBLE HEADER FIELDS
      string HeadKeys[imFields];

      for(int i=0; i<imFields; i++)
      	HeadKeys[i] = Tools::load_str(HeadKeysID[i]);


   	// UNPACK THE HEADER LINE
   	list<string>Headers;                               // The first line is the header

      Used = unpack_tabbed(headline, imFields, Headers);

      if(Used < 3) return IDI_FewFields;						// NAME, GRADE, CLUB mandatory
      if(Used > imFields) return IDI_ManyFields;			// GARBAGE FIELDS IN HEADER

      list<string>::iterator iHead = Headers.begin();
      list<string>::iterator eHead = Headers.end();

      // IDENTIFY THE FIELDS SUPPLIED IN THE HEADER
      for(int f=0; f<imFields; f++)
      	RegFieldMap[f] = ItemMap[f] = -1;

      int Item = 0, ItemCount = 0;								// Index to ItemMap

      while(iHead != eHead)
      {
      	for(int iField=0; iField<imFields; iField++)
         	if(Tools::uppercase(*iHead) == HeadKeys[iField])
            {
            	ItemMap[Item] = iField;						   // Identifies item by column number

               RegFieldMap[iField] = Item;					// Identifies which fields are used

               ++ItemCount;
            }

         ++Item; ++iHead;
      }

      if(ItemCount != Used) return IDI_Unidentified;		// SOME ITEMS NOT IDENTIFIED

      // NAME ANALYSIS
      int NameFieldCount = 0;

      NameAlgorithm = algNames;

      if(M_Family >= 0) { ++NameFieldCount;  NameAlgorithm = algFamily; }
      if(M_Given >= 0)  { ++NameFieldCount;  NameAlgorithm = algGivenFamily; }
      if(M_Name >= 0)   { ++NameFieldCount;  NameAlgorithm = algNameGF; }
      if(M_NameGF >= 0) { ++NameFieldCount;  NameAlgorithm = algNameGF; }
      if(M_NameFG >= 0) { ++NameFieldCount;  NameAlgorithm = algNameFG; }

      switch(NameFieldCount)
      {
      	case 0:	return IDI_NoName;							// NAME IS MANDATORY

         case 1:	if(M_Given>=0)
         				return IDI_MissingFamily;				// BOTH FIELDS GIVEN,FAMILY NEEDED
         			else
                  	break;

         case 2:	if(M_Given<0 || M_Family<0)
         				return IDI_OnlyGivenFamily;
         			else
                  	break;


         default:	return IDI_BadNameFields;					// GARBLED NAME FIELDS
      }

      // CLUB ANALYSIS
      if(M_Club<0) return IDI_NoClub;							// CLUB MANDATORY

      // GRADE MANDATORY
      if(M_Grade<0)return IDI_NoGrade;

      return 0;
   }

   static int ExtractName(string items[imFields], string& name)
   {
   	const string comma(Tools::sComma);

      // FORM COMPACTED NAME
      if(NameAlgorithm == algGivenFamily)
      {
      	const string family = compactify(items[M_Family]);
         const string given = compactify(items[M_Given]);

         name  = !given.empty() ? family + comma + given : family;
      }
      else
      {
      	if(M_Name>=0)   name = compactify(items[M_Name]);
         if(M_NameGF>=0) name = compactify(items[M_NameGF]);
         if(M_NameFG>=0) name = compactify(items[M_NameFG]);
      }

      // NAME IS MANDATORY
      if(name.empty())
      	return IDI_NoName;

      // SYNTAX CHECK
      if(!legal_name_syntax(name))
      	return IDI_PlayerNameSyntax;
      else
      	return 0;
   }

   static int ExtractFields
   (
   	GRegGrid* reggrid,		 // Access

   	string items[imFields],	// Unpacked fields from import file
      string& pname,				// Raw name
      vector<string>& fields	// Register-friendly fields
   )
   {
   	using Tools::uppercase;

   	using TourMaster::RegCode;
   	using RegMaster::family_comma_name;

   	// CLEAR FIELDS - Pid WILL BE INTERNALLY GENERATED
   	for(int f=0; f<rfFields; f++)
      	fields[f] = string();

   	// THE REG FIELD IS OPTIONAL
      fields[rfStatus] = string(RegCode[1]);							// Registered

      if(M_Reg>=0)
      	if(GoTour.State <Tours::tsRegClosed)
      		fields[rfStatus] = trim(uppercase(items[M_Reg]));	// R or empty

      if(M_Reg<0)
      	if(GoTour.State <Tours::tsRegClosed)
      		fields[rfStatus] = string(RegCode[0]);


      // NAME GRADE CLUB ARE MANDATORY
      int nameformat;

      switch(NameAlgorithm)
      {
      	case algNameGF: 		nameformat = TourMaster::nfGivenFamily; break;
         case algNameFG: 		nameformat = TourMaster::nfFamilyGiven; break;
         case algFamily:		nameformat = TourMaster::nfFamily;      break;
         case algGivenFamily: nameformat = TourMaster::nfFamilyGiven; break;

         default:					nameformat = TourMaster::nfUnknown;
      }

      // NAME SIZE IS LIMITED
      string fcname = family_comma_name(nameformat, pname);
      	if(fcname.empty())  return IDI_PlayerNameSyntax;
      fields[rfName]  = fcname;

      fields[rfGrade] = compactify(items[M_Grade]);	if(fields[rfGrade].empty()) return IDI_NoGrade;
      fields[rfClub]  = compactify(items[M_Club]);    if(fields[rfClub].empty())  return IDI_NoClub;

      // COUNTRY DEFAULTS TO UK
      if(M_Land>=0)
      	fields[rfLand]   = compactify(items[M_Land]);
      else
      	fields[rfLand] = S_s(PrefsMaster::PrefsTube.DefLandName);

      // ROUNDS
      if(M_Rounds>=0)
      	fields[rfRounds] = compactify(items[M_Rounds]);
      else
         fields[rfRounds] =  S_s(reggrid->FullRounds());

      // GROUP
      if(M_Group>=0) fields[rfGroup]  = strip_white_space(items[M_Group]);

      // NOTE - ensure note is short
      if(M_Note>=0)
      {
      	string note = trim(items[M_Note]);
      	fields[rfNote] = note;
      }

		//if(M_Rating>=0)fields[rfRating] = strip_white_space(items[M_Rating]);

      return 0;
   }

   static int ParsePlayer
   (
   	GRegGrid* RegGrid,							// Access

   	TStringList* Players, const int p,		// The p'th line in Players
      int& poserr,									// Report line number error
      string& pName,									// Report processed name
      Tours::GPlayer& P								// Set all fields in P
   )
   {
   	using Tools::unpack_tabbed_exact;
      
   	// GET THE PLAYER
      pName = string();  poserr = 0;

      string playerp = trim(S_s(Players->Strings[p]));

      if(playerp.empty()) return -1;  															// GK 6.3.9	 We ignore empty lines

      // UNPACK THE PLAYER
      string PlayerItems[imFields];

      const int nfield =  unpack_tabbed_exact(playerp, imFields, PlayerItems);
      int nempty = 0;																				// GK 6.3.9	Number of empty fields

      for(int f=0; f<nfield; f++)
      	if(PlayerItems[f].empty())
         	nempty += 1;
      if(nempty == nfield) return -1;															// GK 6.3.9	 No visble text

      // if(unpack_tabbed_exact(playerp, imFields, PlayerItems) != FieldCount)	// GK 6.3.9
      if(nfield != FieldCount)
      {
      	poserr = p;
         
      	return IDI_PlayerFieldCount;
      }

      // EXTRACT THE NAME
      int NameReport = ExtractName(PlayerItems, pName);
      	if(NameReport)
         {
      		poserr = p; return NameReport;
         }

      // EXTRACT ALL THE REMAINING FIELDS
      vector<string> pFields(rfFields);

      const int FieldReport = ExtractFields(RegGrid, PlayerItems, pName, pFields);
      	if(FieldReport)
         {
      		poserr = p; return FieldReport;
         }

		// NOW PARSE THE PLAYER FIELDS
      const int Report = RegMaster::parse_player(pFields, P);
      if(Report<rfFields)
      {
      	poserr = p;	return IDI_BadNumberForm+Report;
      }

      // ENSURE PLAYER UNIQUE WITHIN GoTour
      const int qReg = RegMaster::FindPlayerName(Tools::uppercase(P.Name));

      if(qReg>0) // The player name is known in GoTour
		{
      	poserr = p;	return IDI_PlayerTourDup;
      }

		return 0;
   }

   void __fastcall ImportPlayerList			// ENTER PLAYER DETAILS FROM TEXT FILE
   (
      string filename,
      TComboBox* RoundBox,
      RegGUI::GRegGrid* RegGrid,
      PaneGUI::GPairPane* PairPane
   )
   {
   	using Tours::ftGoDraw;
   	using Tours::faOpen;
      using Tours::GTourStream;
      using Tours::stMcMahon;
      using Tours::stBarLow;
      using Tours::style_t;
      using Tours::GPlayer;

      using TourMaster::GoTour;

      TStringList* Players = 0;

      GPlayer* ParsedPlayers = 0;

      try
      {
      	Screen->Cursor = crHourGlass; 					// MAY TAKE A WHILE

         	// LOAD THE FILE OF PLAYERS
            Players = new TStringList();

            if(!Players)
            	Throw(erOperr, IDI_FileImport, IDE_NoMemory);

      		Players->LoadFromFile(s_S(filename));		// filename guaranteed to exist

            if(Players->Count<2)								// No players
            {
               delete Players;

               Throw(erOperr, IDI_FileImport, IDI_BadPlayersFile);
            }

         	// PARS THE HEADER
            string Head = trim(S_s(Players->Strings[0].UpperCase()));

            if(Head.empty())
            {
            	delete Players;

               Throw(erOperr, IDI_FileImport, IDI_EmptyHeader);
            }

            int Report = ParsePlayersHeader(Head, FieldCount);
            if(Report)
            {
            	delete Players;    Players = 0;

               ThrowHeaderSyntax(filename, Head, Report); 		// GK 6.4.1 display culprit
            }

            // CREATE THE LIST OF PARSED PLAYERS
            try
            {
            	ParsedPlayers = new GPlayer[Players->Count];

					for(int p=0; p<Players->Count; p++)
               	ParsedPlayers[p] = GPlayer(p+1, GoTour.Rules.Rounds);
            }
            catch(...)
            {
            	delete Players;    Players = 0;   ParsedPlayers = 0;

            	Throw(erOperr, IDI_FileImport, IDI_ManyPlayers);
            }

            // FIRST PASS:  ANALYSE SYNTAX AND SEARCH FOR DUPLICATE NAMES
            int ImportedCount = 0;

            for(int p=1; p<Players->Count; p++)
            {
            	string Name;  int poserr;

               GPlayer& Player = ParsedPlayers[ImportedCount];

               Report = ParsePlayer
               (
               	RegGrid,
               	Players, p,
                  poserr, Name, Player
               );

               if(Report<0) continue; 									// Gk 6.3.9 ignore empty lines

               if(!Report)
               {
               	const style_t Style = GoTour.Rules.Style;

               	// NO ONE CAN PLAY IN AN EARLIER ROUND
                  const size_t Round = GoTour.Roundl;

                  for(size_t r=0; r<Round; r++)
                  	Player.Games[r].Result = Tours::grSleep;

                  // WE IGNORE THE GROUP FIELD FOR NON-McMAHON TOURNAMENTS
                  if(stMcMahon < Style && Style <stBarLow )
                  	Player.Group = spaces(2);
               }
               else
               {
               	string line = Players->Strings[p].c_str();	// GK 6.4.1 display culprit

            		delete 	    Players;  		   Players = 0;
                  delete ParsedPlayers;	ParsedPlayers = 0;

               	ThrowPlayerError(filename,line, poserr+1, Name, Report);
               }
               if(!Name.empty()) ++ ImportedCount;
            }

            // SECOND PASS - CHECK DUPLICATES IN ParsedPlayers
            for(int p=0; p<ImportedCount; p++)
            {
            	const string pName = uppercase(ParsedPlayers[p].Name);

               for(int q=0; q<p; q++)
               	if(uppercase(ParsedPlayers[q].Name) == pName)
                  {
                  	string line = Players->Strings[p].c_str();

                  	delete Players;				Players = 0;
                     delete[] ParsedPlayers;    ParsedPlayers = 0;

                     ThrowPlayerError(filename, line, p+1, pName, IDI_PlayerListDup);
                  }
            }

            // THIRD PASS: OPEN THE TOUR FILE; ADD PLAYERS; AND CLOSE THE FILE
            try
            {
            	Tours::GTourStream ImportStream(GoTour.FileName, ftGoDraw, faOpen);

               for(int p=1; p<=ImportedCount; p++)
               {
               	GPlayer& Player = ParsedPlayers[p-1];

                  Player.Pid = GoTour.NewPid++;

                  RegMaster::import_player(Player);

                  ImportStream.AppendPlayer(Player);
               }

    				ImportStream.SaveImportedCodes 		 // Newly used codes
               (
               	GoTour, ImportedCount, ParsedPlayers
   				);

               // UPDATE THE HEADER
               GoTour.Players += ImportedCount;

               if(GoTour.State <= Tours::tsTourStart)
               	GoTour.State = Tours::tsRegNow;

               ImportStream.SaveHeader(GoTour);
            }
            catch(...)
            {
            	delete Players;				Players = 0;
               delete[] ParsedPlayers;    ParsedPlayers = 0;

            	Throw(erOperr, IDI_FileImport, IDE_NoMemory);
            }

         // RE-OPEN AND DISPLAY	IN PID ORDER 	GK REGSORT 2015
         //const int RegOrder = GoTour.SortOrder, RegSense = GoTour.SortSense;

         FileMUI::OpenTourFile(s_S(GoTour.FileName), RoundBox,RegGrid, PairPane);

         // GK REGSORT 2015 RegMaster::RegSort(RegOrder, RegSense);
         const int LastRow =  RegGrid->RowCount-1;
         RegGrid->LoadSorted(Tours::skEntry, ToolsGUI::gsUp);
         RegGrid->SelectGridCell(rfName, LastRow);
      	RegGrid->ExposeSelectedCell();

         Screen->Cursor = crDefault;

         delete Players;
         delete[] ParsedPlayers;

         ShowPlayerCount(RegGrid, ImportedCount);
      }
      catch(Exception& Err)
      {
      	delete Players;
         delete[] ParsedPlayers;

      	Screen->Cursor = crDefault;

         ToolsGUI::Serve(Err, IDI_ImportPlayerList, IDI_ImportErr);

         GoTour.ClearTourInfo();
         ToolsGUI::OperatorError(IDI_ImportAbandoned);
      }
      catch(...)
      {
      	delete Players;
         delete[] ParsedPlayers;

      	Screen->Cursor = crDefault;

     		ToolsGUI::ShowWin32Err(IDE_FileMUI, IDI_ImportErr);

         GoTour.ClearTourInfo();
         ToolsGUI::OperatorError(IDI_ImportAbandoned);
      }
   }
}
//------------------------------------------------------------------------------


