////////////////////////////////////////////////////////////////////////////////
// MODULE RegMaster:  RegGrid interface to GTour        G.Kaniuk 13/06/04
//
#include <string>
#include <vector>

#include "Maths.h"
#include "Tour.h"

#ifndef RegMasterH
#define RegMasterH


namespace RegMaster			// EDITING
{
	using std::string;
   using std::vector;
   using Tools::ints;
   using Tours::GPlayer;

   enum
   {
   	pidLegal, pidBad, pidZero, pidBig, pidDeleted
   };

   enum
   {
      prNone,
      prNewMMS,
      prJoin,
      prLeave,
      prRestored,
      prFullyRestored,
      prGradeChanged,
      prCorrupt
   };

   enum 																				// Group code error analysis
   {
   	rgGroupPass,
      rgGroupForbid, rgGroupSpecial, rgGroupSuper, rgGroupOther,
      rgGroupCount
   };

   bool legal_region_syntax(string text);
   
   void capitalise(string& text);

   void rounds_string(const vector<bool>& playing,  string& text);
   void string_rounds(string text,  vector<bool>& playing);

   string name_family(const string name);
   string family_comma_name(int form, const string name);

   bool make_family(string& fullname);
   bool make_region(const int sizecode, string& area, string& areacode);
   bool make_rated_grade(string sgor, string& strength, string& tgrade);

   int parse_pid(string& text, int& pid);
   
   bool parse_grade(string& sgrade, ints& vgrade);
   bool parse_rounds(string& srounds, vector<bool>& vrounds);
   bool parse_group(string& sgroup, char& id, int& type);

   int parse_player(vector<string>& fields, GPlayer& player);
   int edit_player(int prow, bool newplayer,  vector<string>& fields, int& playrep);

   int save_player(bool newcodes, int plist);
   int toggle_reg_status(const int p);
   
   void delete_players(size_t pstart, size_t pend);

   void initial_score(GPlayer& player);

   void player_details_changed(GPlayer& pold, GPlayer& pnow, int& playrep);
   void player_rounds_changed(GPlayer& pold, GPlayer& pnow, int& playrep);
   void player_round_change(bool playing, Tools::intsi& result);

   void start_new_player(GPlayer& player);

   void end_player_edit(GPlayer& pold, GPlayer& pnow, const int ip, int& rep);

   void import_player(GPlayer& playernow);
}
//---------------------------------------------------------------------------

namespace RegMaster			// SORTING
{

   void RegSort(int sortorder, int sortdir);

}
//------------------------------------------------------------------------------

namespace RegMaster 			// SEARCHING
{
	using Tours::GTour;

   enum
   {
   	fiNone,
   	fiName,
      fiGrade,
      fiClub,
      fiLand,
      fiGroup,
      fiNote,
      fiMMS,
      fiRound,
   	fiFields
   };

   struct MFindTarget
   {
   	bool Required;										// Whether this field is required

      String Target;										// The string wanted for this field

      MFindTarget()
      {
      	Required = false; 	Target = String();
      }

      MFindTarget(bool required, String target)
      {
      	Required = required; 	Target = target;
      }
   };

	class MFindTube
   {
   	public:
   		TStrings* Lists[fiFields];

      	MFindTube();

      	String PlayerString(Tours::GPlayerPtr P);

      	Find(MFindTarget targets[fiFields], TStrings* seeked);
   };

   extern MFindTube FindTube;

   size_t ExtractPlayerId(String PackedPlayer);

   int FindPlayerId(size_t pid);

   int FindPlayerName(string name);
}
//---------------------------------------------------------------------------

namespace RegMaster							// CLOSE REGISTER
{
   enum { crDelete, crKeep, crLate };
   
   int MissingGradeCount(void);

   void SetMacGroupCodes(void);

   void CloseUnregistered(const int action);
}
//---------------------------------------------------------------------------
#endif
 