////////////////////////////////////////////////////////////////////////////////
// MODULE  RegPanelGUI    Register Panel Buttons               G.Kaniuk 13/04/04
//
#include <vcl.h>
#pragma hdrstop
#pragma package(smart_init)
//------------------------------------------------------------------------------

#include "GoErrMsgRES.h"
#include "GoInfoRES.h"
#include "GoFormRES.h"
#include "FileRES.h"
#include "TourRES.h"

#include "ETour.h"
#include "TourStream.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "PlayerMaster.h"
#include "DrawMaster.h"

#include "GoDrawGUI.h"
#include "RegPanelGUI.h"
#include "PairPanelGUI.h"
#include "TourMUI.h"

#include "EditPlayerForm.h"
#include "SeekForm.h"
//------------------------------------------------------------------------------

extern void __fastcall LoadPairGrid(void);				// GoDrawForm.cpp
extern void __fastcall LoadResultGrid(void);
extern void __fastcall EditPlayerAndFocus(const int player);
extern bool __fastcall UnsavedPlayerEdit(const int idmsg);
extern void __fastcall ReloadPlayer(void);

//------------------------------------------------------------------------------

namespace RegPanelGUI                                // CLASS MComboBox
{
   __fastcall MComboBox::MComboBox
   (
   	TComponent* Owner,
      const int left, const int top, const int width, int height
   )
   :TComboBox(Owner)
   {
   	Parent = (TWinControl*)Owner;
      
   	const TRect Box(left, top, left+width, top+height);

   	BoundsRect = Box;

      Style = Stdctrls::csDropDown;

      Text = String();

      OnChange = OnChangeEd;
      OnClick  = OnClickList;
   }

   void __fastcall MComboBox::OnChangeEd(TObject* Sender)
   {
   	const String Source = Text;

   	if(FindInList(Source))
      {
         Text = Items->Strings[ItemIndex];

         SelStart = Source.Length();
         SelLength = Text.Length() - SelStart;
      }
      else
      {
         Text = Source;

      	SelStart  = Source.Length();
         SelLength = 0;
      }
   }


   bool __fastcall MComboBox::FindInList(const String TextEd)
   {
   	ItemIndex = -1;

      for(int i = 0; i<Items->Count; i++)
      	if(Items->Strings[i].UpperCase().Pos(TextEd.UpperCase())==1)
         {
         	ItemIndex = i;

            return true;
         }

      return false;
   }


   void __fastcall MComboBox::OnClickList(TObject* Sender)
   {
   	OnChange = 0;
      	Text = Items->Strings[ItemIndex];
      OnChange = OnChangeEd;
   }
}
//------------------------------------------------------------------------------

namespace RegPanelGUI                                // CLASS MListBox
{
   using std::string;
   using std::list;

   __fastcall MListBox::MListBox
   (
   	TComponent* owner,
      const int tabbed, const int* tabpos

   ):TCustomListBox(owner)
   {
      Visible        = false;

      Parent         = (TWinControl*)owner;

      Style          = lbOwnerDrawFixed;
      IntegralHeight = true;
      Sorted         = false;

      TabbedFields   = tabbed;
      FieldPos       = tabpos;

      Background     = Color;

      OnMouseWheel     = MouseWheel;
      OnMouseWheelUp   = MouseWheelUpDn;
      OnMouseWheelDown = MouseWheelUpDn;

      if(TabbedFields>0 && FieldPos)
      {
      	Height = 5*ItemHeight;
         Width = FieldPos[TabbedFields-1];
      }
   }

   void __fastcall MListBox::SetSorted(bool sorted)
   {
   	Sorted = sorted;
   }
   //-------------------------------------------------------------------

   MESSAGE void __fastcall MListBox::CNDrawItem(TMessage& message)
   {
      LPDRAWITEMSTRUCT drawitem = reinterpret_cast<LPDRAWITEMSTRUCT>(message.LParam);

      drawitem->itemState &= ~ODS_FOCUS;

      TCustomListBox::Dispatch(&message);
   }


   void __fastcall MListBox::DrawItem
   (
      int index, const TRect& area, TOwnerDrawState drawstate
   )
   {
      TColor clFont  = Canvas->Font->Color;

      if(drawstate.Contains(odSelected))
      {
         if(clFont==clHighlight)
            Canvas->Font->Color = TColor(clHighlight/2 + 2);

         Canvas->Brush->Color = clHighlight;
      }

      Canvas->FillRect(area);


      int AreaTop = area.top + 2, AreaLeft = area.left + 2;

      if(FieldPos && TabbedFields>0)
      try
      {
         string packed = Tools::S_s(Items->Strings[index]);

         list<string>fields;

         int F = Tools::unpack_tabbed(packed, TabbedFields, fields), f;

         list<string>::iterator Field = fields.begin();

         for(f=0; f<F; f++)
         {
         	string stripped = Tools::strip_crlf(*Field);

            Canvas->TextOut(AreaLeft+FieldPos[f], AreaTop, Tools::s_S(stripped));

            ++Field;
         }
      }
      catch(...)
      {
         Canvas->TextOut(AreaLeft, AreaTop, Items->Strings[index]);
      }
      else
         Canvas->TextOut(AreaLeft, AreaTop, Items->Strings[index]);

      if(drawstate.Contains(odSelected))
      {
         Canvas->Brush->Color = Background;
         Canvas->Font->Color = clFont;
      }
   }
   //---------------------------------------------------------------------------

   void __fastcall MListBox::KeyDownSpecial(TCMWantSpecialKey& Message)
   {
      Message.Result = 1;
   }

   void __fastcall MListBox::KeyDown(Word &Key,TShiftState Shift)
   {
      TCustomListBox::KeyDown(Key, Shift);
   }
}
//------------------------------------------------------------------------------

namespace RegPanelGUI											// TOOLS
{
	using ToolsGUI::OperatorError;

	void __fastcall RegInfo(int MessageId)
   {
   	using TourMaster::GoTour;

   	GoTour.ClearTourInfo();

      GoTour.PostTourInfo(MessageId);
   }
}
//------------------------------------------------------------------------------

namespace RegPanelGUI											// BUTTON new
{
	using TourMaster::GoTour;
   using PlayerMaster::EditPlayerTube;
   /*
	void __fastcall NewPlayerCmd(void)   //OBSOLETE
   {
      switch(EditPlayerTube.State)
      {
         // THE BUTTON CAPTION IS New ...
      	case PlayerMaster::epiNew:
         {
         	EditPlayerTube.PlayerCol = RegMaster::fiName;
            ::EditPlayerAndFocus(GoTour.Players+1);
         }
         break;

         // THE BUTTON CAPTION IS Editing ...
         case PlayerMaster::epiExists:
            ::EditPlayerAndFocus(EditPlayerTube.PlayerRow);
         break;

         default:
         {
         	// THIS SHOULD NEVER HAPPEN
         	EditPlayerTube.PlayerCol = RegMaster::fiName;
            ::EditPlayerAndFocus(GoTour.Players+1);
         }
      }
   }
   */
}
//---------------------------------------------------------------------------

namespace RegPanelGUI											// BUTTON Find
{
	void __fastcall FindCmd(TForm* Seekform, RegGUI::GRegGrid* RegGrid)
   {
   	using RegMaster::FindTube;

      TSeekFrm* SeekFrm = (TSeekFrm*)Seekform;

      if(!SeekFrm)
      {
      	Beep(); RegInfo(IDF_NoSeekFrm);

         return;
      }

      if(!SeekFrm->Started)
      {
      	SeekFrm->ExtractLists();

         SeekFrm->Started = true;
      }

      SeekFrm->Show();
   }
}
//---------------------------------------------------------------------------


namespace RegPanelGUI											// BUTTON Reg All
{
	using TourMaster::GoTour;
   using TourMaster::RegCode;

   using ToolsGUI::EditCell;
   using ToolsGUI::GField;

   using RegGUI::GRegGrid;
   using RegGUI::rfStatus;

   void __fastcall RegPlayersCmd(TButton* RegPlayersBtn,  GRegGrid* RegGrid, int RegMode)
   {
   	try
      {
   		Tours::GTourStream RegStream(GoTour.FileName, Tours::ftGoDraw, Tours::faOpen);

         for(size_t p = 1; p<=GoTour.Players; p++)
         {
         	if(RegMode == rpRegAll)
            {
            	GoTour.Listed[p]->State = Tours::psRegistered;
               RegGrid->Cells[rfStatus][p] = String(RegCode[1]);
            }
            else
            {
            	GoTour.Listed[p]->State = Tours::psEntered;
               RegGrid->Cells[rfStatus][p] = String(RegCode[0]);
            }
            RegStream.SavePlayer(p, GoTour);
         }
         GoTour.PopulationCount();

         if(RegMode == rpRegAll)
         	RegInfo(IDF_RegdAll);
         else
            RegInfo(IDF_UnRegdAll);

         RegGrid->TopRow = 1;	RegGrid->LeftCol = 1;

         RegGrid->NoCellSelected();

         ::ReloadPlayer();
      }
      catch(Exception& Err)
      {
         ToolsGUI::Serve(Err, IDM_RegAllCmd, IDM_FileOpenErr);
      }
      catch(...)
      {
     		ToolsGUI::ShowWin32Err(IDE_RegPanelGUI, IDM_RegAllCmd);
      }
   }
}
//---------------------------------------------------------------------------

namespace RegPanelGUI											// BUTTON Close Reg
{
	// GK REGSORT 2015
   // Major revision for godraw v 6.3.9. There were spurious calls to RegSort without
   // informing RegGrid that the sort order had changed.  This leads to wrong player accessed when register is closed
   // and user clicks on player in the case where the original sort order was opposite to normal.

	using TourMaster::GoTour;

	void __fastcall CloseRegCmd
   (
   	TGroupBox* BarGroup, TGroupBox* PopGroup, GRegGrid* RegGrid
   )
   {
   	using Tours::grade_string;

   	using Tools::ints;
      using Tools::load_str;

      using ToolsGUI::EditCell;
      using ToolsGUI::EditingGrid;

      using RegGUI::rfStatus;
      using RegGUI::rfName;
      using RegGUI::rfGrade;

      const int LastRow = RegGrid->RowCount-1;

       // REQUIRE PLAYERS TO BE REGISTERED
      if(GoTour.RegCount == 0)
      {
         ToolsGUI::OperatorError(IDF_NoRegistered);

         return;
      }

      // ALL PLAYERS MUST HAVE A GRADE
      if(RegMaster::MissingGradeCount() > 0)
      {
      	RegGrid->LoadSorted(Tours::skGrade, ToolsGUI::gsUp);
         RegGrid->SelectGridCell(rfGrade, LastRow-1);
         RegGrid->ExposeSelectedCell();

         ToolsGUI::OperatorError(IDF_MissingGrades);

         return;
      }

      // TWO OR MORE REGISTERED NEEDED FOR A PAIRING
      if(GoTour.PlayNowCount<2)
      {
         ToolsGUI::OperatorError(IDF_TooFewPlayers);

         return;
      }

      // NEED EVEN NUMBER OF REGISTERED PLAYERS FOR FIRST ROUND
      if(GoTour.PlayNowCount%2 != 0)
      {
      	int NewSort = GoTour.SortOrder;

         if(GoTour.RegCount == (int)GoTour.Players)
         {
         	ToolsGUI::OperatorError(IDF_OddPlayers);
            NewSort = Tours::skRounds;
         }
         else
         {
            ToolsGUI::OperatorError(IDF_MissingReg);
            NewSort = Tours::skTicked;
        	}

         RegGrid->LoadSorted(NewSort, ToolsGUI::gsUp);
         RegGrid->SelectGridCell(rfName, LastRow);
         RegGrid->ExposeSelectedCell();

         return;
      }

      // PROCESS UNREGISTERED PLAYERS
      if(GoTour.RegCount != (int)GoTour.Players)
      {
         RegGrid->LoadSorted(Tours::skTicked, ToolsGUI::gsUp);
         RegGrid->SelectGridCell(rfStatus, LastRow);
         RegGrid->ExposeSelectedCell();

         int Report = 0;

         if(TourMUI::CloseRegAct(RegGrid->Owner, Report))
         {
         	const int Action =  Report - (IDT_Delete);

            RegMaster::CloseUnregistered(Action);
         }

         RegInfo(Report);

         switch(Report)
         {
         	case IDT_Resume:
           	return;

            case IDT_NoCloseReg:
            {
            	RegGrid->LoadSorted(GoTour.OldRegOrder, GoTour.OldRegSense);
            	RegGrid->NoCellSelected();
            }
           	return;

            default:	break;
         }
       }

      // SET THE BAR
      ints& MacBar = GoTour.Rules.MacBar; 							// Pick up all the bars
      ints& KomBar = GoTour.Rules.KomiGroup;
      ints& NovBar = GoTour.Rules.MinGrade;

      int BarPop = 0; 														// Set bar and report

      int BarReport = DrawMaster::SetBar(&GoTour, BarPop);     // Sets MacBar and BarPop

      if(BarReport)
      {
      	ToolsGUI::OperatorInfo(IDF_REGBAR + BarReport);

         if(BarReport==DrawMaster::brSleep || BarReport==DrawMaster::brNoBar)
         	return;
      }

      // ENSURE THE KOMI BAR IS BELOW THE McMAHON BAR
      if(GoTour.Rules.MacBar < KomBar)
      {
      	TComboBox*  MacBox = dynamic_cast<TComboBox*>(BarGroup->Controls[1]);

      	if(MacBox)MacBox->SetFocus();

         GoTour.Rules.MacBar = MacBar = KomBar;

         ToolsGUI::OperatorError(IDF_MacOrder);
      }

      // WE CAN NOW CLOSE THE REGISTER
      if(!GoTour.Rooms)										// No rooms allocated
      {
      	GoTour.Rooms = 1;
         GoTour.Boards[0] = (GoTour.Players+1)/2;	// Everybody in room 1
      }

      ints BarDepth=0;
      DrawMaster::SetInitialScores(NovBar, KomBar, MacBar, BarDepth, &GoTour);

		RegMaster::SetMacGroupCodes();

      GoTour.Round = GoTour.Roundl = 0;

      GoTour.State = Tours::tsRegClosed;

      if(GoTour.SaveTournament())
      {
      	GoTour.PopulationCount();

      	RegGrid->LoadSorted(Tours::skGrade, ToolsGUI::gsUp);
      	RegGrid->SelectGridCell(rfName, RegGrid->RowCount-1);		// GK 6.4.0		RegGrid changes size if players deleted
      	RegGrid->ExposeSelectedCell();

         const Tours::style_t pstyle = GoTour.Rules.Style;
         if(pstyle==Tours::stMcMahon || pstyle==Tours::stBarLow)
         {
            string BarReport = load_str(IDF_RegCloseInfo);		// GK 6.3.9
               size_t pBar = BarReport.find('#', 0);
               string sBar = grade_string(GoTour.Rules.MacBar);
            BarReport.replace(pBar, 1, sBar);

            ToolsGUI::ShowMessage(ToolsGUI::erInfo, BarReport);
         }
         GoTour.PostTourInfo(IDF_RegClosed);
      }
   }

   void __fastcall OpenRegCmd(GRegGrid* RegGrid)
   {
   	if(GoTour.State != Tours::tsRegClosed) return;	// This cannot happen!

      GoTour.State = Tours::tsRegNow;

      if(GoTour.SaveTournament())
      {
      	GoTour.PopulationCount();

         GoTour.PostTourInfo(IDF_RegOpen);
      }
   }
}
//---------------------------------------------------------------------------

namespace RegPanelGUI											// BUTTON Round
{
	void __fastcall LoadRounds(TComboBox* RoundBox)
   {
   	RoundBox->Clear();

   	for(size_t r=0; r<GoTour.Roundl; r++)
      	RoundBox->Items->Add(String(r+1));

      RoundBox->ItemIndex = GoTour.Round-1;
   }

   void __fastcall ChangeRoundCmd(TComboBox* RoundBox, RegGUI::GRegGrid* RegGrid, PaneGUI::GPairPane* PairPane)
   {
   	using DrawMaster::SetPlayerStates;

   	size_t RoundReq = RoundBox->ItemIndex + 1;

      TourMaster::GoTour.ClearTourInfo();

      if(RoundReq == GoTour.Round) return;		// Already there

      if(::UnsavedPlayerEdit(IDF_OkCancelEdit)) return;

      if(ToolsGUI::EditingCell())
         if(EditingGrid(PairPane->ResGrid))
       	{
         	OperatorError(IDF_NoLeaveRes);

         	EditCell->SetFocusEd();

            return;
         }

      if(GoTour.Round < GoTour.Roundl)						// No missing results allowed
      	if(GoTour.MissingCount > 0)
         {
         	OperatorError(IDF_NoChangeRoundRes);

            RoundBox->ItemIndex = GoTour.Round - 1;

            return;
         }

   	try
      {
         GoTour.Round = RoundReq;			// Round <= Roundl

      	SetPlayerStates();					// States depends on result this round

         GoTour.PopulationCount();			// Pop depends on Player states

         DrawMaster::ListBoards();			// Boards this round

         PairPane->LoadFromTour();			// Show the boards ASAP

         PairPane->PairGrid->Repaint();
         PairPane->ResGrid->Repaint();

         GoTour.SetState();					// Tour state depends on pop and Roundl

         GoTour.SaveTournament();

         if(GoTour.State == Tours::tsPastRound)
         	GoTour.PostTourInfo(IDF_PastRound);
         else
         	GoTour.PostTourInfo(IDF_LatestRound);

         ::ReloadPlayer();
      }
      catch(...)
      {
      	ToolsGUI::BugReport(IDE_RegPanelGUI,  IDE_ListBoards, IDE_NoMemory);
      }
   }
}
//---------------------------------------------------------------------------

namespace RegPanelGUI											// BUTTON Restore
{
   void __fastcall RestoreCmd(RegGUI::GRegGrid* RegGrid)
   {
   	using TourMaster::GoTour;

      const String NormalRegStatus = TourMaster::RegCode[1];

      int ForcedEdReport = !GoTour.ForcedEdCount ? 1 : 0;

      size_t Round = GoTour.Round;

      if(Round<1 || Round>GoTour.Rules.Rounds)
      	ForcedEdReport = 2;

      if(!ForcedEdReport)
      {
         for(size_t p=1; p<=GoTour.Players; p++)
         {
            Tours::GPlayerPtr Player = GoTour.Listed[p];

            if(DrawMaster::Restore(Player))
            {
            	--GoTour.ForcedEdCount;

               if(GoTour.ForcedEdCount<0)break;

               GoTour.SavePlayer(p);

               RegGrid->Cells[RegGUI::rfStatus][p] = NormalRegStatus;
            }
         }

         if(!GoTour.ForcedEdCount)
         {
         	GoTour.PopulationCount();
            GoTour.SetState();
            GoTour.SaveHeader();

            GoTour.PostTourInfo(IDF_FullyRestored);

            GoTour.RestoreRegStatus = false;

            DrawMaster::ListBoards();
            
            RegGrid->LoadFromTour();

            ::LoadPairGrid();
            ::LoadResultGrid();

            return;
         }
         else
         	ForcedEdReport = 3;

         ::ReloadPlayer();
      }

      ToolsGUI::BugReport(IDE_RegPanelGUI,  IDF_Restore, IDF_Restore+ForcedEdReport);
   }
}

namespace RegPanelGUI	// PLAYER STATE SUPPORT AFTER SAVE DRAW
{
   void __fastcall RestoreRegStatus(RegGUI::GRegGrid* RegGrid)
   {
   	using RegGUI::rfStatus;

      const String NormalRegStatus = TourMaster::RegCode[1];

      for(size_t p=1; p<=GoTour.Players; p++)
         if(!RegGrid->Cells[rfStatus][p].IsEmpty())
         	RegGrid->Cells[rfStatus][p] = NormalRegStatus;

      GoTour.RestoreRegStatus = false;
   }
}
//---------------------------------------------------------------------------



