////////////////////////////////////////////////////////////////////////////////
// MODULE  ResGUI:    Results Pane Support                     G.Kaniuk 07/04/02
//
#include <string>
#include <list>

#include <vcl.h>
#pragma hdrstop

#include "GraphicsAPI.h"
#include "GoDrawRES.h"
#include "GoErrMsgRES.h"
#include "GoInfoRES.h"
#include "GoFormRES.h"
#include "PairRES.h"

#include "Strings.h"
#include "Format.h"

#include "ETour.h"
#include "PrefsMaster.h"
#include "TourMaster.h"
#include "Drawmaster.h"
#include "GoDrawGUI.h"
#include "PairPaneGUI.h"

extern void __fastcall BeginCommand(int IdCmd); // IN GoDrawForm.cpp
extern void __fastcall EndCommand(int IdCmd);
extern void __fastcall ReloadPlayer(void);
extern bool __fastcall UnsavedPlayerEdit(const int idmsg);
extern void __fastcall SetCancelBtnCaption(void);

//---------------------------------------------------------------------------
#pragma package(smart_init)

extern ToolsGUI::GRatio Scale;

namespace ResGUI     // MAGIC CONSTANTS FOR 600 x 800 RESOLUTION SCREEN
{
	using ToolsGUI::OperatorError;
   
   static const int defResHeight = 113, defResWidth = 259;
   static const int defResTop    = 354, defResLeft  = 14;

   static const int cwBoard = 24, cwRes = 14, cwName = 80;

   static const int max_rooms = Tours::size_rooms;

   static ToolsGUI::GGrid::ColSize ColSizes[sfFields] =
   {
      { cwBoard, 0 },
      { cwRes,   0 }, { cwName,  1 },
      { cwRes,  0 },  { cwName,  1 }
   };

   static const int ResHeadID[sfFields] =
   {
      IDL_Board, IDL_Empty, IDL_Black, IDL_Empty, IDL_White
   };

   static char KeyUpper(char Key)
   {
      if( 'a'<= Key && Key <= 'z')
         return Key - 'a' + 'A';
      else
         return Key;
   }

   static bool FilteredResult(const char Key)
   {
      if(0x00 <= Key && Key <= 0x1F){ Beep(); return true; }
      if(Key >= 0x7F)               { Beep(); return true; }

      return false;
   }

}
//---------------------------------------------------------------------------

namespace ResGUI                             // ERROR MANAGEMENT
{
   void Throw(ToolsGUI::EType etype, short idproc, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_ResGUI, idproc, idmsg);
   }

}
//---------------------------------------------------------------------------

namespace ResGUI                                  // CLASS GResEdit
{
   __fastcall GResEdit::GResEdit(GResult* field):ToolsGUI::GEdit(field)
   {
      ResGrid = field->ResGrid;

      AutoSelect = true;
   }

   void __fastcall GResEdit::KeyPress(char &Key)
   {
   	using ToolsGUI::ShowMessage;
      using ToolsGUI::erOperr;

   	using TourMaster::GoTour;

      GResult* ResField = (GResult*)GetField();

      if(FilteredResult(Key)) return;

      if(!ResGrid->Parse((Word)Key))                 // Produces chars
      {
         std::string badresultmsg = Tools::load_str(IDI_ResultKeys);

         badresultmsg[17] = (char)Key;

         ShowMessage(erOperr, badresultmsg);			  // Implant bad key in error message

         SelectText();

         Key = 0;

         return;
      }

      try
      {
         ::BeginCommand(TourMaster::cmdResult);
         {
         	ResGrid->LocateBoard(ResGrid->Row);

            if(!DrawMaster::EditResult(ResGrid->iBoard, ResGrid->BlackR, ResGrid->WhiteR))
            	Throw(ToolsGUI::erBug, IDQ_ResKeyPress, IDQ_NoResEd);

            ::SetCancelBtnCaption();

            bool NewResult = ResGrid->Cells[ResGUI::sfBres][ResGrid->Row][1] == Tools::cNone;

            ResField->Store();	ResGrid->InvalidateRow(ResGrid->Row);

            Key = 0;

            if(!ResGrid->FindNextLocalBoard(NewResult))
            {
               ResField->Dismount();  ResGrid->SetFocus();

               if(!GoTour.MissingCount)
               {
               	GoTour.PopulationCount();

                  GoDraw::SetPopulations(ResGrid->PopGroup);

                  Beep();	GoTour.PostTourInfo(IDI_ResultsDone);

                  ResGrid->SelectGridCell(sfBlack, 1);
               }
               else
               	GoTour.PostTourInfo(IDI_EditResults);

            }
            else
            {
            	ResGrid->MountFieldAfter(GetField());

               GoTour.ClearTourInfo();
            }
         }
         ::EndCommand(TourMaster::cmdResult);

         ::ReloadPlayer();
      }
      catch(...)
      {
      	ToolsGUI::BugReport(IDE_ResGUI, IDQ_ResKeyPress, IDQ_NoResEd);
      }
   }
}

namespace ResGUI                                  // CLASS GResult
{
   using ToolsGUI::GGrid;
   using ToolsGUI::GField;


   __fastcall GResult::GResult(GResGrid* grid, String ename):GField(grid)
   {
      ResGrid = grid;

      Editor  = new GResEdit(this);

      ((TEdit*)Editor)->Name          = ename;
      ((TEdit*)Editor)->MaxLength     = 1;
      ((TEdit*)Editor)->ShowHint      = true;
      ((TEdit*)Editor)->Hint          = LoadStr(IDI_ResultKey_H);
   }

   __fastcall GResult::~GResult( )
   {
      delete Editor;
   }

   void __fastcall GResult::Load()
   {
      ResGrid->BlackRes = ResGrid->Cells[sfBres][RowM];
      ResGrid->WhiteRes = ResGrid->Cells[sfWres][RowM];

      ((TEdit*)Editor)->Text = CellText;
      ((TEdit*)Editor)->Modified = true;
   }

   void __fastcall GResult::Store()
   {
      ResGrid->Cells[sfBres][RowM] = ResGrid->BlackRes;
      ResGrid->Cells[sfWres][RowM] = ResGrid->WhiteRes;
   }

   bool __fastcall GResult::SameValue(void)
   {
      return ((TEdit*)Editor)->Text == CellText;
   }

   void __fastcall GResult::FieldKeyDown(Word &Key, TShiftState Shift)
   {
      switch(Key)
      {
         case VK_DELETE:
         try
         {
         	::BeginCommand(TourMaster::cmdResult);
            {
               ResGrid->Parse(Tools::cNone);

               ResGrid->LocateBoard(ResGrid->Row);

               if(!DrawMaster::EditResult(ResGrid->iBoard, ResGrid->BlackR, ResGrid->WhiteR))
            		Throw(ToolsGUI::erBug, IDQ_ResKeyDown, IDQ_NoResDel);

               ::SetCancelBtnCaption();

               Store();

               ((TEdit*)Editor)->Text = ResGrid->Cells[ColM][RowM];
               ((TEdit*)Editor)->Modified = true;

               Dismount();  ResGrid->SetFocus();

               ResGrid->InvalidateRow(ResGrid->Row);
            }
            ::EndCommand(TourMaster::cmdResult);
         }
         catch(...)
         {
         	ToolsGUI::BugReport(IDE_ResGUI, IDQ_ResKeyDown, IDQ_NoResDel);
         }
         break;

         case VK_ESCAPE:
         {
            Dismount();  ResGrid->SetFocus();
         }
         break;

         case VK_RETURN:
         {
            ResGrid->Fields[ResGrid->Col]->Dismount();

            if(ResGrid->FindNextBatch())
            	ResGrid->Fields[ResGrid->Col]->Mount();
            else
            	if(!TourMaster::GoTour.MissingCount)
               	{ Beep(); ResGrid->ShowInfo(IDI_ResultsDone); }
         }
         break;

         case VK_DOWN:
         {
         	const int NextRow = ResGrid->Row+1;

            if(NextRow<ResGrid->RowCount)
					if(ResGrid->EndedBatchTraverse(Key, NextRow))
               	return;

            ToolsGUI::GField::FieldKeyDown(Key, Shift);
         }
         break;

         case VK_UP:
         {
         	const int PrevRow = ResGrid->Row-1;

            if(PrevRow>0)
					if(ResGrid->EndedBatchTraverse(Key, PrevRow))
               	return;

            ToolsGUI::GField::FieldKeyDown(Key, Shift);
         }
         break;

         default:
            ToolsGUI::GField::FieldKeyDown(Key, Shift);
         return;
      }
      Key = 0;
   }

   void __fastcall GResult::ShowEd()
   {
      if(Editor)
      {
      	Editor->Color = PrefsMaster::PrefsTube.EditColor;
         Editor->Show();
      }
   }

   bool __fastcall GResult::Focussed(void)
   {
      if(State > ToolsGUI::fsDormant)
      {
      	if(!Editor->Visible) Editor->Show();

      	Editor->SetFocus();

         return true;
      }
      else
      	return false;
   }
}
//---------------------------------------------------------------------------

namespace ResGUI                            // CLASS GResGrid IMPLEMENTATION
{
   using ToolsGUI::GGrid;
   using ToolsGUI::GField;
   using ToolsGUI::GRatio;

    __fastcall GResGrid::GResGrid           // Startup constructor
   (
      TGroupBox* parent,
      TStatusBar*  statusbar,
      TGroupBox*   popgroup
   )
   : ToolsGUI::GGrid((TWinControl*)parent, statusbar, sfFields)
   {
             Top = defResTop;
            Left = defResLeft;
          Height = defResHeight;
           Width = defResWidth;

        PopGroup = popgroup;

      BlackR     = WhiteR   = Tours::grNone;
      BlackRes   = WhiteRes = Tools::cNone;

      hScrollBar = false;

      ColWidths[sfBoard] = cwBoard;
      ColWidths[sfBres]  = cwRes;
      ColWidths[sfWres]  = cwRes;

      String EdName = LoadStr(IDL_GRES);

      Fields[sfBres] = new GResult(this, EdName + "B");          // Failure here is trapped in form
      Fields[sfWres] = new GResult(this, EdName + "W");          // and reported to user

      for(int f=0; f<sfFields; f++)
         Cells[f][0] = LoadStr(ResHeadID[f]);

      for(int room=0; room<Tours::size_rooms; room++)
      {
      	BoardCounts[room] = 0;
         GridBoards[room] = 0;
         PairBoards[room] = 0;
      }

      IgnoreClick = false;

      ResultError = false;
   }

   __fastcall GResGrid::~GResGrid()
   {
      delete (GResult*)(Fields[sfBres]);
      delete (GResult*)(Fields[sfWres]);
   }
   //---------------------------------------------------------------------------

   void __fastcall GResGrid::ReBuild()                         // ADJUST SHAPE
   {
      using ToolsGUI::EditCell;
      
      // ROW GEOMETRY
      ScrollBars = ssNone;

      Font->Size = ToolsGUI::defFontSize;              // Do not scale this font

      int BodyRows = 5;

      BodyStride = ClientHeight/(BodyRows+1);

      HeadRowHeight = ClientHeight - BodyRows*BodyStride - GridLineWidth;

      RowHeights[0] = HeadRowHeight;

      for(int j = 1; j<RowCount; j++)
         RowHeights[j] = BodyStride-GridLineWidth;

      // COLUMN GEOMETRY
      if(RowCount-1 > BodyRows)ScrollBars = ssVertical;

      int BoardWidth = cwBoard;
      int ResltWidth = cwRes;
      int NameWidth  = ClientWidth - BoardWidth - 2*ResltWidth;

      if(NameWidth%2 != 0) --NameWidth;

      ColWidths[sfBoard] = ClientWidth - NameWidth - 2*ResltWidth - 5*GridLineWidth;
      ColWidths[sfBlack] = NameWidth/2;
      ColWidths[sfWhite] = NameWidth/2;
      ColWidths[sfBres]  = ResltWidth;
      ColWidths[sfWres]  = ResltWidth;

      if(TourMaster::GoTour.State != Tours::tsEditNow)
         NoCellSelected();

      // IF EDITING IN PROGRESS WE NEED TO RE-ATTACH THE FIELD TO THE CELL
      if(ToolsGUI::EditingGrid(this))   // The rebuilt grid contains the cell in edit
      {
         SelectGridCell(EditCell->ColM, EditCell->RowM);

         ExposeSelectedCell();

         if(EditCell->Editor)
         {
            TRect Cell = CellRect(Col,Row);
            
            EditCell->Editor->Left = Left + Cell.Left + 2*GridLineWidth;
            EditCell->Editor->Top  = Top  + Cell.Top  + 2*GridLineWidth;
         }
      }
   }
   //------------------------------------------------------------------------------

   bool __fastcall GResGrid::Parse(const Word result)
   {
   	// Result KeyCodes are  0 J 1 U D F V _

      using Tours::grLose; using Tours::grWin;  using Tours::grJigo;
      using Tours::grDlos; using Tours::grFwin;
      using Tours::grVoid; using Tours::grUsolv;
      using Tours::grNone;

      switch(Tools::uppercase((char)result))
      {
      	case '1':
         {
         	if(Col==sfBres){ BlackRes = "1";  WhiteRes = "0"; BlackR = grWin;   WhiteR = grLose; }
         	if(Col==sfWres){ BlackRes = "0";  WhiteRes = "1"; BlackR = grLose;  WhiteR = grWin; }
         }
         break;

       	case '0':
         {
         	if(Col==sfWres){ BlackRes = "1";  WhiteRes = "0"; BlackR = grWin;   WhiteR = grLose; }
         	if(Col==sfBres){ BlackRes = "0";  WhiteRes = "1"; BlackR = grLose;  WhiteR = grWin; }
         }
         break;

      	case 'F':
         {
         	if(Col==sfBres){ BlackRes = "F";  WhiteRes = "D"; BlackR = grFwin;  WhiteR = grDlos; }
         	if(Col==sfWres){ BlackRes = "D";  WhiteRes = "F"; BlackR = grDlos;  WhiteR = grFwin; }
         }
         break;

         case 'D':
         {
         	if(Col==sfWres){ BlackRes = "F";  WhiteRes = "D"; BlackR = grFwin;  WhiteR = grDlos; }
         	if(Col==sfBres){ BlackRes = "D";  WhiteRes = "F"; BlackR = grDlos;  WhiteR = grFwin; }
         }
         break;

         case 'J':
         {
         	BlackRes = "J";  WhiteRes = "J";  BlackR = grJigo;  WhiteR = grJigo;
         }
			break;

         case 'U':
         {
         	BlackRes = "U";  WhiteRes = "U";  BlackR = grUsolv;  WhiteR = grUsolv;
         }
         break;

         case 'V':
         {
         	BlackRes = "V";  WhiteRes = "V";  BlackR = grVoid;  WhiteR = grVoid;
         }
         break;

         case '_':
         {
            BlackRes = "_";  WhiteRes = "_";	 BlackR = grNone;  WhiteR = grNone;
         }
         break;

         default:
         	return false;
      }
      return true;
   }
}
//------------------------------------------------------------------------------


namespace ResGUI														// BATCH MANAGEMENT
{
	using Tools::s_S;
   using TourMaster::GoTour;

	bool __fastcall GResGrid::StartFullBatch(const int row)	// CONSISTENCY UPDATE
   {
   	// REQUIRE row IS CONSISTENT WITH iRow, iRoom, iBatch, iBoard
      int fBoard = GridBoards[iRoom];											// First board in room
      int lBoard = fBoard + BoardCounts[iRoom] - 1;						// Last board in room

		int sBoard = fBoard + 5*iBatch;   										// First board this batch
      int eBoard = sBoard+4 <= lBoard ? sBoard+4 : lBoard;				// Last board this batch

      if(sBoard <= row && row <= eBoard)										// row is consistent with room and batch
      {
      	iRow = row;

         iBoard = PairBoards[iRoom] + (iRow - fBoard);
      }
      else																				// Search for room
      {
      	for(iRoom = RoomCount-1; iRoom > 0; iRoom--)
            if(row >= GridBoards[iRoom])
               break;

         fBoard = GridBoards[iRoom];			
         lBoard = fBoard + BoardCounts[iRoom] - 1;

         iRow = row;  iBatch = (iRow - fBoard)/5;

         sBoard = fBoard + 5*iBatch;
         eBoard = sBoard+4 <= lBoard ? sBoard+4 : lBoard;

         iBoard = PairBoards[iRoom] + (iRow - fBoard);
      }

      // SCROLL TO BATCH POSITION
      TopRow = sBoard;


      if(!GoTour.MissingCount) return true;

      // DETERMINE IF THIS BATCH HAS MISSING RESULTS FROM row ON
      for(int b = row; b <= eBoard; b++)				// We guarantee row <= eBoard
      	if(Cells[sfBres][b][1] == Tools::cNone)
         	return false;


      return true;
   }

   bool __fastcall GResGrid::FindNextBatch()			// SEARCH
   {
		int sBoard = GridBoards[iRoom] + 5*iBatch;   // First board this batch

      int rBoard = sBoard + 5;							// First board next batch

      int bCycle = 0;										// We allow one wraparound

      if(rBoard >= RowCount)								// Authorise the wraparound
      {
         ++bCycle;

         if(GoTour.MissingCount) rBoard = 1;
      }

      while(rBoard < RowCount)
      {
         if(!Cells[sfBoard][rBoard].IsEmpty())
            if(Cells[sfBres][rBoard] == Tools::cNone)
            {
               LocateBoard(rBoard);

               SelectGridCell(Col, iRow);

               TopRow = GridBoards[iRoom] + 5*iBatch;

               return true;
            }
         ++rBoard;

         if(rBoard == RowCount)							// We allow one wraparound
         {
         	++bCycle;  if(bCycle>1) return false;

            if(GoTour.MissingCount) rBoard = 1;
         }
      }
      return false;
   }

   bool __fastcall GResGrid::FindNextLocalBoard(bool NewResult)
   {
   	if(!NewResult) return false;									// Only editing - do not advance

		int nBoards = BoardCounts[iRoom];							// Total boards in current room
      int lBoard = GridBoards[iRoom] + nBoards -1;				// Last board in room

		int sBoard = GridBoards[iRoom] + 5*iBatch;				// First board in current batch
      int eBoard = sBoard + 5 - 1;									// Last board in current batch
      				 if(eBoard>lBoard) eBoard = lBoard;

		if(TopRow != sBoard) TopRow = sBoard;

      int NextBoard = Row+1;

      if(NextBoard >= RowCount) return false;      			// No more boards


      while(sBoard<=NextBoard && NextBoard<=eBoard)
      	if(Cells[sfBres][NextBoard][1] == Tools::cNone)		// Next result is missing
         {
            SelectGridCell(Col,NextBoard);

            LocateBoard(NextBoard);

            return true;
         }
         else
         	++NextBoard;

      return false;
   }

   void __fastcall GResGrid::SetupBoards(int& BoardRowCount)
   {
   	iRoom = 0;  BoardRowCount = 0;   RoomCount = 0;

      if(GoTour.State < Tours::tsDrawDone) return;

      const int reqdBoardCount = GoTour.Pair.size() - 1;			// Board numbers start at 1

      // ENSURE THAT WE HAVE ENOUGH BOARDS
      DrawMaster::LayoutBoards();

      // GATHER NON-EMPTY ROOMS
      int sumBoardCounts = 0;

      for(int tRoom=0; tRoom<max_rooms; tRoom++)
      	if(GoTour.Boards[tRoom]>0)
         {
            // SET BOARDS THIS ROOM
            int tBoardCount = GoTour.Boards[tRoom];				// Specified

            sumBoardCounts += tBoardCount;

            if(sumBoardCounts > reqdBoardCount)						// Too many
            {
            	sumBoardCounts -= tBoardCount;

               tBoardCount = reqdBoardCount - sumBoardCounts;	// Exact number in last room

               sumBoardCounts = reqdBoardCount;
            }
            BoardCounts[iRoom] = tBoardCount;

            // ROW NUMBER OF FIRST BOARD IN THIS ROOM
            if(iRoom == 0)
            {
               GridBoards[iRoom] = 1;
               PairBoards[iRoom] = 1;
            }
            else
            {
            	const int prevBoardCount = BoardCounts[iRoom-1];
               const int prevFirstBoard = GridBoards[iRoom-1];

               const int prevRoomRows = ((prevBoardCount+4)/5)*5;

               GridBoards[iRoom] = prevFirstBoard + prevRoomRows;

               PairBoards[iRoom] = PairBoards[iRoom-1] + prevBoardCount;
            }

            // DISPLAY RESULTS IN BATCHES OF 5
            BoardRowCount += ((tBoardCount+4)/5)*5;

            ++iRoom;

            if(sumBoardCounts == reqdBoardCount)
            {
            	RoomCount = iRoom;	break;
            }
         }
   }

   void __fastcall GResGrid::LocateBoard(const int row)
   {
   	if(row == iRow)return;

      int BoardCount = 0, FirstGridBoard = 0;

      // WE HAVE ADVANCED ONE ROW
      if(row == iRow+1)
      {
      	BoardCount = BoardCounts[iRoom];
         FirstGridBoard = GridBoards[iRoom];

         if(iRow == FirstGridBoard+BoardCount-1) 	// Last board in room
         {
         	iBatch = 0;  iRoom += 1;					// Next room

            iRow = GridBoards[iRoom];

            iBoard = PairBoards[iRoom];
         }
         else
         {
            if((iRow - FirstGridBoard + 1)%5 == 0)	// Last board in batch
               ++iBatch;									// Next batch

            ++iRow;  ++iBoard;
         }

         return;
      }

      // SEARCH FOR THE ROOM IN THE GRID CONTAINING THE ROW
      for(iRoom = RoomCount-1; iRoom > 0; iRoom--)
      	if(row >= GridBoards[iRoom])
         	break;

      FirstGridBoard = GridBoards[iRoom];

      const int Offset = row - FirstGridBoard + 1;

      if(Offset > BoardCounts[iRoom])			// No board here
      {
      	iRow = row; iBoard = 0; iBatch = 0;

         return;
      }

      iRow = row;		iBoard = PairBoards[iRoom] + (iRow - FirstGridBoard);

      iBatch = (iRow - FirstGridBoard)/5;
   }


   bool __fastcall GResGrid::EndedBatchTraverse(Word &Key, const int newrow)
   {
   	if(Cells[0][newrow].IsEmpty())					// newrow ends batch
      {
         Fields[Col]->Dismount();

         SetFocus();	KeyDown(Key, TShiftState());

         return true;
      }
      else
      	return false;										// Continue traverse
   }

   int __fastcall GResGrid::LocateRow(const int pair)
   {
   	// PARAMETER board IDENTIFIES A PLAYER IN GoTour.Pair;  WE FIND THE ROOM
      int Room = -1;

      for(int room=0; room<RoomCount; room++)
      {
      	const int& FirstPair = PairBoards[room];

         if(FirstPair==pair){ Room = room;   break; }
      	if(FirstPair>pair) { Room = room-1; break; }
      }
      if(Room<0) return -1;

      // NOW LOCATE GRID ROW FROM RELATIVE POSITION IN ROOM
      const int Offset = pair - PairBoards[Room];				// Guaranteed >= 0

      const int GridRow = GridBoards[Room] + Offset;			// Guaranteed >= 1

      return GridRow < RowCount ? GridRow : -1;
   }
}

namespace ResGUI												// LOAD CELLS
{
   void __fastcall GResGrid::LoadFromTour()
   {
      int BoardRowCount;   SetupBoards(BoardRowCount);

      RowCount = BoardRowCount>0 ? BoardRowCount + 1 : 2;

      Cells[0][0] = String();
      Cells[0][1] = String();

      for(size_t f=1; f<sfFields; f++)
      {
         Cells[f][0] 			= LoadStr(ResHeadID[f]);
         Cells[f][1]          = String();
      }

      // MARCH THROUGH EACH ROOM
      iBoard = 1;   											// True board index to Pair

      for(iRoom=0; iRoom<RoomCount; iRoom++)
      {
      	const int nBoard = BoardCounts[iRoom];
      	const int fBoard = GridBoards[iRoom];
         const int lBoard = fBoard + ((nBoard+4)/5)*5 - 1;

         for(int b=fBoard; b<=lBoard; b++)
         	if(b-fBoard < nBoard)						// Show pair
            {
            	Tours::GBoard& Board = GoTour.Pair[iBoard];

         		Tours::GPlayerPtr Black =  GoTour.Listed[Board.Black];
         		Tours::GPlayerPtr White =  GoTour.Listed[Board.White];

         		Cells[sfBoard][b] = String(iBoard);

         		Cells[sfBlack][b] = s_S(GoTour.ShortName(Black));
         		Cells[sfWhite][b] = s_S(GoTour.ShortName(White));

         		Cells[sfBres][b] = s_S(GoTour.Result(Black));
         		Cells[sfWres][b] = s_S(GoTour.Result(White));

            	++iBoard;
            }
            else												// Show empty pairs
            {
            	Cells[sfBoard][b] = String();

         		Cells[sfBlack][b] = String();
         		Cells[sfWhite][b] = String();

         		Cells[sfBres][b] = String();
         		Cells[sfWres][b] = String();
            }
      }

      ReBuild();

      TopRow = 1;

      NoCellSelected();

      iRoom = 0; iBoard = 1; iRow = 1; iBatch = 0;
   }

   // LOAD AN EXISTING BOARD AFTER A REGISTER EDIT
   void __fastcall GResGrid::LoadResult(int b)
   {
   	// IDENTIFY PLAYER DETAILS
      Tours::GBoard& Board = GoTour.Pair[b];

      Tours::GPlayerPtr Black =  GoTour.Listed[Board.Black];
      Tours::GPlayerPtr White =  GoTour.Listed[Board.White];

      // LOCATE THE ROW IN THE RESULTS GRID
      const int row = LocateRow(b);   if(row<0) return;

      // LOAD THE CELLS
      Cells[sfBoard][row] = String(b);

      Cells[sfBlack][row] = s_S(GoTour.ShortName(Black));
      Cells[sfWhite][row] = s_S(GoTour.ShortName(White));

      Cells[sfBres][row] = s_S(GoTour.Result(Black));
      Cells[sfWres][row] = s_S(GoTour.Result(White));

      InvalidateRow(row);
   }

   bool __fastcall GResGrid::LocatePair(const int row, Tours::GBoard& Pair)
   {
   	LocateBoard(row);

      if(1<=iBoard && iBoard<(int)GoTour.Pair.size())
      	{ Pair = GoTour.Pair[iBoard]; return true; }
      else
      	{ Pair = Tours::GBoard(0,0);	return false; }
   }
}
//------------------------------------------------------------------------------


namespace ResGUI                                      // HANDLERS
{
   void __fastcall GResGrid::DrawHeading(int col, int row, const TRect &cell)
   {
      DrawAlignedText(Canvas->Handle, ToolsGUI::jyCentre, cell, Cells[col][row]);

      int SortColDir = SortDir[col];

      int Ttop = cell.Top + 2;
      int Tleft = cell.Right - SortUp->Width - 2;

      if(SortColDir == ToolsGUI::gsDn) Canvas->Draw( Tleft, Ttop, SortDn);
      if(SortColDir == ToolsGUI::gsUp) Canvas->Draw( Tleft, Ttop, SortUp);
   }

   void __fastcall GResGrid::DrawCell
   (
      int col, int row,
      const TRect &cell,
      TGridDrawState state
   )
   {
      using ToolsGUI::DrawAlignedText;
      using ToolsGUI::jyCentre;
      using ToolsGUI::jyLeft;
      using ToolsGUI::jyRight;

      TRect Cell = cell;

      ToolsGUI::Justify Aligned = jyCentre;

      if(row==0)
      {
         DrawHeading(col,row, cell);   return;
      }

      switch(col)
      {
         case sfBoard:
            Aligned = jyRight; Cell.Right -= 4;                   break;

         case sfBlack: case sfWhite:
            Aligned = jyLeft;  Cell.Left  += 2;  Cell.Right -=2;  break;

         default: Aligned = jyCentre;
      }
      DrawAlignedText(Canvas->Handle, Aligned, Cell, Cells[col][row]);
   }

   bool __fastcall GResGrid::SelectCell(int col, int row)
   {
   	if(Cells[sfBoard][row].IsEmpty())					 // Empty boards cannot be selected
      {
      	IgnoreClick = true;

         return false;
      }
      else
      {
      	IgnoreClick = false;

      	return GGrid::SelectCell(col,row);
      }
   }

   void __fastcall GResGrid::Click()
   {
   	if(IgnoreClick)
      {
      	OperatorError(IDI_NoEmptyEd);

      	IgnoreClick = false;

         return;
      }

      if(::UnsavedPlayerEdit(IDF_OkCancelEdit))
      	return;

      if(!Tag || !EditPossible())					 // Editing forbidden by State
      {                                          // Or already editing something else
      	ToolsGUI::ResetToEditCell();

      	ClearInfo();

         OperatorError(IDI_NoResultEd);

         return;
      }

      GGrid::Click();

      LocateBoard(Row);
   }

   void __fastcall GResGrid::KeyDown(Word &Key, TShiftState Shift)
   {
      if(Col<0 || Row<0) { Key = 0; return; }
      
      ClearInfo();

      switch(Key)
      {
         case VK_PRIOR:	NextResultsPage(-1);	break;
         case VK_NEXT:	NextResultsPage( 1);	break;

         case VK_RETURN:
         {
         	if(Cells[0][Row].IsEmpty())					// Cannot activate
      		{
            	Beep();  break;
            }

            if(Col==sfBres || Col==sfWres)
            {
            	if(!GoTour.MissingCount)
               {
                  Beep(); ShowInfo(IDI_ResultsDone);
               }

               if(StartFullBatch(Row))
               	FindNextBatch();

               Fields[Col]->Mount();
            }
         }
         break;

         default:
         {
            GGrid::KeyDown(Key, Shift);
            return;
         }
      }
      Key = 0;
   }

   void __fastcall GResGrid::NextResultsPage(const int dir)
   {
   	NextGridPage(dir);

      if(!SelectedCellShowing() || !CurFieldId)
      {
      	SetFocus(); return;
      }

      GResult* ResField = (GResult*)Fields[CurFieldId];

      if(ResField->Focussed())
      	return;
      else
      	SetFocus();
   }

   void __fastcall GResGrid::FocusEditCell(void)
   {
   	using ToolsGUI::EditCell;

   	if(!EditCell)return;

      ExposeSelectedCell();		// It may have been scrolled out of view

      switch(EditCell->ColM)		// Identifies mounted field
      {
      	case sfBres: ((GResult*)EditCell)->SetFocusEd();  break;
      	case sfWres: ((GResult*)EditCell)->SetFocusEd();  break;

         default: break;
      }
   }

}
//------------------------------------------------------------------------------


namespace ResGUI                                      // USER INFO
{
   bool __fastcall GResGrid::ColChange(int Control)
   {
      ToolsGUI::ClearEditCell();

      return true;
   }

   void __fastcall GResGrid::GridChange(void)
   {
      ToolsGUI::ClearEditCell();       // Trapped on Save
   }

   void __fastcall GResGrid::NoDownArrow(void)
   {
      ShowInfo(IDI_EndBoards);
   }
}
//------------------------------------------------------------------------------











