////////////////////////////////////////////////////////////////////////////////
// MODULE  ResGUI   Results Pane Support                     G.Kaniuk 07/04/02
//
#include <grids.hpp>
#include <classes.hpp>

#include "Tour.h"
#include "GridGUI.h"

#ifndef ResGUIH
#define ResGUIH

namespace ResGUI
{
   enum GResFields
   {
      sfBoard, sfBres, sfBlack, sfWres, sfWhite, sfFields
   };

   class GResult;
   class GResGrid;

   class GResEdit : public ToolsGUI::GEdit
   {
      private:
         GResGrid* ResGrid;
         
      private:
         DYNAMIC void __fastcall KeyPress(char& Key);

      public:
         __fastcall GResEdit(GResult* Field);

   };

   class GResult : public ToolsGUI::GField
   {
      friend GResEdit;
      
      private:
         GResGrid* ResGrid;

      private:
         void __fastcall Load();
         void __fastcall Store();
         bool __fastcall SameValue();
         void __fastcall ShowEd();

         void __fastcall FieldKeyDown(Word &Key, TShiftState Shift);

      public:
         __fastcall GResult(GResGrid* grid, String ename);
         __fastcall ~GResult();

         bool __fastcall Focussed(void);
   };

   class GResGrid : public ToolsGUI::GGrid
   {
      friend GResEdit;
      friend GResult;
      
      public:
         String BlackRes, WhiteRes;          // Set by Parser
         int    BlackR,   WhiteR;            // Encoded
         bool   ResultError;						// Set by Parse

      private:
         void __fastcall DrawHeading(int col, int row, const TRect &cell);
         
         void __fastcall DrawCell
         (
            int col, int row,                // Identify cell
            const Windows::TRect &rect,      // Cell geometry
            Grids::TGridDrawState state      // Whether selected
         );

         DYNAMIC void __fastcall KeyDown(Word &Key, TShiftState Shift);

         DYNAMIC void __fastcall Click(void);

         TGroupBox* PopGroup;
         
         bool IgnoreClick;
         
         bool __fastcall SelectCell(int col, int row);

		private:											// Batch management
      	int BoardCounts[Tours::size_rooms];	// Required boards each room - no empty rooms
         int GridBoards[Tours::size_rooms];	// Grid row index of first board in room
         int PairBoards[Tours::size_rooms];  // Pair index of first board in room

      	int RoomCount, iRoom;					// Used rooms indexed from 0
         int BatchCount, iBatch;					// Batches per room indexed from 0
         int iBoard;									// Index to GoTour.Pair - the pairing list
         int iRow;									// The last row accessed

         void __fastcall SetupBoards(int& BoardRowCount);
         int  __fastcall LocateRow(const int pair);
         void __fastcall LocateBoard(const int row);

         bool __fastcall FindNextBatch();
         bool __fastcall FindNextLocalBoard(bool NewResult);
         bool __fastcall EndedBatchTraverse(Word &Key, const int row);
         bool __fastcall StartFullBatch(const int row);

      public:
         __fastcall GResGrid(TGroupBox* parent, TStatusBar* statusbar, TGroupBox* popgroup);
         __fastcall ~GResGrid();

         void __fastcall ReBuild();

      private:
         bool __fastcall ColChange(int Control);
         void __fastcall GridChange(void);
         void __fastcall NoDownArrow(void);

         bool __fastcall Parse(const Word result);

      public:
         void __fastcall LoadFromTour();
         void __fastcall LoadResult(int board);
         void __fastcall FocusEditCell(void);
         void __fastcall NextResultsPage(const int dir);

         bool __fastcall LocatePair(const int row, Tours::GBoard& Pair);
   };
}
//---------------------------------------------------------------------------
#endif
 