////////////////////////////////////////////////////////////////////////////////
// MODULE RulesForm:       Rules setup Dialogue        G.Kaniuk 05/04/02
//
#include <vcl.h>
#pragma hdrstop

#include "RulesIDH.h"
#include "SetsRES.h"
#include "ETour.h"

#include "GoDrawGUI.h"

#include "MenuMaster.h"
#include "PairMUI.h"
#include "RulesForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::CalcBoardRanges(void)
{
	int sBoard = 1, eBoard, BoardCount;

	for(int r=1; r<RoomsGrid->RowCount; r++)
   {
   	String Boards = RoomsGrid->Cells[1][r];

      try
      {
      	BoardCount = !Boards.IsEmpty() ? Boards.ToInt() : 0;
      }
      catch(EConvertError& Err)
      {
         BoardCount = 0;

         RoomsGrid->Row = r;
         
         ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::S_s(Err.Message));
      }

      RulesTube.Boards[r-1] = BoardCount;

      eBoard = sBoard + BoardCount;

      RoomsGrid->Cells[2][r] = RulesTube.Range(sBoard, eBoard);

      sBoard = eBoard;
   }
}

__fastcall TRulesFrm::TRulesFrm
(
	TComponent* Owner,							// GoDrawForm
   RulesMaster::MRulesTube& rulestube		// Initialised from GRules
): TForm(Owner), RulesTube(rulestube)
{
	using TourMaster::GoTour;
	using MenuMaster::LoadCombo;

   // CAPTION   
   Caption = LoadStr(IDM_RulesCaption) + Tools::s_S(GoTour.TourNameYear());

   // RULES HELP ID'S
   HelpFile 				  = GoDraw::HelpFilePath(String("Rules.hlp"));

   StyleBox->HelpContext  = IDH_Style;
   RoundsBox->HelpContext = IDH_Rounds;
   TieOneBox->HelpContext = IDH_Tie1;
   TieTwoBox->HelpContext = IDH_Tie2;
   PrizeBox->HelpContext  = IDH_Prize;
   SegGrp->HelpContext    = IDH_BarSeg;
   RuleGrp->HelpContext   = IDH_RuleSet;
   VoidBox->HelpContext   = IDH_Unresolved;
   HcapEd->HelpContext 	  = IDH_Handicap;
   KomiEd->HelpContext 	  = IDH_Komi;
   SupEd->HelpContext 	  = IDH_Supergroup;
   RoomsGrid->HelpContext = IDH_Boards;
   
   AdvancedBtn->HelpContext = IDH_Advanced;

   // LOAD COMBOS FROM SUPPLIED STRING ARRAYS
   LoadCombo(StyleBox,  Tours::stCount, RulesTube.Styles,  RulesTube.iStyle);
   LoadCombo(TieOneBox, Tours::tbSize,  RulesTube.Ties,    RulesTube.iTieOne);
   LoadCombo(TieTwoBox, Tours::tbSize,  RulesTube.Ties, 	 RulesTube.iTieTwo);
   LoadCombo(VoidBox,   3, 				 RulesTube.NoResls, RulesTube.iNoResl);

   // GENERATE THE PRIZES AND ROUNDS
   MenuMaster::LoadPrizes(PrizeBox, RulesTube.Rounds, RulesTube.Prizes);
   MenuMaster::LoadRounds(RoundsBox, Tours::max_rounds, RulesTube.Rounds);

   if(TourMaster::GoTour.Roundl > 0)
   	for(size_t r=0; r<TourMaster::GoTour.Roundl-1; r++)  // Cannot reduce Rounds below
   		RoundsBox->Items->Delete(0);						     // the last drawn round

   // HANDICAP AND SUPERGROUP EDITORS
   HcapEd->Text = String(RulesTube.dHandicap);
   KomiEd->Text = String(RulesTube.dKomi);
   SupEd->Text  = String(RulesTube.dSuper);

   // TOUR STYLE AND BAR SEGREGATION
   SegGrp->ItemIndex = RulesTube.BarSeg;
   RuleGrp->ItemIndex = RulesTube.RuleSet;

   // ROOM LAYOUT
   RoomsGrid->RowCount		  = Tours::size_rooms+1;

   RoomsGrid->Cells[0][0] = LoadStr(IDM_Room);
   RoomsGrid->Cells[1][0] = LoadStr(IDM_Boards);
   RoomsGrid->Cells[2][0] = LoadStr(IDM_BoardRange);

   int sBoard=1, eBoard, Boards;

   for(int r=1; r<=Tours::size_rooms; r++)
   {
   	Boards = RulesTube.Boards[r-1];   eBoard = sBoard + Boards;

   	RoomsGrid->Cells[0][r] = String(r);
      RoomsGrid->Cells[1][r] = Boards>0 ? String(Boards) : String();
      RoomsGrid->Cells[2][r] = RulesTube.Range(sBoard,eBoard);

      sBoard = eBoard;
   }

   
   // WE CAN LOOK, BUT NOT CHANGE IF EDITING THE DRAW
   if(!RulesTube.Enabled)
   {
   	for(int c=0; c<ControlCount; c++)
      	Controls[c]->Enabled = false;

      AdvancedBtn->Enabled = true;
      HelpBtn->Enabled = true;
      CancelBtn->Enabled = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::RoundsBoxClick(TObject *Sender)
{
	const int NewRounds = RoundsBox->ItemIndex + 1;

   const int NewPrizesIndex = 2*NewRounds;

   MenuMaster::LoadPrizes(PrizeBox, NewRounds, NewPrizesIndex);
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::OKBtnClick(TObject *Sender)
{
	using std::string;
   using Tools::load_str;

   using ToolsGUI::ShowMessage;
   using ToolsGUI::erWarn;

   using Tours::size_rooms;

	RulesTube.iStyle  = StyleBox->ItemIndex;
   RulesTube.iTieOne = TieOneBox->ItemIndex;
   RulesTube.iTieTwo = TieTwoBox->ItemIndex;
   RulesTube.iNoResl = VoidBox->ItemIndex;

   RulesTube.Rounds  = RoundsBox->Items->Strings[RoundsBox->ItemIndex].ToInt();
   RulesTube.Prizes  = PrizeBox->ItemIndex;

   RulesTube.BarSeg  = SegGrp->ItemIndex;
   RulesTube.RuleSet = RuleGrp->ItemIndex;

   RulesTube.dHandicap = HcapEd->Text.ToInt();
   RulesTube.dKomi     = KomiEd->Text.ToInt();
   RulesTube.dSuper    = SupEd->Text.ToInt();

   int BoardSum = 0;														// V 6.5.0 Issue message if too low

   for(int r=1; r<=size_rooms; r++)
   {
   	const String Boards = RoomsGrid->Cells[1][r];

      const int BoardCount = Boards.IsEmpty() ? 0 : Boards.ToInt();

      BoardSum += BoardCount;

   	RulesTube.Boards[r-1] = BoardCount;
   }

   string Warning = string();

   while(Warning.empty())
   {
      if(RulesTube.iTieOne == RulesTube.iTieTwo && RulesTube.iTieOne != Tours::tbNone)
      { Warning = load_str(IDM_EqualTie); break; }

      if(RulesTube.iTieOne == Tours::tbCuss && RulesTube.iTieTwo == Tours::tbCusp)
      { Warning = load_str(IDM_CussCusp); break; }

      if(RulesTube.iTieOne == Tours::tbCusp && RulesTube.iTieTwo == Tours::tbCuss)
      { Warning = load_str(IDM_CussCusp); break; }

      if(RulesTube.iTieOne == Tours::tbSoDoS || RulesTube.iTieTwo == Tours::tbSoDoS)
      { Warning = load_str(IDM_SoDoS); break; }

      if(RulesTube.RuleSet==1 && RulesTube.BarSeg==1)
      { Warning = load_str(IDM_EUBarSeg); break; }

      if(RulesTube.RuleSet==0 && RulesTube.BarSeg==1)
      { Warning = load_str(IDM_UKBarSeg); break; }

      if(BoardSum<RulesTube.BoardsRequired)
      { Warning = load_str(IDM_BoardsNeeded); break; }
      
      break;
   }

   if(!Warning.empty())
      if(ShowMessage(erWarn, Warning+load_str(IDM_ExitRules)) != IDOK)
      	return;
         
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::CancelBtnClick(TObject *Sender)
{
   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------


void __fastcall TRulesFrm::RoomsGridClick(TObject *Sender)
{
	CalcBoardRanges();

	if(RoomsGrid->Col==1)
   	RoomsGrid->EditorMode = true;
   else
   {
   	RoomsGrid->EditorMode = false;

      RoomsGrid->LeftCol = 1;
   }
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::RoomsGridGetEditMask
(
	TObject *Sender,
   int col, int row,		// Any row>0 and col == 1 can be edited
   AnsiString &Value    // The digit mask
)
{
	if(row>0 && col==1)
		Value = "999;0; ";
   else
   	Value = String();
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::RoomsGridKeyDown
(
	TObject *Sender, WORD &Key, TShiftState Shift
)
{
	switch(Key)
   {
   	case VK_DOWN: case VK_UP: case VK_RETURN:
      	CalcBoardRanges();
      break;

      default: return;
   };
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::AdvancedBtnClick(TObject *Sender)
{
	using Tools::load_str;
   using ToolsGUI::ShowMessage;
   using ToolsGUI::erOperr;

	int ParsResult = PairMUI::PairParsForm(this, RulesTube.Enabled, RulesTube.PairParam);

   if(ParsResult == PairMUI::ppFail)
   	ShowMessage(erOperr, load_str(IDM_FailAdvanced));
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::HelpBtnClick(TObject *Sender)
{
	try{ Application->HelpCommand(HELP_CONTEXT, IDH_SMRULES); }catch(...){ }
}
//---------------------------------------------------------------------------

void __fastcall TRulesFrm::FormClose(TObject *Sender, TCloseAction &Action)
{
	try{ Application->HelpCommand(HELP_QUIT, 0); }catch(...){ }
}
//---------------------------------------------------------------------------


