////////////////////////////////////////////////////////////////////////////////
// MODULE RulesMaster:  Interface Ruless to GTourMaster  G.Kaniuk 26/08/04
//

#include <vcl.h>
#pragma hdrstop
#pragma package(smart_init)
//---------------------------------------------------------------------------

#include "SetsRES.h"

#include "TourMaster.h"
#include "RulesMaster.h"

namespace RulesMaster
{
	using TourMaster::GoTour;

	MRulesTube::MRulesTube()
   {
   	// CANNOT CHANGE RULES WHILE EDITING
   	Enabled = GoTour.State <= Tours::tsTourDone;

   	// TOURNAMENT STYLES
   	Styles[0] = LoadStr(IDM_RULESTYLE + Tours::stMcMahon);
   	Styles[1] = LoadStr(IDM_RULESTYLE + Tours::stSwiss);
   	Styles[2] = LoadStr(IDM_RULESTYLE + Tours::stSwissHcap);
   	Styles[3] = LoadStr(IDM_RULESTYLE + Tours::stAllPlay);
   	Styles[4] = LoadStr(IDM_RULESTYLE + Tours::stAllPlayHcap);
      Styles[5] = LoadStr(IDM_RULESTYLE + Tours::stBarLow);
      // TIE BREAKS
      Ties[0] = LoadStr(IDM_TIEBREAK + Tours::tbNone);
      Ties[1] = LoadStr(IDM_TIEBREAK + Tours::tbCuss);
      Ties[2] = LoadStr(IDM_TIEBREAK + Tours::tbCusp);
      Ties[3] = LoadStr(IDM_TIEBREAK + Tours::tbSoS);
      Ties[4] = LoadStr(IDM_TIEBREAK + Tours::tbSoSoS);
      Ties[5] = LoadStr(IDM_TIEBREAK + Tours::tbSoDoS);
      Ties[6] = LoadStr(IDM_TIEBREAK + Tours::tbWins);

      // UNRESOLVED
      NoResls[0] = LoadStr(IDM_UNRESOLVED+(2*Tours::gsZero)/Tours::mc_scale);
      NoResls[1] = LoadStr(IDM_UNRESOLVED+(2*Tours::gsHalf)/Tours::mc_scale);
      NoResls[2] = LoadStr(IDM_UNRESOLVED+(2*Tours::gsOne) /Tours::mc_scale);

      // SETTINGS FROM GRules
      iStyle 	 = GoTour.Rules.Style;
      iTieOne 	 = GoTour.Rules.TieOne;
      iTieTwo 	 = GoTour.Rules.TieTwo;
      iNoResl   = (int)(2*GoTour.Rules.Unresolved)/Tours::mc_scale;

      Rounds	 = GoTour.Rules.Rounds;
      Prizes	 = (2*GoTour.Rules.Prizes)/Tours::mc_scale;

      BarSeg    = (int)GoTour.Rules.BarSeg;
      RuleSet   = (int)GoTour.Rules.RuleSet;

      dHandicap = (int)GoTour.Rules.KomiDiff/Tours::mc_scale;
      dKomi     = (int)GoTour.Rules.KomiRate/Tours::mc_scale;
      dSuper	 = (int)GoTour.Rules.SuperDiff/Tours::mc_scale;

      // BOARDS PER ROOM
      BoardsRequired = GoTour.Pair.size()-1;

      for(int b=0; b<Tours::size_rooms; b++)
      	Boards[b] = GoTour.Boards[b];

      // PAIRING PARAMETERS
      for(int p=0; p<Tours::size_param; p++)
      	PairParam[p] = GoTour.PairParam[p];
   }

   String MRulesTube::Range(int sBoard, int eBoard)
   {
   	if(!eBoard || eBoard <= sBoard) return String();

   	const String Hyphen = String(Tools::sMinus);

   	return String(sBoard) + Hyphen + String(eBoard-1);
   }


   void MRulesTube::Harvest(bool& NewRounds, bool& NewStyle)
   {
   	using Tours::mc_scale;
      using Tours::stMcMahon;
      using Tours::stBarLow;

   	// IF ROUNDS CHANGED, A POTENTIAL STATE CHANGE
      // NOTE THAT ROUNDS CANNOT BE REDUCED BELOW THE LAST ROUND DRAWN
      // BUT IT MAY HAVE INCREASED

      NewRounds = false;

      if((size_t)Rounds != GoTour.Rules.Rounds)
      {
      	NewRounds = true;

      	Tours::TourState NewState = GoTour.State;

      	for(size_t p=1; p<=GoTour.Players; p++)
         	GoTour.Listed[p]->Games.resize(Rounds);	// Automatically play in future

         // INCREASE NUMBER OF ROUNDS
         if((size_t)Rounds>GoTour.Rules.Rounds)
         	if(GoTour.State == Tours::tsTourDone)
            	NewState = Tours::tsRoundDone;

         // DECREASE ROUNDS TO LAST ROUND DRAWN
         if((size_t)Rounds==GoTour.Roundl)
         	if(GoTour.State == Tours::tsRoundDone)
         		NewState = Tours::tsTourDone;

         GoTour.State = NewState;
      }

      // IF Style CHANGED WE WILL NEED TO RESET INITIAL AND CURRENT SCORES
      // Style CHANGE IMPORTANT ONLY FOR McMahon OR BarLow
      {
         const Tours::style_t StyleNow =  (Tours::style_t)iStyle;
         const Tours::style_t StyleWas =  GoTour.Rules.Style;

         const bool StyleWasSeeded = StyleWas==stMcMahon || StyleWas==stBarLow;
         const bool StyleNowSeeded = StyleNow==stMcMahon || StyleNow==stBarLow;

         NewStyle = StyleNowSeeded != StyleWasSeeded;

         GoTour.Rules.Style     = (Tours::style_t)iStyle;
      }

      GoTour.Rules.TieOne    = (Tours::tie_t)iTieOne;
      GoTour.Rules.TieTwo    = (Tours::tie_t)iTieTwo;

      GoTour.Rules.RuleSet   = (Tours::rules_t)RuleSet;
      GoTour.Rules.BarSeg    = (bool)BarSeg;
      GoTour.Rules.Rounds    = Rounds;

      GoTour.Rules.KomiDiff  = dHandicap*mc_scale;
      GoTour.Rules.KomiRate  = dKomi*mc_scale;
      GoTour.Rules.SuperDiff = dSuper*mc_scale;

      GoTour.Rules.Prizes     = (Prizes*mc_scale)/2;
      GoTour.Rules.Unresolved = (iNoResl*mc_scale)/2;

      GoTour.Rooms = 0;

      for(int b=0; b<Tours::size_rooms; b++)
      {
      	GoTour.Boards[b] = Boards[b];

         if(Boards[b]) GoTour.Rooms = b + 1;
      }

      for(int p=0; p<Tours::size_param; p++)
      	GoTour.PairParam[p] = PairParam[p];
   }
}
//---------------------------------------------------------------------------


