////////////////////////////////////////////////////////////////////////////////
// MODULE SeekForm:      User Search                  G.Kaniuk 05/04/02
//
#include <vcl.h>

#pragma hdrstop

#include "GoFormRES.h"

#include "Strings.h"
#include "Format.h"

#include "RegPanelGUI.h"
#include "SeekForm.h"
#include "TourMaster.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

//---------------------------------------------------------------------------
void SaveComboText(TComboBox* Box)
{
	String Text = Box->Text.Trim();

	if(Box->Text.IsEmpty()) return;

   if(Box->Items->IndexOf(Text) >= 0) return;

   Box->Items->Add(Text);
}

//---------------------------------------------------------------------------
static const int ColTabs = 5;
static const int ColPos[5] = { 1, 30,155, 184,  267 };

__fastcall SListBox::SListBox
(
   TComponent* owner, RegGUI::GRegGrid* rgrid
):RegPanelGUI::MListBox(owner, ColTabs, ColPos)
{
	RegGrid = rgrid;
}

void __fastcall SListBox::Click()
{
	using RegMaster::ExtractPlayerId;
   using RegMaster::FindPlayerId;

   if(ItemIndex<0) return;

   size_t Pid = ExtractPlayerId(Items->Strings[ItemIndex]);		if(!Pid) return;

   int Rid = FindPlayerId(Pid);											if(!Rid)return;

   RegGrid->TopRow = Rid;
}

void __fastcall SListBox::KeyDown(Word &Key,TShiftState Shift)
{
   RegPanelGUI::MListBox::KeyDown(Key, Shift);

   if(Key==VK_RETURN) Click();
}
//------------------------------------------------------------------------------

__fastcall TSeekFrm::TSeekFrm
(
	TComponent* Owner,
   RegGUI::GRegGrid* rgrid,
   RegMaster::MFindTube& findtube
)
:TForm(Owner)
{
	FoundList = new SListBox(this, rgrid);

   FoundList->Width = FindBtn->Left - NameChk->Left + FindBtn->Width;

   FoundList->Height = 210;
   FoundList->Left  = 8;  		FoundList->Top    = 260;

   FoundList->SetSorted(true);

   FoundList->TabOrder = 30;
   
   CommentLbl->Caption = LoadStr(IDF_SeekLbl);

   Hide();                                            // Modeless form

   Started = false;
}

__fastcall TSeekFrm::~TSeekFrm()
{
	delete FoundList;
}
//---------------------------------------------------------------------------

void __fastcall TSeekFrm::ExtractLists(void)
{
   TStrings* Players = RegMaster::FindTube.Lists[RegMaster::fiName];
   if(Players)
   {
	   NameBox->Clear();

      for(int p=0; p<Players->Count; p++)
      {
         std::string player = Tools::S_s(Players->Strings[p]);

         size_t ctab = player.find(Tools::cTab, 0);

         if(ctab<player.size())
            NameBox->Items->Add(Tools::s_S(player.substr(0, ctab)));
      }
   }

   TStrings* Grades = RegMaster::FindTube.Lists[RegMaster::fiGrade];
   if(Grades)
   {
   	GradeBox->Clear();

      for(int g=0; g<Grades->Count; g++)
      	GradeBox->Items->Add(Grades->Strings[g]);
   }

   TStrings* Clubs = RegMaster::FindTube.Lists[RegMaster::fiClub];
   if(Clubs)
   {
   	ClubBox->Clear();

      for(int c=0; c<Clubs->Count; c++)
      	ClubBox->Items->Add(Clubs->Strings[c]);
   }

   TStrings* Lands = RegMaster::FindTube.Lists[RegMaster::fiLand];
   if(Lands)
   {
   	LandBox->Clear();

      for(int l=0; l<Lands->Count; l++)
      	LandBox->Items->Add(Lands->Strings[l]);
   }

   TStrings* Groups = RegMaster::FindTube.Lists[RegMaster::fiGroup];
   if(Groups)
   {
   	GroupBox->Clear();

      for(int g=0; g<Groups->Count; g++)
      	GroupBox->Items->Add(Groups->Strings[g]);
   }
}

void __fastcall TSeekFrm::FindBtnClick(TObject *Sender)
{
	using RegMaster::fiFields;
   using RegMaster::MFindTarget;

   // CLEAR THE OLD SEARCH RESULTS
   FoundList->Items->Clear();
   FoundList->ItemIndex = -1;

   // SET UP THE ROW OF TARGETS
	MFindTarget Targets[RegMaster::fiFields];
   MFindTarget* iTarget  = Targets + RegMaster::fiName;

   *iTarget = MFindTarget(NameChk->Checked, NameBox->Text);   ++iTarget;
   *iTarget = MFindTarget(GradeChk->Checked, GradeBox->Text); ++iTarget;
   *iTarget = MFindTarget(ClubChk->Checked,  ClubBox->Text);  ++iTarget;
   *iTarget = MFindTarget(LandChk->Checked,  LandBox->Text);  ++iTarget;
   *iTarget = MFindTarget(GroupChk->Checked, GroupBox->Text); ++iTarget;
   *iTarget = MFindTarget(NoteChk->Checked,  NoteBox->Text);  ++iTarget;

   *iTarget = MFindTarget(MmsChk->Checked,   MmsEd->Text);    ++iTarget;
   *iTarget = MFindTarget(SleepChk->Checked, String());       ++iTarget;

   // PATTERN OF TARGET ROUNDS
   std::string RoundsPlayed(TourMaster::GoTour.Rules.Rounds, Tools::cStar);

   for(int i = 0; i<ControlCount; i++)
   {
   	size_t Round = Controls[i]->Tag;

      if(1 <= Round && Round <= TourMaster::GoTour.Rules.Rounds)
      {
      	String iRound = ((TComboBox*)Controls[i])->Text;

         if(!iRound.IsEmpty())
         	RoundsPlayed[Round-1] = iRound[1];
      }
   }
   Targets[RegMaster::fiRound].Target = Tools::s_S(RoundsPlayed);

   // SEARCH
   int Found = RegMaster::FindTube.Find(Targets, FoundList->Items);

	FoundLbl->Caption = LoadStr(IDF_Found) + String(FoundList->Items->Count);

   
   if(Found != fiFields)
   {
   	FoundList->Items->Add(LoadStr(IDF_BADSEEK + (Found<fiFields ? Found : Found-1)));

      return;
   }

   if(!FoundList->Items->Count)
   {
   	FoundList->Items->Add(LoadStr(IDF_NotFound));

      return;
   }

   if(NoteChk->Checked)
   	SaveComboText(NoteBox);

   FoundList->ItemIndex = 0;
   FoundList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TSeekFrm::FormShow(TObject *Sender)
{
	FoundList->Show();
   
	int Rounds = TourMaster::GoTour.Rules.Rounds;

   for(int i=0; i< ControlCount; i++)
   {
   	int Ctag = Controls[i]->Tag;

      if(1<=Ctag && Ctag <= 12)
      {
      	Controls[i]->Visible = Ctag <= Rounds;

         Controls[i]->Hint = LoadStr(IDF_RoundHint);

         Controls[i]->ShowHint = true;
      }

      if(21<=Ctag && Ctag <= 32)
         Controls[i]->Visible = Ctag <= 20 + Rounds;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSeekFrm::OnKeyDnEnter(TObject *Sender, WORD &Key, TShiftState Shift)
{
	if(Key==VK_RETURN)
   	FindBtnClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TSeekFrm::ClearBtnClick(TObject *Sender)
{
	NameChk->Checked = false;	NameBox->Text = String();
	GradeChk->Checked = false;	GradeBox->Text = String();
   ClubChk->Checked = false;	ClubBox->Text = String();
   LandChk->Checked = false;	LandBox->Text = String();
   GroupChk->Checked = false;	GroupBox->Text = String();
   NoteChk->Checked = false;	NoteBox->Text = String();
   MmsChk->Checked = false;	MmsEd->Text = String();

   SleepChk->Checked = false;
   
   for(int i=0; i< ControlCount; i++)
   {
   	int Ctag = Controls[i]->Tag;

      if(1<=Ctag && Ctag <= 12)
      	((TEdit*)(Controls[i]))->Text = "*";
   }

   FoundList->Clear();

   FoundLbl->Caption = LoadStr(IDF_Found) + String(FoundList->Items->Count);
}
//---------------------------------------------------------------------------

void __fastcall TSeekFrm::RoundClick(TObject *Sender)
{
	((TEdit*)Sender)->SelStart = 0;
   ((TEdit*)Sender)->SelLength = 1;
}
//---------------------------------------------------------------------------


void __fastcall TSeekFrm::CancelBtnClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

