////////////////////////////////////////////////////////////////////////////////
// MODULE SettingsMUI:   Settings Menu  Interface       G.Kaniuk 26/08/04
//

#include <vcl.h>
#pragma hdrstop
#pragma package(smart_init)
//------------------------------------------------------------------------------
#include "GoErrMsgRES.h"
#include "GoPrefsRES.h"
#include "SetsRES.h"

#include "Etour.h"
#include "PrinterAPI.h"
#include "TourMaster.h"
#include "RulesMaster.h"
#include "DrawMaster.h"
#include "MenuMaster.h"
#include "PrefsMaster.h"
#include "RulesForm.h"
#include "TrainForm.h"
#include "PrefForm.h"

#include "SettingsMUI.h"


extern void __fastcall SetGridColour(void);  			// GodrawForm.cpp
//------------------------------------------------------------------------------

namespace SettingsMUI                                  // ERROR MANAGEMENT
{
   static void Throw(ToolsGUI::EType etype, short idevent, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_SettingsMUI, idevent, idmsg);
   }

   static int Serve(Exception& Err, short idevent)
   {
      return ToolsGUI::Serve(Err, IDE_SettingsMUI, idevent);
   }
}
//---------------------------------------------------------------------------
 
namespace SettingsMUI
{
	using RegGUI::GRegGrid;
  	using PairGUI::GPairGrid;

	void __fastcall RulesCmd
   (
   	TComponent* Owner,
      GRegGrid* RegGrid, GPairGrid* PairGrid, GResGrid* ResGrid
   )
   {
   	using ToolsGUI::EditCell;
      using ToolsGUI::EditingGrid;
      
   	using TourMaster::GoTour;
      using Tools::load_str;

   	TRulesFrm* RulesForm = 0;

      try
      {
         RulesMaster::MRulesTube RulesTube;				// Initialised from GoTour.Rules

         if(EditCell)
         {
            if(EditingGrid(ResGrid))
            {
               EditCell->HardDismount();

               RulesTube.Enabled = false;
            }
         }

         if(GoTour.State == Tours::tsVirgin)
         	RulesTube.Enabled = false;
            
      	RulesForm = new TRulesFrm(Owner, RulesTube);

         if(RulesForm->ShowModal() == mrOk)
         {
         	bool NewRounds, NewStyle;

         	RulesForm->RulesTube.Harvest(NewRounds, NewStyle);

            if(NewRounds || NewStyle)
            {
            	if(NewRounds)
               	RegGrid->LoadNewRounds();

               if(NewStyle)
               {
            		DrawMaster::ChangeStyle();

                  RegGrid->LoadNewGroups();
               }

            	GoTour.SaveTournament();
            }
            else
            	GoTour.SaveHeader();

            GoTour.PopulationCount();

            PairGrid->LoadFromTour();
            ResGrid->LoadFromTour();
            
            GoTour.PostTourInfo(IDM_RulesDone);
         }
         else
         	GoTour.PostTourInfo(IDM_Ready);

         delete RulesForm;
      }
      catch(Exception& Err)
      {
      	Serve(Err, IDM_Rules);

         delete RulesForm;
      }
      catch(...)
      {
     		ToolsGUI::ShowWin32Err(IDE_SettingsMUI, IDM_Rules);

         delete RulesForm;
      }
   }
}
//---------------------------------------------------------------------------

namespace SettingsMUI					// TRAINING FORM
{
   void __fastcall TrainingCmd(TComponent* Owner, ResGUI::GResGrid* ResGrid)
   {
   	using TourMaster::GoTour;

  		TTrainFrm* TrainForm = NULL;

      // SET UP TUBE AND SHOW THE TRAINING FORM
      try
      {
      	TrainMaster::MTrainTube TrainTube;

      	TrainForm = new TTrainFrm(Owner, ResGrid, TrainTube);

         if(TrainForm->ShowModal()==mrOk)
         {
         	TrainTube.HarvestModes();

            GoTour.SaveHeader();

            GoTour.PostTourInfo(IDM_Ready);
         }
         delete TrainForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_SettingsMUI, IDM_TrainForm);

         delete TrainForm;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_SettingsMUI, IDM_TrainForm);

         delete TrainForm;
      }
   }
}
//------------------------------------------------------------------------------

namespace SettingsMUI					// PREFERENCES FORM
{
   void __fastcall PrefsCmd(TComponent* Owner, GPairGrid* PairGrid)
   {
   	using TourMaster::GoTour;
      using PrefsMaster::PrefsTube;

      TPrefFrm* PrefsForm = NULL;

      try
      {
      	// THE PrefsTube IS PERMANENT - SHOW THE FORM
      	PrefsForm = new TPrefFrm(Owner, PrefsTube);

         const bool oFlagLand = PrefsTube.ScreenFlagLand;
         const bool oFlagClub = PrefsTube.ScreenFlagClub;
         const int  oNameStyle = PrefsTube.ScreenNameStyle;

         if(PrefsForm->ShowModal()==mrOk)
         {
         	// FINALISE THE TUBE BEFORE SAVING TO FILE
         	if(PrefsTube.ReuseWindow)
               PrefsTube.WindowLocn = ((TForm*)Owner)->BoundsRect;

         	PrefsTube.SaveToFile();

            // HARVEST ANY VISIBLES
            PrinterAPI::SetPrinterFont(PrefsTube.PrFont);
            
            ::SetGridColour();

            const bool NewFlags = oFlagLand != PrefsTube.ScreenFlagLand
            										  ||
            							 oFlagClub != PrefsTube.ScreenFlagClub;

            const bool NewStyle = oNameStyle != PrefsTube.ScreenNameStyle;

            if(NewFlags || NewStyle)PairGrid->LoadFromTour();

            GoTour.PostTourInfo(IDM_Ready);
         }
         delete PrefsForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_SettingsMUI, IDU_PrefsForm);

         delete PrefsForm;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_SettingsMUI, IDU_PrefsForm);

         delete PrefsForm;
      }
   }
}
//------------------------------------------------------------------------------


 