////////////////////////////////////////////////////////////////////////////////
// MODULE TextSmith:  Produce all Tab-Delimited text files     G.Kaniuk 05/04/02
//
#include <stdio.h>
#include <vcl.h>
#pragma hdrstop

#include "TourRES.h"
#include "GoDrawRES.h"
#include "GoErrmsgRES.h"
#include "GoFormRES.h"
#include "PrRES.h"

#include "StringsGUI.h"
#include "GoDrawGUI.h"

#include "ETour.h"
#include "Format.h"
#include "Strings.h"
#include "Sort.h"
#include "MenuMaster.h"
#include "PrefsMaster.h"
#include "RegMaster.h"

#include "TextSmith.h"
//------------------------------------------------------------------------------
namespace TextSmith              // TEXT FILE IMPLEMENTATION
{
   using std::string;
   using std::vector;

   static void Throw(ToolsGUI::EType etype, short idrout, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_TextSmith, idrout, idmsg);
   }

   TextFile::TextFile()
   {
   	FileName = string();

   	Tour = NULL;

      Lines = NULL;
   }

   TextFile::TextFile(string& fname, GTourMaster* tour)
   {
      try
      {
         Tour = tour;

         FileName = fname;

         Lines = NULL;  Lines = new TStringList();

         Lines->Sorted = false;
      }
      catch(...)
      {
         Throw(ToolsGUI::erOperr, IDE_CreateTextFile, IDE_NoMemory);
      }
   }

   TextFile::~TextFile()
   {
      try
      {
         if(!FileName.empty() && Lines != NULL)
            Lines->SaveToFile(Tools::s_S(FileName));

         delete Lines;
      }
      catch(...)
      {
         delete Lines;

         Throw(ToolsGUI::erOperr, IDE_CreateTextFile, IDE_NoFileWrite);
      }
   }

   void TextFile::Add(string line)
   {
      try
      {
         Lines->Add(Tools::s_S(line));
      }
      catch(...)
      {
         Throw(ToolsGUI::erOperr, IDE_TextFileAdd, IDE_NoMemory);
      }
   }

   void TextFile::HeadLine()
   {
      // Add one or more lines processed from Tour
   }

   void TextFile::TextLine(int i)
   {
      //Add one or more lines processed from i and Tour
   }

   void TextFile::BuildTextList(int items)
   {
      HeadLine();

      for(int i=0; i<items; i++)
         TextLine(i);
   }
}
//------------------------------------------------------------------------------

namespace TextSmith                          // RANKLIST TEMPLATE
{
   RankFile::RankFile(string& fname, GTourMaster* tour): TextFile(fname,tour)
   {
   	ClubList = new TStringList();
      LandList = new TStringList();

      String ClubCodeFile = GoDraw::FilePath(GoDraw::GoClubCode);

      if(FileExists(ClubCodeFile))
         ClubList->LoadFromFile(ClubCodeFile);
      else
      {
      	delete ClubList;  ClubList = NULL;

         ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDF_NoClubFile));
      }

      String LandCodeFile = GoDraw::FilePath(GoDraw::GoLandCode);

      if(FileExists(LandCodeFile))
         LandList->LoadFromFile(LandCodeFile);
      else
      {
      	delete LandList;  LandList = NULL;

         ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDF_NoLandFile));
      }

   	Active = 0;
   }

   RankFile::~RankFile()
   {
   	delete ClubList;
      delete LandList;
   }

   void RankFile::RankLine
   (
      bool newgrp, int p, int pos,
      GPlayerPtr P
   )
   {
      // Add a line dependent on player id p and player position pos
   }


   void RankFile::BuildRankList()
   {
      using Tools::ints;

      using Tours::GPlayerPtr;

      size_t p, pos=1;

      Active = Tours::RankSort(Tour);

      Tours::GPlayerPtr P = Tour->Listed[Tour->Sorted[1]];

      ints McScore = P->McScore+1;           // Force New McMahon group trigger

      ints TieOne=0, TieTwo=0;

      HeadLine();                            // Supplied by descendents

      for(p=1; p<=Active; p++)
      {
         bool NewGrp = false;

         P = Tour->Listed[Tour->Sorted[p]];

         if
         (                                   // We get a a new McMahon Group ..
            P->McScore!=McScore ||
            P->TieOne!=TieOne ||
            P->TieTwo!=TieTwo
         )
         {
            McScore = P->McScore;

             TieOne = P->TieOne;
             TieTwo = P->TieTwo;
                pos = p;                     // New player position

             NewGrp = true;
         }
         RankLine(NewGrp, p, pos, P);  // Supplied by descendent
      }
   }
}
//------------------------------------------------------------------------------
namespace TextSmith                          // THE STANDARD RANK LIST
{
   using std::string;
   using std::vector;

   using Tools::tab_add;
   using Tools::to_string;

   using PrefsMaster::PrefsTube;
   using PrefsMaster::rcNoClub;
   using PrefsMaster::rlNoLand;

   static const string fmtPno = "####";      // Player number
   static const string fmtPid = "0000";      // Player id
   static const string fmtRnd = "####";        // Round number
   static const string fmtOpp = "####";
   static const string fmtBrd = "###";
   static const string fmtBal = "+##";			// Explicitly signed for balances


   static const string noGame = "   ...";

   static const string errClub = "???";
   static const string errLand = "??";

   StdRankFile::StdRankFile(string& fname, GTourMaster* tour):RankFile(fname,tour)
   {
   }

   void StdRankFile::HeadLine()
   {
   	using Tools::cZero;
      using Tools::cOne;

      using Tools::load_str;
      using Tools::trim;

      const Tours::tie_t TieOne = Tour->Rules.TieOne;
      const Tours::tie_t TieTwo = Tour->Rules.TieTwo;

      const char cTieOne = TieOne>Tours::tbNone ? cOne : cZero;
      const char cTieTwo = TieTwo>Tours::tbNone ? cOne : cZero;

      string formats(4,cZero);

      formats[0] = char((int)cZero + PrefsTube.ClubStyle);
      formats[1] = char((int)cZero + PrefsTube.LandStyle);
      formats[2] = cTieOne;
      formats[3] = cTieTwo;

      string head0 = load_str(IDM_RANKLIST);

      tab_add(head0, load_str(IDM_ROUND));
      tab_add(head0, to_string(Tour->Round));

      tab_add(head0, trim(Tour->TourName));
      tab_add(head0, to_string(Tour->Year));

      tab_add(head0, load_str(IDM_PLAYERS));
      tab_add(head0, to_string(Active));
      tab_add(head0, formats);

      Add(head0);

      string head = load_str(IDL_GPosn);

      tab_add(head, load_str(IDL_GNum));
      tab_add(head, load_str(IDL_Gname));
      tab_add(head, load_str(IDL_Ggrade));

      const string hClub = load_str(IDL_Gclub);
      const string hLand = load_str(IDL_Gland);
      const string hLocn = load_str(IDL_GLocate);

      const string hClubLand = hClub+string("/")+hLand;

      const string hNull = string();
      const string hSpace = string(" ");

      const int sClub = PrefsTube.ClubStyle;					// None, Code, Full
      const int sLand = PrefsTube.LandStyle;

      if(sClub==0 && sLand==0){ tab_add(head, hNull);	    tab_add(head, hNull); }  // No club, no land
      if(sClub==1 && sLand==0){ tab_add(head, hClub);	    tab_add(head, hNull); }  // Club code, no land
      if(sClub==2 && sLand==0){ tab_add(head, hClub);	    tab_add(head, hNull); }  // Club name, no land

      if(sClub==0 && sLand==1){ tab_add(head, hNull);     tab_add(head, hLand); }  // No club, land code
      if(sClub==1 && sLand==1){ tab_add(head, hLocn);     tab_add(head, hNull); }  // Club code, land code
      if(sClub==2 && sLand==1){ tab_add(head, hClubLand); tab_add(head, hNull); }  // Club name, land code

      if(sClub==0 && sLand==2){ tab_add(head, hNull);     tab_add(head, hLand); }  // No club, land name
      if(sClub==1 && sLand==2){ tab_add(head, hClubLand); tab_add(head, hNull); }  // Club code, land name
      if(sClub==2 && sLand==2){ tab_add(head, hClub);     tab_add(head, hLand); }  // Club name, land name

      for(int r=0; r<(int)Tour->Round; r++)      			// Round numbers
         tab_add(head, hSpace + hSpace + to_string(r+1));

      tab_add(head, load_str(IDL_Gwin));
      tab_add(head, load_str(IDL_Gmms));

      const string hTie1 = TieOne>0 ? Tools::load_str(IDL_GNone + TieOne) : hNull;
      const string hTie2 = TieTwo>0 ? Tools::load_str(IDL_GNone + TieTwo) : hNull;

      tab_add(head, hTie1);
      tab_add(head, hTie2);

      Add(head);
   }

   void StdRankFile::RankLine
   (
      bool newgrp, int p, int pos,
      Tours::GPlayerPtr P
   )
   {
      using Tours::mms_string;
      using Tours::wins_string;
      using Tours::grade_string;

      const int Roundn = Tour->Round;

      const string rNull = string();

      string Line;

      if(newgrp)                                   // We have a new position
         Line = to_string(fmtPno, pos);
      else
         Line =  string();                         // Players at same position

      tab_add(Line, to_string(fmtPno, p));         // Player number

      tab_add(Line, Tour->Name(P));

      tab_add(Line, grade_string(P->Grade));

      switch(PrefsTube.ClubStyle)
      {
         case PrefsMaster::rcClubCode:
         {
         	string Club = P->Club;

            if(ClubList)
            	ToolsGUI::OfficialCode(ClubList, Tour->Clubs, Club);

            tab_add(Line, Club);
         }
         break;

         case PrefsMaster::rcClubName: tab_add(Line, Tour->ClubName(P->Club));  break;
         case PrefsMaster::rcNoClub:   tab_add(Line, rNull);                    break;

         default: tab_add(Line, errClub);
      }

      switch(PrefsTube.LandStyle)
      {
      	case PrefsMaster::rlLandCode:
         {
         	string Land = P->Land;

         	if(LandList)
      			ToolsGUI::OfficialCode(LandList, Tour->Lands, Land);

         	tab_add(Line, Land);
         }
         break;

         case PrefsMaster::rlLandName: tab_add(Line, Tour->LandName(P->Land));  break;
         case PrefsMaster::rlNoLand:	tab_add(Line, rNull);                    break;

         default: tab_add(Line, errLand);
      }

      for(int r=0; r<Roundn; r++)
         tab_add(Line, game_string(P->Games[r]));  // "ddddR"

      tab_add(Line, wins_string(Tour->WinsValue(P)));

      tab_add(Line, mms_string(P->McMahonScore(Roundn)));

      if(Tour->Rules.TieOne>Tours::tbNone)
         tab_add(Line, mms_string(P->TieOne));
      else
         tab_add(Line, rNull);

      if(Tour->Rules.TieTwo>Tours::tbNone)
      	tab_add(Line, mms_string(P->TieTwo));
      else
         tab_add(Line, rNull);
         
      Add(Line);
   }

   string StdRankFile::game_string(Tours::GGame& game)
   {
      using Tours::result_char;

      if(game.Oppid>0)
      {
         int opprid = (int)Tour->Listed[game.Oppid]->Rank;

         string sgame;

         if(opprid>0)
            return to_string(fmtOpp, opprid) + string(1, result_char(game.Result));
         else
            return noGame;
      }
      else
      	return noGame;
   }
}
//------------------------------------------------------------------------------

namespace TextSmith                                         // RANKLIST FOR THE WEB
{
   WebRankFile::WebRankFile(string& fname, GTourMaster* tour):StdRankFile(fname,tour)
   {
   }

   string WebRankFile::game_string(Tours::GGame& game)
   {
      using Tools::sBra;
      using Tools::sKet;
      using Tools::to_string;
      using Tools::modulus;


      const string sgame = StdRankFile::game_string(game);
      const int stones   = Tools::modulus(game.Stones)/Tours::mc_scale;
      const string shcap = string(sBra)+to_string(stones)+string(sKet);

      return shcap + Tools::trim_left(sgame);
   }

   void WebRankFile::HeadLine()
   {
      using Tools::load_str;
      using Tools::trim;
      using TourMaster::GoTour;

      // WEB~RANKLIST~<Tournament>~<Year>
      string htour = load_str(IDM_WEB);
      tab_add(htour,load_str(IDM_RANKLIST));
      tab_add(htour, trim(Tour->TourName));
      tab_add(htour, to_string(Tour->Year));
      Add(htour);

      // BAR~<grade>
      string hbar = load_str(IDM_BAR);
      tab_add(hbar, trim(Tours::grade_string(GoTour.Rules.MacBar)));
      Add(hbar);

      // COLUMN HEADER
      string head = load_str(IDL_GPOSN);
      tab_add(head, load_str(IDL_GNO));
      tab_add(head, load_str(IDL_IName));
      tab_add(head, load_str(IDL_IGrade));

      if(PrefsTube.ClubStyle>PrefsMaster::rcNoClub)    // Club Code or Name
         tab_add(head, load_str(IDL_IClub));

      if(PrefsTube.LandStyle>PrefsMaster::rlNoLand)    // Country Code or name
         tab_add(head, load_str(IDL_ILand));

      for(int r=0; r<(int)Tour->Roundl; r++)      // Round numbers
         tab_add(head, to_string(r+1));

      tab_add(head, load_str(IDL_GWIN));
      tab_add(head, load_str(IDL_GMMS));

      Tours::tie_t TieOne = Tour->Rules.TieOne;
      Tours::tie_t TieTwo = Tour->Rules.TieTwo;

      if(TieOne>Tours::tbNone)
      {
         tab_add(head, trim(load_str(IDL_GNONE + TieOne)));

         if(TieTwo>Tours::tbNone)
            tab_add(head, trim(load_str(IDL_GNONE + TieTwo)));
      }

      Add(head);
   }

   void WebRankFile::RankLine
   (
      bool newgrp, int p, int pos,
      Tours::GPlayerPtr P
   )
   {
      using Tours::mms_ustring;
      using Tours::wins_ustring;
      using Tours::grade_ustring;

      string Line;

      if(newgrp)                                   // We have a new position
         Line = to_string(pos);
      else
         Line =  string();                         // Players at same position

      tab_add(Line, to_string(p));                 // Player number

      tab_add(Line, Tour->Name(P));

      tab_add(Line, grade_ustring(P->Grade));

      switch(PrefsTube.ClubStyle)
      {
         case PrefsMaster::rcClubCode:
         {
         	string Club = P->Club;

            if(ClubList)
            	ToolsGUI::OfficialCode(ClubList, Tour->Clubs, Club);

            tab_add(Line, Club);
         }
         break;

         case PrefsMaster::rcClubName: tab_add(Line, Tour->ClubName(P->Club));  break;
         case PrefsMaster::rcNoClub:                                            break;

         default: tab_add(Line, errClub);
      }

      switch(PrefsTube.LandStyle)
      {
         case PrefsMaster::rlLandCode:
         {
         	string Land = P->Land;

         	if(LandList)
      			ToolsGUI::OfficialCode(LandList, Tour->Lands, Land);

         	tab_add(Line, Land);
         }
         break;

         case PrefsMaster::rlLandName: tab_add(Line, Tour->LandName(P->Club));  break;
         case PrefsMaster::rlNoLand:                                            break;

         default: tab_add(Line, errLand);
      }

      for(int r=0; r<(int)Tour->Roundl; r++)
         tab_add(Line, game_string(P->Games[r]));  // "ddddR"


      tab_add(Line, wins_ustring(Tour->WinsValue(P)));

      tab_add(Line, mms_ustring(P->McScore));

      if(Tour->Rules.TieOne>Tours::tbNone)
      {
         tab_add(Line, mms_ustring(P->TieOne));

         if(Tour->Rules.TieTwo>Tours::tbNone)
            tab_add(Line, mms_ustring(P->TieTwo));
      }
      Add(Line);
   }
}
//---------------------------------------------------------------------------

namespace TextSmith                                         // REGISTER PRINT FILE
{
   static const char* rReg = "R";
   static const char* rNoReg = "_";

   RegisterPriFile::RegisterPriFile(std::string& fname, GTourMaster* tour):TextFile(fname,tour)
   {
   	LandList = new TStringList();

      String LandCodeFile = GoDraw::FilePath(GoDraw::GoLandCode);

      if(FileExists(LandCodeFile))
         LandList->LoadFromFile(LandCodeFile);
      else
      {
      	delete LandList;  LandList = NULL;

         ToolsGUI::ShowMessage(ToolsGUI::erOperr, Tools::load_str(IDF_NoLandFile));
      }
   }

   RegisterPriFile::~RegisterPriFile()
   {
   	delete LandList;
   }

   void RegisterPriFile::HeadLine()
   {
   	using Tools::load_str;
      using Tools::to_string;
      using Tools::trim;
      using Tools::sSpace;

      string regtitle = load_str(IDL_RTITLE);
      string regsort  = load_str(IDL_IPlayer + Tour->SortOrder);
      string regtour  = trim(Tour->TourName) + string(sSpace) + to_string(Tour->Year);

      // HEAD 1:REGISTER SORTED BY <SortKey>~<Tournament> <Year>~<Rounds>~PAGE~OF~<PlayerCount>
      string head1 = regtitle+regsort;
      tab_add(head1, regtour);
      tab_add(head1, to_string(Tour->Rules.Rounds));
      tab_add(head1, load_str(IDL_RPAGE));
      tab_add(head1, load_str(IDL_ROF));
      tab_add(head1, to_string(Tour->Players));
      Add(head1);

      // HEAD2:ID REG NAME GRADE CLUB COUNTRY ROUNDS GROUP NOTE
      string head2 = load_str(IDL_IPlayer);
      for(int h=1; h<9; h++)
      	tab_add(head2, load_str(IDL_IPlayer + h));

      Add(head2);
   }

   void RegisterPriFile::TextLine(int p)
   {
   	using RegMaster::FindTube;

   	GPlayerPtr P = Tour->GetPlayerPtr(p+1);	if(!P) return;

      // GET VISIBLE LAND CODE
      string Land = P->Land;

      if(LandList)
      	ToolsGUI::OfficialCode(LandList, Tour->Lands, Land);

   	// LINE:Pid~Tick~Name~Grade~Club~Land~Rounds~Group~Note
   	string Line = to_string(fmtPid, (int)P->Pid);

      string Tick = P->State >= Tours::psRegistered ? rReg : rNoReg;

      tab_add(Line, Tick);

      tab_add(Line, P->Name);
      tab_add(Line, Tour->Grade(P));
      tab_add(Line, Tour->Club(P));
      tab_add(Line, Land);
      tab_add(Line, Tour->Rounds(P));
      tab_add(Line, Tour->Group(P));
      tab_add(Line, Tour->Note(P));

      Add(Line);
   }
}
//---------------------------------------------------------------------------

namespace TextSmith                                         // DRAW/RESULTS PRINT FILE
{
   DrawResPriFile::DrawResPriFile(int mode, std::string& fname, GTourMaster* tour):TextFile(fname,tour)
   {
   	Mode = mode;
   }

   string DrawResPriFile::Name(GPlayerPtr player)
   {
   	using Tools::ints;
      using Tools::spaces;

   	string name = Tour->Name(player);

      string bra  = string(Tools::sBra);
      string ket  = string(Tools::sKet);

      // WE CAN GUARANTEE THAT round >= 0 SINCE DrawResPriFile
      // IS NOT CONSTRUCTED IF Tour->Round <= 0
      const int round = Mode==drDraw ? Tour->Round-1 : Tour->Round;

   	switch(PrefsTube.NameStyle)
      {
      	case PrefsMaster::dpSufGrade:
         	return name + spaces(1) + bra + Tour->Grade(player) + ket;

         case PrefsMaster::dpSufMMS:
         {
            const ints pmms = player->McMahonScore(round);

            return name + spaces(1) + bra + Tours::mms_ustring(pmms) + ket;
         }

         default:
         	return name;
      }
   }

   string DrawResPriFile::Komi(GBoard board)
   {
   	using Tools::modulus;
      using Tools::to_string;
      using Tools::load_str;
      
      int Round = Tour->Round;

      if(Round<1 || !board.Black || !board.White ) return string();

      Tours::GGame Game = Tour->Listed[board.Black]->Games[Round-1];

      int stones = Game.Stones/Tours::mc_scale;
      int komi   = Game.Komi/Tours::mc_scale;

      if(!stones && !komi) return string();

      string Stones = load_str(IDM_stones);
      string Komi   = load_str(IDM_komi);
      string St     = load_str(IDM_st);

      char Handicap[80];

      if(komi>0)
      	sprintf
         (
         	Handicap,	"%s %s + %s %s",
            to_string(stones).c_str(), St.c_str(),
            to_string(komi).c_str(), Komi.c_str()
         );

      if(komi==0)
      {
      	if(stones>1)
            sprintf
            (
               Handicap,	"%s %s",
               to_string(stones).c_str(), Stones.c_str()
            );

         if(stones==1)
         	sprintf(Handicap, "%s", load_str(IDM_no_komi).c_str());
      }

      if(komi<0)
      	sprintf
         (
         	Handicap,	"%s %s - %s %s",
            to_string(stones).c_str(), St.c_str(),
            to_string(-komi).c_str(), Komi.c_str()
         );

      return string(Handicap);
   }


   void DrawResPriFile::HeadLine()
   {
   	using Tools::load_str;
      using Tools::to_string;
      using Tools::trim;

      // HEAD 1:ROUND~<N>~DRAW/RESULTS~<Tournament>~<Year>
      string head1 = load_str(IDM_ROUND);
      tab_add(head1, to_string(Tour->Round));
      tab_add(head1, load_str(IDM_DRAW + Mode));
      tab_add(head1, trim(Tour->TourName));
      tab_add(head1, to_string(Tour->Year));

      Add(head1);

      // HEAD2:ROOMS <n> BOARDS <b>
      const int Boards = Tour->Pair.size()-1;  Tour->Rooms = 0;

      for(int r=0; r<Tours::size_rooms; r++)
      	if(Tour->Boards[r])
         	++Tour->Rooms;

      if(!Tour->Rooms)								// Put all boards in room 1
      {
      	Tour->Rooms = 1;
         Tour->Boards[0] = Boards;
      }

      string head2 = load_str(IDM_ROOMS);
      tab_add(head2, to_string(Tour->Rooms));
      tab_add(head2, load_str(IDM_BOARDS));
      tab_add(head2, to_string(Boards));		// Actual number of boards

      Add(head2);

      // HEAD3*:ROOM  <n> BOARDS <a> TO <b>
      string head3;

      int Fboard = 1, Lboard = 1;

      for(int r=0; r<Tours::size_rooms; r++)
      {
      	head3 = load_str(IDM_ROOM);

      	if(Tour->Boards[r])
         {
         	Lboard = Fboard + Tour->Boards[r] - 1;

         	tab_add(head3, to_string(r+1));
            tab_add(head3, load_str(IDM_BOARDS));
            tab_add(head3, to_string(Fboard));
            tab_add(head3, load_str(IDM_TO));
            tab_add(head3, to_string(Lboard));

         	Add(head3);

            Fboard = Lboard + 1;
         }
      }

      // HEAD4: BOARD BLACK HANDICAP WHITE
      string head4 = load_str(IDM_BOARD);
      tab_add(head4, load_str(IDM_BLACK));
      tab_add(head4, load_str(IDM_HANDICAP));
      tab_add(head4, load_str(IDM_WHITE));

      Add(head4);
   }

   void DrawResPriFile::TextLine(int b)
   {
   	using Tours::GPlayer;
      using Tours::GBoard;

   	GBoard& Board = Tour->Pair[b+1];

      const string NoName = string("? ?");
      const string NoRes  = string("_");

   	GPlayerPtr Black = Tour->Listed[Board.Black];
      GPlayerPtr White = Tour->Listed[Board.White];

      string Bres    = Black ? Tour->Result(Black) : NoRes;
      string Wres    = White ? Tour->Result(White) : NoRes;

      string Kboard  = Komi(Board);

      string Bname   = Black ? Name(Black) : NoName;
      string Wname   = White ? Name(White) : NoName;

      if(Mode == drDraw)
      {
      	Bres = NoRes;	Wres = NoRes;
      }

   	// LINE:<brd>~<res>~Name~<handicap>~<res>~Name
   	string Line = to_string(fmtBrd, b+1);
      tab_add(Line, Bres);
      tab_add(Line, Bname);
      tab_add(Line, Kboard);
      tab_add(Line, Wres);
      tab_add(Line, Wname);

      Add(Line);
   }
}
//---------------------------------------------------------------------------

namespace TextSmith                                         // PRIZE LIST PRINT FILE
{
   PrizePriFile::PrizePriFile(std::string& fname, GTourMaster* tour):TextFile(fname,tour)
   {
   }

   void PrizePriFile::HeadLine()
   {
   	using Tools::load_str;
      using Tools::to_string;
      using Tools::trim;
      
      // HEAD1:PRIZE LIST~ ROUND <n> <Tournament>~<Year>~<PRIZES>~<PrizeCount>~<PLAYERS>
      string head1 = load_str(IDM_PRIZELIST);
      tab_add(head1, load_str(IDM_ROUND));
      tab_add(head1, to_string(Tour->Round));
      tab_add(head1, trim(Tour->TourName));
      tab_add(head1, to_string(Tour->Year));

      tab_add(head1, load_str(IDM_PRIZES));
      tab_add(head1, to_string(Tour->PrizeCount));
      tab_add(head1, load_str(IDM_PLAYERS));
      Add(head1);

      // HEAD3:NO ID REG NAME GRADE CLUB COUNTRY WINS RANK
      string head2 = load_str(IDL_GNO);
      tab_add(head2, load_str(IDL_IPlayer));
      tab_add(head2, load_str(IDL_IName));
      tab_add(head2, load_str(IDL_IGrade));
      tab_add(head2, load_str(IDL_IClub));
      tab_add(head2, load_str(IDL_ILand));

      tab_add(head2, load_str(IDL_GWINS));
      tab_add(head2, load_str(IDL_GRANK));

      Add(head2);
   }

   void PrizePriFile::TextLine(int p)
   {
   	GPlayerPtr P = Tour->GetPlayerPtr(p+1);	if(!P) return;

      string wins = Tours::wins_ustring(P->Wins());

   	// LINE:No~Pid~Name~Grade~Club~Land~Rank~Wins
      string Line = to_string(p+1);
   	tab_add(Line, to_string(fmtPid, (int)P->Pid));
      tab_add(Line, Tour->Name(P));
     	tab_add(Line, Tour->Grade(P));
      tab_add(Line, Tour->Club(P));
      tab_add(Line, Tour->Land(P));
      tab_add(Line, wins);
      tab_add(Line, to_string(P->Rank));

      Add(Line);
   }
}
//---------------------------------------------------------------------------
namespace TextSmith                                         // PRIZE LIST PRINT FILE
{
	using Tools::intsi;

	static string colour_string(intsi colour)
   {
   	switch(colour)
      {
      	case Tours::gcBlack: return string("b");
         case Tours::gcWhite: return string("w");

         default: return string();
      }
   }

   static string parity_string(intsi parity)
   {
   	switch(parity)
      {
      	case Tours::gdDown: return string("-");
         case Tours::gdUp:   return string("+");

         default: return string("o");
      }
   }


   CardsPriFile::CardsPriFile
   (
   	std::string& fname,
      int pfirst, int plast,
      GTourMaster* tour
   ):TextFile(fname,tour)
   {
   	Pfirst = pfirst;
      Plast  = plast;
   }

   void CardsPriFile::HeadLine()
   {
   	using Tools::load_str;
      using Tools::to_string;
      using Tools::trim;
      
      // HEAD1:CARDS~ ROUND~<n>~Of~<m>~<Tournament>~<Year>
      string head1 = load_str(IDM_CARDS);
      tab_add(head1, load_str(IDM_ROUND));
      tab_add(head1, to_string(Tour->Round));
      tab_add(head1, load_str(IDM_Of));
      tab_add(head1, to_string(Tour->Rules.Rounds));
      tab_add(head1, trim(Tour->TourName));
      tab_add(head1, to_string(Tour->Year));
      Add(head1);

      // HEAD2:PLAYERS <first>~TO <last>
      string head2 = load_str(IDM_PLAYERS);
      tab_add(head2, to_string(Pfirst));
      tab_add(head2, load_str(IDM_To));
      tab_add(head2, to_string(Plast));

      Add(head2);
   }

   void CardsPriFile::TextLine(int p)
   {
   	using Tours::result_char;
      using TourMaster::GoTour;
      
   	GPlayerPtr P = Tour->GetPlayerPtr(p+Pfirst);	if(!P) return;

      const string Cbal("C"), Mbal("M"), Mout("U"), Space(" ");

   	// LINE:Posn G Family(Grade)~Club Land
      string Line = to_string(fmtPid, (int)P->Rank)+Space+Space+Tour->ShortNameGrade(P);
      tab_add(Line, Tour->Club(P)+Space+P->Land);

      // LINE +  ~MMS~Cbal~Mbal~Uneven
      tab_add(Line, Tools::load_str(IDL_GMMS));
      tab_add(Line, Tours::mms_ustring(P->McScore));
      tab_add(Line, Cbal+Space+to_string(fmtBal, P->Cbal));
      tab_add(Line, Mbal+Space+to_string(fmtBal, P->Mbal));
      tab_add(Line, Mout+Space+Space+Space+to_string(P->Mout));

      // LINE + GameList
      for(size_t r=0; r<GoTour.Rules.Rounds; r++)
      {
      	Tours::GGame& Game = P->Games[r];

         bool Sleep = Game.Result == Tours::grSleep;
         bool Wait  = r >= GoTour.Roundl && !Sleep;
         bool Play  = r < GoTour.Roundl && !Sleep;

         char Result = result_char(Game.Result);

         if(Result == Tools::cPlus) Result = Tools::cWin;
         if(Result == Tools::cMinus)Result = Tools::cLose;
         if(Result == Tools::cEqual)Result = Tools::cJigo;

         if(Play)
         {
         	int Opp = GoTour.Listed[Game.Oppid]->Rank;

         	tab_add(Line, to_string(fmtPid, Opp));
         	tab_add(Line, colour_string(Game.Colour));
         	tab_add(Line, parity_string(Game.Parity));
         	tab_add(Line, string(1,Result));
         }

         if(Wait)
         	for(int g=0; g<4; g++)
         		tab_add(Line, string(2, Tools::cUline));

         if(Sleep)
         	for(int g=0; g<4; g++)
            	tab_add(Line, string(2, Tools::cDot));
      }
      Add(Line);
   }
}
//---------------------------------------------------------------------------


