////////////////////////////////////////////////////////////////////////////////
// MODULE TourMUI:       Interface to TourMaster::GoTour       G.Kaniuk 08/09/04
//

#include <vcl.h>
#pragma hdrstop
#pragma package(smart_init)
//----------------------------------------------------------------------------

#include "GoErrMsgRES.h"
#include "GoInfoRes.h"
#include "TourRES.h"
#include "SetsRES.h"
#include "GroupRES.h"
#include "EdPlayerRES.h"

#include "GoDrawFormIDH.h"

#include "ETour.h"
#include "RegGUI.h"

#include "TourStream.h"
#include "TourMaster.h"
#include "RegMaster.h"
#include "MenuMaster.h"
#include "PairTool.h"

#include "PairForm.h"
#include "RankForm.h"
#include "PrizeForm.h"
#include "GroupForm.h"
#include "CloseRegForm.h"
#include "JoinDrawForm.h"

#include "TourMUI.h"

extern void __fastcall BeginCommand(int IdCmd); 		// IN GoDrawForm.cpp
extern void __fastcall BeginCommand(void);
extern void __fastcall EndCommand(int IdCmd);
extern void __fastcall SetFormText(void);
extern void __fastcall ClearTourInfo(void);

extern void __fastcall ShowTourStatus(int messid);
//extern void __fastcall EditPlayer(const int player);
extern bool __fastcall DismissedPlayerEdit(const int idmessage);
extern void __fastcall ViewPlayer(const int blackwhite, const int player);
//
//---------------------------------------------------------------------------
//
namespace TourMUI													// TOOLS
{
	using TourMaster::GoTour;

   static void ToCellNow(ToolsGUI::GGrid* Grid)
   {
   	ToolsGUI::GField* Fnow = Grid->GetCurField();

      if(Fnow)
      {
      	Grid->SelectGridCell(Fnow->ColM, Fnow->RowM);

         Fnow->SetFocusEd();
      }

      Grid->ShowInfoStatus(GoTour.TourInfo(), IDI_SVirgin + GoTour.State);
   }
}

namespace TourMUI													// REGISTER POPUP
{
	__fastcall MRegPopup::MRegPopup(RegGUI::GRegGrid* reggrid):TPopupMenu(reggrid->Owner)
   {
   	RegGrid   = reggrid;

      AutoPopup = true;
      OnPopup   = OnRegPopup;

      try
      {
         NewItem = new TMenuItem(this);
         	NewItem->Caption = LoadStr(IDT_PlPopNew);
         	NewItem->OnClick = PlayerEditClick;
         Items->Add(NewItem);

         ExistItem = new TMenuItem(this);
         	ExistItem->Caption = LoadStr(IDT_PlPopExist);
         	ExistItem->OnClick = PlayerEditClick;
         Items->Add(ExistItem);

         Items->InsertNewLineAfter(ExistItem);

         GroupItem = new TMenuItem(this);
         	GroupItem->Caption = LoadStr(IDT_PlPopGroup);
         	GroupItem->OnClick = PlayerGroupClick;
         Items->Add(GroupItem);

         DeleteItem = new TMenuItem(this);
         	DeleteItem->Caption = LoadStr(IDT_PlPopDelete);
         	DeleteItem->OnClick = DeletePlayerClick;
            DeleteItem->Enabled = false;
         Items->Add(DeleteItem);

         Items->InsertNewLineAfter(DeleteItem);

         HelpItem = new TMenuItem(this);
         	HelpItem->Caption = LoadStr(IDT_PlPopHelp);
         	HelpItem->OnClick = HelpClick;
         Items->Add(HelpItem);
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMUI, IDT_RegPopup);

         delete NewItem;
         delete ExistItem;
         delete GroupItem;
         delete DeleteItem;
         delete HelpItem;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_RegPopup);

         delete NewItem;
         delete ExistItem;
         delete GroupItem;
         delete DeleteItem;
         delete HelpItem;
      }
   }

   __fastcall MRegPopup::~MRegPopup()
   {
      Items->Remove(NewItem);		NewItem->Free();
      Items->Remove(ExistItem);	ExistItem->Free();
      Items->Remove(GroupItem);	GroupItem->Free();
      Items->Remove(DeleteItem);	DeleteItem->Free();
      Items->Remove(HelpItem);	HelpItem->Free();
   }

   void __fastcall MRegPopup::OnRegPopup(TObject *Sender)
   {
   	using TourMaster::cmdDel;
      using TourMaster::EnabledCommands;
      using TourMaster::GoTour;

      using Tours::tsRegNow;
      using Tours::tsTourDone;

      using RegGUI::rsDormant;
      using RegGUI::rfName;

      GoTour.ClearTourInfo();

      NewItem->Enabled    = false;
      ExistItem->Enabled  = false;
      DeleteItem->Enabled = false;
      GroupItem->Enabled  = false;
      HelpItem->Enabled   = true;

      SelRows = RegGrid->Selection.Bottom - RegGrid->Selection.Top;
      SelCols = RegGrid->Selection.Right  - RegGrid->Selection.Left;

      // MULTIPLE DELETION
      const int NewPlayerRow = RegGrid->RowCount-1;

      if(SelRows || SelCols)
      {
         if(RegGrid->Selection.Bottom == NewPlayerRow)
            Beep();
         else
				// DeleteItem->Enabled = true; 						// GK 6.3.9 
            DeleteItem->Enabled = EnabledCommands[cmdDel][GoTour.State];
         return;
      }

      // SO WE HAVE RIGHT CLICKED ON GRID OR EMPTY SPACE
      int PopRow, PopCol;

      RegGrid->AnalysePopupCell(PopupPoint, PopCol, PopRow);

      if(PopRow==-1 || PopCol ==-1)
      {
      	PopCol = rfName;  PopRow = NewPlayerRow;
      }

      // RIGHT CLICK ON HEADER OR FIRST COL
      if(PopRow == 0 || PopCol==0) return;


      // RIGHT CLICK SINGLE CELL IN BODY
      const bool FullPopup    = RegGrid->Status != RegGUI::rsFastReg;
      const bool EditPossible = RegGrid->Tag && RegGrid->EditPossible();

      if(FullPopup)
      {
      	if(PopRow >= NewPlayerRow)
            NewItem->Enabled = EditPossible && !GoTour.PastRound();
         else
         {
      		ExistItem->Enabled = EditPossible || GoTour.Players>0 && GoTour.Round == 0;

   			DeleteItem->Enabled = EnabledCommands[cmdDel][GoTour.State];

            if(GoTour.Rules.Style==Tours::stMcMahon || GoTour.Rules.Style==Tours::stBarLow)
               if(tsRegNow<=GoTour.State && GoTour.State<=tsTourDone)
                  GroupItem->Enabled = EditPossible;
         }

      	RegGrid->SelectGridCell(PopCol, PopRow);
      }
   }

   void __fastcall MRegPopup::PlayerEditClick(TObject *Sender)
   {
		using TourMaster::GoTour;
      using Tours::GPlayerPtr;

      GoTour.ClearTourInfo();

      RegGrid->StartPlayerEdit();

      ::SetFormText();

		RegGrid->ShowInfoStatus(GoTour.TourInfo(), IDI_SVirgin + GoTour.State);
   }

   void __fastcall MRegPopup::PlayerGroupClick(TObject *Sender)
   {
		using TourMaster::GoTour;
      using Tours::GPlayerPtr;

      GoTour.ClearTourInfo();

      if(!::DismissedPlayerEdit(IDI_ToolPlayerEd))
      	return;

      if(!SelRows && !SelCols)
      {
      	RegGrid->SelectPopupCell(PopupPoint);

         GroupsForm(this, RegGrid, RegGrid->Row);

			RegGrid->ShowInfoStatus(GoTour.TourInfo(), IDI_SVirgin + GoTour.State);
      }
   }

   void __fastcall MRegPopup::DeletePlayerClick(TObject *Sender)
   {
   	if(!::DismissedPlayerEdit(IDI_DelPlayersEd)) return;

   	::BeginCommand(TourMaster::cmdDel);
      {
         if(!SelRows && !SelCols)
      		RegGrid->SelectPopupCell(PopupPoint);

      	RegGrid->DeleteSelectedPlayers();
      }
      ::EndCommand(TourMaster::cmdDel);
   }

   void __fastcall MRegPopup::HelpClick(TObject *Sender)
   {
		using TourMaster::GoTour;

      GoTour.ClearTourInfo();

      Application->HelpCommand(HELP_CONTEXT, IDH_REGPOP);
   }
}

//------------------------------------------------------------------------------
namespace TourMUI													// PAIR GRID POPUP
{
	using ToolsGUI::GGrid;
	using TourMaster::GoTour;

	__fastcall MPairPopup::MPairPopup
   (
   	TComponent* owner, PaneGUI::GPairPane* pairpane
   )
   :TPopupMenu(owner), PairPane(pairpane)
   {
      AutoPopup = true;

      BlackItem = 0; WhiteItem = 0; PairItem = 0; HelpItem = 0;

      try
      {
         BlackItem = new TMenuItem(this);
         	BlackItem->Caption = LoadStr(IDT_Black);
         	BlackItem->OnClick = BlackClick;
         Items->Add(BlackItem);

         WhiteItem = new TMenuItem(this);
         	WhiteItem->Caption = LoadStr(IDT_White);
         	WhiteItem->OnClick = WhiteClick;
         Items->Add(WhiteItem);

         PairItem = new TMenuItem(this);
         	PairItem->Caption = LoadStr(IDT_Pair);
         	PairItem->OnClick = PairClick;
         Items->Add(PairItem);

         Items->InsertNewLineAfter(PairItem);

         RankItem = new TMenuItem(this);
         	RankItem->Caption = LoadStr(IDT_RankList);
         	RankItem->OnClick = RankClick;
         Items->Add(RankItem);

         PrizeItem = new TMenuItem(this);
         	PrizeItem->Caption = LoadStr(IDT_PrizeList);
         	PrizeItem->OnClick = PrizeClick;
         Items->Add(PrizeItem);

         Items->InsertNewLineAfter(PrizeItem);

         SimuItem = new TMenuItem(this);
         	SimuItem->Caption = LoadStr(IDT_Simulate);
         	SimuItem->OnClick = SimuClick;
            SimuItem->Enabled = false;
         Items->Add(SimuItem);

         ClearItem = new TMenuItem(this);
         	ClearItem->Caption = LoadStr(IDT_Clear);
         	ClearItem->OnClick = ClearClick;
            ClearItem->Enabled = false;
         Items->Add(ClearItem);

         Items->InsertNewLineAfter(ClearItem);

         HelpItem = new TMenuItem(this);
         	HelpItem->Caption = LoadStr(IDT_PlPopHelp);
         	HelpItem->OnClick = HelpClick;
         Items->Add(HelpItem);

         OnPopup = PairPopupClick;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMUI, IDT_PairPopup);

         delete BlackItem;
         delete WhiteItem;
         delete PairItem;
         delete RankItem;
         delete PrizeItem;
         delete SimuItem;
         delete ClearItem;
         delete HelpItem;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_PairPopup);

         delete BlackItem;
         delete WhiteItem;
         delete PairItem;
         delete RankItem;
         delete PrizeItem;
         delete SimuItem;
         delete ClearItem;

         delete HelpItem;
      }
   }

   __fastcall MPairPopup::~MPairPopup()
   {
      Items->Remove(BlackItem);	BlackItem->Free();
      Items->Remove(WhiteItem);	WhiteItem->Free();
      Items->Remove(PairItem);	PairItem->Free();
      Items->Remove(RankItem);	RankItem->Free();
      Items->Remove(PrizeItem);	PrizeItem->Free();
      Items->Remove(SimuItem);	SimuItem->Free();
      Items->Remove(ClearItem);	ClearItem->Free();
      Items->Remove(HelpItem);	HelpItem->Free();
   }

   void __fastcall MPairPopup::PairPopupClick(TObject* Sender)
   {
   	using ToolsGUI::EditingCell;
     	using TrainMaster::MTrainTube;

      const bool CanSimClr  = !EditingCell() && MTrainTube().CanChangeResults();

      SimuItem->Enabled = CanSimClr;
		ClearItem->Enabled = CanSimClr;
   }

   void __fastcall MPairPopup::PairClick(TObject *Sender)
   {
		using TourMaster::GoTour;

      GGrid* Grid = (GGrid*)PopupComponent;

      GoTour.ClearTourInfo();
      	TourMUI::PairDetailsForm(this, Grid, PairPane, PopupPoint);
   }

   void __fastcall MPairPopup::BlackClick(TObject* Sender)
   {
		using TourMaster::GoTour;

      GGrid* Grid = (GGrid*)PopupComponent;

      GoTour.ClearTourInfo();

      TourMUI::BlackWhiteDetailsForm(this, Grid, PairPane, PopupPoint, -1);
   }

   void __fastcall MPairPopup::WhiteClick(TObject *Sender)
   {
		using TourMaster::GoTour;

      GGrid* Grid = (GGrid*)PopupComponent;

      GoTour.ClearTourInfo();

      TourMUI::BlackWhiteDetailsForm(this, Grid, PairPane, PopupPoint, 1);
   }

   void __fastcall MPairPopup::RankClick(TObject *Sender)
   {
		using TourMaster::GoTour;

      GGrid* Grid = (GGrid*)PopupComponent;

//     GK RegSort 2015 GoTour.ClearTourInfo();
//      	TourMUI::RankListForm(this, Grid);
//		 Grid->ShowInfoStatus(GoTour.TourInfo(), IDI_SVirgin + GoTour.State);

		::BeginCommand();
      	TourMUI::RankListForm(this, Grid);
      ::EndCommand(TourMaster::cmdNone);
   }

   void __fastcall MPairPopup::PrizeClick(TObject *Sender)
   {
		using TourMaster::GoTour;

      GGrid* Grid = (GGrid*)PopupComponent;

//     GK RegSort 2015 GoTour.ClearTourInfo();
//			TourMUI::PrizeListForm(this, Grid);
//		Grid->ShowInfoStatus(GoTour.TourInfo(), IDI_SVirgin + GoTour.State);

      ::BeginCommand();
      	TourMUI::PrizeListForm(this, Grid);
      ::EndCommand(TourMaster::cmdNone);

   }

   void __fastcall MPairPopup::SimuClick(TObject *Sender)
   {
      ::BeginCommand(TourMaster::cmdResult);
      	GGrid* Grid = (GGrid*)PopupComponent;
				TourMUI::SimulateResultsCmd(PairPane->ResGrid);
         ToCellNow(Grid);
   	::EndCommand(TourMaster::cmdResult);
   }

   void __fastcall MPairPopup::ClearClick(TObject *Sender)
   {
      ::BeginCommand(TourMaster::cmdResult);
      	GGrid* Grid = (GGrid*)PopupComponent;
   			ClearResultsCmd(PairPane->ResGrid);
         ToCellNow(Grid);
   	::EndCommand(TourMaster::cmdResult);
   }

   void __fastcall MPairPopup::HelpClick(TObject *Sender)
   {
      GGrid* Grid = (GGrid*)PopupComponent;
   		Application->HelpCommand(HELP_CONTEXT, IDH_PAIRPOP);
      ToCellNow(Grid);
   }
}
//---------------------------------------------------------------------------

namespace TourMUI														// PAIR/RESULTS GRID FORMS
{
	using Tours::tsEditNow;
   using Tours::tsForcedPair;
   using Tours::GBoard;

	using ToolsGUI::GGrid;

	using PaneGUI::GPairPane;

   static bool IdentifyPair
   (
   	GGrid* Grid,				// The component triggering the pop-up
      GPairPane* PairPane,		// We identify which grid
      const int row,				// The grid row right-clicked
      GBoard& BWpair				// The identified pair
   )
   {
   	BWpair = GBoard(0,0);

      if(Grid == (GGrid*)PairPane->PairGrid)
      {
         if(GoTour.State==tsEditNow || GoTour.State==tsForcedPair)
            if(1<=row && row < (int)PairPane->Pairs.size())
               BWpair = PairPane->Pairs[row];
            else
            	return false;
         else
            BWpair = GoTour.Pair[row];

      	return true;
      }

      if(Grid == (GGrid*)PairPane->ResGrid)
      	return PairPane->ResGrid->LocatePair(row, BWpair);

      return false;
   }

   void __fastcall BlackWhiteDetailsForm
   (
   	TComponent* Owner, GGrid* Grid, GPairPane* PairPane,
      TPoint PopPoint, int BlackWhite
   )
   {
      int Board,Gcol;

      // ESTABLISH THE ROW THAT WAS RIGHT-CLICKED
      TPoint GridPt = Grid->ScreenToClient(PopPoint);
      Grid->MouseToCell(GridPt.x, GridPt.y, Gcol, Board);
      if(Board<1)
      {
      	ToolsGUI::OperatorError(IDT_NoPlayer); ToCellNow(Grid);	return;
      }

      // IDENTIFY SELECTED PLAYER
      Tours::GBoard BWpair;

      if(!IdentifyPair(Grid, PairPane, Board, BWpair))
      {
      	ToolsGUI::OperatorError(IDT_NoBoardHere); ToCellNow(Grid);	return;
      }

      // LOCATE PLAYER IN CURRENT SORT ORDER
      size_t idPlayer, isPlayer=0;

      if(BlackWhite<0) idPlayer = BWpair.Black;
      if(BlackWhite>0) idPlayer = BWpair.White;

      for(size_t p=1; p<=GoTour.Players; p++)
      	if(GoTour.Sorted[p] == idPlayer)
         {
         	isPlayer = p;  break;
         }

      if(!isPlayer)
      {
      	ToolsGUI::OperatorError(IDT_Player);	 ToCellNow(Grid);	 return;
      }

      // SHOW THE EDITPLAYER FORM IN PASSIVE MODE
      ::ViewPlayer(BlackWhite,isPlayer);

      // RESET SELECTION
      ToCellNow(Grid);
   }
   //---------------------------------------------------------------------------

   void __fastcall PairDetailsForm
   (
   	TComponent* Owner, GGrid* Grid, GPairPane* PairPane, TPoint PopPoint
   )
   {
   	using Tours::GPlayer;
      using Tours::GBoard;

   	using RegGUI::rfStatus;
      using RegGUI::rfGrade;
      using RegGUI::rfRounds;

   	TPairFrm* PairForm = NULL;

      int Board,Col;

      // ESTABLISH THE BOARD REQUIRED
      TPoint GridPt = Grid->ScreenToClient(PopPoint);
      Grid->MouseToCell(GridPt.x, GridPt.y, Col, Board);
      if(Board<1)
      {
      	ToolsGUI::OperatorError(IDT_NoBoard);  ToCellNow(Grid); return;
      }

      // IDENTIFY THE PAIR ON THE BOARD
      GBoard BWpair;

      if(!IdentifyPair(Grid, PairPane, Board, BWpair))
      {
      	ToolsGUI::OperatorError(IDT_NoBoardHere); ToCellNow(Grid);	return;
      }

      GPlayer *Black = 0, *White = 0;

      Black = GoTour.Listed[BWpair.Black];		// NULL if unoccupied
      White = GoTour.Listed[BWpair.White];

      if(!Black && !White)								// Require at least one player
      {
      	ToolsGUI::OperatorError(IDT_NonEmptyBoard);
         ToCellNow(Grid);	return;
      }

      // SET UP TUBE AND SHOW THE DETAILS FORM
      try
      {
      	TourFormMaster::MPairTube PairTube(Board, BWpair);

      	PairForm = new TPairFrm(Owner, PairTube);
         	PairForm->ShowModal();
            GoTour.PostTourInfo(IDT_Ready);
         delete PairForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMUI, IDE_PlayerForm);

         delete PairForm;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_PlayerForm);

         delete PairForm;
      }

      ToCellNow(Grid);
   }
}
//------------------------------------------------------------------------------

namespace TourMUI													// GROUPS FORM
{
	using std::string;

	using Tools::s_S;
   using Tools::S_s;

	using Tours::ftGoDraw;
   using Tours::faOpen;
   using Tours::GTourStream;
   using Tours::GPlayerPtr;
   using Tours::GPlayer;

   using RegGUI::rfGroup;

   void __fastcall GroupsForm
   (
   	TComponent* Owner,
      RegGUI::GRegGrid* reggrid, const int row
   )
   {
  		TGroupFrm* GroupForm = NULL;

      try
      {
      	TourGroupsMaster::MGroupsTube GroupsTube(row);

      	GroupForm = new TGroupFrm(Owner, GroupsTube);
         	GroupForm->ShowModal();

         int GroupReport = IDK_Ready;

         if(GroupForm->ModalResult == mrOk)
         {
            TStrings* GroupItems = GroupForm->PlayerBox->Items;

         	if((int)GoTour.Players == GroupItems->Count)
          	{
            	GTourStream GroupStream(GoTour.FileName, ftGoDraw, faOpen);

            	bool Changed = false;

               for(size_t p=1; p<=GoTour.Players; p++)
               {
               	// THE REGISTER AND GroupItems IN EXACTLY SAME ORDER
                  string group = (S_s(GroupItems->Strings[p-1])).substr(0,2);

                  GPlayerPtr Pnow = GoTour.GetPlayerPtr(p);

                  // WE WRITE TO FILE ONLY IF GROUP HAS CHANGED
                  if(Pnow->Group != group)
                  {
                  	int RegReport = 0;

                  	Pnow->Group = group; 	Changed = true;

                     reggrid->Cells[rfGroup][p] = s_S(group);

                     RegMaster::player_details_changed(*Pnow,*Pnow,RegReport);

                     GroupStream.SavePlayer(GoTour.Sorted[p], GoTour, *Pnow);
                  }
               }

               GroupReport = Changed ? IDK_ReadyDone : IDK_ReadyNoChange;

               if(Changed)
               {
               	GoTour.PopulationCount();

         			::SetFormText();
               }
          	}
            else
         		ToolsGUI::OperatorError(IDK_BadPlayerCount);
         }

         if(GroupReport != IDK_BadPlayerCount)
         	GoTour.PostTourInfo(GroupReport);

         delete GroupForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMUI, IDK_GroupForm);

         delete GroupForm;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_TourMUI, IDK_GroupForm);

         delete GroupForm;
      }
   }
}
//------------------------------------------------------------------------------

namespace TourMUI													// RANKLIST FORM
{
   void __fastcall RankListForm(TComponent* Owner, GGrid* Grid)
   {
  		TRankFrm* RankForm = NULL;

      // SET UP TUBE AND SHOW THE RANKLIST FORM
      // GK 2015 The MRankTube invokesa RankSort which disturbes GoTour.Sorted.
      try
      {
      	TourRankMaster::MRankTube RankTube;

      	RankForm = new TRankFrm(Owner, RankTube);
         	RankForm->ShowModal();
         GoTour.PostTourInfo(IDT_Ready);

         delete RankForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMUI, IDT_RankForm);

         delete RankForm;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_RankForm);

         delete RankForm;
      }
      ToCellNow(Grid);
   }
}
//------------------------------------------------------------------------------

namespace TourMUI					// PRIZELIST FORM
{
   void __fastcall PrizeListForm(TComponent* Owner, GGrid* Grid)
   {
  		TPrizeFrm* PrizeForm = NULL;

      // SET UP TUBE AND SHOW THE PRIZELIST FORM
      // GK 2015 The MPrizeTube invokesa PrizeSort which disturbes GoTour.Sorted.
      try
      {
      	TourPrizeMaster::MPrizeTube PrizeTube;

      	PrizeForm = new TPrizeFrm(Owner, PrizeTube);
         	PrizeForm->ShowModal();
         GoTour.PostTourInfo(IDT_Ready);

         delete PrizeForm;
      }
      catch(Exception& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMUI, IDT_PrizeForm);

         delete PrizeForm;
      }
      catch(...)
      {
      	ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_PrizeForm);

         delete PrizeForm;
      }
      ToCellNow(Grid);
   }
}
//------------------------------------------------------------------------------

namespace TourMUI		// RESULTS SIMULATION
{
   void __fastcall SimulateResultsCmd(ResGUI::GResGrid* ResGrid)
   {
   	using Tours::tsDrawDone;
   	using Tours::tsTourDone;
   	using Tours::tsPastRound;

      TrainMaster::MTrainTube TrainTube;

      GoTour.ClearTourInfo();

      if(!TrainTube.SimulateResults())
      {
         ToolsGUI::BugReport(IDE_TourMUI, IDM_SimulaResCmd, IDM_NoSimulate);

         return;
      }

      ResGrid->LoadFromTour();

      ::SetFormText();
      ::ShowTourStatus(IDM_Ready);
   }

   void __fastcall ClearResultsCmd(ResGUI::GResGrid* ResGrid)
   {
   	using Tours::tsDrawDone;
   	using Tours::tsTourDone;
   	using Tours::tsPastRound;

      GoTour.ClearTourInfo();

      TrainMaster::MTrainTube TrainTube;

      if(!TrainTube.ClearResults())
      {
         ToolsGUI::BugReport(IDE_TourMUI, IDM_ClearResCmd, IDM_NoClear);

         return;
      }

      ResGrid->LoadFromTour();

      ::SetFormText();
      ::ShowTourStatus(IDM_Ready);
   }
}
//------------------------------------------------------------------------------

namespace TourMUI									// UNREGISTERED
{
   bool __fastcall CloseRegAct(TComponent* Owner, int& Report)
   {
		using TourMaster::GoTour;

      TCloseRegFrm* CloseRegForm = NULL;

      Report = IDT_NoCloseReg;

      try
      {
      	TourCloseRegMaster::MCloseRegTube CloseRegTube;

         CloseRegForm = new TCloseRegFrm(Owner, CloseRegTube);

         	CloseRegForm->ShowModal();

            if(CloseRegForm->ModalResult == mrOk)
            	Report = IDT_Delete + CloseRegTube.Action;
            else
            	Report = IDT_Resume;

         delete CloseRegForm;

         return Report < IDT_Resume;
      }
      catch(Exception& Err)
      {
         ToolsGUI::Serve(Err, IDE_TourMUI, IDT_CloseRegForm);

         delete CloseRegForm;
      }
      catch(...)
      {
         ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_CloseRegForm);

         delete CloseRegForm;
      }
      return false;
   }
}
//------------------------------------------------------------------------------

namespace TourMUI									// JOINING DRAW
{
   bool __fastcall JoinDrawAct(TComponent* Owner)
   {
		using TourMaster::GoTour;

      TJoinDrawFrm* JoinDrawForm = NULL;

      bool Confirmed = true;

      try
      {
      	TourJoinMaster::MJoinDrawTube JoinDrawTube;

         JoinDrawForm = new TJoinDrawFrm(Owner, JoinDrawTube);

         	JoinDrawForm->ShowModal();

            Confirmed = JoinDrawForm->ModalResult == mrYes;

         delete JoinDrawForm;
      }
      catch(Exception& Err)
      {
         ToolsGUI::Serve(Err, IDE_TourMUI, IDT_JoinDrawForm);

         Confirmed = false;

         delete JoinDrawForm;
      }
      catch(...)
      {
         ToolsGUI::ShowWin32Err(IDE_TourMUI, IDT_JoinDrawForm);

         Confirmed = false;

         delete JoinDrawForm;
      }
      return Confirmed;
   }
}
//------------------------------------------------------------------------------
