////////////////////////////////////////////////////////////////////////////////
// MODULE TourMaster:       All interface to GTour        G.Kaniuk 03/10/02
//

#include <vcl.h>
#pragma hdrstop

#include "GoErrMsgRES.h"
#include "GoInfoRES.h"

#include "Format.h"

#include "TourMaster.h"
#include "RegMaster.h"
#include "DrawMaster.h"
#include "PairMaster.h"
#include "TourStream.h"


namespace TourMaster                                        // GLOBALS
{
   static void Throw(ToolsGUI::EType etype, short idrout, short idmsg)
   {
      ToolsGUI::ThrowError(etype, IDE_TourMaster, idrout, idmsg);
   }

   char* ResultCodes = "0J1UDFV_";                          // CODES for Result parsing

	const char* RegCode[4] = { "", "R", "J", "L" };     		// CODE for Registered, Joining, Leaving

   // AN ENABLED COMMAND MAY CHANGE STATE; A DISABLED ONE CANNOT
   const bool EnabledCommands[cmdCount][Tours::tsCount] =
   {
   						/* Virgin Start Reg Rclosed Ddone  Res Rdone Tdone Eforced PartD Enow Fpair PastRound */
   	/* None */		{ 	0,		 0,    0,  0,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },
   	/* New */      { 	1,		 1,    1,  1,   	 1,     1,  1,    1,    0,   	   0,   0,   0,    0 },
   	/* Open */     { 	1,		 1,    1,  1,   	 1,     1,  1,    1,    1,   	   0,   0,   0,    0 },
   	/* Enter */    { 	0,		 1,    1,  1,   	 1,     1,  1,    1,    1,   	   0,   0,   0,    1 },
   	/* Del */      { 	0,		 1,    1,  0,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },		// Del suppressed locally
   	/* Close */    { 	0,		 0,    1,  0,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },
   	/* ReOpen */   { 	0,		 0,    0,  1,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },
   	/* Restore */  { 	0,		 0,    0,  0,   	 0,     0,  0,    0,    1,   	   0,   0,   0,    0 },
   	/* Round */    { 	0,		 0,    0,  0,   	 1,     1,  1,    1,    0,   	   0,   0,   0,    1 },
   	/* Draw */     { 	0,		 0,    0,  1,   	 0,     0,  1,    0,    0,   	   0,   0,   1,    0 },
   	/* Forced */   { 	0,		 0,    0,  1,   	 0,     0,  1,    0,    0,   	   0,   0,   0,    0 },
   	/* Edit */     { 	0,		 0,    0,  0,   	 1,     1,  1,    1,    1,   	   1,   0,   0,    1 },
   	/* Save */     { 	0,		 0,    0,  0,   	 0,     0,  0,    0,    0,   	   0,   1,   0,    0 },
   	/* Dcancel */  { 	0,		 0,    0,  0,   	 1,     0,  0,    0,    0,   	   1,   0,   0,    0 },
   	/* Ecancel */  { 	0,		 0,    0,  0,   	 0,     0,  0,    0,    0,   	   0,   1,   1,    0 },
   	/* Result */   { 	0,		 0,    0,  0,   	 1,     1,  1,    1,    0,   	   0,   0,   0,    1 },
      /* CloseTour */{  1,     1,    1,  1,      1,     1,  1,    1,    1,   	   0,   1,   1,    1 },

      /* Find */     {  0,     0,    1,  1,      1,     1,  1,    1,    1,   	   1,   1,   1,    1 },
      /* RegAll */ 	{  0,		 0,    1,  0,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },
      /* UnRegAll */ {  0,		 0,    1,  0,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },
   	/* FastReg */  { 	0,		 0,    1,  0,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },

      /* MacBar */   { 	0,		 1,    1,  1,   	 0,     0,  0,    0,    0,   	   0,   0,   0,    0 },
      /* KomiBar */  { 	0,		 1,    1,  1,   	 0,     0,  1,    0,    0,   	   0,   0,   0,    0 },
      /* NoviBar */  { 	0,		 1,    1,  1,   	 0,     0,  1,    0,    0,   	   0,   0,   0,    0 },

      /* Import */   { 	0,		 1,    1,  0,   	 1,     1,  1,    0,    0,   	   0,   0,   0,    0 }
	};

   GTourMaster GoTour;                                      // THE ONLY GTour
}

namespace TourMaster                                        // CONSTRUCTION & LOADING
{
   using Tours::GTour;

   GTourMaster::GTourMaster(string FileName):GTour(FileName)
   {
   	RestoreRegStatus = false;

      OldRegOrder = SortOrder = 0;									// GK REGSORT 2015
      OldRegSense = SortSense = 0;

      Pair.resize(0);

      ClubPops.clear();	LandPops.clear();
   }

   GTourMaster::GTourMaster():GTour()
   {
   	RestoreRegStatus = false;

      OldRegOrder = SortOrder = 0;									// GK REGSORT 2015
      OldRegSense = SortSense = 0;

      Pair.resize(0);

      ClubPops.clear();	LandPops.clear();
   }


   void GTourMaster::Load(string FileName)
   {
      Tours::Load(FileName, GoTour);

      // NEW PLAYER NUMBERING
      NewPid = 0;

      for(size_t p=1; p<=Players; p++)
      	if(Listed[p]->Pid > (size_t)NewPid)
         	NewPid = Listed[p]->Pid;

      ++NewPid;

      // RECALCULATE TIE BREAKS AS THEY ARE NOT STORED
      SetTieBreaks();

      // THE PAIRING - SET Pairs size
      DrawMaster::ListBoards();

      // THE REGISTER
      RegMaster::RegSort(Tours::skEntry, 1);							// ToolsGUI::gsUp=1

      PopulationCount();
   }

   void GTourMaster::Save(string filename)
   {
      FileName = filename;

      Tours::Save(GoTour);

		// TODO: Backup      
   }

   bool GTourMaster::SaveTournament(void)
   {
   	try
      {
      	Tours::Save(GoTour);

         return true;
      }
      catch(ToolsGUI::ETour& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMaster, IDE_SaveTournament);

         return false;
      }
      catch(...)
      {
      	ToolsGUI::ErrorReport(IDE_TourMaster, IDE_SaveTournament, IDE_NoMemory);

         return false;
      }
   }

   bool GTourMaster::SaveHeader(void)
   {
   	try
      {
   		Tours::GTourStream Stream(GoTour.FileName, Tours::ftGoDraw, Tours::faOpen);

      	Stream.SaveHeader(GoTour);

         return true;
      }
      catch(ToolsGUI::ETour& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMaster, IDE_SaveHeader);

         return false;
      }
      catch(...)
      {
      	ToolsGUI::ErrorReport(IDE_TourMaster, IDE_SaveHeader, IDE_NoMemory);

         return false;
      }
   }

   bool GTourMaster::SavePlayer(int p)
   {
   	try
      {
   		Tours::GTourStream Stream(FileName, Tours::ftGoDraw, Tours::faOpen);

         GTour& Tour = *((GTour*)this);

      	Stream.SavePlayer(p, Tour);

         return true;
      }
      catch(ToolsGUI::ETour& Err)
      {
      	ToolsGUI::Serve(Err, IDE_TourMaster, IDE_SavePlayer);

         return false;
      }
      catch(...)
      {
      	ToolsGUI::ErrorReport(IDE_TourMaster, IDE_SavePlayer, IDE_NoMemory);

         return false;
      }
   }
   //---------------------------------------------------------------------------

   void GTourMaster::NewStart(TourState tState)
   {
   	using Tools::cSpace;
      using PairMaster::SetDefaultParameters;
      
      Pair.resize(0);										// Clear all GTourMaster containers

      ClubPops.clear();	LandPops.clear();

      Clear();													// Clear all single fields,and fixed arrays in GTour

		State 			 = tState;							// tsVirgin or tsTourStart

   	Rules.Rounds 	 = Tours::def_rounds;			// Standard 3 rounder

   	ReBuild(0);												// Clear all GTour Containers to zero players

      Year            = Tours::base_year;
      TourName			 = string(SizeName, cSpace);

      RestoreRegStatus = false;

      Rules.Checked   = false;							// Warned at close register

      Rules.TieOne 	 = Tours::tbSoS;
      Rules.TieTwo 	 = Tours::tbNone;
      Rules.Style 	 = Tours::stMcMahon;
      Rules.RuleSet   = Tours::rtBGA;
      Rules.BarSeg 	 = false;
      Rules.MacBar    = Tours::def_macbar;
      Rules.SupBar    = Tours::sup_bar + Tours::mc_scale*(Rules.Rounds + 1);
      Rules.MinGrade  = Tours::def_mingrade;
      Rules.KomiGroup = Tours::def_komigrp;
      Rules.KomiDiff  = Tours::def_komidiff;
      Rules.KomiRate  = Tours::def_komirate;
      Rules.Prizes    = Tours::def_rounds*Tours::mc_scale;

      SetDefaultParameters(PairParam);

      NewPid          = 1;

      PopulationCount();								// Clear all counters
   }

   void GTourMaster::CleanSeed(void)			// The seed tournament already loaded
   {
   	for(size_t p=1; p<=Players; p++)
      {
      	GPlayerPtr Player = Listed[p];		// Only Name, grade, club country are preserved

         Player->Group = string(2, Tools::cSpace);
         Player->Note  = string();
         Player->Pid   = p;

         RegMaster::initial_score(*Player);

         Player->McScore = Player->MiScore;

         Player->Cbal = Player->Mbal = Player->Mout = 0;

         Player->Rank = 0;  Player->TieOne = Player->TieTwo = 0;

         Player->Spare[0] = Tours::gpAlways;    		// size_spare is 4
         Player->Spare[1] = Tours::gpPrefer;
         Player->Spare[2] = Tours::gpAvoid;
         Player->Spare[3] = Tours::gpNever;

         // PLAYER INITIAL STATE: (psEntered, grNone)
         Player->State = Tours::psEntered;

         for(size_t r=0; r<Player->Games.size(); r++)
         	Player->Games[r] = Tours::GGame();
      }

      Pair.resize(0);

      Round     = 0;
      Roundl    = 0;

      SortOrder = Tours::skEntry;
      SortSense = 1;

      SpareOne  = 0;
      SpareTwo  = 0;

      NewPid    = Players+1;

      Rules.Checked   = false;	// Warned at close register

      PairMaster::SetDefaultParameters(GoTour.PairParam);

      // INITIAL TOURNAMENT STATE
      RestoreRegStatus = false;

      State = Tours::tsRegNow;

      PopulationCount();
   }

   bool GTourMaster::EnabledCommand(int cmd)
   {
   	if(cmd < 0 || cmd >= cmdCount)
      	return false;

      if(State<0 || State >= Tours::tsCount)
      	return false;

   	return EnabledCommands[cmd][State];
   }
}

namespace TourMaster                                        // POPULATION COUNTS
{
   using Tours::tsVirgin;
   using Tours::tsTourStart;
   using Tours::GPlayer;
   using Tours::grSleep;
   using Tours::mc_scale;

   // There are 4 sub-sets of players; these are not disjoint

   // SuperGroup:    g >= SupBar
   // TopGroup:		g >= MacBar
   // KomiGroup:     g <= KomiGrade
   // NoviceGroup    g <= MinGrade

   bool GTourMaster::IncPops(GPlayer& Player)					// ADD PLAYER
   {
   	bool ChangedAreaCounts = false;

      // REGISTERED COUNT
      if(Player.State >= Tours::psRegistered) ++RegCount;

      // CLUB COUNTS
      GCodeCount &club = GCodeCount(Player.Club,1);
      list<GCodeCount>::iterator iClub = find(ClubPops.begin(), ClubPops.end(),club);

      if(iClub == ClubPops.end())
         ClubPops.push_back(club);
      else
         ++iClub->Count;

      if(ClubCount != (int)ClubPops.size())
      {
      	ClubCount = ClubPops.size();
         ChangedAreaCounts = true;
      }

      // COUNTRY COUNTS
      GCodeCount &land = GCodeCount(Player.Land,1);
      list<GCodeCount>::iterator iLand = find(LandPops.begin(), LandPops.end(),land);

      if(iLand == LandPops.end())
         LandPops.push_back(land);
      else
         ++iLand->Count;

      if(LandCount != (int)LandPops.size())
      {
      	LandCount = LandPops.size();
         ChangedAreaCounts = true;
      }

      // PLAYING THIS, NEXT ROUND
      bool PlayNow, PlayNext;

      GoTour.PlayNowNext(Player, PlayNow, PlayNext);

      if(PlayNow)  				 ++PlayNowCount;
      if(PlayNext) 				 ++PlayNextCount;
      if(!PlayNow && PlayNext) ++JoiningCount;

      if(State >= Tours::tsDrawDone)
      {
      	// JOINING/LEAVING/MISSING RESULT THIS ROUND
      	if(Player.Joining(Round))      ++ForcedEdCount;
      	if(Player.Leaving(Round))      ++ForcedEdCount;
      	if(Player.MissingResult(Round))++MissingCount;
      }

      // BAR POPULATIONS
      while(true)
      {
         if(Player.MiScore >= Rules.SupBar)    ++SupPop;
         if(Player.MiScore >= Rules.MacBar)    ++MacPop;

         if(Player.MiScore <= Rules.KomiGroup) ++KomiPop;
         if(Player.MiScore <= Rules.MinGrade)  ++NovicePop;

         break;
      }

     return ChangedAreaCounts;
   }


   bool GTourMaster::DecPops(GPlayer& Player)					// DELETE PLAYER
   {
   	bool ChangedAreaCounts = false;

      // REGISTERED COUNT
      if(Player.State >= Tours::psRegistered) --RegCount;


      // CLUB AND COUNTRY COUNTS
      GCodeCount &club = GCodeCount(Player.Club,1);
      list<GCodeCount>::iterator iClub = find(ClubPops.begin(), ClubPops.end(),club);

      if(iClub != ClubPops.end())--iClub->Count;

      if(iClub->Count == 0)								// No players left in this club
      {
      	ClubPops.erase(iClub);

         ChangedAreaCounts = true;
      }
      ClubCount = ClubPops.size();

      GCodeCount &land = GCodeCount(Player.Land,1);
      list<GCodeCount>::iterator iLand = find(LandPops.begin(), LandPops.end(),land);

      if(iLand != LandPops.end())--iLand->Count;

      if(iLand->Count == 0)								// No players left in this country
      {
      	LandPops.erase(iLand);

         ChangedAreaCounts = true;
      }
      LandCount = LandPops.size();


      // PLAYING NEXT ROUND
      // This routine is called only if player has not played any games

      bool PlayNow, PlayNext;

      GoTour.PlayNowNext(Player, PlayNow, PlayNext);

      if(PlayNow)  				 --PlayNowCount;		// Player would have played this, but is deleted
      if(PlayNext) 				 --PlayNextCount;		// Player would have played next, but is deleted
      if(!PlayNow && PlayNext) --JoiningCount;     // Player would have joined next, but is deleted

      // JOINING/LEAVING
      if(Player.Joining(Round)) --ForcedEdCount;
      if(Player.Leaving(Round)) --ForcedEdCount;

      // WAITING RESULTS
		// Cannot happen as this routine is called only if player not in draw

      // BAR POPULATIONS
      while(true)
      {
         if(Player.MiScore >= Rules.SupBar)    --SupPop;
         if(Player.MiScore >= Rules.MacBar)    --MacPop;
         if(Player.MiScore <= Rules.MinGrade)  --NovicePop;
         if(Player.MiScore <= Rules.KomiGroup) --KomiPop;   

         break;
      }
      return ChangedAreaCounts;
   }

   void GTourMaster::ChangeRegPops(GPlayer& Pa, GPlayer& Pb)	// OBSOLETE EDIT PLAYER STATE
   {
   	using Tours::psRegistered;
      
   	// REGISTERED COUNT - PLAYER COUNT IS INVARIANT
      if(Pa.State < psRegistered && Pb.State >= psRegistered ) ++RegCount;
      if(Pb.State < psRegistered && Pa.State >= psRegistered ) --RegCount;
   }

   bool GTourMaster::ChangePops(GPlayer& Pa, GPlayer& Pb)		// EDIT PLAYER
   {
   	using Tours::psRegistered;
      using Tours::psPlaying;
      using Tours::psSleep;
      using Tours::grSleep;

      bool ChangedAreaCounts = false;

      // REGISTERED COUNT - PLAYER COUNT IS INVARIANT
      if(Pa.State < psRegistered && Pb.State >= psRegistered ) ++RegCount;
      if(Pb.State < psRegistered && Pa.State >= psRegistered ) --RegCount;


      // CLUB COUNTS - THE CLUB FOR Pa EXISTS, NOT SURE ABOUT Pb
      if(Pa.Club != Pb.Club)
      {
      	GCodeCount &aclub = GCodeCount(Pa.Club,0), &bclub = GCodeCount(Pb.Club,0);

      	list<GCodeCount>::iterator aClub = find(ClubPops.begin(), ClubPops.end(),aclub);
      	list<GCodeCount>::iterator bClub = find(ClubPops.begin(), ClubPops.end(),bclub);

         if(--aClub->Count == 0) 			 // Player leaves aclub
         {
         	ClubPops.erase(aClub);			// Club folds

            ChangedAreaCounts = true;
         }

      	if(bClub == ClubPops.end())		// Might be a new club
         {
         	ClubPops.push_back(bclub);

         	++bClub->Count;					// Player joins bclub

            ChangedAreaCounts = true;
         }
      	ClubCount = ClubPops.size();
      }

      // LAND COUNTS - THE LAND FOR Pa EXISTS, NOT SURE ABOUT Pb
      if(Pa.Land != Pb.Land)
      {
      	GCodeCount &aland = GCodeCount(Pa.Land,0), &bland = GCodeCount(Pb.Land,0);

      	list<GCodeCount>::iterator aLand = find(LandPops.begin(), LandPops.end(),aland);
      	list<GCodeCount>::iterator bLand = find(LandPops.begin(), LandPops.end(),bland);

         if(--aLand->Count == 0) 			 // Player leaves aland
         {
         	LandPops.erase(aLand);			// Land folds

            ChangedAreaCounts = true;
         }

      	if(bLand == LandPops.end())		// Might be a new land
         {
         	LandPops.push_back(bland);

         	++bLand->Count;					// Player joins bland

            ChangedAreaCounts = true;
         }
      	LandCount = LandPops.size();
		}

      // PLAYING THIS, NEXT ROUND
      bool aNow, aNext;		PlayNowNext(Pa, aNow, aNext);
      bool bNow, bNext;    PlayNowNext(Pb, bNow, bNext);

      if(!aNow &&  bNow) ++PlayNowCount;
      if( aNow && !bNow) --PlayNowCount;

      if(!aNext &&  bNext) ++PlayNextCount;
      if( aNext && !bNext) --PlayNextCount;

      const bool aJoining = !aNow && aNext, bJoining = !bNow && bNext;

      if(aJoining && !bJoining) --JoiningCount;
      if(!aJoining && bJoining) ++JoiningCount;

      // FORCED EDIT COUNT
      if(!Pa.Awake(Round) && Pb.Joining(Round)) ++ForcedEdCount;	// User edit to add player to draw
      if( Pa.Awake(Round) && Pb.Leaving(Round)) ++ForcedEdCount;  // User edit to remove player from draw

      if(Pa.Joining(Round) && !Pb.Awake(Round)) --ForcedEdCount;	// User edits to undo Joining
      if(Pa.Leaving(Round) &&  Pb.Awake(Round)) --ForcedEdCount;	// User edits to undo Leaving


      // WAITING RESULTS - NO CHANGE IN ACTUAL PRIZES
      if(State > Tours::tsDrawDone)
      {
         if(Pa.State == psSleep   && Pb.State == psPlaying)++MissingCount;
         if(Pa.State == psPlaying && Pb.State == psSleep)--MissingCount;
      }

      // BAR POPULATIONS- IGNORE HERE THAT UPPER BARS ARE DYNAMIC
      while(true)
      {
      	if(Pa.MiScore<Rules.SupBar && Pb.MiScore>=Rules.SupBar) ++SupPop;
       	if(Pa.MiScore>=Rules.SupBar && Pb.MiScore<Rules.SupBar) --SupPop;

         if(Pa.MiScore<Rules.MacBar && Pb.MiScore>=Rules.MacBar) ++MacPop;
       	if(Pa.MiScore>=Rules.MacBar && Pb.MiScore<Rules.MacBar) --MacPop;

       	if(Pa.MiScore>Rules.KomiGroup && Pb.MiScore<=Rules.KomiGroup) ++KomiPop;
         if(Pa.MiScore<=Rules.KomiGroup && Pb.MiScore>Rules.KomiGroup) --KomiPop;

       	if(Pa.MiScore>Rules.MinGrade && Pb.MiScore<=Rules.MinGrade) ++NovicePop;
         if(Pa.MiScore<=Rules.MinGrade && Pb.MiScore>Rules.MinGrade) --NovicePop;

         break;
      }

      return ChangedAreaCounts;
   }

   bool GTourMaster::UpdatePopulationCount(GPlayer& a, GPlayer& b)
   {
   	using Tours::psNew;

      if(a.State == psNew && b.State != psNew)   	// ADD A NEW PLAYER
      	return IncPops(b);

      if(a.State != psNew && b.State == psNew)   	// DELETE A PLAYER
      	return DecPops(a);

      if(a.State != psNew && b.State != psNew)   	// EDIT A PLAYER
      	return ChangePops(a,b);

      return false;											// NEVER HAPPENS
   }

   void GTourMaster::PopulationCount(void)
   {
   	using Tours::GPlayer;
      using Tours::grSleep;
      using Tours::mc_scale;

   	int PlayerCount = 0, PossPrizeCount = 0;

      RegCount = 0;

      ClubCount    = 0;		LandCount     = 0;

      PlayNowCount = 0;   	PlayNextCount = 0;	JoiningCount = 0;
      MissingCount = 0;		PrizeCount    = 0;

      ForcedEdCount = 0;

      SupPop = MacPop = KomiPop = NovicePop = 0;

      ClubPops.clear();	LandPops.clear();

      if(State<=Tours::tsTourStart) return;

      list<GPlayer>::iterator iplayer = PlayerList.begin();
      list<GPlayer>::iterator eplayer = PlayerList.end();

      while(iplayer != eplayer)
      {
      	GPlayer& Player = *iplayer;

      	IncPops(Player);		++PlayerCount;		// ALL COUNTS EXCEPT PRIZES

         if(State >= Tours::tsDrawDone)			// PRIZES
         {
            const int ThisWins = Player.Wins();

            if(ThisWins >= Rules.Prizes)
            	++PrizeCount;

            if(State != Tours::tsTourDone)
            	if(ThisWins >= (Rules.Prizes-Tours::grWin))
               	++PossPrizeCount;
         }
         ++iplayer;
       }
       PrizeCount = (2*PrizeCount + PossPrizeCount)/2;
   }

   void GTourMaster::SetState(void)					// STATE CHANGE AFTER REG/DRAW/RESULT EDIT
   {
   	if(!Round) return;
      
      if(Round<Roundl)
      	{ State = Tours::tsPastRound;	 return; }

		if(MissingCount == 0)
      	State = Round == Rules.Rounds ? Tours::tsTourDone : Tours::tsRoundDone;

     	if(MissingCount > 0)
      	State = Tours::tsResultNow;

      if(MissingCount == PlayNowCount)
      	State = Tours::tsDrawDone;
   }
}
//------------------------------------------------------------------------------

namespace TourMaster                                        // ACCESS
{
   using std::string;
   using std::vector;

   GPlayerPtr GTourMaster::GetPlayerPtr(int p)              // Access sorted list
   {
      if(p==0 || p>(int)Players+1)
         Throw(ToolsGUI::erBug, IDE_GetPlayer, IDE_Range);

      return Listed[Sorted[p]];                             // Players+1 is a dummy
   }

   string GTourMaster::Number(GPlayerPtr P)						// Leading zero form for PlayerId
   {
   	const string numFmt = "0000";

      const int num = P ? P->Pid : 0;

      return Tools::to_string(numFmt, num);
   }

   string GTourMaster::Name(GPlayerPtr P)                   // Given Middle Family
   {
   	string given,middle,family, fullname;

      string wspace = string(Tools::sSpace);

      if(!P) return string();

      if(!Tools::parse_extended_family(P->Name,given,middle,family))
      	return string();

      if(!given.empty())
      	fullname = given;
      else
      	return family;

      if(!middle.empty())
      	fullname += wspace + middle;

      return fullname + wspace + family;
   }

   string GTourMaster::ShortName(GPlayerPtr P)					// I Family(MMS)
   {
      if(!P)																// Missing Player
      	return string();

      string ShortName;

      if(Tools::parse_shortname(P->Name, ShortName))
         return ShortName;
      else
         return P->Name;
   }

   string GTourMaster::ShortNameGrade(GPlayerPtr P)         // I Family(Grade)
   {
      using Tools::sBra;
      using Tools::sKet;
      using Tools::spaces;

      if(!P) return string();

      string Grade, ShortName;

      Grade = string(sBra) + Tours::grade_ustring(P->Grade) + sKet;

      if(Tools::parse_shortname(P->Name, ShortName))
         return ShortName + spaces(1) + Grade;
      else
         return P->Name + spaces(1) + Grade;
   }

   string GTourMaster::ShortNameMMS(GPlayerPtr P)           // I Family(MMS)
   {
      using Tools::sBra;
      using Tools::sKet;
      using Tools::spaces;
      
      string MacScore, ShortName;

      if(!P)return string();									// Missing Player

      if(P->State < Tours::psNew)
      	return string(3,'N');

     if(P->State > Tours::psFinished)
      	return string(4,'F');

      int RawScore = P->McMahonScore(GoTour.Round-1);	// Score at BEGINNING of round

      MacScore = spaces(1) + string(sBra) + Tours::mms_ustring(RawScore) + string(sKet);

      if(Tools::parse_shortname(P->Name, ShortName))
         return ShortName + MacScore;
      else
         return P->Name + MacScore;
   }

   string GTourMaster::ShortLandClub(GPlayerPtr P)							// CC - ClubName
   {
   	using std::string;

      if(!P) return string();

   	string Space = string(Tools::sSpace);
      string Hyphen = string(Tools::sMinus);

   	string LandClub = P->Land + Space + Hyphen + Space;

      return LandClub + Club(P);
   }

   string GTourMaster::Registered(GPlayerPtr P)
   {
      using Tours::psRegistered;

      if(!P) return string();

      if(P->State<psRegistered) return string();

      if(P->Joining(Round)) return string(RegCode[2]);
      if(P->Leaving(Round)) return string(RegCode[3]);

      return string(RegCode[1]);
   }

   string GTourMaster::Family(GPlayerPtr P)                 // Family,Given Middle
   {
   	if(!P) return string();

      return RegMaster::name_family(P->Name);
   }

   string GTourMaster::Grade(GPlayerPtr P)                  // Converted grade
   {
   	if(!P) return string();

      return Tours::grade_ustring(P->Grade);
   }

   string GTourMaster::Club(GPlayerPtr P)                   // Club name from code
   {
   	if(!P) return string();

      string cname;  Clubs.decode(P->Club, cname);

      return cname;
   }

   string GTourMaster::Land(GPlayerPtr P)                   // Country name from code
   {
   	if(!P) return string();

      string lname;  Lands.decode(P->Land, lname);

      return lname;
   }


   string GTourMaster::Rounds(GPlayerPtr P)                 // Rounds played
   {
   	if(!P) return string();

      string splay;

      vector<bool>play(Rules.Rounds);

      for(size_t r=0; r<Rules.Rounds; r++)
         play[r] = P->Games[r].Result != Tours::grSleep;

      RegMaster::rounds_string(play, splay);

      return splay;
   }

   string GTourMaster::Group(GPlayerPtr P)                   // Group code
   {
   	if(!P) return string();

      string group = P->Group;

      return Tools::trim(group);
   }

   string GTourMaster::Note(GPlayerPtr P)                   // Ascii Note
   {
   	if(!P) return string();

      string note= P->Note;

      return Tools::trim(note);
   }


   string GTourMaster::GameNote(Tours::GBoard Board)
   {
      vector<bool>Note(DrawMaster::coCount);

      DrawMaster::GameComment(Board, Note);

      string Comment = string();

      for(int k=0; k<DrawMaster::coCount; k++)
         if(Note[k])
            if(Comment.empty())
               Comment = Tools::load_str(IDI_COMMENT+k);
            else
               Comment += string(Tools::sPlus) +  Tools::load_str(IDI_COMMENT+k);

      return Comment;
   }

   string GTourMaster::Komi(Tours::GBoard Board)
   {
      using Tools::to_string;
      using Tools::sEmpty;
      using Tools::sPlus;

      if(Round<1 || !Board.Black || !Board.White ) return sEmpty;

      GGame Game = Listed[Board.Black]->Games[Round-1];

      const int Stone = Game.Stones/Tours::mc_scale;
      const int Komi  = Game.Komi/Tours::mc_scale;

      if(!Stone && !Komi) return sEmpty;

      const string Stones = to_string(Stone);						// GK 6.4.0 corrected H++K
      const string pKomi  = Komi>0 ? string(sPlus) : string();
      const string Komis  = pKomi + to_string(Komi);

      return (Komi==0) ? Stones : Stones+Komis;
   }

   string GTourMaster::Result(GPlayerPtr P)                   // One char current round
   {
      char Result;

      if(!P)
      	return string();

      switch(P->Games[Round-1].Result)
      {
         case Tours::grLose:  Result = Tools::cLose;   break;
         case Tours::grJigo:  Result = Tools::cJigo;   break;
         case Tours::grWin:   Result = Tools::cWin;    break;
         case Tours::grDlos:  Result = Tools::cDlos;   break;
         case Tours::grFwin:  Result = Tools::cFwin;   break;
         case Tours::grVoid:  Result = Tools::cVoid;   break;
         case Tours::grUsolv: Result = Tools::cUsolv;  break;
         case Tours::grNone:  Result = Tools::cUline;  break;

         default: Result = Tools::cQuery;
      }
      return string(1, Result);
   }

   string GTourMaster::TourNameYear()
   {
   	return Tools::trim(TourName) + "  " + Tools::to_string(Year);
   }
}
//------------------------------------------------------------------------------

namespace TourMaster											// PLAYER STATE SUPPORT
{
   bool GTourMaster::ActivePlayer(int p)				// Active at RegClosed or
   {                                               // Has a result or in draw
   	if(p<1 || p>(int)Players) return false;		// for this round

      if(State<Tours::tsRegClosed) return false;	// All players inactive before round 1

      if(State==Tours::tsRegClosed) return true;	// Waiting to be drawn

   	GPlayerPtr Player = Listed[p];

      for(size_t r=1; r<=Roundl; r++)					// Check all rounds to max played
      	if(Player->Awake(r) )
         	return true;

      return false;											// Sleeping for
   }

   bool GTourMaster::PlayingThisRound(int p)			// Not sleeping at current round
   {
   	if(p<1 || p>(int)Players) return false;

      return Listed[p]->Games[Round-1].Result != Tours::grSleep;
   }
}

////////////////////////////////////////////////////////////////////////////////
// STAND ALONE STRING TOOLS
//

namespace TourMaster
{
   using std::string;
   using std::vector;

   using Tools::ints;
   using Tools::sDot;
   using Tools::sSpace;
   using Tools::sComma;


   bool parse_komi(string skomi, int& stones, int& points)
   {
      using Tools::next_posint;

      stones = points = 0;

      if(skomi.empty()) return true;                  // EVEN

      string::iterator kf = skomi.begin();
      string::iterator ke = skomi.end();

      if(!next_posint(kf, ke, stones))return false;

      if(kf==ke) return true;                         // HCAP

      switch(*kf)
      {
         case '+':                                    // HCAP + POINTS
         {
            ++kf;

            if(!next_posint(kf, ke, points))return false;

            if(kf != ke) return false;
         }
         break;

         case '-':                                    // HCAP - POINTS
         {
            ++kf;

            if(!next_posint(kf, ke, points))return false;

            if(kf != ke) return false;

            points = -points;
         }
         break;

         default: return false;
      }
      return true;
   }

   bool is_result(char key)
   {
      char* reskey = ResultCodes;

      while(*reskey)
      {
         if(*reskey == key) break;

         ++reskey;
      }

      return *reskey != 0;
   }

   bool parse_year(string& yrtext)
   {
   	yrtext = Tools::trim(yrtext); if(yrtext.empty()) return false;
      
   	if(yrtext.size()>4) return false;

   	for(size_t c=0; c<yrtext.size(); c++)
      	if(Tools::lexical_class(yrtext[c]) != Tools::lxDigit)
         	return false;


      if(yrtext.size() == 4) return true;

      string::iterator yrs = yrtext.begin(), yre = yrtext.end();

      int year = Tools::to_posint(yrs, yre);

      if(year < 1000) year += 2000;

      yrtext = Tools::to_string(string("0000"), year);

      return true;
   }

   bool parse_tourname(string& ntext)
   {
   	using Tools::LexClass;
      using Tools::lxUpper;
      using Tools::lxLower;
      using Tools::uppercase;
      using Tools::lowercase;
      using Tools::cSpace;

   	string cntext = Tools::compactify(ntext);		if(cntext.empty()) return false;

      for(size_t c=0; c<cntext.size(); c++)
      {
      	Byte lexic =  Tools::lexical_class(cntext[c]);

      	if(cntext[c] != Tools::cSpace && lexic != lxUpper && lexic != lxLower)
         	return false;
      }

      ntext = lowercase(cntext);

      for(size_t c=0; c<cntext.size(); c++)
      	if(c==0)
         	ntext[0] = uppercase(ntext[0]);
         else
         	if(ntext[c-1] == cSpace)
            	ntext[c] = uppercase(ntext[c]);

      ntext.resize(GTour::SizeName, cSpace);

      return true;
   }
}
//------------------------------------------------------------------------------




