////////////////////////////////////////////////////////////////////////////////
// MODULE  UpsetGUI:    Draw Editing Support for the UpsetList                   G.Kaniuk 23/02/04
//
#include <string>
#include <vcl.h>
#pragma hdrstop

#include "GoInfoRES.h"
#include "GoDrawRES.h"
#include "GoErrMsgRES.h"

#include "GraphicsAPI.h"
#include "Strings.h"
#include "ETour.h"
#include "TourMaster.h"
#include "PairPaneGUI.h"

extern void __fastcall ShowTourInfo(int messid);	// GoDrawForm

namespace UpsetGUI                                  // UPSET LIST
{
	using ToolsGUI::OperatorError;
   using ToolsGUI::GGrid;
   using PairGUI::GPairGrid;

   __fastcall GUpsetList::GUpsetList(TGroupBox* pairgroup)
   :TCustomListBox(pairgroup->Owner)
   {
   	// GEOMETRY
      Parent  			= (TWinControl*)pairgroup;
      Visible 			= false;
      Style   			= lbStandard;
      Sorted  			= false;
      IntegralHeight = true;
      Columns 			= 0;


      // DRAG AND CLICK SUPPORT
      DragKind     = dkDrag;
      DragMode     = dmAutomatic;
      ControlStyle = ControlStyle<<csDisplayDragImage;

      OnStartDrag = DragStartE;
      OnDragOver  = DragOverE;
      OnDragDrop  = DragDropE;
   }

   __fastcall GUpsetList::~GUpsetList()
   {
   }

   void __fastcall GUpsetList::KeyDown(Word &Key, Classes::TShiftState Shift)
   {
      PairPane->PairGrid->ClearInfo();

      switch(Key)
      {
         case VK_ESCAPE:
         {

            if(PairPane->EditState == PaneGUI::psSelect)
            {
               PairPane->ClickPlayers.CancelPairEdit();

               ItemIndex = -1;   PairPane->PairGrid->NoCellSelected();
               Repaint();        PairPane->PairGrid->Repaint();

               PairPane->PairGrid->ShowInfo(IDI_CancelEdit);
            }
         }
         break;

         default:  TCustomListBox::KeyDown(Key, Shift); return;
      }
      Key = 0;
   }


   void __fastcall GUpsetList::ClickPlayer(void)
   {
   	PairPane->PairGrid->ClearInfo();
      
      if(PairPane->DragPlayer->SoIndex<0)
      {
      	::ShowTourInfo(IDI_Select);
         
         return;
      }

      PairPane->ClickPlayers.PairEdit(PaneGUI::soList);
   }

   void __fastcall GUpsetList::Mount(GGrid* resgrid)
   {
      Hide();

        Top = resgrid->Top;        Left = resgrid->Left;
      Width = resgrid->Width;    Height = resgrid->Height;

      Show();
   }

   void __fastcall GUpsetList::AddPlayer(String Name, int Number)
   {
      Items->BeginUpdate();
         Items->Add(Name);

         if(Items->Count <= (int)PairPane->Upsets.size())
         	PairPane->Upsets[Items->Count-1] = Number;
            
      Items->EndUpdate();
   }

   void __fastcall GUpsetList::RemovePlayer(int Item)
   {
      int ItemEnd = Items->Count-1;

      if(Item<0 || Item>ItemEnd) return;

      Items->BeginUpdate();
      	for(int i =Item+1; i<=ItemEnd; i++) // Shuffle up one ...
         {
         	Items->Strings[i-1]   = Items->Strings[i];
            PairPane->Upsets[i-1] = PairPane->Upsets[i];
         }
         Items->Delete(ItemEnd);             // ... delete end item!
         PairPane->Upsets[ItemEnd] = 0;
      Items->EndUpdate();
   }

   void __fastcall GUpsetList::ReplacePlayer(int Item,  String Name, int Number)
   {
      if(Item<0 || Item>Items->Count-1) return;

      Items->Strings[Item] = Name;
      PairPane->Upsets[Item] = Number;
   }
}
//--------------------------------------------------------------------------


namespace UpsetGUI                                  // DRAG IMPLEMENTATION
{
   void __fastcall GUpsetList::DragStartE
   (
      TObject* Source,                             // This PairGrid
      TDragObject* &DragObject                     // GDragPlayer
   )
   {
      PairPane->DragPlayer->StartList = true;
      PairPane->DragPlayer->StartGrid = false;

      PairPane->iSource             = -1;
      PairPane->jSource             = -1;
      PairPane->DragPlayer->SoIndex = -1;

      String Text    = String();
      TRect  Area    = TRect(0,0,0,0);
      TCursor Cursor = crDefault;

      // IDENTIFY SELECTED CELL
      TPoint O =  ClientOrigin;							// Screen co-ordinates
      int iC=0, jC=0; POINT C;                    // POINT is a pair of long's and incompatible with TPoint!

      if(GetCursorPos(&C)){ iC = C.x - O.x;  jC = C.y - O.y; }

      if(iC>0 && jC>0)
      {
         TPoint P(iC,jC);  int index = ItemAtPos(P, true);

         PairPane->iSource             = iC;
         PairPane->jSource             = jC;
         PairPane->DragPlayer->SoIndex = index;

         if(index >= 0)
         {
            Text   = Items->Strings[index];
            Area   = ItemRect(index);
            Cursor = crArrow;
         }
      }

      PairPane->DragPlayer->StartDragPlayer(Source, Cursor, Text, Area);

      DragObject = PairPane->DragPlayer;
   }


   void __fastcall GUpsetList::DragOverE
   (
      TObject* Sink,                               // This PairGrid
      TObject* Source,                             // Either Grid or List
      int i, int j,                                // Current cursor
      TDragState State,                            // dsDragEnter, Leave, Move
      bool &Accept                                 // Possible drop
   )
   {
      if(!PairPane->HasDragged)
         PairPane->HasDragged = PairPane->DragDetected(i,j);

      if(PairPane->DragPlayer->StartGrid)
      {
         if(PairPane->DragPlayer->PhantomDrag)
            PairPane->DragPlayer->CursorNow = crArrow;
         else
            PairPane->DragPlayer->CursorNow = crDrag;

         return;
      }

      PairPane->DragPlayer->CursorNow = crDrag;
   }

   void __fastcall GUpsetList::DragDropE
   (
      TObject* Sink,                               // This PairGrid
      TObject* Source,                             // Either Grid or List
      int i, int j                                 // Current cursor
   )
   {
      PairPane->DragPlayer->Timer->Enabled = false;

      // GDragPlayer::Finish handles all drop processing
   }
}

//---------------------------------------------------------------------------














