////////////////////////////////////////////////////////////////////////////////
// MODULE TestPage: Printer Interface TestPage                 G.Kaniuk 12/03/02
//

#define STRICT

#include <windows.h>
#include <vcl.h>
#pragma hdrstop

#include "PrinterRES.h"
#include "Strings.h"
#include "ETour.h"
#include "PrinterAPI.h"

#include "TestPage.h"

extern ToolsGUI::ErrorLog AppLog;

namespace TestPage
{
/*
   static void Throw(ToolsGUI::EType etype, short idproc, short idmsg)
   {
      using ToolsGUI::Forge;

      std::string message = Forge(etype, IDP_TestPage, idproc, idmsg);

      if(etype==ToolsGUI::erBug)::AppLog.Add(message);

      throw ToolsGUI::ETour(etype, message);
   }
*/
   ////////////////////////////////////////////////////////////////////////////////
   // Tab delimited text data. The first line is the heading printed in bold
   //

   static const int Regfields = 9, Regchars = 115;

   static const int Regcols[] = // SPECFY col posns in characters
   {
      5,       // Player ID
      33,      // Name
      5,       // Grade
      15,      // Club
      7,       // Country
      5,       // Group
      30,      // Rounds
      10,       // Note
      5       // Registered
   };

   static const char* Reg[]=
   {
      "....\tNAME\tG\tCLUB\tLAND\tGP\tROUNDS\tNOTE\tREG",

      "322\tSimone Chevalierz Miro Butler\t1D\tReading\tUK\t\tP P P P P .. .. .. .. ..\tF--\tY",
      "284\tMiroslav Cerny\t1K\tN\tSK\tA\t.. .. .. .. .. .. .. .. .. ..\t-W-\tY",
      "157\tEddie Chang\t5K\tStockholm\tSE\t\t.. .. .. .. .. P P P P P\t-W-\tY",
      "185\tAlain Chevalier\t1K\tMontreuil\tFR\tA\tP P P P P P P P P P\tF-S\tY",
      "175\tYowon Choi\t18K\tAmsterdam\tNL\t\tP P P P P P P P P P\tF-S\tN",

      "216\tTerje Christoffersen\t1D\tOslo\tNO\t\tP P P P P .. .. .. .. ..\tFW-\tN",
      "203\tJim Clare\t3D\tReading\tUK\t\tP P P P P P P P P P\tFWS\tY",
      "311\tMatthew Cocke\t5D\tNorwich\tUK\t1\t.. .. .. .. .. P P P P P\t-W-\tY",
      "238\tRoger Daniel\t3K\tCLGC\tUK\t\tP P P P P P P P P P\tFWS\tY",
      "211\tYuji Deguchi\t1D\tRyokusei\tJP\t\tP P P .. .. .. .. .. .. ..\tF--\tY",

      "239\tGunnar Dickfeld\t1D\tFrankfurt\tDE\t\tP P P P P P P P P ..\tFWS\tY",
      "151\tNiek Van Diepen\t3D\tNijmegen\tNL\t1\tP P .. .. P .. .. .. .. ..\tF--\tY",
      "184\tManfred Diers\t1K\tOldenburg\tDE\tA\tP P P P P .. .. .. .. ..\tFW-\tY",
      "171\tAndreas Drost\t3D\tLeiden\tNL\t1\tP P P P P P P P P P\tF-S\tY",
      "092\tChristophe Dryepondt\t1K\tLevallois\tFR\tA\tP P P P P P P P P P\tFWS\tY"
   };

   static const int Drawfields =  6, Drawchars = 115;

   static const int Drawcols[Drawfields] =
   {
      15,    // Board number
      4,    // Black Result
      37,   // BlackPlayer(grade)
      25,   // Komi
      4,    // White Result
      37   // WhitePlayer(grade)
   };

   static const char* Draw[] =
   {
      "BOARD\t\tBLACK\tKOMI\t\tWHITE",
      "15\t_\tGuido Tautorat(4D)\t\t_\tMasanori Tanaka(6D)",
      "16\t_\tRobert Rehm(5D)\t\t_\tTakashi Miki(6D)",
      "17\t_\tAndreas Drost(3D)\t\t_\tMatthew Cocke(5D)",
      "18\t_\tRichard Hunter(4D)\t\t_\tTobias Berben(3D)",
      "19\t_\tPal Sannes(4D)\t\t_\tKazuma Ishida(6D)",

      "20\t_\tArend-Jan Westhoff(4D)\t\t_\tJoachim Beggerow(2D)",
      "21\t_\tErik Ekholm(3D)\tno komi\t_\tSetsuo Takahashi(6D)",
      "22\t_\tDaniel Gilder(2D)\t\t_\tJens Vygen(4D)",
      "23\t_\tDiana Koszegi(6D)\t\t_\tPiers Shepperson(5D)",
      "24\t_\tRobert Jasiek(5D)\t\t_\tChristian Stodte(3D)",

      "25\t_\tWim Hofman(3D)\t9 st\t_\tTMark Hall(4D)",
      "26\t_\tRoman Pszonka(4D)\t\t_\tDaniela Trinks(3D)",
      "27\t_\tBernd Radmacher(4D)\t\t_\tYoshiyuki Uemura(3D)",
      "28\t_\tLionel Fischer(3D)\t9 st + 24 komi\t_\tSadao Itoh(3D)",
      "29\t_\tJohn O'Malley(2D)\t\t_\tCharles Brideman-Phillips(3D)"
   };

   static const int Rankfields =  19, Rankchars = 160;

   static const int Rankcols[Rankfields] =
   {
      7,                   // Position
      6,                   // Id
      28,                  // Name
      10,                   // Grade
      10,                  // Country
      6,6,6,6,6,6,6,6,6,9, //Results
      5,                   // Wins
      20,                   // MM score
      10,                   // Tie 1
      10                   // Tie 2
   };

   static const char* Rank[] =
   {
      "POSN\tNO\tNAME\tGR\tLAND\t1\t2\t3\t4\t5\t6\t7\t8\t9\t10\tW\tM\tSOS\tSOD",
      "1\t1\tAndrei Kulkov\t5D\tRU\t36+\t10+\t2+\t37+\t4+\t3+\t6+\t9+\t5-\t8+\t9\t13\t103\t92h",
      "2\t2\tKiyoshi Fujita\t6D\tJP\t18+\t9+\t1-\t8-\t35+\t34+\t5+\t7+\t3+\t4+\t8\t12\t103\t80",
      "3\t3\tChristoph Gerlach\t6D\tDE\t58+\t33+\t16+\t7+\t19+\t1-\t4+\t6+\t2-\t5+\t8\t12\t102\t77h",
      "4\t4\tEmil Nijhuis\t5D\tNL\t40+\t6+\t8+\t17+\t1-\t10+\t3-\t11+\t21+\t2-\t7\t11\t104\t67",
      "5\t5\tMatthew Macfadyen\t6D\tUK\t37+\t16-\t12+\t20+\t17+\t9+\t2-\t10+\t1+\t3-\t7\t11\t102\t69h",

      "6\t6\tRudi Verhagen\t5D\tNL\t20+\t4-\t53b\t11+\t8+\t16+\t1-\t3-\t17+\t22+\t7\t11\t100h\t56h",
      "7\t7\tGeert Groenen\t6D\tNL\t56+\t53+\t54+\t3-\t16-\t22+\t8+\t2-\t11+\t12+\t7\t11\t97h\t63h",
      "8\t8\tBela Nagy\t5D\tRO\t34+\t23+\t4-\t2+\t6-\t17+\t7-\t19+\t9+\t1-\t6\t10\t103\t57h",
      "9\t9\tCornel Burzo\t5D\tRO\t35+\t2-\t59+\t23+\t10+\t5-\t16+\t1-\t8-\t20+\t6\t10\t98\t52",
      "10\t10\tTakehiko Hayashi\t5D\tJP\t13+\t1-\t87+\t33+\t9-\t4-\t38+\t5-\t41+\t23+\t6\t10\t94\t49",

      "11\t11\tHirobumi Ohmori\t5D\tJP\t75-\t60+\t61+\t6=\t18+\t38+\t14+\t4-\t7-\t19+\t6\t10\t90h\t50",
      "12\t12\tTakao Yoshida\t6D\tJP\t21+\t54-\t5-\t28-\t61+\t59+\t41+\t16+\t13+\t7-\t6\t10\t90\t50",
      "13\t13\tZoran Mutabzija\t5D\tHR\t10-\t58+\t34-\t35-\t47+\t33+\t42+\t22+\t12-\t25+\t6\t10\t85h\t49",
      "14\t14\tMarco Firnhaber\t5D\tDE\t16-\t62+\t20-\t75+\t52+\t86b\t11-\t17-\t42+\t21+\t6\t10\t84h\t39",
      "15\t15\tNaoyuki Kai\t6D\tJP\t..\t..\t..\t..\t..\t63+\t60+\t36+\t18+\t17+\t5\t10\t40\t40"
   };
}

namespace PrinterAPI
{
  /////////////////////////////////////////////////////////////////////////////
   // PrintTestPage
   //
   // Prints 1 test page containing all variety of features actually used in
   // application printing.
   //
   // Also elementary graphics such as crop marks to show full printable area.
   //
   // The call to this routine is bracketed by either
   //
   //    StartNewPrinter(); PrintTestPage(); EndPrinter(); or
   //
   //    StartPrinter(); PrintTestPage(); EndPrinter();
   //
   // In other words PrintTestPage can assume it already has a DC.
   //


   bool __fastcall PromptTestPrint()
   {
      using namespace Tools;
      
      const char* Caption = load_str(IDP_TestCap).c_str();
      const char* Message = load_str(IDP_TestPrompt).c_str();

      int     Style = MB_YESNO|MB_ICONEXCLAMATION|MB_HELP;

      int Result = Application->MessageBox(Message, Caption, Style);

      return Result == IDYES;
   }


   static void PrintBlock
   (
      PrSheet& Sheet,
      const int Lines, const char* Line[],
      const int Fields, const int Cols[],
      const int Height, const int Chars,
      const int LineStep, int& Row
   )
   {
      using std::string;
      using std::vector;

      vector<int> Column(Fields+1);

      int i, f;

      int Width = (Sheet.Rmargin - Sheet.Lmargin)/Chars, FontWidth, FontHeight;


      // Heading is bold

      Sheet.NewFont(Height, Width, pfBold);  // Create New & delete old if non std

      FontWidth = Sheet.FontWidth; FontHeight = Sheet.FontHeight;

      if(FontWidth*Chars > Sheet.Harea || FontHeight > LineStep)
         Throw(ToolsGUI::erOperr, IDP_PrintBlock, IDP_Test);

      Column[0] = Sheet.Lmargin;

      for(f=1; f<=Fields; f++)Column[f] = Column[f-1] + Cols[f-1]*FontWidth;

      Sheet.TextLineOut(LineStep, Row, Column, string(Line[0]));

      Row += LineStep;


      // Revert to normal for body

      Sheet.NewFont(Height, Width, pfNormal);

      FontWidth = Sheet.FontWidth;  FontHeight = Sheet.FontHeight;

      for(i=1; i<=Lines; i++)
      {
         string Li = string(Line[i]);

         if(Li[Li.size()-1] == 'h')Li[Li.size()-1] = pr_half;

         Sheet.TextLineOut(LineStep, Row, Column, Li);

         Row += LineStep;

         if(i%5==0) Row += LineStep/2;
      }

      // replace the standard ariel font?
   }

   static void DrawCross(PrSheet& sh, int left, int top, int hsize)
   {
      sh.MoveTo(left-hsize, top);         sh.LineTo(left+hsize, top);
      sh.MoveTo(left,       top-hsize);   sh.LineTo(left,      top+hsize);
   }

   static void DrawMarks(PrSheet& sh)
   {
      const int len = 20, marg = 40;

      // CROP MARKS AT PRINTABLE AREA
      sh.MoveTo(0,len);
      sh.LineTo(0,0);                    sh.LineTo(len,          0);
      sh.MoveTo(sh.Harea - 1, len);
      sh.LineTo(sh.Harea-1, 0);          sh.LineTo(sh.Harea-len, 0);

      sh.MoveTo(sh.Harea-1, sh.Varea-len);
      sh.LineTo(sh.Harea-1, sh.Varea-1);   sh.LineTo(sh.Harea-len, sh.Varea-1);
      sh.MoveTo(len,        sh.Varea-1);
      sh.LineTo(0,          sh.Varea-1);   sh.LineTo(0,            sh.Varea-len);

      // CROSSES AT MARGINS
      DrawCross(sh, sh.Lmargin, sh.Tmargin, marg);
      DrawCross(sh, sh.Rmargin, sh.Tmargin, marg);
      DrawCross(sh, sh.Lmargin, sh.Bmargin, marg);
      DrawCross(sh, sh.Rmargin, sh.Bmargin, marg);

      int Hc = (sh.Lmargin + sh.Rmargin)/2, Vc = sh.Bmargin;

      sh.FrameRect(Hc-marg, Vc-marg, Hc+marg, Vc+marg);
   }

   void PrintTestPage()
   {
      using namespace TestPage;

      PrSheet Sheet;

      StartDoc(Tools::load_cstr(IDP_TestDoc));

      // At this point we will normally need text metrics to decide
      // how many pages to print

      StartPage();

      // After start page we now need to reselect all objects like fonts & pens!

      Sheet.SelFont(Sheet.hCurFont);

      int Ht = Sheet.FontHeight;

      int Row = Sheet.Tmargin;

      PrintBlock(Sheet, 15,Reg,  Regfields,Regcols,          Ht,  Regchars, (11*Ht)/10, Row);
      Row += 4*Ht;

      PrintBlock(Sheet, 15,Draw, Drawfields,Drawcols, (11*Ht)/10, Drawchars,    (4*Ht)/3, Row);
      Row += 4*Ht;

      PrintBlock(Sheet, 15,Rank,  Rankfields,Rankcols, (10*Ht)/11, Rankchars,Ht, Row);

      DrawMarks(Sheet);

      EndPage();

      EndDoc();
   }
}

//---------------------------------------------------------------------------


