////////////////////////////////////////////////////////////////////////////////
//                    (C) MS Instruments plc
////////////////////////////////////////////////////////////////////////////////
//
// MODULE:   Format            		CONVERSION OF BASIC TYPES TO STRING
//
// SYSTEM:   MATH DEVELOPMENT
//
// SPEC:     MSI Maths				  ISSUE: A
//
// AUTHOR:   G Kaniuk              DATE:   20 Jul 2001
//
////////////////////////////////////////////////////////////////////////////////
//
// HISTORY:  New at creation date
//
////////////////////////////////////////////////////////////////////////////////
//
// FORMAT   MEANING												  EXAMPLE OUTPUT(* = space)
//
// "###"    integer										         123
// " ###"																*123  -123
// "+###"   															+123  -123
//
// " 000"   zero filled integer				               *012  -012
// "+000"   															+012  -012
// "000"    											            012
//
// " ##.###  real decimal											*12.345  -12.345
// "+##.###  															+12.345  -12.345
// "##.###  															12.345
//
// " ##.###e+00  real floating point							*12.345e-07  -12.345e+07
// "+##.###e+00   													+12.345e-07  -12.345e+07
//
////////////////////////////////////////////////////////////////////////////////

#include <string>

#include "Maths.h"

#ifndef FormatH
#define FormatH


namespace Tools
{
	using std::string;

   using Tools::real;
   
	enum Prefix
   {
   	pxNone,		// Interpreted as positive int or real

      pxHidden, 	// Negatives have prefix -, positive or zero have space prefix

      pxVisible   // All have prefix + or -
   };

   enum Numeric
   {
   	nuZero,		// Zero fill integer from left
      nuInteger,  // Space fill integer from left
      nuDecimal,	// Fixed point real
      nuFloat		// Exponent form for real
   };

   string to_string(const Prefix pre, const Numeric type, const int i, const int value);
   string to_string(const Prefix pre, const int i, const d, const real value);
   string to_string(const Prefix pre, const int i, const d, const e, const real value);

   string to_string(const string format, const int value);
   string to_string(const string format, const real value);

   string to_string(const int value);
   string to_stringi(const int ivalue);
   string to_stringr(const int dprec, const real rvalue);
}

namespace Tools
{
   int to_posint                       // Convert contiguous digit sequence to integer
   (
      const string::iterator btext,    // Start of digit sequence
      const string::iterator etext     // End of contiguous digit sequence
   );

   bool to_int									// Possibly signed integer
   (
      const string::iterator btext,    // Start of digit sequence
      const string::iterator etext,    // End of contiguous digit sequence

      int& value
   );

}
// Format
////////////////////////////////////////////////////////////////////////////////
#endif
