//---------------------------------------------------------------------------
#include <math>
#pragma hdrstop

#include "Maths.h"

namespace Tools
{
   const real epsilon	=  0.00000000000000100000;			                     // Fundamental constants
   const real pi			=	3.14159265358979323846;
   const real halfpi		=  1.57079632679489661923;
   const real twopi     =  6.28318530717958647692;
   const real deg_rad   =  0.01745329251994300000;
   const real rad_deg   = 57.29577951308238000000;
   const real root2     =  1.41421356237309500000;
   const real root3     =  1.73205080756887700000;
   const real log2		=  0.30102999566398119521;

   int  round(real r)         { return (int)( r >=0 ? r + 0.5  : r - 0.5 ); } // Nearest to r
   int  entier(real r)        { return (int)( r >=0 ? r : r - 1 ); }   	      // Largest i < r

   int  modulus(int i)        { return i >=0   ? i : -i; }	    					// |i| - int
   real modulus(real r)	      { return r >=0.0 ? r : -r; }	    				   // |r| - real


   int  sign(int i)	         { return  i > 0 ? 1 : (i<0 ? -1 : 0); }                     	// i/|i|
   real sign(real r)	         { return  r > 0 ? 1 : (r<0 ? -1 : 0); }   	   // r/|r|

   real min(real u, real v)   { return u<v ? u : v; }                         // Pairwise ordering
   real max(real u, real v)   { return u>v ? u : v; }

   void bounds(int si, siBounds& range)                                       // Collection range
   {
      if(si<range.min) range.min = si;
      if(si>range.max) range.max = si;
   }

   void bounds(size_t ui, uiBounds& range)
   {
      if(ui<range.min) range.min = ui;
      if(ui>range.max) range.max = ui;
   }

   void split(real r, int& ir,  real& fr)													  // Decompose r = integral part + fractional part
   {
      if(r>0)
      {
         ir = entier(r);  fr = r - (real)(ir);

         if(fr>=1){ ir += 1; fr -=1; }

         if(fr<0) fr = 0;
      }

      if(r<0)
      {
         split(-r, ir, fr);

         ir = -ir;  fr = -fr;
      }

      if(r==0)
      {
         ir = 0;    fr = 0;
      }
   }


   void split(real r, real& ir,  real& fr)
   {
      if(r>0)
      {
   	   ir = (real)entier(r);  fr = r - ir;

         if(fr>=1){ ir += 1;  fr -=1; }

         if(fr<0) fr = 0;
      }

      if(r<0)
      {
         split(-r, ir, fr);

         ir = -ir;  fr = -fr;
      }

      if(r==0)
      {
         ir = 0;    fr = 0;
      }
   }

   void set_bit(const int bpos, const bool bval, bits& sink)
   {
   	if(bpos<0 || bpos >= 8*sizeof(bits)) return;

      if(bval)
      	sink |= 1<<bpos;  		// Set the bit
      else
      	sink &= ~(1<<bpos);  	// Clear the bit
   }
}
//---------------------------------------------------------------------------

