//---------------------------------------------------------------------------
#ifndef MathsH
#define MathsH

namespace Tools
{
   typedef unsigned char 	byte;                // 8 bit binary pattern
   typedef char          	intsi;					// 8 bit integer
   typedef short int       ints;                // 16 bit integer
   
   typedef unsigned int  	bits;					   // 32 bitwise operations
   typedef unsigned long 	lbits;    			   // 64 bitwise operations

   typedef double        	real;					   // All numerical work

   typedef long double     lreal;				   // Very high precision work

   struct siBounds										// The signed integer interval
   {
   	int min, max;

      siBounds() 					 { min = -1;  max = 1; }
      siBounds(int Min, int Max){ min = Min; max = Max; }

      bool operator()(int i){ return min<=i && i<=max; }

      bool operator()(size_t i){ return min<=(int)i && (int)i<=max; }
   };

   struct uiBounds										// The unsigned signed integer interval
   {
   	size_t min, max;

      uiBounds() 					  { min = 0;  max = 1; }
      uiBounds(int Min, int Max){ min = Min; max = Max; }

      bool operator()(size_t i){ return min<=i && i<=max; }

      bool operator()(int i){ return (int)min<=i && i<=(int)max; }
   };



   extern const real epsilon;			            // Fundamental constants
   extern const real pi;
   extern const real halfpi;
   extern const real twopi;
   extern const real deg_rad;
   extern const real rad_deg;
   extern const real root2;
   extern const real root3;
   extern const real log2;

   int  round(real r);                          // Nearest to r
   int  entier(real r);                         // Largest i < r

   int  modulus(int i);                         // |i| - int
   real modulus(real r);                        // |r| - real


   int  sign(int i);                            // i/|i|
   real sign(real r);                           // r/|r|

   void split(real r, int& ir,  real& fr);		// Decompose r = integral part + fractional part
   void split(real r, real& ir,  real& fr);

   real min(real u, real v);                    // Pairwise ordering
   real max(real u, real v);

   void bounds(int si, siBounds& range);        // Collection range
   void bounds(size_t ui, uiBounds& range);

   void set_bit(const int bpos, const bool bval, bits& sink);
}
//---------------------------------------------------------------------------
#endif
