//---------------------------------------------------------------------------
// MODULE Statistics:         STATISTICAL ANALYSIS     G Kaniuk    31/03/01
//---------------------------------------------------------------------------

#include <math.h>
#pragma hdrstop

#include "Statistics.h"
//---------------------------------------------------------------------------

namespace Tools
{
   void Clear(Moments* M)
   {
      M->n = 0;

      M->x = M->y = 0;

      M->x2 = M->xy = M->y2 = 0;
   }

   void Clear(Model* L)
   {
      L->Slope = L->Offset = 0;

      L->Corr = 0;

      L->Resid = -1;
   }

   
   void AddSample(real X, real Y, Moments* M)               // Accumulate a new sample
   {
      real S = 1/(real)(M->n + 1);
      real R = S*(real)M->n;

       M->x = R*M->x + S*X;       M->y = R*M->y + S*Y;
      M->x2 = R*M->x2 + S*X*X;   M->y2 = R*M->y2 + S*Y*Y;   M->xy = R*M->xy + S*X*Y;

      ++M->n;
   }

   bool LinearFit(Moments M, Model* L)                      // Linear fit assuming that
   {                                                        // x-samples noiseless and
      if(M.n < 1) return false;                             // not all equal

      real vX = M.x2 - M.x*M.x, vY = M.y2 - M.y*M.y;
      real cXY = M.xy - M.x*M.y;

      if(vX <= 0) return false;

      L->Slope = 0;  L->Offset = M.y;
      L->Resid = 0;  L->Corr   = 0;

      if(M.n<2 || vY <= 0) return true;                     // Only one point!

      L->Slope = cXY/vX;  L->Offset = M.y - L->Slope*M.x;

      L->Corr = cXY/sqrt(vX*vY);

      real Lsq = M.y2 + L->Slope*L->Slope*M.x2 + L->Offset*L->Offset
                                       +
               2*(L->Slope*L->Offset*M.x - L->Slope*M.xy - L->Offset*M.y);

      if(Lsq<0) Lsq = 0;                  // Can happen with rounding error

      L->Resid = sqrt(Lsq);

      return true;
   }
}

namespace Tools									// INVERSE HYPERBOLIC FUNCTIONS
{
	real asinh(const real x)
   {
   	return log(x + sqrt(1+x*x));
   }

   real atanh(const real x)
   {
   	if(x>=1-epsilon) return 15;
      if(x<=-1+epsilon) return -15;

      return log((1+x)/(1-x))/2;
   }
}

namespace Tools									// ERROR FUNCTION
{
   static const real a = 1.129324, b = 0.100303;

   real erf(const real x)						// Maths Today
   {
      return tanh(x*(a+b*x*x));
   }

   real inverf(const real x)
   {
   	const real d = sqrt((3*a)/(4*b)), h = 3/(a*d);

      return d*sinh(asinh(h*atanh(x))/3);
   }
}
//---------------------------------------------------------------------------



