////////////////////////////////////////////////////////////////////////////////
// MODULE Strings:      String processing support              G.Kaniuk 07/05/01
////////////////////////////////////////////////////////////////////////////////
//
// EXTREMELY IMPORTANT NOTE!!!
//
// int std::string.find(char, int pos):
//
// If it cannot find char in the string it comes up with some huge
// number beyond the actual size of the string
// So to test if not found we MUST test for result >= string.size()
//
////////////////////////////////////////////////////////////////////////////////

#include <iterator>
#include <vector>
#include <ctype.h>

#pragma hdrstop

#include "Format.h"
#include "Strings.h"

namespace Tools                           // MAGIC CONSTANTS AND STRING INTERFACE
{
   using namespace std;

   const size_t max_item_size = 255;      // Who uses this??

   const char* sEmpty = "";               // Declaring the composites
   const char* sTab   = "\t";             // as string("...") caused
   const char* sCrLf  = "\r\n";           // application to crash!
   const char* sSpace = " ";
   const char* sDot   = ".";
   const char* sTilde = "~";
   const char* sSep   = "\\";
   const char* sUline = "_";
   const char* sEqual = "=";
   const char* sQuery = "?";
   const char* sComma = ",";
   const char* sBra   = "(";
   const char* sKet   = ")";
   const char* sPlus  = "+";
   const char* sMinus = "-";
   const char* sSqBra = "[";
   const char* sSqKet = "]";

   const char cTab   = '\t';
   const char cNull  = '\0';
   const char cUnit  = '\1';

   const char cSpace = ' ';
   const char cDot   = '.';
   const char cTilde = '~';
   const char cUline = '_';
   const char cMinus = '-';
   const char cPlus  = '+';
   const char cStar  = '*';
   const char cComma = ',';


   const char cQuery = '?';
   const char cEqual = '=';
   const char cQuote = '\'';
   const char cSep   = '\\';
   const char cSemic = ';';
   const char cColon = ':';

   const char cZero  = '0';
   const char cOne   = '1';
   const char cNine  = '9';

   const char cGl_a  = 'a';   // cGlya causes problems - used by someone else?
   const char cGl_z  = 'z';

   const char cGlyA  = 'A';
   const char cGlyZ  = 'Z';

   const char cKyu   = 'k';
   const char cDan   = 'd';
   const char cPro   = 'p';

   const char cLose   = '0';
   const char cWin    = '1';
   const char cJigo   = 'J';
   const char cVoid   = 'v';
   const char cDlos   = 'd';
   const char cUsolv  = 'u';
   const char cNone   = '_';
   const char cFwin   = 'f';

   static const char czz = 'z';     // Exposing these in Strings.h
   static const char caa = 'a';     // causes linker problems!

   static const char cAA = 'A';
   static const char cZZ = 'Z';

   const char cQuarter      =  (char)0xbc;  // ANSI
   const char cHalf         =  (char)0xbd;
   const char cThreeQuarter =  (char)0xbe;

   const char cFractional[4] = { cNull, cQuarter, cHalf, cThreeQuarter };

   String s_S(string s) { return String(s.c_str()); }
   string S_s(String S) { return string(S.c_str()); }

   char* load_cstr(int idstr){ return LoadStr(idstr).c_str(); }
   string load_str(int idstr){ return string(LoadStr(idstr).c_str()); }

   bool is_alphabetic(char c)
   {
      if(cAA <= c && c <= cZZ) return true;
      if(caa <= c && c <= czz) return true;

      return false;
   }

   char uppercase(const char c)
   {
      return caa <= c && c <= czz ? c - cSpace : c;
   }

   char lowercase(const char c)
   {
      return cAA <= c && c <= cZZ ? c + cSpace : c;
   }

   bool is_ascii(const char c)
   {
      return cSpace <= c && c <= cTilde;
   }

   bool is_ascii(string text)
   {
   	for(size_t i=0; i<text.size(); i++)
      	if(!is_ascii(text[i]))
         	return false;

      return true;
   }
}

namespace Tools               // LEXICAL ANALYSIS
{
   const  Byte lexAlpha = lxUpper | lxLower;

   static Byte lex_class[256];

   void set_lexical_classes()
   {
      Byte c;

      c = 0xff;                 lex_class[c] = lxOther;
      for(c=0x20; c<0xff; c++)  lex_class[c] = lxOther;

      for(c=0x00; c<=0x1f; c++) lex_class[c] = lxControl;
      for(c=0x30; c<=0x39; c++) lex_class[c] = lxDigit;
      for(c=0x41; c<=0x5a; c++) lex_class[c] = lxUpper;
      for(c=0x61; c<=0x7a; c++) lex_class[c] = lxLower;

      lex_class[cMinus] = lxNameGlyph;
      lex_class[cQuote] = lxNameGlyph;
      lex_class[cSpace] = lxNameGlyph;							// GK 6.5.1

      lex_class[cDot]   = lxFileGlyph;
      lex_class[cSep]   = lxFileGlyph;

      lex_class[cSemic] = lxTermGlyph;
      lex_class[cColon] = lxTermGlyph;
   }

   LexClass lexical_class(Byte c){ return (LexClass)lex_class[c]; }

   bool is_lex_class(const char c, const Byte lexclass)
   {
      return lex_class[c] & lexclass;
   }

   bool isnot_lex_class(const char c, const Byte lexclass)
   {
      return !(lex_class[c] & lexclass);
   }

   Byte lexical_class(const char c){ return lex_class[c]; }
}

namespace Tools               // TAB-DELIMITED STRINGS
{
   void tab_add(string& source, const string& item)
   {
   	if(item.empty())
      {
      	source += sTab;  return;
      }

      string newitem = item;  strip_null(newitem);

      source += sTab + newitem;
   }

   int size_tabbed(const int n, string item[])                    // Tabbed length
   {
      int length = 0;   if(n<=0) return 0;

      for(int i=0; i<n; i++)length += item[i].length();
                                                                  // Allow space
      return length + (n-1);                                      // for tabs
   }

   void pack_tabbed(const int n, string item[], string* packed)      // Pack items
   {
      *packed = string();  if(n<=0) return;

      for(int i=0; i<n-1; i++) *packed += item[i] + sTab;

      *packed += item[n-1];
   }

   void pack_tabbed_line(const int n, string item[], string* packed) // Pack items
   {
      *packed = string();  if(n<=0) return;

      for(int i=0; i<n-1; i++) *packed += item[i] + sTab;

      *packed += item[n-1] + sCrLf;
   }

   int unpack_tabbed(string packed, const int n, string item[])     // Unpack items
   {
      string::iterator P = packed.begin(), E = packed.end();

      if(n<=0)return 0;

      int i=0, ipos = 0, ilen = 0;                 // Keep track of items found

      int count = 0;

      while(P != E && i<n)
      {
         if(*P == cTab)                               // We have an item delimiter
         {
            item[i] = packed.substr(ipos, ilen);      // Extract the item

            if(ilen>0) ++count;                       // Non empty items

            ipos += ilen+1; ilen = 0;                 // Skip past tab

            ++i;  if(i==n) break;                     // More tabs than specified!
         }
         else
            ++ilen;                                   // Another valid data character

         ++P;
      }

      if(i < n)                                       // Up to specified tabs
      {
         item[i] = packed.substr(ipos, ilen);         // Last item terminated by 0

         if(ilen>0) ++count;
      }

      return count;
   }

   int unpack_tabbed_exact(string packed, const int n, string item[])     // Unpack items
   {
      string::iterator P = packed.begin(), E = packed.end();

      if(n<=0)return 0;

      int i=0, ipos = 0, ilen = 0;                 	// Keep track of items found

      int count = 0;

      while(P != E && i<n)
      {
         if(*P == cTab)                               // We have an item delimiter
         {
            item[i] = packed.substr(ipos, ilen);      // Extract the item

            ++count;                       				// Possibly empty items

            ipos += ilen+1; ilen = 0;                 // Skip past tab

            ++i;  if(i==n) break;                     // More tabs than specified!
         }
         else
            ++ilen;                                   // Another valid data character

         ++P;
      }

      if(i < n)                                       // Up to specified tabs
      {
         item[i] = packed.substr(ipos, ilen);         // Last item terminated by 0

         ++count;													// Possibly empty
      }

      return count;
   }

   int unpack_tabbed(string packed, const int n, list<string>& items)
   {
      string::iterator P = packed.begin(), E = packed.end();

      int i=0, ipos = 0, ilen = 0;                 // Keep track of items found

      int count = 0;

      if(items.size()>0)items.resize(0);           // We don't know how many items

      while(P != E)
      {
         if(*P == cTab)                                        // We have an item delimiter
         {
            items.push_back(packed.substr(ipos, ilen));        // Extract the item to the list

            if(ilen>0) ++count;                                // Non empty items

            ipos += ilen+1; ilen = 0;                          // Skip past tab

            ++i; if(i>n)break;
         }
         else
            ++ilen;                                   // Another valid data character

         ++P;
      }
      items.push_back(packed.substr(ipos, ilen));     // Item from last Tab to string end

      if(ilen>0) ++count;

      return count;
   }

}
//---------------------------------------------------------------------------

namespace Tools                           // OPERATIONS ON STRINGS                                                   // STRING OPERATIONS
{
   using std::string;

   string slice(char* stext, char* etext)
   {
      if(etext<stext) return string();

      size_t n = etext - stext, i;

      string text(n, cNull);

      char* s = stext;

      for(i=0; i<n; i++)text[i] = *s++;

      return text;
   }


   string trim_left(string text)                                 // Remove leading, trailing spaces
   {
      if(text.empty()) return text;

      string::iterator f = text.begin(), e = text.end();

      while(f !=e )
      {
         if(*f != cSpace) break;

         ++f;
      }
      if(f==e) return string();

      return string(f,e);
   }

   string trim_right(string text)                                 // Remove leading, trailing spaces
   {
      if(text.empty()) return text;

      string::iterator s = text.begin();
      string::iterator e = text.end();

      while(e != s)
      {
         if(*(e-1) != cSpace)break;

         --e;
      }
      if(e==s) return string();

      return string(s, e);
   }

   string trim(string text)                   // Remove leading, trailing spaces
   {
      if(text.empty()) return text;

      string::iterator f = text.begin(), e = text.end();

      while(f != e)                 // Left trim
      {
         if(*f != cSpace) break;

         ++f;
      }
      if(f==e) return string();

      while(e != f)                 // Right trim backwards
      {
         if(*(e-1) != cSpace)break;

         --e;
      }
      if(e==f) return string();

      return string(f,e);
   }

   string compactify(string text) // Reduce internal spaces to 1
   {
      string tex = trim(text);

      if(tex.empty()) return tex;

      string::iterator ts = tex.begin(), te = tex.end(), tf;

      string heads;                          // text = heads + spaces + text

      tf = ts;

      while(tf != te)                        // the heads: sequence of non-space
      {
         if(*tf==cSpace) break;

         ++tf;
      }
      if(tf == te)return tex;                // No spaces, so text = heads

      heads = string(ts, tf);                // tf IS the heads terminator

      ++tf;                                  // Find start of remaining text

      while(tf != te)
      {
         if(*tf !=cSpace) break;

         ++tf;
      }
      return heads + sSpace + compactify(string(tf, te));
   }

   string compactify_name(string text) // Reduce internal spaces to 1
   {                                   // No space after comm
      string tex = trim(text);

      if(tex.empty()) return tex;

      string::iterator ts = tex.begin(), te = tex.end(), tf;

      string heads;                          // text = heads + spaces + text

      tf = ts;

      while(tf != te)                        // the heads: sequence of non-space
      {
         if(*tf==cSpace) break;

         ++tf;
      }
      if(tf == te)return tex;                // No spaces, so text = heads

      heads = string(ts, tf);                // tf IS the heads terminator

      bool comma = *(tf-1)==cComma;				// Space follows a comma

      ++tf;                                  // Find start of remaining text

      while(tf != te)
      {
         if(*tf !=cSpace) break;

         ++tf;
      }

      if(comma)
      	return heads + compactify(string(tf, te));
      else
         return heads + sSpace + compactify(string(tf, te));
   }

   string pad_right(string text, size_t fieldsize)
   {
      if(fieldsize==0) return string();

      text.resize(fieldsize, cSpace);

      return text;
   }

   string uppercase(string text)
   {
      string u = text;

      string::iterator ui = u.begin();

      while(ui != u.end())
      {
         if( caa <= *ui && *ui <= czz)
            *ui -= cSpace;

         ++ui;
      }
      return u;
   }

   string lowercase(string text)
   {
      string u = text;

      string::iterator ui = u.begin();

      while(ui != u.end())
      {
         if( cAA <= *ui && *ui <= cZZ)
            *ui += cSpace;

         ++ui;
      }
      return u;
   }

   string squeeze(string text)     // Remove anything not alpha
   {
      string squeezed = text;

      string::      iterator qi = squeezed.begin();
      string::const_iterator ti = text.begin();

      size_t qsize = 0;

      while(ti != text.end())
      {
         if(is_alphabetic(*ti))
         {
            *qi = *ti;

            ++qi; ++qsize;
         }
         ++ti;
      }
      squeezed.resize(qsize);

      return squeezed;
   }

   string strip_ext(string filename)
   {
      if(filename.empty()) return filename;

      string::iterator fbegin = filename.begin(), fend = filename.end();
      string::iterator f = fbegin, lastdot=fend;

      while(f != fend)
      {
         if(*f == cDot)lastdot = f;

         ++f;
      }
      if(lastdot == fbegin) return sEmpty;   // No text before the dot
      if(lastdot == fend) return filename;   // No dot at all

      return string(fbegin, lastdot);        // Up to but not including the dot
   }

   string strip_crlf(string text)
   {
      const string crlf = string(sCrLf);

      int tsize = text.size();

      if(tsize<2) return text;

      size_t pos = text.find(crlf, 0);

      if(pos<=(size_t)(tsize-2))
      	return text.substr(0, pos);
      else
      	return text;
   }

   string strip_white_space(string text)
   {
      string stripped = text;

      string::iterator sb = stripped.begin(),se;
      string::iterator it = text.begin();

      se = sb;

      while(it != text.end())
      {
         if(*it && *it != Tools::cSpace)
            *se++ = *it;

         ++it;
      }

      return string(sb, se);
   }

   bool begins_with(string text, string head)
   {
   	if(head.size()>text.size()) return false;

      return uppercase(text).find(uppercase(head), 0) == 0;
   }

   string spaces(const int count)
   {
   	return string(count, cSpace);
   }

   int find_first_change(const string atext, const string btext)
   {
   	if(atext.empty() || btext.empty()) return 0;

   	int pos = 0;

   	string::const_iterator ia = atext.begin(),ib = btext.begin();

      while(ia != atext.end() && ib != btext.end())
      {
      	if(*ia != *ib) return pos;

         ++ia;  ++ib;	++pos;
      }

      return -1;
   }

   int find_first_change_rev(const string atext, const string btext)
   {
   	if(atext.empty() || btext.empty()) return 0;

   	int pos = 0;

   	string::const_reverse_iterator ia = atext.rbegin(),ib = btext.rbegin();

      while(ia != atext.rend() && ib != btext.rend())
      {
      	if(*ia != *ib) return pos;

         ++ia;  ++ib;	++pos;
      }

      return -1;
   }

	using std::string;

	string compactify_tail_space(string text)		// TRIM ALL EXCEPT 1 SPACE
   {
   	if(text.empty()) return text;

      string::const_reverse_iterator it = text.rbegin();
      string::const_reverse_iterator ie = text.rend();

      if(*it != Tools::cSpace)
      	return text;									// There are no trailing spaces

      int spacecount = 1; ++it;
      
      while(it != ie)
      {
      	if(*it != Tools::cSpace) break;

         ++spacecount;	++it;
      }

      // One cannot define a string in terms of reverse iterators!
      return string(text, 0, text.size() - spacecount + 1);
   }

   void strip_null(string& text)								// REMOVE all null chars  GK 6.5.1
   {
   	string otext(text);
   	string::iterator otexti=otext.begin(), otexte=otext.end();
      string newtext = string(sEmpty);
      while(otexti != otexte)
      {
      	if(*otexti != cNull) newtext.append(1, *otexti); ++otexti;
      }
      text = newtext;
   }

   void truncate_field(const size_t fldsize, const char* cpad, string& field) // GK 6.5.1
   {
   	const string spad = string(cpad);
      const size_t zpad = spad.size();

      const size_t fieldz = field.size();

      if(fldsize==0  || fldsize<=zpad)							// Clear the field
      {
      	field=string(); return;
      }

      if(fieldz<=fldsize) return;

      field = field.substr(0, fldsize-zpad) + spad;		// Truncate to fldsize
   }
}
//------------------------------------------------------------------------------

namespace Tools                              // NAME ANALYSIS
{
	// A string may be terminated by  a sequence of zero or more null
   // Find the earliest if it exists

   void first_null_terminator										// GK 6.5.1
   (
   	const string::iterator& bstring,							// SOMESTRING.begin()
      string::iterator& estring									// SOMESTRING.end()
   )
   {
      if(*estring != cNull) return;

      while(bstring != estring)									 // Trim trailing Null
      {
      	--estring;

         if(*estring != cNull) { ++estring;  break; }
      }
   }

   // A name starts and ends with alpha and may have
   // embedded spaces or "'" or -.  If the latter
   // two the previous and next chars are alpha

   bool legal_name_syntax(string& name)      				// Assumes name compactified
   {
   	if(name.empty()) return false;

      string::iterator ns = name.begin(), ne = name.end();

      if(isnot_lex_class(*ns, lexAlpha))return false;

      first_null_terminator(ns,ne);								// Trim trailing Null  GK 6.5.1

      ++ns;

      int commacount=0;

      while(ns != ne)
      {
         switch(lexical_class(*ns))
         {
            case lxNameGlyph:
               if(isnot_lex_class(*(ns-1), lexAlpha))    // ns-1 >= begin() guaranteed
                  return false;                          // Previous MUST be alpha

               if((ns+1)==ne)                            // Unfinished name
                  return false;

               if(isnot_lex_class(*(ns+1), lexAlpha))    // ns+1 <= end() guaranteed
                  return false;                          // Next MUST be alpha

               break;

            case lxLower: case lxUpper:
               break;

            default:
            	if(*ns == cComma)
               {
               	++commacount;

                  if(commacount>1)
                  	return false;
                  else
                  	break;
               }

               //if(*ns != cSpace)								GK 6.5.1
               	return false;
         }
         ++ns;
      }
      return true;
   }

   bool parse_name(string name, string& family)	// FORM: Family
   {
      string::iterator ns = name.begin(), ne = name.end();
      string::iterator cs = ns;

      first_null_terminator(ns,ne);							// GK 6.5.1

      // No commas or spaces
      while(cs != ne)
      {
      	if(*cs==cComma) return false;
         if(*cs==cSpace) return false;

         ++cs;
      }

      // Apostrophe and hyphens
      if(!legal_name_syntax(name))return false;

      family = name;

      return true;
   }

   bool parse_name                           // FORM: Given Middle Family
   (
      string  name,                          // Assumes compact
      string& given, string& middle, string& family
   )
   {
      given = middle = family = string();

      string::iterator ns = name.begin(), ne = name.end();
      string::iterator gs = ns, ge, cs;

      first_null_terminator(ns,ne);							// GK 6.5.1

      // No commas
      cs = gs;

      while(cs != ne)
      {
      	if(*cs==cComma) return false;

         ++cs;
      }

      // The compact name = [Leader + [Interior]] + Trailer
      ge = ns;

      while(ge != ne)                  // Parse the Leader
      {
         if(*ge==cSpace) break;

         ++ge;
      }

      if(ge == ne)                     // So name = Trailer
      {
         family = name;                // Just family name

         return legal_name_syntax(family);
      }
      else
      {
         given = string(gs, ge);

         if(!legal_name_syntax(given)) return false;
      }

      string::iterator fs = ne, fe = ne;

      while(fs != ge)                  // Parse the Trailer backwards
      {
         if(*fs == cSpace) break;

         --fs;
      }
      if(fs == ge)                     // No initials - fs IS space terminator
      {
         family = string(fs+1, fe);    // Just given + family

         return legal_name_syntax(family);
      }

      family = string(fs+1, fe);
               if(!legal_name_syntax(family)) return false;

      middle = string(ge+1, fs);
               if(!legal_name_syntax(middle)) return false;

      return true;
   }

   static bool parse_given_middle(string givenmiddle, string& given, string& middle)
   {
   	// The family parts with trailing comma have been extracted
      // So no more commas are expected

   	string::iterator gs = givenmiddle.begin(), ge = givenmiddle.end();
      string::iterator ms, cs;

      first_null_terminator(gs,ge);							// GK 6.5.1

   	// From now on no more commas
      cs = gs;

      while(cs != ge)
      {
      	if(*cs==cComma) return false;

         ++cs;
      }

      // Look for given name
      ms = gs;                      		// A space ends given

      while(ms != ge)
      {
         if(*ms==cSpace) break;  ++ms;
      }

      given = string(gs,ms);					// Terminated by space or string end

      // Look for middle name
      if(ms == ge)
         return legal_name_syntax(given);	// No initials
      else
         if(!legal_name_syntax(given))
            return false;

      middle = string(ms+1, ge);				// Skip over space terminating given

      if(!middle.empty())
      	return
         	legal_name_syntax(middle);

      return true;
   }

   bool parse_family                // FORM: Family Given Middle
   (
      string name,                  // Assumes compact string
      string& given, string& middle, string& family
   )
   {
      using Tools::cComma;

      given = middle = family = string();

      string::iterator ns = name.begin(), ne = name.end();
      string::iterator gs = ns, ms;

      first_null_terminator(ns,ne);							// GK 6.5.1

      // A space ends family
      while(gs != ne)
      {
         if(*gs==cSpace) break;

         ++gs;
      }
      family = string(ns,gs);                // gs IS one beyond end of name

      if(!legal_name_syntax(family))
      	return false;

      if(gs==ne) return true;               // Entire name is family

      ++gs;  if(gs==ne) return true;        // Skip the ONE space

      // Look for given name
      ms = gs;                              // A space ends given

      while(ms != ne)
      {
         if(*ms==cSpace) break;  ++ms;
      }

      given = string(gs,ms);

      // Look for middle name
      if(ms == ne)
         return legal_name_syntax(given);    // No initials
      else
         if(!legal_name_syntax(given))
            return false;

      middle = string(ms+1, ne);

      if(!middle.empty())
      	return
         	legal_name_syntax(middle);

      return true;
   }

   bool parse_extended_family        // FORM: FAMILY Given Middle
   (                                 // FAMILY:Familyone Familytwo, or Familyone,
      string& name,                  // Assumes compact string
      string& given, string& middle, string& family
   )
   {
      using Tools::cComma;

      given = middle = family = string();

      string::iterator ns = name.begin(), ne = name.end();
      string::iterator gs = ns;

      first_null_terminator(ns,ne); // GK 6.5.1

      // Start and end of name must be alpha
      if(!is_alphabetic(*ns))return false;
      if(!is_alphabetic(*(ne-1)))return false;
      
      // Everything up to a comma is family name including any space
      while(gs != ne)
      {
         if(*gs==cComma)
         {
         	family = string(ns,gs);				// gs IS one beyond end of family parts

         	break;
         }

         ++gs;
      }

      // There is no comma at all in the name
      if(gs==ne)
    		return parse_family(name, given, middle, family);

      if(!legal_name_syntax(family))
      	return false;

      ++gs;												// Skip past the comma
      if(gs==ne) return false;               // There is no given part
      if(*gs == cSpace)++gs;                 // May have a space after comma

      return parse_given_middle(string(gs, ne), given, middle);
   }


   bool parse_shortname(string full, string& shortname)
   {
      string given, middle, family;

      shortname = string();

      if(!parse_extended_family(full, given,middle,family)) return false;

      if(given.empty())
      {
         shortname = family;

         return true;
      }

      shortname = given.substr(0,1) + sSpace + family;

      return true;
   }
}
//------------------------------------------------------------------------------

namespace Tools                      // NUMBER ANALYSIS
{
   bool next_posint
   (
      string::iterator& stext,       // Start of string range
      string::iterator& etext,       // End of string range

      int& i                         // Integer delimited by any non-digit character
   )
   {
      while(stext != etext)
      {
         if(isdigit(*stext))break;

         ++stext;
      }

      if(stext==etext) return false;

      string::iterator ftext = stext;

      while(stext != etext)
      {
         if(!isdigit(*stext))break;

         ++stext;
      }

      i = to_posint(ftext, stext);

      return true;
   }

   bool next_int
   (
      string::iterator& stext,       // Start of string range
      string::iterator& etext,       // End of string range

      int& i                         // Integer delimited by any non-digit character
   )
   {
   	using Tools::cMinus;

      while(stext != etext)
      {
         if(*stext==Tools::cMinus || isdigit(*stext))break;

         ++stext;
      }

      if(stext==etext) return false;

      string::iterator ftext = stext;

      if(*stext==cMinus) ++stext;

      while(stext != etext)
      {
         if(!isdigit(*stext))break;

         ++stext;
      }

      return to_int(ftext, stext, i);
   }

}
//------------------------------------------------------------------------------

namespace Tools                                             // CODE LIST ANALYSIS
{
   using std::string;
   using std::list;

   SCodeList::SCodeList()
   {
      codesize=0;

      maxcodes = 0;

      codelist.clear();
   }

   SCodeList::SCodeList(size_t CodeSize)
   {
      codesize = CodeSize;

      firstcode = string(codesize,cAA);
      lastcode  = string(codesize,cZZ);

      maxcodes = 1;

      for(size_t i=0; i<codesize; i++)
         maxcodes *= 26;

      codelist.clear();
   }

   bool SCodeList::full()
   {
      return codelist.size() == maxcodes;
   }

   bool SCodeList::syntax_check(const string& item)
   {
      if(item[codesize] != cEqual) return false;

      for(size_t i=0; i<codesize; i++)
         if(!is_alphabetic(item[i]))
            return false;

      return true;
   }

   string SCodeList::name_part(const string& item)
   {
      if(item.size() < codesize+1) return sEmpty;

      return item.substr(codesize+1, item.size()-codesize-1);
   }

   string SCodeList::code_part(const string& item)
   {
      if(item.size() < codesize+1) return sEmpty;

      return item.substr(0, codesize);
   }

   bool SCodeList::found_name(const string& name)           // SEARCH THROUGH LIST FOR NAME
   {
      list<string>::iterator itemnow = codelist.begin();

      while(itemnow != codelist.end())
      {
         if(name_part(*itemnow) == name)
            return true;

         ++itemnow;
      }
      return false;
   }

   bool SCodeList::found_code(const string& code)           // SEARCH THROUGH LIST FOR CODE
   {
      list<string>::iterator itemnow = codelist.begin();

      if(code.size() != codesize) return false;

      while(itemnow != codelist.end())
      {
         if(code_part(*itemnow)== code)
            return true;

         ++itemnow;
      }
      return false;
   }

   void SCodeList::successor(string& code)                  // code += 1
   {
      string::reverse_iterator ecode = code.rbegin();

      while(ecode != code.rend())   // Start at least significant end
      {
         if(*ecode!= cZZ)           // Its OK to add 1
         {
            *ecode += cUnit;        // There is no carry

            return;                 // We are done
         }

         *ecode = cAA;              // Z + 1 = A carry 1

         ++ecode;                   // Do the carry
      }
   }


   void SCodeList::predecessor(string& code)                // code -= 1
   {
      string::reverse_iterator ecode = code.rbegin();

      while(ecode != code.rend())   // Start at least significant end
      {
         if(*ecode != cAA)          // Its OK to subtract 1
         {
            *ecode -= cUnit;        // There is no borrow

            return;                 // We are done
         }

         *ecode = cZZ;              // A - 1 = Z borrow 1

         ++ecode;                   // Do the borrow
      }
   }

   string SCodeList::make_code(const string& name)          // GENERATE A UNIQUE CODE
   {
      // The code is generated from the first characters of the name.
      // If the resulting code is not unique we find a gap for the
      // required code in the list sorted alphabetically.

      if(full())
         return string(codesize,cQuery);

      // Since the list is not full the codes have the following property:
      // It is impossible for first code=AAA and last code=ZZZ

      string sname = squeeze(name);                         // Accept pure alphabetic

      if(sname.empty())                                     // Ensure there is at least one alpha
         return string(codesize,cQuery);

      string code = uppercase(sname.substr(0, codesize));   // A potentially unique code

      if(code.size()<codesize)
         code.resize(codesize, cGlyA);

      codelist.sort();                                      // The existing codes are unique,
                                                            // so the code part defines the lexical order
      list<string>::iterator itemnow = codelist.begin();    // SEARCH THROUGH LIST FOR CONSTRUCTED CODE

      while(itemnow != codelist.end())
      if(code == code_part(*itemnow))                      // The code has already been used
         break;
      else
         ++itemnow;                                         // Next item

      if(itemnow==codelist.end())                           // The code has not been used
         return code;
         
                                                            // CODE IS USED - HUNT FOR GAP IN USED CODES
      list<string>::iterator itemcode = itemnow;            // Remember where we are

      ++itemnow;  successor(code);

      while(itemnow != itemcode)          // SEARCH TREATING codelist AS A CIRCULAR LIST
      {
         if(itemnow == codelist.end())
         {
            if(code != firstcode)         // code is successor of last code on list
               return code;               // So all codes beyond that are available if not wrapped

            itemnow = codelist.begin();   // Here code is first_code, and list has wrapped
         }

         if(code != code_part(*itemnow))     // The codelist is sorted, therefore
            return code;                     // succcessor(code_part(itemnow))<= code_part(++itemnow)

         ++itemnow;  successor(code);
      }
      return code;
   }


   bool SCodeList::decode                                   // FIND NAME GIVEN CODE
   (
      const string& sourcecode,                             // The given code
      string& name                                          // The name found or ???
   )
   {
      list<string>::iterator itemnow = codelist.begin();

      string item=sEmpty, code=sEmpty;

      name = sEmpty;

      while(itemnow != codelist.end())                      // SEARCH FOR sourcecode IN LIST
      {
         item = *itemnow;

         size_t isize = item.size();

         if(isize<=codesize+1)                              // Garbage codelist - name is empty
            return false;

         code = code_part(item);                            // First part of CODE=NAME gives CODE
         name = name_part(item);                            // Strip off CODE= to leave name

         if(code==sourcecode)                               // We have found the given code and are done
            return true;

         ++itemnow;
      }

      name = string(codesize, cQuery);                      // Given code is not in list

      return false;
   }



   SCodeList::Query SCodeList::encode                       // GIVEN A NAME, ESTABLISH CORRESPONDING CODE
   (
      const string& sourcename,                             // Search for sourcename in list
      string& code                                          // Corresponding code found or generated
   )
   {
      list<string>::iterator itemnow = codelist.begin();

      string item=sEmpty, name=sEmpty;

      code = string(codesize, cQuery);

      if(sourcename.empty())
         return qyCorrupt;

      while(itemnow != codelist.end())                      // SEARCH FOR sourcename IN LIST
      {
         if((*itemnow).size()<=codesize+1)                  // Cannot be CODE=Name
            return qyCorrupt;

         code = code_part(*itemnow);                        // Leading characters are the CODE
         name = name_part(*itemnow);                        // Strip off CODE= to get Name

         if(name==sourcename)                               // Name is known
            return qyKnownName;

         ++itemnow;
      }

      // NAME IS NOT KNOWN - GENERATE A UNIQUE CODE 

      if(full())                                            // Cannot add more items
         return qyFull;

      code = make_code(sourcename);                         // Generate unique code

      codelist.push_back(code + sEqual + sourcename);       // Add new item CODE=Name

      return qyNewName;
   }

   string SCodeList::code_item                              // RETURN ITEM OR EMPTY, GIVEN CODE
   (
      const string& sourcecode                              // The given code
   )
   {
      list<string>::iterator itemnow = codelist.begin();

      string item=sEmpty, code=sEmpty;

      while(itemnow != codelist.end())                      // SEARCH FOR sourcecode IN LIST
      {
         item = *itemnow;

         if(item.size()<=codesize+1)                        // Garbage codelist - name is empty
            return sEmpty;

         if(code_part(item)==sourcecode)                    // We have found the given code
            return item;                                    // So return the item = "code=name"

         ++itemnow;
      }
      return sEmpty;
   }
}
//------------------------------------------------------------------------------




