////////////////////////////////////////////////////////////////////////////////
// MODULE Strings:      String processing support              G.Kaniuk 07/05/01
//

#include <vcl.h>                 // REQUIRED ONLY FOR CLASS String

#include <string>                // ALL TOOLS HERE USE string, NOT String
#include <list>

#ifndef StringsH
#define StringsH

namespace Tools                              // MAGIC CONSTANTS AND STRING INTERFACE
{
   using std::string;

   extern const size_t max_item_size;                 // Item length limit

   extern const char *sEmpty,*sSpace,*sTab, *sComma;
   extern const char *sEqual,*sDot,*sTilde,*sSep, *sCrLf, *sUline, *sQuery;
   extern const char *sBra,*sKet, *sPlus, *sMinus, *sSqBra, *sSqKet;

   extern const char cTab, cNull, cUnit, cSpace, cDot, cTilde, cUline, cSep, cComma;
   extern const char cMinus, cPlus, cStar, cQuery, cEqual, cQuote, cSemic, cColon;

   extern const char cZero, cOne, cNine;
   extern const char cGlyA, cGlyZ, cGlya, cGly_z;
   extern const char cKyu,  cDan, cPro;

   extern const char cLose, cWin, cJigo, cVoid, cDlos, cUsolv, cNone, cFwin;

   extern const char cQuarter,cHalf,cThreeQuarter;

   extern const char cFractional[4];    

   char uppercase(const char c);
   char lowercase(const char c);
   bool is_ascii(const char c);
   bool is_alphabetic(const char c);
   
   bool is_ascii(string text);
   
   String s_S(std::string s);       // THE ONLY VCL CREATURE ALLOWED IN HERE
   std::string S_s(String S);       // AND THIS IS THE ONLY USE OF String

   std::string load_str(int idstr); // STRING TABLE INTERFACE
   char* load_cstr(int idstr);
}

namespace Tools                        // LEXICAL ANALYSIS
{
   enum LexClass
   {
      lxControl=1,                     // Codes 0 to 31
      lxDigit=2, lxUpper=4, lxLower=8, // "0..9", "A..Z", "a..z"
      lxFileGlyph=16,                  // ".", "\"
      lxNameGlyph=32,                  // " ", "-", "'"
      lxTermGlyph=64,                  // ":", ";"
      lxOther=128
   };

   extern const  Byte lexAlpha;
   
   void set_lexical_classes();

   bool is_lex_class(const char c, const Byte lexclass);
   bool isnot_lex_class(const char c, const Byte lexclass);

   Byte lexical_class(const char c); 
}

namespace Tools                  // TABBED STRING PROCESSING
{
   using std::string;
   using std::list;

   void tab_add(string& source, const string& item);                    // Generate packed tabbed string

   int size_tabbed(const int n, string items[]);                        // Tabbed length

   void pack_tabbed(const int n, string items[], string* packed);       // Pack items
   void pack_tabbed_line(const int n, string items[], string* packed);  // Pack items + CrLf

   int unpack_tabbed(string packed, const int n, string items[]);			// Unpack items known no
   int unpack_tabbed_exact(string packed, const int n, string item[]);	// Unpack possibly empty items
   int unpack_tabbed(string packed, const int n, list<string>& items);  // Unpack - unknown no
}

namespace Tools                           // STRING OPERATIONS
{
   string slice(char* stext, char* etext);

   string trim_left(string text);
   string trim_right(string text);
   string trim(string text);

   string compactify(string text);
   string compactify_name(string text);
   
   string pad_right(string text, size_t fieldsize);

   string squeeze(string text);           // REMOVE anything not alpha
   string strip_white_space(string text); // REMOVE cNull and cSpace
   string strip_ext(string filename);     // REMOVE .ext
   string strip_crlf(string text);			// REMOVE cRlF and any trailing chars

   string uppercase(string text);
   string lowercase(string text);

   bool begins_with(string text, string head);

   string spaces(const int count);

   int find_first_change(const string atext, const string btext);
   int find_first_change_rev(const string atext, const string btext);

   string compactify_tail_space(string text);

    void strip_null(string& text);			// REMOVE ALL cNull
}

namespace Tools                           // STRING PARSERS
{
   void truncate_field							// Maintain fixed field size for user generated data   GK 6.5.1
   (
   	const size_t fldsize,							// Required field size
      const char* cpad,									// Indicate truncation
      string& field										// Modified field
   );

	void first_null_terminator
   (
   	const string::iterator& bstring,
      string::iterator& estring
   );

   bool legal_name_syntax(string& name);

   bool parse_name(string full, string& family);

   bool parse_name
   (
      string full,
      string& given, string& middle, string& family
   );

   bool parse_family
   (
      string full,
      string& given, string& middle, string& family
   );

   bool parse_extended_family
   (
      string& fullname,                  
      string& given, string& middle, string& family
   );

   bool parse_shortname(string full, string& shortname);

   bool next_posint
   (
      string::iterator& stext,       // Start of string range
      string::iterator& etext,       // End of string range

      int& i                         // Integer delimited by any non-digit character
   );

   bool next_int							 // Accepts +ddd, ddd, -ddd
   (
      string::iterator& stext,       // Start of string range
      string::iterator& etext,       // End of string range

      int& i                         // Integer delimited by any non-digit character
   );

}

namespace Tools                                             // CODE LIST SUPPORT
{
   using std::string;
   using std::list;
   
   class SCodeList
   {
      public:
         enum Query
         {
            qyEncoded,
            qyNewName,
            qyKnownName,
            qyFull,
            qyCorrupt
         };

         size_t maxcodes;        // Total number of possible codes 26^codesize
         size_t codesize;        // Number of uppercase alpha characters in the code

         string firstcode;       // All 'A'
         string lastcode;        // All 'Z'
         
         list<string> codelist;  // Each string in list has format "CODE=Name"

      private:
         string make_code(const string& name);              // Generate code from name
         void successor(string& code);
         void predecessor(string& code);

      public:
         SCodeList();                                       // Default constructor
         SCodeList(size_t CodeSize);                        // Constructor defines empty list

         bool syntax_check(const string& item);             // Require code=name
         string name_part(const string& item);              // Extract name from code=name
         string code_part(const string& item);              // Extract code from code=name

         bool found_name(const string& name);               // Search for name
         bool found_code(const string& code);               // Search for code

         bool full();                                       // All codes used

         bool decode(const string& code, string& name);     // Search for given code

         Query encode(const string& name, string& code);    // Search for given name and generate
                                                            // new CODE=name on list if required

         string SCodeList::code_item(const string& code);   // Search for given code, return whole item                              // RETURN ITEM OR EMPTY GIVEN CODE
   };
}

//---------------------------------------------------------------------------
#endif
