////////////////////////////////////////////////////////////////////////////////
// MODULE StringsGUI:      Interface to  String, TString        G.Kaniuk 7/05/02
//

#include <vcl.h>
#pragma hdrstop

#include "ETour.h"
#include "ToolsRES.h"
#include "Strings.h"
#include "StringsGUI.h"

extern ToolsGUI::ErrorLog AppLog;
////////////////////////////////////////////////////////////////////////////////

namespace ToolsGUI                                   // ERROR MANAGER
{
   void Throw(EType etype, short idproc, short idmsg)
   {
      ThrowError(etype, IDG_StringsGUI, idproc, idmsg);
   }
}
////////////////////////////////////////////////////////////////////////////////

namespace ToolsGUI												// TStrings SUPPORT
{
	using std::string;
   using Tools::S_s;
   using Tools::s_S;

   using Tools::SCodeList;

   void LoadCodeList                                 // LOAD THE CODELIST AFRESH
   (
      TStrings* newcodes,                            // List of Code=Name from
      SCodeList& encoded                             // The code list to be loaded
   )
   {
      encoded.codelist.clear();

      for(int i=0; i<newcodes->Count; i++)
         encoded.codelist.push_back(S_s(newcodes->Strings[i]));
   }

   void LoadCodeStrings                              // LOAD CodeList TO TStrings
   (
      SCodeList& encoded,
      TStrings* codes
   )
   {
      using std::list;

      codes->Clear();
      codes->BeginUpdate();

      list<string>::iterator is = encoded.codelist.begin();
      list<string>::iterator ie = encoded.codelist.end();

      while(is != ie)
      {
         codes->Add(s_S(*is));

         ++is;
      }

      codes->EndUpdate();
   }

   SCodeList::Query UpdateCodeList                    // APPEND NEW CODENAMES TO CODELIST
   (
      TStrings* newcodes,                             // List of Code=Name from .gdi
      SCodeList& codes                                // Current used codes
   )
   {
      for(int i=0; i<newcodes->Count; i++)
      {
         string item = S_s(newcodes->Strings[i]);

         if(!codes.syntax_check(item))             // Require code=name syntax
            return SCodeList::qyCorrupt;           // with valid no of code chars

         string code = codes.code_part(item);
         string name = codes.name_part(item);
         
         SCodeList::Query encoded = codes.encode(name, code);

         switch(encoded)
         {
            case SCodeList::qyNewName:
               // NewName is freshly encoded
               // So the generated code is unofficial
            break;

            case SCodeList::qyKnownName:
               // KnownName is on the existing codelist
               // So no new code is generated, and the code
               // could be official or unofficial
            break;

            case SCodeList::qyFull:
               // Cannot add any more items
               return SCodeList::qyFull;

            default:
               // Syntax error
               return SCodeList::qyCorrupt;
         }
      }
      return SCodeList::qyEncoded;
   }

   void ExtractNames(Tools::SCodeList& encoded, TStrings* names)
   {
      using std::list;

      list<string>::iterator is = encoded.codelist.begin();
      list<string>::iterator ie = encoded.codelist.end();

      names->BeginUpdate();

      names->Clear();

      while(is != ie)
      {
         size_t codesize = encoded.codesize;

         string item = *is;

         string name = item.substr(codesize+1, item.size()-codesize-1);

         names->Add(s_S(name));

         ++is;
      }

      names->EndUpdate();
   }

   void OfficialCode(TStrings* Official, SCodeList& actual, string& code)
   {
   	string value, query(code.size(), Tools::cQuery);

      if(!actual.decode(code, value))
      {
         code = value;  return;        // Fill code with ?
      }

      String Value = s_S(value), Code = s_S(query);

      for(int i=0; i<Official->Count; i++)
         if(Official->Values[Official->Names[i]]==Value)
         {
            Code = Official->Names[i];

            break;
         }

      code = S_s(Code);
   }

   void OfficialClubCode(TStringList* official, SCodeList& clubs, string& clubcode)
   {
   	// THE clubcode HERE IS A 3-LETTER INTERNALLY GENERATED CODE
      // WE IDENTIFY THE FULLNAME THEN LOOK UP THE OFFICIAL 4-LETTER
      // IN THE OFFICIAL LIST

   	string clubname, query(4, Tools::cQuery);

      if(!clubs.decode(clubcode, clubname))
      {
         clubcode = clubname;		// Empty for bad code
         								// Filled with ? if not in list
         return;
      }

      // SEARCH FOR clubname IN THE official TAB DELIMITED LIST
      for(int i=0; i<official->Count; i++)
      {
      	const string line 	 = S_s(official->Strings[i]);
         const size_t linesize = line.size();

         // FIND NEXT LINE CONTAINING A TAB
         size_t tabpos = line.find(Tools::cTab, 0);				// End of code part

         if(tabpos < linesize)
         {
         	size_t namestart = tabpos+1;								// Start of name part
            size_t namesize  = linesize-namestart;

            const string& realname = line.substr(namestart, namesize);

            if(realname==clubname)
            {
            	clubcode = string(line, 0, tabpos);

               return;
            }
         }
      }

      clubcode = query;
   }

   string PublicCode(TStrings* CodeList, string publicname)
   {
   	using Tools::uppercase;

   	string Template = S_s(CodeList->Strings[0]);

      int NamePos = Template.find(Tools::cEqual, 0) + 1;

      if(!(NamePos==3 || NamePos==4)) return string(2, Tools::cQuery);

   	string PublicName = uppercase(publicname);

   	for(int i=0; i<CodeList->Count; i++)
      {
      	string ilisted = S_s(CodeList->Strings[i]);
      	string iListed = uppercase(ilisted);

         int NameFound = iListed.find(PublicName, NamePos);

         if(NameFound == NamePos)
         	return ilisted.substr(0, NamePos-1);
      }
      return string(NamePos-2, Tools::cQuery);
   }
}

namespace ToolsGUI
{
	int FindNearest(const String text, TStrings* list, bool& exact)
   {
   	const String target = text.UpperCase();
      const int targetlen = target.Length();

      exact = false;

   	if(target.IsEmpty()) return -1;

      int item = 0;

      for(int i=0; i<list->Count; i++)
      {
      	const String ilist = list->Strings[i].UpperCase();
         const int ilistlen = ilist.Length();

         if(targetlen <= ilistlen)					// Target MAY start ilist
         	if(ilist.Pos(target)==1)				// Target DOES start ilist
            {
            	exact = targetlen == ilistlen;	// Exact match

               item = i;								// Item found

            	break;
            }
      }

      return item;
   }

   int FindInTabbedList(const String text, TStrings* list)
   {
   	const String Tab    = String(Tools::sTab);
   	const String target = text.UpperCase();
      const int targetlen = target.Length();

   	if(target.IsEmpty()) return -1;

      int item = -1;

      for(int i=0; i<list->Count; i++)
      {
      	const String ilist = list->Strings[i].UpperCase();
         const int ilistlen = ilist.Length();

         if(targetlen <= ilistlen)						// Target MAY start ilist
         	if(ilist.Pos(target)==1)					// Target DOES start ilist
            	if(ilist.Pos(Tab)==targetlen+1) 		// Exact match
               {
               	item = i;								// Item found

                  break;
               }
      }

      return item;
   }

   void AddNewItem(const String item, TStrings* items)
   {
   	if(item.IsEmpty())
      	return;

   	if(items->Count==0)					// List is empty
        items->Add(item);
      else
         if(items->IndexOf(item)<0)  	// Add only if item not in list
            items->Add(item);
   }
}
//---------------------------------------------------------------------------

namespace ToolsGUI												// String SUPPORT
{
   bool WhiteSpace(const String S)
   {
   	if(S.IsEmpty()) return true;

      for(int c=1; c<=S.Length(); c++)
      	if(S[c] != Tools::cSpace)
         	return false;

      return true;
   }
}

