////////////////////////////////////////////////////////////////////////////////
// MODULE ByteStream:  GByteStream tools for laying      G.Kaniuk 5/07/02
//                     objects onto a buffer

#include <vcl.h>
#include <algorithm>
#pragma hdrstop

#include "ETourRES.h"
#include "TourRES.h"
#include "Strings.h"
#include "ByteStream.h"

namespace Tours                                          // CONSTRUCTION
{
   ByteStream::ByteStream(size_t bsize)
   {
      stream = pstream = estream = NULL;

      try
      {
         stream = new Byte[bsize];

         streamsize =  bsize;

         pstream = stream;

         estream = stream + bsize;
      }
      catch(...)
      {
         Throw(ToolsGUI::erOperr, IDT_ByteStream, IDS_NoMemory);
      }
   }

   ByteStream::~ByteStream()
   {
      delete[] stream;
   }
}

namespace Tours                                          // ELEMENTARY TYPES
{
   using ToolsGUI::erBug;

   void ByteStream::put_bytes(size_t count, Byte* bytes)
   {
      if(pstream+count > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      for(size_t i=0; i<count; i++)
      	*pstream++ = bytes[i];
   }

   void ByteStream::get_bytes(size_t count, Byte* bytes)
   {
      if(pstream+count > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      for(size_t i=0; i<count; i++)
      	bytes[i] = *pstream++;
   }

   void ByteStream::put_string(std::string text)
   {
      size_t textsize = text.size();

      if(pstream+textsize > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      for(size_t i=0; i<textsize; i++)
         *pstream++ = (Byte)text[i];
   }

   void ByteStream::get_string(std::string& text)
   {
      size_t textsize = text.size();

      if(pstream+textsize > estream)
         Throw(erBug, IDT_ByteStream_get, IDT_BadByteSize);

      for(size_t i=0; i<textsize; i++)
         text[i] = (char)*pstream++;
   }

   void ByteStream::put_String(String text)
   {
      size_t textsize = text.Length();

      if(pstream+textsize > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      for(size_t i=0; i<textsize; i++)
         *pstream++ = (Byte)text[i+1];
   }

   void ByteStream::get_String(String& text)
   {
      size_t textsize = text.Length();

      if(pstream+textsize > estream)
         Throw(erBug, IDT_ByteStream_get, IDT_BadByteSize);

      for(size_t i=0; i<textsize; i++)
         text[i+1] = (char)*pstream++;
   }
}


namespace Tours                                          // RULES
{
   using ToolsGUI::erBug;

   void ByteStream::put_GRules(GRules& rules)
   {
      size_t rulesize = rules.Size();

      if(pstream+rulesize > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      put_value(rules.Checked);
      put_value(rules.Rounds);

      put_value(rules.TieOne);      put_value(rules.TieTwo);
      put_value(rules.Style);       put_value(rules.RuleSet);
      put_value(rules.Prizes);

      put_value(rules.BarSeg);

      put_value(rules.MacBar);      put_value(rules.SupBar);
      put_value(rules.MinGrade);

      put_value(rules.KomiGroup);   put_value(rules.KomiDiff);
      put_value(rules.KomiRate);

      put_value(rules.Unresolved);	put_value(rules.SuperDiff);
   }

   void ByteStream::get_GRules(GRules& rules)
   {
      size_t rulesize = rules.Size();

      if(pstream+rulesize > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      get_value(rules.Checked);  get_value(rules.Rounds);
      get_value(rules.TieOne);   get_value(rules.TieTwo);
      get_value(rules.Style);    get_value(rules.RuleSet);
      get_value(rules.Prizes);

      get_value(rules.BarSeg);

      get_value(rules.MacBar);
      get_value(rules.SupBar);
      get_value(rules.MinGrade);

      get_value(rules.KomiGroup);
      get_value(rules.KomiDiff);
      get_value(rules.KomiRate);

      get_value(rules.Unresolved);
      get_value(rules.SuperDiff);
   }
}

namespace Tours                                          // GAME & PLAYER
{
	using Tools::truncate_field;
   using ToolsGUI::erBug;

   void ByteStream::put_GGame(GGame& game)
   {
      if(pstream+game.Size() > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      put_value(game.Board);
      put_value(game.Oppid);

      put_value(game.Komi);
      put_value(game.Stones);

      put_value(game.Colour);
      put_value(game.Parity);
      put_value(game.Result);
   }

   void ByteStream::get_GGame(GGame& game)
   {
      if(pstream+game.Size() > estream)
         Throw(erBug, IDT_ByteStream_get, IDT_BadByteSize);

      get_value(game.Board);
      get_value(game.Oppid);


      get_value(game.Komi);
      get_value(game.Stones);

      get_value(game.Colour);
      get_value(game.Parity);
      get_value(game.Result);
   }

   void ByteStream::put_GPlayer(GPlayer& player)
   {
      using Tools::sTab; using Tools::cNull;

      if(pstream+player.Size() > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      string namenote = player.Name + string(sTab) + player.Note;
      size_t nnsize = namenote.size();

      //if(nnsize > player_text_size)
         //Throw(ToolsGUI::erOperr, IDT_ByteStream_put, IDT_BadByteSize);

      if(nnsize > player_text_size)
      	truncate_field(player_text_size, "ZZZ", namenote);						// GK 6.5.1  
      if(nnsize < player_text_size)
      	namenote.resize(player_text_size, cNull);

      put_value(player.State);

      put_value(player.Pid);

      put_string(namenote);

      put_string(player.Club);         put_string(player.Land);
      put_string(player.Group);

      put_value(player.Grade);          put_value(player.MiScore);
      put_value(player.McScore);

      put_value(player.Cbal);          put_value(player.Mbal);
      put_value(player.Mout);

      const size_t nround = player.Games.size();
      const size_t sizegames = nround*GGame().Size();
      if(pstream + sizegames != estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      for(size_t r=0; r<nround; r++)
         put_GGame(player.Games[r]);
   }

   void ByteStream::get_GPlayer(GPlayer& player)
   {
      using std::string;
      using std::find;
      
      using Tools::sTab;
      using Tools::cTab;
      using Tools::cNull;

      if(pstream+player.Size() > estream)
         Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

      string namenote(player_text_size, cNull);

      get_value(player.State);

      get_value(player.Pid);				get_string(namenote);

      get_string(player.Club);         get_string(player.Land);
      get_string(player.Group);

      get_value(player.Grade);         get_value(player.MiScore);
      get_value(player.McScore);

      get_value(player.Cbal);          get_value(player.Mbal);
      get_value(player.Mout);

      for(size_t r=0; r<player.Games.size(); r++)
          get_GGame(player.Games[r]);

      string::iterator ns = namenote.begin();
      string::iterator ne = namenote.end();

      string::iterator nt;

      nt = find(ns,ne, cTab);    player.Name = string(ns,nt);

      ns = ++nt;

      nt = find(ns,ne, cNull);   player.Note = string(ns,nt);

      player.TieOne = 0;
      player.TieTwo = 0;      
   }

   void ByteStream::put_GPlayerPtr(GPlayerPtr player)
   {
      put_GPlayer(*player);
   }

   void ByteStream::get_GPlayerPtr(GPlayerPtr player)
   {
      get_GPlayer(*player);
   }
}
// ByteStream
////////////////////////////////////////////////////////////////////////////////


