////////////////////////////////////////////////////////////////////////////////
// MODULE ByteStream:  GByteStream tools for laying      G.Kaniuk 5/07/02
//                     objects onto a buffer

#include "Tour.h"

#ifndef ByteStreamH
#define ByteStreamH

namespace Tours
{
   class ByteStream
   {
      private:
         size_t streamsize;   // Number of bytes in stream

         Byte* stream;        // The stream is on the heap
         Byte* pstream;       // Current position in the stream
         Byte* estream;       // One past the last stream byte

      public:
         ByteStream(size_t bsize);
         ~ByteStream();

         Byte* bytes(){ return stream; }

         size_t size(){ return streamsize; }

         template <class T>
         void get_value(T& val)
         {
            size_t valsize = sizeof(T);

            if(pstream+valsize > estream)
               Throw(erBug, IDT_ByteStream_get, IDT_BadByteSize);

            val = *((T*)pstream);    pstream += valsize;
         }

         template <class T>
         void put_value(T val)
         {
            size_t valsize = sizeof(T);

            if(pstream+valsize > estream)
               Throw(erBug, IDT_ByteStream_put, IDT_BadByteSize);

            *((T*)pstream) = val;

            pstream += valsize;
         }

         void put_bytes(size_t count, Byte* bytes);
         void get_bytes(size_t count, Byte* bytes);

         void put_string(std::string val);
         void get_string(std::string& val);

         void put_String(String val);
         void get_String(String& val);

         void put_GRules(GRules& rules);
         void get_GRules(GRules& rules);

         void put_GGame(GGame& game);
         void get_GGame(GGame& game);

         void put_GPlayer(GPlayer& player);
         void get_GPlayer(GPlayer& player);

         void put_GPlayerPtr(GPlayerPtr player);
         void get_GPlayerPtr(GPlayerPtr player);
   };
}

#endif
// ByteStream
////////////////////////////////////////////////////////////////////////////////

 