////////////////////////////////////////////////////////////////////////////////
// MODULE TourStream:  Implementation of GTourStream           G.Kaniuk 8/06/02
//                     Provides all file services for GTour

#include "ETour.h"
#include "Tour.h"

#ifndef TourStreamH
#define TourStreamH

namespace Tours                        // STANDARD FILES, IDENTIFICATION
{
   using std::string;
   
   enum                                   // SIZES
   {
      CodeFileSize = 8192,                // Combined Land and Club codes

      OtherSize
   };

   enum file_t
   {
      ftGodraw,                           // Any DOSgodraw file .dda
      ftGoDraw,                           // .gdt file
      ftUnknown                           // Unrecognisable extension
   };

   enum filact_t
   {
      faCreate,                           // Create destroying any previous contents
      faOpen                              // Open file for read/write, exclusive
   };

   extern const string TourNow;          // Default filename
   extern const string TourExt;          // Tournament file - binary
   extern const string TourVer;          // Software version

   file_t IdentifyFile(string FileName);
}


namespace Tours                         // CLASS GTourStream
{
   using std::string;

   class GTour;

   class ByteStream;

   class GTourStream
   {
      public:
         string FileName;                 // Full path
         
         int OpenSize;                    // The size at file open
         int HeadSize;                    // Size of header

         int StartPos;                    // Start position in file
         int SizePos;                     // Position of filesize
         int CodesPos;                    // Start of codes
         int HeaderPos;                   // Start of header
         int PlayerPos;                   // Start of first player

         TFileStream* Stream;

      private:
         void SaveFileSize();                                        // Update at file close
         void SaveFileStamp();                                       // Version control
         void VerifyFileStamp();
         void SaveEmptyCodes();

      public:
         __fastcall ~GTourStream();          // Close file
         __fastcall GTourStream              // Create or open
         (
            string FileName,
            file_t FileType,
            filact_t Action
         );

         void StreamThrow(ToolsGUI::EType etype, short idproc, short idmsg);

         void SaveCodes(GTour& Tour);
         void SaveImportedCodes  
   		(
   			GTour& Tour, const int PlayerCount, GPlayer* Players
   		);
         void LoadCodes(GTour& Tour);

         void SaveHeader(GTour& Tour);
         void LoadHeader(GTour& Tour);

         void SavePlayer(GPlayer& Player);
         void AppendPlayer(GPlayer& Player);
         void LoadPlayer(GPlayer& Player);

         void SavePlayer(size_t p, GTour& Tour);
         void LoadPlayer(size_t p, GTour& Tour);

         void SavePlayer(size_t p, GTour& Tour, GPlayer& Player);
         void LoadPlayer(size_t p, GTour& Tour, GPlayer& Player);

         void LoadTour(GTour& Tour);

         virtual bool VerifyVersion(string version);
   };
}

namespace Tours                        // TOURNAMENT FILE INTERFACE
{
   class GTour;

   void Save(GTour& Tour);                // Tour knows the filename

   void Load
   (
      std::string FileName,              // Supplied by user or ini file
      GTour& Tour
   );
}

#endif
////////////////////////////////////////////////////////////////////////////////
